package com.agilex.healthcare.mobilehealthplatform.consult.dao.cdw;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;

import com.agilex.healthcare.mobilehealthplatform.consult.dao.Mapper;
import com.agilex.healthcare.mobilehealthplatform.consult.dao.Transformable;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.Consult;
import com.agilex.healthcare.mobilehealthplatform.domain.Consults;
import com.agilex.healthcare.mobilehealthplatform.domain.DomainTransferObject;

@Repository
public class ConsultDaoCdw extends GenericDaoCdw<Consult, Long> {

    public Consults fetchPatientConsults(String patientICN, List<String> statuses,
                                         ConsultFilter filter) {
        boolean includeStatus = (statuses.size() > 0);
        boolean dateRange = (filter.getStartDate() != null && filter.getEndDate() != null);
        String sql =  getPatientConsultsQuery(includeStatus, dateRange);

        Query query = this.cdwEntityManager.createNativeQuery(sql, ConsultPo.class);
        query.setParameter("patientId", patientICN);
        if(dateRange) {
            query.setParameter("startDate", filter.getStartDate());
            query.setParameter("endDate", filter.getEndDate());
        }
        else {
            query.setParameter("daysOld", filter.getDaysFromCreatedDate());
        }

        if (includeStatus){
            query.setParameter("statuses", statuses);
        }

        @SuppressWarnings("unchecked")
        List<ConsultPo> resultList = query.getResultList();
        List<Transformable<? extends DomainTransferObject>> transformableList = createTransformableList(resultList);
        Consults consults = new Consults();
        Mapper.add(transformableList, consults);
        return consults;
    }

    private String getPatientConsultsQuery(boolean includeStatus, boolean dateRange){
        String sql = "";
        if(dateRange)   {
            sql = "SELECT c.* FROM App.Consult c WHERE (VistaCreateDate >= :startDate and VistaCreateDate <= :endDate) " +
                    "and c.PatientICN = :patientId";
        }
        else {
            sql = "SELECT c.* FROM App.Consult c WHERE VistaCreateDate > (GETDATE() - :daysOld) and c.PatientICN = :patientId";
        }

        if (includeStatus){
            sql = sql.concat(" and c.OrderStatus in (:statuses)");
        }
        sql = sql.concat(" order by VistaCreateDate desc");
        return sql;
    }


    private List<Transformable<? extends DomainTransferObject>> createTransformableList(List<ConsultPo> results) {
		List<Transformable<? extends DomainTransferObject>> transformableList = new ArrayList<Transformable<? extends DomainTransferObject>>();
		transformableList.addAll(results);
		return transformableList;
	}
	
	public Consults fetchConsultsWrittenByProvider(String providerIEN, String Sta3n, List<String> statuses, ConsultFilter filter){
		
		Query query = constructConsultsByProviderQuery(providerIEN, Sta3n, statuses, filter);
		
		@SuppressWarnings("unchecked")
		List<ConsultPo> resultList = query.getResultList();
		List<Transformable<? extends DomainTransferObject>> transformableList = createTransformableList(resultList);
		Consults consults = new Consults();
		Mapper.add(transformableList, consults);
		
		return consults;
	}

	private Query constructConsultsByProviderQuery(String providerIEN, String Sta3n, List<String> statuses, ConsultFilter filter) {
		boolean includeStatus = (statuses.size() > 0);
		boolean isQueryByDateRange = filter!= null && filter.doesDateRangeExist();
		String  consultsByProviderQuery;
		
		if(isQueryByDateRange){
			consultsByProviderQuery = "SELECT c.* FROM App.Consult c WHERE (VistaCreateDate >= :startDate and VistaCreateDate <= :endDate) and OrderingProviderIEN = :orderingProviderIEN and Sta3n = :facilityCode";
		}else{
			consultsByProviderQuery = "SELECT c.* FROM App.Consult c WHERE OrderingProviderIEN = :orderingProviderIEN and Sta3n = :facilityCode";
		}
		
		if (includeStatus){
			consultsByProviderQuery = consultsByProviderQuery.concat(" and c.OrderStatus in (:statuses)");
		}
		
		consultsByProviderQuery = consultsByProviderQuery.concat(" order by VistaCreateDate desc");
		
		Query query = this.cdwEntityManager.createNativeQuery(consultsByProviderQuery, ConsultPo.class);
		
		query.setParameter("orderingProviderIEN", providerIEN);
		query.setParameter("facilityCode", Sta3n);
		
		if(isQueryByDateRange){
            query.setParameter("startDate", filter.getStartDate());
            query.setParameter("endDate", filter.getEndDate());
		}
		
		if (includeStatus){
			query.setParameter("statuses", statuses);
		}
		return query;
	}
	
	
	public Consults fetchConsultsBySpecialties(List<String> specialtySIDs, ConsultFilter filter, String facilityCode){
		
		Query query = createConsultsBySpecialitiesQuery(specialtySIDs, filter, facilityCode);
		
		@SuppressWarnings("unchecked")
		List<ConsultPo> resultList = query.getResultList();
		List<Transformable<? extends DomainTransferObject>> transformableList = createTransformableList(resultList);
		Consults consults = new Consults();
		Mapper.add(transformableList, consults);
	
		return consults;
	}

	private Query createConsultsBySpecialitiesQuery(List<String> specialtySIDs, ConsultFilter filter, String facilityCode) {
		
		final String consultsBySpecialitiesQuery;
		boolean isQueryByDateRange = filter!= null && filter.doesDateRangeExist();
		
		if(isQueryByDateRange){
			consultsBySpecialitiesQuery = "SELECT * FROM App.Consult WHERE (VistaCreateDate >= :startDate and VistaCreateDate <= :endDate) and Sta3n = :facilityCode and ToRequestServiceSID in (:specialtySIDs) order by VistaCreateDate desc";
		}else{
			consultsBySpecialitiesQuery = "SELECT * FROM App.Consult WHERE Sta3n = :facilityCode and ToRequestServiceSID in (:specialtySIDs) order by VistaCreateDate desc";
		}
		
		Query query = this.cdwEntityManager.createNativeQuery(consultsBySpecialitiesQuery, ConsultPo.class);
		
		if(isQueryByDateRange){
            query.setParameter("startDate", filter.getStartDate());
            query.setParameter("endDate", filter.getEndDate());
		}
		
		query.setParameter("specialtySIDs", specialtySIDs);
		query.setParameter("facilityCode", facilityCode);
		return query;
	}
	
	
	
}