package com.agilex.healthcare.mobilehealthplatform.consult.dao.cdw;

import com.agilex.healthcare.mobilehealthplatform.consult.dao.Transformable;
import com.agilex.healthcare.mobilehealthplatform.domain.Consult;
import com.agilex.healthcare.mobilehealthplatform.enumeration.ConsultStatus;
import com.agilex.healthcare.mobilehealthplatform.enumeration.InOutpatientCare;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@Entity
@Table(name = "App.Consult")
@XStreamAlias("Consult")
public class ConsultPo implements Transformable<Consult>, Serializable {
	
	private static final long serialVersionUID = -6685853437786624261L;
	
	@Id
	@Column(name = "ConsultSID")
	private Long consultId;
	
	@Column(name = "PatientICN")
	private String patientId;
	
	@Column(name = "VistaCreateDate")
	@Temporal(TemporalType.DATE)
	private Date createdDate;
	
	@Column(name = "OrderStatus")
	private String status;	
	
	@Column(name = "OrderingProviderName")
	private String orderingProviderName;
	
	@Column(name = "ConsultingProviderName")
	private String consultingProviderName;
	
	@Column(name = "PlaceOfConsultation")
	private String placeOfConsultation;
	
	@Column(name = "RequestDateTime")
	private Date requestDate;
	
	@Column(name = "ToRequestServiceName")
	private String requestServiceName;
	
	@Column(name = "Urgency")
	private String urgency;

	@Column(name = "ProvisionalDiagnosis")
	private String provisionalDiagnosis;

	@Column(name = "InpatOutpat")
	private char inpatOutpat;

	@Column(name = "PatientName")
	private String patientName;

	@Column(name = "PatientIEN")
	private String patientIEN;

	@Column(name = "ConsultIEN")
	private String consultIEN;
	
	@Column(name = "ToRequestServiceSID")
	private Long requestServiceSID;
	
	public ConsultPo() {}

	public ConsultPo(Consult dto) {
		this.setConsultId(dto.getConsultId());
		this.setPatientId(dto.getPatientId());
		this.setCreatedDate(dto.getCreatedDate());
		this.setStatus(dto.getStatus().name());
		this.setOrderingProviderName(dto.getOrderingProviderName());
		this.setConsultingProviderName(dto.getConsultingProviderName());
		this.setPlaceOfConsultation(dto.getPlaceOfConsultation());
		this.setRequestDate(dto.getRequestDate());	
		this.setRequestServiceName(dto.getRequestServiceName());
		this.setUrgency(dto.getUrgency());
		this.setProvisionalDiagnosis(dto.getProvisionalDiagnosis());
		this.setInpatOutpat(dto.getInOutpatientCare().getCode());
		this.setPatientName(dto.getPatientName());
		this.setPatientIEN(dto.getPatientIEN());
		this.setConsultIEN(dto.getConsultIEN());
		this.setRequestServiceSID(dto.getRequestServiceSID());
	}
	
	public Consult create() {
		Consult consult = new Consult();
		consult.setConsultId(this.getConsultId());
		consult.setPatientId(this.getPatientId());
		consult.setCreatedDate(this.getCreatedDate());
		consult.setStatus(ConsultStatus.lookUpByName(this.status));
		consult.setOrderingProviderName(this.getOrderingProviderName());
		consult.setConsultingProviderName(this.getConsultingProviderName());
		consult.setPlaceOfConsultation(this.getPlaceOfConsultation());	
		consult.setRequestDate(this.getRequestDate());
		consult.setRequestServiceName(this.getRequestServiceName());
		consult.setUrgency(this.getUrgency());
		consult.setProvisionalDiagnosis(this.getProvisionalDiagnosis());
		consult.setInOutpatientCare(InOutpatientCare.lookUpByCode(this.getInpatOutpat()));
		consult.setPatientName(this.getPatientName());
		consult.setPatientIEN(this.patientIEN);
		consult.setConsultIEN(this.consultIEN);
		consult.setRequestServiceSID(this.getRequestServiceSID());
		return consult;
	}	
	
	public Long getConsultId() {
		return this.consultId;
	}
	public void setConsultId(Long id) {
		this.consultId = id;
	}
	
	public String getPatientId() {
		return this.patientId;
	}
	public void setPatientId(String patientId) {
		this.patientId = patientId;
	}	
	
	public Date getCreatedDate() {
		return this.createdDate;
	}
	public void setCreatedDate(Date createdDate) {
		this.createdDate = createdDate;
	}

	public String getStatus() {
		return this.status;
	}
	public void setStatus(String status) {
		this.status = status;
	}

	public String getOrderingProviderName() {
		return orderingProviderName;
	}
	public void setOrderingProviderName(String orderingProviderName) {
		this.orderingProviderName = orderingProviderName;
	}

	public String getConsultingProviderName() {
		return consultingProviderName;
	}
	public void setConsultingProviderName(String consultingProviderName) {
		this.consultingProviderName = consultingProviderName;
	}
	
	public String getPlaceOfConsultation() {
		return this.placeOfConsultation;
	}
	public void setPlaceOfConsultation(String placeOfConsultation) {
		this.placeOfConsultation = placeOfConsultation;
	}	
	
	public Date getRequestDate() {
		return this.requestDate;
	}
	public void setRequestDate(Date requestDate) {
		this.requestDate = requestDate;
	}
	
	public String getRequestServiceName() {
		return requestServiceName;
	}
	public void setRequestServiceName(String requestServiceName) {
		this.requestServiceName = requestServiceName;
	}
	public String getUrgency() {
		return urgency;
	}

	public void setUrgency(String urgency) {
		this.urgency = urgency;
	}

	public String getProvisionalDiagnosis() {
		return provisionalDiagnosis;
	}

	public void setProvisionalDiagnosis(String provisionalDiagnosis) {
		this.provisionalDiagnosis = provisionalDiagnosis;
	}

	public char getInpatOutpat() {
		return inpatOutpat;
	}

	public void setInpatOutpat(char inpatOutpat) {
		this.inpatOutpat = inpatOutpat;
	}
	public String getPatientName() {
		return patientName;
	}

	public void setPatientName(String patientName) {
		this.patientName = patientName;
	}
	public String getPatientIEN() {
		return patientIEN;
	}

	public void setPatientIEN(String patientIEN) {
		this.patientIEN = patientIEN;
	}

	public String getConsultIEN() {
		return consultIEN;
	}

	public void setConsultIEN(String consultIEN) {
		this.consultIEN = consultIEN;
	}

	public Long getRequestServiceSID() {
		return requestServiceSID;
	}

	public void setRequestServiceSID(Long requestServiceSID) {
		this.requestServiceSID = requestServiceSID;
	}

}