package com.agilex.healthcare.mobilehealthplatform.consult.datalayer;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;

import com.agilex.healthcare.mobilehealthplatform.consult.dao.cdw.ConsultDaoCdw;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.Consults;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.code.AssigningAuthorityCode;
import com.agilex.healthcare.mobilehealthplatform.enumeration.ConsultStatus;
import com.agilex.healthcare.utility.PatientIdentityHelper;

@Service
public class ConsultDataLayerCdw implements ConsultDataLayer {

	@Resource
	ConsultDaoCdw consultDao;

    @Override
    public Consults fetchPatientConsults(PatientIdentifier patientId, ConsultFilter filter) {
        checkAssigningAuthorityAsICN(patientId);
        // Strip of the hash from the ICN for CDW
        PatientIdentifier alteredPatientIdentifier = new PatientIdentityHelper().truncateICN(patientId);
        List<String> statusValues = convertConsultStatusToValues(filter.getStatuses());

        Consults patientConsults = consultDao.fetchPatientConsults(alteredPatientIdentifier.getUniqueId(), statusValues,
                 filter);
        return patientConsults;
    }

	private void checkAssigningAuthorityAsICN(PatientIdentifier patientId) throws IllegalArgumentException{
		if (patientId.getAssigningAuthority() == null)
			throw new IllegalArgumentException("Assigning Authority is null.");
		if (! patientId.getAssigningAuthority().toUpperCase().contains(AssigningAuthorityCode.ICN))
			throw new IllegalArgumentException("Assigning Authority is not ICN.");
	}

	private List<String> convertConsultStatusToValues(List <ConsultStatus> statuses){
		List<String> statusValues = new ArrayList<String>();
		for (int i = 0; i < statuses.size(); i++) {
			statusValues.add(statuses.get(i).name());
		}
		return statusValues;
	}

	@Override
	public Consults fetchConsultsWrittenByProvider(ProviderIdentifier providerIdentifier, ConsultFilter filter) {
		List<String> statusValues = convertConsultStatusToValues(filter.getStatuses());
		
		Consults consults = consultDao.fetchConsultsWrittenByProvider(providerIdentifier.getStaffId(), providerIdentifier.getFacilityCode(), 
																		statusValues, filter) ;
		return consults;
	}
	
	@Override
	public Consults fetchConsultsBySpecialties(ConsultFilter filter, String facilityCode) {
		return consultDao.fetchConsultsBySpecialties(filter.getSpecialties(), filter, facilityCode);
	}
}