package com.agilex.healthcare.facilitydefinition.datalayer.clinic;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import javax.annotation.Resource;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.domain.Clinic;
import com.agilex.healthcare.mobilehealthplatform.domain.Clinics;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class ClinicDataLayerMongoTest {
	@Resource
	ClinicDataLayer clinicDataLayer;

	@Test
	public void testFetchEmptyClinics() {
		Clinics clinics = clinicDataLayer.fetchClinics("invalid", "");
		assertEquals(0, clinics.size());
	}

	@Test
	public void testFetchClinics() {
		Clinics clinics = clinicDataLayer.fetchClinics("500", "");
		assertTrue(clinics.size() > 0);

		String previousName = "";
		for (Clinic clinic : clinics) {
			assertNotNull(clinic.getId());
			assertNotNull(clinic.getFacilityCode());
			assertNotNull(clinic.getName());
			assertTrue("sorted in ascending order", previousName.compareTo(clinic.getName()) < 0);
			previousName = clinic.getName();
		}
	}

	@Test
	public void testFetchClinicsWithFilter() {
		String searchString = "Card";
		//ClinicDataLayer dataLayer = new ClinicDataLayerMongo();

		Clinics clinics = clinicDataLayer.fetchClinics("500", searchString);
		assertTrue(clinics.size() > 0);

		String previousName = "";
		for (Clinic clinic : clinics) {
			assertTrue(clinic.getName().toLowerCase().contains(searchString.toLowerCase()));
			assertNotNull(clinic.getId());
			assertNotNull(clinic.getFacilityCode());
			assertNotNull(clinic.getName());
			assertTrue("sorted in ascending order", previousName.compareTo(clinic.getName()) < 0);
			previousName = clinic.getName();
		}
	}
}