package com.agilex.healthcare.facilitydefinition.datalayer.lab;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.lab.LabTestsDataLayerVistaImpl;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTest;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTests;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;

@ContextConfiguration(locations = {"classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class LabTestsDataLayerVistaImplTest {
	private static final String SITE_777 = "777";
	
    @Resource
    LabTestsDataLayerVistaImpl labTestsDataLayerVistaImpl;

    @BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("777", "cprs1234", "cprs4321$", "1");
	}
    @AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}

    @Test
	public void getLabTests() {
    	PatientIdentifier patientIdentifier = new PatientIdentifier("dfn-777", "1");
    	
		LabTests result = labTestsDataLayerVistaImpl.getLabTests(patientIdentifier, SITE_777, null);
		assertNotNull(result);
		assertTrue(result.size() > 0);
	}

    @Test
	public void getLabTestsByLabCategory() {
    	PatientIdentifier patientIdentifier = new PatientIdentifier("dfn-777", "1");
    	List<String> labCategoryList = new ArrayList<String>();
    	labCategoryList.add("HE");
    	
		LabTests result = labTestsDataLayerVistaImpl.getLabTests(patientIdentifier, SITE_777, labCategoryList);
		assertNotNull(result);
		assertTrue(result.size() > 0);
		
		for (int i=0; i<result.size(); i++) {
			LabTest lt = result.get(i);
			String as = lt.getAccessionSubscript();
			if (as == null || as.isEmpty() || as.contains("HE") == false)
				fail("Found a result that didn't contain the Category.");
		}
	}

    @Test
	public void getLabTestsDate() {
    	PatientIdentifier patientIdentifier = new PatientIdentifier("dfn-777", "1");
    	
		String labTestIen = "1";
		String labTestName = "";
		String startDate = "";
		String endDate = "";
		LabTest result = labTestsDataLayerVistaImpl.getLabResults(patientIdentifier, labTestIen, labTestName, SITE_777, startDate, endDate);
		assertNotNull(result);
	}

    @Test
	public void getLabTestsTestName() {
    	PatientIdentifier patientIdentifier = new PatientIdentifier("dfn-777", "1");
    	
		String labTestIen = "1";
		String labTestName = "BRUCE WAYNE";
		String startDate = "";
		String endDate = "";
		LabTest result = labTestsDataLayerVistaImpl.getLabResults(patientIdentifier, labTestIen, labTestName, SITE_777, startDate, endDate);
		assertNotNull(result);
		assertEquals(labTestName, result.getTestname());
	}

    @Test
	public void getLabTestsDateWithStartDateOnly() {
    	PatientIdentifier patientIdentifier = new PatientIdentifier("dfn-777", "1");
    	
		String labTestIen = "1";
		String labTestName = "";
		String startDate = "10/06/1990";
		String endDate = "";
		LabTest result = labTestsDataLayerVistaImpl.getLabResults(patientIdentifier, labTestIen, labTestName, SITE_777, startDate, endDate);
		assertNotNull(result);
		assertNotNull(result.getLabResults());
		assertTrue(result.getLabResults().size() > 0);
	}

    @Test
	public void getLabTestsDateWithEndDateOnly() {
    	PatientIdentifier patientIdentifier = new PatientIdentifier("dfn-777", "1");
    	
		String labTestIen = "1";
		String labTestName = "";
		String startDate = "";
		String endDate = "10/06/2004";
		LabTest result = labTestsDataLayerVistaImpl.getLabResults(patientIdentifier, labTestIen, labTestName, SITE_777, startDate, endDate);
		assertNotNull(result);
		assertNotNull(result.getLabResults());
		assertTrue(result.getLabResults().size() > 0);
	}

    @Test
	public void getLabTestsDateWithStartAndEndDate() {
    	PatientIdentifier patientIdentifier = new PatientIdentifier("dfn-777", "1");
    	
		String labTestIen = "1";
		String labTestName = "";
		String startDate = "10/06/1990";
		String endDate = "10/06/2004";
		LabTest result = labTestsDataLayerVistaImpl.getLabResults(patientIdentifier, labTestIen, labTestName, SITE_777, startDate, endDate);
		assertNotNull(result);
		assertNotNull(result.getLabResults());
		assertTrue(result.getLabResults().size() > 0);
	}

    @Test
	public void getLabTestsDatesOutOfRange() {
    	PatientIdentifier patientIdentifier = new PatientIdentifier("dfn-777", "1");
    	
		String labTestIen = "1";
		String labTestName = "";
		String startDate = "10/06/2004";
		String endDate = "10/06/2004";
		LabTest result = labTestsDataLayerVistaImpl.getLabResults(patientIdentifier, labTestIen, labTestName, SITE_777, startDate, endDate);
		assertNotNull(result);
		assertNotNull(result.getLabResults());
		assertEquals(0, result.getLabResults().size());
	}
}
