package com.agilex.healthcare.facilitydefinition.datalayer.order;

import static org.junit.Assert.*;

import java.util.Date;

import javax.annotation.Resource;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.order.OrderServicesDataLayerMongo;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderService;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderServices;
import com.mongodb.MongoClient;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class OrderServicesDataLayerMongoTest {
	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(OrderServicesDataLayerMongoTest.class);

	@Resource
	MongoClient mongoClient;

	@Resource
	OrderServicesDataLayerMongo orderServicesDataLayerMongo;
	
	@Before
	public void setup() {
		OrderUtilities.removeServicesTestDataFromMongo(mongoClient,LOGGER);
	}
	
	@Test
	public void fetchEmptyServices() {
		OrderServices services = orderServicesDataLayerMongo.fetchServices(OrderUtilities.TEST_SITE);
		assertEquals(0, services.size());
	}
	
	@Test
	public void fetchEmptyLastUpdated() {
		Date lastUpdated = orderServicesDataLayerMongo.fetchServicesLastUpdate(OrderUtilities.TEST_SITE);
		assertNull(lastUpdated);
	}
	
	@Test
	public void updateServices() {
		OrderServices orderServices = new OrderServices();
		int numOrders = 2;
		addOrderServices(orderServices, numOrders);
		
		orderServicesDataLayerMongo.updateServices(OrderUtilities.TEST_SITE, orderServices);
		OrderServices savedServices = orderServicesDataLayerMongo.fetchServices(OrderUtilities.TEST_SITE);
		
		assertOrderServices(numOrders, orderServices, savedServices);
	}
	
	@Test
	public void fetchServicesLastUpdate() {
		OrderServices orderServices = new OrderServices();
		addOrderServices(orderServices, 1);
		
		orderServicesDataLayerMongo.updateServices(OrderUtilities.TEST_SITE, orderServices);
		Date lastUpdated = orderServicesDataLayerMongo.fetchServicesLastUpdate(OrderUtilities.TEST_SITE);
		OrderServices savedServices = orderServicesDataLayerMongo.fetchServices(OrderUtilities.TEST_SITE);
		assertNotNull(lastUpdated);
		assertOrderServices(1, orderServices, savedServices);
		
		OrderServices newOrderServices = new OrderServices();
		addOrderServices(newOrderServices, 3);
		
		orderServicesDataLayerMongo.updateServices(OrderUtilities.TEST_SITE, newOrderServices);
		Date newLastUpdated = orderServicesDataLayerMongo.fetchServicesLastUpdate(OrderUtilities.TEST_SITE);
		OrderServices newSavedServices = orderServicesDataLayerMongo.fetchServices(OrderUtilities.TEST_SITE);
		assertNotNull(lastUpdated);
		assertOrderServices(3, newOrderServices, newSavedServices);
		
		assertTrue("Expected the new date to be after the previously saved date.", newLastUpdated.after(lastUpdated));
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
	private void addOrderServices(OrderServices orderServices, int num) {
		for (int i=0; i<num; i++) {
			OrderService os = new OrderService();
			os.setIen("TEST IEN - " + (i+1));
			os.setName("TEST NAME - " + (i+1));
			os.setParent("TEST PARENT - " + (i+1));
			orderServices.add(os);
		}
	}
	private void assertOrderServices(int num, OrderServices orderServices, OrderServices savedServices) {
		assertEquals(num, savedServices.size());
		
		for (int i=0; i<num; i++) {
			assertEquals(orderServices.get(i).getIen(), savedServices.get(i).getIen());
			assertEquals(orderServices.get(i).getName(), savedServices.get(i).getName());
			assertEquals(orderServices.get(i).getParent(), savedServices.get(i).getParent());
		}
	}
}
