package com.agilex.healthcare.facilitydefinition.datalayer.order;

import static org.junit.Assert.*;

import java.util.Date;

import javax.annotation.Resource;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.order.OrderStatusesDataLayerMongo;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderStatusValue;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderStatuses;
import com.mongodb.MongoClient;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class OrderStatusesDataLayerMongoTest {
	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(OrderStatusesDataLayerMongoTest.class);

	@Resource
	MongoClient mongoClient;

	@Resource
	OrderStatusesDataLayerMongo orderStatusesDataLayerMongo;
	
	@Before
	public void setup() {
		OrderUtilities.removeStatusesTestDataFromMongo(mongoClient,LOGGER);
	}
	
	@Test
	public void fetchEmptyStatuses() {
		OrderStatuses Statuses = orderStatusesDataLayerMongo.fetchStatuses(OrderUtilities.TEST_SITE);
		assertEquals(0, Statuses.size());
	}
	
	@Test
	public void fetchEmptyLastUpdated() {
		Date lastUpdated = orderStatusesDataLayerMongo.fetchStatusesLastUpdate(OrderUtilities.TEST_SITE);
		assertNull(lastUpdated);
	}
	
	@Test
	public void updateStatuses() {
		OrderStatuses orderStatuses = new OrderStatuses();
		int numOrders = 2;
		addOrderStatuses(orderStatuses, numOrders);
		
		orderStatusesDataLayerMongo.updateStatuses(OrderUtilities.TEST_SITE, orderStatuses);
		OrderStatuses savedStatuses = orderStatusesDataLayerMongo.fetchStatuses(OrderUtilities.TEST_SITE);
		
		assertOrderStatuses(numOrders, orderStatuses, savedStatuses);
	}
	
	@Test
	public void fetchStatusesLastUpdate() {
		OrderStatuses orderStatuses = new OrderStatuses();
		addOrderStatuses(orderStatuses, 1);
		
		orderStatusesDataLayerMongo.updateStatuses(OrderUtilities.TEST_SITE, orderStatuses);
		Date lastUpdated = orderStatusesDataLayerMongo.fetchStatusesLastUpdate(OrderUtilities.TEST_SITE);
		OrderStatuses savedStatuses = orderStatusesDataLayerMongo.fetchStatuses(OrderUtilities.TEST_SITE);
		assertNotNull(lastUpdated);
		assertOrderStatuses(1, orderStatuses, savedStatuses);
		
		OrderStatuses newOrderStatuses = new OrderStatuses();
		addOrderStatuses(newOrderStatuses, 3);
		
		orderStatusesDataLayerMongo.updateStatuses(OrderUtilities.TEST_SITE, newOrderStatuses);
		Date newLastUpdated = orderStatusesDataLayerMongo.fetchStatusesLastUpdate(OrderUtilities.TEST_SITE);
		OrderStatuses newSavedStatuses = orderStatusesDataLayerMongo.fetchStatuses(OrderUtilities.TEST_SITE);
		assertNotNull(lastUpdated);
		assertOrderStatuses(3, newOrderStatuses, newSavedStatuses);
		
		assertTrue("Expected the new date to be after the previously saved date.", newLastUpdated.after(lastUpdated));
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
	private void addOrderStatuses(OrderStatuses orderStatuses, int num) {
		for (int i=0; i<num; i++) {
			OrderStatusValue os = new OrderStatusValue();
			os.setIen("TEST IEN - " + (i+1));
			os.setName("TEST NAME - " + (i+1));
			os.setParent("TEST PARENT - " + (i+1));
			orderStatuses.add(os);
		}
	}
	private void assertOrderStatuses(int num, OrderStatuses orderStatuses, OrderStatuses savedStatuses) {
		assertEquals(num, savedStatuses.size());
		
		for (int i=0; i<num; i++) {
			assertEquals(orderStatuses.get(i).getIen(), savedStatuses.get(i).getIen());
			assertEquals(orderStatuses.get(i).getName(), savedStatuses.get(i).getName());
			assertEquals(orderStatuses.get(i).getParent(), savedStatuses.get(i).getParent());
		}
	}
}
