package com.agilex.healthcare.facilitydefinition.datalayer.ward;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import javax.annotation.Resource;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.domain.Ward;
import com.agilex.healthcare.mobilehealthplatform.domain.Wards;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class WardDataLayerMongoTest {

	@Resource
	WardDataLayer wardDataLayer;

	@Test
	public void testFetchEmptyWards() {
		Wards wards = wardDataLayer.fetchWards("invalid", "");
		assertEquals(0, wards.size());
	}

	@Test
	public void testFetchWards() {
		Wards wards = wardDataLayer.fetchWards("500", "");
		assertTrue(wards.size() > 0);

		String previousName = "";
		for (Ward ward : wards) {
			assertNotNull(ward.getId());
			assertNotNull(ward.getFacilityCode());
			assertNotNull(ward.getName());
			assertTrue("sorted in ascending order", previousName.compareTo(ward.getName()) < 0);
			previousName = ward.getName();
		}
	}


	@Test
	public void testFetchWardsWithFilter() {
		String searchString = "NHC";
		Wards wards = wardDataLayer.fetchWards("500", searchString);
		assertTrue(wards.size() > 0);

		String previousName = "";
		for (Ward ward : wards) {
			assertTrue(ward.getName().toLowerCase().contains(searchString.toLowerCase()));
			assertNotNull(ward.getId());
			assertNotNull(ward.getFacilityCode());
			assertNotNull(ward.getName());
			assertTrue("sorted in ascending order", previousName.compareTo(ward.getName()) < 0);
			previousName = ward.getName();
		}
	}
}