package com.agilex.healthcare.facilitydefinition.datalayer.clinic;

import com.agilex.healthcare.facilitydefinition.translator.ClinicTranslator;
import com.agilex.healthcare.mobilehealthplatform.domain.Clinic;
import com.agilex.healthcare.mobilehealthplatform.domain.Clinics;
import com.agilex.healthcare.mongo.MongoClientFactory;
import com.agilex.healthcare.mongo.SharedMongoClient;
import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;

public class ClinicDataLayerMongo implements ClinicDataLayer {

	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(ClinicDataLayerMongo.class);

	private static DBCollection collection;
	private static DB db;
	private static String facilityCodeKey = "facility";
	private static String nameKey = "name";

	public ClinicDataLayerMongo(MongoClient mongoClient) {
		try {
			db = mongoClient.getDB("facility-definition");
			collection = db.getCollection("clinics");
		} catch (com.mongodb.MongoException e) {
			LOGGER.error("Unable to connect to Mongo DB - Mongo Exception", e);
			throw new RuntimeException("Unable to connect to Mongo DB - Mongo Exception", e);
		}
	}

    public ClinicDataLayerMongo(String databaseName, String collectionName) {
        SharedMongoClient sharedMongoClient = MongoClientFactory.getInstance().getMongoClient();
        db = sharedMongoClient.getDB(databaseName);
        collection = db.getCollection(collectionName);
        collection.dropIndexes();
    }
    

	/**
	 * Returns the Clinics filtered such that the search string in the filter removes any Clinics whose name does not
	 * contain the search string.  This can be done because we are not actually calling mdws to retrieve any data from
	 * their rest services.  We have previously used a TableSync to populate mongo with this data so we interact with
	 * it as a database instead of being forced to the constraints of how the mdws web services return data.
	 */
	public Clinics fetchClinics(String facilityCode, String searchString) {
		ClinicTranslator translator = new ClinicTranslator();
		ClinicFilterer filterer = new ClinicFilterer();
		Clinics clinics = new Clinics();
		BasicDBObject query = new BasicDBObject(facilityCodeKey, facilityCode);

		DBCursor iterator = collection.find(query).sort(new BasicDBObject(nameKey, 1));

		while(iterator.hasNext()) {
			DBObject dbClinic = iterator.next();
			Clinic clinic = translator.translate(dbClinic);

			if (filterer.matches(clinic, searchString)) {
				clinics.add(clinic);
			}
		}

		return clinics;
	}
}