package com.agilex.healthcare.facilitydefinition.datalayer.clinic;

import com.agilex.healthcare.mobilehealthplatform.domain.Clinics;
import com.agilex.healthcare.mobilehealthplatform.mdws.SearchIndex;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.MhpObjectFactory;

public class ClinicDataService {
	/**
	 * Returns the Clinics starting at searchIndex.
	 * 
	 * @param searchIndex
	 * The searchIndex should be thought of as being identical to looking up a name in a telephone book.
	 * When you do a lookup in a phone book you pick someone's name and then start scanning down the list of names
	 * from there.  When you perform a lookup here and you receive results, it is like opening the phone book and picking
	 * a name in it.  It just selects the first item that matches closest to the one you supplied and then it returns
	 * you the rest of the list of all items from that point to the end of the list.  Thus, this is not filtering
	 * anything but is providing you a way to start your list at a point somewhere within the entire list.<br/><br/>
	 * 
	 * If searchIndex.getSearchString() is null or the empty string, this will return you ONLY the first 44 records.<br/><br/>
	 * 
	 * When it comes to anything that involves pagination in MDWS, if there are more than 44 results, we get back
	 * exactly 44 records. At that point, you should call the exact same MDWS call again passing in the value from the
	 * last record (the 44th record). This would continue until we receive less than 44 records.  Remember the
	 * telephone book analogy - this is like going down 44 records in that list and returning 44 more.<br/><br/>
	 * 
	 * <font color="red"><b>Example</b></font><br/>
	 * Suppose you had the following set of data: {'benefit', 'bladder', 'bones', 'brain'}.<br/>
	 * If you searched for 'bl', you'd get back {'bladder', 'bones', 'brain'}.<br/>
	 * If you searched for 'bm', you'd get back {'bones', 'brain'}.  This is the first index of data that occurs after
	 * what you searched for.<br/><br/>
	 */
	public Clinics fetchClinics(String facilityCode, SearchIndex searchIndex) {
		ClinicDataLayer dataLayer = MhpObjectFactory.getInstance().getMdwsClinicDataLayer();
		String searchString = "";
		if (searchIndex != null && searchIndex.getSearchString() != null)
			searchString = searchIndex.getSearchString();
		Clinics retvalue = dataLayer.fetchClinics(facilityCode, searchString);
		return retvalue;
	}
	
	/**
	 * Returns the Clinics filtered such that the search string in the filter removes any Clinics whose name does not
	 * contain the search string.  This can be done because we are not actually calling mdws to retrieve any data from
	 * their rest services.  We have previously used a TableSync to populate mongo with this data so we interact with
	 * it as a database instead of being forced to the constraints of how the mdws web services return data.
	 */
	public Clinics fetchClinics(String facilityCode, ClinicFilter filter) {
		ClinicDataLayer dataLayer = MhpObjectFactory.getInstance().getClinicDataLayer();
		String searchString = filter.getSearchString();
		Clinics retvalue = dataLayer.fetchClinics(facilityCode, searchString);
		return retvalue;
	}
}
