package com.agilex.healthcare.facilitydefinition.datalayer.specialty;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import com.agilex.healthcare.facilitydefinition.translator.SpecialtyTranslator;
import com.agilex.healthcare.mobilehealthplatform.domain.Specialties;
import com.agilex.healthcare.mobilehealthplatform.domain.Specialty;
import com.agilex.healthcare.mongo.MongoClientFactory;
import com.agilex.healthcare.mongo.SharedMongoClient;
import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;

public class SpecialtyDataLayerMongo implements SpecialtyDataLayer{

    private final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(SpecialtyDataLayerMongo.class);

    private DBCollection collection;
    private DB db;
    private String facilityCodeKey = "facility";
    private String nameKey = "name";
    private SpecialtyTranslator specialtyTranslator = new SpecialtyTranslator();

    public SpecialtyDataLayerMongo(MongoClient mongoClient){
        try {
            db = mongoClient.getDB("facility-definition");
            collection = db.getCollection("specialtyServiceToClinic");
        } catch (com.mongodb.MongoException e) {
            LOGGER.error("Unable to connect to Mongo DB - Mongo Exception", e);
            throw new RuntimeException("Unable to connect to Mongo DB - Mongo Exception", e);
        }
    }

    public SpecialtyDataLayerMongo(String databaseName, String collectionName) {
        SharedMongoClient sharedMongoClient = MongoClientFactory.getInstance().getMongoClient();
        db = sharedMongoClient.getDB(databaseName);
        collection = db.getCollection(collectionName);
        collection.dropIndexes();
    }

    @Override
    public Specialties fetchSpecialties(String facilityCode) {

		BasicDBObject query = new BasicDBObject(facilityCodeKey, facilityCode);
		DBCursor iterator = collection.find(query).sort(new BasicDBObject(nameKey, 1));

        Specialties specialties = new Specialties();
        while(iterator.hasNext()) {
			DBObject specialty = iterator.next();
			specialties.add(specialtyTranslator.translate(specialty));
		}

		return specialties;
    }

    @Override
    public Specialty fetchSpecialty(String facilityCode, String specialtyName){

        BasicDBObject andQuery = new BasicDBObject();
        List<BasicDBObject> obj = new ArrayList<BasicDBObject>();
        obj.add(new BasicDBObject(facilityCodeKey, facilityCode));
        obj.add(new BasicDBObject(nameKey, getSpecialtyRegExp(specialtyName)));
        andQuery.put("$and", obj);

        DBObject specialty = collection.findOne(andQuery);

       return specialtyTranslator.translate(specialty);

    }

    private Pattern getSpecialtyRegExp(String specialtyName){
        String pattern = "^ $";
        if(specialtyName!=null){
            pattern = "^"+specialtyName+"$";
        }
        return Pattern.compile(pattern, Pattern.CASE_INSENSITIVE);
    }



}
