package com.agilex.healthcare.facilitydefinition.datalayer.ward;

import com.agilex.healthcare.facilitydefinition.translator.WardTranslator;
import com.agilex.healthcare.mobilehealthplatform.domain.Ward;
import com.agilex.healthcare.mobilehealthplatform.domain.Wards;
import com.agilex.healthcare.mongo.MongoClientFactory;
import com.agilex.healthcare.mongo.SharedMongoClient;
import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;

public class WardDataLayerMongo implements WardDataLayer {

	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(WardDataLayerMongo.class);

	private static DBCollection collection;
	private static DB db;
	private static String facilityCodeKey = "facility";
	private static String nameKey = "name";

	public WardDataLayerMongo(MongoClient mongoClient) {
		try {
			db = mongoClient.getDB("facility-definition");
			collection = db.getCollection("wards");
		} catch (com.mongodb.MongoException e) {
			LOGGER.error("Unable to connect to Mongo DB - Mongo Exception", e);
			throw new RuntimeException("Unable to connect to Mongo DB - Mongo Exception", e);
		}
	}

    public WardDataLayerMongo(String databaseName, String collectionName) {
        SharedMongoClient sharedMongoClient = MongoClientFactory.getInstance().getMongoClient();
        db = sharedMongoClient.getDB(databaseName);
        collection = db.getCollection(collectionName);
        collection.dropIndexes();
    }

	public Wards fetchWards(String facilityCode, String searchString) {
		BasicDBObject query = new BasicDBObject(facilityCodeKey, facilityCode);
		DBCursor iterator = collection.find(query).sort(new BasicDBObject(nameKey, 1));

		WardTranslator translator = new WardTranslator();
		WardFilterer filterer = new WardFilterer();
		Wards wards = new Wards();
		
		while(iterator.hasNext()) {
			DBObject dbWard = iterator.next();
			Ward ward = translator.translate(dbWard);
			if (filterer.matches(ward, searchString))
				wards.add(ward);
		}

		return wards;
	}
}