package com.agilex.healthcare.mobilehealthplatform.aopaspect;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;

import com.yammer.metrics.Metrics;
import com.yammer.metrics.core.Counter;
import com.yammer.metrics.core.Timer;
import com.yammer.metrics.core.TimerContext;

@Aspect
public class MetricsAspect {

	@Pointcut("execution(* com.agilex.healthcare.mobilehealthplatform.restservice.*.*(..))")
	public void RestServicePointcut() {
	}

	@Around("RestServicePointcut()")
	public Object recordMetrics(ProceedingJoinPoint joinPoint) throws Throwable {
		Class<? extends Object> restendpointClass = joinPoint.getTarget().getClass();
		String eventName = getEventName(joinPoint);

		final TimerContext contextForThisMethod = startTimer(restendpointClass, eventName);
		List<Counter> counters = incrementCounters(restendpointClass, eventName);

		Object result;
		try {
			result = joinPoint.proceed();
		} finally {
			stopTimer(contextForThisMethod);
			decrementCounters(counters);
		}

		return result;
	}

	private List<Counter> incrementCounters(Class<? extends Object> restendpointClass, String eventName) {
		ArrayList<Counter> counters = new ArrayList<Counter>();

		final Counter totalPendingJobsCounter = Metrics.newCounter(MetricsAspect.class, "restendpoint-total-pendingjobs");
		totalPendingJobsCounter.inc();
		counters.add(totalPendingJobsCounter);

		final Counter pendingJobsForThisMethod = Metrics.newCounter(restendpointClass, "restendpoint-" + eventName + "-pendingjobs");
		pendingJobsForThisMethod.inc();
		counters.add(pendingJobsForThisMethod);

		return counters;
	}

	private void decrementCounters(List<Counter> counters) {
		for (Counter counter : counters)
			counter.dec();
	}

	private void stopTimer(final TimerContext contextForThisMethod) {
		contextForThisMethod.stop();
	}

	private TimerContext startTimer(Class<? extends Object> restendpointClass, String eventName) {
		final Timer timerForThisMethod = Metrics.newTimer(restendpointClass, "restendpoint-" + eventName + "-timer", TimeUnit.MILLISECONDS, TimeUnit.SECONDS);
		final TimerContext contextForThisMethod = timerForThisMethod.time();
		return contextForThisMethod;
	}

	private String getEventName(ProceedingJoinPoint joinPoint) {
		return joinPoint.getSignature().getName();
	}
}
