package com.agilex.healthcare.mobilehealthplatform.appointmentrequestmessage;

import com.agilex.healthcare.mobilehealthplatform.datalayer.preference.PatientPreferenceDataService;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientPreference;
import com.agilex.messaging.core.MessageHandler;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Created with IntelliJ IDEA.
 * User: clarkee
 * Date: 2/22/13
 * Time: 10:53 AM
 * To change this template use File | Settings | File Templates.
 */
public class AppointmentRequestMessageCreatedMessageHandler implements MessageHandler<AppointmentRequestMessageCreatedMessage> {
	private final static Log logger = LogFactory.getLog(AppointmentRequestMessageCreatedMessageHandler.class);

    private static String dailyEmailPreference = "Daily";
    private static String neverEmailPreference = "Never";
    private static String eachEmailPreference = "Each new message";
    private EmailDefaults emailDefaults;


    public AppointmentRequestMessageCreatedMessageHandler(EmailDefaults emailDefaults) {
        this.emailDefaults = emailDefaults;
    }

    @Override
    public void handle(AppointmentRequestMessageCreatedMessage appointmentRequestMessageCreatedMessage) {
        //Retrieve patient preferences
//        create new messages for email and sms as appropriate
        logger.debug("Received " + appointmentRequestMessageCreatedMessage.getClass().getName());

        PatientPreferenceDataService preferenceDataService = new PatientPreferenceDataService();
        final PatientPreference patientPreference = preferenceDataService.fetchPatientPreference(appointmentRequestMessageCreatedMessage.getPatientIdentifier());

        try {
            createEmailMessage(patientPreference);
        } catch (MessagePublishException e) {
            logger.error(e.getMessage());
        }

    }

    private void createEmailMessage(PatientPreference patientPreference) throws MessagePublishException {

        if (patientPreference.isEmailAllowed()){
            assert(patientPreference.getEmailAddress() != null);

            String frequency = patientPreference.getNotificationFrequency();
            if (frequency == null || frequency.equals(neverEmailPreference)){
                logger.debug("Email Notification Message not sent due to patient preference");
            }
            else if (frequency.equals(dailyEmailPreference)){
                logger.debug("Creating Email Notification Message for Daily");
                BatchEmailNotificationMessage message = new BatchEmailNotificationMessage(patientPreference.getEmailAddress(), emailDefaults);
                BatchEmailNotificationGateway gateway = MessagingFactory.getBatchEmailNotificationGateway();

                gateway.send(message);
                logger.debug("Email Notification Message sent");
            }
            else if (frequency.equals(eachEmailPreference)){
                logger.debug("Creating Email Notification Message");

                EmailNotificationMessage message = new EmailNotificationMessage(patientPreference.getEmailAddress(), emailDefaults);
                EmailNotificationGateway gateway = MessagingFactory.getEmailNotificationGateway();

                gateway.send(message);
                logger.debug("Email Notification Message sent");
            }
        }
    }
}
