package com.agilex.healthcare.mobilehealthplatform.datalayer.admission;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestHandler;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.ResponseMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataFetchRequestReader;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataResponseBuilder;
import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifiers;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNotes;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.DataSystem;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.DomainServiceRegistry;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.MhpObjectFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;

import java.util.Collection;

public class FetchDischargeSummariesRequestHandler implements RequestHandler {

    @Override
    public ResponseMessage handle(RequestMessage requestMessage) {

        DischargeSummariesDataLayer dataLayer =  getDataLayer(requestMessage);
        PatientDataFetchRequestReader requestReader = PatientDataFetchRequestReader.fromRequest(requestMessage);
        PatientIdentifier patientIdentifier = translatePatientIdentifier(determineDataSystemList(requestReader.getScopeFilter(), requestReader.getDomain()), requestReader.getPatientIdentifier());
        ProviderNotes providerNotes = dataLayer.fetchPatientDischargeSummaries(patientIdentifier, requestReader.getDateFilter());
        return PatientDataResponseBuilder.fromPatientData(providerNotes).build();
    }

    private DischargeSummariesDataLayer getDataLayer(RequestMessage requestMessage) {
        DomainServiceRegistry serviceRegistry = new DomainServiceRegistry();
        return serviceRegistry.getDataLayerByMessage(requestMessage);
    }

    private PatientIdentifier translatePatientIdentifier(Collection<DataSystem> dataSystems, PatientIdentifier patientIdentifier){
        PatientIdentifiers patientIdentifiers = MhpObjectFactory.getInstance().getPatientCorrelationService().getCorrespondIdentifiers(patientIdentifier);
        if(patientIdentifiers != null && patientIdentifiers.size() > 0){
            for (DataSystem targetDataSystem : dataSystems) {
                String targetSystemIdentifier = targetDataSystem.getSystemIdentifier();
                String targetAssigningAuthority = targetDataSystem.getPatientAssigningAuthority();
                if ("vista".equalsIgnoreCase(targetSystemIdentifier)) {
                    MhpUser currentUser = getCurrentUser();
                    targetAssigningAuthority = String.format(targetDataSystem.getPatientAssigningAuthority(), currentUser.getVistaLocation());
                }
                for (PatientIdentifier userIdentifier : patientIdentifiers) {
                    if (userIdentifier.getAssigningAuthority().equals(targetAssigningAuthority)) {
                        return userIdentifier;
                    }
                }
            }
        }
        return patientIdentifier;
    }

    private Collection<DataSystem> determineDataSystemList(ScopeFilter scopeFilter, String domain) {
        DomainServiceRegistry registry = new DomainServiceRegistry();
        return registry.getDataSystems(scopeFilter, domain);
    }

    private MhpUser getCurrentUser(){
        return MhpUserFactory.createFromSecurityContext();
    }
}
