package com.agilex.healthcare.mobilehealthplatform.datalayer.allergy;

import java.util.Collections;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.ResponseMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.Router;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataFetchRequestBuilder;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataResponseReader;
import com.agilex.healthcare.mobilehealthplatform.domain.Allergies;
import com.agilex.healthcare.mobilehealthplatform.domain.Allergy;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.Domain;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;


public class AllergyDataService {

	private Router router;

	public AllergyDataService() {
		this.router = new Router();
	}

	public AllergyDataService(Router router) {
		this.router = new Router();
	}

	public Allergies fetchPatientAllergies(PatientIdentifier patientIdentifier) {
		return fetchPatientAllergies(patientIdentifier, ScopeFilter.getInstanceForLongitudinalScope());
	}

	public Allergies fetchPatientAllergies(PatientIdentifier patientIdentifier, DateFilter dateFilter, ScopeFilter scopeFilter) {
		RequestMessage request = PatientDataFetchRequestBuilder.forRetrieveList().forDomain(Domain.allergy).forPatientIdentifier(patientIdentifier).forDateFilter(dateFilter).forScopeFilter(scopeFilter).build();
		ResponseMessage responseMessage = router.execute(request);
		Allergies allergies = PatientDataResponseReader.<Allergies, Allergy> fromResponse(responseMessage).getDataListNoNull(Allergies.class);
		Collections.sort(allergies);
		return allergies;
	}
	
	/**
	 * Please transition to using the signature with {@link DateFilter}
	 */
	@Deprecated
	public Allergies fetchPatientAllergies(PatientIdentifier patientIdentifier, ScopeFilter scopeFilter) {
		RequestMessage request = PatientDataFetchRequestBuilder.forRetrieveList().forDomain(Domain.allergy).forPatientIdentifier(patientIdentifier).forScopeFilter(scopeFilter).build();
		ResponseMessage responseMessage = router.execute(request);
		Allergies allergies = PatientDataResponseReader.<Allergies, Allergy> fromResponse(responseMessage).getDataListNoNull(Allergies.class);
		Collections.sort(allergies);
		return allergies;
	}
}
