package com.agilex.healthcare.mobilehealthplatform.datalayer.consult;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.ResponseMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.Router;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataResponseReader;
import com.agilex.healthcare.mobilehealthplatform.domain.Consult;
import com.agilex.healthcare.mobilehealthplatform.domain.Consults;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderIdentifier;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.Domain;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.MhpObjectFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;

public class ConsultDataService {

	private Router router;
//    private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(ConsultDataService.class);

	public ConsultDataService() {
		this.router = new Router();
	}

	public ConsultDataService(Router router) {
		this.router = router;
	}

	public Consults fetchPatientConsults(PatientIdentifier patientIdentifier, ConsultFilter filter, ScopeFilter scopeFilter) {
		RequestMessage request = ConsultFetchRequestBuilder.forRetrieveList().forConsultFilter(filter).forDomain(Domain.consult).forPatientIdentifier(patientIdentifier).forScopeFilter(scopeFilter).build();
		ResponseMessage response = router.execute(request);
		Consults consults = PatientDataResponseReader.<Consults, Consult> fromResponse(response).getDataListNoNull(Consults.class);
		Consults.sortConsultsOldestFirst(consults);
		return consults;
	}
	
	public Consults fetchConsultsByProvider(ProviderIdentifier providerIdentifier, ConsultFilter filter) {
		ConsultDataLayer dataLayer = MhpObjectFactory.getInstance().getConsultDataLayer();
		Consults consults = dataLayer.fetchConsultsWrittenByProvider(providerIdentifier, filter);
		Consults.sortConsultsOldestFirst(consults);
		return consults;
	}
	
	public Consults fetchConsultsBySpecialties(ConsultFilter filter, String facilityCode) {
		ConsultDataLayer dataLayer = MhpObjectFactory.getInstance().getConsultDataLayer();
		Consults consults = dataLayer.fetchConsultsBySpecialties(filter, facilityCode);
		Consults.sortConsultsOldestFirst(consults);
		return consults;
	}
}
