package com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestHandler;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.ResponseMessage;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientData;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientDataCollection;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.DomainServiceRegistry;

public abstract class AbstractFetchPatientDataByRetrieveAllAndFilterRequestHandler<TDataLayer, Ts extends PatientDataCollection<T>, T extends PatientData> implements RequestHandler {
	@Override
	public ResponseMessage handle(RequestMessage requestMessage) {
		PatientDataFetchRequestReader messageReader = PatientDataFetchRequestReader.fromRequest(requestMessage);
		TDataLayer datalayer = getDataLayer(requestMessage);
		Ts patientData = invokeDataLayerToFetchList(datalayer, requestMessage, messageReader);

		T match = patientData.findMatchById(messageReader.getDataIdentifier());
		return PatientDataResponseBuilder.fromPatientData(match).build();
	}

	protected abstract Ts invokeDataLayerToFetchList(TDataLayer datalayer, RequestMessage requestMessage, PatientDataFetchRequestReader messageReader);

	@SuppressWarnings("unchecked")
    private TDataLayer getDataLayer(RequestMessage requestMessage) {
		DomainServiceRegistry serviceRegistry = new DomainServiceRegistry();
		return (TDataLayer) serviceRegistry.getDataLayerByMessage(requestMessage);
	}
}
