package com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.ResponseMessage;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderPickListItem;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderPickListsCollection;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.MhpObjectFactory;

public class OrderPickListsResponseBuilder {
	private ResponseMessage response;

	private OrderPickListsResponseBuilder(ResponseMessage response) {
		this.response = response;
	}

	public static OrderPickListsResponseBuilder newInstance() {
		ResponseMessage response = new ResponseMessage();
		return new OrderPickListsResponseBuilder(response);
	}

	public static <Ts extends OrderPickListsCollection<T>, T extends OrderPickListItem> OrderPickListsResponseBuilder fromOrderPickListsValues(Ts dataList) {
		ResponseMessage response = new ResponseMessage();
		OrderPickListsResponseBuilder builder = new OrderPickListsResponseBuilder(response);
		builder.appendOrderPickListsValueData(dataList);
		return builder;
	}

	public <Ts extends OrderPickListsCollection<T>, T extends OrderPickListItem> OrderPickListsResponseBuilder appendOrderPickListsValueData(Ts dataList) {
		if (dataList != null) {

			@SuppressWarnings("unchecked")
            Ts existingDataList = response.getValue("data-list");
			if (existingDataList == null) {
				// note: the need to create a new collection prevents us from
				// modifying the source collection. In case of caching or
				// in-memory
				// mock this can lead to unindented effects
				@SuppressWarnings("unchecked")
				Ts aggregatedCollection = (Ts) MhpObjectFactory.initializeByType(dataList.getClass());
				aggregatedCollection.addAll(dataList);

				response.setValue("data-list", aggregatedCollection);
			} else {
				existingDataList.addAll(dataList);
			}
		}

		return this;
	}

	public ResponseMessage build() {
		return response;
	}

}
