package com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.ResponseMessage;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderStatusValue;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderStatusValueCollection;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.MhpObjectFactory;

import java.util.List;

public class OrderStatusValueResponseBuilder {
	private ResponseMessage response;

	private OrderStatusValueResponseBuilder(ResponseMessage response) {
		this.response = response;
	}

	public static OrderStatusValueResponseBuilder newInstance() {
		ResponseMessage response = new ResponseMessage();
		return new OrderStatusValueResponseBuilder(response);
	}

	public static <Ts extends OrderStatusValueCollection<T>, T extends OrderStatusValue> OrderStatusValueResponseBuilder fromOrderStatusValues(Ts dataList) {
		ResponseMessage response = new ResponseMessage();
		OrderStatusValueResponseBuilder builder = new OrderStatusValueResponseBuilder(response);
		builder.appendOrderStatusValueData(dataList);
		return builder;
	}

	public <Ts extends OrderStatusValueCollection<T>, T extends OrderStatusValue> OrderStatusValueResponseBuilder appendOrderStatusValueData(Ts dataList) {
		if (dataList != null) {

			@SuppressWarnings("unchecked")
            Ts existingDataList = response.getValue("data-list");
			if (existingDataList == null) {
				// note: the need to create a new collection prevents us from
				// modifying the source collection. In case of caching or
				// in-memory
				// mock this can lead to unindented effects
				@SuppressWarnings("unchecked")
				Ts aggregatedCollection = (Ts) MhpObjectFactory.initializeByType(dataList.getClass());
				aggregatedCollection.addAll(dataList);

				response.setValue("data-list", aggregatedCollection);
			} else {
				existingDataList.addAll(dataList);
			}
		}

		return this;
	}

	public ResponseMessage build() {
		return response;
	}

}
