package com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestMessage;
import com.agilex.healthcare.mobilehealthplatform.domain.DataIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.DataSystem;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;
import com.agilex.healthcare.utility.NullChecker;

public class PatientDataFetchRequestBuilder {
	protected RequestMessage request;

	protected PatientDataFetchRequestBuilder(RequestMessage request) {
		this.request = request;
	}

    public static PatientDataFetchRequestBuilder forRetrieveList() {
        RequestMessage request = new RequestMessage();
        PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(request);
        request.setType("FetchListOfPatientData");
        return builder;
    }

    public static PatientDataFetchRequestBuilder forClinics() {
        RequestMessage request = new RequestMessage();
        PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(request);
        request.setType("FetchOrderClinics");
        return builder;
    }

    public static PatientDataFetchRequestBuilder forLocations() {
        RequestMessage request = new RequestMessage();
        PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(request);
        request.setType("FetchOrderLocations");
        return builder;
    }

	public static PatientDataFetchRequestBuilder forRetrieveSingleById() {
		RequestMessage request = new RequestMessage();
		PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(request);
		request.setType("FetchPatientDataById");
		return builder;
	}
	
	public static PatientDataFetchRequestBuilder cloneFromParent(RequestMessage parentRequest) {
		RequestMessage childRequest = parentRequest.createChildCopy();
		return new PatientDataFetchRequestBuilder(childRequest);
	}

	public PatientDataFetchRequestBuilder forType(String type) {
		request.setType(type);
		return this;
	}

	public PatientDataFetchRequestBuilder forPatientIdentifier(PatientIdentifier patientIdentifier) {
		request.setValue("patient-identifier", patientIdentifier);
		return this;
	}

    public PatientDataFetchRequestBuilder forSite(String site) {
        request.setValue("site", site);
        return this;
    }

    public PatientDataFetchRequestBuilder forTarget(String target) {
        request.setValue("target", target);
        return this;
    }

    public PatientDataFetchRequestBuilder forDirection(String direction) {
        request.setValue("direction", direction);
        return this;
    }

    public PatientDataFetchRequestBuilder forProcedureIen(String procedureIen) {
        request.setValue("procedureIen", procedureIen);
        return this;
    }

    public PatientDataFetchRequestBuilder forOrderIen(String orderIen) {
        request.setValue("orderIen", orderIen);
        return this;
    }

    public PatientDataFetchRequestBuilder forDisplayGroup(String displayGroup) {
        request.setValue("displayGroup", displayGroup);
        return this;
    }

    public PatientDataFetchRequestBuilder forCategory(String category) {
        request.setValue("category", category);
        return this;
    }

    public PatientDataFetchRequestBuilder forStartDate(String startDate) {
        request.setValue("start-date", startDate);
        return this;
    }

    public PatientDataFetchRequestBuilder forEndDate(String endDate) {
        request.setValue("end-date", endDate);
        return this;
    }

    public PatientDataFetchRequestBuilder forStatus(String status) {
        request.setValue("status", status);
        return this;
    }

    public PatientDataFetchRequestBuilder forService(String service) {
        request.setValue("service", service);
        return this;
    }

    public PatientDataFetchRequestBuilder forOrderType(String orderType) {
        request.setValue("order-type", orderType);
        return this;
    }

    public PatientDataFetchRequestBuilder forDesiredDateTime(String desiredDateTime) {
        request.setValue("desired-date-time", desiredDateTime);
        return this;
    }

    public PatientDataFetchRequestBuilder forPatientLocation(String patientLocation) {
        request.setValue("patient-location", patientLocation);
        return this;
    }

	public PatientDataFetchRequestBuilder forDomain(String domain) {
		request.setValue("domain", domain);
		return this;
	}

	public PatientDataFetchRequestBuilder forScopeFilter(ScopeFilter scopeFilter) {
		request.setValue("scope-filter", scopeFilter);
		return this;
	}

	public PatientDataFetchRequestBuilder forTargetDataSystem(DataSystem dataSystem) {
		ScopeFilter scopeFilter = ScopeFilter.getInstanceForSystem(dataSystem);
		request.setValue("scope-filter", scopeFilter);
		return this;
	}

	public PatientDataFetchRequestBuilder forDataIdentifier(DataIdentifier dataIdentifier) {
		request.setValue("data-identifier", dataIdentifier);
		return this;
	}

	public PatientDataFetchRequestBuilder includeDetails() {
		request.setValue("include-details", "true");
		return this;
	}

	public PatientDataFetchRequestBuilder excludeDetails() {
		request.setValue("include-details", "false");
		return this;
	}

	public RequestMessage build() {
		if (NullChecker.isNullish(request.getMessageId()))
			throw new IllegalArgumentException("ID is a required field");
		if (NullChecker.isNullish(request.getType()))
			throw new IllegalArgumentException("Type is a required field");
		return request;
	}

	public PatientDataFetchRequestBuilder forDateFilter(DateFilter dateFilter) {
		request.setValue("date-filter", dateFilter);
		return this;
	}

	public PatientDataFetchRequestBuilder setValue(String key, String value) {
		request.setValue(key, value);
		return this;
	}

	public PatientDataFetchRequestBuilder setChildMessageType(String childMessageType) {
		request.setValue("childMessageType", childMessageType);
		return this;
	}

    public static PatientDataFetchRequestBuilder forPatientDischargeSummaries(){
        RequestMessage request = new RequestMessage();
        PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(request);
        request.setType("FetchPatientDischargeSummaries");
        return builder;
    }

    public static PatientDataFetchRequestBuilder forPatientProgressNotes(){
        RequestMessage request = new RequestMessage();
        PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(request);
        request.setType("FetchPatientProgressNotes");
        return builder;
    }

    public static PatientDataFetchRequestBuilder forPatientSurgeryReports(){
        RequestMessage request = new RequestMessage();
        PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(request);
        request.setType("FetchPatientSurgeryReports");
        return builder;
    }

    public static PatientDataFetchRequestBuilder forPatientPathologyReports(){
        RequestMessage request = new RequestMessage();
        PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(request);
        request.setType("FetchPatientPathologyReports");
        return builder;
    }

    public static PatientDataFetchRequestBuilder forOrdersByCustom(){
        RequestMessage request = new RequestMessage();
        PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(request);
        request.setType("FetchOrdersByCustom");
        return builder;
    }

    public static PatientDataFetchRequestBuilder forOrderStatuses() {
        RequestMessage request = new RequestMessage();
        PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(request);
        request.setType("FetchOrderStatuses");
        return builder;
    }

    public static PatientDataFetchRequestBuilder forCategorySupplements() {
        RequestMessage request = new RequestMessage();
        PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(request);
        request.setType("FetchCategorySupplements");
        return builder;
    }
    
    public static PatientDataFetchRequestBuilder forPatientAllReports(){
        RequestMessage requestMessage = new RequestMessage();
        PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(requestMessage);
        requestMessage.setType("FetchAllReportDocumentsForPatient");
        return builder;
    }

    public static PatientDataFetchRequestBuilder forAcceptOrder(){
        RequestMessage requestMessage = new RequestMessage();
        PatientDataFetchRequestBuilder builder = new PatientDataFetchRequestBuilder(requestMessage);
        requestMessage.setType("FetchAcceptOrder");
        return builder;
    }
}
