package com.agilex.healthcare.mobilehealthplatform.datalayer.lab;

import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.ResponseMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.Router;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataFetchRequestBuilder;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataResponseReader;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResult;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResults;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTest;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTestGroups;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTests;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.Domain;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;
import com.agilex.healthcare.utility.NullChecker;

public class ChemistryLabDataService extends AbstractLabDataService {

	protected static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(ChemistryLabDataService.class);
	private Router router;

	public ChemistryLabDataService() {
		this.router = new Router();
	}

	public ChemistryLabDataService(Router router) {
		this.router = router;
	}

	public LabResults getLabResults(PatientIdentifier patientIdentifier, DateFilter dateFilter, ScopeFilter scopeFilter) {

		RequestMessage request = PatientDataFetchRequestBuilder.forRetrieveList().forDomain(Domain.chemLabResult).forPatientIdentifier(patientIdentifier).forScopeFilter(scopeFilter).forDateFilter(dateFilter).build();
		ResponseMessage responseMessage = router.execute(request);
		LabResults labResults = PatientDataResponseReader.<LabResults, LabResult> fromResponse(responseMessage).getDataListNoNull(LabResults.class);
        labResultsGraphable(labResults);
		calculateResultAbnormalityFlag(labResults);
		labResults.sortDescending("resultedDate");

		return labResults;
	}


	public LabResults getLabResults(PatientIdentifier patientIdentifier, String groupingId, DateFilter dateFilter, ScopeFilter scopeFilter) {
		LabResults results = getLabResults(patientIdentifier, dateFilter, scopeFilter);
		return filter(results, groupingId);
	}

	private LabResults filter(LabResults labResults, String groupingId) {
		LabResultsFilter filter = new LabResultsFilter();
		return filter.filter(labResults, groupingId);
	}
	
	public LabTests getLabTestsWithMostRecentResult(PatientIdentifier patientIdentifier, DateFilter dateFilter, ScopeFilter scopeFilter) {
		LabResults labResults = getLabResults(patientIdentifier, dateFilter, scopeFilter);

		LabTests labTests;
		if (NullChecker.isNotNullish(labResults)) {
			// map to store the most recent lab result for a given lab test
			LabTestFilterByMostRecentResult filter = new LabTestFilterByMostRecentResult();
			HashMap<String, LabResult> labResultsIndexedByTestId = filter.getMostRecentResultsByTest(labResults);
			labTests = getLabTestsFromResults(labResultsIndexedByTestId);
		} else {
			labTests = new LabTests();
		}

		return labTests;
	}

	/**
	 * This gets gets the LabTests for MostRecentResult and then calls
	 * extractAndSortLabGroupsByLabTests
	 * 
	 * @see #extractAndSortLabGroupsByLabTests(LabTests)
	 * 
	 * @param patientIdentifier
	 * @return
	 */
	public LabTestGroups getLabTestsInGroups(PatientIdentifier patientIdentifier, DateFilter dateFilter, ScopeFilter scopeFilter) {
		LabTests labTests = getLabTestsWithMostRecentResult(patientIdentifier, dateFilter, scopeFilter);
		LabTestGroups labTestGroups = extractAndSortLabGroupsByLabTests(labTests);

		return labTestGroups;
	}

	private LabTests getLabTestsFromResults(HashMap<String, LabResult> labTestHistory) {
		LabTests labTests = new LabTests();

		for (LabResult result : labTestHistory.values()) {
			LabTest labTest = new LabTest();
			labTest.setMostRecentLabResult(result);
			labTest.setUniqueId(result.getTestId());
			labTest.setDataIdentifier(result.getDataIdentifier().getSystemId(), result.getTestId());
			labTest.setPatientIdentifier(result.getPatientIdentifier());
			labTest.setTestname(result.getTestname());
			labTest.setSpecimenId(result.getSpecimenId());
			labTest.setSpecimenName(result.getSpecimenName());
			labTest.setTestDescription(result.getDisplayDescription());
			labTest.setLoinc(result.getLoinc());

			labTests.add(labTest);
		}
		return labTests;
	}

    // Check if the labResult value field contains only Text
    // if so sets the graphable to false else to true
    private void labResultsGraphable (LabResults labResults) {
        for (LabResult labResult : labResults) {
            String value = labResult.getValue();
            Pattern pattern = Pattern.compile("^[a-zA-Z]+$");
            Matcher matcher = pattern.matcher(value);
            if(matcher.find())
                labResult.setGraphable(false) ;
            else
                labResult.setGraphable(true);
        }
    }
    
    public LabResult getCreatineLabTestInfo(PatientIdentifier patientIdentifier, DateFilter dateFilter, ScopeFilter scopeFilter,String site) {
		LabResults labResults = getLabResults(patientIdentifier, dateFilter, scopeFilter);
		LabResult labResult;
		CreatinineTestDataService dataService = new CreatinineTestDataService();
		int ien = dataService.fetchCreatinineTestLookUpInfo(site);
		if (NullChecker.isNotNullish(labResults)) {
			// map to store the most recent lab result for a given lab test
			LabTestFilterByMostRecentResult filter = new LabTestFilterByMostRecentResult();
			HashMap<String, LabResult> labResultsIndexedByTestId = filter.getMostRecentResultsByTestIrrespectiveOfSpecimen(labResults);
			labResult = labResultsIndexedByTestId.get(String.valueOf(ien));
		} else {
			labResult = new LabResult();
		}
		return labResult;
	}
}
