package com.agilex.healthcare.mobilehealthplatform.datalayer.lab;

import java.util.Date;
import java.util.List;

import com.agilex.healthcare.mdws.connection.EmrConnectionDataLayer;
import com.agilex.healthcare.mdws.connection.EmrMdwsConnection;
import com.agilex.healthcare.mdws.connection.MdwsConfiguration;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.TaggedGraphingItemDataArray;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.TaggedItemForGraphingTypeArray;
import com.agilex.healthcare.mdws.translator.LabTestTranslator;
import com.agilex.healthcare.mobilehealthplatform.domain.DataIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResults;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTest;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTests;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterImpl;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterer;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.MutableDateFilter;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;

public class LabTestsDataLayerVistaImpl extends EmrConnectionDataLayer implements LabTestsDataLayer {
	public final String GRAPHING_TYPE = "63";

	public LabTestsDataLayerVistaImpl(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

	public LabTests getLabTests(PatientIdentifier patientIdentifier, String siteCode, List<String> labCategoryList) {
		EmrMdwsConnection connection = super.getConnection(patientIdentifier.getUniqueId(), siteCode);
		TaggedItemForGraphingTypeArray taggedItemForGraphingTypeArray = connection.getService().getItemsForGraphingType(GRAPHING_TYPE);

		LabTestTranslator translator = new LabTestTranslator();
		LabTests labTests = translator.translateTaggedItemForGraphingTypeArray(patientIdentifier, siteCode, taggedItemForGraphingTypeArray);
		LabTests filteredTests = filterTestsByLabCategory(labCategoryList, labTests);
		return filteredTests;
	}

	private LabTests filterTestsByLabCategory(List<String> labCategoryList, LabTests labTests) {
		LabTests filteredTests = new LabTests();

		if (labCategoryList != null && !labCategoryList.isEmpty()) {
			for (LabTest labTest : labTests) {
				if (labCategoryList.contains(labTest.getAccessionSubscript())) {
					filteredTests.add(labTest);
				}
			}
		} else {
			filteredTests = labTests;
		}

		return filteredTests;
	}

	public LabTest getLabResults(PatientIdentifier patientIdentifier, String labTestIen, String labTestName, String siteCode, String startDate, String endDate) {
		String maxDate = DateHelper.formatDateTimeInVistaFormat(new Date());

		EmrMdwsConnection connection = super.getConnection(patientIdentifier.getUniqueId(), siteCode);
		TaggedGraphingItemDataArray taggedGraphingItemDataArray = connection.getService().getGraphingItemData(GRAPHING_TYPE, labTestIen, maxDate);

		LabTestTranslator translator = new LabTestTranslator();
		LabTest test = new LabTest();

		test.setTestname(labTestName);

		MutableDateFilter dateFilter = new DateFilterImpl();
		if (NullChecker.isNotNullish(startDate)) {
			dateFilter.setStartDate(DateHelper.parseDate(startDate));
		}
		if (NullChecker.isNotNullish(endDate)) {
			dateFilter.setEndDate(DateHelper.parseDate(endDate));
		}

		LabResults labResults = translator.translateTaggedGraphingItemDataArray(patientIdentifier, taggedGraphingItemDataArray, labTestName);
		LabResults filteredLabResults = DateFilterer.filterByDate(labResults, dateFilter);

		test.setLabResults(filteredLabResults);
		test.setSiteCode(siteCode);
		test.setPatientIdentifier(patientIdentifier);
		test.setDataIdentifier(new DataIdentifier(siteCode, labTestIen));
		return test;
	}
}
