package com.agilex.healthcare.mobilehealthplatform.datalayer.medication;

import java.util.Collections;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.ResponseMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.Router;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataFetchRequestBuilder;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataResponseReader;
import com.agilex.healthcare.mobilehealthplatform.domain.DataIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.Medication;
import com.agilex.healthcare.mobilehealthplatform.domain.Medications;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.Domain;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;

public class MedicationDataService {

	private Router router;

	public MedicationDataService() {
		this.router = new Router();
	}

	public MedicationDataService(Router router) {
		this.router = router;
	}

	public Medications getActivePatientMedications(PatientIdentifier patientIdentifier, MedicationFilter filter, ScopeFilter scopeFilter) {
		filter.addActiveStatus(true);
		Medications medications = fetchPatientMedications(patientIdentifier, filter, scopeFilter);

		return medications;
	}
	
	public Medications getActiveAndRecentlyExpiredPatientMedications(PatientIdentifier patientIdentifier, MedicationFilter filter, ScopeFilter scopeFilter) {
		filter.addMedicationSources(MedicationFilter.OUTPATIENT);
		
		Medications medications = getPatientMedicationHistory(patientIdentifier, filter, scopeFilter);

		return medications;
	}
	
	public Medications getPatientMedicationHistory(PatientIdentifier patientIdentifier, MedicationFilter filter, ScopeFilter scopeFilter) {
		filter.addStatus(MedicationFilter.STATUS_ACTIVE);
		filter.addStatus(MedicationFilter.STATUS_EXPIRED);
		
		Medications medications = fetchPatientMedications(patientIdentifier, filter, scopeFilter);

		return medications;
	}

	public Medications fetchPatientMedications(PatientIdentifier patientIdentifier, MedicationFilter filter, ScopeFilter scopeFilter) {
		RequestMessage request = MedicationFetchRequestBuilder.forRetrieveList().forMedicationFilter(filter).forDomain(Domain.medication).forPatientIdentifier(patientIdentifier).forScopeFilter(scopeFilter).build();
		ResponseMessage response = router.execute(request);
		Medications medications = PatientDataResponseReader.<Medications, Medication> fromResponse(response).getDataListNoNull(Medications.class);

		medications = MedicationFilterer.filter(medications, filter);
		Collections.sort(medications);

		return medications;
	}

	public Medication retrieveMedicationDetail(PatientIdentifier patientIdentifier, DataIdentifier dataIdentifier) {
		RequestMessage request = PatientDataFetchRequestBuilder.forRetrieveSingleById().forDomain(Domain.medication).forPatientIdentifier(patientIdentifier).forDataIdentifier(dataIdentifier).build();
		ResponseMessage responseMessage = router.execute(request);
		return PatientDataResponseReader.<Medications, Medication> fromResponse(responseMessage).getDataItem();
	}

}
