package com.agilex.healthcare.mobilehealthplatform.datalayer.order;

import java.util.Collection;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestHandler;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.ResponseMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataFetchRequestReader;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataResponseBuilder;
import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.Orders;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifiers;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.DataSystem;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.DomainServiceRegistry;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.MhpObjectFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;

public class FetchOrdersByCustomRequestHandler implements RequestHandler {
    @Override
    public ResponseMessage handle(RequestMessage requestMessage) {

        OrderDataLayer orderDataLayer = getDataLayer(requestMessage);
        PatientDataFetchRequestReader requestReader = PatientDataFetchRequestReader.fromRequest(requestMessage);
        PatientIdentifier patientIdentifier = translatePatientIdentifier(determineDataSystemList(requestReader.getScopeFilter(), requestReader.getDomain()), requestReader.getPatientIdentifier());

        String startDate = requestMessage.getValueAsString("start-date");
		String endDate = requestMessage.getValueAsString("end-date");
		String status = requestMessage.getValueAsString("status");
		String service = requestMessage.getValueAsString("service");
		Orders orders = orderDataLayer.fetchOrdersByCustom(patientIdentifier, startDate, endDate, status, service);

        ResponseMessage responseMessage = PatientDataResponseBuilder.fromPatientData(orders).build();

        return responseMessage;
    }

    private  OrderDataLayer getDataLayer(RequestMessage requestMessage) {
        DomainServiceRegistry serviceRegistry = new DomainServiceRegistry();
        return serviceRegistry.getDataLayerByMessage(requestMessage);
    }

    private PatientIdentifier translatePatientIdentifier(Collection<DataSystem> dataSystems, PatientIdentifier patientIdentifier){
        PatientIdentifiers patientIdentifiers = MhpObjectFactory.getInstance().getPatientCorrelationService().getCorrespondIdentifiers(patientIdentifier);
        if(patientIdentifiers != null && patientIdentifiers.size() > 0){
            for (DataSystem targetDataSystem : dataSystems) {
                String targetSystemIdentifier = targetDataSystem.getSystemIdentifier();
                String targetAssigningAuthority = targetDataSystem.getPatientAssigningAuthority();
                if ("vista".equalsIgnoreCase(targetSystemIdentifier)) {
                    MhpUser currentUser = getCurrentUser();
                    targetAssigningAuthority = String.format(targetDataSystem.getPatientAssigningAuthority(), currentUser.getVistaLocation());
                }
                for (PatientIdentifier userIdentifier : patientIdentifiers) {
                    if (userIdentifier.getAssigningAuthority().equals(targetAssigningAuthority)) {
                        return userIdentifier;
                    }
                }
            }
        }
        return patientIdentifier;
    }

    private Collection<DataSystem> determineDataSystemList(ScopeFilter scopeFilter, String domain) {
        DomainServiceRegistry registry = new DomainServiceRegistry();
        return registry.getDataSystems(scopeFilter, domain);
    }

    private MhpUser getCurrentUser(){
        return MhpUserFactory.createFromSecurityContext();
    }
}
