package com.agilex.healthcare.mobilehealthplatform.datalayer.order;

import com.agilex.healthcare.mobilehealthplatform.domain.Order;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderResultsHistory;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ValidationError;
import com.agilex.healthcare.mobilehealthplatform.domain.ValidationResult;
import com.agilex.healthcare.mobilehealthplatform.restservice.exception.InvalidOrderException;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.MhpObjectFactory;

public class OrderResultsHistoryDataService {
	private OrderDataLayerMdws orderDataLayerMdws = null;
	
	public OrderResultsHistoryDataService() {
		orderDataLayerMdws = MhpObjectFactory.getInstance().getOrderDataLayerMdws();
	}
	
	public OrderResultsHistory fetchPatientOrderResultHistory(PatientIdentifier patientIdentifier, String orderIen) {
		validate(orderIen);
		OrderResultsHistory retvalue = orderDataLayerMdws.fetchPatientOrderResultHistory(patientIdentifier, orderIen);
		return retvalue;
	}
	
	private void validate(String orderIen) {
		ValidationResult<Order> validationResult = new ValidationResult<Order>();

		validateNotNullIEN(validationResult, "order-ien", orderIen);
		
		if (validationResult.getErrors().size() > 0) {
			throw new InvalidOrderException(validationResult.getErrors());
		}
	}
	
	private void validateNotNullIEN(ValidationResult<Order> validationResult, String fieldName, String value) {
		if (value == null || value.isEmpty()) {
			ValidationError error = new ValidationError();
			error.setErrorMessage("Must not be null or empty.");
			error.setFieldName(fieldName);

			validationResult.addIfNotNull(error);
		}
	}
}
