package com.agilex.healthcare.mobilehealthplatform.datalayer.order;

import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.OrderService;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderServices;
import com.agilex.healthcare.mongo.MongoClientFactory;
import com.agilex.healthcare.mongo.SharedMongoClient;
import com.mongodb.BasicDBList;
import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;

public class OrderServicesDataLayerMongo {

	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(OrderServicesDataLayerMongo.class);

	private static final String SITE_CODE_KEY = "site";
	private static final String LAST_MODIFIED_DATE = "lastModifiedDate";
	private static final String ORDER_SERVICES = "orderServices";
	
	private static final String IEN = "ien";
	private static final String NAME = "name";
	private static final String PARENT = "parent";
	
	private static DBCollection collection;
	private static DB db;
	
	public OrderServicesDataLayerMongo(MongoClient mongoClient) {
		try {
			db = mongoClient.getDB("orders");
			collection = db.getCollection("services");
		} catch (com.mongodb.MongoException e) {
			LOGGER.error("Unable to connect to Mongo DB - Mongo Exception", e);
			throw new RuntimeException("Unable to connect to Mongo DB - Mongo Exception", e);
		}
	}

    public OrderServicesDataLayerMongo(String databaseName, String collectionName) {
        SharedMongoClient sharedMongoClient = MongoClientFactory.getInstance().getMongoClient();
        db = sharedMongoClient.getDB(databaseName);
        collection = db.getCollection(collectionName);
        collection.dropIndexes();
    }

	public OrderServices fetchServices(String siteCode) {
		BasicDBObject query = new BasicDBObject(SITE_CODE_KEY, siteCode);
		DBObject dbObj = collection.findOne(query);
		if (dbObj == null)
			return new OrderServices();
		DBObject dbOrderServices = (DBObject)dbObj.get(ORDER_SERVICES);
		OrderServices orderServices = translate(dbOrderServices);
		return orderServices;
	}
	
	public Date fetchServicesLastUpdate(String siteCode) {
		BasicDBObject query = new BasicDBObject(SITE_CODE_KEY, siteCode);
		DBObject dbObj = collection.findOne(query);
		if (dbObj == null)
			return null;
		Date lastModifiedDate = (Date)dbObj.get(LAST_MODIFIED_DATE);
		return lastModifiedDate;
	}
	
	public void updateServices(String siteCode, OrderServices orderServices) {
		BasicDBObject query = new BasicDBObject(SITE_CODE_KEY, siteCode);
		
		BasicDBObject dbObj = new BasicDBObject();
		dbObj.put(SITE_CODE_KEY, siteCode);
		dbObj.put(LAST_MODIFIED_DATE, Calendar.getInstance().getTime());
		DBObject orderServicesDbObj = translate(orderServices);
		dbObj.put(ORDER_SERVICES, orderServicesDbObj);
		collection.update(query, dbObj, true, false);
	}
	
	private DBObject translate(OrderServices orderServices) {
		BasicDBList dbList = new BasicDBList();
		for (int i=0; i<orderServices.size(); i++) {
			OrderService os = orderServices.get(i);
			String ien = os.getIen();
			String name = os.getName();
			String parent = os.getParent();
			
			BasicDBObject dbObj = new BasicDBObject();
			dbObj.put(IEN, ien);
			dbObj.put(NAME, name);
			dbObj.put(PARENT, parent);
			dbList.add(dbObj);
		}
		return dbList;
	}
	
	private OrderServices translate(DBObject dbObj) {
		OrderServices retvalue = new OrderServices();
		Iterator<Object> it = ((BasicDBList)dbObj).iterator();
		while (it.hasNext()) {
		    BasicDBObject po = (BasicDBObject) it.next();
		    String ien = po.getString(IEN);
			String name = po.getString(NAME);
			String parent = po.getString(PARENT);
			
			OrderService os = new OrderService();
			os.setIen(ien);
			os.setName(name);
			os.setParent(parent);
			retvalue.add(os);
		}
		return retvalue;
	}
}