package com.agilex.healthcare.mobilehealthplatform.datalayer.order;

import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.OrderStatusValue;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderStatuses;
import com.agilex.healthcare.mongo.MongoClientFactory;
import com.agilex.healthcare.mongo.SharedMongoClient;
import com.mongodb.BasicDBList;
import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;

public class OrderStatusesDataLayerMongo {

	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(OrderStatusesDataLayerMongo.class);

	private static final String SITE_CODE_KEY = "site";
	private static final String LAST_MODIFIED_DATE = "lastModifiedDate";
	private static final String ORDER_STATUSES = "statuses";
	
	private static final String IEN = "ien";
	private static final String NAME = "name";
	private static final String PARENT = "parent";
	
	private static DBCollection collection;
	private static DB db;
	
	public OrderStatusesDataLayerMongo(MongoClient mongoClient) {
		try {
			db = mongoClient.getDB("orders");
			collection = db.getCollection("statuses");
		} catch (com.mongodb.MongoException e) {
			LOGGER.error("Unable to connect to Mongo DB - Mongo Exception", e);
			throw new RuntimeException("Unable to connect to Mongo DB - Mongo Exception", e);
		}
	}

    public OrderStatusesDataLayerMongo(String databaseName, String collectionName) {
        SharedMongoClient sharedMongoClient = MongoClientFactory.getInstance().getMongoClient();
        db = sharedMongoClient.getDB(databaseName);
        collection = db.getCollection(collectionName);
        collection.dropIndexes();
    }

	public OrderStatuses fetchStatuses(String siteCode) {
		BasicDBObject query = new BasicDBObject(SITE_CODE_KEY, siteCode);
		DBObject dbObj = collection.findOne(query);
		if (dbObj == null)
			return new OrderStatuses();
		DBObject dbOrderStatuses = (DBObject)dbObj.get(ORDER_STATUSES);
		OrderStatuses orderStatuses = translate(dbOrderStatuses);
		return orderStatuses;
	}
	
	public Date fetchStatusesLastUpdate(String siteCode) {
		BasicDBObject query = new BasicDBObject(SITE_CODE_KEY, siteCode);
		DBObject dbObj = collection.findOne(query);
		if (dbObj == null)
			return null;
		Date lastModifiedDate = (Date)dbObj.get(LAST_MODIFIED_DATE);
		return lastModifiedDate;
	}
	
	public void updateStatuses(String siteCode, OrderStatuses orderStatuses) {
		BasicDBObject query = new BasicDBObject(SITE_CODE_KEY, siteCode);
		
		BasicDBObject dbObj = new BasicDBObject();
		dbObj.put(SITE_CODE_KEY, siteCode);
		dbObj.put(LAST_MODIFIED_DATE, Calendar.getInstance().getTime());
		DBObject orderStatusesDbObj = translate(orderStatuses);
		dbObj.put(ORDER_STATUSES, orderStatusesDbObj);
		collection.update(query, dbObj, true, false);
	}
	
	private DBObject translate(OrderStatuses orderStatuses) {
		BasicDBList dbList = new BasicDBList();
		for (int i=0; i<orderStatuses.size(); i++) {
			OrderStatusValue os = orderStatuses.get(i);
			String ien = os.getIen();
			String name = os.getName();
			String parent = os.getParent();
			
			BasicDBObject dbObj = new BasicDBObject();
			dbObj.put(IEN, ien);
			dbObj.put(NAME, name);
			dbObj.put(PARENT, parent);
			dbList.add(dbObj);
		}
		return dbList;
	}
	
	private OrderStatuses translate(DBObject dbObj) {
		OrderStatuses retvalue = new OrderStatuses();
		Iterator<Object> it = ((BasicDBList)dbObj).iterator();
		while (it.hasNext()) {
		    BasicDBObject po = (BasicDBObject) it.next();
		    String ien = po.getString(IEN);
			String name = po.getString(NAME);
			String parent = po.getString(PARENT);
			
			OrderStatusValue os = new OrderStatusValue();
			os.setIen(ien);
			os.setName(name);
			os.setParent(parent);
			retvalue.add(os);
		}
		return retvalue;
	}
}