package com.agilex.healthcare.mobilehealthplatform.datalayer.order;

import java.util.Calendar;
import java.util.Date;

import agilex.commons.helper.PropertyHelper;

import com.agilex.healthcare.mobilehealthplatform.domain.OrderStatuses;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.MhpObjectFactory;

public class OrderStatusesDataService {
	private OrderDataLayerMdws orderDataLayerMdws = null;
	private OrderStatusesDataLayerMongo orderStatusesDataLayerMongo = null;
	private int cacheInDays = 30;
	
	public OrderStatusesDataService() {
		orderDataLayerMdws = MhpObjectFactory.getInstance().getOrderDataLayerMdws();
		orderStatusesDataLayerMongo = MhpObjectFactory.getInstance().getOrderStatusesDataLayerMongo();
		
		PropertyHelper propertyHelper = MhpObjectFactory.getInstance().getPropertyHelper();
		//If this is set to -1, then we won't cache or use cache with Mongo but will query MDWS directly every time.
		cacheInDays = Integer.parseInt(propertyHelper.getProperty("order_statuses_days_to_cache"));
	}
	
	/**
	 * Calls the MDWS service OrderMgmtSvc.getOrdersStatuses<br/>
	 * If the property order_statuses_days_to_cache is -1, we will no longer cache the data in Mongo.  Otherwise, after
	 * retrieving this data from MDWS we cache it in Mongo.  We then compare the dates of every call to what we have
	 * stored as the last cached date in Mongo.  If the timespan between today and this last cached date is less than
	 * the property we give them the data from Mongo.  If it's more, we retrieve the data from MDWS, update Mongo, and
	 * give them the new data.
	 */
	public OrderStatuses fetchOrderStatuses(String siteCode) {
		boolean refresh = checkIfNDaysOld(siteCode);
		
		if (refresh == false) {
			OrderStatuses cachedOrderServices = orderStatusesDataLayerMongo.fetchStatuses(siteCode);
			return cachedOrderServices;
		}
		
		OrderStatuses freshOrderServices = orderDataLayerMdws.fetchOrderStatuses(siteCode);
		if (cacheInDays != -1)
			orderStatusesDataLayerMongo.updateStatuses(siteCode, freshOrderServices);
		return freshOrderServices;
	}

	private boolean checkIfNDaysOld(String siteCode) {
		if (cacheInDays == -1)
			return true;
		
		boolean refresh = true;
		Date lastCached = orderStatusesDataLayerMongo.fetchStatusesLastUpdate(siteCode);
		
		//Compare N days ago to when we last cached the data in Mongo.
		//If the last time we cached in mongo is after N days ago, then we don't need to refresh the data.
		if (lastCached != null) {
			Calendar cal = Calendar.getInstance();
			cal.add(Calendar.DATE, cacheInDays*-1);
			Date nDaysAgo = cal.getTime();
			if (lastCached.after(nDaysAgo))
				refresh = false;
		}
		
		return refresh;
	}
}
