package com.agilex.healthcare.mobilehealthplatform.datalayer.pathology;


import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.AbstractFetchPatientDataListRequestHandler;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataFetchRequestReader;
import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.DataSystem;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.DomainServiceRegistry;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.MhpObjectFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;

import java.util.Collection;

public class FetchPathologyReportsRequestHandler extends AbstractFetchPatientDataListRequestHandler<PathologyDataLayer, PathologyReports, PathologyReport>{

    @Override
    protected PathologyReports invokeDataLayerToFetchList(PathologyDataLayer datalayer, RequestMessage requestMessage, PatientDataFetchRequestReader messageReader) {
        PatientIdentifier patientIdentifier = translatePatientIdentifier(determineDataSystemList(messageReader.getScopeFilter(), messageReader.getDomain()), messageReader.getPatientIdentifier());
        return datalayer.getPatientPathologyReports(patientIdentifier, messageReader.getDateFilter());
    }

    private PatientIdentifier translatePatientIdentifier(Collection<DataSystem> dataSystems, PatientIdentifier patientIdentifier){
        PatientIdentifiers patientIdentifiers = MhpObjectFactory.getInstance().getPatientCorrelationService().getCorrespondIdentifiers(patientIdentifier);
        if(patientIdentifiers != null && patientIdentifiers.size() > 0){
            for (DataSystem targetDataSystem : dataSystems) {
                String targetSystemIdentifier = targetDataSystem.getSystemIdentifier();
                String targetAssigningAuthority = targetDataSystem.getPatientAssigningAuthority();
                if ("vista".equalsIgnoreCase(targetSystemIdentifier)) {
                    MhpUser currentUser = getCurrentUser();
                    targetAssigningAuthority = String.format(targetDataSystem.getPatientAssigningAuthority(), currentUser.getVistaLocation());
                }
                for (PatientIdentifier userIdentifier : patientIdentifiers) {
                    if (userIdentifier.getAssigningAuthority().equals(targetAssigningAuthority)) {
                        return userIdentifier;
                    }
                }
            }
        }
        return patientIdentifier;
    }

    private Collection<DataSystem> determineDataSystemList(ScopeFilter scopeFilter, String domain) {
        DomainServiceRegistry registry = new DomainServiceRegistry();
        return registry.getDataSystems(scopeFilter, domain);
    }

    private MhpUser getCurrentUser(){
        return MhpUserFactory.createFromSecurityContext();
    }
}
