package com.agilex.healthcare.mobilehealthplatform.datalayer.patient;

import com.agilex.healthcare.mobilehealthplatform.datalayer.scheduling.SchedulingDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientSearchResults;
import com.agilex.healthcare.mobilehealthplatform.domain.Patients;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.MhpObjectFactory;
import com.agilex.healthcare.utility.NullChecker;
import com.agilex.healthcare.utility.PatientUtility;

public class PatientClinicDataServiceMDWS {

	private final static org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(PatientClinicDataServiceMDWS.class);

	private SchedulingDataLayerMdws schedulingDataLayerMdws = null;
	
	public PatientClinicDataServiceMDWS() {
		schedulingDataLayerMdws = MhpObjectFactory.getInstance().getSchedulingSvcDataLayerMdws();
	}
	
	/**
	 * 
	 * @param siteCode The ID for the site.
	 * @param visitDate A Date in yyyyMMdd format
	 * @param pagingInfo
	 * @param scopeFilter
	 * @return
	 */
	public PatientSearchResults patientSearchByClinic(PatientSearchCriteria patientSearchCriteria, String siteCode, PatientSearchPaging pagingInfo) {
		String clinicId = patientSearchCriteria.getClinic();
		String visitDate = patientSearchCriteria.getVisitDate();
		
		if (NullChecker.isNotNullish(clinicId)) {
			logger.info("performing patient search by clinic/date [" + clinicId + "/" + visitDate + "]");
		}
		
		//yyyyMMdd.HHmmss (.HHmmss portion can be left out if desired)
		String startDate = visitDate + ".000000";
		String stopDate = visitDate + ".235959";
		Patients patients = schedulingDataLayerMdws.getPatientsByClinic(siteCode, clinicId, startDate, stopDate);
		
		PatientSearchCriteria criteria = PatientDataService.populateCriteriaFromSearchString(patientSearchCriteria);
		Patients translatedPatients = PatientUtility.filter(patients, criteria);
		
		if(translatedPatients == null) {
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_PATIENT, "Translated Patients is null.");
		}

		Patients filteredPatients = PatientDataLayerMdws.applyPostNameFiltering(criteria, translatedPatients);
		
		PatientFormatter formatter = new PatientFormatter();
		formatter.formatPatients(filteredPatients);
		formatter.sort(filteredPatients);
		
		PatientSearchResults searchResults = enforceMaxRecords(filteredPatients, pagingInfo);
		return searchResults;
	}

	private PatientSearchResults enforceMaxRecords(Patients patients, PatientSearchPaging pagingInfo) {
		Paging paging = new Paging();
		return paging.enforceMaxRecords(patients, pagingInfo);
	}

	
}
