package com.agilex.healthcare.mobilehealthplatform.datalayer.vital;

import java.util.Collection;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.*;
import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.*;

public class FetchVitalEntriesRequestHandler extends AbstractFetchPatientDataListRequestHandler<VitalDataLayer, VitalEntries, VitalEntry>{
	
	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory
			.getLog(FetchVitalEntriesRequestHandler.class);
	

	@Override
	protected VitalEntries invokeDataLayerToFetchList(VitalDataLayer datalayer, RequestMessage requestMessage, PatientDataFetchRequestReader messageReader) {
		PatientIdentifier translatedIdentifier = translatePatientIdentifier(requestMessage);
		
		VitalEntries vitalEntries = null;
		if(translatedIdentifier == null){
			vitalEntries = new VitalEntries();
		}else{
			vitalEntries = datalayer.fetchVitalEntries(translatedIdentifier, getFilter(requestMessage));
			fixIdentifiers(vitalEntries, translatedIdentifier);
		}
		return vitalEntries;
	}
	
	private VitalEntryFilter getFilter(RequestMessage request) {
		return request.getValue("vitals-filter");
	}
	
	private ScopeFilter getScopeFilter(RequestMessage request) {
		return request.getValue("scope-filter");
	}
	
	private String getDomain(RequestMessage request) {
		return request.getValueAsString("domain");
	}
	
	private void fixIdentifiers(VitalEntries vitalEntries, PatientIdentifier patientIdentifier) {
		for(VitalEntry entry : vitalEntries) {
			entry.setPatientIdentifier(patientIdentifier);
		}
	}

	private Collection<DataSystem> determineDataSystemList(RequestMessage requestMessage) {
		DomainServiceRegistry registry = new DomainServiceRegistry();
		return registry.getDataSystems(getScopeFilter(requestMessage), getDomain(requestMessage));
	}
	
	private PatientIdentifier translatePatientIdentifier(RequestMessage requestMessage) {
		PatientIdentifier patientIdentifier = requestMessage.getValue("patient-identifier");
		
		Collection<DataSystem> dataSystems = determineDataSystemList(requestMessage);
		
		for (DataSystem targetDataSystem : dataSystems) {
			
			String targetAssigningAuthority = targetDataSystem.getPatientAssigningAuthority();
			
			if (targetDataSystem.getSystemIdentifier().equalsIgnoreCase("vista")){
				MhpUser currentUser = getCurrentUser();
				targetAssigningAuthority = String.format(targetDataSystem.getPatientAssigningAuthority(),currentUser.getVistaLocation()); 
			} 
			
			for (PatientIdentifier targetPatientIdentifier : translatePatientIdentifier(patientIdentifier,
					targetAssigningAuthority)) {
				
				return targetPatientIdentifier;
			}
		}
		
		return null;
	}
	
	private MhpUser getCurrentUser(){
		return MhpUserFactory.createFromSecurityContext();
	}
	
	private PatientIdentifiers translatePatientIdentifier(PatientIdentifier patientIdentifier, String targetAssigningAuthority) {
		
		PatientIdentifiers matcingPatientIdentifiers = new PatientIdentifiers();

		if(targetAssigningAuthority.equals(patientIdentifier.getAssigningAuthority())){
			logger.debug(String.format("No translations are neccessary as patient's assigning authority %s and target assigning authority %s are same for the patient identifier %s", 
					patientIdentifier.getAssigningAuthority(), targetAssigningAuthority, patientIdentifier));
			matcingPatientIdentifiers.add(patientIdentifier);
			return matcingPatientIdentifiers;
		}
		
		// Get all the identifiers for given patient identifier 
		PatientIdentifiers correspondingIdentifiers = MhpObjectFactory.getInstance().getPatientCorrelationService()
				.getCorrespondIdentifiers(patientIdentifier);
		
		
		// Get all the identifiers matching the target assigning authority
		for (PatientIdentifier possibleMatch : correspondingIdentifiers) {
			if (possibleMatch.getAssigningAuthority().toLowerCase().contains(targetAssigningAuthority.toLowerCase())) {
				matcingPatientIdentifiers.add(possibleMatch);
			}
		}
		
		logger.debug(String.format("translated patient identifier %s for assigning authority %s into %s corresponding identifiers", patientIdentifier,
				targetAssigningAuthority, matcingPatientIdentifiers.size()));
		
		return matcingPatientIdentifiers;
	}
	
	
}
