package com.agilex.healthcare.mobilehealthplatform.datalayer.vital;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.*;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.*;
import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.*;

public class VitalDataService {

	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(VitalDataService.class);
	private Router router;

	public VitalDataService() {
		this.router = new Router();
	}

	public VitalDataService(Router router) {
		this.router = router;
	}

	public VitalEntries fetchVitalEntries(PatientIdentifier patientIdentifier, VitalEntryFilter vitalFilter, ScopeFilter scopeFilter) {
		logger.debug(String.format("Retieving VitalEntries [patientIdentifier: %s][filter: %s]", patientIdentifier, vitalFilter));
		
		RequestMessage request = VitalEntryRequestBuilder.forVitalEntries().forDomain(Domain.vital).forFilter(vitalFilter).forPatientIdentifier(patientIdentifier).forScope(scopeFilter).build();
		ResponseMessage response = router.execute(request);
		VitalEntries vitalEntries = PatientDataResponseReader.<VitalEntries, VitalEntry>fromResponse(response).getDataList();
		vitalEntries.sortDescending("entryDate");
		
		return vitalEntries;
	}
	
	public VitalEntry fetchVitalEntry(PatientIdentifier patientIdentifier, DataIdentifier dataIdentifier, ScopeFilter scopeFilter) {
		logger.debug(String.format("Retieving a single VitalEntry [patientIdentifier: %s][dataIdentifier: %s]", patientIdentifier, dataIdentifier));
		
		RequestMessage request = VitalEntryRequestBuilder.forVitalEntry().forDomain(Domain.vital).forDataIdentifier(dataIdentifier).forPatientIdentifier(patientIdentifier).forScope(scopeFilter).build();
		ResponseMessage response = router.execute(request);
		VitalEntry vitalEntry = PatientDataResponseReader.<VitalEntries, VitalEntry>fromResponse(response).getDataItem();
		
		return vitalEntry;
	}

}
