package com.agilex.healthcare.mobilehealthplatform.restservice.exception;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;
import javax.ws.rs.ext.ExceptionMapper;
import javax.ws.rs.ext.Provider;

import com.agilex.healthcare.mdws.datalayer.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.domain.ExternalSystemError;

@Provider
public class MdwsResponseExceptionEncounterMDWSMapper implements ExceptionMapper<MdwsResponseException> {
	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(MdwsResponseExceptionEncounterMDWSMapper.class);
	
	@Override
	public Response toResponse(MdwsResponseException ex) {
		MdwsResponseException.MdwsResponseExceptionLogType typeOfLog = MdwsResponseException.logException(logger, ex);
		
		ExternalSystemError error = new ExternalSystemError();
		error.setMessage(ex.getMessage());
		
		//Only give the end user a UUID if it is actually being logged to a file.
		boolean setUuid = false;
		if (MdwsResponseException.MdwsResponseExceptionLogType.ERROR == typeOfLog && logger.isErrorEnabled())
			setUuid = true;
		else if (MdwsResponseException.MdwsResponseExceptionLogType.WARN == typeOfLog && logger.isWarnEnabled())
			setUuid = true;
		else if (MdwsResponseException.MdwsResponseExceptionLogType.INFO == typeOfLog && logger.isInfoEnabled())
			setUuid = true;
		else if (MdwsResponseException.MdwsResponseExceptionLogType.DEBUG == typeOfLog && logger.isDebugEnabled())
			setUuid = true;
		
		if (setUuid)
			error.setUuid(ex.getUuid());
		
		error.setDetailedDescription(ex.getExpandedMessage());
		
		Response response;
		switch (ex.getUserMessage()) {
			case CONNECTIONS_NOT_READY:
				response = Response.status(Status.FORBIDDEN).entity(error).type(MediaType.APPLICATION_JSON).build(); //403 error
			break;
				
			case NOT_AUTHORIZED:
			case INVALID_ESIG:
			case INVALID_ACCESS_VERIFY:
			case INVALID_PATIENT:
			case INVALID_INPUT:
			case INVALID_FACILITY_CODE:
			case INVALID_LOCATION_IEN:
			case INVALID_NOTE_TITLE_IEN:
			case INVALID_CPT_CODE:
			case INVALID_ICD_CODE:
			case MISSING_CPT_CODE:
			case MISSING_ICD_CODE:
			case INVALID_VISIT_RELATED_TOS:
			case INVALID_NOTE_BODY:
			case INVALID_PROVIDER_IEN:
			case INVALID_RECORD_ID:
			case INVALID_SERVICE_CATEGORY:
			case INVALID_INPATIENT:
			case INVALID_DATE:
			case INVALID_PROCEDURES:
			case INVALID_COMMENT:
			case INVALID_CONSULT_IEN:
			case INVALID_DIAGNOSES:
			case INVALID_HEALTH_FACTORS:
				response = Response.status(Status.BAD_REQUEST).entity(error).type(MediaType.APPLICATION_JSON).build(); //400 error
			break;
			
			case INVALID_DATA_RETURNED:
			case INVALID_WSDL:
			case CONNECTION_ERROR:
			case GENERIC_ERROR:
			case TIMEOUT:
			default:
				response = Response.status(Status.INTERNAL_SERVER_ERROR).entity(error).type(MediaType.APPLICATION_JSON).build(); //500 error
			break;
		}
		
		return response;
	}
}
