package com.agilex.healthcare.mobilehealthplatform.serviceregistry;

import agilex.commons.helper.PropertyHelper;

import com.agilex.healthcare.facilitydefinition.datalayer.clinic.ClinicDataLayer;
import com.agilex.healthcare.facilitydefinition.datalayer.specialty.SpecialtyDataLayer;
import com.agilex.healthcare.facilitydefinition.datalayer.ward.WardDataLayer;
import com.agilex.healthcare.mdws.datalayer.EncounterDataLayer;
import com.agilex.healthcare.mdws.datalayer.TIUNotesDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.admission.AdmissionDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.allergy.AllergyDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentMetricsDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentRequestDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentRequestInProcessDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.DetailCodeDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.audit.AuditLogDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.calendar.CalendarEventDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.calendar.CalendarKeyDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.contactlog.ContactLogDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.document.DocumentDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.facility.FacilityDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.facility.FacilityInternalDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.feedback.UserFeedbackDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.lab.ChemistryLabDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.lab.CreatinineTestDataLayerMongo;
import com.agilex.healthcare.mobilehealthplatform.datalayer.lab.micro.MicrobiologyLabDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.launchpad.LaunchpadItemDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.medication.MedicationDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.medication.MedicationRefillDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.medication.MedicationRefillRequestDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.metric.MetricDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.notification.DeviceRegistrationDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.notification.NotificationCommunicationDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.notification.NotificationDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.order.OrderDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.order.OrderServicesDataLayerMongo;
import com.agilex.healthcare.mobilehealthplatform.datalayer.order.OrderStatusesDataLayerMongo;
import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientInternalDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientMetadataDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.preference.PatientPreferenceDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.problem.ProblemDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.radiology.RadiologyDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.rightofaccess.RightOfAccessDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.scheduling.SchedulingDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.securemessage.SecureMessageDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.staffdirectory.StaffDirectoryDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.staffuserdisclaimer.StaffUserDisclaimerDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.surgery.SurgeryDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vital.VitalDataLayer;
import com.agilex.healthcare.mobilehealthplatform.domain.DataSourceSystem;
import com.agilex.healthcare.notewriter.datalayer.ClinicalEventDataLayer;
import com.agilex.healthcare.notewriter.datalayer.FacilityProviderDataLayer;
import com.agilex.healthcare.notewriter.datalayer.NoteTitleDataLayer;
import com.agilex.healthcare.patientsearch.datalayer.PatientSearchDataLayer;

public interface DataLayerFactory {
	PatientDataLayer getPatientDataLayer();

	AllergyDataLayer getAllergyDataLayer();

	DocumentDataLayer getDocumentDataLayer();

	ProblemDataLayer getProblemDataLayer();

	MedicationDataLayer getMedicationDataLayer();

	MedicationRefillDataLayer getMedicationRefillDataLayer();

	VitalDataLayer getVitalDataLayer(String source);

	ChemistryLabDataLayer getChemistryLabDataLayer();

	MicrobiologyLabDataLayer getMicrobiologyLabDataLayer();

	ContactLogDataLayer getContactLogDataLayer();

	AppointmentDataLayer getAppointmentDataLayer();

	AppointmentRequestDataLayer getAppointmentRequestDataLayer();

	AppointmentMetricsDataLayer getAppointmentMetricsDataLayer();

	PatientPreferenceDataLayer getPatientPreferenceDataLayer();

	AdmissionDataLayer getAdmissionDataLayer();

	SurgeryDataLayer getSurgeryDataLayer();

	RadiologyDataLayer getRadiologyDataLayer();

	NotificationDataLayer getNotificationDataLayer();

	NotificationCommunicationDataLayer getNotificationCommunicationDataLayer();

	DeviceRegistrationDataLayer getDeviceRegistrationDataLayer();

	CalendarEventDataLayer getCalendarEventDataLayer();

	CalendarKeyDataLayer getCalendarKeyDataLayer();

	SecureMessageDataLayer getSecureMessageDataLayer();

	RightOfAccessDataLayer getRightOfAccessDataLayer();

	AuditLogDataLayer getAuditLogDataLayer();

	UserFeedbackDataLayer getUserFeedbackDataLayer();

	PatientMetadataDataLayer getPatientMetadataDataLayer();

	FacilityDataLayer getFacilityDataLayer();

	DetailCodeDataLayer getDetailCodeDataLayer();

	FacilityInternalDataLayer getFacilityInternalDataLayer();

	PatientInternalDataLayer getPatientInternalDataLayer();

	AppointmentRequestInProcessDataLayer getAppointmentRequestInProcessDataLayer();

	MetricDataLayer getMetricDataLayer();

	LaunchpadItemDataLayer getLaunchpadItemDataLayer();

	StaffUserDisclaimerDataLayer getStaffUserDisclaimerDataLayer();

	StaffDirectoryDataLayer getStaffDirectoryDataLayer();


	ClinicDataLayer getMdwsClinicDataLayer();
	ClinicDataLayer getClinicDataLayer();

	WardDataLayer getMdwsWardDataLayer();
	WardDataLayer getWardDataLayer();

	ConsultDataLayer getConsultDataLayer();

	PatientSearchDataLayer getPatientSearchDataLayer();

	ClinicalEventDataLayer getClinicalEventDataLayer();
	NoteTitleDataLayer getNoteTitleDataLayer();

	SpecialtyDataLayer getSpecialtyDataLayer();
    SpecialtyDataLayer getSpecialtyDataLayer(DataSourceSystem dataSource);

    EncounterDataLayer getMdwsEncounterDataLayer();
    TIUNotesDataLayer getMdwsTIUNotesDataLayer();

    MedicationRefillRequestDataLayer getMedicationRefillRequestDataLayer();

    FacilityProviderDataLayer getFacilityProviderDataLayer();

    SchedulingDataLayerMdws getSchedulingSvcDataLayerMdws();
    
    public AppointmentDataLayerMdws getAppointmentDataLayerMdws();

    CreatinineTestDataLayerMongo getCreatinineTestDataLayer();
    public OrderDataLayerMdws getOrderDataLayerMdws();

	OrderServicesDataLayerMongo getOrderServicesDataLayerMongo();
	OrderStatusesDataLayerMongo getOrderStatusesDataLayerMongo();

	PropertyHelper getPropertyHelper();
}
