 package com.agilex.healthcare.mobilehealthplatform.serviceregistry;

import java.util.HashMap;
import java.util.List;

import com.agilex.healthcare.mdws.datalayer.TIUNotesDataLayerMdws;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import agilex.commons.helper.PropertyHelper;

import com.agilex.healthcare.facilitydefinition.datalayer.clinic.ClinicDataLayer;
import com.agilex.healthcare.facilitydefinition.datalayer.specialty.SpecialtyDataLayer;
import com.agilex.healthcare.facilitydefinition.datalayer.ward.WardDataLayer;
import com.agilex.healthcare.mdws.datalayer.EncounterDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.admission.AdmissionDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.allergy.AllergyDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentMetricsDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentRequestDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentRequestInProcessDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.DetailCodeDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.audit.AuditLogDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.calendar.CalendarEventDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.calendar.CalendarKeyDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.contactlog.ContactLogDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.document.DocumentDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.facility.FacilityDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.facility.FacilityInternalDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.feedback.UserFeedbackDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.lab.ChemistryLabDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.lab.CreatinineTestDataLayerMongo;
import com.agilex.healthcare.mobilehealthplatform.datalayer.lab.micro.MicrobiologyLabDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.launchpad.LaunchpadItemDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.medication.MedicationDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.medication.MedicationRefillDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.medication.MedicationRefillRequestDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.metric.MetricDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.notification.DeviceRegistrationDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.notification.NotificationCommunicationDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.notification.NotificationDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.order.OrderDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.order.OrderServicesDataLayerMongo;
import com.agilex.healthcare.mobilehealthplatform.datalayer.order.OrderStatusesDataLayerMongo;
import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientInternalDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientMetadataDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.preference.PatientPreferenceDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.problem.ProblemDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.radiology.RadiologyDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.rightofaccess.RightOfAccessDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.scheduling.SchedulingDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.securemessage.SecureMessageDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.staffdirectory.StaffDirectoryDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.staffuserdisclaimer.StaffUserDisclaimerDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.surgery.SurgeryDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vital.VitalDataLayer;
import com.agilex.healthcare.mobilehealthplatform.domain.DataSourceSystem;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTestGroup;
import com.agilex.healthcare.mobilehealthplatform.patientcorrelation.PatientCorrelationService;
import com.agilex.healthcare.notewriter.datalayer.ClinicalEventDataLayer;
import com.agilex.healthcare.notewriter.datalayer.FacilityProviderDataLayer;
import com.agilex.healthcare.notewriter.datalayer.NoteTitleDataLayer;
import com.agilex.healthcare.patientsearch.datalayer.PatientSearchDataLayer;
import com.agilex.healthcare.utility.NullSafeStringComparer;
import com.agilex.system.health.HealthCheckRunner;

public class MhpObjectFactory implements DataLayerFactory, ApplicationContextAware {

    private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(MhpObjectFactory.class);

	private static final String PATIENTDATALAYER_BEAN = "patientDataLayer";
	private static final String PROBLEMDATALAYER_BEAN = "problemDataLayer";
	private static final String ALLERGYDATALAYER_BEAN = "allergyDataLayer";
	private static final String MEDICATIONDATALAYER_BEAN = "medicationDataLayer";
	private static final String MEDICATIONREFILLDATALAYER_BEAN = "medicationRefillDataLayer";
	private static final String CHEMISTRYLABDATALAYER_BEAN = "chemistryLabDataLayer";
	private static final String MICROBIOLOGYLABDATALAYER_BEAN = "microbiologyLabDataLayer";
	private static final String CHEMISTRY_LAB_GROUP_DEFINITION_BEAN = "chemistryLabGroupDefinitions";
	private static final String HEALTHCHECKCOMPONENT = "healthCheckComponent";
	private static final String VITALDATALAYER_BEAN = "vitalDataLayer";
	private static final String VITALPROVIDERDATALAYER_BEAN = "vitalProviderDataLayer";
	private static final String CONTACTLOGDATALAYER_BEAN = "contactLogDataLayer";
	private static final String APPOINTMENTSDATALAYER_BEAN = "appointmentsDataLayer";
	private static final String APPOINTMENTREQUESTDATALAYER_BEAN = "appointmentRequestDataLayer";
	private static final String ADMISSIONDATALAYER_BEAN = "admissionDataLayer";
	private static final String SURGERYDATALAYER_BEAN = "surgeryDataLayer";
	private static final String RADIOLOGYDATALAYER_BEAN = "radiologyDataLayer";
	private static final String DOCUMENTDATALAYER_BEAN = "documentDataLayer";
	private static final String NOTIFICATIONDATALAYER_BEAN = "notificationDataLayer";
	private static final String NOTIFICATIONCOMMUNICATIONDATALAYER_BEAN = "notificationCommunicationDataLayer";
	private static final String DEVICEREGISTRATIONDATALAYER_BEAN = "deviceRegistrationDataLayer";
	private static final String CALENDARKEYDATALAYER_BEAN = "calendarKeyDataLayer";
	private static final String SECUREMESSAGEDATALAYER_BEAN = "secureMessageDataLayer";
	private static final String CALENDAR_EVENT_DATALAYER_BEAN = "calendarEventDataLayer";
	private static final String RIGHT_OF_ACCESS_BEAN = "rightOfAccessDataLayer";
	private static final String AUDIT_LOG_BEAN = "auditLogDataLayer";
	private static final String PATIENT_CORRELATION_BEAN = "patient-correlation";
	private static final String USERFEEDBACK_DATALAYER_BEAN = "userFeedbackDataLayer";
	private static final String PATIENT_METADATA_DATALAYER_BEAN = "patientMetadataDataLayer";
	private static final String FACILITY_DATALAYER_BEAN = "facilityDataLayer";
	private static final String DETAIL_CODE_DATALAYER = "detailCodeDataLayer";
	private static final String FACILITY_INTERNAL_DATALAYER = "facilityInternalDataLayer";
	private static final String PATIENT_INTERNAL_DATALAYER = "patientInternalDataLayer";
	private static final String APPOINTMENT_METRICS_DATALAYER = "appointmentMetricsDataLayer";
	private static final String APPOINTMENTREQUESTINPROCESS_DATALAYER = "appointmentRequestInProcessDataLayer";
	private static final String METRIC_DATALAYER = "metricDataLayer";
	private static final String LAUNCHPAD_ITEM_DATALAYER = "launchpadItemDataLayer";
	private static final String PATIENT_PREFERENCE_DATALAYER = "patientPreferenceDataLayer";
	private static final String STAFF_USER_DISCLAIMER_DATALAYER = "staffUserDisclaimerDataLayer";
	private static final String STAFF_DIRECTORY_DATALAYER = "staffDirectoryDataLayer";

	private static final String CONSULT_DATALAYER = "consultDataLayer";
	private static final String MDWS_CLINIC_DATALAYER = "mdwsClinicDataLayer";
	private static final String CLINIC_DATALAYER = "clinicDataLayer";
	private static final String MDWS_WARD_DATALAYER = "mdwsWardDataLayer";
	private static final String WARD_DATALAYER = "wardDataLayer";
	private static final String FACILITY_PROVIDER_DATALAYER = "facilityProviderDataLayer";

	private static final String PATIENT_SEARCH_DATALAYER = "patientSearchDataLayer";
	private static final String MY_LIST_SEARCH_DATALAYER = "myListSearchDataLayer";
	private static final String CLINICAL_EVENT_DATALAYER = "clinicalEventDataLayer";
	private static final String NOTE_TITLE_DATALAYER = "noteTitleDataLayer";
	private static final String APPOINTMENT_DATALAYER_MDWS = "appointmentDataLayerMdws";
    //private static final String WAITLIST_DATALAYER_BEAN = "waitListDataLayer";
    private static final String SPECIALTY_DATALAYER_BEAN = "specialtyDataLayer";
    private static final String SPECIALTY_DATALAYER_BEAN_CDW = "cdw.specialtyDataLayer";

    private static final String MDWS_ENCOUNTER_DATALAYER = "mdwsEncounterDataLayer";
    private static final String MDWS_TIUNOTES_DATALAYER = "mdwsTIUNotesDataLayer";

    private static final String PATIENT_CLINIC_DATALAYER_MDWS = "patientClinicDataLayerMdws";
    private static final String SCHEDULING_SVC_DATALAYER_MDWS = "schedulingDataLayerMdws";

    private static final String MEDICATION_REFILL_REQUEST_DATALAYER = "medicationRefillRequestDataLayer";

    private static final String CREATININE_TEST_DATALAYER_MONGO = "creatinineDataLayer";
    private static final String MDWS_ORDER_DATALAYER = "vista.patient-orders.datalayer";

	private static final String ORDER_SERVICES_DATALAYER_MONGO = "orderServicesDataLayerMongo";
	private static final String ORDER_STATUSES_DATALAYER_MONGO = "orderStatusesDataLayerMongo";
	private static final String PROPERTY_HELPER = "propertyHelper";

    private static BeanFactory beanFactory = null;
    private static MhpObjectFactory instance = null;
    private static ApplicationContext springContext = null;

    public MhpObjectFactory() {
        LOGGER.debug("invoked default constructor");
    }
    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        springContext = applicationContext;
    }

    //This uses the existing application context to avoid loading the context twice
    private static BeanFactory getBeanFactory() {
        synchronized (MhpObjectFactory.class) {
            if (beanFactory == null) {
                LOGGER.debug("bean factory not initialized, get app context");
                ApplicationContext appContext = springContext;
                beanFactory = appContext;
                LOGGER.debug("bean factory now set");
            }
        }
        return beanFactory;
    }

	public static MhpObjectFactory getInstance() {
		if (instance == null) {
			initializeInstance();
		}
		return instance;
	}

	private synchronized static void initializeInstance() {
		if (instance == null) {
			LOGGER.debug("initalizing mhp object factory");
			instance = new MhpObjectFactory();
		}
	}

	@SuppressWarnings("unchecked")
	public <T> T getBean(String beanName) {
		T bean = (T) getBeanFactory().getBean(beanName);
		LOGGER.debug(String.format("loaded bean (without type safety) [beanname='%s']=>[bean=%s]", beanName, bean));
		return bean;
	}

	public <T> T getBean(String beanName, Class<T> requiredType) {
		T bean = (T) getBeanFactory().getBean(beanName, requiredType);
		LOGGER.debug(String.format("loaded bean (with type safety) [beanname='%s']=>[bean=%s]", beanName, bean));
		return bean;
	}

	@Override
	public PatientDataLayer getPatientDataLayer() {
		return getBean(PATIENTDATALAYER_BEAN, PatientDataLayer.class);
	}

	@Override
	public AllergyDataLayer getAllergyDataLayer() {
		return getBean(ALLERGYDATALAYER_BEAN, AllergyDataLayer.class);
	}

	@Override
	public DocumentDataLayer getDocumentDataLayer() {
		return getBean(DOCUMENTDATALAYER_BEAN, DocumentDataLayer.class);
	}

	@Override
	public ProblemDataLayer getProblemDataLayer() {
		return getBean(PROBLEMDATALAYER_BEAN, ProblemDataLayer.class);
	}

	@Override
	public MedicationDataLayer getMedicationDataLayer() {
		return getBean(MEDICATIONDATALAYER_BEAN, MedicationDataLayer.class);
	}

	@Override
	public VitalDataLayer getVitalDataLayer(String source) {
		LOGGER.debug(String.format("determine vital data layer for source %s", source));
		String beanname;

		if (NullSafeStringComparer.areEqual(source, "patient")) {
			beanname = VITALDATALAYER_BEAN;
		} else if (NullSafeStringComparer.areEqual(source, "provider")) {
			beanname = VITALPROVIDERDATALAYER_BEAN;
		} else {
			beanname = VITALDATALAYER_BEAN;
		}

		LOGGER.debug(String.format("determined vital data layer for source %s, will use bean %s", source, beanname));
		return getBean(beanname, VitalDataLayer.class);
	}

	@Override
	public SurgeryDataLayer getSurgeryDataLayer() {
		return getBean(SURGERYDATALAYER_BEAN, SurgeryDataLayer.class);
	}

	@Override
	public ChemistryLabDataLayer getChemistryLabDataLayer() {
		return getBean(CHEMISTRYLABDATALAYER_BEAN, ChemistryLabDataLayer.class);
	}

	@Override
	public MicrobiologyLabDataLayer getMicrobiologyLabDataLayer() {
		return getBean(MICROBIOLOGYLABDATALAYER_BEAN, MicrobiologyLabDataLayer.class);
	}

	@Override
	public AppointmentDataLayer getAppointmentDataLayer() {
		return getBean(APPOINTMENTSDATALAYER_BEAN, AppointmentDataLayer.class);
	}

	@Override
	public AppointmentRequestDataLayer getAppointmentRequestDataLayer() {
		return getBean(APPOINTMENTREQUESTDATALAYER_BEAN, AppointmentRequestDataLayer.class);
	}

	public HealthCheckRunner getHealthCheckComponent() {
		return getBean(HEALTHCHECKCOMPONENT, HealthCheckRunner.class);
	}

	@Override
	public ContactLogDataLayer getContactLogDataLayer() {
		return getBean(CONTACTLOGDATALAYER_BEAN, ContactLogDataLayer.class);
	}

	@SuppressWarnings("unchecked")
	public HashMap<LabTestGroup, List<String>> chemistryLabGroupDefinitions() {
		return getBean(CHEMISTRY_LAB_GROUP_DEFINITION_BEAN, HashMap.class);
	}

	@Override
	public AdmissionDataLayer getAdmissionDataLayer() {
		return getBean(ADMISSIONDATALAYER_BEAN, AdmissionDataLayer.class);
	}

	@Override
	public RadiologyDataLayer getRadiologyDataLayer() {
		return getBean(RADIOLOGYDATALAYER_BEAN, RadiologyDataLayer.class);
	}

	@Override
	public NotificationDataLayer getNotificationDataLayer() {
		return getBean(NOTIFICATIONDATALAYER_BEAN, NotificationDataLayer.class);
	}

	@Override
	public NotificationCommunicationDataLayer getNotificationCommunicationDataLayer() {
		return getBean(NOTIFICATIONCOMMUNICATIONDATALAYER_BEAN, NotificationCommunicationDataLayer.class);
	}

	@Override
	public DeviceRegistrationDataLayer getDeviceRegistrationDataLayer() {
		return getBean(DEVICEREGISTRATIONDATALAYER_BEAN, DeviceRegistrationDataLayer.class);
	}

	@Override
	public CalendarKeyDataLayer getCalendarKeyDataLayer() {
		return getBean(CALENDARKEYDATALAYER_BEAN, CalendarKeyDataLayer.class);
	}

	@Override
	public MedicationRefillDataLayer getMedicationRefillDataLayer() {
		return getBean(MEDICATIONREFILLDATALAYER_BEAN, MedicationRefillDataLayer.class);
	}

	@Override
	public SecureMessageDataLayer getSecureMessageDataLayer() {
		return getBean(SECUREMESSAGEDATALAYER_BEAN, SecureMessageDataLayer.class);
	}

	public CalendarEventDataLayer getCalendarEventDataLayer() {
		return getBean(CALENDAR_EVENT_DATALAYER_BEAN, CalendarEventDataLayer.class);
	}

	@Override
	public RightOfAccessDataLayer getRightOfAccessDataLayer() {
		return getBean(RIGHT_OF_ACCESS_BEAN, RightOfAccessDataLayer.class);
	}

	@Override
	public AuditLogDataLayer getAuditLogDataLayer() {
		return getBean(AUDIT_LOG_BEAN, AuditLogDataLayer.class);
	}

	@Override
	public UserFeedbackDataLayer getUserFeedbackDataLayer() {
		return getBean(USERFEEDBACK_DATALAYER_BEAN, UserFeedbackDataLayer.class);
	}

	@Override
	public PatientMetadataDataLayer getPatientMetadataDataLayer() {
		return getBean(PATIENT_METADATA_DATALAYER_BEAN, PatientMetadataDataLayer.class);
	}

	public PatientCorrelationService getPatientCorrelationService () {
		return getBean(PATIENT_CORRELATION_BEAN, PatientCorrelationService.class);
	}

	@Override
	public FacilityDataLayer getFacilityDataLayer() {
		return getBean(FACILITY_DATALAYER_BEAN, FacilityDataLayer.class);
	}

	@Override
	public DetailCodeDataLayer getDetailCodeDataLayer() {
		return getBean(DETAIL_CODE_DATALAYER, DetailCodeDataLayer.class);
	}

	@Override
	public FacilityInternalDataLayer getFacilityInternalDataLayer() {
		return getBean(FACILITY_INTERNAL_DATALAYER, FacilityInternalDataLayer.class);
	}

	@Override
	public PatientInternalDataLayer getPatientInternalDataLayer() {
		return getBean(PATIENT_INTERNAL_DATALAYER, PatientInternalDataLayer.class);
	}

	@Override
	public AppointmentMetricsDataLayer getAppointmentMetricsDataLayer() {
		return getBean(APPOINTMENT_METRICS_DATALAYER, AppointmentMetricsDataLayer.class);
	}

	@Override
	public PatientPreferenceDataLayer getPatientPreferenceDataLayer(){
		return getBean(PATIENT_PREFERENCE_DATALAYER,PatientPreferenceDataLayer.class);
	}

	@Override
	public AppointmentRequestInProcessDataLayer getAppointmentRequestInProcessDataLayer() {
		return getBean(APPOINTMENTREQUESTINPROCESS_DATALAYER, AppointmentRequestInProcessDataLayer.class);
	}

	@Override
	public MetricDataLayer getMetricDataLayer() {
		return getBean(METRIC_DATALAYER, MetricDataLayer.class);
	}

	@Override
	public LaunchpadItemDataLayer getLaunchpadItemDataLayer() {
		return getBean(LAUNCHPAD_ITEM_DATALAYER, LaunchpadItemDataLayer.class);
	}

	@Override
	public StaffUserDisclaimerDataLayer getStaffUserDisclaimerDataLayer() {
		return getBean(STAFF_USER_DISCLAIMER_DATALAYER, StaffUserDisclaimerDataLayer.class);
	}

	@Override
	public StaffDirectoryDataLayer getStaffDirectoryDataLayer() {
		return getBean(STAFF_DIRECTORY_DATALAYER, StaffDirectoryDataLayer.class);
	}

	@Override
	public MedicationRefillRequestDataLayer getMedicationRefillRequestDataLayer() {
		return getBean(MEDICATION_REFILL_REQUEST_DATALAYER, MedicationRefillRequestDataLayer.class);
	}

	@Override
	public ConsultDataLayer getConsultDataLayer() {
		return getBean(CONSULT_DATALAYER, ConsultDataLayer.class);
	}

	@Override
	public ClinicDataLayer getMdwsClinicDataLayer() {
		return getBean(MDWS_CLINIC_DATALAYER, ClinicDataLayer.class);
	}
	@Override
	public ClinicDataLayer getClinicDataLayer() {
		return getBean(CLINIC_DATALAYER, ClinicDataLayer.class);
	}
	@Override
	public WardDataLayer getMdwsWardDataLayer() {
		return getBean(MDWS_WARD_DATALAYER, WardDataLayer.class);
	}
	@Override
	public WardDataLayer getWardDataLayer() {
		return getBean(WARD_DATALAYER, WardDataLayer.class);
	}
    @Override
    public SpecialtyDataLayer getSpecialtyDataLayer() {
        return getSpecialtyDataLayer(DataSourceSystem.MONGODB);
    }

    @Override
	public SpecialtyDataLayer getSpecialtyDataLayer(DataSourceSystem dataSource) {
    	String beanName = SPECIALTY_DATALAYER_BEAN;
		if (dataSource.equals(DataSourceSystem.CDW)){
			beanName = SPECIALTY_DATALAYER_BEAN_CDW;
		}
		return getBean(beanName, SpecialtyDataLayer.class);
	}

	@Override
	public PatientSearchDataLayer getPatientSearchDataLayer() {
		return getBean(PATIENT_SEARCH_DATALAYER, PatientSearchDataLayer.class);
	}

	@Override
	public ClinicalEventDataLayer getClinicalEventDataLayer() {
		return getBean(CLINICAL_EVENT_DATALAYER, ClinicalEventDataLayer.class);
	}

	@Override
	public NoteTitleDataLayer getNoteTitleDataLayer() {
		return getBean(NOTE_TITLE_DATALAYER, NoteTitleDataLayer.class);
	}

	@Override
	public AppointmentDataLayerMdws getAppointmentDataLayerMdws() {
		return getBean(APPOINTMENT_DATALAYER_MDWS, AppointmentDataLayerMdws.class);
	}

	public static <T> T initializeByType(Class<T> type) {
		T object;
		try {
			object = type.newInstance();
		} catch (InstantiationException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		}
		return object;
	}
	@Override
	public FacilityProviderDataLayer getFacilityProviderDataLayer() {
		return getBean(FACILITY_PROVIDER_DATALAYER, FacilityProviderDataLayer.class);
	}

	@Override
	public EncounterDataLayer getMdwsEncounterDataLayer() {
		return getBean(MDWS_ENCOUNTER_DATALAYER, EncounterDataLayer.class);
	}

    @Override
    public TIUNotesDataLayerMdws getMdwsTIUNotesDataLayer() {
        return getBean(MDWS_TIUNOTES_DATALAYER, TIUNotesDataLayerMdws.class);
    }

	@Override
	public SchedulingDataLayerMdws getSchedulingSvcDataLayerMdws() {
		return getBean(SCHEDULING_SVC_DATALAYER_MDWS, SchedulingDataLayerMdws.class);
	}
	
	@Override
	public CreatinineTestDataLayerMongo getCreatinineTestDataLayer(){
		return getBean(CREATININE_TEST_DATALAYER_MONGO,CreatinineTestDataLayerMongo.class);
	}

	@Override
	public OrderDataLayerMdws getOrderDataLayerMdws() {
		return getBean(MDWS_ORDER_DATALAYER, OrderDataLayerMdws.class);
	}

	@Override
	public OrderServicesDataLayerMongo getOrderServicesDataLayerMongo() {
		return getBean(ORDER_SERVICES_DATALAYER_MONGO, OrderServicesDataLayerMongo.class);
	}

	@Override
	public OrderStatusesDataLayerMongo getOrderStatusesDataLayerMongo() {
		return getBean(ORDER_STATUSES_DATALAYER_MONGO, OrderStatusesDataLayerMongo.class);
	}

	@Override
	public PropertyHelper getPropertyHelper() {
		return getBean(PROPERTY_HELPER, PropertyHelper.class);
	}
}
