package com.agilex.healthcare.mobilehealthplatform.translator;

import java.net.URI;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import com.agilex.healthcare.mobilehealthplatform.domain.Address;
import com.agilex.healthcare.mobilehealthplatform.domain.Addresses;
import com.agilex.healthcare.mobilehealthplatform.domain.AtomLink;
import com.agilex.healthcare.mobilehealthplatform.domain.AtomLinks;
import com.agilex.healthcare.mobilehealthplatform.domain.DataIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.NextOfKin;
import com.agilex.healthcare.mobilehealthplatform.domain.Order;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.Orders;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientDemographics;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.PhoneNumber;
import com.mongodb.BasicDBList;
import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;

public class OrderMongoTranslator {

	private static final String       ERROR_MESSAGE = "error_message";
    private static final String       ORDER_DETAIL = "order_detail";
    private static final String       ORDER_ID = "order_Id";
    private static final String       ORDER_SUB_TYPE = "order_Sub_Type";
    private static final String       ORDER_TEXT = "order_Text";
    private static final String       ORDER_TYPE = "order_Type";
    private static final String       ORDER_TYPE_ID = "order_Type_Id";
    private static final String       STATUS = "status";
    private static final String       VERIFYING_CLERK = "verifying_clerk";
    private static final String       VERIFYING_NURSE = "verifying_nurse";
    private static final String       ASSIGNING_AUTHORITY = "assigning_authority";
    private static final String       PATIENT = "patient";
    private static final String       PATIENT_ID = "patient_id";
    private static final String       OBJECT_TYPE = "object_type";
    private static final String       SOURCE_SYSTEM = "source_system";
    private static final String       SURROGATE_ID = "surrogate_id";
    private static final String       SYSTEM_ID = "system_id";
    private static final String       UNIQUE_ID = "unique_id";
    private static final String       DATE = "date";
    private static final String       DATE_SIGNED = "date_signed";
    private static final String       DATE_VERIFIED = "date_verified";
    private static final String       ORDERED_DATE = "ordered_date";
    private static final String       START_DATE = "start_date";
    private static final String       STOP_DATE = "stop_date";
    private static final String       ORDER_GROUPING = "order_grouping";
    private static final String       ORDER_LOCATION = "order_location";
    private static final String       ORDER_PROVIDER = "order_provider";
    private static final String       ORDER_SIGN_STATUS = "order_sign_status";
    private static final String       ORDER_STATUS = "order_status";
    private static final String       PROVIDER = "provider";
    private static final String       DATA_IDENTIFIER = "data_identifier";
    private static final String       LINK = "link";
    private static final String       PATIENT_IDENTIFIER = "patient_identifier";
    private static final String       SELF_LINK = "self_link";
    private static final String       SURROGATE_IDENTIFIER = "surrogate_identifier";
    private static final String       SELF_URI = "self_uri";
    private static final String       ID = "id";
    private static final String       NAME = "name";

    private static final String       DIGITAL_PAGER = "digital_pager";
    private static final String       DOMAIN = "domain";
    private static final String       USER_NAME = "user_name";
    private static final String       USER_CLASS = "user_class";
    private static final String       TITLE = "title";
    private static final String       SSN = "ssn";
    private static final String       SITE_MESSAGE = "site_message";
    private static final String       SITE_ID = "site_id";
    private static final String       SERVICE = "service";
    private static final String       PHONE = "phone";
    private static final String       PAGER = "pager";
    private static final String       ORDER_ROLE = "order_role";
    private static final String       OFFICE = "office";
    private static final String       IDS = "ids";
    private static final String       GREETING = "greeting";
    private static final String       EMAIL_ADDRESS = "email_address";
    private static final String       DUZ = "duz";
    private static final String       VALUE = "value";
    private static final String       CODE = "code";
    private static final String       SHORT_NAME = "short_name";
    private static final String       ADDRESS_TYPE = "address_type";
    private static final String       CITY = "city";
    private static final String       FORMATTED_DISPLAY_ADDRESS_1 = "formatted_display_address_1";
    private static final String       FORMATTED_DISPLAY_ADDRESS_2 = "formatted_display_address_2";
    private static final String       STREET_ADDRESS_LINE_1 = "street_address_line_1";
    private static final String       STREET_ADDRESS_LINE_2 = "street_address_line_2";
    private static final String       STREET_ADDRESS_LINE_3 = "street_address_line_3";
    private static final String       STATE = "state";
    private static final String       ZIP_CODE = "zip_code";
    private static final String       ADDRESSES = "addresses";
    private static final String       ADDRESS = "address";
    private static final String       HREF = "href";
    private static final String       HREFLANG = "hreflang";
    private static final String       LENGTH = "length";
    private static final String       REL = "rel";
    private static final String       TYPE = "type";
    private static final String       PHONE_NUMBER_WORK = "phone_number_work";
    private static final String       PHONE_NUMBER_MOBILE = "phone_number_mobile";
//    private static final String       PHONE_NUMBER_PAGER = "phone_number_pager";
//    private static final String       PHONE_NUMBER_HOME = "phone_number_home";
    private static final String       PHONE_NUMBER = "phone_number";
    private static final String       PHONE_NUMBERS = "phone_numbers";
    private static final String       RELATIONSHIPP = "relationship";
    private static final String       NUMBER = "number";
    private static final String       NEXT_OF_KIN = "next_of_kin";
    private static final String       FIRST_NAME = "first_name";
    private static final String       LAST_NAME = "last_name";
    private static final String       RAW_PATH = "raw_path";
    private static final String       QUERY = "query";
    private static final String       AUTHORITY = "authority";
    private static final String       FRAGMENT = "fragment";
    private static final String       RAW_QUERY = "raw_query";
    private static final String       RAW_AUTHORITY = "raw_authority";
    private static final String       RAW_FRAGMENT = "raw_fragment";
    private static final String       SCHEME_SPECIFIC_PART = "scheme_specific_part";
    private static final String       SCHEME = "scheme";
    private static final String       RAW_USER_INFO = "raw_user_info";
    private static final String       USER_INFO = "user_info";
//    private static final String       RAW_SCHEME_SPECIFIC_PART = "raw_scheme_specific_part";
    private static final String       HOST = "host";
    private static final String       PORT = "port";
    private static final String       PATH = "path";
    private static final String       DEMOGRAPHICS = "demographics";
    private static final String       ALERGIES = "alergies";
    private static final String       PROBLEMS = "problems";
    private static final String       MEDICATIONS = "medications";
//    private static final String       NAME = "name";
//    private static final String       NAME = "name";
//    private static final String       NAME = "name";
//    private static final String       NAME = "name";
//    private static final String       NAME = "name";
//    private static final String       NAME = "name";
//    private static final String       NAME = "name";
//    private static final String       NAME = "name";
//    private static final String       NAME = "name";
//    private static final String       NAME = "name";

    public List<DBObject> convertToDBObjectList(Orders  orders) {
        ArrayList<DBObject>       returnList = new ArrayList<DBObject>();
        DBObject                  dbObject;
        DBObject                  subObject;
        DBObject                  subObject2;
        DBObject                  subObject3;
        BasicDBList               subObjectList;
        Iterator<?>                  orderIterator;
        Iterator<?>                  addressIterator;
        Iterator<?>                  phoneNumberIterator;
        PatientDemographics       demographics;
        OrderProvider             orderProvider;
        MhpUser                   mhpUser;
        Addresses                 addresses;
        Address                   address;
        DataIdentifier            dataIdentifier;


        orderIterator = orders.getOrder().iterator();
        while(orderIterator.hasNext()) {
            Order     order = (Order)orderIterator.next();

            dbObject = new BasicDBObject();

            dbObject.put(ERROR_MESSAGE, order.getErrorMessage());
            dbObject.put(ORDER_DETAIL, order.getOrderDetail());
            dbObject.put(ORDER_ID, order.getOrderId());
            dbObject.put(ORDER_SUB_TYPE, order.getOrderSubType());
            dbObject.put(ORDER_TEXT, order.getOrderText());
            dbObject.put(ORDER_TYPE, order.getOrderType());
            dbObject.put(ORDER_TYPE_ID, order.getOrderTypeId());
            dbObject.put(STATUS, order.getStatus());
            dbObject.put(VERIFYING_CLERK, order.getVerifyingClerk());
            dbObject.put(VERIFYING_NURSE, order.getVerifyingNurse());
            dbObject.put(ASSIGNING_AUTHORITY, order.getAssigningAuthority());
            dbObject.put(PATIENT_ID, order.getPatientId());
            dbObject.put(OBJECT_TYPE, order.getObjectType());
            dbObject.put(SOURCE_SYSTEM, order.getSourceSystem());
            dbObject.put(SURROGATE_ID, order.getSurrogateId());
            dbObject.put(SYSTEM_ID, order.getSystemId());
            dbObject.put(UNIQUE_ID, order.getUniqueId());
            dbObject.put(DATE, order.getDate());
            dbObject.put(DATE_SIGNED, order.getDateSigned());
            dbObject.put(DATE_VERIFIED, order.getDateVerified());
            dbObject.put(ORDERED_DATE, order.getOrderedDate());
            dbObject.put(START_DATE, order.getStartDate());
            dbObject.put(STOP_DATE, order.getStopDate());

            subObject = new BasicDBObject();

//            subObject.put(ORDER_GROUPING, order.getOrderGrouping().get);
            dbObject.put(ORDER_GROUPING, subObject);


            subObject = new BasicDBObject();

            subObject.put(ID, order.getOrderLocation().getId());
            subObject.put(NAME, order.getOrderLocation().getName());
            dbObject.put(ORDER_LOCATION, subObject);

            subObject = new BasicDBObject();
            subObject2 = new BasicDBObject();
            subObjectList = new BasicDBList();

            orderProvider = order.getOrderProvider();
            if(orderProvider != null) {
                demographics = orderProvider.getDemographics();

                if(demographics != null) {
                    addresses = demographics.getAddresses();
                    if(addresses != null) {
                        addressIterator = addresses.iterator();
                        while(addressIterator.hasNext()) {
                            address = (Address)addressIterator.next();

                            subObjectList.add(loadAddressIntoDbObject(address));
                        }
                        subObject2.put(ADDRESSES, subObjectList);
                    }
                    subObject2.put(LINK, loadAtomLinksIntoDbObject(demographics.getLink()));


                    subObjectList = new BasicDBList();

                    phoneNumberIterator = demographics.getPhoneNumbers().getPhoneNumber().iterator();
                    while(phoneNumberIterator.hasNext()) {
                        PhoneNumber       number = (PhoneNumber)phoneNumberIterator.next();

                        subObjectList.add(loadPhoneNumberIntoDbObject(number));
                    }
                    subObject2.put(PHONE_NUMBERS, subObjectList);

                    subObject2.put(ADDRESS, loadAddressIntoDbObject(demographics.getAddress()));
                    subObject2.put(SELF_LINK, loadAtomLinkIntoDbObject(demographics.getSelfLink()));
                    subObject2.put(SURROGATE_IDENTIFIER, loadPatientIdentifierIntoDbObject(demographics.getSurrogateIdentifier()));
                    subObject2.put(SURROGATE_ID, demographics.getSurrogateId());
                    subObject2.put(LINK, loadAtomLinksIntoDbObject(demographics.getLink()));
                    subObject2.put(SELF_URI, loadUriIntoDbObject(demographics.getSelfUri()));
                    subObject2.put(OBJECT_TYPE, demographics.getObjectType());
                    subObject2.put(EMAIL_ADDRESS, demographics.getEmailAddress());
                    subObject2.put(NEXT_OF_KIN, loadNextOfKinIntoDbObject(demographics.getNextOfKin()));
                    subObject2.put(PHONE_NUMBER_WORK, demographics.getPhoneNumberHome());
                    subObject2.put(PHONE_NUMBER_WORK, demographics.getPhoneNumberWork());
                    subObject2.put(PHONE_NUMBER_MOBILE, demographics.getPhoneNumberMobile());
                    subObject2.put(EMAIL_ADDRESS, demographics.getEmailAddress());
                    subObject2.put(ASSIGNING_AUTHORITY, demographics.getAssigningAuthority());
                    subObject2.put(PHONE_NUMBER_MOBILE, demographics.getPhoneNumberPager());
                    subObject2.put(PATIENT_ID, demographics.getPatientId());
                    subObject2.put(PATIENT_IDENTIFIER, loadPatientIdentifierIntoDbObject(demographics.getPatientIdentifier()));
                    subObject.put(DEMOGRAPHICS, subObject2);

                    subObject3 = new BasicDBObject();

                    subObject3.put(UNIQUE_ID, demographics.getDataIdentifier().getUniqueId());
                    subObject3.put(SYSTEM_ID, demographics.getDataIdentifier().getSystemId());
                    subObject2.put(DATA_IDENTIFIER, subObject3);

                    subObject2.put(SELF_URI, loadUriIntoDbObject(demographics.getSelfUri()));
                }



                subObject.put(NAME, order.getOrderProvider().getName());
                subObject.put(DIGITAL_PAGER, order.getOrderProvider().getDigitalPager());
                subObject.put(DOMAIN, order.getOrderProvider().getDomain());
                subObject.put(SELF_URI, loadUriIntoDbObject(order.getOrderProvider().getSelfUri()));
                subObject.put(SELF_LINK, order.getOrderProvider().getSelfLink());
                subObject.put(OBJECT_TYPE, order.getOrderProvider().getObjectType());
                subObject.put(LINK, order.getOrderProvider().getLink());
                subObject.put(USER_NAME, order.getOrderProvider().getUsername());
                subObject.put(USER_CLASS, order.getOrderProvider().getUserClass());
                subObject.put(TITLE, order.getOrderProvider().getTitle());
                subObject.put(SSN, order.getOrderProvider().getSsn());
                subObject.put(SITE_MESSAGE, order.getOrderProvider().getSiteMessage());
                subObject.put(SITE_ID, order.getOrderProvider().getSiteId());
                subObject.put(SERVICE, order.getOrderProvider().getService());
                subObject.put(PHONE, order.getOrderProvider().getPhone());
                subObject.put(USER_NAME, order.getOrderProvider().getUsername());
                subObject.put(USER_CLASS, order.getOrderProvider().getUserClass());
                subObject.put(TITLE, order.getOrderProvider().getTitle());
                subObject.put(PAGER, order.getOrderProvider().getPager());
                subObject.put(ORDER_ROLE, order.getOrderProvider().getOrderRole());
                subObject.put(OFFICE, order.getOrderProvider().getOffice());
                subObject.put(IDS, order.getOrderProvider().getIds());
                subObject.put(GREETING, order.getOrderProvider().getGreeting());
                subObject.put(EMAIL_ADDRESS, order.getOrderProvider().getEmailAddress());
                subObject.put(DUZ, order.getOrderProvider().getDuz());
                dbObject.put(ORDER_PROVIDER, subObject);
            }


            subObject = new BasicDBObject();

            subObject.put(NAME, order.getOrderSignStatus().getName());
            subObject.put(VALUE, order.getOrderSignStatus().getValue());
            dbObject.put(ORDER_SIGN_STATUS, subObject);


            subObject = new BasicDBObject();

            subObject.put(LINK, order.getOrderStatus().getLink());
            subObject.put(OBJECT_TYPE, order.getOrderStatus().getObjectType());
            subObject.put(VALUE, order.getOrderStatus().getValue());
            subObject.put(SELF_LINK, order.getOrderStatus().getSelfLink());
            subObject.put(SELF_URI, loadUriIntoDbObject(order.getOrderStatus().getSelfUri()));
            subObject.put(CODE, order.getOrderStatus().getCode());
            subObject.put(NAME, order.getOrderStatus().getName());
            subObject.put(SHORT_NAME, order.getOrderStatus().getShortName());
            dbObject.put(ORDER_STATUS, subObject);


            subObject = new BasicDBObject();

            mhpUser = order.getProvider();


            if(mhpUser != null) {
                subObject.put(SELF_LINK, loadAtomLinkIntoDbObject(mhpUser.getSelfLink()));
                subObject.put(SELF_URI, loadUriIntoDbObject(mhpUser.getSelfUri()));
                subObject.put(PATIENT_ID, mhpUser.getPatientId());
                subObject.put(PATIENT, loadPatientIntoDbObject(mhpUser.getPatient()));
//            subObject.put(PROVIDER, mhpUser.get);
//            subObject.put(PROVIDER, mhpUser.get);
//            subObject.put(PROVIDER, mhpUser.get);
            }
            dbObject.put(PROVIDER, subObject);


            subObject = new BasicDBObject();

            dataIdentifier = order.getDataIdentifier();

            if(dataIdentifier != null) {
                subObject.put(DATA_IDENTIFIER, dataIdentifier.getSystemId());
                subObject.put(DATA_IDENTIFIER, dataIdentifier.getUniqueId());
                dbObject.put(DATA_IDENTIFIER, subObject);
            }


            subObject = new BasicDBObject();

            subObject.put(LINK, loadAtomLinksIntoDbObject(order.getLink()));
            dbObject.put(LINK, subObject);


            subObject = new BasicDBObject();

            subObject.put(PATIENT_IDENTIFIER, loadPatientIdentifierIntoDbObject(order.getPatientIdentifier()));
            dbObject.put(PATIENT_IDENTIFIER, subObject);

            subObject = new BasicDBObject();

            subObject.put(SELF_LINK, loadAtomLinkIntoDbObject(order.getSelfLink()));
            dbObject.put(SELF_LINK, subObject);


            subObject = new BasicDBObject();

            subObject.put(SURROGATE_ID, order.getSurrogateId());
            dbObject.put(SURROGATE_ID, subObject);


//            subObject = new BasicDBObject();
//
//            subObject.put(SURROGATE_IDENTIFIER, order.getSurrogateIdentifier().get);
//            dbObject.put(SELF_LINK, subObject);
//
//
//            subObject = new BasicDBObject();
//
//            subObject.put(SELF_LINK, order.getSelfUri().get);
//            dbObject.put(SELF_LINK, subObject);
//
//
//            subObject = new BasicDBObject();
//
//            subObject.put(SELF_URI, order.getSelfUri().get);
//            dbObject.put(SELF_URI, subObject);


            returnList.add(dbObject);
        }

        return returnList;
    }

    private DBObject loadPatientIntoDbObject(Patient patient) {
        DBObject    dbObject = new BasicDBObject();


        if(patient != null) {
            dbObject.put(SELF_URI, loadUriIntoDbObject(patient.getSelfUri()));
            dbObject.put(ALERGIES, loadUriIntoDbObject(patient.getAllergiesUri()));
            dbObject.put(DEMOGRAPHICS, loadUriIntoDbObject(patient.getDemographicsUri()));
            dbObject.put(MEDICATIONS, loadUriIntoDbObject(patient.getMedicationsUri()));
            dbObject.put(PROBLEMS, loadUriIntoDbObject(patient.getProblemsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getVitalsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getActiveMedicationsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getAssessmentResultsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getInpatientMedicationsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getLabResultsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getLongitudinalVitalsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getMedicationRefillsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getOperationalVitalsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getOutpatientMedicationsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getPatientImageUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getAssessmentGraphDataUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getPatientOrdersUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getCurrentInpatientMedicationsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getCurrentOutpatientMedicationsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getLabTestHistoryUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getMentalHealthAppointmentUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getMedicationRefillsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getMicroLabResultsUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getLabTestHistoryByGroupUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getProviderEnteredVitalsGraphDataUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getProviderEnteredVitalsTableDataUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getMicroLabTestHistoryByGroupUri()));
//        dbObject.put(, loadUriIntoDbObject(patient.getMicroLabTestHistoryUri()));
//        dbObject.put(, patient.getObjectType());
//        dbObject.put(, loadPatientIdentifierIntoDbObject(patient.getPatientIdentifier()));
//        dbObject.put(, loadAtomLinkIntoDbObject(patient.getSelfLink()));
            dbObject.put(LINK, loadAtomLinksIntoDbObject(patient.getLink()));
//        dbObject.put(, patient.getAdmittedDateTime());
//        dbObject.put(, patient.getAge());
//        dbObject.put(, patient.getCwad());
//        dbObject.put(, patient.getDateOfBirth());
//        dbObject.put(, patient.getDisplayName());
//        dbObject.put(, patient.getRoombed());
//        dbObject.put(, patient.getEnterpriseId());
            dbObject.put(FIRST_NAME, patient.getFirstName());
//        dbObject.put(, patient.getGender());
//        dbObject.put(, patient.getIcn());
            dbObject.put(PATIENT_ID, patient.getId());
            dbObject.put(LAST_NAME, patient.getLastName());
//        dbObject.put(, patient.getLocationIEN());
//        dbObject.put(, patient.getWardLocation());
//        dbObject.put(, patient.getMiddleInitial());
//        dbObject.put(, patient.getMiddleName());
//        dbObject.put(, patient.getServiceConnectedPercent());
//        dbObject.put(, patient.getSsn());
//        dbObject.put(, patient.getTextMessagingPhoneNumber());
        }

        return dbObject;
    }

    private DBObject loadUriIntoDbObject(URI uri) {
        DBObject    dbObject = new BasicDBObject();

        if(uri != null) {
            dbObject.put(RAW_PATH, uri.getRawPath());
            dbObject.put(AUTHORITY, uri.getAuthority());
            dbObject.put(FRAGMENT, uri.getFragment());
            dbObject.put(HOST, uri.getHost());
            dbObject.put(PATH, uri.getPath());
            dbObject.put(PORT, uri.getPort());
            dbObject.put(QUERY, uri.getQuery());
            dbObject.put(RAW_AUTHORITY, uri.getRawAuthority());
            dbObject.put(RAW_FRAGMENT, uri.getRawFragment());
            dbObject.put(RAW_QUERY, uri.getRawQuery());
            dbObject.put(RAW_FRAGMENT, uri.getRawSchemeSpecificPart());
            dbObject.put(RAW_USER_INFO, uri.getRawUserInfo());
            dbObject.put(SCHEME, uri.getScheme());
            dbObject.put(SCHEME_SPECIFIC_PART, uri.getSchemeSpecificPart());
            dbObject.put(USER_INFO, uri.getUserInfo());
        }

        return dbObject;
    }

    private DBObject loadPatientIdentifierIntoDbObject(PatientIdentifier identifier) {
        DBObject    dbObject = new BasicDBObject();

        if(identifier != null) {
            dbObject.put(ASSIGNING_AUTHORITY, identifier.getAssigningAuthority());
            dbObject.put(FIRST_NAME, identifier.getFirstName());
            dbObject.put(LAST_NAME, identifier.getLastName());
            dbObject.put(SSN, identifier.getSsn());
            dbObject.put(UNIQUE_ID, identifier.getUniqueId());
        }

        return dbObject;
    }

    private DBObject loadPhoneNumberIntoDbObject(PhoneNumber number) {
        DBObject    dbObject = new BasicDBObject();

        if(number != null) {
            dbObject.put(OBJECT_TYPE, number.getObjectType());
            dbObject.put(NUMBER, number.getNumber());
            dbObject.put(TYPE, number.getType());
            dbObject.put(LINK, loadAtomLinksIntoDbObject(number.getLink()));
            dbObject.put(SELF_LINK, number.getSelfLink());
        }

        return dbObject;
    }

    private DBObject loadNextOfKinIntoDbObject(NextOfKin nextOfKin) {
        DBObject    dbObject = new BasicDBObject();

        if(nextOfKin != null) {
            dbObject.put(RELATIONSHIPP, nextOfKin.getRelationship());
            dbObject.put(CITY, nextOfKin.getCity());
            dbObject.put(PHONE_NUMBER_WORK, nextOfKin.getPhoneNumberWork());
            dbObject.put(PHONE_NUMBER, nextOfKin.getPhoneNumber());
            dbObject.put(STREET_ADDRESS_LINE_1, nextOfKin.getStreetAddressLine1());
            dbObject.put(STREET_ADDRESS_LINE_2, nextOfKin.getStreetAddressLine2());
            dbObject.put(STREET_ADDRESS_LINE_3, nextOfKin.getStreetAddressLine3());
            dbObject.put(STATE, nextOfKin.getState());
            dbObject.put(NAME, nextOfKin.getName());
        }

        return dbObject;
    }

    private DBObject  loadAtomLinksIntoDbObject(AtomLinks  links) {
        BasicDBList       subObjectList = new BasicDBList();
        Iterator<?>          linkIterator;


        if(links != null) {
            linkIterator = links.iterator();
            while(linkIterator.hasNext()) {
                AtomLink    link = (AtomLink)linkIterator.next();

                subObjectList.add(loadAtomLinkIntoDbObject(link));
            }
        }

        return subObjectList;
    }

    private DBObject loadAtomLinkIntoDbObject(AtomLink selfLink) {
        DBObject    dbObject;

        dbObject = new BasicDBObject();

        if(selfLink != null) {
            dbObject.put(HREF, selfLink.getHref());
            dbObject.put(HREFLANG, selfLink.getHreflang());
            dbObject.put(LENGTH, selfLink.getLength());
            dbObject.put(REL, selfLink.getRel());
            dbObject.put(OBJECT_TYPE, selfLink.getObjectType());
            dbObject.put(TITLE, selfLink.getTitle());
            dbObject.put(TYPE, selfLink.getType());
        }

        return dbObject;
    }

    private DBObject loadAddressIntoDbObject(Address address) {
        DBObject    dbObject = new BasicDBObject();

        if(address != null) {
            dbObject.put(OBJECT_TYPE, address.getObjectType());
            dbObject.put(SELF_URI, address.getSelfUri());
            dbObject.put(LINK, address.getLink());
            dbObject.put(SELF_LINK, address.getSelfLink());
            dbObject.put(ADDRESS_TYPE, address.getAddressType());
            dbObject.put(CITY, address.getCity());
            dbObject.put(FORMATTED_DISPLAY_ADDRESS_1, address.getFormattedDisplayAddress1());
            dbObject.put(FORMATTED_DISPLAY_ADDRESS_2, address.getFormattedDisplayAddress2());
            dbObject.put(STREET_ADDRESS_LINE_1, address.getStreetAddressLine1());
            dbObject.put(STREET_ADDRESS_LINE_2, address.getStreetAddressLine2());
            dbObject.put(STREET_ADDRESS_LINE_3, address.getStreetAddressLine3());
            dbObject.put(STATE, address.getState());
            dbObject.put(ZIP_CODE, address.getZipCode());
        }

        return dbObject;
    }


    public Orders convertFromDbObjectList(List<DBObject>  dbObjectList) {
        Orders      orders = new Orders();
        Order       order;
        Iterator<DBObject>    objectIterator;


        objectIterator = dbObjectList.iterator();
        while(objectIterator.hasNext()) {
            DBObject    dbObject = (DBObject)objectIterator.next();

            order = new Order();
            order.setErrorMessage((String)dbObject.get(ERROR_MESSAGE));
            order.setOrderDetail((String)dbObject.get(ORDER_DETAIL));
            order.setOrderId((String)dbObject.get(ORDER_ID));
            order.setOrderSubType((String)dbObject.get(ORDER_SUB_TYPE));
            order.setOrderText((String)dbObject.get(ORDER_TEXT));
            order.setOrderType((String)dbObject.get(ORDER_TYPE));
            order.setOrderTypeId((String)dbObject.get(ORDER_TYPE_ID));
            order.setStatus((String)dbObject.get(STATUS));
            order.setVerifyingClerk((String)dbObject.get(VERIFYING_CLERK));
            order.setVerifyingNurse((String)dbObject.get(VERIFYING_NURSE));
            order.setAssigningAuthority((String)dbObject.get(ASSIGNING_AUTHORITY));
            order.setPatientId((String)dbObject.get(PATIENT_ID));
//            order.setObjectType(dbObject.get(OBJECT_TYPE));
            order.setSourceSystem((String)dbObject.get(SOURCE_SYSTEM));
//            dbObject.get(SURROGATE_ID, order.setSurrogateId());
            order.setSystemId((String)dbObject.get(SYSTEM_ID));
            order.setUniqueId((String)dbObject.get(UNIQUE_ID));
//            dbObject.get(DATE, order.setDate());
            order.setDateSigned((Date)dbObject.get(DATE_SIGNED));
            order.setDateVerified((Date)dbObject.get(DATE_VERIFIED));
            order.setOrderedDate((Date)dbObject.get(ORDERED_DATE));
            order.setStartDate((Date)dbObject.get(START_DATE));
            order.setStopDate((Date)dbObject.get(STOP_DATE));

            orders.add(order);
        }

        return orders;
    }
}
