package com.agilex.healthcare.mobilehealthplatform.translator;

import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.utils.StringUtil;
import com.agilex.healthcare.utility.DateHelper;
import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

public class ReportDocumentMongoTranslator {
    public static final String      DISCHARGE_SUMMARY = "DISCHARGE SUMMARY";
    public static final String      PATHOLOGY_REPORT = "PATHOLOGY REPORT";
    public static final String      PROGRESS_NOTE = "PROGRESS NOTE";
    public static final String      RADIOLOGY_REPORT = "RADIOLOGY REPORT";
    public static final String      SURGERY_REPORT = "SURGERY REPORT";

    private static final String       DOCUMENT_TYPE = "documentType";
    private static final String       ID = "recordId";
    private static final String       TITLE = "title";
    private static final String       TIMESTAMP = "timestamp";
    private static final String       ENTRY_DATE = "entryDate";
    private static final String       DISCHARGE_DATE = "dischargeDate";
    private static final String       LOCAL_TITLE = "localTitle";
    private static final String       STANDARD_TITLE = "standardTitle";

    private static final String       AUTHOR = "author";
    private static final String       AUTHOR_ID = "authorId";
    private static final String       AUTHOR_NAME = "authorName";
    private static final String       AUTHOR_SIGNATURE = "authorSignature";
    private static final String       APPROVED_BY = "approvedBy";

    private static final String       TEXT = "text";
	private static final String       LOCATION = "location";

	private static final String       FACILITY = "facility";
	private static final String       FACILITY_NAME = "facilityName";
	private static final String       NAME = "name";
	private static final String       TYPE = "type";
	private static final String       STATE = "state";
	private static final String       CITY = "city";
	private static final String       ADDRESS = "address";
	private static final String       PARENT_SITE_CODE = "parentSiteCode";

	private static final String       SPECIMEN = "specimen";
	private static final String       SPECIMEN_ID = "specimenId";
	private static final String       SPECIMEN_NAME = "specimenName";
	private static final String       SPECIMEN_COLLECTION_DATE = "specimenCollectionDate";
	private static final String       SPECIMEN_ACCESSION_NUMBER = "specimenAccessionNumber";

	private static final String       EXAM = "exam";

	private static final String       TEST_ID = "testId";
	private static final String       REPORT = "report";
	private static final String       TEST_NAME = "testName";
	private static final String       SOURCE_SYSTEM = "sourceSystem";
	private static final String       RESULTED_DATE = "resultedDate";

    public ArrayList<DBObject> translateReportDocumentListToArrayList(ReportDocuments reportDocuments) {
        ArrayList<DBObject>       dbObjectList = new ArrayList<DBObject>();
        Iterator                  documentIterator;

        documentIterator = reportDocuments.getReportDocument().iterator();
        while(documentIterator.hasNext()) {
            ReportDocument      document = (ReportDocument)documentIterator.next();

            if(document.getDocumentType().equals(DISCHARGE_SUMMARY)) {
                dbObjectList.add(translateDischargeSummaryOrProgessNotes(document));
            }
            else if(document.getDocumentType().equals(PATHOLOGY_REPORT)) {
                dbObjectList.add(translatePathologyReport(document));
            }
            else if(document.getDocumentType().equals(PROGRESS_NOTE)) {
                dbObjectList.add(translateDischargeSummaryOrProgessNotes(document));
            }
            else if(document.getDocumentType().equals(RADIOLOGY_REPORT)) {
                dbObjectList.add(translateRadiologyReport(document));
            }
            else if(document.getDocumentType().equals(SURGERY_REPORT)) {
                dbObjectList.add(translateSurgeryReport(document));
            }

        }

        return dbObjectList;
    }

    public ReportDocuments translateArrayListToReportDocumentList(List<DBObject> dbObjectList) {
        ReportDocuments       reportDocuments = new ReportDocuments();
        Iterator              documentIterator;

        documentIterator = dbObjectList.iterator();
        while(documentIterator.hasNext()) {
            DBObject    dbObject = (DBObject)documentIterator.next();

            if(dbObject.get(DOCUMENT_TYPE).equals(DISCHARGE_SUMMARY)) {
                reportDocuments.add(translateToDischargeSummaryOrProgessNotes(dbObject));
            }
            else if(dbObject.get(DOCUMENT_TYPE).equals(PATHOLOGY_REPORT)) {
                reportDocuments.add(translateToPathologyReport(dbObject));
            }
            else if(dbObject.get(DOCUMENT_TYPE).equals(PROGRESS_NOTE)) {
                reportDocuments.add(translateToDischargeSummaryOrProgessNotes(dbObject));
            }
            else if(dbObject.get(DOCUMENT_TYPE).equals(RADIOLOGY_REPORT)) {
                reportDocuments.add(translateToRadiologyReport(dbObject));
            }
            else if(dbObject.get(DOCUMENT_TYPE).equals(SURGERY_REPORT)) {
                reportDocuments.add(translateToSurgeryReport(dbObject));
            }

        }

        return reportDocuments;
    }

    public DBObject translateDischargeSummaryOrProgessNotes(ReportDocument reportDocument){
    	DBObject    dbObject = new BasicDBObject();

    	dbObject.put(ID, reportDocument.getRecordId());
    	dbObject.put(TIMESTAMP, reportDocument.getTimestamp());
    	dbObject.put(ENTRY_DATE, reportDocument.getEntryDate());
    	dbObject.put(DISCHARGE_DATE, reportDocument.getDischargeDate());
    	dbObject.put(LOCAL_TITLE, reportDocument.getLocalTitle());
    	dbObject.put(STANDARD_TITLE,reportDocument.getStandardTitle());
    	if(reportDocument.getAuthor()!=null){
    		dbObject.put(AUTHOR, translateAuthor(reportDocument.getAuthor()));
    	}
        dbObject.put(TEXT, reportDocument.getText());
        if(reportDocument.getApprovedBy()!=null){
        	dbObject.put(APPROVED_BY, translateAuthor(reportDocument.getApprovedBy()));
        }
        dbObject.put(DOCUMENT_TYPE, reportDocument.getDocumentType());
        return dbObject;
	}

	public ReportDocument translateToDischargeSummaryOrProgessNotes(DBObject dbObject){
		ReportDocument      reportDocument = new ReportDocument();

		reportDocument.setRecordId(StringUtil.objectToString(dbObject.get(ID)));
		reportDocument.setTimestamp((Date)dbObject.get(TIMESTAMP));
		reportDocument.setEntryDate((Date)dbObject.get(ENTRY_DATE));
		reportDocument.setDischargeDate((Date)dbObject.get(DISCHARGE_DATE));
		reportDocument.setLocalTitle(StringUtil.objectToString(dbObject.get(LOCAL_TITLE)));
		reportDocument.setStandardTitle(StringUtil.objectToString(dbObject.get(STANDARD_TITLE)));
		if(dbObject.get(AUTHOR)!=null){
			reportDocument.setAuthor(translateToAuthor((BasicDBObject)dbObject.get(AUTHOR)));
		}
		reportDocument.setText(StringUtil.objectToString(dbObject.get(TEXT)));
		if(dbObject.get(APPROVED_BY)!=null){
			reportDocument.setApprovedBy(translateToAuthor((BasicDBObject)dbObject.get(APPROVED_BY)));
		}
		reportDocument.setDocumentType(StringUtil.objectToString(dbObject.get(DOCUMENT_TYPE)));
		return reportDocument;
	}

	public DBObject translatePathologyReport(ReportDocument reportDocument){
		DBObject    dbObject = new BasicDBObject();
		dbObject.put(TITLE, reportDocument.getTitle());
		dbObject.put(TIMESTAMP,reportDocument.getTimestamp());
		if(reportDocument.getFacility()!=null){
			dbObject.put(FACILITY,translateFacility(reportDocument.getFacility()));
		}
		if(reportDocument.getSpecimen()!=null){
			dbObject.put(SPECIMEN,translateSpecimen(reportDocument.getSpecimen()));
		}
		dbObject.put(EXAM, reportDocument.getExam());
	    dbObject.put(DOCUMENT_TYPE, reportDocument.getDocumentType());
	    return dbObject;
	}

	public ReportDocument translateToPathologyReport(DBObject dbObject){
		ReportDocument      reportDocument = new ReportDocument();
		reportDocument.setTitle(StringUtil.objectToString(dbObject.get(TITLE)));
		reportDocument.setTimestamp((Date)dbObject.get(TIMESTAMP));
		if(dbObject.get(FACILITY)!=null){
			reportDocument.setFacility(translateToFacility((BasicDBObject)dbObject.get(FACILITY)));
		}
		if(dbObject.get(SPECIMEN)!=null){
			reportDocument.setSpecimen(translateToSpecimen((BasicDBObject)(dbObject.get(SPECIMEN))));
		}
		reportDocument.setExam(StringUtil.objectToString(dbObject.get(EXAM)));
		reportDocument.setDocumentType(StringUtil.objectToString(dbObject.get(DOCUMENT_TYPE)));
		return reportDocument;
	}

	public DBObject translateRadiologyReport(ReportDocument reportDocument){
		DBObject    dbObject = new BasicDBObject();
		dbObject.put(TITLE, reportDocument.getTitle());
		dbObject.put(ID, reportDocument.getRecordId());
		dbObject.put(TIMESTAMP,reportDocument.getTimestamp());
		dbObject.put(FACILITY_NAME, reportDocument.getFacilityName());
		dbObject.put(TEST_ID,reportDocument.getTestId());
		dbObject.put(REPORT,reportDocument.getReport());
		dbObject.put(TEST_NAME,reportDocument.getTestName());
		dbObject.put(SOURCE_SYSTEM, reportDocument.getSourceSystem());
		dbObject.put(RESULTED_DATE, DateHelper.formatDateTime(reportDocument.getResultedDate()));
		dbObject.put(DOCUMENT_TYPE, reportDocument.getDocumentType());
		return dbObject;

	}
	public ReportDocument translateToRadiologyReport(DBObject dbObject){
		ReportDocument      reportDocument = new ReportDocument();
		reportDocument.setRecordId(StringUtil.objectToString(dbObject.get(ID)));
		reportDocument.setTitle(StringUtil.objectToString(dbObject.get(TITLE)));
		reportDocument.setTimestamp((Date)dbObject.get(TIMESTAMP));
		reportDocument.setFacilityName(StringUtil.objectToString(dbObject.get(FACILITY_NAME)));
		reportDocument.setTestId(StringUtil.objectToString(dbObject.get(TEST_ID)));
		reportDocument.setReport(StringUtil.objectToString(dbObject.get(REPORT)));
		reportDocument.setTestName(StringUtil.objectToString(dbObject.get(TEST_NAME)));
		reportDocument.setSourceSystem(StringUtil.objectToString(dbObject.get(SOURCE_SYSTEM)));
		reportDocument.setResultedDate(DateHelper.parseDateTime((String)dbObject.get(RESULTED_DATE)));
		reportDocument.setDocumentType(StringUtil.objectToString(dbObject.get(DOCUMENT_TYPE)));
		return reportDocument;
	}

	public DBObject translateSurgeryReport(ReportDocument reportDocument){
		DBObject    dbObject = new BasicDBObject();
		dbObject.put(TITLE, reportDocument.getTitle());
		dbObject.put(ID, reportDocument.getRecordId());
		dbObject.put(TIMESTAMP,reportDocument.getTimestamp());
		if(reportDocument.getAuthor()!=null){
			dbObject.put(AUTHOR, translateAuthor(reportDocument.getAuthor()));
		}
		dbObject.put(TEXT,reportDocument.getText());
		dbObject.put(LOCATION, reportDocument.getLocation());
		dbObject.put(DOCUMENT_TYPE, reportDocument.getDocumentType());
		return dbObject;
	}
	public ReportDocument translateToSurgeryReport(DBObject dbObject){
		ReportDocument      reportDocument = new ReportDocument();
		reportDocument.setRecordId(StringUtil.objectToString(dbObject.get(ID)));
		reportDocument.setTitle(StringUtil.objectToString(dbObject.get(TITLE)));
		reportDocument.setTimestamp((Date)dbObject.get(TIMESTAMP));
		reportDocument.setText(StringUtil.objectToString(dbObject.get(TEXT)));
		if(dbObject.get(AUTHOR)!=null){
			reportDocument.setAuthor(translateToAuthor((BasicDBObject)dbObject.get(AUTHOR)));
		}
		reportDocument.setLocation(StringUtil.objectToString(dbObject.get(LOCATION)));
		reportDocument.setDocumentType(StringUtil.objectToString(dbObject.get(DOCUMENT_TYPE)));
		return reportDocument;
	}

    private DBObject translateAuthor(Author author){
    	DBObject    dbObject = new BasicDBObject();
    	dbObject.put(AUTHOR_ID, author.getId());
    	dbObject.put(AUTHOR_NAME, author.getName());
    	dbObject.put(AUTHOR_SIGNATURE,author.getSignature());
    	return dbObject;
    }
    private Author translateToAuthor(DBObject dbObject){
    	Author      author = new Author();
    	author.setId(StringUtil.objectToString(dbObject.get(AUTHOR_ID)));
    	author.setName(StringUtil.objectToString(dbObject.get(AUTHOR_NAME)));
    	author.setSignature(StringUtil.objectToString(dbObject.get(AUTHOR_SIGNATURE)));
    	return author;
    }

	private DBObject translateFacility(Facility facility){
		DBObject    dbObject = new BasicDBObject();
		dbObject.put(NAME, facility.getName());
		dbObject.put(TYPE, facility.getType());
		dbObject.put(STATE, facility.getState());
		dbObject.put(CITY, facility.getCity());
		dbObject.put(ADDRESS, facility.getAddress());
		dbObject.put(PARENT_SITE_CODE, facility.getParentSiteCode());
		return dbObject;
	}
	private Facility translateToFacility(DBObject dbObject){
		Facility    facility = new Facility();
		facility.setName(StringUtil.objectToString(dbObject.get(NAME)));
		facility.setType(StringUtil.objectToString(dbObject.get(TYPE)));
		facility.setState(StringUtil.objectToString(dbObject.get(STATE)));
		facility.setCity(StringUtil.objectToString(dbObject.get(CITY)));
		facility.setAddress(StringUtil.objectToString(dbObject.get(ADDRESS)));
		facility.setParentSiteCode(StringUtil.objectToString(dbObject.get(PARENT_SITE_CODE)));
		return facility;
	}

	private DBObject translateSpecimen(Specimen specimen){
		DBObject    dbObject = new BasicDBObject();
		dbObject.put(SPECIMEN_ID, specimen.getId());
		dbObject.put(SPECIMEN_NAME, specimen.getName());
		dbObject.put(SPECIMEN_COLLECTION_DATE, DateHelper.formatDateTime(specimen.getCollectionDate()));
		dbObject.put(SPECIMEN_ACCESSION_NUMBER, specimen.getAccessionNumber());
		return dbObject;
	}
	private Specimen translateToSpecimen(DBObject dbObject){
		Specimen    specimen = new Specimen();
		specimen.setId(StringUtil.objectToString(dbObject.get(SPECIMEN_ID)));
		specimen.setName(StringUtil.objectToString(dbObject.get(SPECIMEN_NAME)));
		specimen.setCollectionDate(DateHelper.parseDateTime(StringUtil.objectToString(dbObject.get(SPECIMEN_COLLECTION_DATE))));
		specimen.setAccessionNumber(StringUtil.objectToString(dbObject.get(SPECIMEN_ACCESSION_NUMBER)));
		return specimen;
	}
}
