package com.agilex.healthcare.mobilehealthplatform.utils;

import java.net.URI;

import com.agilex.healthcare.mobilehealthplatform.domain.PatientData;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientDataCollection;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.Problem;
import com.agilex.healthcare.mobilehealthplatform.domain.Problems;
import com.agilex.healthcare.utility.NullChecker;

public class PatientIdentifierHelper {
	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(PatientIdentifierHelper.class);

	public static String getPatientIdFromUri(URI uri) {
		String id = null;
		if (uri != null)
			id = getPatientIdFromUri(uri.toString());
		return id;
	}

	public static String getPatientIdFromUri(String uri) {
		String id = null;
		if (NullChecker.isNotNullish(uri)) {
			final String patientPattern = "/patient/";
			int locationOfPattern = uri.indexOf(patientPattern);
			if (locationOfPattern > 0) {
				int idStart = locationOfPattern + patientPattern.length();
				id = uri.substring(idStart);
			}
		}
		return id;
	}

	public static void updatePatientIdentifier(PatientDataCollection<?> dataCollection, PatientIdentifier patientIdentifier) {
		logger.debug(String.format("updating patient identifier for collection [%s][%s] ", dataCollection, patientIdentifier));
		dataCollection.setPatientIdentifier(patientIdentifier);

		for (PatientData data : dataCollection) {
			updatePatientIdentifier(data, patientIdentifier);
		}
	}

	public static void updatePatientIdentifier(PatientData data, PatientIdentifier patientIdentifier) {
		if (data != null)
			data.setPatientIdentifier(patientIdentifier);
	}

	public static void setAssigningAuthority(PatientDataCollection<?> dataCollection, String assigningAuthority) {
		for (PatientData data : dataCollection) {
			setAssigningAuthority(data, assigningAuthority);
		}
	}

	public static void setAssigningAuthority(PatientData data, String assigningAuthority) {
		data.getPatientIdentifier().setAssigningAuthority(assigningAuthority);
	}

	public static void updateCommunity(Problems problems, String communityId) {
		for (Problem problem : problems) {
			problem.getDataIdentifier().setSystemId(communityId);
		}
	}

}
