package com.agilex.healthcare.mobilehealthplatform.utils.uriformaters;

import java.net.URI;

import javax.annotation.Resource;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response.Status;
import javax.ws.rs.core.UriBuilder;

import agilex.commons.helper.PropertyHelper;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import com.agilex.healthcare.mobilehealthplatform.domain.AtomLink;
import com.agilex.healthcare.mobilehealthplatform.domain.LinkTitles;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteWriterResourceDirectory;
import com.agilex.healthcare.utility.NullChecker;

@Service
@Scope(value=ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class ClinicalEventResourceDirectoryBuilder {

	private static String AUTO_SAVE_KEY="notewriter.autosaveIntervalinMinutes";

	@Resource
	private PropertyHelper propertyHelper;

	public ClinicalEventResourceDirectoryBuilder() {}

	public NoteWriterResourceDirectory getResourceDirectory(URI systemBaseUri, String facilityCode, String staffId, String assigningAuthority, String patientId) {

		if (NullChecker.isNullish(facilityCode)
				|| NullChecker.isNullish(staffId)
				|| NullChecker.isNullish(assigningAuthority)
				|| NullChecker.isNullish(patientId)) {
			throw new WebApplicationException(Status.BAD_REQUEST);
		}

		NoteWriterResourceDirectory directory = new NoteWriterResourceDirectory();
		int autoSaveInterval = Integer.parseInt(propertyHelper.getProperty(AUTO_SAVE_KEY));
		directory.setAutoSaveIntervalinMinutes(autoSaveInterval);

		AtomLink link;

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("clinical-event-service").path("site").path(facilityCode).path("id").path(staffId).path("resource-directory").queryParam("assigningAuthority", assigningAuthority).queryParam("patientId", patientId).build());
		link.setTitle("resource-directory");
		directory.getLink().setSelfLink(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("clinical-event-service").path("site").path(facilityCode).path("id").path(staffId).path("patient").path(assigningAuthority).path(patientId).path("clinical-events").build());
		link.setTitle(LinkTitles.ClinicalEvents);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("clinical-event-service").path("site").path(facilityCode).path("id").path(staffId).path("patient").path(assigningAuthority).path(patientId).path("clinical-events").path("count").build());
		link.setTitle(LinkTitles.ClinicalEventsCount);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("clinical-event-service").path("site").path(facilityCode).path("id").path(staffId).path("patient").path(assigningAuthority).path(patientId).path("clinical-events").path("sign").build());
		link.setTitle(LinkTitles.SignNote);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("clinical-event-service").path("site").path(facilityCode).path("id").path(staffId).path("note-titles").path("progress-notes").build());
		link.setTitle(LinkTitles.NoteTitles);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("clinical-event-service").path("site").path(facilityCode).path("id").path(staffId).path("note-titles").path("mdws-progress-notes").build());
		link.setTitle(LinkTitles.MdwsNoteTitles);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("clinical-event-service").path("site").path(facilityCode).path("id").path(staffId).path("note-title").path("is-consult").build());
		link.setTitle(LinkTitles.NoteTitleIsConsult);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("clinical-event-service").path("site").path(facilityCode).path("id").path(staffId).path("note-title").path("is-patient-record-flag").build());
		link.setTitle(LinkTitles.NoteTitleIsPRF);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("clinical-event-service").path("site").path(facilityCode).path("id").path(staffId).path("note-title").path("is-one-visit-note").build());
		link.setTitle(LinkTitles.NoteTitleIsOneVisitNote);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("clinical-event-service").path("site").path(facilityCode).path("id").path(staffId).path("note-title").path("is-surgery-note").build());
		link.setTitle(LinkTitles.NoteTitleIsSurgeryNote);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("clinical-event-service").path("site").path(facilityCode).path("id").path(staffId).path("note-titles").path("last-selected-progress-note-title").build());
		link.setTitle(LinkTitles.LastSelectedNoteTitle);
		directory.getLink().add(link);

		// encounter form service
		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("encounter-form-service").path("site").path(facilityCode).path("providers").build());
		link.setTitle(LinkTitles.Providers);
		directory.getLink().add(link);
		
		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("encounter-form-service").path("site").path(facilityCode).path("mdws-providers").build());
		link.setTitle(LinkTitles.MdwsProviders);
		directory.getLink().add(link);
		
		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("encounter-form-service").path("site").path(facilityCode).path("visit-types").build());
		link.setTitle(LinkTitles.VisitTypes);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("encounter-form-service").path("site").path(facilityCode).path("id").path(staffId).path("patient").path(assigningAuthority).path(patientId).path("visit-related-to").build());
		link.setTitle(LinkTitles.VisitRelatedTo);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("encounter-form-service").path("site").path(facilityCode).path("procedures").build());
		link.setTitle(LinkTitles.Procedures);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("encounter-form-service").path("site").path(facilityCode).path("modifiers").build());
		link.setTitle(LinkTitles.Modifiers);
		directory.getLink().add(link);

		link = new AtomLink();
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("encounter-form-service").path("site").path(facilityCode).path(LinkTitles.Diagnoses).build());
		link.setTitle(LinkTitles.Diagnoses);
		directory.getLink().add(link);

		link = new AtomLink();
		//                                                                                                      site/{facility-code}/id/{staff-id}/patient/{assigning-authority}/{patient-id}/problem-list-diagnoses
		link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("encounter-form-service").path("site").path(facilityCode).path("id").path(staffId).path("patient").path(assigningAuthority).path(patientId).path("problem-list-diagnoses").build());
		link.setTitle(LinkTitles.ProblemListDiagnoses);
		directory.getLink().add(link);

        // patient
        link = new AtomLink();
        link.setHref(UriBuilder.fromUri(systemBaseUri).path("note-writer").path("patient").path(assigningAuthority).path(patientId).path("appointments").path("visits").build());
        link.setTitle(LinkTitles.NoteWriterClinicAppointments);
        directory.getLink().add(link);

        return directory;
	}

}