package com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import com.agilex.healthcare.mobilehealthplatform.domain.AtomLink;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEvent;
import com.agilex.healthcare.mobilehealthplatform.domain.LinkTitles;
import com.agilex.healthcare.mobilehealthplatform.domain.Note;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;

public class ClinicalEventLinkBuilder {
	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(ClinicalEventLinkBuilder.class);

	private URI baseUri;

	public ClinicalEventLinkBuilder(URI baseUri) {
		this.baseUri = baseUri;
	}

	public URI createClinicalEventSelfUri(String facilityCode, String staffId, PatientIdentifier patientIdentifier, String clinicalEventId) {
		URI retValue = UriBuilder.fromUri(this.baseUri).path("note-writer").path("site").path(facilityCode).path("id").path(staffId).path("patient").path(patientIdentifier.getAssigningAuthority()).path(patientIdentifier.getUniqueId()).path("clinical-events").path("id").path(clinicalEventId).build();
		if (LOGGER.isDebugEnabled())
			LOGGER.debug(retValue.toString());
		return retValue;
	}
	
	public void fillLinks(ClinicalEvent clinicalEvent) {
		ClinicalEventLinkBuilder linkBuilder = new ClinicalEventLinkBuilder(baseUri);
		clinicalEvent.setSelfUri(linkBuilder.createClinicalEventSelfUri(clinicalEvent.getProviderIdentifier().getFacilityCode(), clinicalEvent.getProviderIdentifier().getStaffId(), clinicalEvent.getPatientIdentifier(), clinicalEvent.getId()));
		
		Note note = clinicalEvent.getNote();
		NoteWriterLinkBuilder noteLinkBuilder = new NoteWriterLinkBuilder(baseUri);
		String facilityCode = clinicalEvent.getProviderIdentifier().getFacilityCode();
		String staffId = clinicalEvent.getProviderIdentifier().getStaffId();
		PatientIdentifier patientIdentifier = new PatientIdentifier(clinicalEvent.getPatientIdentifier().getAssigningAuthority(), clinicalEvent.getPatientIdentifier().getUniqueId());
		
		if (note.getSignedDate() == null) {
			appendRelatedLink(note, LinkTitles.SignNote, noteLinkBuilder.createInProgressNoteSignUri(facilityCode, staffId, patientIdentifier, clinicalEvent.getId()));
		}
	}
	
	private AtomLink appendRelatedLink(Note note, String uriTitle, URI uri) {
		return note.getLink().addRelatedLink(uriTitle, uri);
	}
}