package com.agilex.healthcare.notewriter.datalayer;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import com.agilex.healthcare.mongo.MongoClientFactory;
import com.agilex.healthcare.mongo.SharedMongoClient;
import com.agilex.healthcare.notewriter.datalayer.procedure.ProceduresTranslator;
import com.agilex.healthcare.notewriter.domain.EncounterCodes;
import com.agilex.healthcare.utility.NullChecker;
import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;


public abstract class ClinicalCodeDataLayerMongo implements ClinicalCodeDataLayer{

    private final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(ClinicalCodeDataLayerMongo.class);

    private DBCollection collection;
    private String codeType;
    private DB db;
    private String field2SearchIfNotFoundInKey;
    public static final String FACILITY_CODE_KEY = "FacilityCode";

//    @Resource
//    private PropertyHelper propertyHelper = null;

    /**
     *
     * @param collectionName Which collection you want to search in.
     * @param field2SearchIfNotFoundInKey This is the field that will be used in an or statement.
     * The findCodes methods attempt to look for the search string within {@link CodeTranslator.CODE_KEY}
     * or within the field passed in here.
     */
    public ClinicalCodeDataLayerMongo(MongoClient mongoClient,String collectionName, String field2SearchIfNotFoundInKey){
        try {
        	this.codeType = collectionName.toUpperCase();
            db = mongoClient.getDB("encounter");
            collection = db.getCollection(collectionName);
            this.field2SearchIfNotFoundInKey = field2SearchIfNotFoundInKey;
        } catch (com.mongodb.MongoException e) {
            LOGGER.error("Unable to connect to Mongo DB - Mongo Exception", e);
            throw new RuntimeException("Unable to connect to Mongo DB - Mongo Exception", e);
        }
    }

    public ClinicalCodeDataLayerMongo(String databaseName, String collectionName) {
        SharedMongoClient sharedMongoClient = MongoClientFactory.getInstance().getMongoClient();
        db = sharedMongoClient.getDB(databaseName);
        collection = db.getCollection(collectionName);
        collection.dropIndexes();
    }

	@Override
	public EncounterCodes findCodes(Integer facilityCode) {
		return findCodes(facilityCode, null);
	}

	@Override
	public EncounterCodes findCodes(Integer facilityCode, SearchFilter filter) {
		int maxRecords = 0;

		BasicDBObject query = new BasicDBObject(FACILITY_CODE_KEY, facilityCode);

		if (filter != null) {
			String searchString = filter.getSearchString();
			if (NullChecker.isNotNullish(searchString)) {
				List<BasicDBObject> obj = new ArrayList<BasicDBObject>();
				obj.add(new BasicDBObject(ProceduresTranslator.CODE_KEY, formatAsRegExp(searchString)));
				obj.add(new BasicDBObject(field2SearchIfNotFoundInKey, formatAsRegExp(searchString)));
				query.put("$or", obj);
			}

			maxRecords = filter.getMaxRecords();
		}

		DBCursor iterator = collection.find(query).limit(maxRecords);
		return  translateToCodes(iterator);
	}

    private EncounterCodes translateToCodes(DBCursor iterator){
    	EncounterCodes codes = new EncounterCodes();
    	ProceduresTranslator translator = new ProceduresTranslator();
        while(iterator.hasNext()) {
			DBObject code = iterator.next();
			codes.add(translator.translate(this.codeType, code));
		}

		return codes;
    }

    private Pattern formatAsRegExp(String searchName){
        return Pattern.compile(searchName, Pattern.CASE_INSENSITIVE);
    }

}
