package com.agilex.healthcare.notewriter.datalayer.facilityprovider;

import java.util.regex.Pattern;

import com.agilex.healthcare.mobilehealthplatform.domain.FacilityProviders;
import com.agilex.healthcare.mongo.MongoClientFactory;
import com.agilex.healthcare.mongo.SharedMongoClient;
import com.agilex.healthcare.notewriter.datalayer.FacilityProviderDataLayer;
import com.agilex.healthcare.notewriter.datalayer.SearchFilter;
import com.agilex.healthcare.utility.NullChecker;
import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;

public class FacilityProviderDataLayerMongo implements FacilityProviderDataLayer {


    private final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(FacilityProviderDataLayerMongo.class);

    private DBCollection collection;
    private DB db;
        
    /**
     *
     * @param collectionName Which collection you want to search in.
     * @param field2SearchIfNotFoundInKey This is the field that will be used in an or statement.
     * The findCodes methods attempt to look for the search string within {@link CodeTranslator.CODE_KEY}
     * or within the field passed in here.
     */

    public FacilityProviderDataLayerMongo(MongoClient mongoClient){
        try {
            db = mongoClient.getDB("note-writer");
            collection = db.getCollection("providers");
        } catch (com.mongodb.MongoException e) {
            LOGGER.error("Unable to connect to Mongo DB - Mongo Exception", e);
            throw new RuntimeException("Unable to connect to Mongo DB - Mongo Exception", e);
        }
    }

    public FacilityProviderDataLayerMongo(String databaseName, String collectionName) {
        SharedMongoClient sharedMongoClient = MongoClientFactory.getInstance().getMongoClient();
        db = sharedMongoClient.getDB(databaseName);
        collection = db.getCollection(collectionName);
        collection.dropIndexes();
    }

	@Override
	public FacilityProviders findFacilityProviders(String facilityCode) {
		return findFacilityProviders(facilityCode, null);
	}

	@Override
	public FacilityProviders findFacilityProviders(String facilityCode, SearchFilter filter) {
		int maxRecords = 0;

		BasicDBObject query = new BasicDBObject(FacilityProviderTranslator.FACILITY_CODE_KEY, facilityCode);

		if (filter != null) {
			String searchString = filter.getSearchString();
			if (NullChecker.isNotNullish(searchString)) {
				query.put(FacilityProviderTranslator.NAME_KEY, formatAsRegExp(searchString));
			}

			maxRecords = filter.getMaxRecords();
		}

		DBCursor iterator = collection.find(query).limit(maxRecords);
		return  translateToProviders(iterator);
	}

    private FacilityProviders translateToProviders(DBCursor iterator){
    	FacilityProviders providers = new FacilityProviders();
    	FacilityProviderTranslator translator = new FacilityProviderTranslator();
        while(iterator.hasNext()) {
			DBObject obj = iterator.next();
			providers.add(translator.translate(obj));
		}
		return providers;
    }

    private Pattern formatAsRegExp(String searchName){
        return Pattern.compile(searchName, Pattern.CASE_INSENSITIVE);
    }

}
