package com.agilex.healthcare.notewriter.translator;

import java.util.Date;
import java.util.Iterator;

import org.bson.types.ObjectId;

import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEvent;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEventState;
import com.agilex.healthcare.mobilehealthplatform.domain.Encounter;
import com.agilex.healthcare.mobilehealthplatform.domain.EncounterProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.EncounterProviders;
import com.agilex.healthcare.mobilehealthplatform.domain.FacilityProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.HealthFactor;
import com.agilex.healthcare.mobilehealthplatform.domain.HealthFactors;
import com.agilex.healthcare.mobilehealthplatform.domain.Modifier;
import com.agilex.healthcare.mobilehealthplatform.domain.Modifiers;
import com.agilex.healthcare.mobilehealthplatform.domain.Note;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitle;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitRelatedTo;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitRelatedTos;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitType;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitTypes;
import com.agilex.healthcare.mobilehealthplatform.utils.StringUtil;
import com.agilex.healthcare.notewriter.domain.EncounterDiagnosisICDCode;
import com.agilex.healthcare.notewriter.domain.EncounterDiagnosisICDCodes;
import com.agilex.healthcare.notewriter.domain.EncounterProcedureCPTCode;
import com.agilex.healthcare.notewriter.domain.EncounterProcedureCPTCodes;
import com.agilex.healthcare.utility.NullChecker;
import com.mongodb.BasicDBList;
import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;

public class ClinicalEventTranslator {
	public static final String ID = "id";
	public static final String FACILITY_CODE = "facilityCode";
	public static final String ELECTRONIC_SIGNATURE_CODE = "electronicSignatureCode";
	public static final String LAST_MODIFIED_DATE = "lastModifiedDate";
	public static final String CLINICAL_EVENT_STATE = "clinicalEventState";
	public static final String ENCOUNTER_EDITED = "encounterEdited";
	public static final String ENCOUNTER_VALID = "encounterValid";
	

	public static final String PATIENT_IDENTIFIER = "patientIdentifier";
	public static final String PATIENT_ASSIGNING_AUTHORITY = "assigningAuthority";
	public static final String PATIENT_UNIQUE_ID = "uniqueId";
	public static final String PATIENT_FIRST_NAME = "firstName";
	public static final String PATIENT_LAST_NAME = "lastName";
	public static final String PATIENT_SSN = "ssn";

	public static final String PROVIDER_IDENTIFIER = "providerIdentifier";
	public static final String PROVIDER_FACILITY_CODE = "facilityCode";
	public static final String PROVIDER_STAFF_ID = "staffId";

    public static final String FIRST_NAME = "firstName";
    public static final String LAST_NAME = "lastName";
    public static final String SSN = "ssn";

	public static final String NOTE = "note";
	public static final String NOTE_TITLE = "title";
	public static final String NOTE_TITLE_FACILITY = "facility";
	public static final String NOTE_TITLE_IEN = "ien";
	public static final String NOTE_TITLE_NAME = "name";
	public static final String NOTE_TITLE_IS_CONSULT = "isConsult";
	public static final String NOTE_TITLE_IS_PRF = "isPrf";
	public static final String NOTE_BODY = "body";
	public static final String NOTE_SIGNED_DATE = "signedDate";
	public static final String NOTE_CONSULT_IEN = "consultIen";

	public static final String ENCOUNTER = "encounter";
	public static final String ENCOUNTER_TIMESTAMP = "timestamp";
	public static final String ENCOUNTER_RESUME_DATE = "date";
	public static final String ENCOUNTER_RESUME_TIME = "time";
	public static final String ENCOUNTER_LOCATION_ID = "locationId";
	public static final String ENCOUNTER_LOCATION_TITLE = "locationTitle";
	public static final String ENCOUNTER_CURRENT_ADMISSION = "currentAdmission";
	public static final String ENCOUNTER_IS_HISTORICAL = "isHistorical";
	public static final String ENCOUNTER_SERVICE_CATEGORY = "serviceCategory";
	public static final String ENCOUNTER_APPOINTMENT_TYPE = "appointmentType";
	public static final String ENCOUNTER_LINK_TYPE = "linkType";
	public static final String ENCOUNTER_ENCOUNTER_TYPE = "encounterType";

	public static final String ENCOUNTER_VISIT_RELATED_TOS = "visitRelatedTos";
	public static final String ENCOUNTER_VIS_TITLE = "title";
	public static final String ENCOUNTER_VIS_CHECKED = "checked";
	public static final String ENCOUNTER_VIS_ENABLED = "enabled";
	public static final String ENCOUNTER_VIS_CODE = "code";

	public static final String ENCOUNTER_PROVIDERS = "providers";
	public static final String ENCOUNTER_PROV_PRIMARY = "primary";
	public static final String ENCOUNTER_PROV_SELECTED = "selected";
	public static final String ENCOUNTER_PROV_PROVIDER = "facilityProvider";
	public static final String ENCOUNTER_PROV_IDENTIFIER = "providerIdentifier";
	public static final String ENCOUNTER_PROV_FACILITY_CODE = "facilityCode";
	public static final String ENCOUNTER_PROV_STAFF_ID = "providerIen";
	public static final String ENCOUNTER_PROV_NAME = "providerName";
	
	public static final String ENCOUNTER_PROCEDURE_CODES = "procedureCodes";
	public static final String ENCOUNTER_DIAGNOSIS_CODES = "diagnosisCodes";
	public static final String ENCOUNTER_VISIT_TYPES = "visitTypes";
	public static final String ENCOUNTER_VISIT_TYPES_NAME = "name";
	public static final String ENCOUNTER_VISIT_TYPES_SELECTED = "selected";
	public static final String ENCOUNTER_VISIT_TYPES_SECTIONS = "sections";
	public static final String ENCOUNTER_VISIT_TYPES_SECTIONS_SELECTED = "selected";
	public static final String ENCOUNTER_VISIT_TYPES_SECTIONS_MODIFIERS_SELECTED = "selected";
	public static final String ENCOUNTER_CODE = "code";
	public static final String ENCOUNTER_CODE_TYPE = "codeType";
	public static final String ENCOUNTER_CODE_DESCRIPTION = "description";
	public static final String ENCOUNTER_CODE_IEN = "ien";
	public static final String ENCOUNTER_CODE_NAME = "name";
	public static final String ENCOUNTER_CODE_SHORT_DESCRIPTION = "shortDescription";
	
	public static final String ENCOUNTER_DIAGNOSIS_ADD_TO_PROBLEM_LIST = "addToProblemList";
	public static final String ENCOUNTER_DIAGNOSIS_PRIMARY_DIAGNOSIS = "primaryDiagnosis";
	public static final String ENCOUNTER_DIAGNOSIS_FROM_PROBLEM_LIST = "fromProblemList";
	
	public static final String ENCOUNTER_CPTCODE_MODIFIERS = "modifiers";

    public static final String HEALTH_FACTORS = "healthFactors";
    public static final String HEALTH_FACTOR_IEN = "healthFactorIen";
    public static final String HEALTH_FACTOR_CODE = "healthFactorCode";
    public static final String HEALTH_FACTOR_DESCRIPTION = "healthFactorDescription";
    public static final String HEALTH_FACTOR_COMMENT = "healthFactorComment";
    public static final String HEALTH_FACTOR_PARENT_CATEGORY = "parentCategory";
    public static final String HEALTH_FACTOR_CATEGORY = "category";
    public static final String HEALTH_FACTOR_SEVERITY = "healthFactorSeverity";

	public ClinicalEvent translateClinicalEvent(DBObject clinicalEventPo) {
		ClinicalEvent clinicalEvent = null;

		if (clinicalEventPo != null) {
			clinicalEvent = new ClinicalEvent();

			clinicalEvent.setId(StringUtil.objectToString(clinicalEventPo.get(ID)));
			clinicalEvent.setClinicalEventState(ClinicalEventState.fromValue((StringUtil.objectToString(clinicalEventPo.get(CLINICAL_EVENT_STATE)))));
			clinicalEvent.setFacilityCode(StringUtil.objectToString(clinicalEventPo.get(FACILITY_CODE)));
			clinicalEvent.setEncounterEdited(StringUtil.objectToBoolean(clinicalEventPo.get(ENCOUNTER_EDITED)));
			clinicalEvent.setEncounterValid(StringUtil.objectToBoolean(clinicalEventPo.get(ENCOUNTER_VALID)));

			PatientIdentifier patientIdentifier = new PatientIdentifier();
			clinicalEvent.setPatientIdentifier(patientIdentifier);
			clinicalEvent.getPatientIdentifier().setAssigningAuthority(StringUtil.objectToString(((DBObject)clinicalEventPo.get(PATIENT_IDENTIFIER)).get(PATIENT_ASSIGNING_AUTHORITY)));
			clinicalEvent.getPatientIdentifier().setUniqueId(StringUtil.objectToString(((DBObject)clinicalEventPo.get(PATIENT_IDENTIFIER)).get(PATIENT_UNIQUE_ID)));
			clinicalEvent.getPatientIdentifier().setFirstName(StringUtil.objectToString(((DBObject)clinicalEventPo.get(PATIENT_IDENTIFIER)).get(FIRST_NAME)));
			clinicalEvent.getPatientIdentifier().setLastName(StringUtil.objectToString(((DBObject)clinicalEventPo.get(PATIENT_IDENTIFIER)).get(LAST_NAME)));
			clinicalEvent.getPatientIdentifier().setSsn(StringUtil.objectToString(((DBObject)clinicalEventPo.get(PATIENT_IDENTIFIER)).get(SSN)));

			ProviderIdentifier providerIdentifier = new ProviderIdentifier();
			clinicalEvent.setProviderIdentifier(providerIdentifier);
			clinicalEvent.getProviderIdentifier().setFacilityCode(StringUtil.objectToString(((DBObject)clinicalEventPo.get(PROVIDER_IDENTIFIER)).get(PROVIDER_FACILITY_CODE)));
			clinicalEvent.getProviderIdentifier().setStaffId(StringUtil.objectToString(((DBObject)clinicalEventPo.get(PROVIDER_IDENTIFIER)).get(PROVIDER_STAFF_ID)));

			//If it's null, translateNote will return null.
			clinicalEvent.setNote(translateNote((DBObject)clinicalEventPo.get(NOTE)));

			//If it's null, translateEncounter will return null.
			clinicalEvent.setEncounter(translateEncounter((DBObject)clinicalEventPo.get(ENCOUNTER)));

			Date lastModifiedDate = (Date)clinicalEventPo.get(LAST_MODIFIED_DATE);
			if (NullChecker.isNotNullish(lastModifiedDate)) {
				clinicalEvent.setLastModifiedDate(lastModifiedDate);
			}
            if(null!=clinicalEventPo.get(HEALTH_FACTORS))
                clinicalEvent.setHealthFactors(populateHealthFactors((BasicDBList)clinicalEventPo.get(HEALTH_FACTORS)));
		}
		return clinicalEvent;
	}

	public DBObject translateClinicalEvent(ClinicalEvent clinicalEvent) {
		DBObject dbObject = new BasicDBObject(8);

		//id
		if (NullChecker.isNullish(clinicalEvent.getId()))
			dbObject.put(ID, new ObjectId().toString());
		else
			dbObject.put(ID, clinicalEvent.getId());

		//facility code
		dbObject.put(FACILITY_CODE, clinicalEvent.getFacilityCode());
		dbObject.put(ENCOUNTER_VALID, clinicalEvent.isEncounterValid());
		dbObject.put(ENCOUNTER_EDITED, clinicalEvent.isEncounterEdited());

		// last modified date
		Date lastModifiedDate = clinicalEvent.getLastModifiedDate();
		if (lastModifiedDate != null) {
			dbObject.put(LAST_MODIFIED_DATE, clinicalEvent.getLastModifiedDate());
		}

		dbObject.put(CLINICAL_EVENT_STATE, clinicalEvent.getClinicalEventState().toString());

		//patientIdentifier
		DBObject patientDbObject = new BasicDBObject(5);
		patientDbObject.put(PATIENT_UNIQUE_ID, clinicalEvent.getPatientIdentifier().getUniqueId());
		patientDbObject.put(PATIENT_ASSIGNING_AUTHORITY, clinicalEvent.getPatientIdentifier().getAssigningAuthority());
		patientDbObject.put(PATIENT_FIRST_NAME, clinicalEvent.getPatientIdentifier().getFirstName());
		patientDbObject.put(PATIENT_LAST_NAME, clinicalEvent.getPatientIdentifier().getLastName());
		patientDbObject.put(PATIENT_SSN, clinicalEvent.getPatientIdentifier().getSsn());
		dbObject.put(PATIENT_IDENTIFIER, patientDbObject);

		//providerIdentifier
		DBObject providerDbObject = new BasicDBObject(2);
		providerDbObject.put(PROVIDER_FACILITY_CODE, clinicalEvent.getProviderIdentifier().getFacilityCode());
		providerDbObject.put(PROVIDER_STAFF_ID, clinicalEvent.getProviderIdentifier().getStaffId());
		dbObject.put(PROVIDER_IDENTIFIER, providerDbObject);

		if (clinicalEvent.getNote() != null)
			dbObject.put(NOTE, translateNote(clinicalEvent.getNote()));

		if (clinicalEvent.getEncounter() != null)
			dbObject.put(ENCOUNTER, translateEncounter(clinicalEvent.getEncounter()));

        if(null != clinicalEvent.getHealthFactors())
            dbObject.put(HEALTH_FACTORS, populateDbObjectHealthFactors(clinicalEvent.getHealthFactors()));

		return dbObject;
	}

	public Note translateNote(DBObject notePo) {
		Note note = null;

		if (notePo != null) {
			note = new Note();

			note.setTitle(translateTitle((DBObject)notePo.get(NOTE_TITLE)));

			String body = StringUtil.objectToString(notePo.get(NOTE_BODY));
			note.setBody(body == null ? "" : body);

			String consultIen = StringUtil.objectToString(notePo.get(NOTE_CONSULT_IEN));
			note.setConsultIen(consultIen);
			
			Date signedDate = (Date)notePo.get(NOTE_SIGNED_DATE);
			if (NullChecker.isNotNullish(signedDate)) {
				note.setSignedDate(signedDate);
			}
		}

		return note;
	}

	protected DBObject translateNote(Note note) {
		DBObject dbObject = new BasicDBObject(3);

		dbObject.put(NOTE_TITLE, translateTitle(note.getTitle()));
		dbObject.put(NOTE_BODY, note.getBody() == null ? "" : note.getBody());
		dbObject.put(NOTE_CONSULT_IEN, note.getConsultIen());
		if (note.getSignedDate() != null) {
			dbObject.put(NOTE_SIGNED_DATE, note.getSignedDate());
		}

		return dbObject;
	}

	public Encounter translateEncounter(DBObject encounterPo) {
		Encounter encounter = null;

		if (encounterPo != null) {
			encounter = new Encounter();

			Date timeStamp = (Date)encounterPo.get(ENCOUNTER_TIMESTAMP);
			if (NullChecker.isNotNullish(timeStamp)) {
				encounter.setTimestamp(timeStamp);
			}
			
			encounter.setDate(StringUtil.objectToString(encounterPo.get(ENCOUNTER_RESUME_DATE)));
			encounter.setTime(StringUtil.objectToString(encounterPo.get(ENCOUNTER_RESUME_TIME)));
			
			encounter.setLocationId(StringUtil.objectToString(encounterPo.get(ENCOUNTER_LOCATION_ID)));
			encounter.setLocationTitle(StringUtil.objectToString(encounterPo.get(ENCOUNTER_LOCATION_TITLE)));	
			encounter.setLinkType(StringUtil.objectToString(encounterPo.get(ENCOUNTER_LINK_TYPE)));
			encounter.setCurrentAdmission(StringUtil.objectToBoolean(encounterPo.get(ENCOUNTER_CURRENT_ADMISSION)));
			encounter.setHistorical(StringUtil.objectToBoolean(encounterPo.get(ENCOUNTER_IS_HISTORICAL)));			
			encounter.setServiceCategory(StringUtil.objectToString(encounterPo.get(ENCOUNTER_SERVICE_CATEGORY)));
			encounter.setAppointmentType(StringUtil.objectToString(encounterPo.get(ENCOUNTER_APPOINTMENT_TYPE)));
			encounter.setEncounterType(StringUtil.objectToString(encounterPo.get(ENCOUNTER_ENCOUNTER_TYPE)));

			//If it's null, populateVisitRelatedTos will return null.
			encounter.setVisitRelatedTos(populateVisitRelatedTos((BasicDBList)encounterPo.get(ENCOUNTER_VISIT_RELATED_TOS)));
			
			//If it's null, populateEncounterProviders will return null.
			encounter.setProviders(populateEncounterProviders((BasicDBList) encounterPo.get(ENCOUNTER_PROVIDERS)));
			
			//If it's null, populateVisitSectionCodes will return null.
			encounter.setVisitTypes(populateVisitTypes((BasicDBList) encounterPo.get(ENCOUNTER_VISIT_TYPES)));
			
			//If it's null, populateProcedureCodes will return null.
			encounter.setProcedureCodes(populateProcedureCodes((BasicDBList) encounterPo.get(ENCOUNTER_PROCEDURE_CODES)));
			
			//If it's null, populateDiagnosisCodes will return null.
			encounter.setDiagnosisCodes(populateDiagnosisCodes((BasicDBList) encounterPo.get(ENCOUNTER_DIAGNOSIS_CODES)));
		}

		return encounter;
	}

	private VisitRelatedTos populateVisitRelatedTos(BasicDBList visitRelatedTosList) {
		VisitRelatedTos visitRelatedTos = null;

		if (visitRelatedTosList != null) {
			visitRelatedTos = new VisitRelatedTos();

			Iterator<Object> itVisitRelatedTosList = visitRelatedTosList.iterator();
			while (itVisitRelatedTosList.hasNext()) {
			    BasicDBObject visitRelatedToPo = (BasicDBObject) itVisitRelatedTosList.next();
			    VisitRelatedTo visitRelatedTo = new VisitRelatedTo();
			    //ENCOUNTER_VISIT_RELATED_TO
			    if (visitRelatedToPo.get(ENCOUNTER_VIS_CHECKED) != null)
			    	visitRelatedTo.setChecked(visitRelatedToPo.getBoolean(ENCOUNTER_VIS_CHECKED));

			    if (visitRelatedToPo.get(ENCOUNTER_VIS_ENABLED) != null)
			    	visitRelatedTo.setEnabled(visitRelatedToPo.getBoolean(ENCOUNTER_VIS_ENABLED));

			    visitRelatedTo.setTitle(StringUtil.objectToString(visitRelatedToPo.get(ENCOUNTER_VIS_TITLE)));
			    visitRelatedTo.setCode(StringUtil.objectToString(visitRelatedToPo.get(ENCOUNTER_VIS_CODE)));
			    visitRelatedTos.add(visitRelatedTo);
			}
		}
		return visitRelatedTos;
	}
	
	private VisitTypes populateVisitTypes(BasicDBList encounterVisitTypeList) {
		VisitTypes visitTypes = null;
		
		if (encounterVisitTypeList != null) {
			visitTypes = new VisitTypes();
			Iterator<Object> itCodes = encounterVisitTypeList.iterator();
			while (itCodes.hasNext()) {
			    BasicDBObject visitTypePo = (BasicDBObject) itCodes.next();
			    VisitType visitType = new VisitType();
			    
			    visitType.setName(StringUtil.objectToString(visitTypePo.get(ENCOUNTER_VISIT_TYPES_NAME)));
			    visitType.setSelected(StringUtil.objectToBoolean(visitTypePo.get(ENCOUNTER_VISIT_TYPES_SELECTED)));
			    
			    BasicDBList encounterProcedureCPTCodesList = (BasicDBList)visitTypePo.get(ENCOUNTER_VISIT_TYPES_SECTIONS);
			    EncounterProcedureCPTCodes encounterProcedureCPTCodes = populateProcedureCodes(encounterProcedureCPTCodesList);
			    visitType.setSections(encounterProcedureCPTCodes);
			    visitTypes.add(visitType);
			}
		}
		return visitTypes;
	}
	
	private EncounterProcedureCPTCodes populateProcedureCodes(BasicDBList encounterProcedureCPTCodesList) {
		EncounterProcedureCPTCodes cptCodes = null;

		if (encounterProcedureCPTCodesList != null) {
			cptCodes = new EncounterProcedureCPTCodes();

			Iterator<Object> itCodes = encounterProcedureCPTCodesList.iterator();
			while (itCodes.hasNext()) {
			    BasicDBObject cptCodePo = (BasicDBObject) itCodes.next();
			    EncounterProcedureCPTCode cptCode = new EncounterProcedureCPTCode();
			    
			    cptCode.setCode(StringUtil.objectToString(cptCodePo.get(ENCOUNTER_CODE)));
			    cptCode.setCodeType(StringUtil.objectToString(cptCodePo.get(ENCOUNTER_CODE_TYPE)));
			    cptCode.setDescription(StringUtil.objectToString(cptCodePo.get(ENCOUNTER_CODE_DESCRIPTION)));
			    cptCode.setIen(StringUtil.objectToString(cptCodePo.get(ENCOUNTER_CODE_IEN)));
			    cptCode.setShortDescription(StringUtil.objectToString(cptCodePo.get(ENCOUNTER_CODE_SHORT_DESCRIPTION)));
			    cptCode.setSelected(StringUtil.objectToBoolean(cptCodePo.get(ENCOUNTER_VISIT_TYPES_SECTIONS_SELECTED)));
			    
			    if (cptCodePo.get(ENCOUNTER_CPTCODE_MODIFIERS) != null) {
			    	Modifiers modifiers = new Modifiers();
			    	BasicDBList dbListModifiers = (BasicDBList)cptCodePo.get(ENCOUNTER_CPTCODE_MODIFIERS);
			    	Iterator<Object> itModifiers = dbListModifiers.iterator();
			    	while (itModifiers.hasNext()) {
			    		BasicDBObject modifierPo = (BasicDBObject) itModifiers.next();
			    		Modifier modifier = new Modifier();
			    		modifier.setCode(StringUtil.objectToString(modifierPo.get(ENCOUNTER_CODE)));
			    		modifier.setIen(StringUtil.objectToString(modifierPo.get(ENCOUNTER_CODE_IEN)));
			    		modifier.setName(StringUtil.objectToString(modifierPo.get(ENCOUNTER_CODE_NAME)));
			    		modifier.setSelected(StringUtil.objectToBoolean(modifierPo.get(ENCOUNTER_VISIT_TYPES_SECTIONS_MODIFIERS_SELECTED)));
			    		
			    		modifiers.add(modifier);
			    	}
			    	cptCode.setModifiers(modifiers);
			    }

			    cptCodes.add(cptCode);
			}
		}
		return cptCodes;
	}
	
	private EncounterDiagnosisICDCodes populateDiagnosisCodes(BasicDBList encounterDiagnosisICDCodesList) {
		EncounterDiagnosisICDCodes icdCodes = null;

		if (encounterDiagnosisICDCodesList != null) {
			icdCodes = new EncounterDiagnosisICDCodes();

			Iterator<Object> itCodes = encounterDiagnosisICDCodesList.iterator();
			while (itCodes.hasNext()) {
			    BasicDBObject icdCodePo = (BasicDBObject) itCodes.next();
			    EncounterDiagnosisICDCode icdCode = new EncounterDiagnosisICDCode();
			    
			    icdCode.setCode(StringUtil.objectToString(icdCodePo.get(ENCOUNTER_CODE)));
			    icdCode.setCodeType(StringUtil.objectToString(icdCodePo.get(ENCOUNTER_CODE_TYPE)));
			    icdCode.setDescription(StringUtil.objectToString(icdCodePo.get(ENCOUNTER_CODE_DESCRIPTION)));
			    icdCode.setShortDescription(StringUtil.objectToString(icdCodePo.get(ENCOUNTER_CODE_SHORT_DESCRIPTION)));
			    icdCode.setIen(StringUtil.objectToString(icdCodePo.get(ENCOUNTER_CODE_IEN)));
			    
			    icdCode.setAddToProblemList(StringUtil.objectToBoolean(icdCodePo.get(ENCOUNTER_DIAGNOSIS_ADD_TO_PROBLEM_LIST)));
			    icdCode.setPrimaryDiagnosis(StringUtil.objectToBoolean(icdCodePo.get(ENCOUNTER_DIAGNOSIS_PRIMARY_DIAGNOSIS)));
			    icdCode.setFromProblemList(StringUtil.objectToBoolean(icdCodePo.get(ENCOUNTER_DIAGNOSIS_FROM_PROBLEM_LIST)));
			    
			    icdCodes.add(icdCode);
			}
		}
		return icdCodes;
	}

	private EncounterProviders populateEncounterProviders(BasicDBList providersList) {
		EncounterProviders providers = null;

		if (providersList != null) {
			providers = new EncounterProviders();

			Iterator<Object> it = providersList.iterator();
			while (it.hasNext()) {
			    BasicDBObject encounterProviderPo = (BasicDBObject) it.next();

				EncounterProvider encounterProvider = new EncounterProvider();
				FacilityProvider facilityProvider = new FacilityProvider();

				encounterProvider.setPrimary(encounterProviderPo.getBoolean(ENCOUNTER_PROV_PRIMARY));
				encounterProvider.setSelected(encounterProviderPo.getBoolean(ENCOUNTER_PROV_SELECTED));
				
				DBObject providerPo = ((DBObject)encounterProviderPo.get(ENCOUNTER_PROV_PROVIDER));
				String facilityCode = StringUtil.objectToString(((DBObject)providerPo.get(ENCOUNTER_PROV_IDENTIFIER)).get(ENCOUNTER_PROV_FACILITY_CODE));
				String staffId = StringUtil.objectToString(((DBObject)providerPo.get(ENCOUNTER_PROV_IDENTIFIER)).get(ENCOUNTER_PROV_STAFF_ID));
				String providerName = StringUtil.objectToString(((DBObject)providerPo.get(ENCOUNTER_PROV_IDENTIFIER)).get(ENCOUNTER_PROV_NAME));
				facilityProvider.setFacilityCode(facilityCode);
				facilityProvider.setProviderIen(staffId);
				facilityProvider.setProviderName(providerName);
				encounterProvider.setProvider(facilityProvider);

				providers.add(encounterProvider);
			}
		}
		return providers;
	}

	public DBObject translateEncounter(Encounter encounter) {
		DBObject dbObject = new BasicDBObject(15);

		if (encounter.getTimestamp() != null) {
			dbObject.put(ENCOUNTER_TIMESTAMP, encounter.getTimestamp());
		}

		dbObject.put(ENCOUNTER_RESUME_DATE, encounter.getDate());
		dbObject.put(ENCOUNTER_RESUME_TIME, encounter.getTime());

		dbObject.put(ENCOUNTER_LOCATION_ID, encounter.getLocationId());
		dbObject.put(ENCOUNTER_LOCATION_TITLE, encounter.getLocationTitle());
		dbObject.put(ENCOUNTER_CURRENT_ADMISSION, encounter.isCurrentAdmission());
		dbObject.put(ENCOUNTER_IS_HISTORICAL, encounter.isHistorical());		
		dbObject.put(ENCOUNTER_SERVICE_CATEGORY, encounter.getServiceCategory());
		dbObject.put(ENCOUNTER_APPOINTMENT_TYPE, encounter.getAppointmentType());
		dbObject.put(ENCOUNTER_ENCOUNTER_TYPE, encounter.getEncounterType());
		dbObject.put(ENCOUNTER_LINK_TYPE, encounter.getLinkType());

		//VisitRelatedTos
		if (encounter.getVisitRelatedTos() != null)
			dbObject.put(ENCOUNTER_VISIT_RELATED_TOS, populateDbObjectVisitRelatedTos(encounter));

		//providers
		if (encounter.getProviders() != null)
			dbObject.put(ENCOUNTER_PROVIDERS, populateDbObjectProviders(encounter));
		
		if (encounter.getVisitTypes() != null)
			dbObject.put(ENCOUNTER_VISIT_TYPES, populateDbObjectVisitTypes(encounter.getVisitTypes()));
		if (encounter.getProcedureCodes() != null)
			dbObject.put(ENCOUNTER_PROCEDURE_CODES, populateDbObjectProcedureCodes(encounter.getProcedureCodes()));
		if (encounter.getDiagnosisCodes() != null)
			dbObject.put(ENCOUNTER_DIAGNOSIS_CODES, populateDbObjectDiagnosisCodes(encounter));

		return dbObject;
	}

	private BasicDBList populateDbObjectVisitRelatedTos(Encounter encounter) {
		BasicDBList dbList = new BasicDBList();
		
		if (encounter.getVisitRelatedTos() == null)
			return dbList;

		for (int i = 0; i < encounter.getVisitRelatedTos().size(); i++) {
			VisitRelatedTo visitRelatedTo = encounter.getVisitRelatedTos().get(i);
			DBObject dbObject = new BasicDBObject(4);
			dbObject.put(ENCOUNTER_VIS_TITLE, visitRelatedTo.getTitle());
			dbObject.put(ENCOUNTER_VIS_ENABLED, visitRelatedTo.getEnabled());
	    	dbObject.put(ENCOUNTER_VIS_CHECKED, visitRelatedTo.getChecked());
	    	dbObject.put(ENCOUNTER_VIS_CODE, visitRelatedTo.getCode());

			dbList.add(dbObject);
		}
		return dbList;
	}
	
	private BasicDBList populateDbObjectVisitTypes(VisitTypes visitTypes) {
		BasicDBList dbList = new BasicDBList();
		
		if (visitTypes == null)
			return dbList;

		for (int i = 0; i < visitTypes.size(); i++) {
			VisitType visitType = visitTypes.get(i);
			DBObject dbObject = new BasicDBObject(2);
			dbObject.put(ENCOUNTER_VISIT_TYPES_NAME, visitType.getName());
			dbObject.put(ENCOUNTER_VISIT_TYPES_SELECTED, visitType.isSelected());
			
			BasicDBList sections = populateDbObjectProcedureCodes(visitType.getSections());
			dbObject.put(ENCOUNTER_VISIT_TYPES_SECTIONS, sections);
			
			dbList.add(dbObject);
		}
		
		return dbList;
	}
	
	private BasicDBList populateDbObjectProcedureCodes(EncounterProcedureCPTCodes procedureCodes) {
		BasicDBList dbList = new BasicDBList();
		
		if (procedureCodes == null)
			return dbList;

		for (int i = 0; i < procedureCodes.size(); i++) {
			EncounterProcedureCPTCode code = procedureCodes.get(i);
			DBObject dbObject = new BasicDBObject(6);
			dbObject.put(ENCOUNTER_CODE_DESCRIPTION, code.getDescription());
			dbObject.put(ENCOUNTER_CODE, code.getCode());
			dbObject.put(ENCOUNTER_CODE_IEN, code.getIen());
			dbObject.put(ENCOUNTER_CODE_TYPE, code.getCodeType());
			dbObject.put(ENCOUNTER_CODE_SHORT_DESCRIPTION, code.getShortDescription());
			dbObject.put(ENCOUNTER_VISIT_TYPES_SECTIONS_SELECTED, code.isSelected());
			

			if (code.getModifiers() != null && code.getModifiers().size() > 0) {
				BasicDBList dbListModifiers = new BasicDBList();
				for (int k=0; k<code.getModifiers().size(); k++) {
					DBObject dbObjectModifier = new BasicDBObject(3);
					dbObjectModifier.put(ENCOUNTER_CODE, code.getModifiers().get(k).getCode());
					dbObjectModifier.put(ENCOUNTER_CODE_IEN, code.getModifiers().get(k).getIen());
					dbObjectModifier.put(ENCOUNTER_CODE_NAME, code.getModifiers().get(k).getName());
					dbObjectModifier.put(ENCOUNTER_VISIT_TYPES_SECTIONS_SELECTED, code.getModifiers().get(k).isSelected());
					
					dbListModifiers.add(dbObjectModifier);
				}
				dbObject.put(ENCOUNTER_CPTCODE_MODIFIERS, dbListModifiers);
			}

			dbList.add(dbObject);
		}
		return dbList;
	}
	
	private BasicDBList populateDbObjectDiagnosisCodes(Encounter encounter) {
		BasicDBList dbList = new BasicDBList();
		
		if (encounter.getDiagnosisCodes() == null)
			return dbList;

		for (int i = 0; i < encounter.getDiagnosisCodes().size(); i++) {
			EncounterDiagnosisICDCode code = encounter.getDiagnosisCodes().get(i);
			DBObject dbObject = new BasicDBObject(8);
			dbObject.put(ENCOUNTER_CODE_DESCRIPTION, code.getDescription());
			dbObject.put(ENCOUNTER_CODE, code.getCode());
			dbObject.put(ENCOUNTER_CODE_IEN, code.getIen());
			dbObject.put(ENCOUNTER_CODE_TYPE, code.getCodeType());
			dbObject.put(ENCOUNTER_CODE_SHORT_DESCRIPTION, code.getShortDescription());
			dbObject.put(ENCOUNTER_DIAGNOSIS_PRIMARY_DIAGNOSIS, code.isPrimaryDiagnosis());
			dbObject.put(ENCOUNTER_DIAGNOSIS_ADD_TO_PROBLEM_LIST, code.isAddToProblemList());
			dbObject.put(ENCOUNTER_DIAGNOSIS_FROM_PROBLEM_LIST, code.isFromProblemList());
			
			dbList.add(dbObject);
		}
		return dbList;
	}

	private BasicDBList populateDbObjectProviders(Encounter encounter) {
		//Providers
		BasicDBList dbList = new BasicDBList();
		
		if (encounter.getProviders() == null)
			return dbList;

		for (int i = 0; i < encounter.getProviders().size(); i++) {
			EncounterProvider encounterProvider = encounter.getProviders().get(i);

			//providerIdentifier, facilityCode, staffId
			DBObject dbObjectProvider = new BasicDBObject(1);
			DBObject dbObjectProviderId = new BasicDBObject(3);
			if (encounterProvider.getProvider() != null) {
				dbObjectProviderId.put(ENCOUNTER_PROV_STAFF_ID, encounterProvider.getProvider().getProviderIen());
				dbObjectProviderId.put(ENCOUNTER_PROV_FACILITY_CODE, encounterProvider.getProvider().getFacilityCode());
				dbObjectProviderId.put(ENCOUNTER_PROV_NAME, encounterProvider.getProvider().getProviderName());
			}
			dbObjectProvider.put(ENCOUNTER_PROV_IDENTIFIER, dbObjectProviderId);

			DBObject dbObject = new BasicDBObject(3);
			dbObject.put(ENCOUNTER_PROV_PROVIDER, dbObjectProvider);
			dbObject.put(ENCOUNTER_PROV_PRIMARY, encounterProvider.isPrimary());
			dbObject.put(ENCOUNTER_PROV_SELECTED, encounterProvider.isSelected());

			dbList.add(dbObject);
		}
		return dbList;
	}

	public NoteTitle translateTitle(DBObject noteTitlePo) {
		NoteTitle title = null;

		if (noteTitlePo != null) {
			title = new NoteTitle();

			title.setIen(StringUtil.objectToString(noteTitlePo.get(NOTE_TITLE_IEN)));
			String name = StringUtil.objectToString(noteTitlePo.get(NOTE_TITLE_NAME));
			title.setName(name == null ? "" : name.trim());
			title.setConsult(StringUtil.objectToBoolean(noteTitlePo.get(NOTE_TITLE_IS_CONSULT)));
			title.setPrf(StringUtil.objectToBoolean(noteTitlePo.get(NOTE_TITLE_IS_PRF)));
		}

		return title;
	}

	protected DBObject translateTitle(NoteTitle title) {
		DBObject dbObject = new BasicDBObject(2);

		dbObject.put(NOTE_TITLE_NAME, title.getName() == null ? "" : title.getName());

		if (NullChecker.isNullish(title.getIen())) {
			dbObject.put(NOTE_TITLE_IEN, "");
		} else {
			dbObject.put(NOTE_TITLE_IEN, title.getIen());
		}

		dbObject.put(NOTE_TITLE_IS_CONSULT, title.isConsult());
		dbObject.put(NOTE_TITLE_IS_PRF, title.isPrf());
		
		return dbObject;
	}

	/**
	 * NOTE: The DBObject returned here differs from the domain model object.
	 * It adds the facilityCode and staffId to the object to be saved by Mongo.
	 */
	public DBObject translateLastSelectedTitle(String facilityCode, String staffId, NoteTitle title) {
		DBObject dbObject = new BasicDBObject(3);

		DBObject providerDbObject = new BasicDBObject(2);
		providerDbObject.put(PROVIDER_FACILITY_CODE, facilityCode);
		providerDbObject.put(PROVIDER_STAFF_ID, staffId);
		dbObject.put(PROVIDER_IDENTIFIER, providerDbObject);

		if (NullChecker.isNullish(title.getIen())) {
			dbObject.put(NOTE_TITLE_IEN, new ObjectId().toString());
		} else {
			dbObject.put(NOTE_TITLE_IEN, title.getIen());
		}

		dbObject.put(NOTE_TITLE_NAME, title.getName() == null ? "" : title.getName());

		dbObject.put(NOTE_TITLE_IS_CONSULT, title.isConsult());
		
		dbObject.put(NOTE_TITLE_IS_PRF, title.isPrf());
		
		return dbObject;
	}

    private HealthFactors populateHealthFactors(BasicDBList healthFactorsList){
        HealthFactors healthFactors = new HealthFactors();
        for(Object obj: healthFactorsList){
            HealthFactor healthFactor = new HealthFactor();
            DBObject dbObject = (DBObject)obj;
            healthFactor.setIen(StringUtil.objectToString(dbObject.get(HEALTH_FACTOR_IEN)));
            healthFactor.setHealthFactorCode(StringUtil.objectToString(dbObject.get(HEALTH_FACTOR_CODE)));
            healthFactor.setHealthFactorDescription(StringUtil.objectToString(dbObject.get(HEALTH_FACTOR_DESCRIPTION)));
            if(null!=dbObject.get(HEALTH_FACTOR_COMMENT))
                healthFactor.setHealthFactorComment(StringUtil.objectToString(dbObject.get(HEALTH_FACTOR_COMMENT)));
            healthFactor.setParentCategory(StringUtil.objectToString(dbObject.get(HEALTH_FACTOR_PARENT_CATEGORY)));
            healthFactor.setCategory((Boolean)dbObject.get(HEALTH_FACTOR_CATEGORY));
            healthFactors.add(healthFactor);
        }
        return healthFactors;
    }

    private BasicDBList populateDbObjectHealthFactors(HealthFactors healthFactors){
        BasicDBList dbList = new BasicDBList();
        for(HealthFactor healthFactor : healthFactors){
            DBObject dbObject = new BasicDBObject();
            dbObject.put(HEALTH_FACTOR_IEN, healthFactor.getIen());
            dbObject.put(HEALTH_FACTOR_CODE, healthFactor.getHealthFactorCode());
            dbObject.put(HEALTH_FACTOR_DESCRIPTION, healthFactor.getHealthFactorDescription());
            if(null!=healthFactor.getHealthFactorComment())
                dbObject.put(HEALTH_FACTOR_COMMENT, healthFactor.getHealthFactorComment());
            dbObject.put(HEALTH_FACTOR_PARENT_CATEGORY, healthFactor.getParentCategory());
            dbObject.put(HEALTH_FACTOR_CATEGORY, healthFactor.isCategory());
            dbList.add(dbObject);
        }
        return dbList;
    }
}
