package com.agilex.healthcare.notewriter.translator;

import java.util.Iterator;

import com.agilex.healthcare.mdws.datalayer.MdwsResponseException;
import com.agilex.healthcare.mdws.domain.BasicCode;
import com.agilex.healthcare.mdws.domain.Code;
import com.agilex.healthcare.mdws.domain.Codes;
import com.agilex.healthcare.mdws.domain.EncounterNote;
import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.notewriter.domain.EncounterDiagnosisICDCode;
import com.agilex.healthcare.notewriter.domain.EncounterDiagnosisICDCodes;
import com.agilex.healthcare.notewriter.domain.EncounterProcedureCPTCode;
import com.agilex.healthcare.notewriter.domain.EncounterProcedureCPTCodes;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.vamf.json.util.NullChecker;

public class ClinicalNoteTranslator {
	public static EncounterNote translateToEncounterNote(ClinicalEvent clinicalEvent){
		EncounterNote note = new EncounterNote();
    	note.setPatientIen(clinicalEvent.getPatientIdentifier().getUniqueId());
    	note.setFacilityCode(clinicalEvent.getFacilityCode());
    	note.setLocationIen(clinicalEvent.getEncounter().getLocationId());
		note.setNoteTitleIen(clinicalEvent.getNote().getTitle().getIen());
		note.setServiceCategory(clinicalEvent.getEncounter().getServiceCategory());
		note.setInpatient(clinicalEvent.getEncounter().isCurrentAdmission() ? "1" : "0");
		note.setNoteBody(clinicalEvent.getNote().getBody().toString());
		note.setConsultIen(clinicalEvent.getNote().getConsultIen());
		note.setDate(DateHelper.format(clinicalEvent.getNote().getSignedDate(), DateHelper.VISTA_DATETIME_FORMAT));
		translateProviders(clinicalEvent, note);
		translateProcedures(clinicalEvent.getEncounter().getProcedureCodes(), note, false);
		translateDiagnoses(clinicalEvent, note);
		translateVisitRelatedCodes(clinicalEvent, note);
		if (clinicalEvent.getEncounter().getVisitTypes() != null) {
			for (int i=0; i<clinicalEvent.getEncounter().getVisitTypes().size(); i++)
				translateProcedures(clinicalEvent.getEncounter().getVisitTypes().get(i).getSections(), note, true);
		}
        translateHealthFactors(clinicalEvent, note);

		return note;

	}

	private static void translateProviders(ClinicalEvent clinicalEvent, EncounterNote note) {
		String providers = "";
		if (clinicalEvent.getEncounter().getProviders() != null) {
			Iterator<EncounterProvider> it = clinicalEvent.getEncounter().getProviders().iterator();
			while (it.hasNext()) {
				EncounterProvider ep = it.next();
				
				//Only send those providers that have actually been selected.
				if (ep.isSelected() == false)
					continue;
				
				String id = ep.getProvider().getProviderIen();
				if (providers.length() == 0)
					providers += id;
				else if (ep.isPrimary())					//If you use more than one, the first one in
					providers = id + "," + providers;		//the list will be interpreted as the primary provider
				else
					providers += "," + id;
			}
		}
		note.setProviders(providers);
	}

	private static void translateProcedures(EncounterProcedureCPTCodes procedures, EncounterNote note, boolean bIsVisitType) {
		if (procedures != null){
			com.agilex.healthcare.mdws.domain.EncounterProcedureCPTCodes mdwsProcedures = new com.agilex.healthcare.mdws.domain.EncounterProcedureCPTCodes();
			for (int i=0; i<procedures.size(); i++) {
				com.agilex.healthcare.mdws.domain.EncounterProcedureCPTCode mdwsProcedureCode = new com.agilex.healthcare.mdws.domain.EncounterProcedureCPTCode();
				EncounterProcedureCPTCode procedureCode = procedures.get(i);
				
				String code = procedureCode.getCode();
				
				if (!NullChecker.isNotNullish(code)) {
					if (bIsVisitType)
						throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_CPT_CODE, "The cpt code for the visit type was null or empty.");
					else
						throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_CPT_CODE, "The cpt code for the procedure was null or empty.");
				}
				String description = procedureCode.getShortDescription();
				
				BasicCode basicCode = new BasicCode();
				basicCode.setCode(code);
				basicCode.setDescription(description);
				mdwsProcedureCode.setBasicCode(basicCode);
				
				if (procedureCode.getModifiers() != null) {
					Codes modifierCodes = new Codes();
					for (int k=0; k<procedureCode.getModifiers().size(); k++) {
						Modifier modifier = procedureCode.getModifiers().get(k);
						
						String modifierCodeType = "CPT";
						String modifierIen = modifier.getIen();
						String modifierDescription = modifier.getName();
						String modifierCode = modifier.getCode();
						
						Code cptModifier = new Code();
						cptModifier.setCode(modifierCode);
						cptModifier.setCodeType(modifierCodeType);
						cptModifier.setDescription(modifierDescription);
						cptModifier.setIen(modifierIen);
						
						modifierCodes.add(cptModifier);
					}
					mdwsProcedureCode.setModifiers(modifierCodes);
				}
				mdwsProcedures.add(mdwsProcedureCode);
			}
			
			com.agilex.healthcare.mdws.domain.EncounterProcedureCPTCodes currentProcedures = note.getProcedures();
			if (currentProcedures == null || currentProcedures.size() == 0) {
				note.setProcedures(mdwsProcedures);
			}
			else {
				currentProcedures.addAll(mdwsProcedures);
				note.setProcedures(currentProcedures);
			}
		}
	}
	
	private static void translateDiagnoses(ClinicalEvent clinicalEvent, EncounterNote note) {
		if (clinicalEvent.getEncounter().getDiagnosisCodes() != null){
			com.agilex.healthcare.mdws.domain.EncounterDiagnosisICDCodes mdwsDiagnoses = new com.agilex.healthcare.mdws.domain.EncounterDiagnosisICDCodes();
			EncounterDiagnosisICDCodes diagnoses = clinicalEvent.getEncounter().getDiagnosisCodes();
			for (int i=0; i<diagnoses.size(); i++) {
				com.agilex.healthcare.mdws.domain.EncounterDiagnosisICDCode mdwsDiagnosisCode = new com.agilex.healthcare.mdws.domain.EncounterDiagnosisICDCode();
				EncounterDiagnosisICDCode diagnosisCode = diagnoses.get(i);
				
				String code = diagnosisCode.getCode();
				String description = diagnosisCode.getShortDescription();
				boolean addToProblemList = diagnosisCode.isAddToProblemList();
				boolean primary = diagnosisCode.isPrimaryDiagnosis();
				
				BasicCode basicCode = new BasicCode();
				basicCode.setCode(code);
				basicCode.setDescription(description);
				mdwsDiagnosisCode.setBasicCode(basicCode);
				mdwsDiagnosisCode.setAddToProblemList(addToProblemList);
				mdwsDiagnosisCode.setPrimary(primary);
				
				mdwsDiagnoses.add(mdwsDiagnosisCode);
			}
			note.setDiagnoses(mdwsDiagnoses);
		}
	}
	
	private static void translateVisitRelatedCodes(ClinicalEvent clinicalEvent, EncounterNote note) {
		String visitRelatedCodes = "";
		if (clinicalEvent.getEncounter().getVisitRelatedTos() != null) {
			Iterator<VisitRelatedTo> it = clinicalEvent.getEncounter().getVisitRelatedTos().iterator();
			while (it.hasNext()) {
				VisitRelatedTo vrt = it.next();
				
				String code = vrt.getCode();
				if (visitRelatedCodes.length() == 0)
					visitRelatedCodes += code;
				else
					visitRelatedCodes += "," + code;
			}
		}
		note.setVisitRelatedCodes(visitRelatedCodes);
	}

    private static void translateHealthFactors(ClinicalEvent clinicalEvent, EncounterNote note){
        if(null!= clinicalEvent.getHealthFactors()){
            StringBuilder builder = new StringBuilder("");
            for(HealthFactor healthFactor: clinicalEvent.getHealthFactors()){
                builder.append(healthFactor.toDelimitedStringForVista());
                builder.append(";");
            }
            if(builder.charAt(builder.length()-1) == ';')
                builder.deleteCharAt(builder.length()-1);
            note.setHealthFactors(builder.toString());
        }else{
            note.setHealthFactors("");
        }
    }
}
