package com.agilex.healthcare.patientsearch.datalayer;

import java.util.Date;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response.Status;

import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.RecentlySelectedPatients;
import com.agilex.healthcare.mobilehealthplatform.domain.SelectedPatient;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.MhpObjectFactory;
import com.agilex.healthcare.utility.NullChecker;

public class PatientSearchDataService {
	PatientSearchDataLayer dataLayer = null;
	
	public PatientSearchDataService() {
		dataLayer = MhpObjectFactory.getInstance().getPatientSearchDataLayer();
	}
	
	public PatientSearchDataService(PatientSearchDataLayer dataLayer) {
		this.dataLayer = dataLayer;
	}
	
	public RecentlySelectedPatients fetchRecentlySelectedPatients(String facilityCode, String staffId) {
		return dataLayer.fetchRecentlySelectedPatients(facilityCode, staffId);
	}

	public SelectedPatient addSelectedPatient(String facilityCode, String staffId, Patient patient) {
		if (!isAddSelectedPatientRequestValid(facilityCode, staffId, patient)) {
			throw new WebApplicationException(Status.BAD_REQUEST);
		}
		Date searchedTime = new Date();
		SelectedPatient pat = dataLayer.addSelectedPatient(searchedTime, facilityCode, staffId, patient);
		return pat;
	}
	
	private boolean isAddSelectedPatientRequestValid(String facilityCode, String staffId, Patient patient) {
		boolean valid = true;
		if (NullChecker.isNullish(facilityCode) 
				|| NullChecker.isNullish(staffId) 
				|| patient == null 
				|| patient.getPatientIdentifier() == null 
				|| NullChecker.isNullish(patient.getPatientIdentifier().getUniqueId()) 
				|| NullChecker.isNullish(patient.getPatientIdentifier().getAssigningAuthority())
				//|| NullChecker.isNullish(patient.getSsn())
				//|| NullChecker.isNullish(patient.getGender())
				//|| patient.getDateOfBirth() == null
				|| NullChecker.isNullish(patient.getDisplayName())
				|| patient.getDisplayName().equals("null, null")) {
			valid = false;
		}
		return valid;
	}
}
