package com.agilex.healthcare.patientsearch.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.DOBDate;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.RecentlySelectedPatients;
import com.agilex.healthcare.mobilehealthplatform.domain.SelectedPatient;
import com.agilex.healthcare.mobilehealthplatform.utils.StringUtil;
import com.agilex.healthcare.utility.DateHelper;
import com.mongodb.BasicDBList;
import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;

public class PatientSearchTranslator {
	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(PatientSearchTranslator.class);
	
	private final String SEARCHED_TIME_KEY = "searchedTime";
	
	private final String PROVIDER_KEY = "provider";
	private final String FACILITY_CODE_KEY = "facilityCode";
	private final String STAFF_ID_KEY = "staffId";
	
	private final String PATIENTS_KEY = "patients";
	private final String PATIENT_KEY = "patient";
	private final String PATIENT_ID_KEY = "patientId";
	private final String ASSIGNING_AUTHORITY_KEY = "assigningAuthority";
	private final String FIRST_NAME_KEY = "firstName";
	private final String MIDDLE_NAME_KEY = "middleName";
	private final String LAST_NAME_KEY = "lastName";
	private final String SSN_KEY = "ssn";
	private final String DATE_OF_BIRTH_KEY = "dateOfBirth";
	private final String GENDER_KEY = "gender";
	private final String INPATIENT_KEY = "inpatient";
	
	public RecentlySelectedPatients translate(DBObject po) {
		RecentlySelectedPatients recentlySearchedPatients = new RecentlySelectedPatients();
		
		DBObject poProv = (DBObject)po.get(PROVIDER_KEY);
		ProviderIdentifier provId = new ProviderIdentifier();
		provId.setFacilityCode(StringUtil.objectToString(poProv.get(FACILITY_CODE_KEY)));
		provId.setStaffId(StringUtil.objectToString(poProv.get(STAFF_ID_KEY)));
		recentlySearchedPatients.setProvider(provId);

		BasicDBList poPats = (BasicDBList) po.get(PATIENTS_KEY);
		
		if (poPats != null) {
			Iterator<Object> it = poPats.iterator();
			while (it.hasNext()) {
				BasicDBObject dbObj = (BasicDBObject)it.next();
				SelectedPatient selPat = new SelectedPatient();
				Patient patient = new Patient();
				patient.setPatientIdentifier(new PatientIdentifier());
				selPat.setPatient(patient);
				selPat.setSearchedTime((Date)dbObj.get(SEARCHED_TIME_KEY));
				
				selPat.getPatient().getPatientIdentifier().setUniqueId(StringUtil.objectToString(((DBObject)dbObj.get(PATIENT_KEY)).get(PATIENT_ID_KEY)));
				selPat.getPatient().getPatientIdentifier().setAssigningAuthority(StringUtil.objectToString(((DBObject)dbObj.get(PATIENT_KEY)).get(ASSIGNING_AUTHORITY_KEY)));
				// JUSTIFICATION: data layer should know the facility and staffId via parameters, so they are not mapped here
				selPat.getPatient().setFirstName(StringUtil.objectToString(((DBObject)dbObj.get(PATIENT_KEY)).get(FIRST_NAME_KEY)));
				selPat.getPatient().setMiddleName(StringUtil.objectToString(((DBObject)dbObj.get(PATIENT_KEY)).get(MIDDLE_NAME_KEY)));
				selPat.getPatient().setLastName(StringUtil.objectToString(((DBObject)dbObj.get(PATIENT_KEY)).get(LAST_NAME_KEY)));
				selPat.getPatient().setSsn(StringUtil.objectToString(((DBObject)dbObj.get(PATIENT_KEY)).get(SSN_KEY)));
				selPat.getPatient().setGender(StringUtil.objectToString(((DBObject)dbObj.get(PATIENT_KEY)).get(GENDER_KEY)));
				selPat.getPatient().setInpatient(StringUtil.objectToBoolean(((DBObject)dbObj.get(PATIENT_KEY)).get(INPATIENT_KEY)));
				String dateOfBirth = StringUtil.objectToString(((DBObject)dbObj.get(PATIENT_KEY)).get(DATE_OF_BIRTH_KEY));
				DOBDate dob = new DOBDate(null, dateOfBirth);
				selPat.getPatient().setDateOfBirth(dob);
				
				setAge(selPat.getPatient());

				recentlySearchedPatients.add(selPat);
			}
		}
		
		
		return recentlySearchedPatients;
	}

	public void setAge(Patient pat) {
		try {
			int age = DateHelper.calculateDeltaInYears(pat.getDateOfBirth().getDate(), new Date());
			pat.setAge(String.valueOf(age));
		} catch (Exception e) {
			LOGGER.error("Error calculating patient's age", e);
		}
	}
	
	public DBObject translate(String facilityCode, String staffId, RecentlySelectedPatients pats) {
		DBObject dbObject = new BasicDBObject(2);
		
		DBObject providerDbObject = new BasicDBObject(2);
		providerDbObject.put(FACILITY_CODE_KEY, facilityCode);
		providerDbObject.put(STAFF_ID_KEY, staffId);
		dbObject.put(PROVIDER_KEY, providerDbObject);
		
		BasicDBList poPats = new BasicDBList();
		
		for (int i=0; i<pats.size(); i++) {
			SelectedPatient selectedPatient = pats.get(i);
			DBObject patientAndTime = new BasicDBObject();
			patientAndTime.put(SEARCHED_TIME_KEY, selectedPatient.getSearchedTime());
			
			DBObject patientDbObject = new BasicDBObject(8);
			patientDbObject.put(PATIENT_ID_KEY, selectedPatient.getPatient().getPatientIdentifier().getUniqueId());
			patientDbObject.put(ASSIGNING_AUTHORITY_KEY, selectedPatient.getPatient().getPatientIdentifier().getAssigningAuthority());
			patientDbObject.put(FIRST_NAME_KEY, selectedPatient.getPatient().getFirstName());
			patientDbObject.put(MIDDLE_NAME_KEY, selectedPatient.getPatient().getMiddleName());
			patientDbObject.put(LAST_NAME_KEY, selectedPatient.getPatient().getLastName());
			patientDbObject.put(SSN_KEY, selectedPatient.getPatient().getSsn());
			patientDbObject.put(GENDER_KEY, selectedPatient.getPatient().getGender());
			patientDbObject.put(INPATIENT_KEY, selectedPatient.getPatient().isInpatient());
			DOBDate dateOfBirth = selectedPatient.getPatient().getDateOfBirth();
			if (dateOfBirth != null)
				patientDbObject.put(DATE_OF_BIRTH_KEY, dateOfBirth.toString());
			patientAndTime.put(PATIENT_KEY, patientDbObject);
			
			poPats.add(patientAndTime);
		}
		
		dbObject.put(PATIENTS_KEY, poPats);
		return dbObject;
	}
}
