package com.agilex.healthcare.mobilehealthplatform.translator.documents;

import junit.framework.Assert;

import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.domain.Author;
import com.agilex.healthcare.mobilehealthplatform.domain.Facility;
import com.agilex.healthcare.mobilehealthplatform.domain.ReportDocument;
import com.agilex.healthcare.mobilehealthplatform.domain.Specimen;
import com.agilex.healthcare.mobilehealthplatform.translator.ReportDocumentMongoTranslator;
import com.agilex.healthcare.mobilehealthplatform.utils.StringUtil;
import com.agilex.healthcare.utility.DateHelper;
import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;

import java.util.Date;

public class ReportDocumentMongoTranslatorTest {
    private static final String DOCUMENT_TYPE = "documentType";
    private static final String RECORD_ID = "recordId";
    private static final String TITLE = "title";
    private static final String TIME_STAMP = "timestamp";
    private static final String ENTRY_DATE = "entryDate";
    private static final String DISCHARGE_DATE = "dischargeDate";
    private static final String LOCAL_TITLE = "localTitle";
    private static final String STANDARD_TITLE = "standardTitle";

    private static final String AUTHOR = "author";
    private static final String AUTHOR_ID = "authorId";
    private static final String AUTHOR_NAME = "authorName";
    private static final String AUTHOR_SIGNATURE = "authorSignature";
    private static final String APPROVED_BY = "approvedBy";

    private static final String TEXT = "text";
	private static final String LOCATION = "location";

	private static final String FACILITY = "facility";
	private static final String FACILITY_NAME = "facilityName";
	private static final String NAME = "name";
	private static final String TYPE = "type";
	private static final String STATE = "state";
	private static final String CITY = "city";
	private static final String ADDRESS = "address";
	private static final String PARENT_SITE_CODE = "parentSiteCode";

	private static final String SPECIMEN = "specimen";
	private static final String SPECIMEN_ID = "specimenId";
	private static final String SPECIMEN_NAME = "specimenName";
	private static final String SPECIMEN_COLLECTION_DATE = "specimenCollectionDate";
	private static final String SPECIMEN_ACCESSION_NUMBER = "specimenAccessionNumber";

	private static final String EXAM = "exam";

	private static final String TEST_ID = "testId";
	private static final String REPORT = "report";
	private static final String TEST_NAME = "testName";
	private static final String SOURCE_SYSTEM = "sourceSystem";
	private static final String RESULTED_DATE = "resultedDate";

	@Test
	public void DischargeSummaryOrProgressNotesTranslatorTest(){
		ReportDocument reportDocument = new ReportDocument();
		reportDocument.setRecordId("1234");
    	reportDocument.setTimestamp(DateHelper.parseVistaDefaultDateTIme("20140201.100819"));
    	reportDocument.setEntryDate(DateHelper.parseVistaDefaultDateTIme("20140202.100819"));
    	reportDocument.setDischargeDate(DateHelper.parseVistaDefaultDateTIme("20140203.100819"));
    	reportDocument.setLocalTitle("Discharge Summary Local Title");
    	reportDocument.setStandardTitle("Discharge Summary Standard Title");
    	reportDocument.setAuthor(authorInfo());
        reportDocument.setText("Discharge Summary Text");
        reportDocument.setApprovedBy(approvedByInfo());
        reportDocument.setDocumentType(ReportDocumentMongoTranslator.DISCHARGE_SUMMARY);

        ReportDocumentMongoTranslator translator = new ReportDocumentMongoTranslator();
        DBObject dbObject = translator.translateDischargeSummaryOrProgessNotes(reportDocument);

        Assert.assertEquals("Check id:", "1234", StringUtil.objectToString(dbObject.get(RECORD_ID)));
        Assert.assertEquals("Check timestamp:", "02/01/2014 10:08:19", DateHelper.formatDateTime((Date)dbObject.get(TIME_STAMP)));
        Assert.assertEquals("Check EntryDate:", "02/02/2014 10:08:19", DateHelper.formatDateTime((Date)dbObject
                .get(ENTRY_DATE)));
        Assert.assertEquals("Check DischargeDate:", "02/03/2014 10:08:19", DateHelper.formatDateTime((Date)dbObject
                .get(DISCHARGE_DATE)));
        Assert.assertEquals("Check LocalTitle:", "Discharge Summary Local Title", StringUtil.objectToString(dbObject.get(LOCAL_TITLE)));
        Assert.assertEquals("Check StandardTitle:", "Discharge Summary Standard Title", StringUtil.objectToString(dbObject.get(STANDARD_TITLE)));
        DBObject authorTest= (BasicDBObject)dbObject.get(AUTHOR);
        Assert.assertEquals("Check AuthorId:", "1", StringUtil.objectToString(authorTest.get(AUTHOR_ID)));
        Assert.assertEquals("Check AuthorName:", "Abc", StringUtil.objectToString(authorTest.get(AUTHOR_NAME)));
        Assert.assertEquals("Check AuthorSignature:", "ABC", StringUtil.objectToString(authorTest.get(AUTHOR_SIGNATURE)));
        Assert.assertEquals("Check Text:", "Discharge Summary Text", StringUtil.objectToString(dbObject.get(TEXT)));
        DBObject approvedByTest= (BasicDBObject)dbObject.get(APPROVED_BY);
        Assert.assertEquals("Check AuthorId:", "2", StringUtil.objectToString(approvedByTest.get(AUTHOR_ID)));
        Assert.assertEquals("Check AuthorName:", "Abc2", StringUtil.objectToString(approvedByTest.get(AUTHOR_NAME)));
        Assert.assertEquals("Check AuthorSignature:", "ABC2", StringUtil.objectToString(approvedByTest.get(AUTHOR_SIGNATURE)));
        Assert.assertEquals("Check DocumentType:", ReportDocumentMongoTranslator.DISCHARGE_SUMMARY, StringUtil.objectToString(dbObject.get(DOCUMENT_TYPE)));

        ReportDocument rd = translator.translateToDischargeSummaryOrProgessNotes(dbObject);

        Assert.assertEquals("Check id:", "1234", rd.getRecordId());
        Assert.assertEquals("Check timestamp:", "02/01/2014 10:08:19", DateHelper.formatDateTime(rd.getTimestamp()));
        Assert.assertEquals("Check EntryDate:", "02/02/2014 10:08:19", DateHelper.formatDateTime(rd.getEntryDate()));
        Assert.assertEquals("Check DischargeDate:", "02/03/2014 10:08:19", DateHelper.formatDateTime(rd.getDischargeDate()));
        Assert.assertEquals("Check LocalTitle:", "Discharge Summary Local Title", rd.getLocalTitle());
        Assert.assertEquals("Check StandardTitle:", "Discharge Summary Standard Title", rd.getStandardTitle());

        Author authorRdTest= rd.getAuthor();
        Assert.assertEquals("Check AuthorId:", "1", authorRdTest.getId());
        Assert.assertEquals("Check AuthorName:", "Abc", authorRdTest.getName());
        Assert.assertEquals("Check AuthorSignature:", "ABC",authorRdTest.getSignature());

        Assert.assertEquals("Check Text:", "Discharge Summary Text", rd.getText());

        Author approvedByRdTest= rd.getApprovedBy();
        Assert.assertEquals("Check AuthorId:", "2", approvedByRdTest.getId());
        Assert.assertEquals("Check AuthorName:", "Abc2", approvedByRdTest.getName());
        Assert.assertEquals("Check AuthorSignature:", "ABC2", approvedByRdTest.getSignature());
        Assert.assertEquals("Check DocumentType:", ReportDocumentMongoTranslator.DISCHARGE_SUMMARY, rd.getDocumentType());

	}

	@Test
	public void PathologyReportTranslatorTest(){
		ReportDocument reportDocument = new ReportDocument();
		reportDocument.setTitle("Pathology report Title");
		reportDocument.setTimestamp(DateHelper.parseDateTime("02/01/2014 10:08:19"));
    	reportDocument.setFacility(facilityInfo());
    	reportDocument.setSpecimen(specimenInfo());
    	reportDocument.setExam("Pathology report Exam");
        reportDocument.setDocumentType(ReportDocumentMongoTranslator.PATHOLOGY_REPORT);

        ReportDocumentMongoTranslator translator = new ReportDocumentMongoTranslator();
        DBObject dbObject = translator.translatePathologyReport(reportDocument);

        Assert.assertEquals("Check timestamp:", "02/01/2014 10:08:19", DateHelper.formatDateTime((Date)dbObject.get(TIME_STAMP)));
        Assert.assertEquals("Check Title:", "Pathology report Title", StringUtil.objectToString(dbObject.get(TITLE)));
        DBObject facilityTest= (BasicDBObject)dbObject.get(FACILITY);
        Assert.assertEquals("Check Facility Name:", "MNO Test", StringUtil.objectToString(facilityTest.get(NAME)));
        Assert.assertEquals("Check Facility Type:", "General", StringUtil.objectToString(facilityTest.get(TYPE)));
        Assert.assertEquals("Check Facility State:", "VA", StringUtil.objectToString(facilityTest.get(STATE)));
        Assert.assertEquals("Check Facility City:", "Chantilly", StringUtil.objectToString(facilityTest.get(CITY)));
        Assert.assertEquals("Check Facility Address:", "123 OneTwoThree street", StringUtil.objectToString(facilityTest.get(ADDRESS)));
        Assert.assertEquals("Check Facility Parent Site Code", "777", StringUtil.objectToString(facilityTest.get(PARENT_SITE_CODE)));
        Assert.assertEquals("Check Exam:", "Pathology report Exam", StringUtil.objectToString(dbObject.get(EXAM)));
        DBObject specimenTest= (BasicDBObject)dbObject.get(SPECIMEN);
        Assert.assertEquals("Check Specimen Id:", "1234567", StringUtil.objectToString(specimenTest.get(SPECIMEN_ID)));
        Assert.assertEquals("Check Specimen Name:", "ABC germs", StringUtil.objectToString(specimenTest.get(SPECIMEN_NAME)));
        Assert.assertEquals("Check Specimen Collection Date:", "02/04/2014 10:08:19", StringUtil.objectToString(specimenTest.get(SPECIMEN_COLLECTION_DATE)));
        Assert.assertEquals("Check Specimen Accession Number:", "1010101010", StringUtil.objectToString(specimenTest.get(SPECIMEN_ACCESSION_NUMBER)));
        Assert.assertEquals("Check DocumentType:", ReportDocumentMongoTranslator.PATHOLOGY_REPORT, StringUtil.objectToString(dbObject.get(DOCUMENT_TYPE)));

        ReportDocument rd = translator.translateToPathologyReport(dbObject);
        Assert.assertEquals("Check timestamp:", "02/01/2014 10:08:19", DateHelper.formatDateTime(rd.getTimestamp()));
        Assert.assertEquals("Check Title:", "Pathology report Title", rd.getTitle());
        Facility facility= rd.getFacility();
        Assert.assertEquals("Check Facility Name:", "MNO Test", facility.getName());
        Assert.assertEquals("Check Facility Type:", "General",facility.getType());
        Assert.assertEquals("Check Facility State:", "VA",facility.getState());
        Assert.assertEquals("Check Facility City:", "Chantilly",facility.getCity());
        Assert.assertEquals("Check Facility Address:", "123 OneTwoThree street",facility.getAddress());
        Assert.assertEquals("Check Facility Parent Site Code:", "777",facility.getParentSiteCode());
        Assert.assertEquals("Check Exam:", "Pathology report Exam", rd.getExam());
        Specimen specimen= rd.getSpecimen();
        Assert.assertEquals("Check Specimen Id:", "1234567", specimen.getId());
        Assert.assertEquals("Check Specimen Name:", "ABC germs", specimen.getName());
        Assert.assertEquals("Check Specimen Collection Date:", "02/04/2014 10:08:19", DateHelper.formatDateTime(specimen.getCollectionDate()));
        Assert.assertEquals("Check Specimen Accession Number:", "1010101010", specimen.getAccessionNumber());
        Assert.assertEquals("Check DocumentType:", ReportDocumentMongoTranslator.PATHOLOGY_REPORT, rd.getDocumentType());
	}

	@Test
	public void RadiologyReportTranslatorTest(){
		ReportDocument reportDocument = new ReportDocument();
		reportDocument.setTitle("Radiology report Title");
		reportDocument.setRecordId("1234");
		reportDocument.setTimestamp(DateHelper.parseDateTime("02/01/2014 10:08:19"));
    	reportDocument.setFacilityName("123 Facility");
    	reportDocument.setTestId("6789");
    	reportDocument.setReport("Radiology Report");
    	reportDocument.setTestName("Radiology Testing");
    	reportDocument.setSourceSystem("Vista Source System");
    	reportDocument.setResultedDate(DateHelper.parseDateTime("02/05/2014 10:08:19"));
        reportDocument.setDocumentType(ReportDocumentMongoTranslator.RADIOLOGY_REPORT);

        ReportDocumentMongoTranslator translator = new ReportDocumentMongoTranslator();
        DBObject dbObject = translator.translateRadiologyReport(reportDocument);

        Assert.assertEquals("Check Id:", "1234",StringUtil.objectToString(dbObject.get(RECORD_ID)));
        Assert.assertEquals("Check Title:", "Radiology report Title",StringUtil.objectToString(dbObject.get(TITLE)));
        Assert.assertEquals("Check Timestamp:", "02/01/2014 10:08:19",DateHelper.formatDateTime((Date)dbObject.get(TIME_STAMP)));
        Assert.assertEquals("Check Facility Name:", "123 Facility",StringUtil.objectToString(dbObject.get(FACILITY_NAME)));
        Assert.assertEquals("Check Test Id:", "6789",StringUtil.objectToString(dbObject.get(TEST_ID)));
        Assert.assertEquals("Check Report:", "Radiology Report",StringUtil.objectToString(dbObject.get(REPORT)));
        Assert.assertEquals("Check Test Name:", "Radiology Testing",StringUtil.objectToString(dbObject.get(TEST_NAME)));
        Assert.assertEquals("Check Source System:", "Vista Source System",StringUtil.objectToString(dbObject.get(SOURCE_SYSTEM)));
        Assert.assertEquals("Check Result Date:", "02/05/2014 10:08:19",StringUtil.objectToString(dbObject.get(RESULTED_DATE)));
        Assert.assertEquals("Check Document Type:", ReportDocumentMongoTranslator.RADIOLOGY_REPORT,StringUtil.objectToString(dbObject.get(DOCUMENT_TYPE)));

        ReportDocument rd = translator.translateToRadiologyReport(dbObject);
        Assert.assertEquals("Check Title:", "Radiology report Title", rd.getTitle());
        Assert.assertEquals("Check Id:", "1234", rd.getRecordId());
        Assert.assertEquals("Check timestamp:", "02/01/2014 10:08:19",DateHelper.formatDateTime(rd.getTimestamp()));
        Assert.assertEquals("Check Facility Name:", "123 Facility", rd.getFacilityName());
        Assert.assertEquals("Check Test Id:", "6789",rd.getTestId());
        Assert.assertEquals("Check Report:", "Radiology Report",rd.getReport());
        Assert.assertEquals("Check Test Name:", "Radiology Testing",rd.getTestName());
        Assert.assertEquals("Check Source System:", "Vista Source System", rd.getSourceSystem());
        Assert.assertEquals("Check Resulted Date:", "02/05/2014 10:08:19", DateHelper.formatDateTime(rd.getResultedDate()));
        Assert.assertEquals("Check Document Type:", ReportDocumentMongoTranslator.RADIOLOGY_REPORT, rd.getDocumentType());
	}

	@Test
	public void SurgeryReportTranslatorTest(){
		ReportDocument reportDocument = new ReportDocument();
		reportDocument.setTitle("Surgery report Title");
		reportDocument.setRecordId("1234");
		reportDocument.setTimestamp(DateHelper.parseDateTime("02/01/2014 10:08:19"));
		reportDocument.setAuthor(authorInfo());
    	reportDocument.setText("Surgery Text");
    	reportDocument.setLocation("Surgery Location");
    	reportDocument.setDocumentType(ReportDocumentMongoTranslator.SURGERY_REPORT);

        ReportDocumentMongoTranslator translator = new ReportDocumentMongoTranslator();
        DBObject dbObject = translator.translateSurgeryReport(reportDocument);

        Assert.assertEquals("Check Id:", "1234",StringUtil.objectToString(dbObject.get(RECORD_ID)));
        Assert.assertEquals("Check Title:", "Surgery report Title",StringUtil.objectToString(dbObject.get(TITLE)));
        Assert.assertEquals("Check Timestamp:", "02/01/2014 10:08:19",DateHelper.formatDateTime((Date)dbObject.get(TIME_STAMP)));
        DBObject authorTest= (BasicDBObject)dbObject.get(AUTHOR);
        Assert.assertEquals("Check AuthorId:", "1", StringUtil.objectToString(authorTest.get(AUTHOR_ID)));
        Assert.assertEquals("Check AuthorName:", "Abc", StringUtil.objectToString(authorTest.get(AUTHOR_NAME)));
        Assert.assertEquals("Check AuthorSignature:", "ABC", StringUtil.objectToString(authorTest.get(AUTHOR_SIGNATURE)));
        Assert.assertEquals("Check Text:", "Surgery Text",StringUtil.objectToString(dbObject.get(TEXT)));
        Assert.assertEquals("Check Location:", "Surgery Location",StringUtil.objectToString(dbObject.get(LOCATION)));
        Assert.assertEquals("Check Document Type:", ReportDocumentMongoTranslator.SURGERY_REPORT,StringUtil.objectToString(dbObject.get(DOCUMENT_TYPE)));

        ReportDocument rd = translator.translateToSurgeryReport(dbObject);
        Assert.assertEquals("Check Title:", "Surgery report Title", rd.getTitle());
        Assert.assertEquals("Check Id:", "1234", rd.getRecordId());
        Assert.assertEquals("Check timestamp:", "02/01/2014 10:08:19",DateHelper.formatDateTime((Date)rd.getTimestamp()));
        Author authorRdTest= rd.getAuthor();
        Assert.assertEquals("Check AuthorId:", "1", authorRdTest.getId());
        Assert.assertEquals("Check AuthorName:", "Abc", authorRdTest.getName());
        Assert.assertEquals("Check AuthorSignature:", "ABC",authorRdTest.getSignature());
        Assert.assertEquals("Check Text:", "Surgery Text",rd.getText());
        Assert.assertEquals("Check Location:", "Surgery Location",rd.getLocation());
        Assert.assertEquals("Check Document Type:", ReportDocumentMongoTranslator.SURGERY_REPORT, rd.getDocumentType());
	}

	private Author authorInfo(){
		Author author = new Author();
		author.setId("1");
		author.setName("Abc");
		author.setSignature("ABC");
		return author;
	}

	private Author approvedByInfo(){
		Author author = new Author();
		author.setId("2");
		author.setName("Abc2");
		author.setSignature("ABC2");
		return author;
	}

	private Facility facilityInfo(){
		Facility facility = new Facility();
		facility.setAddress("123 OneTwoThree street");
		facility.setCity("Chantilly");
		facility.setFacilityCode("123xyz");
		facility.setName("MNO Test");
		facility.setParentSiteCode("777");
		facility.setState("VA");
		facility.setType("General");
		return facility;
	}
	private Specimen specimenInfo(){
		Specimen specimen = new Specimen();
		specimen.setId("1234567");
		specimen.setName("ABC germs");
		specimen.setCollectionDate(DateHelper.parseVistaDefaultDateTIme("20140204.100819"));
		specimen.setAccessionNumber("1010101010");
		return specimen;
	}

}
