package com.agilex.healthcare.mobilehealthplatform.translator.lab;

import static org.junit.Assert.assertEquals;

import java.util.Iterator;
import java.util.List;

import org.junit.Test;

import com.agilex.healthcare.mdws.generatedwsdl.emrservice.ArrayOfGraphingItemDataTO;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.ArrayOfItemForGraphingTypeTO;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.GraphingItemDataTO;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.ItemForGraphingTypeTO;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.TaggedGraphingItemDataArray;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.TaggedItemForGraphingTypeArray;
import com.agilex.healthcare.mdws.translator.LabTestTranslator;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResult;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResults;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTest;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTests;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.utility.DateHelper;

public class LabTestTranslatorTest {
	private final String SITE_CODE="999";
	private final String GRAPHING_TYPE="63";
	@Test
	public void translateLVTaggedItemForGraphingTypeArrayTest() {
		PatientIdentifier patientIdentifier = new PatientIdentifier("TESTAUTHORITY", "TESTID");

		LabTestTranslator labTestTranslator = new LabTestTranslator();
		LabTests labTests = labTestTranslator.translateTaggedItemForGraphingTypeArray(patientIdentifier, SITE_CODE, mockMdwsLabTests());

		assertEquals(labTests.getLabTest().size(), 3);

		Iterator<LabTest> iterator = labTests.getLabTest().iterator();
		LabTest labTest1 = iterator.next();
		assertEquals(labTest1.getIen(), "1");
		assertEquals(labTest1.getTestname(), "WBC");
		assertEquals(labTest1.getAccessionAreaIen(), "10");
		assertEquals(labTest1.getAccessionAreaName(), "lab-HEMATOLOGY");
		assertEquals(labTest1.getAccessionSubscript(), "HE");
		assertEquals(labTest1.getSiteCode(), SITE_CODE);
		assertEquals(labTest1.getMostCurrentResultDate(), DateHelper.parseAnyVistaDateString(DateHelper.convertPureVistaDateToKnownVistaDateFormat("3140903.105404")));
		assertEquals(labTest1.getPatientIdentifier().getAssigningAuthority(), "TESTAUTHORITY");
		assertEquals(labTest1.getPatientIdentifier().getUniqueId(), "TESTID");
		assertEquals(labTest1.getDataIdentifier().getSystemId(), SITE_CODE);
		assertEquals(labTest1.getDataIdentifier().getUniqueId(), "1");

		LabTest labTest2 = iterator.next();
		assertEquals(labTest2.getIen(), "2");
		assertEquals(labTest2.getTestname(), "WBC");
		assertEquals(labTest2.getAccessionAreaIen(), "10");
		assertEquals(labTest2.getAccessionAreaName(), "lab-HEMATOLOGY");
		assertEquals(labTest2.getAccessionSubscript(), "HE");
		assertEquals(labTest2.getSiteCode(), SITE_CODE);
		assertEquals(labTest2.getMostCurrentResultDate(), DateHelper.parseAnyVistaDateString(DateHelper.convertPureVistaDateToKnownVistaDateFormat("3140303.105404")));
		assertEquals(labTest2.getPatientIdentifier().getAssigningAuthority(), "TESTAUTHORITY");
		assertEquals(labTest2.getPatientIdentifier().getUniqueId(), "TESTID");
		assertEquals(labTest2.getDataIdentifier().getSystemId(), SITE_CODE);
		assertEquals(labTest2.getDataIdentifier().getUniqueId(), "2");

		LabTest labTest3 = iterator.next();
		assertEquals(labTest3.getIen(), "3");
		assertEquals(labTest3.getTestname(), "ABC");
		assertEquals(labTest3.getAccessionAreaIen(), "11");
		assertEquals(labTest3.getAccessionAreaName(), "lab-CHEMISTRY");
		assertEquals(labTest3.getAccessionSubscript(), "CH");
		assertEquals(labTest3.getSiteCode(), SITE_CODE);
		assertEquals(labTest3.getMostCurrentResultDate(), DateHelper.parseAnyVistaDateString(DateHelper.convertPureVistaDateToKnownVistaDateFormat("3140403.105606")));
		assertEquals(labTest3.getPatientIdentifier().getAssigningAuthority(), "TESTAUTHORITY");
		assertEquals(labTest3.getPatientIdentifier().getUniqueId(), "TESTID");
		assertEquals(labTest3.getDataIdentifier().getSystemId(), SITE_CODE);
		assertEquals(labTest3.getDataIdentifier().getUniqueId(), "3");
	}

	@Test
	public void translateItemForGraphingTypeTest() {
		PatientIdentifier patientIdentifier = new PatientIdentifier("TESTAUTHORITY", "TESTID");

		LabTestTranslator labTestTranslator = new LabTestTranslator();
		LabTest labTest = labTestTranslator.translateItemForGraphingType(patientIdentifier, SITE_CODE, mockMdwsLabTest(
				"1",
				"WBC",
				"10",
				"lab-HEMATOLOGY",
				"HE",
				GRAPHING_TYPE,
				"3140903.105404"));

		assertEquals(labTest.getPatientIdentifier().getAssigningAuthority(), "TESTAUTHORITY");
		assertEquals(labTest.getPatientIdentifier().getUniqueId(), "TESTID");
		assertEquals(labTest.getDataIdentifier().getSystemId(), SITE_CODE);
		assertEquals(labTest.getDataIdentifier().getUniqueId(), "1");
		assertEquals(labTest.getIen(), "1");
		assertEquals(labTest.getTestname(), "WBC");
		assertEquals(labTest.getAccessionAreaIen(), "10");
		assertEquals(labTest.getAccessionAreaName(), "lab-HEMATOLOGY");
		assertEquals(labTest.getAccessionSubscript(), "HE");
		assertEquals(labTest.getSiteCode(), SITE_CODE);
		assertEquals(labTest.getMostCurrentResultDate(), DateHelper.parseAnyVistaDateString(DateHelper.convertPureVistaDateToKnownVistaDateFormat("3140903.105404")));
	}

	@Test
	public void translateLVTaggedGraphingItemDataArrayTest() {
		LabTestTranslator labTestTranslator = new LabTestTranslator();
		PatientIdentifier patientIdentifier = new PatientIdentifier("TESTAUTHORITY", "TESTID");

		LabResults labResults = labTestTranslator.translateTaggedGraphingItemDataArray(patientIdentifier, mockMdwsLabResults(), "WBC");

		assertEquals(labResults.getLabResult().size(), 2);

		Iterator<LabResult> iterator = labResults.getLabResult().iterator();
		LabResult labResult1 = iterator.next();
		assertEquals(labResult1.getPatientIdentifier().getAssigningAuthority(), "TESTAUTHORITY");
		assertEquals(labResult1.getPatientIdentifier().getUniqueId(), "TESTID");
		assertEquals(labResult1.getValue(), "11.1");
		assertEquals(labResult1.getFlag(), "H");
		assertEquals(labResult1.getSpecimenId(), "70");
		assertEquals(labResult1.getSpecimenName(), "BLOOD");
		assertEquals(labResult1.getReferenceHigh(), "8.3");
		assertEquals(labResult1.getReferenceLow(), "3.5");
		assertEquals(labResult1.getValueUnits(), "k/cmm");
		assertEquals("WBC", labResult1.getTestname());
		assertEquals(labResult1.getResultedDate(), DateHelper.parseAnyVistaDateString(DateHelper.convertPureVistaDateToKnownVistaDateFormat("3140903.105404")));

		LabResult labResult2 = iterator.next();
		assertEquals(labResult2.getPatientIdentifier().getAssigningAuthority(), "TESTAUTHORITY");
		assertEquals(labResult2.getPatientIdentifier().getUniqueId(), "TESTID");
		assertEquals(labResult2.getValue(), "8.1");
		assertEquals(labResult2.getFlag(), "L");
		assertEquals(labResult2.getSpecimenId(), "80");
		assertEquals(labResult2.getSpecimenName(), "BLOOD");
		assertEquals(labResult2.getReferenceHigh(), "3.3");
		assertEquals(labResult2.getReferenceLow(), "2.5");
		assertEquals(labResult2.getValueUnits(), "k/cmm");
		assertEquals(labResult2.getResultedDate(), DateHelper.parseAnyVistaDateString(DateHelper.convertPureVistaDateToKnownVistaDateFormat("3140303.105404")));
	}

	@Test
	public void translateLVGraphingItemDataTest() {
		LabTestTranslator labTestTranslator = new LabTestTranslator();
		PatientIdentifier patientIdentifier = new PatientIdentifier("TESTAUTHORITY", "TESTID");

		LabResult labResult = labTestTranslator.translateGraphingItemData(patientIdentifier, mockMdwsLabResult("1",
				"3140903.105404",
				"11.1",
				"H",
				"70",
				"BLOOD",
				"3.5!8.3",
				"k/cmm"), "WBC");

		assertEquals(labResult.getPatientIdentifier().getAssigningAuthority(), "TESTAUTHORITY");
		assertEquals(labResult.getPatientIdentifier().getUniqueId(), "TESTID");
		assertEquals(labResult.getValue(), "11.1");
		assertEquals(labResult.getFlag(), "H");
		assertEquals(labResult.getSpecimenId(), "70");
		assertEquals(labResult.getSpecimenName(), "BLOOD");
		assertEquals(labResult.getReferenceHigh(), "8.3");
		assertEquals(labResult.getReferenceLow(), "3.5");
		assertEquals(labResult.getValueUnits(), "k/cmm");
		assertEquals("WBC", labResult.getTestname());
		assertEquals(labResult.getResultedDate(), DateHelper.parseAnyVistaDateString(DateHelper.convertPureVistaDateToKnownVistaDateFormat("3140903.105404")));
	}

	private TaggedItemForGraphingTypeArray mockMdwsLabTests() {
		TaggedItemForGraphingTypeArray taggedItemForGraphingTypeArray = new TaggedItemForGraphingTypeArray();

		ArrayOfItemForGraphingTypeTO arrayOfItemForGraphingType = new ArrayOfItemForGraphingTypeTO();
		taggedItemForGraphingTypeArray.setItems(arrayOfItemForGraphingType);

		List<ItemForGraphingTypeTO> itemForGraphingTypeList = arrayOfItemForGraphingType.getItemForGraphingTypeTO();

		itemForGraphingTypeList.add(mockMdwsLabTest(
				"1",
				"WBC",
				"10",
				"lab-HEMATOLOGY",
				"HE",
				GRAPHING_TYPE,
				"3140903.105404"
		));

		itemForGraphingTypeList.add(mockMdwsLabTest(
				"2",
				"WBC",
				"10",
				"lab-HEMATOLOGY",
				"HE",
				GRAPHING_TYPE,
				"3140303.105404"
		));

		itemForGraphingTypeList.add(mockMdwsLabTest(
				"3",
				"ABC",
				"11",
				"lab-CHEMISTRY",
				"CH",
				GRAPHING_TYPE,
				"3140403.105606"
		));


		return taggedItemForGraphingTypeArray;
	}

	private ItemForGraphingTypeTO mockMdwsLabTest(String itemIen,
												  String itemName,
												  String accAreaIen,
												  String accAreaName,
												  String accSubscript,
												  String graphingType,
												  String mostCurrentResultDate) {
		ItemForGraphingTypeTO itemForGraphingType = new ItemForGraphingTypeTO();

		itemForGraphingType.setAccAreaIen(accAreaIen);
		itemForGraphingType.setAccAreaName(accAreaName);
		itemForGraphingType.setAccSubscript(accSubscript);
		itemForGraphingType.setGraphingType(graphingType);
		itemForGraphingType.setItemIen(itemIen);
		itemForGraphingType.setItemName(itemName);
		itemForGraphingType.setMostCurrentResultDate(mostCurrentResultDate);

		return itemForGraphingType;
	}

	private TaggedGraphingItemDataArray mockMdwsLabResults() {
		TaggedGraphingItemDataArray taggedGraphingItemDataArray = new TaggedGraphingItemDataArray();

		ArrayOfGraphingItemDataTO arrayOfLVGraphingItemData = new ArrayOfGraphingItemDataTO();
		taggedGraphingItemDataArray.setItems(arrayOfLVGraphingItemData);

		List<GraphingItemDataTO> lvGraphingItemDataList = arrayOfLVGraphingItemData.getGraphingItemDataTO();

		lvGraphingItemDataList.add(
				mockMdwsLabResult("1",
						"3140903.105404",
						"11.1",
						"H",
						"70",
						"BLOOD",
						"3.5!8.3",
						"k/cmm")
		);

		lvGraphingItemDataList.add(
				mockMdwsLabResult("2",
						"3140303.105404",
						"8.1",
						"L",
						"80",
						"BLOOD",
						"2.5!3.3",
						"k/cmm")
		);

		return taggedGraphingItemDataArray;
	}

	private GraphingItemDataTO mockMdwsLabResult(String itemIen,
												 String resultDate,
												 String result,
												 String flag,
												 String specimenIen,
												 String specimenName,
												 String range,
												 String units) {
		GraphingItemDataTO graphingItemData = new GraphingItemDataTO();
		graphingItemData.setItemIen(itemIen);
		graphingItemData.setResultDate(resultDate);
		graphingItemData.setResult(result);
		graphingItemData.setFlag(flag);
		graphingItemData.setSpecimenIen(specimenIen);
		graphingItemData.setSpecimenName(specimenName);
		graphingItemData.setRange(range);
		graphingItemData.setUnits(units);

		return graphingItemData;
	}
}
