package com.agilex.healthcare.notewriter.datalayer;

import junit.framework.Assert;

import org.junit.Test;

import com.agilex.healthcare.mdws.domain.Code;
import com.agilex.healthcare.notewriter.domain.EncounterCode;

public class EncounterCodeTranslatorTest {
	@Test
	public void extractCodes() {
		//Failures
		testExtractCode("This is a test", "", "");
		testExtractCode("This (is a (test)", "", ""); //Assert code empty due to mismatched parenthesis
		testExtractCode("This (is) a (test)", "", ""); //Assert code empty because no space in test.

		//Success
		testExtractCode("This is) (a test)", "a", "test");
		testExtractCode("This (is a test)", "is a", "test");
		testExtractCode("This is (a test)", "a", "test");
		testExtractCode("This (is) (a test)", "a", "test");
		testExtractCode("Appendectomy (CPT-4 44950)", "CPT-4", "44950");
	}
	
	private void testExtractCode(String description, String expectedCodeType, String expectedCode) {
		Code mdwsCode = new Code();
		mdwsCode.setDescription(description);
		mdwsCode.setCode(expectedCode);
		mdwsCode.setCodeType(expectedCodeType);
		EncounterCode code = EncounterCodeTranslator.translateMdwsCodeToEncounterCode(mdwsCode);
		Assert.assertEquals(expectedCode, code.getCode());
		Assert.assertEquals(expectedCodeType, code.getCodeType());
		String expected = description;
		String codeAndType = " (" + expectedCodeType + " " + expectedCode + ")";
		if (!expected.endsWith(codeAndType))
			expected += codeAndType;
		Assert.assertEquals(expected, code.getDescription());
	}
}
