package com.agilex.healthcare.notewriter.translator;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import junit.framework.Assert;

import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEvent;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEventState;
import com.agilex.healthcare.mobilehealthplatform.domain.Encounter;
import com.agilex.healthcare.mobilehealthplatform.domain.EncounterProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.EncounterProviders;
import com.agilex.healthcare.mobilehealthplatform.domain.FacilityProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.Note;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitle;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitRelatedTo;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitRelatedTos;
import com.agilex.healthcare.mobilehealthplatform.utils.StringUtil;
import com.mongodb.BasicDBList;
import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;


public class ClinicalEventTranslatorTest {
	@Test
	public void translateClinicalEvent() {
		Date time = Calendar.getInstance().getTime();
		ClinicalEvent event = ClinicalEventUtilities.createClinicalEvent(time);
		ClinicalEventTranslator translator = new ClinicalEventTranslator();
		
		DBObject dbObject = translator.translateClinicalEvent(event);
		
		String orgTranslation = dbObject.toString();
//		System.out.println(orgTranslation);
		
		ClinicalEvent translatedClinicalEvent = translator.translateClinicalEvent(dbObject);
		DBObject dbObjectFromTranslated = translator.translateClinicalEvent(translatedClinicalEvent);
		String reTranslated = dbObjectFromTranslated.toString();
//		System.out.println(reTranslated);
		
		Assert.assertEquals(orgTranslation, reTranslated);
		ClinicalEventUtilities.assertClinicalEvent(translatedClinicalEvent, time);
	}

	
	@Test
	public void translateExistingClinicalEventWithSignedDateToDbObject() {
		String facilityCode = "500";
		String staffId = "staff001";
		ProviderIdentifier providerIdentifier = new ProviderIdentifier(facilityCode, staffId);
		String assigningAuthority = "EDIPI";
		String patientId = "D123401";
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		String clinicalEventId = "TEST_ID";
		NoteTitle noteTitle = new NoteTitle("1267", "PHARMACY  <TRANSITIONAL PHARMACY BENEFIT NOTE>", true, true);
		String noteBody = "TEST BODY";
		Date signedDate = new Date();
//		String electronicSignatureCode = "DOCTOR1";
		String locationId = "456";
		String locationTitle = "LocationTitle for 456";
		Boolean currentAdmission = false;
		Boolean isHistorical = false;
		String serviceCategory = "A";
		String appointmentType = "Primary";
		String linkType = "Appointment";
		String encounterType = "Not sure";
		Date encounterTimestamp = new Date();
		Date lastModifiedDate = new Date();
		boolean encounterValid = true;
		boolean encounterEdited = true;
		
		VisitRelatedTos visitRelatedTos = new VisitRelatedTos();
		visitRelatedTos.add(new VisitRelatedTo("Title1", Boolean.TRUE, Boolean.FALSE));
		visitRelatedTos.add(new VisitRelatedTo("Title2", Boolean.FALSE, Boolean.FALSE));
		visitRelatedTos.add(new VisitRelatedTo("Title3", Boolean.TRUE, null));
		visitRelatedTos.add(new VisitRelatedTo("Title4", Boolean.FALSE, null));
		EncounterProviders encounterProviders = new EncounterProviders();
		EncounterProvider encProvider = new EncounterProvider();
		encProvider.setPrimary(true);
		FacilityProvider facilityProvider = new FacilityProvider();
		facilityProvider.setProviderIen("12345");
		facilityProvider.setFacilityCode("abcdef");
		encProvider.setProvider(facilityProvider );
		encounterProviders.add(encProvider);
		
		ClinicalEvent clinicalEvent = new ClinicalEvent();
		clinicalEvent.setId(clinicalEventId);
		clinicalEvent.setFacilityCode(facilityCode);
		clinicalEvent.setPatientIdentifier(patientIdentifier);
		clinicalEvent.setProviderIdentifier(providerIdentifier);
		clinicalEvent.setLastModifiedDate(lastModifiedDate);
		clinicalEvent.setEncounterValid(encounterValid);
		clinicalEvent.setEncounterEdited(encounterEdited);
		
		Note note = new Note();
		note.setTitle(noteTitle);
		note.setBody(noteBody);
		note.setSignedDate(signedDate);
		
		Encounter encounter = new Encounter();
		encounter.setAppointmentType(appointmentType);
		encounter.setLocationId(locationId);
		encounter.setLocationTitle(locationTitle);
		encounter.setLinkType(linkType);
		encounter.setHistorical(isHistorical);
		encounter.setCurrentAdmission(currentAdmission);
		encounter.setServiceCategory(serviceCategory);
		encounter.setEncounterType(encounterType);
		encounter.setVisitRelatedTos(visitRelatedTos);
		encounter.setProviders(encounterProviders);
		encounter.setTimestamp(encounterTimestamp);
		
		clinicalEvent.setEncounter(encounter);
		clinicalEvent.setNote(note);
		
		ClinicalEventTranslator translator = new ClinicalEventTranslator();
		
		DBObject dbObject = translator.translateClinicalEvent(clinicalEvent);

		// assert clinical event
		assertEquals(clinicalEventId, StringUtil.objectToString(dbObject.get(ClinicalEventTranslator.ID)));
		assertEquals(facilityCode, StringUtil.objectToString(dbObject.get(ClinicalEventTranslator.FACILITY_CODE)));
		assertEquals(facilityCode, StringUtil.objectToString(((DBObject)dbObject.get(ClinicalEventTranslator.PROVIDER_IDENTIFIER)).get(ClinicalEventTranslator.PROVIDER_FACILITY_CODE)));
		assertEquals(staffId, StringUtil.objectToString(((DBObject)dbObject.get(ClinicalEventTranslator.PROVIDER_IDENTIFIER)).get(ClinicalEventTranslator.PROVIDER_STAFF_ID)));
		assertEquals(assigningAuthority, StringUtil.objectToString(((DBObject)dbObject.get(ClinicalEventTranslator.PATIENT_IDENTIFIER)).get(ClinicalEventTranslator.PATIENT_ASSIGNING_AUTHORITY)));
		assertEquals(patientId, StringUtil.objectToString(((DBObject)dbObject.get(ClinicalEventTranslator.PATIENT_IDENTIFIER)).get(ClinicalEventTranslator.PATIENT_UNIQUE_ID)));
		assertEquals(clinicalEvent.getLastModifiedDate(), (Date)dbObject.get(ClinicalEventTranslator.LAST_MODIFIED_DATE));
		assertEquals(encounterValid, StringUtil.objectToBoolean((dbObject.get(ClinicalEventTranslator.ENCOUNTER_VALID))));
		assertEquals(encounterEdited, StringUtil.objectToBoolean((dbObject.get(ClinicalEventTranslator.ENCOUNTER_EDITED))));
		
		
		// assert note
		NoteTitle translatedNoteTitle = translator.translateTitle(((DBObject)((DBObject)dbObject.get(ClinicalEventTranslator.NOTE)).get(ClinicalEventTranslator.NOTE_TITLE)));
		assertEquals(noteTitle.getIen(), translatedNoteTitle.getIen());
		assertEquals(noteTitle.getName(), translatedNoteTitle.getName());
		assertEquals(noteTitle.isConsult(), translatedNoteTitle.isConsult());
		assertEquals(noteTitle.isPrf(), translatedNoteTitle.isPrf());
		assertEquals(noteBody, StringUtil.objectToString(((DBObject)dbObject.get(ClinicalEventTranslator.NOTE)).get(ClinicalEventTranslator.NOTE_BODY)));
		assertEquals(signedDate, (Date)((DBObject)dbObject.get(ClinicalEventTranslator.NOTE)).get(ClinicalEventTranslator.NOTE_SIGNED_DATE));
		
		// assert encounter
		Encounter translatedEncounter = translator.translateEncounter((DBObject)((DBObject)dbObject.get(ClinicalEventTranslator.ENCOUNTER)));
		assertEquals(appointmentType, translatedEncounter.getAppointmentType());
		assertEquals(locationId, translatedEncounter.getLocationId());
		assertEquals(locationTitle, translatedEncounter.getLocationTitle());
		assertEquals(linkType, translatedEncounter.getLinkType());
		assertEquals(isHistorical, translatedEncounter.isHistorical());
		assertEquals(currentAdmission, translatedEncounter.isCurrentAdmission());
		assertEquals(serviceCategory, translatedEncounter.getServiceCategory());
		assertEquals(encounterType, translatedEncounter.getEncounterType());
		assertEquals(encounterTimestamp, translatedEncounter.getTimestamp());
		
		List<VisitRelatedTo> translatedVisitRelatedTos = new LinkedList<VisitRelatedTo>(translatedEncounter.getVisitRelatedTos());
		Iterator<VisitRelatedTo> iterator = translatedVisitRelatedTos.iterator();
		for(VisitRelatedTo visitRelatedTo : visitRelatedTos) {
			boolean found = false;
			
			while (iterator.hasNext()) {
				VisitRelatedTo translatedVisitRelatedTo = iterator.next();
				if (visitRelatedTo.getTitle().equals(translatedVisitRelatedTo.getTitle())) {
					assertEquals(visitRelatedTo.getChecked(), translatedVisitRelatedTo.getChecked());
					assertEquals(visitRelatedTo.getEnabled(), translatedVisitRelatedTo.getEnabled());
					iterator.remove();
					found = true;
					break;
				}
			}
			
			if (!found) {
				fail(String.format("Corresponding Visit Related To (title=%s)not found", visitRelatedTo.getTitle()));
			}
		}
		assertEquals(0, translatedVisitRelatedTos.size());
		
		List<EncounterProvider> translatedEncounterProviders = new LinkedList<EncounterProvider>(translatedEncounter.getProviders());
		Iterator<EncounterProvider> encounterProviderIterator = translatedEncounterProviders.iterator();
		for(EncounterProvider encounterProvider : encounterProviders) {
			boolean found = false;
			
			while (encounterProviderIterator.hasNext()) {
				EncounterProvider translatedEncounterProvider = encounterProviderIterator.next();
				if (encounterProvider.getProvider() != null && encounterProvider.getProvider().getFacilityCode().equals(translatedEncounterProvider.getProvider().getFacilityCode())
						&& encounterProvider.getProvider().getProviderIen().equals(translatedEncounterProvider.getProvider().getProviderIen())) {
					encounterProviderIterator.remove();
					found = true;
					break;
				}
			}
			
			if (!found) {
				fail(String.format("Corresponding Encounter Provider (%s#%s) To not found",
						(encounterProvider.getProvider() == null ? "<null>" : encounterProvider.getProvider().getFacilityCode()),
						(encounterProvider.getProvider() == null ? "<null>" : encounterProvider.getProvider().getProviderIen())));
			}
		}
		assertEquals(0, translatedVisitRelatedTos.size());
		
	}
	
	@Test
	public void translateNewClinicalEventToDbObject() {
		String facilityCode = "500";
		String staffId = "staff001";
		ProviderIdentifier providerIdentifier = new ProviderIdentifier(facilityCode, staffId);
		String assigningAuthority = "EDIPI";
		String patientId = "D123401";
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		NoteTitle noteTitle = new NoteTitle("1267", "PHARMACY  <TRANSITIONAL PHARMACY BENEFIT NOTE>");
		String noteBody = "TEST BODY";
		Boolean encounterValid = true;
		Boolean encounterEdited = true;
		
		ClinicalEvent clinicalEvent = new ClinicalEvent();
		clinicalEvent.setFacilityCode(facilityCode);
		clinicalEvent.setPatientIdentifier(patientIdentifier);
		clinicalEvent.setProviderIdentifier(providerIdentifier);
		clinicalEvent.setEncounterEdited(encounterEdited);
		clinicalEvent.setEncounterValid(encounterValid);
		
		Note note = new Note();
		note.setTitle(noteTitle);
		note.setBody(noteBody);
		
		clinicalEvent.setNote(note);
		
		ClinicalEventTranslator translator = new ClinicalEventTranslator();
		
		DBObject dbObject = translator.translateClinicalEvent(clinicalEvent);

		// assert clinical event
		assertEquals(facilityCode, StringUtil.objectToString(dbObject.get(ClinicalEventTranslator.FACILITY_CODE)));
		assertEquals(facilityCode, StringUtil.objectToString(((DBObject)dbObject.get(ClinicalEventTranslator.PROVIDER_IDENTIFIER)).get(ClinicalEventTranslator.PROVIDER_FACILITY_CODE)));
		assertEquals(staffId, StringUtil.objectToString(((DBObject)dbObject.get(ClinicalEventTranslator.PROVIDER_IDENTIFIER)).get(ClinicalEventTranslator.PROVIDER_STAFF_ID)));
		assertEquals(assigningAuthority, StringUtil.objectToString(((DBObject)dbObject.get(ClinicalEventTranslator.PATIENT_IDENTIFIER)).get(ClinicalEventTranslator.PATIENT_ASSIGNING_AUTHORITY)));
		assertEquals(patientId, StringUtil.objectToString(((DBObject)dbObject.get(ClinicalEventTranslator.PATIENT_IDENTIFIER)).get(ClinicalEventTranslator.PATIENT_UNIQUE_ID)));
		assertEquals(encounterValid, StringUtil.objectToBoolean((dbObject.get(ClinicalEventTranslator.ENCOUNTER_VALID))));
		assertEquals(encounterEdited, StringUtil.objectToBoolean((dbObject.get(ClinicalEventTranslator.ENCOUNTER_EDITED))));

		
		assertEquals(clinicalEvent.getLastModifiedDate(), (Date)dbObject.get(ClinicalEventTranslator.LAST_MODIFIED_DATE));
		
		// assert note
		NoteTitle translatedNoteTitle = translator.translateTitle(((DBObject)((DBObject)dbObject.get(ClinicalEventTranslator.NOTE)).get(ClinicalEventTranslator.NOTE_TITLE)));
		assertEquals(noteTitle.getIen(), translatedNoteTitle.getIen());
		assertEquals(noteTitle.getName(), translatedNoteTitle.getName());
		assertEquals(noteTitle.isConsult(), translatedNoteTitle.isConsult());
		assertEquals(noteTitle.isPrf(), translatedNoteTitle.isPrf());
		assertEquals(noteBody, StringUtil.objectToString(((DBObject)dbObject.get(ClinicalEventTranslator.NOTE)).get(ClinicalEventTranslator.NOTE_BODY)));
	}
	
	@Test
	public void translateDbObjectToClinicalEvent() {
		String facilityCode = "500";
		ClinicalEventState clinicalEventState = ClinicalEventState.DRAFT;
		//String staffId = "staff001";
		//ProviderIdentifier providerIdentifier = new ProviderIdentifier(facilityCode, staffId);
		String assigningAuthority = "EDIPI";
		String patientId = "D123401";
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		String clinicalEventId = "TEST_ID";
		NoteTitle noteTitle = new NoteTitle("1267", "PHARMACY  <TRANSITIONAL PHARMACY BENEFIT NOTE>", false, false);
		String noteBody = "TEST BODY";
		Date signedDate = new Date();
		String electronicSignatureCode = "DOCTOR1";
		String locationId = "456";
		String locationTitle = "Title for 456";
		String linkType = "Appointment";
		Boolean currentAdmission = false;
		Boolean isHistorical = false;		
		String serviceCategory = "A";
		String appointmentType = "Primary";
		String encounterType = "Not sure";
		Date encounterTimestamp = new Date();
		Date lastModifiedDate = new Date();
		boolean encounterValid = true;
		boolean encounterEdited = true;
		
		VisitRelatedTos visitRelatedTos = new VisitRelatedTos();
		visitRelatedTos.add(new VisitRelatedTo("Title1", Boolean.TRUE, Boolean.FALSE));
		visitRelatedTos.add(new VisitRelatedTo("Title2", Boolean.FALSE, Boolean.FALSE));
		visitRelatedTos.add(new VisitRelatedTo("Title3", Boolean.TRUE, null));
		visitRelatedTos.add(new VisitRelatedTo("Title4", Boolean.FALSE, null));
		EncounterProviders encounterProviders = new EncounterProviders();
		EncounterProvider encProvider = new EncounterProvider();
		encProvider.setPrimary(true);
		FacilityProvider facilityProvider = new FacilityProvider();
		facilityProvider.setProviderIen("12345");
		facilityProvider.setFacilityCode("abcdef");
		encProvider.setProvider(facilityProvider );
		encounterProviders.add(encProvider);

		ClinicalEventTranslator translator = new ClinicalEventTranslator();
		
		DBObject po = new BasicDBObject();
		po.put(ClinicalEventTranslator.ID, clinicalEventId);
		po.put(ClinicalEventTranslator.FACILITY_CODE, facilityCode);
		po.put(ClinicalEventTranslator.ELECTRONIC_SIGNATURE_CODE, electronicSignatureCode);
		po.put(ClinicalEventTranslator.CLINICAL_EVENT_STATE, clinicalEventState);
		po.put(ClinicalEventTranslator.ENCOUNTER_VALID, encounterValid);
		po.put(ClinicalEventTranslator.ENCOUNTER_EDITED, encounterEdited);
		DBObject patientDbObject = new BasicDBObject(2);
		patientDbObject.put(ClinicalEventTranslator.PATIENT_ASSIGNING_AUTHORITY, patientIdentifier.getAssigningAuthority());
		patientDbObject.put(ClinicalEventTranslator.PATIENT_UNIQUE_ID, patientIdentifier.getUniqueId());
		po.put(ClinicalEventTranslator.PATIENT_IDENTIFIER, patientDbObject);
		DBObject providerDbObject = new BasicDBObject(2);
		providerDbObject.put(ClinicalEventTranslator.PROVIDER_FACILITY_CODE, facilityCode);
		//providerDbObject.put(ClinicalEventTranslator.PROVIDER_STAFF_ID, providerIdentifier.getStaffId());
		po.put(ClinicalEventTranslator.PROVIDER_IDENTIFIER, providerDbObject);
		po.put(ClinicalEventTranslator.LAST_MODIFIED_DATE, lastModifiedDate);
		
		DBObject notePo = new BasicDBObject();
		notePo.put(ClinicalEventTranslator.NOTE_TITLE, translator.translateTitle(noteTitle));
		notePo.put(ClinicalEventTranslator.NOTE_BODY, noteBody);
		notePo.put(ClinicalEventTranslator.NOTE_SIGNED_DATE, signedDate);
		
		po.put(ClinicalEventTranslator.NOTE, notePo);

		DBObject encounterPo = new BasicDBObject();
		encounterPo.put(ClinicalEventTranslator.ENCOUNTER_TIMESTAMP, encounterTimestamp);
		encounterPo.put(ClinicalEventTranslator.ENCOUNTER_LOCATION_ID, locationId);
		encounterPo.put(ClinicalEventTranslator.ENCOUNTER_LOCATION_TITLE, locationTitle);
		encounterPo.put(ClinicalEventTranslator.ENCOUNTER_LINK_TYPE, linkType);
		encounterPo.put(ClinicalEventTranslator.ENCOUNTER_IS_HISTORICAL, isHistorical);
		encounterPo.put(ClinicalEventTranslator.ENCOUNTER_CURRENT_ADMISSION, currentAdmission);
		encounterPo.put(ClinicalEventTranslator.ENCOUNTER_SERVICE_CATEGORY, serviceCategory);
		encounterPo.put(ClinicalEventTranslator.ENCOUNTER_APPOINTMENT_TYPE, appointmentType);
		encounterPo.put(ClinicalEventTranslator.ENCOUNTER_ENCOUNTER_TYPE, encounterType);
		encounterPo.put(ClinicalEventTranslator.ENCOUNTER_VISIT_RELATED_TOS, populateDbObjectVisitTos(visitRelatedTos));
		encounterPo.put(ClinicalEventTranslator.ENCOUNTER_PROVIDERS, populateDbObjectProviders(encounterProviders));
		po.put(ClinicalEventTranslator.ENCOUNTER, encounterPo);
		
		ClinicalEvent clinicalEvent = translator.translateClinicalEvent(po);
		

		assertEquals(clinicalEventId, clinicalEvent.getId());
		assertEquals(facilityCode, clinicalEvent.getFacilityCode());
		assertEquals(lastModifiedDate, clinicalEvent.getLastModifiedDate());
		assertEquals(clinicalEventState, clinicalEvent.getClinicalEventState());
		assertEquals(encounterValid, clinicalEvent.isEncounterValid());
		assertEquals(encounterEdited, clinicalEvent.isEncounterEdited());
		
		
		assertEquals(facilityCode, clinicalEvent.getProviderIdentifier().getFacilityCode());
//		assertEquals(staffId, clinicalEvent.getProviderIdentifier().getStaffId());
		
		assertEquals(assigningAuthority, clinicalEvent.getPatientIdentifier().getAssigningAuthority());
		assertEquals(patientId, clinicalEvent.getPatientIdentifier().getUniqueId());
		
		assertEquals(noteTitle.getIen(), clinicalEvent.getNote().getTitle().getIen());
		assertEquals(noteTitle.getName(), clinicalEvent.getNote().getTitle().getName());
		assertEquals(noteTitle.isConsult(), clinicalEvent.getNote().getTitle().isConsult());
		assertEquals(noteTitle.isPrf(), clinicalEvent.getNote().getTitle().isPrf());
		assertEquals(noteBody, clinicalEvent.getNote().getBody());
		assertEquals(signedDate, clinicalEvent.getNote().getSignedDate());
		
		assertEquals(encounterTimestamp, clinicalEvent.getEncounter().getTimestamp());
		assertEquals(locationId, clinicalEvent.getEncounter().getLocationId());
		assertEquals(locationTitle, clinicalEvent.getEncounter().getLocationTitle());
		assertEquals(linkType, clinicalEvent.getEncounter().getLinkType());
		assertEquals(isHistorical, clinicalEvent.getEncounter().isHistorical());
		assertEquals(currentAdmission, clinicalEvent.getEncounter().isCurrentAdmission());		
		assertEquals(serviceCategory, clinicalEvent.getEncounter().getServiceCategory());
		assertEquals(appointmentType, clinicalEvent.getEncounter().getAppointmentType());
		assertEquals(encounterType, clinicalEvent.getEncounter().getEncounterType());
		
		List<VisitRelatedTo> translatedVisitRelatedTos = new LinkedList<VisitRelatedTo>(clinicalEvent.getEncounter().getVisitRelatedTos());
		Iterator<VisitRelatedTo> iterator = translatedVisitRelatedTos.iterator();
		for(VisitRelatedTo visitRelatedTo : visitRelatedTos) {
			boolean found = false;
			
			while (iterator.hasNext()) {
				VisitRelatedTo translatedVisitRelatedTo = iterator.next();
				if (visitRelatedTo.getTitle().equals(translatedVisitRelatedTo.getTitle())) {
					assertEquals(visitRelatedTo.getChecked(), translatedVisitRelatedTo.getChecked());
					assertEquals(visitRelatedTo.getEnabled(), translatedVisitRelatedTo.getEnabled());
					iterator.remove();
					found = true;
					break;
				}
			}
			
			if (!found) {
				fail(String.format("Corresponding Visit Related To (title=%s)not found", visitRelatedTo.getTitle()));
			}
		}
		assertEquals(0, translatedVisitRelatedTos.size());
		
		List<EncounterProvider> translatedEncounterProviders = new LinkedList<EncounterProvider>(clinicalEvent.getEncounter().getProviders());
		Iterator<EncounterProvider> encounterProviderIterator = translatedEncounterProviders.iterator();
		for(EncounterProvider encounterProvider : encounterProviders) {
			boolean found = false;
			
			while (encounterProviderIterator.hasNext()) {
				EncounterProvider translatedEncounterProvider = encounterProviderIterator.next();
				if (encounterProvider.getProvider() != null && encounterProvider.getProvider().getFacilityCode().equals(translatedEncounterProvider.getProvider().getFacilityCode())
						&& encounterProvider.getProvider().getProviderIen().equals(translatedEncounterProvider.getProvider().getProviderIen())) {
					encounterProviderIterator.remove();
					found = true;
					break;
				}
			}
			
			if (!found) {
				fail(String.format("Corresponding Encounter Provider (%s#%s) To not found",
						(encounterProvider.getProvider() == null ? "<null>" : encounterProvider.getProvider().getFacilityCode()),
						(encounterProvider.getProvider() == null ? "<null>" : encounterProvider.getProvider().getProviderIen())));
			}
		}
		assertEquals(0, translatedVisitRelatedTos.size());
	}
	

	private BasicDBList populateDbObjectVisitTos(VisitRelatedTos visitRelatedTos) {
		BasicDBList dbList = new BasicDBList();
		
		for (int i = 0; i < visitRelatedTos.size(); i++) {
			VisitRelatedTo visitRelatedTo = visitRelatedTos.get(i);
			DBObject dbObject = new BasicDBObject(2);
			dbObject.put(ClinicalEventTranslator.ENCOUNTER_VIS_TITLE, visitRelatedTo.getTitle());

		    if (visitRelatedTo.getChecked() != null) {
		    	dbObject.put(ClinicalEventTranslator.ENCOUNTER_VIS_CHECKED, visitRelatedTo.getChecked());
		    }

		    if (visitRelatedTo.getEnabled() != null) {
		    	dbObject.put(ClinicalEventTranslator.ENCOUNTER_VIS_ENABLED, visitRelatedTo.getEnabled());
		    }
			
			dbList.add(dbObject);
		}
		return dbList;
	}

	private BasicDBList populateDbObjectProviders(EncounterProviders encounterProviders) {
		//Providers
		BasicDBList dbList = new BasicDBList();
		
		for (int i = 0; i < encounterProviders.size(); i++) {
			EncounterProvider encounterProvider = encounterProviders.get(i);
			
			//providerIdentifier, facilityCode, staffId
			DBObject dbObjectProvider = new BasicDBObject(1);
			DBObject dbObjectProviderId = new BasicDBObject(2);
			if (encounterProvider.getProvider() != null) {
				dbObjectProviderId.put(ClinicalEventTranslator.ENCOUNTER_PROV_FACILITY_CODE, encounterProvider.getProvider().getFacilityCode());
				dbObjectProviderId.put(ClinicalEventTranslator.ENCOUNTER_PROV_STAFF_ID, encounterProvider.getProvider().getProviderIen());
			}
			dbObjectProvider.put(ClinicalEventTranslator.ENCOUNTER_PROV_IDENTIFIER, dbObjectProviderId);
			
			DBObject dbObject = new BasicDBObject(2);
			dbObject.put(ClinicalEventTranslator.ENCOUNTER_PROV_PRIMARY, encounterProvider.isPrimary());
			dbObject.put(ClinicalEventTranslator.ENCOUNTER_PROV_PROVIDER, dbObjectProvider);
			
			dbList.add(dbObject);
		}
		return dbList;
	}

	@Test
	public void translateLastSelectedNoteTitleToDBObject() {
		ClinicalEventTranslator translator = new ClinicalEventTranslator();
		String facilityCode = "code";
		String staffId = "staffid";
		String ien = "ienn";
		String name = "title";
		NoteTitle title = new NoteTitle(ien, name);
		
		DBObject dbObject = translator.translateLastSelectedTitle(facilityCode, staffId, title);
		
		assertEquals(facilityCode, StringUtil.objectToString(((DBObject)dbObject.get(ClinicalEventTranslator.PROVIDER_IDENTIFIER)).get(ClinicalEventTranslator.PROVIDER_FACILITY_CODE)));
		assertEquals(staffId, StringUtil.objectToString(((DBObject)dbObject.get(ClinicalEventTranslator.PROVIDER_IDENTIFIER)).get(ClinicalEventTranslator.PROVIDER_STAFF_ID)));
		assertEquals(ien, StringUtil.objectToString(dbObject.get(ClinicalEventTranslator.NOTE_TITLE_IEN)));
		assertEquals(name, StringUtil.objectToString(dbObject.get(ClinicalEventTranslator.NOTE_TITLE_NAME)));
	}
	
	@Test
	public void translateDBObjectToLastSelectedNoteTitle() {
		ClinicalEventTranslator translator = new ClinicalEventTranslator();
		String facilityCode = "code";
		String staffId = "staffid";
		String ien = "ienn";
		String name = "title";
		
		DBObject po = new BasicDBObject();
		po.put(ClinicalEventTranslator.NOTE_TITLE_IEN, ien);
		po.put(ClinicalEventTranslator.NOTE_TITLE_NAME, name);
		
		DBObject provider = new BasicDBObject();
		provider.put(ClinicalEventTranslator.PROVIDER_FACILITY_CODE, facilityCode);
		provider.put(ClinicalEventTranslator.PROVIDER_STAFF_ID, staffId);
		po.put(ClinicalEventTranslator.PROVIDER_IDENTIFIER, provider);
		
		NoteTitle title = translator.translateTitle(po);

		assertEquals(ien, title.getIen());
		assertEquals(name, title.getName());
	}
	
	@Test
	public void translateNullDbObjectToNote() {
		DBObject po = null;
		ClinicalEventTranslator translator = new ClinicalEventTranslator();
		ClinicalEvent clinicalEvent = translator.translateClinicalEvent(po);
		
		assertTrue(clinicalEvent == null);
	}
}
