package com.agilex.healthcare.notewriter.translator;

import java.util.Date;

import org.junit.Assert;

import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEvent;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEventState;
import com.agilex.healthcare.mobilehealthplatform.domain.Encounter;
import com.agilex.healthcare.mobilehealthplatform.domain.EncounterProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.EncounterProviders;
import com.agilex.healthcare.mobilehealthplatform.domain.FacilityProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.Modifier;
import com.agilex.healthcare.mobilehealthplatform.domain.Modifiers;
import com.agilex.healthcare.mobilehealthplatform.domain.Note;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitle;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitRelatedTo;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitRelatedTos;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitType;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitTypes;
import com.agilex.healthcare.notewriter.domain.EncounterDiagnosisICDCode;
import com.agilex.healthcare.notewriter.domain.EncounterDiagnosisICDCodes;
import com.agilex.healthcare.notewriter.domain.EncounterProcedureCPTCode;
import com.agilex.healthcare.notewriter.domain.EncounterProcedureCPTCodes;
import com.agilex.healthcare.utility.NullChecker;
import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;

public class ClinicalEventUtilities {

   // @Resource
   // private static PropertyHelper propertyHelper;


	public static void removeTestDataFromMongo(MongoClient mongoClient,org.apache.commons.logging.Log log) {
		try {

			DB db = mongoClient.getDB("note-writer");
			DBCollection clinicalEventsCollection = db.getCollection("clinical-events");

			BasicDBObject query = new BasicDBObject();
			query.put("facilityCode", java.util.regex.Pattern.compile("^TESTDATA - my fac - "));


			DBCursor iterator = clinicalEventsCollection.find(query);
			log.debug("Starting removal of test data from Mongo");
			while (iterator.hasNext()) {
				DBObject dbObject = iterator.next();
//				log.debug(dbObject.toString());
				clinicalEventsCollection.remove(dbObject);
			}
			log.debug("Finished removal of test data from Mongo");
		}
		catch (com.mongodb.MongoException e) {
			Assert.fail(e.getMessage());
		}
	}

	public static ClinicalEvent createClinicalEventMinimal(Date time) {
		String somethingUnique = "" + time.getTime();
		ClinicalEvent clinicalEvent = new ClinicalEvent();

		clinicalEvent.setId(null); //This should be updated by a Save in the DataService (not in the mongo data service).
		clinicalEvent.setLastModifiedDate(null); //This should be updated by a Save or an Update in the DataService (not in the mongo data service)

		clinicalEvent.setFacilityCode("TESTDATA - my fac - " + somethingUnique);
		clinicalEvent.setClinicalEventState(ClinicalEventState.DRAFT);

		PatientIdentifier patientIdentifier = new PatientIdentifier();
		patientIdentifier.setAssigningAuthority("TESTDATA - my assignation - " + somethingUnique);
		patientIdentifier.setUniqueId("TESTDATA - hey, I'm unique - " + somethingUnique);
		patientIdentifier.setFirstName("TESTDATA - patient first name - " + somethingUnique);
		patientIdentifier.setLastName("TESTDATA - patient last name - " + somethingUnique);
		patientIdentifier.setSsn("TESTDATA - hey, patient ssn - " + somethingUnique);
		clinicalEvent.setPatientIdentifier(patientIdentifier);

		ProviderIdentifier providerIdentifier = new ProviderIdentifier();
		providerIdentifier.setFacilityCode("TESTDATA - my prov fac - " + somethingUnique);
		providerIdentifier.setStaffId("TESTDATA - hey, I'm on staff - " + somethingUnique);
		clinicalEvent.setProviderIdentifier(providerIdentifier);

		return clinicalEvent;
	}

	public static ClinicalEvent createClinicalEvent(Date time) {
		String somethingUnique = "" + time.getTime();
		ClinicalEvent clinicalEvent = createClinicalEventMinimal(time);

		Note note = new Note();
		NoteTitle title = new NoteTitle();
		title.setIen("TESTDATA - my ien - " + somethingUnique);
		title.setName("TESTDATA - My title has a first name, it's O-s-c-a-r - " + somethingUnique);
		title.setConsult(true);
		title.setPrf(true);
		note.setTitle(title);
		note.setBody("TESTDATA - Hey, it's my body - " + somethingUnique);
		note.setConsultIen("TESTDATA - Hey, it's my consult - " + somethingUnique);
		note.setSignedDate(null);
		clinicalEvent.setNote(note);
		clinicalEvent.setEncounterValid(true);
		clinicalEvent.setEncounterEdited(true);

		Encounter encounter = new Encounter();
		encounter.setTimestamp(time);
		encounter.setTime("TESTDATA - my time for resume - " + somethingUnique);
		encounter.setDate("TESTDATA - my date for resume - " + somethingUnique);
		encounter.setLocationId("TESTDATA - my location abcdef - " + somethingUnique);
		encounter.setLocationTitle("TESTDATA - my location Title - " + somethingUnique);
		encounter.setLinkType("Appointment");
		encounter.setCurrentAdmission(false);
		encounter.setHistorical(false);
		encounter.setServiceCategory("TESTDATA - my service category - " + somethingUnique);
		encounter.setAppointmentType("TESTDATA - my appointment type - " + somethingUnique);
		encounter.setEncounterType("TESTDATA - my encounter type - " + somethingUnique);

		EncounterDiagnosisICDCodes diagnosisCodes = new EncounterDiagnosisICDCodes();
		for (int i=0; i<10; i++) {
			EncounterDiagnosisICDCode diagnosisCode = new EncounterDiagnosisICDCode();
			diagnosisCode.setAddToProblemList(i%2 == 0);
			diagnosisCode.setPrimaryDiagnosis(i%2 == 1);
			diagnosisCode.setFromProblemList(i%2 == 0);
			diagnosisCode.setCode("TESTDATA - DiagnosisCode (" + i + ") - " + somethingUnique);
			diagnosisCode.setCodeType("TESTDATA - DiagnosisCodeType (" + i + ") - " + somethingUnique);
			diagnosisCode.setDescription("TESTDATA - DiagnosisCodeDescription (" + i + ") - " + somethingUnique);
			diagnosisCode.setIen("TESTDATA - DiagnosisCodeIen (" + i + ") - " + somethingUnique);
			diagnosisCode.setShortDescription("TESTDATA - DiagnosisCodeShortDescription (" + i + ") - " + somethingUnique);
			diagnosisCodes.add(diagnosisCode);
		}
		encounter.setDiagnosisCodes(diagnosisCodes);

		EncounterProcedureCPTCodes procedureCodes = new EncounterProcedureCPTCodes();
		for (int i=0; i<10; i++) {
			EncounterProcedureCPTCode procedureCode = new EncounterProcedureCPTCode();
			procedureCode.setCode("TESTDATA - ProcedureCode (" + i + ") - " + somethingUnique);
			procedureCode.setCodeType("TESTDATA - ProcedureCodeType (" + i + ") - " + somethingUnique);
			procedureCode.setDescription("TESTDATA - ProcedureCodeDescription (" + i + ") - " + somethingUnique);
			procedureCode.setIen("TESTDATA - ProcedureCodeIen (" + i + ") - " + somethingUnique);
			procedureCode.setShortDescription("TESTDATA - ProcedureCodeShortDescription (" + i + ") - " + somethingUnique);

			Modifiers modifiers = new Modifiers();
			for (int k=0; k<10; k++) {
				Modifier modifier = new Modifier();
				modifier.setCode("TESTDATA - ProcedureCodeModifier code (" + i + ") - " + somethingUnique);
				modifier.setIen("TESTDATA - ProcedureCodeModifier ien (" + i + ") - " + somethingUnique);
				modifier.setName("TESTDATA - ProcedureCodeModifier name (" + i + ") - " + somethingUnique);
				modifiers.add(modifier);
			}
			procedureCode.setModifiers(modifiers);
			procedureCodes.add(procedureCode);
		}
		encounter.setProcedureCodes(procedureCodes);

		VisitTypes visitTypes = new VisitTypes();
		for (int m=0; m<10; m++) {
			VisitType visitType = new VisitType();

			EncounterProcedureCPTCodes visitSectionCodes = new EncounterProcedureCPTCodes();
			for (int i=0; i<10; i++) {
				EncounterProcedureCPTCode visitSectionCode = new EncounterProcedureCPTCode();
				visitSectionCode.setCode("TESTDATA - VisitSectionCode (" + i + ") - " + somethingUnique);
				visitSectionCode.setCodeType("TESTDATA - VisitSectionCodeType (" + i + ") - " + somethingUnique);
				visitSectionCode.setDescription("TESTDATA - VisitSectionCodeDescription (" + i + ") - " + somethingUnique);
				visitSectionCode.setIen("TESTDATA - VisitSectionCodeIen (" + i + ") - " + somethingUnique);
				visitSectionCode.setShortDescription("TESTDATA - VisitSectionCodeShortDescription (" + i + ") - " + somethingUnique);
				visitSectionCode.setSelected(i%2 == 0);

				Modifiers modifiers = new Modifiers();
				for (int k=0; k<10; k++) {
					Modifier modifier = new Modifier();
					modifier.setCode("TESTDATA - VisitSectionCodeModifier code (" + i + ") - " + somethingUnique);
					modifier.setIen("TESTDATA - VisitSectionCodeModifier ien (" + i + ") - " + somethingUnique);
					modifier.setName("TESTDATA - VisitSectionCodeModifier name (" + i + ") - " + somethingUnique);
					modifier.setSelected(k%2 == 0);
					modifiers.add(modifier);
				}
				visitSectionCode.setModifiers(modifiers);
				visitSectionCodes.add(visitSectionCode);
			}
			visitType.setSections(visitSectionCodes);
			visitType.setName("TESTDATA - VisitType name (" + m + ") - " + somethingUnique);
			visitType.setSelected(m%2 == 0);
			visitTypes.add(visitType);
		}
		encounter.setVisitTypes(visitTypes);

		VisitRelatedTos visitRelatedTos = new VisitRelatedTos();
		for (int i=0; i<10; i++) {
			VisitRelatedTo visitRelatedTo = new VisitRelatedTo();
			visitRelatedTo.setChecked(i%2 == 0);
			visitRelatedTo.setTitle("TESTDATA - My Visit (" + i + ") - " + somethingUnique);
			visitRelatedTo.setCode("TESTDATA - Code (" + i + ") - " + somethingUnique);
			visitRelatedTo.setEnabled(i%2 == 0);
			visitRelatedTos.add(visitRelatedTo);
		}
		encounter.setVisitRelatedTos(visitRelatedTos);

		EncounterProviders providers = new EncounterProviders();
		for (int i=0; i<10; i++) {
			FacilityProvider facilityProvider = new FacilityProvider();
			facilityProvider.setFacilityCode("TESTDATA - my enc prov fac - " + somethingUnique);
			facilityProvider.setProviderIen("TESTDATA - hey, I'm on staff too - " + somethingUnique);
			facilityProvider.setProviderName("TESTDATA - My Name Is John Jacob Jingleheimer Smith - " + somethingUnique);

			EncounterProvider encProvider = new EncounterProvider();
			encProvider.setPrimary(i%2 == 0);
			encProvider.setSelected(i%2 == 1);
			encProvider.setProvider(facilityProvider);

			providers.add(encProvider);
		}
		encounter.setProviders(providers);


		clinicalEvent.setEncounter(encounter);
		return clinicalEvent;
	}

	public static void assertClinicalEvent(ClinicalEvent clinicalEvent, Date time) {
		Assert.assertNotNull(clinicalEvent);
		Assert.assertTrue(NullChecker.isNotNullish(clinicalEvent.getId()));
		//Assert.assertTrue(NullChecker.isNotNullish(clinicalEvent.getLastModifiedDate())); //Doesn't happen in translator but in ClinicalEventDataService.

		String somethingUnique = "" + time.getTime();
		Assert.assertEquals("TESTDATA - my fac - " + somethingUnique, clinicalEvent.getFacilityCode());
		Assert.assertEquals(ClinicalEventState.DRAFT, clinicalEvent.getClinicalEventState());


		Assert.assertEquals(true, clinicalEvent.isEncounterValid());
		Assert.assertEquals(true, clinicalEvent.isEncounterEdited());

		Assert.assertNotNull(clinicalEvent.getPatientIdentifier());
		Assert.assertEquals("TESTDATA - my assignation - " + somethingUnique, clinicalEvent.getPatientIdentifier().getAssigningAuthority());
		Assert.assertEquals("TESTDATA - hey, I'm unique - " + somethingUnique, clinicalEvent.getPatientIdentifier().getUniqueId());

		Assert.assertEquals("TESTDATA - patient first name - " + somethingUnique, clinicalEvent.getPatientIdentifier().getFirstName());
		Assert.assertEquals("TESTDATA - patient last name - " + somethingUnique, clinicalEvent.getPatientIdentifier().getLastName());
		Assert.assertEquals("TESTDATA - hey, patient ssn - " + somethingUnique, clinicalEvent.getPatientIdentifier().getSsn());

		Assert.assertNotNull(clinicalEvent.getProviderIdentifier());
		Assert.assertEquals("TESTDATA - my prov fac - " + somethingUnique, clinicalEvent.getProviderIdentifier().getFacilityCode());
		Assert.assertEquals("TESTDATA - hey, I'm on staff - " + somethingUnique, clinicalEvent.getProviderIdentifier().getStaffId());

		Assert.assertNotNull(clinicalEvent.getNote());
		Assert.assertEquals("TESTDATA - my ien - " + somethingUnique, clinicalEvent.getNote().getTitle().getIen());
		Assert.assertTrue(clinicalEvent.getNote().getTitle().isConsult());
		Assert.assertTrue(clinicalEvent.getNote().getTitle().isPrf());
		Assert.assertEquals("TESTDATA - My title has a first name, it's O-s-c-a-r - " + somethingUnique, clinicalEvent.getNote().getTitle().getName());
		Assert.assertEquals("TESTDATA - Hey, it's my body - " + somethingUnique, clinicalEvent.getNote().getBody());
		Assert.assertEquals("TESTDATA - Hey, it's my consult - " + somethingUnique, clinicalEvent.getNote().getConsultIen());

		Assert.assertNotNull(clinicalEvent.getEncounter());
		Assert.assertEquals(time, clinicalEvent.getEncounter().getTimestamp());
		Assert.assertEquals("TESTDATA - my time for resume - " + somethingUnique, clinicalEvent.getEncounter().getTime());
		Assert.assertEquals("TESTDATA - my date for resume - " + somethingUnique, clinicalEvent.getEncounter().getDate());
		Assert.assertEquals("TESTDATA - my location abcdef - " + somethingUnique, clinicalEvent.getEncounter().getLocationId());
		Assert.assertEquals("TESTDATA - my location Title - " + somethingUnique, clinicalEvent.getEncounter().getLocationTitle());
		Assert.assertEquals("Appointment", clinicalEvent.getEncounter().getLinkType());
		Assert.assertEquals(false, clinicalEvent.getEncounter().isCurrentAdmission());
		Assert.assertEquals(false, clinicalEvent.getEncounter().isHistorical());
		Assert.assertEquals("TESTDATA - my service category - " + somethingUnique, clinicalEvent.getEncounter().getServiceCategory());
		Assert.assertEquals("TESTDATA - my appointment type - " + somethingUnique, clinicalEvent.getEncounter().getAppointmentType());
		Assert.assertEquals("TESTDATA - my encounter type - " + somethingUnique, clinicalEvent.getEncounter().getEncounterType());

		Assert.assertNotNull(clinicalEvent.getEncounter().getDiagnosisCodes());
		Assert.assertEquals(10, clinicalEvent.getEncounter().getDiagnosisCodes().size());
		for (int i=0; i<clinicalEvent.getEncounter().getDiagnosisCodes().size(); i++) {
			Assert.assertEquals("TESTDATA - DiagnosisCode (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getDiagnosisCodes().get(i).getCode());
			Assert.assertEquals("TESTDATA - DiagnosisCodeType (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getDiagnosisCodes().get(i).getCodeType());
			Assert.assertEquals("TESTDATA - DiagnosisCodeDescription (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getDiagnosisCodes().get(i).getDescription());
			Assert.assertEquals("TESTDATA - DiagnosisCodeIen (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getDiagnosisCodes().get(i).getIen());
			Assert.assertEquals("TESTDATA - DiagnosisCodeShortDescription (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getDiagnosisCodes().get(i).getShortDescription());
			Assert.assertEquals(new Boolean(i%2 == 0), clinicalEvent.getEncounter().getDiagnosisCodes().get(i).isAddToProblemList());
			Assert.assertEquals(new Boolean(i%2 == 1), clinicalEvent.getEncounter().getDiagnosisCodes().get(i).isPrimaryDiagnosis());
			Assert.assertEquals(new Boolean(i%2 == 0), clinicalEvent.getEncounter().getDiagnosisCodes().get(i).isFromProblemList());
		}

		Assert.assertNotNull(clinicalEvent.getEncounter().getProcedureCodes());
		Assert.assertEquals(10, clinicalEvent.getEncounter().getProcedureCodes().size());
		for (int i=0; i<clinicalEvent.getEncounter().getProcedureCodes().size(); i++) {
			Assert.assertEquals("TESTDATA - ProcedureCode (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getProcedureCodes().get(i).getCode());
			Assert.assertEquals("TESTDATA - ProcedureCodeType (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getProcedureCodes().get(i).getCodeType());
			Assert.assertEquals("TESTDATA - ProcedureCodeDescription (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getProcedureCodes().get(i).getDescription());
			Assert.assertEquals("TESTDATA - ProcedureCodeIen (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getProcedureCodes().get(i).getIen());
			Assert.assertEquals("TESTDATA - ProcedureCodeShortDescription (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getProcedureCodes().get(i).getShortDescription());

			Assert.assertNotNull(clinicalEvent.getEncounter().getProcedureCodes().get(i).getModifiers());
			Assert.assertEquals(10, clinicalEvent.getEncounter().getProcedureCodes().get(i).getModifiers().size());
			for (int k=0; k<clinicalEvent.getEncounter().getProcedureCodes().get(i).getModifiers().size(); k++) {
				Assert.assertEquals("TESTDATA - ProcedureCodeModifier code (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getProcedureCodes().get(i).getModifiers().get(k).getCode());
				Assert.assertEquals("TESTDATA - ProcedureCodeModifier ien (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getProcedureCodes().get(i).getModifiers().get(k).getIen());
				Assert.assertEquals("TESTDATA - ProcedureCodeModifier name (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getProcedureCodes().get(i).getModifiers().get(k).getName());
			}
		}

		Assert.assertNotNull(clinicalEvent.getEncounter().getVisitTypes());
		Assert.assertEquals(10, clinicalEvent.getEncounter().getVisitTypes().size());
		for (int m=0; m<clinicalEvent.getEncounter().getVisitTypes().size(); m++) {
			Assert.assertNotNull(clinicalEvent.getEncounter().getVisitTypes().get(m).getSections());
			Assert.assertEquals(10, clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().size());
			Assert.assertEquals("TESTDATA - VisitType name (" + m + ") - " + somethingUnique, clinicalEvent.getEncounter().getVisitTypes().get(m).getName());
			Assert.assertEquals(m%2 == 0, clinicalEvent.getEncounter().getVisitTypes().get(m).isSelected());

			for (int i=0; i<clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().size(); i++) {
				Assert.assertEquals("TESTDATA - VisitSectionCode (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).getCode());
				Assert.assertEquals("TESTDATA - VisitSectionCodeType (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).getCodeType());
				Assert.assertEquals("TESTDATA - VisitSectionCodeDescription (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).getDescription());
				Assert.assertEquals("TESTDATA - VisitSectionCodeIen (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).getIen());
				Assert.assertEquals(i%2 == 0, clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).isSelected());
				Assert.assertEquals("TESTDATA - VisitSectionCodeShortDescription (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).getShortDescription());

				Assert.assertNotNull(clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).getModifiers());
				Assert.assertEquals(10, clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).getModifiers().size());
				for (int k=0; k<clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).getModifiers().size(); k++) {
					Assert.assertEquals("TESTDATA - VisitSectionCodeModifier code (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).getModifiers().get(k).getCode());
					Assert.assertEquals("TESTDATA - VisitSectionCodeModifier ien (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).getModifiers().get(k).getIen());
					Assert.assertEquals("TESTDATA - VisitSectionCodeModifier name (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).getModifiers().get(k).getName());
					Assert.assertEquals(k%2 == 0, clinicalEvent.getEncounter().getVisitTypes().get(m).getSections().get(i).getModifiers().get(k).isSelected());
				}
			}
		}

		Assert.assertNotNull(clinicalEvent.getEncounter().getVisitRelatedTos());
		Assert.assertEquals(10, clinicalEvent.getEncounter().getVisitRelatedTos().size());
		for (int i=0; i<clinicalEvent.getEncounter().getVisitRelatedTos().size(); i++) {
			Assert.assertEquals(new Boolean(i%2 == 0), clinicalEvent.getEncounter().getVisitRelatedTos().get(i).getChecked());
			Assert.assertEquals(new Boolean(i%2 == 0), clinicalEvent.getEncounter().getVisitRelatedTos().get(i).getEnabled());
			Assert.assertEquals("TESTDATA - My Visit (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getVisitRelatedTos().get(i).getTitle());
			Assert.assertEquals("TESTDATA - Code (" + i + ") - " + somethingUnique, clinicalEvent.getEncounter().getVisitRelatedTos().get(i).getCode());
		}

		Assert.assertNotNull(clinicalEvent.getEncounter().getProviders());
		Assert.assertEquals(10, clinicalEvent.getEncounter().getProviders().size());
		for (int i=0; i<clinicalEvent.getEncounter().getProviders().size(); i++) {
			Assert.assertEquals(i%2 == 0, clinicalEvent.getEncounter().getProviders().get(i).isPrimary());
			Assert.assertEquals(i%2 == 1, clinicalEvent.getEncounter().getProviders().get(i).isSelected());
			Assert.assertEquals("TESTDATA - my enc prov fac - " + somethingUnique, clinicalEvent.getEncounter().getProviders().get(i).getProvider().getFacilityCode());
			Assert.assertEquals("TESTDATA - hey, I'm on staff too - " + somethingUnique, clinicalEvent.getEncounter().getProviders().get(i).getProvider().getProviderIen());
			Assert.assertEquals("TESTDATA - My Name Is John Jacob Jingleheimer Smith - " + somethingUnique, clinicalEvent.getEncounter().getProviders().get(i).getProvider().getProviderName());
		}
	}
}
