package com.agilex.healthcare.patientsearch.translator;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.Date;

import junit.framework.Assert;

import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.domain.DOBDate;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.RecentlySelectedPatients;
import com.agilex.healthcare.mobilehealthplatform.domain.SelectedPatient;
import com.agilex.healthcare.mobilehealthplatform.utils.StringUtil;
import com.agilex.healthcare.utility.DateHelper;
import com.mongodb.BasicDBList;
import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;

public class PatientSearchTranslatorTest {
	private final String SEARCHED_TIME_KEY = "searchedTime";
	
	private final String PROVIDER_KEY = "provider";
	private final String FACILITY_CODE_KEY = "facilityCode";
	private final String STAFF_ID_KEY = "staffId";
	private final String INPATIENT_KEY = "inpatient";
	private final String PATIENTS_KEY = "patients";
	private final String PATIENT_KEY = "patient";
	private final String PATIENT_ID_KEY = "patientId";
	private final String ASSIGNING_AUTHORITY_KEY = "assigningAuthority";
	private final String FIRST_NAME_KEY = "firstName";
	private final String MIDDLE_NAME_KEY = "middleName";
	private final String LAST_NAME_KEY = "lastName";
	private final String SSN_KEY = "ssn";
	private final String DATE_OF_BIRTH_KEY = "dateOfBirth";
	private final String GENDER_KEY = "gender";
	
	@Test
	public void translateSearchedPatientToDbObject() {
		String facilityCode = "500";
		String staffId = "staff001";
		DOBDate dob = new DOBDate(null, "MAY 05, 1900");
		Date now = new Date();
		Patient patient = new Patient();
		patient.setPatientIdentifier(new PatientIdentifier("EDIPI", "D123401"));
		patient.setDateOfBirth(dob);
		patient.setFirstName("zztest");
		patient.setLastName("patient01");
		patient.setSsn("999999901");
		patient.setGender("MALE");
		patient.setInpatient(true);
		
		SelectedPatient searchedPatient = new SelectedPatient();
		searchedPatient.setSearchedTime(now);
		searchedPatient.setPatient(patient);
		
		RecentlySelectedPatients recentlySelectedPatients = new RecentlySelectedPatients();
		recentlySelectedPatients.add(searchedPatient);
		PatientSearchTranslator translator = new PatientSearchTranslator();
		
		DBObject dbObject = translator.translate(facilityCode, staffId, recentlySelectedPatients);
		
		assertEquals(facilityCode, StringUtil.objectToString(((DBObject)dbObject.get(PROVIDER_KEY)).get(FACILITY_CODE_KEY)));
		assertEquals(staffId, StringUtil.objectToString(((DBObject)dbObject.get(PROVIDER_KEY)).get(STAFF_ID_KEY)));
		
		
		BasicDBList poPats = (BasicDBList)dbObject.get(PATIENTS_KEY);
		Assert.assertTrue(poPats.size() > 0);
		
		for (int i=0; i<poPats.size(); i++) {
			DBObject dbPatObject = (DBObject)poPats.get(i);
			assertEquals(searchedPatient.getSearchedTime(), dbPatObject.get(SEARCHED_TIME_KEY));
			assertEquals(searchedPatient.getPatient().getPatientIdentifier().getAssigningAuthority(), StringUtil.objectToString(((DBObject)dbPatObject.get(PATIENT_KEY)).get(ASSIGNING_AUTHORITY_KEY)));
			assertEquals(searchedPatient.getPatient().getPatientIdentifier().getUniqueId(), StringUtil.objectToString(((DBObject)dbPatObject.get(PATIENT_KEY)).get(PATIENT_ID_KEY)));
			assertEquals(searchedPatient.getPatient().getFirstName(), StringUtil.objectToString(((DBObject)dbPatObject.get(PATIENT_KEY)).get(FIRST_NAME_KEY)));
			assertEquals(searchedPatient.getPatient().getMiddleName(), StringUtil.objectToString(((DBObject)dbPatObject.get(PATIENT_KEY)).get(MIDDLE_NAME_KEY)));
			assertEquals(searchedPatient.getPatient().getLastName(), StringUtil.objectToString(((DBObject)dbPatObject.get(PATIENT_KEY)).get(LAST_NAME_KEY)));
			assertEquals(searchedPatient.getPatient().getSsn(), StringUtil.objectToString(((DBObject)dbPatObject.get(PATIENT_KEY)).get(SSN_KEY)));
			assertEquals(searchedPatient.getPatient().getDateOfBirth().toString(), StringUtil.objectToString(((DBObject)dbPatObject.get(PATIENT_KEY)).get(DATE_OF_BIRTH_KEY)));
			assertEquals(searchedPatient.getPatient().getGender(), StringUtil.objectToString(((DBObject)dbPatObject.get(PATIENT_KEY)).get(GENDER_KEY)));
			assertEquals(searchedPatient.getPatient().isInpatient(), StringUtil.objectToBoolean(((DBObject)dbPatObject.get(PATIENT_KEY)).get(INPATIENT_KEY)));
		}
	}
	
	@Test
	public void translateDbObjectToSearchedPatient() {
		String facilityCode = "500";
		String staffId = "staff046";
		String assigningAuthority = "EDIPI";
		String patientId = "D123401";
		String dob = "MAY 05, 1900";
		DOBDate dobDate = new DOBDate(null, dob);
		String age = String.valueOf(DateHelper.calculateDeltaInYears(dobDate.getDate(), new Date()));
		Date now = new Date();
		String firstName = "zztest";
		String middleName = null;
		String lastName = "patient01";
		String ssn = "999999901";
		String gender = "MALE";
		boolean isInpatient = true;
		
		
		BasicDBList poPats = new BasicDBList();
		DBObject po = new BasicDBObject();
		
		DBObject provider = new BasicDBObject();
		provider.put(FACILITY_CODE_KEY, facilityCode);
		provider.put(STAFF_ID_KEY, staffId);
		po.put(PROVIDER_KEY, provider);
		
		
		DBObject patientAndTime = new BasicDBObject();
		patientAndTime.put(SEARCHED_TIME_KEY, now);
		
		DBObject patient = new BasicDBObject();
		patient.put(ASSIGNING_AUTHORITY_KEY, assigningAuthority);
		patient.put(PATIENT_ID_KEY, patientId);
		patient.put(FIRST_NAME_KEY, firstName);
		patient.put(MIDDLE_NAME_KEY, middleName);
		patient.put(LAST_NAME_KEY, lastName);
		patient.put(SSN_KEY, ssn);
		patient.put(DATE_OF_BIRTH_KEY, dob);
		patient.put(GENDER_KEY, gender);
		patient.put(INPATIENT_KEY,  isInpatient);
		patientAndTime.put(PATIENT_KEY, patient);
		
		poPats.add(patientAndTime);
		po.put(PATIENTS_KEY, poPats);
		
		PatientSearchTranslator translator = new PatientSearchTranslator();
		RecentlySelectedPatients recentlySelectedPatients = translator.translate(po);
		SelectedPatient searchedPatient = recentlySelectedPatients.get(0);
		
		assertNotNull(searchedPatient);
		assertEquals(now, searchedPatient.getSearchedTime());
		assertEquals(assigningAuthority, searchedPatient.getPatient().getPatientIdentifier().getAssigningAuthority());
		assertEquals(patientId, searchedPatient.getPatient().getPatientIdentifier().getUniqueId());
		
		assertEquals(firstName, searchedPatient.getPatient().getFirstName());
		assertEquals(middleName, searchedPatient.getPatient().getMiddleName());
		assertEquals(lastName, searchedPatient.getPatient().getLastName());
		assertEquals(ssn, searchedPatient.getPatient().getSsn());
		assertEquals(dobDate, searchedPatient.getPatient().getDateOfBirth());
		assertEquals(age, searchedPatient.getPatient().getAge());
		assertEquals(gender, searchedPatient.getPatient().getGender());
		assertEquals(isInpatient, searchedPatient.getPatient().isInpatient());
	}
}
