package com.agilex.healthcare.mdws.datalayer;

import static org.junit.Assert.assertNotNull;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import com.agilex.healthcare.mdws.datalayer.util.AuthenticationUtil;
import com.agilex.healthcare.mdws.datalayer.util.SpringBasedIntegrationRepo;
import com.agilex.healthcare.mdws.domain.Code;
import com.agilex.healthcare.mdws.domain.Codes;
import com.agilex.healthcare.mdws.domain.ProblemLists;
import com.agilex.healthcare.mdws.domain.Providers;
import com.agilex.healthcare.mdws.domain.SvcConditionRelatedTos;
import com.agilex.healthcare.mdws.domain.UserMenuOptions;
import com.agilex.healthcare.mdws.domain.VisitTypes;

public class EncounterDataLayerMdwsTest extends SpringBasedIntegrationRepo{
	private static final String SITE_777 = "777";
	
    @Resource
    EncounterDataLayer encounterDataLayer;

    @BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("777", "cprs1234", "cprs4321$", "1", "FPS");
	}
    @AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser("FPS");
	}
    
//----------------------------------------------------------------------------------------------------------------------
	
	@Test
	public void getModifierCodes() {
		Codes result = encounterDataLayer.getModifierCodes(SITE_777, "00100");
		assertNotNull(result);
		Assert.assertTrue(result.size() > 0);
	}
	
	@Test
	public void getModifierCodesNothingReturned() {
		Codes result = encounterDataLayer.getModifierCodes(SITE_777, "~!@#"); //12
		assertNotNull(result);
		Assert.assertEquals(0, result.size());
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
	@Test
	public void getVisitTypeByClinic() {
		VisitTypes result = encounterDataLayer.getVisitTypeByClinic(SITE_777, "195", null);
		assertNotNull(result);
		Assert.assertTrue(result.size() > 0);
	}
	
	@Test
	public void getVisitTypeByClinicNothingReturned() {
		VisitTypes result = encounterDataLayer.getVisitTypeByClinic(SITE_777, "~!@#", null); //12
		assertNotNull(result);
		Assert.assertEquals(0, result.size());
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
	@Ignore("The VA updated VistA to only contain ICD-10-CM codes and our contract was to only return ICD-9-CM codes, so we are commenting out the tests for this as directed by Visu")
	@Test
	public void getDiagnosisSubCodes() {
		Codes result = encounterDataLayer.getDiagnosisSubCodes(SITE_777, "999", "");
		assertNotNull(result);
		Assert.assertTrue(result.size() > 0);
	}

	@Test
	public void getDiagnosisSubCodesNothingReturned() {
		Codes result = encounterDataLayer.getDiagnosisSubCodes(SITE_777, "~!@#", ""); //Q2039
		assertNotNull(result);
		Assert.assertEquals(0, result.size());
	}
	
	@Ignore("The VA updated VistA to only contain ICD-10-CM codes and our contract was to only return ICD-9-CM codes, so we are commenting out the tests for this as directed by Visu")
	@Test
	public void getDiagnosisSubCodesSpecificValueReturned() {
		Codes result = encounterDataLayer.getDiagnosisSubCodes(SITE_777, "nipp", "");
		assertNotNull(result);
		Assert.assertTrue(result.size() > 0);
		
		Assert.assertTrue(findCode("757.6", result));
	}
	
	private boolean findCode(String codeToFind, Codes result) {
		if (codeToFind == null || codeToFind.isEmpty())
			return false;
		
		boolean found = false;
		for (int i=0; i<result.size(); i++) {
			Code code = result.get(i);
			if (codeToFind.equals(code.getCode())) {
				found = true;
				break;
			}
		}
		return found;
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
	@Test
	public void getProceduresSubCodes() {
		Codes result = encounterDataLayer.getProceduresSubCodes(SITE_777, "756", "");
		assertNotNull(result);
		Assert.assertTrue(result.size() > 0);
	}

	@Test
	public void getProceduresSubCodesNothingReturned() {
		Codes result = encounterDataLayer.getProceduresSubCodes(SITE_777, "~!@#", ""); //abcdefghijk
		assertNotNull(result);
		Assert.assertEquals(0, result.size());
	}
	
	@Test
	public void getProceduresSubCodesSpecificValueReturned() {
		Codes result = encounterDataLayer.getProceduresSubCodes(SITE_777, "nipp", "");
		assertNotNull(result);
		Assert.assertTrue(result.size() > 0);
		
		Assert.assertTrue(findCode("19355", result));
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
	@Test
	public void getPatientProblemListDiagnoses() {
		ProblemLists result = encounterDataLayer.getPatientProblemListDiagnoses(SITE_777, "1", null); //1,3,8,16,17,19,25
		assertNotNull(result);
		Assert.assertTrue(result.size() > 0);
	}
	
	@Test
	public void getPatientProblemListDiagnosesNothingReturned() {
		ProblemLists result = encounterDataLayer.getPatientProblemListDiagnoses(SITE_777, "2", null);
		assertNotNull(result);
		Assert.assertEquals(0, result.size());
	}
	
	@Test
	public void getPatientProblemListDiagnosesInvalidPatient() {
		MdwsResponseException.UserMessages expected = MdwsResponseException.UserMessages.INVALID_RECORD_ID;
		try {
			encounterDataLayer.getPatientProblemListDiagnoses(SITE_777, "~!@#", null);
		}
		catch (MdwsResponseException e) {
			MdwsResponseException.UserMessages um = e.getUserMessage();
			if (!expected.equals(um))
				Assert.fail("The Exception thrown was not '" + expected.name() + "' but was '" + (um == null ? "<null>" : um.name()) + "'");
			return;
		}
		Assert.fail("Expected this exception to be thrown: " + expected.name());
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
	@Test
	public void getProvidersBySite() {
		Providers result = encounterDataLayer.getProvidersBySite(SITE_777);
		assertNotNull(result);
		Assert.assertTrue(result.size() > 0);
	}
	
//----------------------------------------------------------------------------------------------------------------------

	/**
	 * We will always have results returned - unless invalid data is passed in.  Currently, we are seeing 8 values
	 * always returned in SvcConditionRelatedTos
	 */
	@Test
	public void getSvcConnectedRelatedToConditions() {
		SvcConditionRelatedTos result = encounterDataLayer.getSvcConnectedRelatedToConditions(SITE_777, "2", "01/01/2013", "32");
		assertNotNull(result);
		Assert.assertTrue(result.size() > 0);
	}

//----------------------------------------------------------------------------------------------------------------------

	@Test
	public void getSvcConnectedRelatedToConditionsInvalidPatientIen() {
		MdwsResponseException.UserMessages expected = MdwsResponseException.UserMessages.INVALID_RECORD_ID;
		try {
			encounterDataLayer.getSvcConnectedRelatedToConditions(SITE_777, "~!@#", "01/01/2013", "32");
		}
		catch (MdwsResponseException e) {
			MdwsResponseException.UserMessages um = e.getUserMessage();
			if (!expected.equals(um))
				Assert.fail("The Exception thrown was not '" + expected.name() + "' but was '" + (um == null ? "<null>" : um.name()) + "'");
			return;
		}
		Assert.fail("Expected this exception to be thrown: " + expected.name());
	}
	
//----------------------------------------------------------------------------------------------------------------------

	@Test
	public void getSvcConnectedRelatedToConditionsInvalidDate() {
		MdwsResponseException.UserMessages expected = MdwsResponseException.UserMessages.INVALID_DATE;
		try {
			encounterDataLayer.getSvcConnectedRelatedToConditions(SITE_777, "2", "~!@#", "32");
		}
		catch (MdwsResponseException e) {
			MdwsResponseException.UserMessages um = e.getUserMessage();
			if (!expected.equals(um))
				Assert.fail("The Exception thrown was not '" + expected.name() + "' but was '" + (um == null ? "<null>" : um.name()) + "'");
			return;
		}
		Assert.fail("Expected this exception to be thrown: " + expected.name());
	}
	
//----------------------------------------------------------------------------------------------------------------------
	//TODO: MDWS refactoring
	@Ignore
	@Test
	public void getSvcConnectedRelatedToConditionsInvalidDateEmpty() {
		//the VA's RPC that's requiring the date to be zero if it is not passed in.
		SvcConditionRelatedTos result = encounterDataLayer.getSvcConnectedRelatedToConditions(SITE_777, "2", "", "32");
		assertNotNull(result);
		Assert.assertTrue(result.size() > 0);
	}

//----------------------------------------------------------------------------------------------------------------------
	//TODO: MDWS refactoring expects a fileman date now. 
	@Ignore
	@Test
	public void getSvcConnectedRelatedToConditionsInvalidLocationIen() {
		MdwsResponseException.UserMessages expected = MdwsResponseException.UserMessages.INVALID_RECORD_ID;
		//"01/01/2013"
		try {
			encounterDataLayer.getSvcConnectedRelatedToConditions(SITE_777, "2", "3140530.081055", "~!@#");
		}
		catch (MdwsResponseException e) {
			MdwsResponseException.UserMessages um = e.getUserMessage();
			if (!expected.equals(um))
				Assert.fail("The Exception thrown was not '" + expected.name() + "' but was '" + (um == null ? "<null>" : um.name()) + "'");
			return;
		}
		Assert.fail("Expected this exception to be thrown: " + expected.name());
	}
	
//----------------------------------------------------------------------------------------------------------------------

	@Test
	public void getUserMenuOptions() {
		UserMenuOptions result = encounterDataLayer.getUserMenuOptions(SITE_777, "1");
		assertNotNull(result);
		Assert.assertTrue(result.size() > 0);
	}

//----------------------------------------------------------------------------------------------------------------------

	@Test
	public void getUserMenuOptionsNothingReturned() {
		UserMenuOptions result = encounterDataLayer.getUserMenuOptions(SITE_777, "2");
		assertNotNull(result);
		Assert.assertEquals(0, result.size());
	}

}
