package com.agilex.healthcare.mdws.datalayer;

import static org.junit.Assert.*;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mdws.datalayer.util.AuthenticationUtil;
import com.agilex.healthcare.mdws.datalayer.util.EncounterNoteUtil;
import com.agilex.healthcare.mdws.datalayer.util.SpringBasedIntegrationRepo;
import com.agilex.healthcare.mdws.domain.EncounterDiagnosisICDCodes;
import com.agilex.healthcare.mdws.domain.EncounterNote;
import com.agilex.healthcare.mdws.domain.EncounterProcedureCPTCodes;

public class EncounterNoteSign2000Test extends SpringBasedIntegrationRepo {
	private static final String SITE_777 = "777";
	private static final String SITE_777_USER = "cprs1234";
	private static final String SITE_777_PASSWORD = "cprs4321$";
	private static final String SITE_777_ESIG = "CPRS1234";
    @Resource
    TIUNotesDataLayer encounterDataLayer;
	
    @BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser(SITE_777, SITE_777_USER, SITE_777_PASSWORD, "1", "TIU");
	}
    @AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser("TIU");
	}
    
//----------------------------------------------------------------------------------------------------------------------
	
	@Test
	public void signEncounterNote() {
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		
		EncounterDiagnosisICDCodes diagnoses = encounterNote.getDiagnoses();
		populate11000PlusCharacterDiagnoses(diagnoses);
		
		EncounterProcedureCPTCodes cptCodes = encounterNote.getProcedures();
		populate16000PlusCharacterProcedures(cptCodes);
		
		encounterNote.setNoteBody(get2574CharacterString());
		
		//TODO: When the mdws side is fixed, delete this line and uncomment the next
		encounterNote.setHealthFactors("500122^DENIAL OF NEED FOR EDUCATION^H^TEST");
		//encounterNote.setHealthFactors("500122^DENIAL OF NEED FOR EDUCATION^H^" + get2574CharacterString());
		
		EncounterNote result = encounterDataLayer.signEncounterNote(encounterNote, SITE_777_ESIG);
		assertNotNull(result);
	}
	
	@Test
	public void signEncounterNoteIsFailingWithLargeHealthFactors() {
		//TODO: When the mdws side is fixed, this test can be removed.  For now it is here to let you know that this is happening.
		boolean receivedError = false;
		try {
			EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
			encounterNote.setHealthFactors("500122^DENIAL OF NEED FOR EDUCATION^H^" + get2574CharacterString());
			
			EncounterNote result = encounterDataLayer.signEncounterNote(encounterNote, SITE_777_ESIG);
			assertNotNull(result);
		}
		catch (Exception e) {
			receivedError = true;
		}
		if (receivedError == false)
			fail("Expected mdws to fail with large health factors, is mdws now fixed?");
	}
	
	private void populate16000PlusCharacterProcedures(EncounterProcedureCPTCodes cptCodes) {
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29530", "Strapping of Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27599", "Femur or Knee Surgery"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27598", "Disarticulation at Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29870", "Diagnostic Arthroscopy of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27415", "Osteochondral Allograft, Knee, open"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("73562", "Radiologic Examination, Knee; 3 Views"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27580", "Arthrodesis of Knee using any Technique"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27370", "Injection Procedure for Knee Arthrography"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27412", "Autologous Chondrocyte Implantation, Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27365", "Radical Resection of Tumor, Femur or Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01404", "Anesthesia for Disarticulation at the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("73560", "Radiologic Examination, Knee; 1 or 2 Views"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27403", "Arthrotomy with Meniscus Repair of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27440", "Arthroplasty of the Tibial Plateau of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01400", "Anesthesia for open Procedures on the Knee Joint"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27330", "Arthrotomy for Synovial Biopsy involving the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29873", "Arthroscopy, Knee, Surgical; with Lateral Release"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01380", "Anesthesia for closed Procedures of the Knee Joint"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29405", "Application of Short Leg Cast (below Knee to Toes"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27570", "Manipulation of Knee Joint Under General Anesthesia"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27427", "Extra-Articular Ligamentous Reconstruction of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27435", "Capsulotomy of the Knee for Posterior Capsular Release"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27550", "Closed Treatment of Knee Dislocation without Anesthesia"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("73564", "Radiologic Examination, Knee; Complete, 4 or more Views"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01382", "Anesthesia for Arthroscopic Procedures of the Knee Joint"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29884", "Surgical Arthroscopy of the Knee with Lysis of Adhesions"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27334", "Arthrotomy, with Synovectomy, Knee; Anterior or Posterior"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27552", "Closed Treatment of Knee Dislocation Requiring Anesthesia"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29875", "Surgical Arthroscopy of the Knee with Limited Synovectomy"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27323", "Superficial Biopsy of the Soft Tissue of Thigh or Knee area"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27428", "Open Intra-Articular Ligamentous Reconstruction of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27390", "Open Tenotomy involving a single Tendon Hamstring (Knee to Hip"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01382", "Anesthesia for Diagnostic Arthroscopic Procedures of Knee Joint"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27391", "Tenotomy, open, Hamstring, Knee to Hip; Multiple Tendons, 1 Leg"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27442", "Arthroplasty of the Femoral Condyles or Tibial Plateaus of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29871", "Surgical Arthroscopy of the Knee with Infection Lavage and Drainage"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01420", "Anesthesia for Cast Application, Removal or Repair involving the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27407", "Primary Repair of A Torn Cruciate Ligament and/or Capsule of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01402", "Anesthesia for open Procedures on Knee Joint; Total Knee Arthroplasty"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27405", "Primary Repair of A Torn Collateral Ligament and/or Capsule of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29867", "Arthroscopy, Knee, Surgical; Osteochondral Allograft (eg, Mosaicplasty"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29874", "Surgical Arthroscopy of the Knee with Removal of A Loose or Foreign Body"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27445", "Arthroplasty of the Knee involving A Hinged Prosthesis (eg Walldius Type"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29882", "Surgical Arthroscopy of the Knee with Meniscus Repair (Medial or Lateral"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27347", "Excision of Lesion of Meniscus or Capsule (eg Cyst, Ganglion) of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27498", "Decompression Fasciotomy of Multiple Compartments of the Thigh and/or Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29883", "Surgical Arthroscopy of the Knee with Meniscus Repair (Medial and Lateral"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01440", "Anesthesia for Procedures involving Arteries of the Knee and Popliteal area"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("73565", "Radiologic Examination of the Knees (Both Knees, Standing, Anteroposterior"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27486", "Revision of Total Knee Arthroplasty, with or without Allograft; 1 Component"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01430", "Anesthesia for Procedures involving the Veins of the Knee and Popliteal area"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27392", "Open Tenotomy involving Multiple Tendons, Hamstrings (Knee to Hip, Bilateral"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27372", "Removal of A Foreign Body involving the Deep Tissues of the Thigh or Knee area"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29425", "Application of Short Leg Cast (below Knee to Toes, Walking or Ambulatory Type"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27429", "Open Intra-Articular and Extra-Articular Ligamentous Reconstruction of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29877", "Surgical Arthroscopy of the Knee with Debridement/Shaving of Articular Cartilage"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27327", "Excision, Tumor, Soft Tissue of Thigh or Knee area, Subcutaneous; less than 3 cm"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27337", "Excision, Tumor, Soft Tissue of Thigh or Knee area, Subcutaneous; 3 cm or greater"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27324", "Deep Biopsy of the Soft Tissue (Subfascial or Intramuscular) of Thigh or Knee area"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27409", "Primary Repair of Torn Collateral and Cruciate Ligaments and/or Capsule of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27538", "Closed Treatment of Intercondylar Spine(s) and/or Tuberosity Fracture(s) of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27441", "Arthroplasty of the Tibial Plateau of the Knee with Debridement and Partial Synovectomy"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01432", "Anesthesia for Procedures involving Arteriovenous Fistula of the Knee and Popliteal area"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29886", "Surgical Arthroscopy of the Knee with Drilling for Intact Osteochondritis Dissecan Lesion"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29581", "Application of Multi-Layer Compression System; Leg (below Knee), including Ankle and Foot"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27310", "Arthrotomy of the Knee with Exploration, Drainage or Removal of Foreign Body (eg Infection"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27496", "Decompression Fasciotomy, Thigh and/or Knee, 1 Compartment (Flexor or Extensor or Adductor"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27301", "Incision and Drainage of Deep Abscess, Bursa, or Hematoma involving the Thigh or Knee Region"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01400", "Anesthesia for open or Surgical Arthroscopic Procedures on Knee Joint; not otherwise specified"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01402", "Anesthesia for open or Surgical Arthroscopic Procedures on Knee Joint; Total Knee Arthroplasty"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01404", "Anesthesia for open or Surgical Arthroscopic Procedures on Knee Joint; Disarticulation at Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29876", "Arthroscopy, Knee, Surgical; Synovectomy, major, 2 or more Compartments (eg, Medial or Lateral"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27331", "Arthrotomy involving the Knee, including Exploration, Biopsy, or Removal of Loose Foreign Bodies"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("73580", "Radiologic Examination of the Knee by Arthrography (Radiological Supervision and Interpretation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27416", "Osteochondral Autograft(s), Knee, open (eg, Mosaicplasty) (includes Harvesting of Autograft [S"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27335", "Anterior and Posterior Arthrotomy for Synovectomy involving the Knee Joint and the Popliteal area"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29885", "Surgical Arthroscopy of the Knee with Drilling for Osteochondritis Dissecans (with Bone Grafting"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27328", "Excision, Tumor, Soft Tissue of Thigh or Knee area, Subfascial (eg, Intramuscular); less than 5 cm"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27339", "Excision, Tumor, Soft Tissue of Thigh or Knee area, Subfascial (eg, Intramuscular); 5 cm or greater"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27446", "Arthroplasty of the Medial or Lateral Compartment of the Femoral Condyle and Tibial Plateau of the Knee"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27332", "Medial or Lateral Arthrotomy with Excision of Semilunar Cartilage (Meniscectomy) involving the Knee Joint"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27333", "Medial and Lateral Arthrotomy for Excision of Semilunar Cartilage (Meniscectomy) involving the Knee Joint"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27303", "Deep Incision with Opening of the Bone Cortex involving the Femur or Knee (eg Osteomyelitis or Bone Abscess"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27443", "Arthroplasty of the Femoral Condyles or Tibial Plateaus of the Knee with Debridement and Partial Synovectomy"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27487", "Revision of A Total Knee Arthroplasty, with/without Allograft, involving the Femoral or Entire Tibial Component"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29887", "Surgical Arthroscopy of the Knee with Drilling for Intact Osteochondritis Dissecan Lesion and Internal Fixation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("37722", "Ligation, Division, and Stripping, Long (Greater) Saphenous Veins from Saphenofemoral Junction to Knee or below"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27557", "Open Treatment of Knee Dislocation, includes Internal Fixation, when Performed; with Primary Ligamentous Repair"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27488", "Removal of Knee Prosthesis, including Total Knee Prosthesis, Methylmethacrylate, with/without Insertion of Spacer"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01320", "Anesthesia for Procedures involving the Nerves, Muscles, Tendons, Fascia and Bursae of the Knee and/or Popliteal area"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29868", "Arthroscopy, Knee, Surgical; Meniscal Transplantation (includes Arthrotomy for Meniscal Insertion), Medial or Lateral"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29866", "Arthroscopy, Knee, Surgical; Osteochondral Autograft(s) (eg, Mosaicplasty) (includes Harvesting of the Autograft [S"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01482", "Anesthesia for open Procedure on Bones of lower Leg, Ankle, and Foot, Radical Resection (including below Knee Amputation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27499", "Decompression Fasciotomy of Multiple Compartments of the Thigh and/or Knee with Debridement of Nonviable Muscle and/or Nerve"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27540", "Open Treatment of Intercondylar Spine(s) and/or Tuberosity Fracture(s) of the Knee, includes Internal Fixation, when Performed"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29879", "Arthroscopy, Knee, Surgical; Abrasion Arthroplasty (includes Chondroplasty where Necessary) or Multiple Drilling or Microfracture"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29851", "Arthroscopically Aided Treatment of Intercondylar Spine(s) and/or Tuberosity Fracture(s) of the Knee with Internal or External Fixation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29850", "Arthroscopically Aided Treatment of Intercondylar Spine(s) and/or Tuberosity Fracture(s) of the Knee without Internal or External Fixation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27497", "Decompression Fasciotomy, Thigh and/or Knee, 1 Compartment (Flexor or Extensor or Adductor); with Debridement of Nonviable Muscle and/or Nerve"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27556", "Open Treatment of Knee Dislocation, includes Internal Fixation, when Performed; without Primary Ligamentous Repair or Augmentation/Reconstruction"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27558", "Open Treatment of Knee Dislocation, includes Internal Fixation, when Performed; with Primary Ligamentous Repair, with Augmentation/Reconstruction"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27457", "Osteotomy, Proximal Tibia, including Fibular Excision or Osteotomy (includes Correction of Genu Varus [Bowleg] or Genu Valgus [Knock-Knee]); after Epiphyseal Closure"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27455", "Osteotomy, Proximal Tibia, including Fibular Excision or Osteotomy (includes Correction of Genu Varus [Bowleg] or Genu Valgus [Knock-Knee]); before Epiphyseal Closure"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("1006F", "Osteoarthritis Symptoms and Functional Status Assessed (may Include the use of A Standardized Scale or the Completion of an Assessment Questionnaire, such as the SF-36, Aaos Hip & Knee Questionnaire"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29881", "Arthroscopy, Knee, Surgical; with Meniscectomy (Medial or Lateral, including any Meniscal Shaving) including Debridement/Shaving of Articular Cartilage (Chondroplasty), same or Separate Compartment(s), when Performed"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29880", "Arthroscopy, Knee, Surgical; with Meniscectomy (Medial and Lateral, including any Meniscal Shaving) including Debridement/Shaving of Articular Cartilage (Chondroplasty), same or Separate Compartment(s), when Performed"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27425", "Lateral Retinacular Release"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27340", "Excision of the Prepatellar Bursa"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27350", "Patellectomy or Hemi-Patellectomy"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01212", "Anesthesia for Hip Disarticulation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27305", "Open Iliotibial Fasciotomy/Tenotomy"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27495", "Prophylactic Treatment of the Femur"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27590", "Amputation of the Thigh through Femur"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01232", "Anesthesia for Amputation of the Femur"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01272", "Anesthesia for Femoral Artery Ligation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01474", "Anesthesia for Gastrocnemius Recession"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01486", "Anesthesia for Total Ankle Replacement"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27380", "Primary Suture of Infrapatellar Tendon"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27566", "Open Treatment of Patellar Dislocation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27465", "Osteoplasty for Shortening of the Femur"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27393", "Lengthening of a single Hamstring Tendon"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27466", "Osteoplasty for Lengthening of the Femur"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27596", "Re-Amputation of the Thigh through Femur"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01274", "Anesthesia for Femoral Artery Embolectomy"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01502", "Anesthesia for Embolectomy of the lower Leg"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27438", "Arthroplasty of the Patella with Prosthesis"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01234", "Anesthesia for Radical Resection of the Femur"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27430", "Quadricepsplasty (eg Bennet or Thompson Type"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01442", "Anesthesia for Popliteal Thromboendarterectomy"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27437", "Arthroplasty of the Patella without Prosthesis"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01250", "Anesthesia for Procedures involving the upper Leg"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01472", "Anesthesia for Repair of Ruptured Achilles Tendon"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01522", "Anesthesia for Venous Thrombectomy of the lower Leg"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27395", "Bilateral Lengthening of Multiple Hamstring Tendons"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01210", "Anesthesia for open Procedures involving the Hip Joint"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01340", "Anesthesia for closed Procedures on lower 1/3 of Femur"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01202", "Anesthesia for Arthroscopic Procedures of the Hip Joint"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01200", "Anesthesia for closed Procedures involving the him Joint"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27385", "Primary Suture of Quadriceps or Hamstring Muscle Rupture"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01464", "Anesthesia for Arthroscopic Procedures of the Ankle Joint"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27418", "Anterior Tibial Tubercleplasty (eg Maquet Type Procedure"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27507", "Open Treatment of Femoral Shaft Fracture with Plate/Screws"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27520", "Closed Treatment of Patellar Fracture without Manipulation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01230", "Anesthesia for open Procedures involving upper 2/3 of Femur"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27355", "Excision or Curettage of Bone Cyst or Benign Tumor of Femur"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27560", "Closed Treatment of Patellar Dislocation without Anesthesia"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27502", "Closed Treatment of Femoral Shaft Fracture with Manipulation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01220", "Anesthesia for closed Procedures involving upper 2/3 of Femur"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01270", "Anesthesia for Procedures involving Arteries of the upper Leg"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27562", "Closed Treatment of Patellar Dislocation Requiring Anesthesia"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01260", "Anesthesia for Procedures involving the Veins of the upper Leg"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01520", "Anesthesia for Procedures involving the Veins of the lower Leg"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27500", "Closed Treatment of Femoral Shaft Fracture without Manipulation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27420", "Reconstruction of Dislocating Patella (eg Hauser Type Procedure"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01462", "Anesthesia for closed Procedures on the lower Leg, Ankle and Foot"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01500", "Anesthesia for Procedures involving the Arteries of the lower Leg"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27345", "Excision of A Synovial Cyst of the Popliteal Space (Baker's Cyst"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01484", "Anesthesia for Osteotomy or Osteoplasty of the Tibia and/or Fibula"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27424", "Reconstruction for Recurrent Dislocating Patella with Patellectomy"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27450", "Osteotomy of the Shaft or Supracondylar of the Femur with Fixation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("35152", "Direct Repair for Ruptured Aneurysm involving the Popliteal Artery"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("20610", "Arthrocentesis, Aspiration and/or Injection of major Joint or Bursa"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01490", "Anesthesia for Cast Application, Removal or Repair for the lower Leg"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27591", "Amputation of the Thigh through Femur by Immediate Fitting Technique"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27448", "Osteotomy of the Shaft or Supracondylar of the Femur without Fixation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27306", "Percutaneous Tenotomy involving a single Tendon (Hamstring or Adductor"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27307", "Percutaneous Tenotomy involving Multiple Tendons (Adductors or Hamstrings"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27356", "Excision or Curettage of Bone Cyst or Benign Tumor of Femur with Allograft"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27381", "Secondary Suture of Infrapatellar Tendon including Fascial or Tendon Graft"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27517", "Closed Treatment of Distal Femoral Epiphyseal Separation with Manipulation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01392", "Anesthesia for open Procedures on upper Ends of Tibia, Fibula, and/or Patella"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01480", "Anesthesia for open Procedures on the Bones of the lower Leg, Ankle, and Foot"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27516", "Closed Treatment of Distal Femoral Epiphyseal Separation without Manipulation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27592", "Open Amputation of the Thigh through Femur by Circular (Guillotine) Technique"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01390", "Anesthesia for closed Procedures on upper Ends of Tibia, Fibula, and/or Patella"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01444", "Anesthesia for Popliteal Excision and Graft or Repair for Occlusion or Aneurysm"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27477", "Epiphyseal Arrest of the Proximal Tibia and Fibula by Epiphysiodesis or Stapling"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27506", "Open Treatment of Femoral Shaft Fracture with Insertion of Intramedullary Implant"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27530", "Closed Treatment of A Tibial Fracture of the Proximal Plateau without Manipulation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27532", "Closed Treatment of A Tibial Fracture of the Proximal Plateau with Skeletal Traction"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27503", "Closed Treatment of Supracondylar or Transcondylar Femoral Fracture with Manipulation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27400", "Tendon or Muscle Transfer involving the Hamstrings to the Femur (eg Egger's Procedure"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27594", "Secondary Closure or Scar Revision Following the Amputation of the Thigh through Femur"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29888", "Arthroscopically Aided Anterior Cruciate Ligament Repair/Augmentation or Reconstruction"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27501", "Closed Treatment of Supracondylar or Transcondylar Femoral Fracture without Manipulation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("29889", "Arthroscopically Aided Posterior Cruciate Ligament Repair/Augmentation or Reconstruction"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27470", "Repair of Nonunion or Malunion of the Femur from Distal to the Head and Neck without Graft"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27386", "Secondary Suture of Quadriceps or Hamstring Muscle Rupture including Fascial or Tendon Graft"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27358", "Excision or Curettage of Bone Cyst or Benign Tumor of Femur with Internal Fixation add-on Code"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27510", "Closed Treatment of Femoral Fracture (Distal end, Medial or Lateral Condyle) with Manipulation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27508", "Closed Treatment of Femoral Fracture (Distal end, Medial or Lateral Condyle) without Manipulation"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("Osteotomy", "Osteotomy of the Proximal Tibia"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27468", "Osteoplasty for the Combined Lengthening and Shortening of the Femur with Femoral Segment Transfer"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("Osteotomy", "Osteotomy of the Proximal Tibia"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27485", "Hemiepiphyseal Arrest of the Distal Femur or Proximal Leg, Tibia or Fibula (eg Genu Varus or Valgus"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27524", "Open Treatment of Patellar Fracture with Internal Fixation and/or Patellectomy and Soft Tissue Repair"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27357", "Excision or Curettage of Bone Cyst or Benign Tumor of Femur; with Autograft (includes Obtaining Graft"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27479", "Epiphyseal Arrest of the Combined Distal Femur, Proximal Tibia and Fibula by Epiphysiodesis or Stapling"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27454", "Osteotomy, Multiple, of the Femoral Shaft (eg Sofield Type Procedure) with Realignment on Intramedullary Rod"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("01470", "Anesthesia for Procedures involving the Nerves, Muscles, Tendons and Fascia of the lower Leg, Ankle, and Foot"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27472", "Repair of Nonunion or Malunion of the Femur from Distal to the Head and Neck with Iliac or other Autogenous Bone Graft"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27509", "Percutaneous Skeletal Fixation of Femoral Fracture (Distal end, Medial or Lateral Condyle, or Supracondylar or Transcondylar"));
		cptCodes.add(EncounterNoteUtil.createEncounterProcedureCPTCode("27422", "Reconstruction of Dislocating Patella with Extensor Realignment and/or Muscle Advancement or Release (eg Campbell, Goldwaite Type Procedure"));
	}
	
	private void populate11000PlusCharacterDiagnoses(EncounterDiagnosisICDCodes diagnoses) {
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "388.44", "Recruitment"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.42", "Rectal pain"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "787.99", "Rectal mass"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.0", "Rectal polyp"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "947.3", "Burn of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "565.1", "Rectal fistula"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "566", "Rectal abscess"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.41", "Ulcer of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.1", "Rectal prolapse"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "728.84", "Diastasis recti"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "863.45", "Injury of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.3", "Rectal hemorrhage"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.49", "Rupture of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "566", "Rectal cellulitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "211.4", "Adenoma of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "523.20", "Gingival recession"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.49", "Disorder of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.2", "Stenosis of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "646.30", "Recurrent abortion"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "239.0", "Neoplasm of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "595.9", "Recurrent cystitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "619.1", "Rectovulval fistula"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.2", "Stricture of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "372.45", "Recurrent pterygium"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.49", "Granuloma of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "473.9", "Recurrent sinusitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "784.59", "Receptive dysphasia"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "619.1", "Rectovaginal fistula"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.30", "Recurrent depression"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "576.1", "Recurrent cholangitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.49", "Perforation of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.0", "Anal and rectal polyp"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "327.13", "Recurrent hypersomnia"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "937", "Foreign body in rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "528.9", "Recurrent mouth ulcers"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "364.02", "Recurrent iridocyclitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "V13.09", "H/O: recurrent cystitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "054.9", "Recurrent herpes simplex"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "736.5", "Acquired genu recurvatum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.10", "Recurrent manic episodes"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "494.0", "Recurrent bronchiectasis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "315.32", "Receptive language delay"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "464.4", "Recurrent allergic croup"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "154.1", "Adenocarcinoma of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "562.11", "Diverticulitis of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "562.10", "Diverticulosis of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "789.00", "Recurrent abdominal pain"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "555.1", "Crohn's disease of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.41", "Stercoral ulcer of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "461.9", "Recurrent acute sinusitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "211.4", "Villous adenoma of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "154.1", "Malignant tumor of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.30", "Recurrent major depression"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "364.77", "Recession of chamber angle"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "754.40", "Congenital genu recurvatum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "371.42", "Recurrent erosion of cornea"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "230.4", "Carcinoma in situ of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "463", "Recurrent acute tonsillitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.2", "Stenosis of rectum and anus"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "315.39", "Receptive language disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "473.9", "Chronic recurrent sinusitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "382.9", "Recurrent acute otitis media"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "577.0", "Acute recurrent pancreatitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "595.9", "Recurrent infective cystitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "154.1", "Malignant melanoma of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "595.89", "Recurrent bacterial cystitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.35", "Recurrent dislocation of hip"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "415.19", "Recurrent pulmonary embolism"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.34", "Recurrent dislocation of hand"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.3", "Hemorrhage of rectum and anus"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "041.9", "Recurrent bacterial infection"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.36", "Recurrent Dislocation of Knee"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.30", "Recurrent dislocation of joint"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "576.1", "Recurrent pyogenic cholangitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.11", "Recurrent manic episodes, mild"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.32", "Recurrent Dislocation of Elbow"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "562.11", "Diverticulitis of rectosigmoid"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "507.0", "Recurrent aspiration pneumonia"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "562.10", "Diverticulosis of rectosigmoid"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.31", "Mild recurrent major depression"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "361.05", "Recent total retinal detachment"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "937", "Foreign body in anus and rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "848.8", "Strain of rectus femoris muscle"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "307.21", "Recurrent transient tic disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "560.89", "Recurrent intestinal obstruction"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "054.10", "Recurrent genital herpes simplex"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "599.0", "Recurrent urinary tract infection"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "054.13", "Recurrent herpes simplex of penis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "646.30", "Antenatal Care: Recurrent Aborter"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.12", "Recurrent manic episodes, moderate"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "361.05", "Recent subtotal retinal detachment"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "459.0", "Secondary and recurrent hemorrhage"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "327.43", "Recurrent isolated sleep paralysis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "550.93", "Bilateral recurrent inguinal hernia"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.32", "Moderate recurrent major depression"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "309.0", "Recurrent brief depressive disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "556.3", "Chronic ulcerative rectosigmoiditis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.30", "Recurrent major depressive episodes"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "617.4", "Endometriosis of rectovaginal septum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "550.91", "Unilateral recurrent inguinal hernia"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "154.1", "Primary malignant neoplasm of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.36", "Recurrent subluxation of the patella"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "590.80", "Recurrent obstructive pyelonephritis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.2/E926.9", "Stricture of rectum due to radiation"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "197.5", "Secondary malignant neoplasm of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "154.0", "Carcinoma of the rectosigmoid junction"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.39", "Recurrent dislocation of multiple sites"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.35", "Recurrent major depression in remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "154.0", "Adenocarcinoma of rectosigmoid junction"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "550.01", "Recurrent inguinal hernia with gangrene"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "V10.06", "History of malignant neoplasm of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.31", "Recurrent dislocation of shoulder region"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "211.4", "Benign neoplasm of rectosigmoid junction"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "235.2", "Neoplasm of uncertain behavior of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "154.0", "Malignant tumor of rectosigmoid junction"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "277.4", "Benign recurrent intrahepatic cholestasis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.31", "Recurrent major depressive episodes, mild"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.37", "Recurrent dislocation of ankle AND/OR foot"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "724.02", "Stenosis of lateral recess of lumbar spine"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.16", "Recurrent manic episodes, in full remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "553.8", "Recurrent hernia of anterior abdominal wall"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "315.32", "Mixed receptive-expressive language disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "153.9", "Local recurrence of malignant tumor of colon"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.32", "Recurrent major depressive episodes, moderate"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "174.9", "Local recurrence of malignant tumor of breast"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "154.1", "Local recurrence of malignant tumor of rectum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.35", "Recurrent major depression in partial remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "552.01", "Unilateral recurrent irreducible femoral hernia"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.36", "Recurrent major depression in complete remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "158.8", "Primary malignant neoplasm of rectouterine pouch"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "305.00", "Alcohol intake above recommended sensible limits"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.14", "Recurrent manic episodes, severe, with psychosis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "550.11", "Unilateral recurrent irreducible inguinal hernia"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.38", "Recurrent dislocation of temporomandibular joint"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.38", "Recurrent subluxation of temporomandibular joint"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "664.20", "Perineal laceration involving rectovaginal septum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.00", "Bipolar I disorder, most recent episode hypomanic"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "550.03", "Bilateral recurrent inguinal hernia with gangrene"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "551.01", "Unilateral recurrent femoral hernia with gangrene"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.36", "Recurrent dislocation of the patellofemoral joint"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Bipolar II disorder, most recent episode hypomanic"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "550.01", "Unilateral recurrent inguinal hernia with gangrene"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "383.32", "Recurrent cholesteatoma of postmastoidectomy cavity"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "154.0", "Primary malignant neoplasm of rectosigmoid junction"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "361.01", "Partial recent retinal detachment with single defect"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "863.55", "Injury of rectum with open wound into abdominal cavity"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.36", "Recurrent major depressive episodes, in full remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "361.02", "Partial recent retinal detachment with multiple defects"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "361.04", "Partial recent retinal detachment with retinal dialysis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "V15.89", "History of recurrent dislocation of hip joint prosthesis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Bipolar II disorder, most recent episode major depressive"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.34", "Severe recurrent major depression with psychotic features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "863.45", "Injury of rectum without open wound into abdominal cavity"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.34", "Recurrent major depressive episodes, severe, with psychosis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.33", "Severe recurrent major depression without psychotic features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "552.8", "Recurrent hernia of anterior abdominal wall with obstruction"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "154.8", "Overlapping malignant neoplasm of rectum, anus and anal canal"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Mild bipolar II disorder, most recent episode major depressive"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Chronic bipolar II disorder, most recent episode major depressive"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "550.03", "Bilateral recurrent inguinal hernia with gangrene AND obstruction"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Moderate bipolar II disorder, most recent episode major depressive"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "550.13", "Bilateral recurrent inguinal hernia with obstruction but no gangrene"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "552.01", "Unilateral recurrent femoral hernia with obstruction but no gangrene"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "550.11", "Unilateral recurrent inguinal hernia with obstruction but no gangrene"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "958.2", "Secondary AND/OR recurrent hemorrhage as early complication of trauma"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "996.39", "Mechanical complication due to prosthetic reconstruction of vas deferens"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.34", "Severe recurrent major depression with psychotic features, mood-congruent"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "553.03", "Bilateral recurrent femoral hernia without obstruction AND without gangrene"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.34", "Severe recurrent major depression with psychotic features, mood-incongruent"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "553.01", "Unilateral recurrent femoral hernia without obstruction AND without gangrene"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Severe bipolar II disorder, most recent episode major depressive, in remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "648.40/296.89", "Bipolar II disorder, most recent episode major depressive with postpartum onset"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Bipolar II disorder, most recent episode major depressive with atypical features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Bipolar II disorder, most recent episode major depressive with catatonic features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Severe bipolar II disorder, most recent episode major depressive, in full remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Bipolar II disorder, most recent episode major depressive with melancholic features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "863.56", "Injury of multiple sites in colon AND/OR rectum with open wound into abdominal cavity"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.55", "Severe bipolar II disorder, most recent episode major depressive, in partial remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "863.46", "Injury of multiple sites in colon AND/OR rectum without open wound into abdominal cavity"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.7", "Severe bipolar II disorder, most recent episode major depressive with psychotic features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "997.1/414.00", "Recurrent coronary arteriosclerosis after percutaneous transluminal coronary angioplasty"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Severe bipolar II disorder, most recent episode major depressive without psychotic features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Severe bipolar II disorder, most recent episode major depressive with psychotic features, mood-congruent"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.89", "Severe bipolar II disorder, most recent episode major depressive with psychotic features, mood-incongruent"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "249.80", "Rare form of secondary diabetes mellitus, due to disorder other than malnutrition, protein deficiency, pancreatic disease, hormonal disease, drugs, receptor abnormality, OR genetic syndrome"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "578.1", "Melena"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "578.1", "Hematochezia"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "784.3", "Global aphasia"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "560.32", "Fecal impaction"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "087.9", "Relapsing fever"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "754.81", "Pectus excavatum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "708.8", "Chronic urticaria"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "270.0", "Cystinuria, type 1"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "996.00", "Retraction of stoma"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "378.54", "Abducens nerve palsy"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "757.39", "Epidermolysis bullosa"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "784.3", "Psycho-sensory aphasia"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "787.99", "Urgent desire for stool"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "378.54", "Abducens nerve disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.60", "Mixed bipolar I disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.40", "Manic bipolar I disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "646.70", "Cholestasis of pregnancy"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "237.71", "Neurofibromatosis, type 1"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "738.3", "Acquired pectus excavatum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "359.3", "Familial periodic paralysis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "754.81", "Congenital pectus excavatum"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "277.31", "Familial Mediterranean fever"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.50", "Depressed bipolar I disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "556.2", "Chronic ulcerative proctitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.61", "Mild mixed bipolar I disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.41", "Mild manic bipolar I disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "272.0", "Familial hypercholesterolemia"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "288.1", "Hyperimmunoglobulin E Syndrome"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "352.3", "Vagus nerve laryngeal paralysis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "569.49/909.2", "Perforating radiation proctitis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "664.20", "Third degree perineal laceration"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "754.40", "Hyperextension deformity of knee"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.62", "Moderate mixed bipolar I disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.51", "Mild depressed bipolar I disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "718.20", "Pathological dislocation of joint"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.42", "Moderate manic bipolar I disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "756.52", "Infantile malignant osteopetrosis"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "V15.81", "Noncompliance with dietary regimen"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.66", "Mixed bipolar I disorder in remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.46", "Manic bipolar I disorder in remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.52", "Moderate depressed bipolar I disorder"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "753.14", "Polycystic kidney disease, infantile type"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.56", "Depressed bipolar I disorder in remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.46", "Manic bipolar I disorder in full remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "618.04", "Female proctocele without uterine prolapse"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.66", "Mixed bipolar I disorder in full remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "696.1", "Pustular psoriasis of the palms AND/OR soles"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.65", "Mixed bipolar I disorder in partial remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "277.86", "Rhizomelic chondrodysplasia punctata syndrome"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.45", "Manic bipolar I disorder in partial remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.56", "Depressed bipolar I disorder in full remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "680.9", "Furunculosis of skin AND/OR subcutaneous tissue"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.55", "Depressed bipolar I disorder in partial remission"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "580.89", "Acute benign hemorrhagic glomerulonephritic syndrome"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.64", "Severe mixed bipolar I disorder with psychotic features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.44", "Severe manic bipolar I disorder with psychotic features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.43", "Severe manic bipolar I disorder without psychotic features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.63", "Severe mixed bipolar I disorder without psychotic features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.54", "Severe depressed bipolar I disorder with psychotic features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.53", "Severe depressed bipolar I disorder without psychotic features"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.64", "Severe mixed bipolar I disorder with psychotic features, mood-congruent"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.44", "Severe manic bipolar I disorder with psychotic features, mood-congruent"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.64", "Severe mixed bipolar I disorder with psychotic features, mood-incongruent"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.44", "Severe manic bipolar I disorder with psychotic features, mood-incongruent"));
		diagnoses.add(EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "296.54", "Severe depressed bipolar I disorder with psychotic features, mood-congruent"));
	}
	
	private String get2574CharacterString() {
		String str = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa" +
		"bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb" +
		"ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
		"ddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd" +
		"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee" +
		"fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff" +
		"ggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggg" +
		"hhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhh" +
		"iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii" +
		"jjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjj" +
		"kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk" +
		"lllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllll" +
		"mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm" +
		"nnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnn" +
		"ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo" +
		"ppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppp" +
		"qqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqq" +
		"rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr" +
		"sssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss" +
		"ttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttt" +
		"uuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuuu" +
		"vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv" +
		"wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww" +
		"xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" +
		"yyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyy" +
		"zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";
		return str;
	}
}
