package com.agilex.healthcare.mdws.datalayer;

import static org.junit.Assert.assertNotNull;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mdws.datalayer.util.AuthenticationUtil;
import com.agilex.healthcare.mdws.datalayer.util.EncounterNoteUtil;
import com.agilex.healthcare.mdws.datalayer.util.SpringBasedIntegrationRepo;
import com.agilex.healthcare.mdws.domain.BasicCode;
import com.agilex.healthcare.mdws.domain.EncounterDiagnosisICDCode;
import com.agilex.healthcare.mdws.domain.EncounterDiagnosisICDCodes;
import com.agilex.healthcare.mdws.domain.EncounterNote;
import com.agilex.healthcare.mdws.domain.EncounterProcedureCPTCode;
import com.agilex.healthcare.mdws.domain.EncounterProcedureCPTCodes;

public class EncounterNoteSignTest extends SpringBasedIntegrationRepo {
	private static final String SITE_777 = "777";
	private static final String SITE_777_USER = "cprs1234";
	private static final String SITE_777_PASSWORD = "cprs4321$";
	private static final String SITE_777_ESIG = "CPRS1234";
    @Resource
    TIUNotesDataLayer encounterDataLayer;
	
    @BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser(SITE_777, SITE_777_USER, SITE_777_PASSWORD, "1", "TIU");
	}
    @AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser("TIU");
	}
    
//----------------------------------------------------------------------------------------------------------------------
	
	@Test
	public void signEncounterNote() {
		EncounterNote result = encounterDataLayer.signEncounterNote(EncounterNoteUtil.createEncounterNote(), SITE_777_ESIG);
		assertNotNull(result);
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
    @Test
	public void failFacilityCode() {
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setFacilityCode("");
		failureConnecting("", SITE_777_USER, SITE_777_PASSWORD, "1", MdwsResponseException.UserMessages.INVALID_FACILITY_CODE, encounterNote); //The facilityCode was null or empty.
		encounterNote.setFacilityCode(null);
		failureConnecting(null, SITE_777_USER, SITE_777_PASSWORD, "1", MdwsResponseException.UserMessages.INVALID_FACILITY_CODE, encounterNote); //The facilityCode was null or empty.
		encounterNote.setFacilityCode("ABCDEFG");
		failureConnecting("ABCDEFG", SITE_777_USER, SITE_777_PASSWORD, "1", MdwsResponseException.UserMessages.CONNECTION_ERROR, encounterNote); //Site not in site table
	}
	
    @Test
	public void failPatientIen() {
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setPatientIen("");
		failure(MdwsResponseException.UserMessages.INVALID_PATIENT, encounterNote); //Missing local PID
		encounterNote.setPatientIen(null);
		failure(MdwsResponseException.UserMessages.INVALID_PATIENT, encounterNote); //patientIen cannot be null
		encounterNote.setPatientIen("ABCDEFG");
		failure(MdwsResponseException.UserMessages.INVALID_RECORD_ID, encounterNote); //Invalidly formed record ID: ABCDEFG
	}
	
    @Test
	public void failLocationIen() {
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setLocationIen("");
		failure(MdwsResponseException.UserMessages.INVALID_LOCATION_IEN, encounterNote); //Must supply locationIen
		encounterNote.setLocationIen(null);
		failure(MdwsResponseException.UserMessages.INVALID_LOCATION_IEN, encounterNote); //The location IEN provided was not valid.
		encounterNote.setLocationIen("ABCDEFG");
		failure(MdwsResponseException.UserMessages.INVALID_RECORD_ID, encounterNote); //Invalidly formed record ID: ABCDEFG
	}
	
    @Test
	public void failNoteTitleIen() {
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setNoteTitleIen("");
		failure(MdwsResponseException.UserMessages.INVALID_NOTE_TITLE_IEN, encounterNote); //Must supply noteTitleIen
		encounterNote.setNoteTitleIen(null);
		failure(MdwsResponseException.UserMessages.INVALID_NOTE_TITLE_IEN, encounterNote); //An Error Occurred with Note Title IEN
		encounterNote.setNoteTitleIen("ABCDEFG");
		failure(MdwsResponseException.UserMessages.INVALID_RECORD_ID, encounterNote); //Invalidly formed record ID: ABCDEFG
	}
	
    @Test
	public void failServiceCategory() {
		//'A' (ambulatory), 'H' (hospitalization), or 'E' (event historical) (defaults to 'A')
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setServiceCategory(null);
		failure(MdwsResponseException.UserMessages.INVALID_SERVICE_CATEGORY, encounterNote); //Input string for service category was not in a correct format.
		encounterNote.setServiceCategory("X");
		failure(MdwsResponseException.UserMessages.INVALID_SERVICE_CATEGORY, encounterNote); //serviceCategory must be 'A' (ambulatory), 'H' (hospitalization), or 'E' (event historical)
	}
	
    @Test
	public void failInpatient() {
		//'1' for true or '0' for false (defaults to '0').
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setInpatient(null);
		failure(MdwsResponseException.UserMessages.INVALID_INPATIENT, encounterNote); //Input string for inpatient was not in a correct format.
		encounterNote.setInpatient("ABCDEFG");
		failure(MdwsResponseException.UserMessages.INVALID_INPATIENT, encounterNote); //inpatient must be '1' for true or '0' for false
	}
	
    @Test
	public void passProceduresE() {
		//If your encounter has no CPT codes, you can pass a null for this parameter.
		//Note that this is only allowed if you are using service category "E" or inpatient value "1".
		//Otherwise, passing null for cptCodes will return an error.
		
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setProcedures(null);
		encounterNote.setServiceCategory("E");
		encounterNote.setInpatient("0");
		EncounterNote result = encounterDataLayer.signEncounterNote(encounterNote, SITE_777_ESIG);
		assertNotNull(result);
	}
    
    @Test
	public void passProcedures1() {
		//If your encounter has no CPT codes, you can pass a null for this parameter.
		//Note that this is only allowed if you are using service category "E" or inpatient value "1".
		//Otherwise, passing null for cptCodes will return an error.
		
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setProcedures(null);
		encounterNote.setServiceCategory("A");
		encounterNote.setInpatient("1");
		EncounterNote result = encounterDataLayer.signEncounterNote(encounterNote, SITE_777_ESIG);
		assertNotNull(result);
	}
    
    @Test
	public void passProceduresE1() {
		//If your encounter has no CPT codes, you can pass a null for this parameter.
		//Note that this is only allowed if you are using service category "E" or inpatient value "1".
		//Otherwise, passing null for cptCodes will return an error.
		
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setProcedures(null);
		encounterNote.setServiceCategory("E");
		encounterNote.setInpatient("1");
		EncounterNote result = encounterDataLayer.signEncounterNote(encounterNote, SITE_777_ESIG);
		assertNotNull(result);
	}
    
	@Test
	public void failProcedures() {
		//If your encounter has no CPT codes, you can pass a null for this parameter.
		//Note that this is only allowed if you are using service category "E" or inpatient value "1".
		//Otherwise, passing null for cptCodes will return an error.
		
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setProcedures(null);
		encounterNote.setServiceCategory("A");
		encounterNote.setInpatient("0");
		failure(MdwsResponseException.UserMessages.INVALID_PROCEDURES, encounterNote);
		
		EncounterProcedureCPTCodes codes = new EncounterProcedureCPTCodes();
		encounterNote.setProcedures(codes);
		failure(MdwsResponseException.UserMessages.INVALID_PROCEDURES, encounterNote);
		
		EncounterProcedureCPTCode code = new EncounterProcedureCPTCode();
		BasicCode basicCode = null;
		code.setBasicCode(basicCode);
		codes.add(code);
		failure(MdwsResponseException.UserMessages.MISSING_CPT_CODE, encounterNote);
		
		codes.remove(code);
		basicCode = new BasicCode();
		code.setBasicCode(basicCode);
		codes.add(code);
		failure(MdwsResponseException.UserMessages.MISSING_CPT_CODE, encounterNote);
	}
	
	@Test
	public void failDiagnoses() {
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		
		//The diagnoses can be null or can contain an empty array of codes.
		
		EncounterDiagnosisICDCodes codes = new EncounterDiagnosisICDCodes();
		encounterNote.setDiagnoses(codes);
		
		EncounterDiagnosisICDCode code = new EncounterDiagnosisICDCode();
		BasicCode basicCode = null;
		code.setBasicCode(basicCode);
		codes.add(code);
		failure(MdwsResponseException.UserMessages.MISSING_ICD_CODE, encounterNote);
		
		codes.remove(code);
		basicCode = new BasicCode();
		code.setBasicCode(basicCode);
		codes.add(code);
		failure(MdwsResponseException.UserMessages.MISSING_ICD_CODE, encounterNote);
	}
	
	@Test
	public void failVisitRelatedTos() {
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setVisitRelatedCodes(null);
		failure(MdwsResponseException.UserMessages.INVALID_VISIT_RELATED_TOS, encounterNote); //An Error Occurred with Visit Related Tos
		encounterNote.setVisitRelatedCodes("SAC");
		failure(MdwsResponseException.UserMessages.INVALID_VISIT_RELATED_TOS, encounterNote); //invalid visitRelatedTo entry: SAC
		encounterNote.setVisitRelatedCodes("XYZ");
		failure(MdwsResponseException.UserMessages.INVALID_VISIT_RELATED_TOS, encounterNote); //invalid visitRelatedTo entry: XYZ
	}
	
	@Test
	public void failNoteBody() {
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setNoteBody("");
		failure(MdwsResponseException.UserMessages.INVALID_NOTE_BODY, encounterNote); //Must supply note body
		encounterNote.setNoteBody(null);
		failure(MdwsResponseException.UserMessages.INVALID_NOTE_BODY, encounterNote); //Must supply note body
	}
	
	@Test
	public void failComment() {
		//You can pass in null or an empty string and it won't matter.
		Assert.assertTrue(true);
	}
	
	@Test
	public void failDate() {
		//should be formatted yyyymmdd.hhmmss
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setDate(null);
		failure(MdwsResponseException.UserMessages.INVALID_DATE, encounterNote); //Date cannot be null
		encounterNote.setDate("ABCDEFG");
		failure(MdwsResponseException.UserMessages.INVALID_INPUT, encounterNote); //Input string was not in a correct format.
	}

	@Test
	public void failProviders() {
		EncounterNote encounterNote = EncounterNoteUtil.createEncounterNote();
		encounterNote.setProviders("");
		failure(MdwsResponseException.UserMessages.INVALID_PROVIDER_IEN, encounterNote); //Must supply at least one providerIEN
		encounterNote.setProviders(null);
		failure(MdwsResponseException.UserMessages.INVALID_PROVIDER_IEN, encounterNote); //Must supply at least one providerIEN
		encounterNote.setProviders("5,119,541");
		failure(MdwsResponseException.UserMessages.NOT_AUTHORIZED, encounterNote); //User not authorized to sign note
	}
	
	private void failure(MdwsResponseException.UserMessages expected, EncounterNote encounterNote) {
		try {
			encounterDataLayer.signEncounterNote(encounterNote, SITE_777_ESIG);
		}
		catch (MdwsResponseException e) {
			MdwsResponseException.UserMessages um = e.getUserMessage();
			if (!expected.equals(um))
				Assert.fail("The Exception thrown was not '" + expected.name() + "' but was '" + (um == null ? "<null>" : um.name()) + "'");
			return;
		}
		Assert.fail("Expected this exception to be thrown: " + expected.name());
	}
	
	private void failureConnecting(String siteCode, String accessCode, String verifyCode, String uniqueId, MdwsResponseException.UserMessages expected, EncounterNote encounterNote) {
		try {
			AuthenticationUtil.loginUser(siteCode, accessCode, verifyCode, uniqueId, "TIU");
			encounterDataLayer.signEncounterNote(encounterNote, SITE_777_ESIG);
		}
		catch (MdwsResponseException e) {
			MdwsResponseException.UserMessages um = e.getUserMessage();
			if (!expected.equals(um))
				Assert.fail("The Exception thrown was not '" + expected.name() + "' but was '" + (um == null ? "<null>" : um.name()) + "'");
			return;
		}
		Assert.fail("Expected this exception to be thrown: " + expected.name());
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
}
