package com.agilex.healthcare.mdws.datalayer;

import static org.junit.Assert.assertNotNull;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mdws.datalayer.util.AuthenticationUtil;
import com.agilex.healthcare.mdws.datalayer.util.SpringBasedIntegrationRepo;
import com.agilex.healthcare.mdws.domain.EncounterImmunization;

public class ImmunizationSignTest extends SpringBasedIntegrationRepo {
	private static final String SITE_777 = "777";
	private static final String SITE_777_USER = "cprs1234";
	private static final String SITE_777_PASSWORD = "cprs4321$";
	private static final String SITE_777_ESIG = "CPRS1234";
	
	@Resource
	TIUNotesDataLayer encounterDataLayer;
	
	@BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser(SITE_777, SITE_777_USER, SITE_777_PASSWORD, "1", "TIU");
	}
    @AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser("TIU");
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
	@Test
	public void signImmunization() {
		EncounterImmunization result = encounterDataLayer.signImmunization(createImmunization(), SITE_777_ESIG);
		assertNotNull(result);
	}

//----------------------------------------------------------------------------------------------------------------------
	
	@Test
	public void failFacilityCode() {
		EncounterImmunization immunization = createImmunization();
		immunization.setFacilityCode("");
		failureConnecting("", SITE_777_USER, SITE_777_PASSWORD, "1", MdwsResponseException.UserMessages.INVALID_FACILITY_CODE, immunization); //The facilityCode was null or empty.
		immunization.setFacilityCode(null);
		failureConnecting(null, SITE_777_USER, SITE_777_PASSWORD, "1", MdwsResponseException.UserMessages.INVALID_FACILITY_CODE, immunization); //The facilityCode was null or empty.
		immunization.setFacilityCode("ABCDEFG");
		failureConnecting("ABCDEFG", SITE_777_USER, SITE_777_PASSWORD, "1", MdwsResponseException.UserMessages.CONNECTION_ERROR, immunization); //Site not in site table
	}
	
	@Test
	public void failPatientIen() {
		EncounterImmunization immunization = createImmunization();
		immunization.setPatientIen("");
		failure(MdwsResponseException.UserMessages.INVALID_PATIENT, immunization); //Missing local PID
		immunization.setPatientIen(null);
		failure(MdwsResponseException.UserMessages.INVALID_PATIENT, immunization); //patientIen cannot be null
		immunization.setPatientIen("ABCDEFG");
		failure(MdwsResponseException.UserMessages.INVALID_RECORD_ID, immunization); //Invalidly formed record ID: ABCDEFG
	}
	
    @Test
	public void failLocationIen() {
		EncounterImmunization immunization = createImmunization();
		immunization.setLocationIen("");
		failure(MdwsResponseException.UserMessages.INVALID_LOCATION_IEN, immunization); //Must supply locationIen
		immunization.setLocationIen(null);
		failure(MdwsResponseException.UserMessages.INVALID_LOCATION_IEN, immunization); //The location IEN provided was not valid.
		immunization.setLocationIen("ABCDEFG");
		failure(MdwsResponseException.UserMessages.INVALID_RECORD_ID, immunization); //Invalidly formed record ID: ABCDEFG
	}
	
    @Test
	public void failNoteTitleIen() {
		EncounterImmunization immunization = createImmunization();
		immunization.setNoteTitleIen("");
		failure(MdwsResponseException.UserMessages.INVALID_NOTE_TITLE_IEN, immunization); //Must supply noteTitleIen
		immunization.setNoteTitleIen(null);
		failure(MdwsResponseException.UserMessages.INVALID_NOTE_TITLE_IEN, immunization); //An Error Occurred with Note Title IEN
		immunization.setNoteTitleIen("ABCDEFG");
		failure(MdwsResponseException.UserMessages.INVALID_RECORD_ID, immunization); //Invalidly formed record ID: ABCDEFG
	}
	
    @Test
	public void failServiceCategory() {
		//'A' (ambulatory), 'H' (hospitalization), or 'E' (event historical) (defaults to 'A')
		EncounterImmunization immunization = createImmunization();
		immunization.setServiceCategory(null);
		failure(MdwsResponseException.UserMessages.INVALID_SERVICE_CATEGORY, immunization); //Input string for service category was not in a correct format.
		immunization.setServiceCategory("X");
		failure(MdwsResponseException.UserMessages.INVALID_SERVICE_CATEGORY, immunization); //serviceCategory must be 'A' (ambulatory), 'H' (hospitalization), or 'E' (event historical)
	}
	
    @Test
	public void failDate() {
		//should be formatted yyyymmdd.hhmmss
		EncounterImmunization immunization = createImmunization();
		immunization.setDate(null);
		failure(MdwsResponseException.UserMessages.INVALID_DATE, immunization); //Date cannot be null
		immunization.setServiceCategory("E");
		immunization.setDate("_+*&");
		failure(MdwsResponseException.UserMessages.INVALID_DATE, immunization); //Input string was not in a correct format.
	}

	@Test
	public void failProcedures() {
		EncounterImmunization immunization = createImmunization();
		immunization.setCptCode(null);
		failure(MdwsResponseException.UserMessages.INVALID_PROCEDURES, immunization);
		immunization.setCptCode("_+*&");
		failure(MdwsResponseException.UserMessages.INVALID_CPT_CODE, immunization);
	}
	
	@Test
	public void failVisitRelatedTos() {
		EncounterImmunization immunization = createImmunization();
		immunization.setVisitRelatedTos(null);
		failure(MdwsResponseException.UserMessages.INVALID_VISIT_RELATED_TOS, immunization); //An Error Occurred with Visit Related Tos
		immunization.setVisitRelatedTos("SAC");
		failure(MdwsResponseException.UserMessages.INVALID_VISIT_RELATED_TOS, immunization); //invalid visitRelatedTo entry: SAC
		immunization.setVisitRelatedTos("XYZ");
		failure(MdwsResponseException.UserMessages.INVALID_VISIT_RELATED_TOS, immunization); //invalid visitRelatedTo entry: XYZ
	}
	
	@Test
	public void failComment() {
		EncounterImmunization immunization = createImmunization();
		immunization.setComment(null);
		failure(MdwsResponseException.UserMessages.INVALID_COMMENT, immunization); //Must supply comment
		immunization.setComment("");
		failure(MdwsResponseException.UserMessages.INVALID_COMMENT, immunization); //Must supply comment
	}
	
	private void failure(MdwsResponseException.UserMessages expected, EncounterImmunization encounterImmunization) {
		try {
			encounterDataLayer.signImmunization(encounterImmunization, SITE_777_ESIG);
		}
		catch (MdwsResponseException e) {
			MdwsResponseException.UserMessages um = e.getUserMessage();
			if (!expected.equals(um))
				Assert.fail("The Exception thrown was not '" + expected.name() + "' but was '" + (um == null ? "<null>" : um.name()) + "'");
			return;
		}
		Assert.fail("Expected this exception to be thrown: " + expected.name());
	}
	
	private void failureConnecting(String siteCode, String accessCode, String verifyCode, String uniqueId, MdwsResponseException.UserMessages expected, EncounterImmunization encounterImmunization) {
		try {
			AuthenticationUtil.loginUser(siteCode, accessCode, verifyCode, uniqueId, "TIU");
			encounterDataLayer.signImmunization(encounterImmunization, SITE_777_ESIG);
		}
		catch (MdwsResponseException e) {
			MdwsResponseException.UserMessages um = e.getUserMessage();
			if (!expected.equals(um))
				Assert.fail("The Exception thrown was not '" + expected.name() + "' but was '" + (um == null ? "<null>" : um.name()) + "'");
			return;
		}
		Assert.fail("Expected this exception to be thrown: " + expected.name());
	}

//----------------------------------------------------------------------------------------------------------------------
		
	private EncounterImmunization createImmunization() {
		EncounterImmunization imm = new EncounterImmunization();
		imm.setPatientIen("1");
		imm.setFacilityCode(SITE_777);
		imm.setLocationIen("32");
		imm.setNoteTitleIen("16");
		imm.setServiceCategory("A");
		imm.setVisitRelatedTos("SC,CV,AO,IR,EC,SHD,MST,HNC");
		imm.setDate("");
		imm.setComment("My Comment");
		imm.setCptCode("90654");
		imm.setImmunizationIen("341366");
		imm.setImmunizationDescription("Influenza Virus Vaccine, Split Virus, Preservative-Free, for Intradermal use (CPT-4 90654)");
		return imm;
	}
}
