package com.agilex.healthcare.mdws.datalayer;

import static org.junit.Assert.assertNotNull;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mdws.connection.ConnectionManager;
import com.agilex.healthcare.mdws.connection.ConnectionManagerFactory;
import com.agilex.healthcare.mdws.connection.EncounterMdwsConnection;
import com.agilex.healthcare.mdws.connection.MdwsConfiguration;
import com.agilex.healthcare.mdws.connection.TIUNotesMdwsConnection;
import com.agilex.healthcare.mdws.datalayer.util.AuthenticationUtil;
import com.agilex.healthcare.mdws.datalayer.util.SpringBasedIntegrationRepo;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.TaggedModifiersByCptArrays;
import com.agilex.healthcare.mdws.generatedwsdl.tiunotesservice.TaggedTextArray;

public class MdwsResponseExceptionTest extends SpringBasedIntegrationRepo {
private static final String SITE_777 = "777";
	
    @Resource
    MdwsConfiguration encountersMdwsConfiguration;

    @Resource
    MdwsConfiguration tiunotesMdwsConfiguration;
    
    @BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("777", "cprs1234", "cprs4321$", "1", "FPS");
		AuthenticationUtil.loginUser("777", "cprs1234", "cprs4321$", "1", "TIU");
	}
    @AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser("FPS");
		AuthenticationUtil.logoffUser("TIU");
	}
    
//----------------------------------------------------------------------------------------------------------------------
	
    
	@Test
	public void getSiteNotReadyForFPSConnections() {
		EncounterMdwsConnection connection = getConnection(SITE_777);
		//In order to receive a connections not ready error, let's disconnect before we try making any calls.
		connection.getService().disconnect();
		
		TaggedModifiersByCptArrays result = connection.getService().getModifiersByCpt("1");
		assertNotNull(result.getFault());
		
		try {
			MdwsResponseValidator.validateMdwsObject(result);
			Assert.fail("Expected an MdwsResponseException, received a valid object");
		}
		catch (MdwsResponseException e) {
			Assert.assertEquals(MdwsResponseException.UserMessages.CONNECTIONS_NOT_READY, e.getUserMessage());
		}
		catch (Exception e) {
			Assert.fail("Expected a MdwsResponseException");
		}
	}

	@Test
	public void getSiteNotReadyForTIUNotesConnections() {
		TIUNotesMdwsConnection connection = getTIUNotesConnection(SITE_777);
		//In order to receive a connections not ready error, let's disconnect before we try making any calls.
		connection.getService().disconnect();
		
		TaggedTextArray result = connection.getService().getNotesEncounterString(SITE_777, "123");
		assertNotNull(result.getFault());
		
		try {
			MdwsResponseValidator.validateMdwsObject(result);
			Assert.fail("Expected an MdwsResponseException, received a valid object");
		}
		catch (MdwsResponseException e) {
			Assert.assertEquals(MdwsResponseException.UserMessages.CONNECTIONS_NOT_READY, e.getUserMessage());
		}
		catch (Exception e) {
			Assert.fail("Expected a MdwsResponseException");
		}
	}
	
	protected EncounterMdwsConnection getConnection(String siteCode) {
		ConnectionManager connectionManager = ConnectionManagerFactory.getInstance();
		return (EncounterMdwsConnection) connectionManager.getEncounterConnection(encountersMdwsConfiguration, siteCode);
	}
	
	protected TIUNotesMdwsConnection getTIUNotesConnection(String siteCode) {
		ConnectionManager connectionManager = ConnectionManagerFactory.getInstance();
		return (TIUNotesMdwsConnection) connectionManager.getTIUNotesConnection(tiunotesMdwsConfiguration, siteCode);
	}
	
}
