package com.agilex.healthcare.mdws.datalayer.util;

import junit.framework.Assert;

import org.junit.Test;

import com.agilex.healthcare.mdws.domain.EncounterNote;
import com.agilex.healthcare.mdws.domain.EncounterProcedureCPTCode;
import com.agilex.healthcare.mdws.domain.EncounterProcedureCPTCodes;

public class CptCodeUtilTest {
	@Test
	public void testCptCodeUtil() {
		EncounterNote note = EncounterNoteUtil.createEncounterNote();
		String actual = CptCodeUtil.translate(note.getProcedures());
    	String expected = "47500^Injection Procedure for Percutaneous Transhepatic Cholangiography (CPT-4 47500)";
		expected += ";" + "D4341^Periodontal Scaling and Root Planing (per Quadrant) (HCPCS D4341)";
		expected += ";" + "23420^Reconstruction of Complete Shoulder (Rotator) Cuff Avulsion, Chronic (includes Acromioplasty) (CPT-4 23420)^^47505|299344";
		expected += ";" + "19355^Correction of Inverted Nipple(s) (CPT-4 19355)^^GJ|367^47|9^50|10";
		Assert.assertEquals(expected, actual);
	}
	
	@Test
	public void testSemicolonsInDescription() {
		EncounterProcedureCPTCode proc1 = EncounterNoteUtil.createEncounterProcedureCPTCode("00528", "Anesthesia for closed Chest Procedures; Mediastinoscopy and Diagnostic Thoracoscopy not Utilizing 1 Lung Ventilation");
		EncounterProcedureCPTCode proc2 = EncounterNoteUtil.createEncounterProcedureCPTCode("D4341", "Periodontal Scaling and Root Planing (per Quadrant) (HCPCS D4341)");
		EncounterProcedureCPTCodes procs = new EncounterProcedureCPTCodes();
		procs.add(proc1);
		procs.add(proc2);
		
		String actual = CptCodeUtil.translate(procs);
    	String expected = "00528^Anesthesia for closed Chest Procedures\\; Mediastinoscopy and Diagnostic Thoracoscopy not Utilizing 1 Lung Ventilation";
    	expected += ";" + "D4341^Periodontal Scaling and Root Planing (per Quadrant) (HCPCS D4341)";
		Assert.assertEquals(expected, actual);
	}
}
