package com.agilex.healthcare.mdws.datalayer.util;

import junit.framework.Assert;

import org.junit.Test;

import com.agilex.healthcare.mdws.domain.EncounterDiagnosisICDCode;
import com.agilex.healthcare.mdws.domain.EncounterNote;
import com.agilex.healthcare.mdws.domain.EncounterDiagnosisICDCodes;

public class DiagnosesUtilTest {
	@Test
	public void testDiagnosesUtil() {
		EncounterNote note = EncounterNoteUtil.createEncounterNote();
		String actual = DiagnosesUtil.translate(note.getDiagnoses());
		String expected = "756.10^Anomaly of spine, congenital (ICD-9-CM 756.10)^1^1";
		expected += ";" + "303.00^Alcohol Intoxication (DSM-IV 303.00/ICD-9-CM 303.00/305.00)^1^0";
		expected += ";" + "309.81^Posttraumatic Stress Disorder * (DSM-IV 309.81/ICD-9-CM 309.81)^0^1";
		expected += ";" + "942.22^Blisters with epidermal loss due to burn (second degree) of chest wall, excluding breast and nipple (ICD-9-CM 942.22)^0^0";
		Assert.assertEquals(expected, actual);
	}
	
	@Test
	public void testSemicolonsInDescription() {
		EncounterDiagnosisICDCode icd1 = EncounterNoteUtil.createEncounterDiagnosisICDCode(true, true, "XXX.XX", "Something Bogus; With a semi-colon in it.");
		EncounterDiagnosisICDCode icd2 = EncounterNoteUtil.createEncounterDiagnosisICDCode(false, false, "303.00", "Alcohol Intoxication (DSM-IV 303.00/ICD-9-CM 303.00/305.00)");
		EncounterDiagnosisICDCodes icds = new EncounterDiagnosisICDCodes();
		icds.add(icd1);
		icds.add(icd2);
		
		String actual = DiagnosesUtil.translate(icds);
    	String expected = "XXX.XX^Something Bogus\\; With a semi-colon in it.^1^1";
    	expected += ";" + "303.00^Alcohol Intoxication (DSM-IV 303.00/ICD-9-CM 303.00/305.00)^0^0";
		Assert.assertEquals(expected, actual);
	}
}
