package com.agilex.healthcare.mdws.connection;

import java.io.Serializable;
import java.util.Date;
import java.util.UUID;

public abstract class MdwsConnection<T> implements Serializable {

	private static final long serialVersionUID = 7121050209087208049L;
	private final String id = UUID.randomUUID().toString();
	private Date refreshedTimestamp;
	private MdwsConfiguration configuration;

	public MdwsConnection(MdwsConfiguration configuration){
		this.configuration = configuration;
	}

	public Date getRefreshedTimestamp() {
		return refreshedTimestamp;
	}

	public void refreshTimestamp() {
		this.refreshedTimestamp = new Date();
	}

	public String getId() {
		return id;
	}

	public abstract T getService();
	
	protected abstract T establishConnection(MdwsConfiguration configuration, String siteCode);

	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		MdwsConnection other = (MdwsConnection) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public MdwsConfiguration getConfiguration() {
		return configuration;
	}
	public void setConfiguration(MdwsConfiguration configuration) {
		this.configuration = configuration;
	}
	
}
