package com.agilex.healthcare.mdws.datalayer;

import com.agilex.healthcare.mdws.connection.BasicConnectionManager;
import com.agilex.healthcare.mdws.connection.EmrMdwsConnection;
import com.agilex.healthcare.mdws.connection.MdwsConfiguration;
import com.agilex.healthcare.mdws.datalayer.util.MdwsConfigurationHelper;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.PatientTO;

import java.util.concurrent.atomic.AtomicBoolean;

public class EmrDataLayerMdws {

	protected MdwsConfiguration mdwsConfiguration;
	private final AtomicBoolean isConnectedToCDW = new AtomicBoolean(false);

	protected EmrDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		this.mdwsConfiguration = mdwsConfiguration;
	}

	protected EmrMdwsConnection getAuthenticatedConnection(String siteCode) {
		BasicConnectionManager connectionManager = new BasicConnectionManager();
		EmrMdwsConnection connection = connectionManager.getEmrConnection(mdwsConfiguration, siteCode);
		boolean connectedToCDW = (siteCode.equals(MdwsConfigurationHelper.CDW_SITECODE));
		isConnectedToCDW.set(connectedToCDW);
		return connection;
	}

	protected EmrMdwsConnection getAuthenticatedConnectionWithPatientSelect(PatientIdentifier patientIdentifier) {
		return getAuthenticatedConnectionWithPatientSelect(getSiteCodeFromPatientIdentifier(patientIdentifier), patientIdentifier.getUniqueId());
	}

	/**
	 * Calls the MDWS service EmrSvc.select
	 */
	protected EmrMdwsConnection getAuthenticatedConnectionWithPatientSelect(String siteCode, String patientId) {
        EmrMdwsConnection connection = getAuthenticatedConnection(siteCode);
        PatientTO result = null;
		try {
			result = connection.getService().select(patientId);
		}
		catch (Exception e) {
			throw new MdwsResponseException(MdwsResponseFaultTOFilter.getResponseType(e.getMessage()), e.getMessage());
		}
        if (result.getFault() != null) {
        	throw new MdwsResponseException(result.getFault().getMessage());
        }
        return connection;
    }

    protected String getSiteCodeFromPatientIdentifier(PatientIdentifier patientIdentifier){
		String siteCode = new MdwsConfigurationHelper().extractSiteCodeFromAssigningAuthority(patientIdentifier.getAssigningAuthority());
		return siteCode;
	}

    /**
     * This method exists in this class instead of @EmrConnection because the logic in method @getSiteCodeFromPatientIdentifier
     * which swaps the site code to 204 if the patient identifier is ICN.
     */
    protected void disconnectFromCdwIfApplicable(EmrMdwsConnection emrConnection) {
		if (isConnectedToCDW.get() == true){
			emrConnection.getService().disconnect();
			isConnectedToCDW.compareAndSet(true, false);
		}
	}

}
