package com.agilex.healthcare.mdws.datalayer.util;

import com.agilex.healthcare.mdws.datalayer.MdwsResponseException;
import com.agilex.healthcare.mdws.domain.EncounterProcedureCPTCodes;

public class CptCodeUtil {

	/**
	 * <b>EncounterNote.procedures</b> (OPTIONAL): Procedures is a delimited string.
	 * Use semicolons to delimit the individual procedures; carets to delimit the fields within each procedure
	 * (each modifier may be treated as a separate field); and pipes to delimit the modifierCode from the modifierIen.
	 * If your encounter has no procedures (or visit types), you can pass an empty string for this parameter.
	 * Note that passing an empty string is only allowed if you are using service category 'E' and/or
	 * inpatient value '1'. Otherwise, passing an empty string for procedures will return an error.<br/>
	 * <code>
	 * 	<b>Format:</b><br/>
	 * 		cpt1Code^cpt1Description^cpt1Comment^cpt1Modifier1Code|cpt1Modifier1Ien^cpt1Modifier2Code|cpt1Modifier2Ien^cpt1Modifier3Code|cpt1Modifier3Ien;cpt2Code^cpt2Description^cpt2Comment;cpt3Code^cpt3Description<br/>
	 * 	<b>Example:</b><br/>
	 * 		90662^Influenza Virus Vaccine, Split Virus, Preservative Free, Enhanced Immunogenicity via Increased Antigen Content, for Intramuscular use (CPT-4 90662)^flu shot^AK|497^EY|421^GR|250^KX|423;93530^Right Heart Catheterization, for Congenital Cardiac Anomalies (CPT-4 93530)^heart catheterization<br/>
	 * </code>
	 * <b>cptCode and cptDescription</b> are required for each procedure. Comment and modifiers may be omitted.
	 * Any modifier that is included must have both a code and an IEN.
	 */
	public static String translate(EncounterProcedureCPTCodes cptCodes) {
		if (cptCodes == null || cptCodes.size() == 0)
			return "";
		
		StringBuffer sb = new StringBuffer();
		
		for (int i=0; i<cptCodes.size(); i++) {
			if (i > 0)
				sb.append(";");

			if (cptCodes.get(i).getBasicCode() == null)
				throw new MdwsResponseException(MdwsResponseException.UserMessages.MISSING_CPT_CODE, "A procedure was populated without codes");

			String code = cptCodes.get(i).getBasicCode().getCode();
			String description = cptCodes.get(i).getBasicCode().getDescription();
			
			if (code == null || code.isEmpty())
				throw new MdwsResponseException(MdwsResponseException.UserMessages.MISSING_CPT_CODE, "The array of procedures contained a cptCode that was null or empty");
			
			if (description == null || description.isEmpty())
				throw new MdwsResponseException(MdwsResponseException.UserMessages.MISSING_CPT_CODE, "The array of procedures contained a cptDescription that was null or empty");
			
			description = description.replaceAll(";", "\\\\;");
			
			sb.append(code);
			sb.append("^");
			sb.append(description);
			
			if (cptCodes.get(i).getModifiers() != null && cptCodes.get(i).getModifiers().size() > 0) {
				sb.append("^");
				sb.append(""); //Comment
				sb.append("^");
				
				for (int k=0; k<cptCodes.get(i).getModifiers().size(); k++) {
					if (k > 0)
						sb.append("^");

					String modifierCode = cptCodes.get(i).getModifiers().get(k).getCode();
					String modifierIen = cptCodes.get(i).getModifiers().get(k).getIen();
					sb.append(modifierCode);
					sb.append("|");
					sb.append(modifierIen);
				}
			}
			
		}
		return sb.toString();
	}

}
