package com.agilex.healthcare.mdws.datalayer.util;

import com.agilex.healthcare.mdws.datalayer.MdwsResponseException;
import com.agilex.healthcare.mdws.domain.EncounterDiagnosisICDCodes;

public class DiagnosesUtil {

	/**
	 * <b>EncounterNote.diagnoses</b> (OPTIONAL): Diagnoses is a delimited string.
	 * Use semicolons to delimit the individual diagnoses; carets to delimit the fields within each diagnosis.
	 * If your encounter has no diagnoses, you can pass an empty string for this parameter.
	 * <code>
	 * 	<b>Format:</b><br/>
	 * 		diagnosis1Code^diagnosis1Description^diagnosis1IsPrimary^diagnosis1AddToProblemList^diagnosis1Comment;diagnosis2Code^diagnosis2Description^diagnosis2IsPrimary^diagnosis2AddToProblemList;diagnosis3Code^diagnosis3Description<br/>
	 * 	<b>Example:</b><br/>
	 * 		427.5^Cardiac Arrest * (ICD-9-CM 427.5)^1^1^heart arrest;668.11^Cardiac complications of anesthesia or other sedation in labor and delivery, delivered (ICD-9-CM 668.11)^0^1^complications;V12.53^Personal History of Sudden Cardiac Arrest (ICD-9-CM V12.53)^0^0^personal history<br/>
	 * </code>
	 * <b>icd9Code and description</b> are required for each diagnosis.
	 * isPrimary and addToProblemList should be '1' for true and '0' for false, and will be set to '0' if they are
	 * omitted or invalid. Comment is optional.
	 */
	public static String translate(EncounterDiagnosisICDCodes diagnoses) {
		if (diagnoses == null || diagnoses.size() == 0)
			return "";
		
		StringBuffer sb = new StringBuffer();
		
		for (int i=0; i<diagnoses.size(); i++) {
			if (i > 0)
				sb.append(";");
			
			boolean addToProblemList = diagnoses.get(i).isAddToProblemList();
			boolean primary = diagnoses.get(i).isPrimary();
			
			if (diagnoses.get(i).getBasicCode() == null)
				throw new MdwsResponseException(MdwsResponseException.UserMessages.MISSING_ICD_CODE, "A diagnosis was populated without codes");
			String icd9Code = diagnoses.get(i).getBasicCode().getCode();
			String icd9Description = diagnoses.get(i).getBasicCode().getDescription();
			
			if (icd9Code == null || icd9Code.isEmpty())
				throw new MdwsResponseException(MdwsResponseException.UserMessages.MISSING_ICD_CODE, "The array of diagnoses contained a icd9Code that was null or empty");
			if (icd9Description == null || icd9Description.isEmpty())
				throw new MdwsResponseException(MdwsResponseException.UserMessages.MISSING_ICD_CODE, "The array of diagnoses contained a icd9Description that was null or empty");
			
			icd9Description = icd9Description.replaceAll(";", "\\\\;");
			
			sb.append(icd9Code);
			sb.append("^");
			sb.append(icd9Description);
			sb.append("^");
			sb.append(primary ? "1" : "0");
			sb.append("^");
			sb.append(addToProblemList ? "1" : "0");
//			sb.append("^");
//			sb.append(""); //Comment
		}
		return sb.toString();
	}

}
