package com.agilex.healthcare.mdws.datalayer.util;

import java.util.Iterator;

import com.agilex.healthcare.mdws.datalayer.MdwsResponseException;
import com.agilex.healthcare.mdws.domain.Code;
import com.agilex.healthcare.mdws.domain.Codes;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.LexiconEntryTO;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.TaggedLexiconEntryArray;

public class LexiconCodeLookupUtil {
	public static Codes translateLVTaggedLexiconEntryArray(TaggedLexiconEntryArray arr, String[] filter) {

		//Perform Error Checking
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedLexiconCodeLookupArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault().getMessage());
		if (arr.getEntries() == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedLexiconCodeLookupArrays.getEntries was null");
		if (arr.getEntries().getLexiconEntryTO() == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedLexiconCodeLookupArrays.getEntries.getLVLexiconEntryTO was null");

		Codes codes = new Codes();

		Iterator<LexiconEntryTO> itLVLexiconEntryTO = arr.getEntries().getLexiconEntryTO().iterator();
		while (itLVLexiconEntryTO.hasNext()) {
			LexiconEntryTO lexiconEntryTO = itLVLexiconEntryTO.next();

			//Perform Error Checking
			if (lexiconEntryTO.getFault() != null)
				throw new MdwsResponseException(lexiconEntryTO.getFault().getMessage());
			//For some weird reason, they've decided that an IEN of -1 means they couldn't find anything.
			if ("-1".equals(lexiconEntryTO.getLexIen()))
				return codes;

			if ( isItemInFilterList(filter, lexiconEntryTO) ){
				codes.add(translate(lexiconEntryTO));
			}
		}

		return codes;
	}
	
	private static boolean isItemInFilterList(String[] filter, LexiconEntryTO lexiconEntryTO){
		boolean inList = false;
		
		// If the array length is 0 then there is nothing to filter out.
		if (filter.length == 0)
			return true;
		
		for (int i=0; i<filter.length; i++) {
			String s = filter[i];
			if (s.equalsIgnoreCase(lexiconEntryTO.getCodeSys()) || s.equalsIgnoreCase(lexiconEntryTO.getIcdVer())) {
				return true;
			}
		}
		return inList;
	}
	
	private static Code translate(LexiconEntryTO lexiconEntryTO){
		Code code = new Code();
		code.setIen(lexiconEntryTO.getLexIen());
		code.setDescription(lexiconEntryTO.getPrefText());
		code.setCode(lexiconEntryTO.getConceptID());
		code.setCodeType(lexiconEntryTO.getCodeSys());
		return code;
	}
	
}
