package com.agilex.healthcare.mdws.datalayer.util;

import java.util.Iterator;

import com.agilex.healthcare.mdws.datalayer.MdwsResponseException;
import com.agilex.healthcare.mdws.domain.Code;
import com.agilex.healthcare.mdws.domain.Codes;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.ArrayOfModifiersByCptTO;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.ModifiersByCptTO;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.TaggedModifiersByCptArray;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.TaggedModifiersByCptArrays;

public class ModifierUtil {
	public static Codes translateLVTaggedModifiersByCptArrays(TaggedModifiersByCptArrays arr) {
		//Perform Error Checking
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "LVTaggedModifiersByCptArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault().getMessage());
		if (arr.getArrays() == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "LVTaggedModifiersByCptArrays.getArrays was null");
		if (arr.getArrays().getTaggedModifiersByCptArray() == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "LVTaggedModifiersByCptArrays.getArrays.getLVTaggedModifiersByCptArray was null");

		Codes retValue = new Codes();

		//The object returned sends the data (LVModifiersByCptTO) back in an array of statuses objects
		//LVTaggedModifiersByCptArrays | arrays | LVTaggedModifiersByCptArray | statuses | LVModifiersByCptTO
		//If no values were found, the statuses object is not there.
		//The LVTaggedModifiersByCptArrays and LVTaggedModifiersByCptArray are both arrays that in all of our testing
		//do not return data (but because the potential is there for them to return data we just collect everything to
		//put in our object to return.
		Iterator<TaggedModifiersByCptArray> itTaggedModifiers = arr.getArrays().getTaggedModifiersByCptArray().iterator();
		while (itTaggedModifiers.hasNext()) {
			TaggedModifiersByCptArray taggedModifiers = itTaggedModifiers.next();
			//Perform Error Checking
			if (taggedModifiers.getFault() != null)
				throw new MdwsResponseException(taggedModifiers.getFault().getMessage());

			ArrayOfModifiersByCptTO modifiersArr = taggedModifiers.getStatuses();
			//Perform Error Checking
			if (modifiersArr == null || modifiersArr.getModifiersByCptTO() == null)
				continue;

			Iterator<ModifiersByCptTO> itModifier = modifiersArr.getModifiersByCptTO().iterator();
			while (itModifier.hasNext()) {
				ModifiersByCptTO modifiers = itModifier.next();
				//Perform Error Checking
				if (modifiers.getFault() != null)
					throw new MdwsResponseException(modifiers.getFault().getMessage());

				Code mc = new Code();
				mc.setCode(modifiers.getCptmodifiercode());
				mc.setCodeType("CPT Modifier");
				mc.setDescription(modifiers.getDescription());
				mc.setIen(modifiers.getIen());
				retValue.add(mc);
			}
		}

		return retValue;
	}
}
