package com.agilex.healthcare.mdws.datalayer.util;

import java.util.Iterator;

import com.agilex.healthcare.mdws.datalayer.MdwsResponseException;
import com.agilex.healthcare.mdws.domain.Code;
import com.agilex.healthcare.mdws.domain.ProblemList;
import com.agilex.healthcare.mdws.domain.ProblemLists;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.ArrayOfPatientProblemListDiagnosesTO;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.PatientProblemListDiagnosesTO;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.TaggedPatientProblemListDiagnosesArray;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.TaggedPatientProblemListDiagnosesArrays;

public class ProblemListUtil {

	public static ProblemLists translateLVTaggedPatientProblemListDiagnosesArrays(TaggedPatientProblemListDiagnosesArrays arr) {
		//Perform Error Checking
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "LVTaggedPatientProblemListDiagnosesArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault().getMessage());
		if (arr.getArrays() == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "LVTaggedPatientProblemListDiagnosesArrays.getArrays was null");
		if (arr.getArrays().getTaggedPatientProblemListDiagnosesArray() == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "LVTaggedModifiersByCptArrays.getArrays.getLVTaggedPatientProblemListDiagnosesArray was null");

		ProblemLists retValue = new ProblemLists();

		Iterator<TaggedPatientProblemListDiagnosesArray> itPatientProblemList = arr.getArrays().getTaggedPatientProblemListDiagnosesArray().iterator();

		//<LVTaggedPatientProblemListDiagnosesArrays>
		//  <arrays>
		//    <LVTaggedPatientProblemListDiagnosesArray>
		//      <statuses>
		//        <LVPatientProblemListDiagnosesTO>
		while (itPatientProblemList.hasNext()) {
			TaggedPatientProblemListDiagnosesArray arPatientProblemList = itPatientProblemList.next();
			//Perform Error Checking
			if (arPatientProblemList.getFault() != null)
				throw new MdwsResponseException(arPatientProblemList.getFault().getMessage());

			ArrayOfPatientProblemListDiagnosesTO statusesArr = arPatientProblemList.getStatuses();
			//Perform Error Checking
			if (statusesArr == null || statusesArr.getPatientProblemListDiagnosesTO() == null)
				continue;

			Iterator<PatientProblemListDiagnosesTO> itProblems = statusesArr.getPatientProblemListDiagnosesTO().iterator();
			while (itProblems.hasNext()) {
				PatientProblemListDiagnosesTO problem = itProblems.next();
				//Perform Error Checking
				if (problem.getFault() != null)
					throw new MdwsResponseException(problem.getFault().getMessage());

				ProblemList pl = new ProblemList();
				Code code = new Code();
				code.setCodeType("ICD-9-CM");
				code.setCode(problem.getIcdCode());
				code.setDescription(problem.getDescription());
				code.setIen(problem.getProblemIen());
				pl.setCode(code);
				pl.setDateOnset(problem.getDateOnset());
				pl.setProblemFlag(problem.getProblemFlag());
				pl.setServiceConnected(problem.getServiceConnected());
				pl.setServiceConnectedYN(problem.getServiceConnectedYN());
				pl.setSpecialExposure(problem.getSpecialExposure());

				retValue.add(pl);
			}
		}

		return retValue;
	}

}
