package com.agilex.healthcare.mdws.datalayer.util;

import java.util.Iterator;

import com.agilex.healthcare.mdws.datalayer.MdwsResponseException;
import com.agilex.healthcare.mdws.domain.BasicCode;
import com.agilex.healthcare.mdws.domain.BasicCodes;
import com.agilex.healthcare.mdws.domain.VisitType;
import com.agilex.healthcare.mdws.domain.VisitTypes;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.TaggedVisitTypeCptByClinicArray;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.TaggedVisitTypeCptByClinicArrays;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.VisitSubType;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.VisitTypeCptByClinicTO;

public class VisitTypeUtil {
	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(VisitTypeUtil.class);

	public static VisitTypes translateLVTaggedVisitTypeCptByClinicArrays(TaggedVisitTypeCptByClinicArrays arr) {
		//Perform Error Checking
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "LVTaggedVisitTypeCptByClinicArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault().getMessage());
		if (arr.getArrays() == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "LVTaggedVisitTypeCptByClinicArrays.getArrays was null");
		if (arr.getArrays().getTaggedVisitTypeCptByClinicArray() == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "LVTaggedVisitTypeCptByClinicArrays.getArrays.getLVTaggedVisitTypeCptByClinicArray was null");

		VisitTypes retValue = new VisitTypes();


		//<LVTaggedVisitTypeCptByClinicArrays>
		//  <arrays>
		//    <LVTaggedVisitTypeCptByClinicArray>
		//      <statuses>
		//        <LVVisitTypeCptByClinicTO>
		//          <visitsubtypes>
		Iterator<TaggedVisitTypeCptByClinicArray> itTaggedVisitType = arr.getArrays().getTaggedVisitTypeCptByClinicArray().iterator();
		while (itTaggedVisitType.hasNext()) {
			TaggedVisitTypeCptByClinicArray taggedVisitType = itTaggedVisitType.next();
			//Perform Error Checking
			if (taggedVisitType.getFault() != null)
				throw new MdwsResponseException(taggedVisitType.getFault().getMessage());
			if (taggedVisitType.getStatuses() == null || taggedVisitType.getStatuses().getVisitTypeCptByClinicTO() == null)
				continue;

			Iterator<VisitTypeCptByClinicTO> itVisitType = taggedVisitType.getStatuses().getVisitTypeCptByClinicTO().iterator();
			while (itVisitType.hasNext()) {
				VisitTypeCptByClinicTO visitType = itVisitType.next();
				//Perform Error Checking
				if (visitType.getFault() != null)
					throw new MdwsResponseException(visitType.getFault().getMessage());

				VisitType vt = new VisitType();
				vt.setDescription(visitType.getVisittypedescription());

				if (visitType.getVisittypedescription() == null || visitType.getVisittypedescription().isEmpty()) {
					logger.error("VisitTypeUtil unexpectedly received a description that was null");
					continue;
				}

				retValue.add(vt);

				if (visitType.getVisitsubtypes() == null || visitType.getVisitsubtypes().getVisitSubType() == null) {
					continue;
				}

				BasicCodes codes = new BasicCodes();
				Iterator<VisitSubType> itSubTypes = visitType.getVisitsubtypes().getVisitSubType().iterator();
				while (itSubTypes.hasNext()) {
					VisitSubType subType = itSubTypes.next();
					BasicCode code = new BasicCode();
					code.setCode(subType.getVisitSubTypeCptCode());
					code.setDescription(subType.getVisitSubTypeDescription());
					codes.add(code);
				}
				vt.setCodes(codes);
			}
		}

		return retValue;
	}
}
