package com.agilex.healthcare.mdws.translator;

import java.util.Date;
import java.util.List;

import com.agilex.healthcare.mdws.datalayer.MdwsResponseException;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.ArrayOfGraphingItemDataTO;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.ArrayOfItemForGraphingTypeTO;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.GraphingItemDataTO;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.ItemForGraphingTypeTO;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.TaggedGraphingItemDataArray;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.TaggedItemForGraphingTypeArray;
import com.agilex.healthcare.mobilehealthplatform.domain.DataIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResult;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResults;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTest;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTests;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;

public class LabTestTranslator {
	public LabTests translateTaggedItemForGraphingTypeArray(PatientIdentifier patientIdentifier, String siteCode, TaggedItemForGraphingTypeArray taggedItemForGraphingTypeArray) {
		if (taggedItemForGraphingTypeArray == null) {
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "taggedItemForGraphingTypeArray was null");
		} else if (taggedItemForGraphingTypeArray.getFault() != null) {
			throw new MdwsResponseException(taggedItemForGraphingTypeArray.getFault().getMessage());
		}

		LabTests labTests = new LabTests();

		labTests.setPatientIdentifier(patientIdentifier);

		ArrayOfItemForGraphingTypeTO arrayOfItemForGraphingType = taggedItemForGraphingTypeArray.getItems();
		if (arrayOfItemForGraphingType != null) {
			List<ItemForGraphingTypeTO> itemForGraphingTypeList = arrayOfItemForGraphingType.getItemForGraphingTypeTO();

			for (ItemForGraphingTypeTO itemForGraphingType : itemForGraphingTypeList) {
				LabTest labTest = translateItemForGraphingType(patientIdentifier, siteCode, itemForGraphingType);
				labTests.add(labTest);
			}
		}

		return labTests;
	}

	public LabTest translateItemForGraphingType(PatientIdentifier patientIdentifier, String siteCode, ItemForGraphingTypeTO itemForGraphingType) {
		LabTest labTest = new LabTest();

		labTest.setPatientIdentifier(patientIdentifier);

		String itemIen = itemForGraphingType.getItemIen();
		labTest.setIen(itemIen);

		DataIdentifier dataIdentifier = new DataIdentifier(siteCode, itemIen);
		labTest.setDataIdentifier(dataIdentifier);

		String testName = itemForGraphingType.getItemName();
		labTest.setTestname(testName);

		String accessionAreaIen = itemForGraphingType.getAccAreaIen();
		labTest.setAccessionAreaIen(accessionAreaIen);

		String accessionAreaName = itemForGraphingType.getAccAreaName();
		labTest.setAccessionAreaName(accessionAreaName);

		String accessionSubscript = itemForGraphingType.getAccSubscript();
		labTest.setAccessionSubscript(accessionSubscript);

		labTest.setSiteCode(siteCode);

		String mostCurrentDateString = itemForGraphingType.getMostCurrentResultDate();
		if (NullChecker.isNotNullish(mostCurrentDateString)) {
			String convertedMostCurrentDateString = DateHelper.convertPureVistaDateToKnownVistaDateFormat(mostCurrentDateString);
			Date parsedVistaDate = DateHelper.parseAnyVistaDateString(convertedMostCurrentDateString);
			labTest.setMostCurrentResultDate(parsedVistaDate);
		}

		return labTest;
	}

	public LabResults translateTaggedGraphingItemDataArray(PatientIdentifier patientIdentifier, TaggedGraphingItemDataArray taggedGraphingItemDataArray, String labTestName) {
		if (taggedGraphingItemDataArray == null) {
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "translateTaggedGraphingItemDataArray was null");
		} else if (taggedGraphingItemDataArray.getFault() != null) {
			throw new MdwsResponseException(taggedGraphingItemDataArray.getFault().getMessage());
		}

		LabResults labResults = new LabResults();

		ArrayOfGraphingItemDataTO arrayOfGraphingItemData = taggedGraphingItemDataArray.getItems();

		if (arrayOfGraphingItemData != null) {
			List<GraphingItemDataTO> graphingItemDataList = arrayOfGraphingItemData.getGraphingItemDataTO();
			for (GraphingItemDataTO graphingItemData : graphingItemDataList) {
				LabResult labResult = translateGraphingItemData(patientIdentifier, graphingItemData, labTestName);
				labResults.add(labResult);
			}
		}

		return labResults;
	}

	public LabResult translateGraphingItemData(PatientIdentifier patientIdentifier, GraphingItemDataTO graphingItemData, String labTestName) {
		LabResult labResult = new LabResult();

		labResult.setPatientIdentifier(patientIdentifier);
		labResult.setTestname(labTestName);

		String testId = graphingItemData.getItemIen();
		labResult.setTestId(testId);

		String value = graphingItemData.getResult();
		labResult.setValue(value);

		String flag = graphingItemData.getFlag();
		labResult.setFlag(flag);

		String specimenId = graphingItemData.getSpecimenIen();
		labResult.setSpecimenId(specimenId);

		String specimenName = graphingItemData.getSpecimenName();
		labResult.setSpecimenName(specimenName);

		String valueUnits = graphingItemData.getUnits();
		labResult.setValueUnits(valueUnits);

		String range = graphingItemData.getRange();
		if (NullChecker.isNotNullish(range)) {
			String[] rangeTokens = range.split("!");
			if(rangeTokens.length > 0 && NullChecker.isNotNullish(rangeTokens[0])) {
				labResult.setReferenceLow(rangeTokens[0]);
			}
			if(rangeTokens.length > 1 && NullChecker.isNotNullish(rangeTokens[1])) {
				labResult.setReferenceHigh(rangeTokens[1]);
			}
		}

		String resultDate = graphingItemData.getResultDate();
		if (NullChecker.isNotNullish(resultDate)) {
			String convertedResultDate = DateHelper.convertPureVistaDateToKnownVistaDateFormat(resultDate);
			Date parsedVistaDate = DateHelper.parseAnyVistaDateString(convertedResultDate);
			labResult.setResultedDate(parsedVistaDate);
		}

		return labResult;
	}
}
