package com.agilex.healthcare.mobilehealthplatform.client.datalayer;

import com.agilex.healthcare.mobilehealthplatform.client.MobileHealthClient;
import com.agilex.healthcare.mobilehealthplatform.datalayer.allergy.AllergyDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultFilter;
import com.agilex.healthcare.mobilehealthplatform.datalayer.medication.MedicationDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.medication.MedicationFilter;
import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientSearchCriteria;
import com.agilex.healthcare.mobilehealthplatform.datalayer.problem.ProblemDataLayer;
import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.utility.NullChecker;
import com.agilex.healthcare.utility.ResourceUriEncoder;
import org.apache.commons.lang.NotImplementedException;

import java.net.URI;
import java.util.Collection;

public class PatientsResourceClient implements PatientDataLayer, AllergyDataLayer, ProblemDataLayer, 
	MedicationDataLayer, AppointmentDataLayer, ConsultDataLayer {
	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(PatientsResourceClient.class);

	private MobileHealthClient mhpClient;

	public PatientsResourceClient(MobileHealthClient mhpClient) {
		this.mhpClient = mhpClient;
	}

	@Override
	public Patients patientSearch(PatientSearchCriteria criteria) {
		return this.mhpClient.patientsResource().queryParam("searchstring", criteria.getSearchString()).get(Patients.class);
	}

	@Override
	public PatientDemographics getDemographics(PatientIdentifier patientIdentifier) {
		Patient patient = getPatient(patientIdentifier);
		URI uri = patient.getDemographicsUri();
		return this.mhpClient.resource(uri).get(PatientDemographics.class);
	}

	@Override
	public Patient getPatient(PatientIdentifier patientIdentifier) {
		Patients patients = this.mhpClient.patientsResource().queryParam("patientid", patientIdentifier.getUniqueId()).get(Patients.class);
		Patient patient = null;
		if (NullChecker.isNotNullish(patients))
			patient = patients.get(0);
		return patient;
	}

	public Allergies getPatientAllergies(String patientId) {
		throw new NotImplementedException();
		
	}

	@Override
	public Appointments getProviderAppointments(String clinicId, DateFilter dateFilter) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Appointments getClinicAppointments(String clinicId, DateFilter dateFilter) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Medications getMedications(PatientIdentifier patientId, boolean activeOnly) {
		return getPatientDataList(patientId, LinkTitles.PatientMedications, Medications.class, Medication.class);
	}

	@Override
	public Medication getMedicationLite(PatientIdentifier patientId, String orderNumber) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Medication getMedicationDetail(PatientIdentifier patientId, String orderNumber) {
		throw new UnsupportedOperationException();
	}

	private <Tl extends Collection<Td>, Td extends PatientData> Tl getPatientDataList(PatientIdentifier patientId, String uriTitle, Class<Tl> listClass, Class<Td> dataClass) {
		logger.debug(String.format("get patient [%s] to determine uri for [%s]", patientId, uriTitle));
		Patient patient = getPatient(patientId);
		logger.debug(String.format("retrieved patient [%s]", patient));
		URI uri = patient.getLink().getUriByTitle(uriTitle);
		logger.debug(String.format("retrieve data [%s] by uri [%s]", uriTitle, uri.toString()));
		Tl dataList = this.mhpClient.resource(uri).get(listClass);
		logger.debug(String.format("retrieved data [%s] by uri [%s] => [%s]", uriTitle, uri.toString(), dataList));

		updateUniqueIdToSelfUri((Collection<Td>) dataList);
		logger.debug("set unique id's to encoded uri");

		return dataList;
	}

	// private <Td> Td getPatientData(String dataId, Class<Td> dataClass) {
	// logger.debug(String.format("decoding data id [%s] to get data uri",
	// dataId));
	// String decodedDataId = ResourceUriEncoder.decodeUri(dataId);
	// logger.debug(String.format("decoded data id [%s]->[%s]", dataId,
	// decodedDataId));
	// URI uri = UriBuilder.fromPath(decodedDataId).build();
	// logger.debug(String.format("retrieve data by uri [%s]", uri.toString()));
	// Td data = this.mhpClient.resource(uri).get(dataClass);
	// logger.debug(String.format("retrieved data [%s]", data));
	// return data;
	// }

	private <Td extends PatientData> void updateUniqueIdToSelfUri(Collection<Td> dataList) {
		for (Object data : dataList) {
			if (data instanceof PatientData) {
				PatientData patientData = (PatientData) data;
				String patientDataSelfUri = patientData.getSelfUri().toString();
				if (NullChecker.isNotNullish(patientDataSelfUri)) {
					String encodedUri = ResourceUriEncoder.encodeUri(patientDataSelfUri);
					patientData.getDataIdentifier().setUniqueId(encodedUri);
				}
			}
		}
	}

    @Override
    public Appointments fetchAppointmentsByPatient(PatientIdentifier patientIdentifier, DateFilter filter) {
        throw new NotImplementedException();
    }

    @Override
    public Appointments fetchAppointmentsByPatient(PatientIdentifier patientIdentifier, DateFilter filter, boolean shouldIncludeDetails) {
        throw new NotImplementedException();
    }

    @Override
    public Appointment fetchAppointment(PatientIdentifier patientIdentifier, String appointmentId) {
        throw new NotImplementedException();
    }

	@Override
	public Medications getMedications(PatientIdentifier patientId, MedicationFilter medicationFilter) {
		throw new NotImplementedException();
	}


	public Allergies fetchAllergies(String patientId, DateFilter filter) {
		throw new NotImplementedException();
	}

	@Override
	public Allergies fetchAllergies(PatientIdentifier patientIdentifier, DateFilter filter) {
		return getPatientDataList(patientIdentifier, LinkTitles.PatientAllergies, Allergies.class, Allergy.class);
	}

	@Override
	public Problems fetchProblems(PatientIdentifier patientIdentifier, DateFilter dateFilter) {
		throw new NotImplementedException();
	}

	@Override
	public Consults fetchPatientConsults(PatientIdentifier patientId, ConsultFilter filter) {
		throw new NotImplementedException();
	}

	@Override
	public Consults fetchConsultsWrittenByProvider(ProviderIdentifier providerIdentifier, ConsultFilter filter) {
		throw new NotImplementedException();
	}

	@Override
	public Consults fetchConsultsBySpecialties(ConsultFilter filter, String facilityCode) {
		throw new NotImplementedException();
	}

	@Override
	public Appointments getVisits(PatientIdentifier patientIdentifier, DateFilter filter) {
		throw new NotImplementedException();
	}

}
