package com.agilex.healthcare.mobilehealthplatform.datalayer.consult;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.agilex.healthcare.mobilehealthplatform.enumeration.ConsultStatus;
import com.agilex.healthcare.utility.NullChecker;

public class ConsultFilter {
	private static final Integer DEFAULT_DAYS_FROM_CREATED_DATE = 730;

	private List<ConsultStatus> statuses;
	private Integer daysFromCreatedDate = DEFAULT_DAYS_FROM_CREATED_DATE;
	private List<String> specialties;
    private Date startDate;
    private Date endDate;
	
	public ConsultFilter() {
		statuses = new ArrayList<ConsultStatus>();
		specialties = new ArrayList<String>();
	}

	public ConsultFilter(List<ConsultStatus> statuses) {
		this.statuses = statuses;
	}

	public List<ConsultStatus> getStatuses() {
		return statuses;
	}

	public void setStatuses(List<ConsultStatus> statuses) {
		this.statuses = statuses;
	}
		
	public void addStatus(ConsultStatus status) {
		if (NullChecker.isNullish(statuses)) {
			statuses = new ArrayList<ConsultStatus>();
		}
		statuses.add(status);
	}

	public void addStatusByName(String name) {
		statuses.add(ConsultStatus.lookUpByName(name));
	}
	
	public void addSpecialtyById(String specialtyUniqueId) {
		specialties.add(specialtyUniqueId);
	}

	public Integer getDaysFromCreatedDate() {
		return daysFromCreatedDate;
	}

	public void setDaysFromCreatedDate(Integer daysFromCreatedDate) {
		if (daysFromCreatedDate == null){
			throw new IllegalArgumentException(String.format("Please set a filter value for daysFromCreatedDate, an example %s.", DEFAULT_DAYS_FROM_CREATED_DATE));
		}
		this.daysFromCreatedDate = daysFromCreatedDate;
	}

	public List<String> getSpecialties() {
		return specialties;
	}

    public Date getStartDate() {
        return startDate;
    }

    public void setStartDate(Date startDate) {
        this.startDate = startDate;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }
    
    public boolean doesDateRangeExist(){
    	return (this.getStartDate() != null && this.getEndDate() != null);
    }
}
