package com.agilex.healthcare.mobilehealthplatform.datalayer.consult;

import java.net.URI;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.agilex.healthcare.mobilehealthplatform.utils.QueryParameters;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;

public class ConsultFilterFactory {

	public static ConsultFilter createEmptyFilter() {
		ConsultFilter filter = new ConsultFilter();
		return filter;
	}
	
	public static ConsultFilter createFilterFromUri(URI uri) {
		QueryParameters queryParams = new QueryParameters(uri.getQuery());
		ConsultFilter filter = createFilter(queryParams);
		return filter;
	}

	private static ConsultFilter createFilter(QueryParameters queryParams) {
		String parameterValue = queryParams.getValue("statuses");		
		ConsultFilter filter = createEmptyFilter();
		if (NullChecker.isNotNullish(parameterValue)){
			filter = addStatusesToFilter(filter, parseCommaSeparatedStrings(parameterValue));
		}
		
		parameterValue = queryParams.getValue("daysFromCreatedDate");
		if (NullChecker.isNotNullish(parameterValue)){
			filter.setDaysFromCreatedDate(Integer.valueOf(parameterValue));
		}	

		parameterValue = queryParams.getValue("specialties");	
		if (NullChecker.isNotNullish(parameterValue)){
			filter = addSpecialtiesToFilter(filter, parseCommaSeparatedStrings(parameterValue));
		}

        parameterValue = queryParams.getValue("startDate");
        if (NullChecker.isNotNullish(parameterValue)){
            Date startDate = getValueAsDate(queryParams, "startDate", false);
			filter.setStartDate(startDate);
        }

        parameterValue = queryParams.getValue("endDate");
        if (NullChecker.isNotNullish(parameterValue)){
            Date endDate = getValueAsDate(queryParams, "endDate", true);
			filter.setEndDate(endDate);
        }
		return filter;
	}
	
	public static Date getValueAsDate(QueryParameters queryParams, String key, boolean endOfDay) {
		String rawValue = queryParams.getValue(key);
		Date dtValue = null;
		if (NullChecker.isNotNullish(rawValue))
			dtValue = DateHelper.parseDate(rawValue);
		if (dtValue != null && endOfDay == true) {
			Calendar cal = Calendar.getInstance();
			cal.setTime(dtValue);
			cal.set(Calendar.HOUR_OF_DAY, 23);
			cal.set(Calendar.MINUTE, 59);
			cal.set(Calendar.SECOND, 59);
			cal.set(Calendar.MILLISECOND, 59);
			dtValue = cal.getTime();
		}
		return dtValue;
	}
	
	private static ConsultFilter addSpecialtiesToFilter(ConsultFilter addToFilter, List<String> specialties){
		for (int i = 0; i < specialties.size(); i++) {
			addToFilter.addSpecialtyById(specialties.get(i));
		}
		return addToFilter;
	}	
	
	private static ConsultFilter addStatusesToFilter(ConsultFilter addToFilter, List<String> statuses){
		for (int i = 0; i < statuses.size(); i++) {
			addToFilter.addStatusByName(statuses.get(i));
		}
		return addToFilter;
	}
	
	private static List<String> parseCommaSeparatedStrings(String values) {
		if (NullChecker.isNotNullish(values)) {
			stripTrailingComma(values);
			String[] splits = values.split(",");
			List<String> parsedValues = Arrays.asList(splits);
			return parsedValues;
		} else {
			return null;
		}
	}

	private static void stripTrailingComma(String value) {
		if(value.charAt(value.length()-1) == ',') {
			value = value.substring(0, value.length()-1);
		}
	}
	
}
