package com.agilex.healthcare.mobilehealthplatform.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import java.util.Date;

@XmlRootElement(name = "clinicalEvent", namespace = Namespace.MobileHealthPlatform)
@XmlAccessorType(value = XmlAccessType.NONE)
public class ClinicalEvent extends DomainTransferObject {
	private static final long serialVersionUID = 5738058134152874529L;

	@XmlElement
	private String id;
	
	@XmlElement
	private String facilityCode;
	
	@XmlElement
	private PatientIdentifier patientIdentifier;

	@XmlElement
	private ProviderIdentifier providerIdentifier;
	
	@XmlElement
	private Note note;

	@XmlElement
	private Encounter encounter;
	
	@XmlElement
	private boolean encounterValid;

	@XmlElement
	private boolean encounterEdited;
	
	@XmlElement
	private String electronicSignatureCode;
	
	@XmlElement
	private Date lastModifiedDate;
	
	@XmlElement(required = true)
	private ClinicalEventState clinicalEventState = ClinicalEventState.DRAFT;

    @XmlElement
    private HealthFactors healthFactors;
	
	@Override
	public String toString() {
		return "ClinicalEvent [id=" + id + ", facilityCode=" + facilityCode
				+ ", patientIdentifier=" + patientIdentifier
				+ ", providerIdentifier=" + providerIdentifier + ", note="
				+ note + ", encounter=" + encounter + ", encounterValid="
				+ encounterValid + ", encounterEdited=" + encounterEdited
				+ ", electronicSignatureCode=" + electronicSignatureCode
				+ ", lastModifiedDate=" + lastModifiedDate
				+ ", clinicalEventState=" + clinicalEventState + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime
				* result
				+ ((clinicalEventState == null) ? 0 : clinicalEventState
						.hashCode());
		result = prime
				* result
				+ ((electronicSignatureCode == null) ? 0
						: electronicSignatureCode.hashCode());
		result = prime * result
				+ ((encounter == null) ? 0 : encounter.hashCode());
		result = prime * result + (encounterEdited ? 1231 : 1237);
		result = prime * result + (encounterValid ? 1231 : 1237);
		result = prime * result
				+ ((facilityCode == null) ? 0 : facilityCode.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime
				* result
				+ ((lastModifiedDate == null) ? 0 : lastModifiedDate.hashCode());
		result = prime * result + ((note == null) ? 0 : note.hashCode());
		result = prime
				* result
				+ ((patientIdentifier == null) ? 0 : patientIdentifier
						.hashCode());
		result = prime
				* result
				+ ((providerIdentifier == null) ? 0 : providerIdentifier
						.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ClinicalEvent other = (ClinicalEvent) obj;
		if (clinicalEventState != other.clinicalEventState)
			return false;
		if (electronicSignatureCode == null) {
			if (other.electronicSignatureCode != null)
				return false;
		} else if (!electronicSignatureCode
				.equals(other.electronicSignatureCode))
			return false;
		if (encounter == null) {
			if (other.encounter != null)
				return false;
		} else if (!encounter.equals(other.encounter))
			return false;
		if (encounterEdited != other.encounterEdited)
			return false;
		if (encounterValid != other.encounterValid)
			return false;
		if (facilityCode == null) {
			if (other.facilityCode != null)
				return false;
		} else if (!facilityCode.equals(other.facilityCode))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (lastModifiedDate == null) {
			if (other.lastModifiedDate != null)
				return false;
		} else if (!lastModifiedDate.equals(other.lastModifiedDate))
			return false;
		if (note == null) {
			if (other.note != null)
				return false;
		} else if (!note.equals(other.note))
			return false;
		if (patientIdentifier == null) {
			if (other.patientIdentifier != null)
				return false;
		} else if (!patientIdentifier.equals(other.patientIdentifier))
			return false;
		if (providerIdentifier == null) {
			if (other.providerIdentifier != null)
				return false;
		} else if (!providerIdentifier.equals(other.providerIdentifier))
			return false;
		return true;
	}
	
	public ClinicalEventState getClinicalEventState() {
		return clinicalEventState;
	}

	public void setClinicalEventState(ClinicalEventState clinicalEventState) {
		this.clinicalEventState = clinicalEventState;
	}

	public String getFacilityCode() {
		return facilityCode;
	}

	public void setFacilityCode(String facilityCode) {
		this.facilityCode = facilityCode;
	}
	
	public void setProviderIdentifier(ProviderIdentifier providerIdentifier) {
		this.providerIdentifier = providerIdentifier;
	}
	
	public ProviderIdentifier getProviderIdentifier() {
        synchronized(this) {
            if (providerIdentifier == null) {
                providerIdentifier = new ProviderIdentifier();
            }
        }

		return providerIdentifier;
	}
	
	public PatientIdentifier getPatientIdentifier() {
        synchronized(this) {
            if (patientIdentifier == null) {
                patientIdentifier = new PatientIdentifier();
            }
		}
		
		return patientIdentifier;
	}

	public void setPatientIdentifier(PatientIdentifier patientIdentifier) {
		this.patientIdentifier = patientIdentifier;
	}
	
	public boolean isEncounterValid() {
		return encounterValid;
	}

	public void setEncounterValid(boolean encounterValid) {
		this.encounterValid = encounterValid;
	}

	public boolean isEncounterEdited() {
		return encounterEdited;
	}

	public void setEncounterEdited(boolean encounterEdited) {
		this.encounterEdited = encounterEdited;
	}

	public Note getNote() {
		return note;
	}

	public void setNote(Note note) {
		this.note = note;
	}

	public Encounter getEncounter() {
		return encounter;
	}

	public void setEncounter(Encounter encounter) {
		this.encounter = encounter;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public Date getLastModifiedDate() {
		return lastModifiedDate;
	}

	public void setLastModifiedDate(Date lastModifiedDate) {
		this.lastModifiedDate = lastModifiedDate;
	}

    public HealthFactors getHealthFactors() {
        return healthFactors;
    }

    public void setHealthFactors(HealthFactors healthFactors) {
        this.healthFactors = healthFactors;
    }
}