package com.agilex.healthcare.mobilehealthplatform.domain;

import java.net.URI;
import java.util.Date;
import javax.xml.bind.annotation.*;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;
import com.agilex.healthcare.mobilehealthplatform.enumeration.ConsultStatus;
import com.agilex.healthcare.mobilehealthplatform.enumeration.InOutpatientCare;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.datemarshallers.DateTimeMarshaller;
import com.agilex.healthcare.mobilehealthplatform.AdapterCDATA;

@XmlAccessorType(value = XmlAccessType.NONE)
@XmlRootElement(name = "consult", namespace = Namespace.MobileHealthPlatform)
public class Consult extends PatientData {
	private static final long serialVersionUID = -4859740862225608396L;
	
	@XmlElement
	private Long consultId;
	
	@XmlElement
	@XmlJavaTypeAdapter(DateTimeMarshaller.class)
	private Date createdDate;
	
	@XmlElement
	private ConsultStatus status;	
	
	@XmlElement
	private String orderingProviderName;
	
	@XmlElement
	private String consultingProviderName;
	
	@XmlElement
	private String placeOfConsultation;
	
	@XmlElement
	@XmlJavaTypeAdapter(DateTimeMarshaller.class)
	private Date requestDate;
	
	@XmlElement
	private String requestServiceName;
	
	@XmlJavaTypeAdapter(AdapterCDATA.class)
	private String text;
	
	@XmlElement
	private Integer daysOnList;
	
	@XmlElement
	private String provisionalDiagnosis;
	
	@XmlElement
	private String urgency;
	
	@XmlElement
	private InOutpatientCare inOutpatientCare;
			
	@XmlElement
	private String title;

	@XmlElement
	private String patientName;
	
	@XmlElement
	private String patientIEN;
	
	@XmlElement
	private String consultIEN;
	
	@XmlElement
	private Long requestServiceSID;

    @XmlElement
    private String statusDescription;

    @XmlElement
    private Boolean statusIcon = false;
    
	public Long getConsultId() {
		return consultId;
	}
	public void setConsultId(Long consultId) {
		this.consultId = consultId;
	}
	
	public Date getCreatedDate() {
		return createdDate;
	}
	public void setCreatedDate(Date createdDate) {
		this.createdDate = createdDate;
		calculateDaysOnList();
	}
	private void calculateDaysOnList(){
		Date endOfCreatedDate = DateHelper.endOfDay(this.createdDate);
		Date today = DateHelper.getToday();
		Date endOfToday = DateHelper.endOfDay(today);
		int calculateDeltaInDays = DateHelper.calculateDeltaInDays(endOfCreatedDate, endOfToday);
		Integer daysBetweenTodayAndCreatedDate = Integer.valueOf(calculateDeltaInDays);
		this.setDaysOnList(daysBetweenTodayAndCreatedDate);
	}
	
	public ConsultStatus getStatus() {
		return status;
	}
	public void setStatus(ConsultStatus status) {
		this.status = status;
        updateStatusDescription();
	}

    public String getStatusDescription() { return this.statusDescription; }
    public void setStatusDescription(String statusDescription) {
        this.statusDescription = statusDescription;
    }

	public String getOrderingProviderName() {
		return orderingProviderName;
	}
	public void setOrderingProviderName(String orderingProviderName) {
		this.orderingProviderName = orderingProviderName;
	}
	
	public String getConsultingProviderName() {
		return consultingProviderName;
	}
	public void setConsultingProviderName(String consultingProviderName) {
		this.consultingProviderName = consultingProviderName;
	}
	
	public String getPlaceOfConsultation() {
		return placeOfConsultation;
	}
	public void setPlaceOfConsultation(String placeOfConsultation) {
		this.placeOfConsultation = placeOfConsultation;
	}
	
	public Date getRequestDate() {
		return requestDate;
	}
	public void setRequestDate(Date requestDate) {
		this.requestDate = requestDate;
	}
	
	public String getRequestServiceName() {
		return requestServiceName;
	}
	public void setRequestServiceName(String requestServiceName) {
		this.requestServiceName = requestServiceName;
	}
	
	public String getText() {
		return text;
	}

	public void setText(String text) {
		this.text = text;
	}
	
	public Integer getDaysOnList() {
		if (daysOnList == null && getCreatedDate() != null){
			this.calculateDaysOnList();
		}
		return daysOnList;
	}
	public void setDaysOnList(Integer daysOnList) {
		this.daysOnList = daysOnList;
	}
	
	public String getProvisionalDiagnosis() {
		return provisionalDiagnosis;
	}
	public void setProvisionalDiagnosis(String provisionalDiagnosis) {
		this.provisionalDiagnosis = provisionalDiagnosis;
	}
	
	public InOutpatientCare getInOutpatientCare() {
		return inOutpatientCare;
	}
	public void setInOutpatientCare(InOutpatientCare inOutpatientCare) {
		this.inOutpatientCare = inOutpatientCare;
	}
	
	public String getUrgency() {
		return urgency;
	}
	public void setUrgency(String urgency) {
		this.urgency = urgency;
	}
	
	public String getTitle() {
		return title;
	}
	public void setTitle(String title) {
		this.title = title;
	}
	
	public String getPatientName() {
		return patientName;
	}
	public void setPatientName(String patientName) {
		this.patientName = patientName;
	}

	public String getPatientIEN() {
		return patientIEN;
	}
	public void setPatientIEN(String patientIEN) {
		this.patientIEN = patientIEN;
	}
	
	public String getConsultIEN() {
		return consultIEN;
	}
	public void setConsultIEN(String consultIEN) {
		this.consultIEN = consultIEN;
	}

	public AtomLink getConsultDetailUri() {
		return getLink().getLinkByTitle(LinkTitles.PatientConsultDetail);
	}
	public void setConsultDetailUri(URI uri) {
		getLink().addRelatedLink(LinkTitles.PatientConsultDetail, uri);
	}
	public Long getRequestServiceSID() {
		return requestServiceSID;
	}

	public void setRequestServiceSID(Long toRequestServiceSID) {
		this.requestServiceSID = toRequestServiceSID;
	}

    public Boolean getStatusIcon() {
        return statusIcon;
    }

    public void setStatusIcon(Boolean statusIcon) {
        this.statusIcon = statusIcon;
    }

    private void updateStatusDescription()  {
        String description = "";
        Boolean iconStatus = false;
        if(status != ConsultStatus.CANCELLED && status != ConsultStatus.DISCONTINUED)  {
            if ((daysOnList >= 8 && daysOnList <= 37) &&
                (status != ConsultStatus.SCHEDULED && status != ConsultStatus.COMPLETE)) {
                description = " - Past Due for Scheduling";
                iconStatus = true;
            }
            else if ((daysOnList >= 38 ) && status != ConsultStatus.COMPLETE) {
                description = " - Past Due for Completion";
                iconStatus = true;
            }
        }
        descriptionAndIconStatus(description, status, iconStatus);
    }

    private void descriptionAndIconStatus(String description, ConsultStatus _consultStatus, Boolean statusIcon) {
        setStatusDescription(new StringBuffer().append((status == ConsultStatus.CANCELLED) ? "CANCELED" : status.name()).
                append(description).toString());
        setStatusIcon(statusIcon);
    }
}
