package com.agilex.healthcare.mobilehealthplatform.domain;

import java.util.Date;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.agilex.healthcare.notewriter.domain.EncounterDiagnosisICDCodes;
import com.agilex.healthcare.notewriter.domain.EncounterProcedureCPTCodes;

@XmlRootElement(name = "encounter", namespace = Namespace.MobileHealthPlatform)
@XmlAccessorType(value = XmlAccessType.NONE)
public class Encounter extends DomainTransferObject {
	private static final long serialVersionUID = 6118839982260602204L;

	@XmlElement
	private Date timestamp;
	
	/**
	 * Date is used by the UI only for saving and resuming the text for new appointments - it is not sent to mdws.
	 * This is because they need the ability to save without having filled in a date or a time and they are two
	 * separate entities within the UI (and javascript can't create a date object without having both entities populated).
	 */
	@XmlElement
	private String date;
	
	/**
	 * Time is used by the UI only for saving and resuming the text for new appointments - it is not sent to mdws.
	 * This is because they need the ability to save without having filled in a date or a time and they are two
	 * separate entities within the UI (and javascript can't create a date object without having both entities populated).
	 */
	@XmlElement
	private String time;
	
	@XmlElement
	private String locationId;
	
	@XmlElement
	private String locationTitle;
	
	@XmlElement
	private boolean currentAdmission;
	
	@XmlElement
	private boolean isHistorical;

	@XmlElement
	private String linkType;

	// ServiceCategory (A for clinic encounters/I if inpatient, H for admission, E for historical)
	@XmlElement
	private String serviceCategory;

	// AppointmentType (if appointment, primary or mental health I'm guessing)
	@XmlElement
	private String appointmentType;
	
	@XmlElement
	private String encounterType;
	
	@XmlElement
	private VisitRelatedTos visitRelatedTos;
	
	@XmlElement
	private EncounterProviders providers;
	
	@XmlElement
	private VisitTypes visitTypes;
	
	@XmlElement
	private EncounterProcedureCPTCodes procedureCodes;
	
	@XmlElement
	private EncounterDiagnosisICDCodes diagnosisCodes;
	
	@Override
	public String toString() {
		return "Encounter [timestamp=" + timestamp + ", date=" + date + ", time=" + time + ", locationId="
				+ locationId + ", locationTitle=" + locationTitle + ", currentAdmission=" + currentAdmission + ", isHistorical="
				+ isHistorical + ", linkType=" + linkType + ", serviceCategory=" + serviceCategory + ", appointmentType=" + appointmentType
				+ ", encounterType=" + encounterType + ", visitRelatedTos=" + visitRelatedTos + ", providers=" + providers
				+ ", visitTypes=" + visitTypes + ", procedureCodes=" + procedureCodes + ", diagnosisCodes=" + diagnosisCodes + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((appointmentType == null) ? 0 : appointmentType.hashCode());
		result = prime * result + (currentAdmission ? 1231 : 1237);
		result = prime * result + ((date == null) ? 0 : date.hashCode());
		result = prime * result + ((diagnosisCodes == null) ? 0 : diagnosisCodes.hashCode());
		result = prime * result + ((encounterType == null) ? 0 : encounterType.hashCode());
		result = prime * result + (isHistorical ? 1231 : 1237);
		result = prime * result + ((linkType == null) ? 0 : linkType.hashCode());
		result = prime * result + ((locationId == null) ? 0 : locationId.hashCode());
		result = prime * result + ((locationTitle == null) ? 0 : locationTitle.hashCode());
		result = prime * result + ((procedureCodes == null) ? 0 : procedureCodes.hashCode());
		result = prime * result + ((providers == null) ? 0 : providers.hashCode());
		result = prime * result + ((serviceCategory == null) ? 0 : serviceCategory.hashCode());
		result = prime * result + ((time == null) ? 0 : time.hashCode());
		result = prime * result + ((timestamp == null) ? 0 : timestamp.hashCode());
		result = prime * result + ((visitRelatedTos == null) ? 0 : visitRelatedTos.hashCode());
		result = prime * result + ((visitTypes == null) ? 0 : visitTypes.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Encounter other = (Encounter) obj;
		if (appointmentType == null) {
			if (other.appointmentType != null)
				return false;
		}
		else if (!appointmentType.equals(other.appointmentType))
			return false;
		if (currentAdmission != other.currentAdmission)
			return false;
		if (date == null) {
			if (other.date != null)
				return false;
		}
		else if (!date.equals(other.date))
			return false;
		if (diagnosisCodes == null) {
			if (other.diagnosisCodes != null)
				return false;
		}
		else if (!diagnosisCodes.equals(other.diagnosisCodes))
			return false;
		if (encounterType == null) {
			if (other.encounterType != null)
				return false;
		}
		else if (!encounterType.equals(other.encounterType))
			return false;
		if (isHistorical != other.isHistorical)
			return false;
		if (linkType == null) {
			if (other.linkType != null)
				return false;
		}
		else if (!linkType.equals(other.linkType))
			return false;
		if (locationId == null) {
			if (other.locationId != null)
				return false;
		}
		else if (!locationId.equals(other.locationId))
			return false;
		if (locationTitle == null) {
			if (other.locationTitle != null)
				return false;
		}
		else if (!locationTitle.equals(other.locationTitle))
			return false;
		if (procedureCodes == null) {
			if (other.procedureCodes != null)
				return false;
		}
		else if (!procedureCodes.equals(other.procedureCodes))
			return false;
		if (providers == null) {
			if (other.providers != null)
				return false;
		}
		else if (!providers.equals(other.providers))
			return false;
		if (serviceCategory == null) {
			if (other.serviceCategory != null)
				return false;
		}
		else if (!serviceCategory.equals(other.serviceCategory))
			return false;
		if (time == null) {
			if (other.time != null)
				return false;
		}
		else if (!time.equals(other.time))
			return false;
		if (timestamp == null) {
			if (other.timestamp != null)
				return false;
		}
		else if (!timestamp.equals(other.timestamp))
			return false;
		if (visitRelatedTos == null) {
			if (other.visitRelatedTos != null)
				return false;
		}
		else if (!visitRelatedTos.equals(other.visitRelatedTos))
			return false;
		if (visitTypes == null) {
			if (other.visitTypes != null)
				return false;
		}
		else if (!visitTypes.equals(other.visitTypes))
			return false;
		return true;
	}

	public Date getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(Date timestamp) {
		this.timestamp = timestamp;
	}

	/**
	 * Date is used by the UI only for saving and resuming the text for new appointments - it is not sent to mdws.
	 * This is because they need the ability to save without having filled in a date or a time and they are two
	 * separate entities within the UI (and javascript can't create a date object without having both entities populated).
	 */
	public String getDate() {
		return date;
	}

	/**
	 * Date is used by the UI only for saving and resuming the text for new appointments - it is not sent to mdws.
	 * This is because they need the ability to save without having filled in a date or a time and they are two
	 * separate entities within the UI (and javascript can't create a date object without having both entities populated).
	 */
	public void setDate(String date) {
		this.date = date;
	}

	/**
	 * Time is used by the UI only for saving and resuming the text for new appointments - it is not sent to mdws.
	 * This is because they need the ability to save without having filled in a date or a time and they are two
	 * separate entities within the UI (and javascript can't create a date object without having both entities populated).
	 */
	public String getTime() {
		return time;
	}

	/**
	 * Time is used by the UI only for saving and resuming the text for new appointments - it is not sent to mdws.
	 * This is because they need the ability to save without having filled in a date or a time and they are two
	 * separate entities within the UI (and javascript can't create a date object without having both entities populated).
	 */
	public void setTime(String time) {
		this.time = time;
	}

	public String getServiceCategory() {
		return serviceCategory;
	}

	public void setServiceCategory(String serviceCategory) {
		this.serviceCategory = serviceCategory;
	}

	public String getAppointmentType() {
		return appointmentType;
	}

	public void setAppointmentType(String appointmentType) {
		this.appointmentType = appointmentType;
	}

	public List<VisitRelatedTo> getVisitRelatedTos() {
		return visitRelatedTos;
	}

	public void setVisitRelatedTos(VisitRelatedTos visitRelatedTos) {
		this.visitRelatedTos = visitRelatedTos;
	}

	public List<EncounterProvider> getProviders() {
		return providers;
	}

	public void setProviders(EncounterProviders providers) {
		this.providers = providers;
	}

	public String getLocationId() {
		return locationId;
	}

	public void setLocationId(String locationId) {
		this.locationId = locationId;
	}

	public String getLocationTitle() {
		return locationTitle;
	}

	public void setLocationTitle(String locationTitle) {
		this.locationTitle = locationTitle;
	}
	
	public String getLinkType() {
		return linkType;
	}

	public void setLinkType(String linkType) {
		this.linkType = linkType;
	}
	
	public boolean isCurrentAdmission() {
		return currentAdmission;
	}

	public void setCurrentAdmission(boolean currentAdmission) {
		this.currentAdmission = currentAdmission;
	}

	public boolean isHistorical() {
		return isHistorical;
	}

	public void setHistorical(boolean isHistorical) {
		this.isHistorical = isHistorical;
	}

	public String getEncounterType() {
		return encounterType;
	}

	public void setEncounterType(String encounterType) {
		this.encounterType = encounterType;
	}

	public EncounterProcedureCPTCodes getProcedureCodes() {
		return procedureCodes;
	}

	public void setProcedureCodes(EncounterProcedureCPTCodes procedureCodes) {
		this.procedureCodes = procedureCodes;
	}

	public VisitTypes getVisitTypes() {
		return visitTypes;
	}

	public void setVisitTypes(VisitTypes visitTypes) {
		this.visitTypes = visitTypes;
	}

	public EncounterDiagnosisICDCodes getDiagnosisCodes() {
		return diagnosisCodes;
	}

	public void setDiagnosisCodes(EncounterDiagnosisICDCodes diagnosisCodes) {
		this.diagnosisCodes = diagnosisCodes;
	}

}