package com.agilex.healthcare.mobilehealthplatform.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

@XmlRootElement(name = "encounterProvider")
@XmlAccessorType(value = XmlAccessType.NONE)
public class EncounterProvider implements Comparable<EncounterProvider> {

	@XmlElement
	private FacilityProvider facilityProvider;
	
	@XmlElement
	private boolean primary = false;
	
	/**
	 * For resume, this is used to display the selected and unselected providers.
	 * For sign/submit only providers with selected: true should be sent to MDWS
	 */
	@XmlElement
	private boolean selected = false;

	public EncounterProvider() {
		
	}
	
	public EncounterProvider(FacilityProvider facilityProvider) {
		this.facilityProvider = facilityProvider;
	}
	
	@Override
	public String toString() {
		return "EncounterProvider [facilityProvider=" + facilityProvider + ", primary=" + primary + ", selected=" + selected + "]";
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((facilityProvider == null) ? 0 : facilityProvider.hashCode());
		result = prime * result + (primary ? 1231 : 1237);
		result = prime * result + (selected ? 1231 : 1237);
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		EncounterProvider other = (EncounterProvider) obj;
		if (facilityProvider == null) {
			if (other.facilityProvider != null)
				return false;
		}
		else if (!facilityProvider.equals(other.facilityProvider))
			return false;
		if (primary != other.primary)
			return false;
		if (selected != other.selected)
			return false;
		return true;
	}

	public FacilityProvider getProvider() {
		return facilityProvider;
	}

	public void setProvider(FacilityProvider facilityProvider) {
		this.facilityProvider = facilityProvider;
	}

	public boolean isPrimary() {
		return primary;
	}

	public void setPrimary(boolean primary) {
		this.primary = primary;
	}

	@Override
	public int compareTo(EncounterProvider that) {
		if (this.getProvider() == null && that.getProvider() == null) {
			return 0;
		} else if (this.getProvider() == null) {
			return -1;
		} else if (that.getProvider() == null) {
			return 1;
		} else {			
			return this.getProvider().getProviderName().compareTo(that.getProvider().getProviderName());
		}
	}

	public boolean isSelected() {
		return selected;
	}

	public void setSelected(boolean selected) {
		this.selected = selected;
	}
}
