package com.agilex.healthcare.mobilehealthplatform.domain;

import com.agilex.healthcare.utility.NullChecker;

import javax.xml.bind.annotation.*;
import java.net.URI;
import java.util.Date;

@XmlAccessorType(value = XmlAccessType.NONE)
@XmlRootElement(name = "labtest", namespace = Namespace.MobileHealthPlatform)
public class LabTest extends PatientData implements Comparable<LabTest> {

	private static final long serialVersionUID = 7975019513233324247L;

	@XmlElement
	private String specimenId = null;
	@XmlElement
	private String specimenName = null;

	private String testname = null;
	@XmlElement
	private String loinc = "";
	@XmlElement
	private String testDescription = null;

	@XmlElement
	private String ien = null;
	@XmlElement
	private Date mostCurrentResultDate = null;
	@XmlElement
	private String accessionAreaIen = null;
	@XmlElement
	private String accessionAreaName = null;
	@XmlElement
	private String accessionSubscript = null;
	@XmlElement
	private String siteCode = null;

	private int sortOrder = Integer.MAX_VALUE;

	@XmlElement(name = "labResult", namespace = Namespace.MobileHealthPlatform)
	private LabResult mostRecentLabResult = null;

	@XmlElement(name = "labResult")
	private LabResults labResults = new LabResults();

	@XmlAttribute
	public String getTestname() {
		return getDisplayName();
	}

	public void setTestname(String testname) {
		this.testname = testname;
	}

	public void setMostRecentLabResult(LabResult mostRecentLabResult) {
		this.mostRecentLabResult = mostRecentLabResult;
	}

	public LabResult getMostRecentLabResult() {
		return mostRecentLabResult;
	}

	public void setTestLabResultsUri(URI uri) {
		getLink().addRelatedLink(LinkTitles.PatientChemHemLabResultsForTest, uri);
	}

	public AtomLink getTestLabResultsLink() {
		return getLink().getLinkByTitle(LinkTitles.PatientChemHemLabResultsForTest);
	}

	public URI getTestLabResultsUri() {
		return AtomLink.extractUri(getTestLabResultsLink());
	}

	public void setSpecimenId(String specimen) {
		this.specimenId = specimen;
	}

	public String getSpecimenId() {
		return specimenId;
	}

	public String getSpecimenName() {
		return specimenName;
	}

	public void setSpecimenName(String specimenName) {
		this.specimenName = specimenName;
	}

	@XmlAttribute
	public String getDisplayName() {
		String displayName = testname;

		if (NullChecker.isNotNullish(specimenName)) {
			return displayName + " (" + specimenName + ")";
		}

		return displayName;
	}

	public LabResults getLabResults() {
		return labResults;
	}

	public void setLabResults(LabResults labResults) {
		this.labResults = labResults;
	}

	public String getTestDescription() {
		return testDescription;
	}

	public void setTestDescription(String testDescription) {
		this.testDescription = testDescription;
	}

	public String getLoinc() {
		return loinc;
	}

	public void setLoinc(String loinc) {
		this.loinc = loinc;
	}

	public String getIen() {
		return ien;
	}

	public void setIen(String ien) {
		this.ien = ien;
	}

	public Date getMostCurrentResultDate() {
		return mostCurrentResultDate;
	}

	public void setMostCurrentResultDate(Date mostCurrentResultDate) {
		this.mostCurrentResultDate = mostCurrentResultDate;
	}

	public String getAccessionAreaIen() {
		return accessionAreaIen;
	}

	public void setAccessionAreaIen(String accessionAreaIen) {
		this.accessionAreaIen = accessionAreaIen;
	}

	public String getAccessionAreaName() {
		return accessionAreaName;
	}

	public void setAccessionAreaName(String accessionAreaName) {
		this.accessionAreaName = accessionAreaName;
	}

	public String getAccessionSubscript() {
		return accessionSubscript;
	}

	public void setAccessionSubscript(String accessionSubscript) {
		this.accessionSubscript = accessionSubscript;
	}

	public String getSiteCode() {
		return siteCode;
	}

	public void setSiteCode(String siteCode) {
		this.siteCode = siteCode;
	}

	public String getGroupingId() {
		if (NullChecker.isNotNullish(getLoinc()))
			return getLoinc();
		else if (NullChecker.isNotNullish(getUniqueId()) && NullChecker.isNotNullish(getSpecimenName()))
			return getUniqueId() + "-" + getSpecimenName();

		return testname;
	}

	@Override
	public int compareTo(LabTest that) {
		int result = this.testname.compareTo(that.testname);
		if (result == 0) {
			result = this.getLabResults().get(0).getResultedDate().compareTo(that.getLabResults().get(0).getResultedDate()) * -1;
		}

		return result;
	}

	public int getSortOrder() {
		return sortOrder;
	}

	public void setSortOrder(int sortOrder) {
		this.sortOrder = sortOrder;
	}
}
