package com.agilex.healthcare.mobilehealthplatform.domain;

import com.agilex.healthcare.mobilehealthplatform.domain.atom.rel;
import com.agilex.healthcare.mobilehealthplatform.domain.sorting.PatientSortingAlgorithm;
import com.agilex.healthcare.utility.DOBDateMarshaller;
import com.agilex.healthcare.utility.NullChecker;
import com.agilex.healthcare.utility.datemarshallers.DateTimeMarshaller;

import javax.xml.bind.annotation.*;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;
import java.net.URI;
import java.util.Date;

/**
 * 
 * @author j.ray
 */
@XmlRootElement(name = "patient", namespace = Namespace.MobileHealthPlatform)
@XmlAccessorType(value = XmlAccessType.NONE)
public class Patient extends DomainTransferObject implements Comparable<Patient> {

	private static final long serialVersionUID = -3040524557861665627L;

	private String displayName = null;
	@XmlElement
	private String firstName;
	@XmlElement
	private String lastName;
	@XmlElement
	private String middleName;
	@XmlElement
	@XmlJavaTypeAdapter(DOBDateMarshaller.class)
	private DOBDate dateOfBirth;
	@XmlElement
	private String gender;
	@XmlElement
	private PatientIdentifier patientIdentifier = new PatientIdentifier();
	@XmlElement
	private String ssn;
	@XmlElement
	private String middleInitial;
	@XmlElement
	private String age = null;
	@XmlElement
	private String wardLocation = null;
	@XmlElement
	private String roombed = null;
	@XmlElement
	private String enterpriseId;
	@XmlElement
	private boolean inpatient;
	@XmlElement
	private boolean textMessagingAllowed;
	@XmlElement
	private String textMessagingPhoneNumber;
    @XmlElement
    private String locationIEN;
    @XmlElement
    private String cwad;
    @XmlElement
    private String icn;
    @XmlElement
    private boolean serviceConnected;
    @XmlElement
    private String serviceConnectedPercent;
    @XmlElement
    private boolean sensitive;
    @XmlElement
    @XmlJavaTypeAdapter(DateTimeMarshaller.class)
    private Date admittedDateTime;

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return this.lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getSsn() {
		return this.ssn;
	}

	public void setSsn(String ssn) {
		this.ssn = ssn;
	}

	public String getMiddleName() {
		return middleName;
	}

	public void setMiddleName(String middleName) {
		this.middleName = middleName;
	}

	public DOBDate getDateOfBirth() {
		return dateOfBirth;
	}

	public void setDateOfBirth(DOBDate dateOfBirth) {
		this.dateOfBirth = dateOfBirth;
	}

	public String getGender() {
		return gender;
	}

	public void setGender(String gender) {
		this.gender = gender;
	}

	@XmlTransient
	public URI getDemographicsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientDemographics);
		return AtomLink.extractUri(link);
	}

	public void setDemographicsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientDemographics);
		getLink().setLinkByTitle(link);
	}

	public String getId() {
		return getPatientIdentifier().getUniqueId();
	}

	public void setId(String id) {
		getPatientIdentifier().setUniqueId(id);
	}

	public String getWardLocation() {
		return wardLocation;
	}

	public void setWardLocation(String wardLocation) {
		this.wardLocation = wardLocation;
	}

	public String getRoombed() {
		return roombed;
	}

	public void setRoombed(String roombed) {
		this.roombed = roombed;
	}

	public URI getPatientImageUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientImage);
		return AtomLink.extractUri(link);
	}

	public void setPatientImageUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientImage);
		getLink().setLinkByTitle(link);
	}

	public URI getProblemsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientProblems);
		return AtomLink.extractUri(link);
	}

	public void setProblemsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientProblems);
		getLink().setLinkByTitle(link);
	}

	public void setOutpatientProblemsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientOutpatientProblems);
		getLink().setLinkByTitle(link);
	}

	public void setInpatientProblemsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientInpatientProblems);
		getLink().setLinkByTitle(link);
	}

	public void setMiddleInitial(String middleInitial) {
		this.middleInitial = middleInitial;
	}

	public String getMiddleInitial() {
		return middleInitial;
	}

	public URI getAllergiesUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientAllergies);
		return AtomLink.extractUri(link);
	}

	public void setAllergiesUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientAllergies);
		getLink().setLinkByTitle(link);
	}

	public URI Surgeries() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientSurgeries);
		return AtomLink.extractUri(link);
	}

	public void setSurgeriesUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientSurgeries);
		getLink().setLinkByTitle(link);
	}

	public void setConsultsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientConsults);
		getLink().setLinkByTitle(link);
	}

	public void setImmunizationUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.ImmunizationCampaign);
		getLink().setLinkByTitle(link);
	}

	public URI getVitalsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientVitals);
		return AtomLink.extractUri(link);
	}
	
	public URI getLongitudinalVitalsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientVitalsLongitudinal);
		return AtomLink.extractUri(link);
	}
	
	public URI getOperationalVitalsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientVitalsOperational);
		return AtomLink.extractUri(link);
	}
	
	public URI getProviderEnteredVitalsTableDataUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientVitalsTableLongitudinal);
		return AtomLink.extractUri(link);
	}

	public URI getProviderEnteredVitalsGraphDataUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientVitalsGraphLongitudinal);
		return AtomLink.extractUri(link);
	}

	public void setAge(String age) {
		this.age = age;
	}

	public String getAge() {
		return age;
	}

	public void setDisplayName(String displayName) {
		this.displayName = displayName;
	}

	@XmlElement
	public String getDisplayName() {
		if(NullChecker.isNullish(displayName)){
			displayName=NullChecker.isNullish(middleName)?lastName + ", " + firstName :lastName + ", " + firstName+" "+middleName;
		}
		return displayName;
	}

	@Override
	public int compareTo(Patient patient) {
		PatientSortingAlgorithm algorithm = new PatientSortingAlgorithm();
		return algorithm.compare(this, patient);
	}

	public void setPatientIdentifier(PatientIdentifier patientIdentifier) {
		this.patientIdentifier = patientIdentifier;
	}

	public PatientIdentifier getPatientIdentifier() {
		if (patientIdentifier == null)
			patientIdentifier = new PatientIdentifier();
		return patientIdentifier;
	}

	public static String serializePatientIdentifier(Patient patient) {
		String s = null;
		if (!(patient == null))
			s = patient.getPatientIdentifier().toString();
		return s;
	}

	public void setEnterpriseId(String enterpriseId) {
		this.enterpriseId = enterpriseId;
	}

	public String getEnterpriseId() {
		return enterpriseId;
	}

	public boolean isInpatient() {
		return inpatient;
	}

	public void setInpatient(boolean inpatient) {
		this.inpatient = inpatient;
	}

	public URI getMicroLabResultsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientMicrobiologyLabResults);
		return AtomLink.extractUri(link);
	}

	public void setMicroLabResultsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientMicrobiologyLabResults);
		getLink().setLinkByTitle(link);
	}

	public URI getMicroLabTestHistoryUri() {
		return getLink().getUriByTitle(LinkTitles.PatientMicrobiologyLabTestHistory);
	}

	public void setMicroLabTestHistoryUri(URI uri) {
		getLink().addRelatedLink(LinkTitles.PatientMicrobiologyLabTestHistory, uri);
	}

	public URI getMicroLabTestHistoryByGroupUri() {
		return getLink().getUriByTitle(LinkTitles.PatientMicrobiologyLabTestHistoryByGroup);
	}

	public void setMicroLabTestHistoryByGroupUri(URI uri) {
		getLink().addRelatedLink(LinkTitles.PatientMicrobiologyLabTestHistoryByGroup, uri);
	}

	public URI getMedicationsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientMedications);
		return AtomLink.extractUri(link);
	}

	public void setMedicationsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientMedications);
		getLink().setLinkByTitle(link);
	}

	public URI getActiveMedicationsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientActiveMedications);
		return AtomLink.extractUri(link);
	}

	public void setActiveMedicationsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientActiveMedications);
		getLink().setLinkByTitle(link);
	}

	public URI getMedicationRefillsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientMedicationRefills);
		return AtomLink.extractUri(link);
	}

	public void setMedicationRefillsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientMedicationRefills);
		getLink().setLinkByTitle(link);
	}

	public URI getOutpatientMedicationsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientOutpatientMedications);
		return AtomLink.extractUri(link);
	}

	public void setOutpatientMedicationsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientOutpatientMedications);
		getLink().setLinkByTitle(link);
	}

	public URI getCurrentOutpatientMedicationsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientActiveOutpatientMedications);
		return AtomLink.extractUri(link);
	}

	public void setCurrentOutpatientMedicationsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientActiveOutpatientMedications);
		getLink().setLinkByTitle(link);
	}

	public URI getInpatientMedicationsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientInpatientMedications);
		return AtomLink.extractUri(link);
	}

	public void setInpatientMedicationsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientInpatientMedications);
		getLink().setLinkByTitle(link);
	}

	public URI getCurrentInpatientMedicationsUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientActiveInpatientMedications);
		return AtomLink.extractUri(link);
	}

	public void setCurrentInpatientMedicationsUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientActiveInpatientMedications);
		getLink().setLinkByTitle(link);
	}
	
	public URI getMentalHealthAppointmentUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientMentalHealthAppointments);
		return AtomLink.extractUri(link);
	}
	
	public URI getPatientOrdersUri() {
		AtomLink link = getLink().getLinkByTitle(LinkTitles.PatientOrders);
		return AtomLink.extractUri(link);
	}
	
	public void setPatientOrdersUri(URI uri) {
		AtomLink link = new AtomLink();
		link.setHref(uri);
		link.setRel(rel.related);
		link.setTitle(LinkTitles.PatientOrders);
		getLink().setLinkByTitle(link);
	}
	
	public boolean isTextMessagingAllowed() {
		return textMessagingAllowed;
	}

	public void setTextMessagingAllowed(boolean textMessagingAllowed) {
		this.textMessagingAllowed = textMessagingAllowed;
	}

	public String getTextMessagingPhoneNumber() {
		return textMessagingPhoneNumber;
	}

	public void setTextMessagingPhoneNumber(String textMessagingPhoneNumber) {
		this.textMessagingPhoneNumber = textMessagingPhoneNumber;
	}

    public String getLocationIEN() {
        return locationIEN;
    }

    public void setLocationIEN(String locationIEN) {
        this.locationIEN = locationIEN;
    }

    public String getCwad() {
        return cwad;
    }

    public void setCwad(String cwad) {
        this.cwad = cwad;
    }

    public String getIcn() {
        return icn;
    }

    public void setIcn(String icn) {
        this.icn = icn;
    }

    public boolean isServiceConnected() {
        return serviceConnected;
    }

    public void setServiceConnected(boolean serviceConnected) {
        this.serviceConnected = serviceConnected;
    }

    public String getServiceConnectedPercent() {
        return serviceConnectedPercent;
    }

    public void setServiceConnectedPercent(String serviceConnectedPercent) {
        this.serviceConnectedPercent = serviceConnectedPercent;
    }

    public boolean isSensitive() {
        return sensitive;
    }

    public void setSensitive(boolean sensitive) {
        this.sensitive = sensitive;
    }

    public Date getAdmittedDateTime() {
        return admittedDateTime;
    }

    public void setAdmittedDateTime(Date admittedDateTime) {
        this.admittedDateTime = admittedDateTime;
    }
}