package com.agilex.healthcare.mobilehealthplatform.domain;

import java.util.HashSet;
import java.util.Set;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.agilex.healthcare.utility.NullChecker;

@XmlRootElement(name = "provider")
@XmlAccessorType(value = XmlAccessType.NONE)
public class Provider {

	@XmlElement
	private ProviderIdentifier providerIdentifier;
	
	@XmlElement
	protected String lastName;
	@XmlElement
	protected String firstName;
	@XmlElement
	protected String middleInitials;
	private Set<String> vistaLocations;
	@XmlElement
	private String title;
	private String cellPhone;
	private String officePhone;
	private String voicePager;
	private String digitalPager;
	private String emailAddress;
	private String officeLocation;
	
	public Provider() {
	}
	
	public Provider(ProviderIdentifier providerIdentifier) {
		this.providerIdentifier = providerIdentifier;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((cellPhone == null) ? 0 : cellPhone.hashCode());
		result = prime * result + ((digitalPager == null) ? 0 : digitalPager.hashCode());
		result = prime * result + ((emailAddress == null) ? 0 : emailAddress.hashCode());
		result = prime * result + ((firstName == null) ? 0 : firstName.hashCode());
		result = prime * result + ((lastName == null) ? 0 : lastName.hashCode());
		result = prime * result + ((middleInitials == null) ? 0 : middleInitials.hashCode());
		result = prime * result + ((officeLocation == null) ? 0 : officeLocation.hashCode());
		result = prime * result + ((officePhone == null) ? 0 : officePhone.hashCode());
		result = prime * result + ((providerIdentifier == null) ? 0 : providerIdentifier.hashCode());
		result = prime * result + ((title == null) ? 0 : title.hashCode());
		result = prime * result + ((vistaLocations == null) ? 0 : vistaLocations.hashCode());
		result = prime * result + ((voicePager == null) ? 0 : voicePager.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Provider other = (Provider) obj;
		if (cellPhone == null) {
			if (other.cellPhone != null)
				return false;
		}
		else if (!cellPhone.equals(other.cellPhone))
			return false;
		if (digitalPager == null) {
			if (other.digitalPager != null)
				return false;
		}
		else if (!digitalPager.equals(other.digitalPager))
			return false;
		if (emailAddress == null) {
			if (other.emailAddress != null)
				return false;
		}
		else if (!emailAddress.equals(other.emailAddress))
			return false;
		if (firstName == null) {
			if (other.firstName != null)
				return false;
		}
		else if (!firstName.equals(other.firstName))
			return false;
		if (lastName == null) {
			if (other.lastName != null)
				return false;
		}
		else if (!lastName.equals(other.lastName))
			return false;
		if (middleInitials == null) {
			if (other.middleInitials != null)
				return false;
		}
		else if (!middleInitials.equals(other.middleInitials))
			return false;
		if (officeLocation == null) {
			if (other.officeLocation != null)
				return false;
		}
		else if (!officeLocation.equals(other.officeLocation))
			return false;
		if (officePhone == null) {
			if (other.officePhone != null)
				return false;
		}
		else if (!officePhone.equals(other.officePhone))
			return false;
		if (providerIdentifier == null) {
			if (other.providerIdentifier != null)
				return false;
		}
		else if (!providerIdentifier.equals(other.providerIdentifier))
			return false;
		if (title == null) {
			if (other.title != null)
				return false;
		}
		else if (!title.equals(other.title))
			return false;
		if (vistaLocations == null) {
			if (other.vistaLocations != null)
				return false;
		}
		else if (!vistaLocations.equals(other.vistaLocations))
			return false;
		if (voicePager == null) {
			if (other.voicePager != null)
				return false;
		}
		else if (!voicePager.equals(other.voicePager))
			return false;
		return true;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getMiddleInitials() {
		return middleInitials;
	}

	public void setMiddleInitials(String middleInitials) {
		this.middleInitials = middleInitials;
	}

	public String getDisplayName() {
		StringBuilder sb = new StringBuilder();
		sb.append(firstName);
		if (NullChecker.isNotNullish(middleInitials)) {
			sb.append(" ");
			sb.append(middleInitials);
		}
		sb.append(" ");
		sb.append(lastName);
		
		return sb.toString();
	}
	
    public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getCellPhone() {
		return cellPhone;
	}

	public void setCellPhone(String cellPhone) {
		this.cellPhone = cellPhone;
	}

	public String getOfficePhone() {
		return officePhone;
	}

	public void setOfficePhone(String officePhone) {
		this.officePhone = officePhone;
	}

	public String getVoicePager() {
		return voicePager;
	}

	public void setVoicePager(String voicePager) {
		this.voicePager = voicePager;
	}

	public String getDigitalPager() {
		return digitalPager;
	}

	public void setDigitalPager(String digitalPager) {
		this.digitalPager = digitalPager;
	}

	public String getEmailAddress() {
		return emailAddress;
	}

	public void setEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
	}

	public Set<String> getVistaLocations() {
		if (this.vistaLocations == null) {
			this.vistaLocations = new HashSet<String>();
		}
		return vistaLocations;
	}

	public void setVistaLocations(Set<String> vistaLocations) {
		this.vistaLocations = vistaLocations;
	}

	public void addVistaLocation(String vistaLocation) {
		if (this.vistaLocations == null) {
			this.vistaLocations = new HashSet<String>();
		}
		this.vistaLocations.add(vistaLocation);
	}
	
	public void removeVistaLocation(String vistaLocation) {
		if (this.vistaLocations != null) {
			this.vistaLocations.remove(vistaLocation);	
		}
	}

	public String getOfficeLocation() {
		return officeLocation;
	}

	public void setOfficeLocation(String officeLocation) {
		this.officeLocation = officeLocation;
	}

	public ProviderIdentifier getProviderIdentifier() {
		return providerIdentifier;
	}

	public void setProviderIdentifier(ProviderIdentifier providerIdentifier) {
		this.providerIdentifier = providerIdentifier;
	}
}
