package com.agilex.healthcare.mobilehealthplatform.enumeration;

import java.util.HashMap;
import java.util.Map;

/**
 * The status a Consult can be in: <br/>
 * ACTIVE<br/>
 * PENDING<br/>
 * HOLD<br/>
 * FLAGGED<br/>
 * RENEWED<br/>
 * NO_STATUS<br/>
 * CANCELLED<br/>
 * DISCONTINUED<br/>
 * SCHEDULED<br/>
 * COMPLETE<br/>
 * PARTIAL_RESULTS<br/>
 * EXPIRED<br/>
 * UNKNOWN 
 */
public enum ConsultStatus {
	ACTIVE("Active"), PENDING("Pending"), 
	HOLD("Hold"), FLAGGED("Flagged"), 
	RENEWED("Renewed"), NO_STATUS("No Status"),
	CANCELLED("Cancelled"), DISCONTINUED("Denied"), 
	SCHEDULED("Scheduled"), COMPLETE("Complete"), 
	PARTIAL_RESULTS("Partial Results"), EXPIRED("Expired"), 
	UNKNOWN("Unknown");
	 
	 private String representation;
	 private static Map<String, ConsultStatus> representationToStatusMapping;
	 
	 private ConsultStatus(String representation) {
	   this.representation = representation;
	 }
	 
	 public String getRepresentation() {
	   return this.representation;
	 }
	 
	 public static ConsultStatus lookUpByRepresentation(String representation){
		 ConsultStatus consultStatus = ConsultStatus.UNKNOWN;
		 if (representationToStatusMapping == null) {
			 initMapping();
		 }
		 consultStatus = representationToStatusMapping.get(representation);
		 return consultStatus;	 
	 }
	 
	 private static void initMapping() {
		 representationToStatusMapping = new HashMap<String, ConsultStatus>();
	     for (ConsultStatus s : values()) {
	    	 representationToStatusMapping.put(s.getRepresentation(), s);
	     }
	 }
	 
	 public static ConsultStatus lookUpByName(String name){
		 ConsultStatus consultStatus = ConsultStatus.UNKNOWN;
		 String statusName = name.trim().toUpperCase();
		 try
	     	{
			 	if (statusName.equals("PARTIAL RESULTS")){
					 consultStatus = ConsultStatus.PARTIAL_RESULTS;
				 }else	if (statusName.equals("NO STATUS")){
					 consultStatus = ConsultStatus.NO_STATUS;
				 }else{
					 consultStatus = getValue(statusName);
				 }
	        } catch( NumberFormatException e ) {
	        	consultStatus = ConsultStatus.UNKNOWN;
	        }
		 return consultStatus;	 
	 }
	 
	 public static ConsultStatus getValue(String name){
		 for(ConsultStatus status : ConsultStatus.values()){
			 if(status.name().equals(name)){
				 return ConsultStatus.valueOf(name);
			 }		 
		 }
		 return ConsultStatus.UNKNOWN;
	 }	 
}
