package com.agilex.healthcare.mobilehealthplatform.mdws;

import java.io.Serializable;


public class SearchIndex implements Serializable {
	private static final long serialVersionUID = 6714176155668483303L;
	
	private String searchString = "";
	private Direction direction = Direction.DEFAULT;

	public SearchIndex() {
		this.searchString = "";
		this.direction = Direction.DEFAULT;
	}
	
	public SearchIndex(String searchString, Direction direction) {
		super();
		this.searchString = searchString;
		this.direction = direction;
	}

	public String getSearchString() {
		if (searchString == null)
			return "";
		else
			return searchString;
	}

	public void setSearchString(String searchString) {
		this.searchString = searchString;
	}

	/**
	 * If a MDWS lookup supports direction, this indicates whether the results will contain values after or before
	 * the searchString.  If BACKWARD, the results returned will not include the index in the list of data that
	 * searchString will return and the data will be in a reversed order.
	 */
	public Direction getDirection() {
		if (direction == null)
			return Direction.DEFAULT;
		else
			return direction;
	}

	/**
	 * If a MDWS lookup supports direction, this indicates whether the results will contain values after or before
	 * the searchString.  If BACKWARD, the results returned will not include the index in the list of data that
	 * searchString will return and the data will be in a reversed order.
	 */
	public void setDirection(Direction direction) {
		this.direction = direction;
	}

//---------------------------------Enum---------------------------------------------------------------------------------
	public enum Direction {
		FORWARD("1"),
		BACKWARD("-1"),
		DEFAULT("");
		
		private final String value;
		
		private Direction(String value) {
			this.value = value;
		}
		
		@Override
		public String toString() {
			return this.value;
		}
		
		public static Direction fromValue(String value) {
			if (value.equals("1"))
				return FORWARD;
			else if (value.equals("-1"))
				return BACKWARD;
			else if (value.equals(""))
				return DEFAULT;
			else
				throw new IllegalArgumentException("SearchIndex.Direction can only be \"\", \"-1\" or \"1\", it cannot be " + value);
		}
	}
//---------------------------------End Enum-----------------------------------------------------------------------------
}
