package com.agilex.healthcare.mobilehealthplatform.mdws;

import java.io.UnsupportedEncodingException;
import java.net.URI;

import com.agilex.healthcare.mobilehealthplatform.utils.QueryParameters;
import com.agilex.healthcare.utility.NullChecker;

public class SearchIndexFactory {
	
	public SearchIndex createEmptySearchIndex() {
		SearchIndex searchIndex = new SearchIndex();
		
		return searchIndex;
	}
	
	public SearchIndex createFromUriDecoded(URI u) {
		try {
			SearchIndex searchIndex = createEmptySearchIndex();
			
			QueryParameters queryParameters = new QueryParameters(u.getRawQuery());
			
			String searchString = queryParameters.getValue("searchstring");
			
			if (NullChecker.isNotNullish(searchString)) {
				searchString = java.net.URLDecoder.decode(searchString, "UTF-8");
				searchIndex.setSearchString(searchString);
			}
			else {
				searchIndex.setSearchString("");
			}
			
			String direction = queryParameters.getValue("direction");
			
			if (NullChecker.isNotNullish(direction)) {
				direction = java.net.URLDecoder.decode(direction, "UTF-8");
				if (direction.equalsIgnoreCase("1"))
					searchIndex.setDirection(SearchIndex.Direction.FORWARD);
				else if (direction.equalsIgnoreCase("-1"))
					searchIndex.setDirection(SearchIndex.Direction.BACKWARD);
				else
					throw new IllegalArgumentException("direction can only be \"\", \"-1\" or \"1\", it cannot be " + direction);
			}
			else {
				searchIndex.setDirection(SearchIndex.Direction.FORWARD);
			}
			
			return searchIndex;
		}
		catch (UnsupportedEncodingException e) {
			throw new IllegalArgumentException(e.getMessage(), e);
		}
	}
	
	public SearchIndex createFromSearchString(String searchString) {
		SearchIndex searchIndex = createEmptySearchIndex();
		
		if (NullChecker.isNotNullish(searchString)) {
			searchIndex.setSearchString(searchString);
		}
		
		return searchIndex;
	}
	
	public SearchIndex createFromSearchStringAndDirection(String searchString, SearchIndex.Direction direction) {
		SearchIndex searchIndex = createFromSearchString(searchString);
		searchIndex.setDirection(direction);
		return searchIndex;
	}
}
