package com.agilex.healthcare.notewriter.datalayer;

import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitles;

public interface NoteTitleDataLayer {
	/**
	 * Returns the NoteTitles starting at searchString.
	 * 
	 * @param searchString
	 * The searchString should be thought of as being identical to looking up a name in a telephone book.
	 * When you do a lookup in a phone book you pick someone's name and then start scanning down the list of names
	 * from there.  When you perform a lookup here and you receive results, it is like opening the phone book and picking
	 * a name in it.  It just selects the first item that matches closest to the one you supplied and then it returns
	 * you the rest of the list of all items from that point to the end of the list.  Thus, this is not filtering
	 * anything but is providing you a way to start your list at a point somewhere within the entire list.<br/><br/>
	 * 
	 * If searchString is null or the empty string, this will return you ONLY the first 44 records.<br/><br/>
	 * 
	 * When it comes to anything that involves pagination in MDWS, if there are more than 44 results, we get back
	 * exactly 44 records. At that point, you should call the exact same MDWS call again passing in the value from the
	 * last record (the 44th record). This would continue until we receive less than 44 records.  Remember the
	 * telephone book analogy - this is like going down 44 records in that list and returning 44 more.<br/><br/>
	 * 
	 * <font color="red"><b>Example</b></font><br/>
	 * Suppose you had the following set of data: {'benefit', 'bladder', 'bones', 'brain'}.<br/>
	 * If you searched for 'bl', you'd get back {'bladder', 'bones', 'brain'}.<br/>
	 * If you searched for 'bm', you'd get back {'bones', 'brain'}.  This is the first index of data that occurs after
	 * what you searched for.<br/><br/>
	 * 
	 * 
	 * @param direction You can pass in &quot;&quot;, &quot;-1&quot;, or &quot;1&quot;.
	 * This tells the system whether to return the previous (&quot;-1&quot;) or the next (&quot;1&quot;) set of data.
	 * If you don't pass anything in &quot;&quot; or null, it's similar to passing in &quot;1&quot;.
	 * When returning the previous set of data, it does not include the index you searched for and the order of
	 * elements in the returned list is in a reversed order.<br/><br/>
	 * 
	 * <font color="red"><b>Example of using the (optional) direction</b></font><br/>
	 * Suppose you had the following set of data: {'benefit', 'bladder', 'bones', 'brain'}.<br/>
	 * If you searched for 'bo' with a direction of 1, you'd get back {'bones', 'brain'}.<br/>
	 * If you searched for 'bp' with a direction of 1, you'd get back {'brain'}.<br/>
	 * If you searched for 'bo' with a direction of -1, you'd get back {'bladder', 'benefit'}.<br/>
	 * If you searched for 'bp' with a direction of -1, you'd get back {'bones', 'bladder', 'benefit'}<br/>
	 * Notice that the list elements are reversed when direction is -1.<br/><br/>
	 */
	NoteTitles getNoteTitles(String searchString, String direction);
	
	/**
	 * Returns whether the note IEN is for a Consult
	 */
	boolean isConsultNote(String noteDefinitionIEN);

	/**
	 * Returns whether the note IEN is for a Patient Record Flag
	 */
	boolean isPrfNote(String noteDefinitionIEN);

	/**
	 * Returns whether the note IEN, noteTitle, locationIen, vistaTimestamp, & serviceCategory combination are for a OneVisitNote
	 * 
	 * @param serviceCategory 'A' (ambulatory), 'H' (hospitalization), or 'E' (event historical)
	 */
	boolean isOneVisitNote(String patientId, String noteDefinitionIEN, String noteTitle, String locationIen, String vistaTimestamp, String serviceCategory);

	/**
	 * Returns whether the note IEN is for a Surgery Note
	 */
	boolean isSurgeryNote(String noteDefinitionIEN);
}
