package com.agilex.healthcare.notewriter.datalayer;

import java.io.UnsupportedEncodingException;
import java.net.URI;

import com.agilex.healthcare.mobilehealthplatform.utils.QueryParameters;
import com.agilex.healthcare.utility.NullChecker;

public class SearchFilterFactory {
	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(SearchFilterFactory.class);
	
	public SearchFilter createEmptySearchFilter() {
		SearchFilter searchFilter = new SearchFilter();
		
		return searchFilter;
	}
	
	public SearchFilter createFromUri(URI u) {
		SearchFilter searchFilter = createEmptySearchFilter();
		
		QueryParameters queryParameters = new QueryParameters(u.getQuery());
		String inputValue = queryParameters.getValue("searchstring");
		if (NullChecker.isNotNullish(inputValue)) {
			searchFilter.setSearchString(inputValue);
		}
		
		inputValue = queryParameters.getValue("maxrecords");
		if (NullChecker.isNotNullish(inputValue)) {
			try {
				int maxRecords = Integer.valueOf(inputValue);
				searchFilter.setMaxRecords(maxRecords);
			} catch (NumberFormatException e) {
				LOGGER.error(String.format("%s is not a number (only numbers are allowed for max records)",inputValue));
			}
		}
		
		return searchFilter;
	}
	
	public SearchFilter createFromUriDecoded(URI u) {
		try {
			SearchFilter searchFilter = createEmptySearchFilter();
			
			QueryParameters queryParameters = new QueryParameters(u.getRawQuery()); //Notice this is a raw query
			String inputValue = queryParameters.getValue("searchstring");
			
			if (NullChecker.isNotNullish(inputValue)) {
				inputValue = java.net.URLDecoder.decode(inputValue, "UTF-8");
				searchFilter.setSearchString(inputValue);
			}
			
			inputValue = queryParameters.getValue("maxrecords");
			if (NullChecker.isNotNullish(inputValue)) {
				inputValue = java.net.URLDecoder.decode(inputValue, "UTF-8");
				try {
					int maxRecords = Integer.valueOf(inputValue);
					searchFilter.setMaxRecords(maxRecords);
				} catch (NumberFormatException e) {
					LOGGER.error(String.format("%s is not a number (only numbers are allowed for max records)",inputValue));
				}
			}
			
			return searchFilter;
		}
		catch (UnsupportedEncodingException e) {
			throw new IllegalArgumentException(e.getMessage(), e);
		}
	}
	
	public SearchFilter createFromSearchString(String searchString) {
		SearchFilter searchFilter = createEmptySearchFilter();
		
		if (NullChecker.isNotNullish(searchString)) {
			searchFilter.setSearchString(searchString);
		}
		
		return searchFilter;
	}
	
	public SearchFilter createFromSearchString(String searchString, int maxRecords) {
		SearchFilter searchFilter = createEmptySearchFilter();
		
		if (NullChecker.isNotNullish(searchString)) {
			searchFilter.setSearchString(searchString);
		}
		if (maxRecords > 0) {
			searchFilter.setMaxRecords(maxRecords);
		}
		
		return searchFilter;
	}
}
