package com.agilex.healthcare.utility.objectmapper;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.agilex.healthcare.mobilehealthplatform.domain.VitalEntries;
import com.agilex.healthcare.mobilehealthplatform.domain.VitalEntry;
import com.agilex.healthcare.mobilehealthplatform.domain.VitalObservation;
import com.agilex.healthcare.mobilehealthplatform.domain.VitalTableData;
import com.agilex.healthcare.mobilehealthplatform.domain.VitalTableRow;
import com.agilex.healthcare.mobilehealthplatform.domain.code.ObservationTypeCode;
import com.agilex.healthcare.mobilehealthplatform.domain.code.VitalSectionCode;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;

public class VitalTableDataGenerator {
	public VitalTableData generateVitalTableData(VitalEntries vitalEntries) {
		VitalTableData tableData = new VitalTableData();

		if (NullChecker.isNullish(vitalEntries)) {
			return tableData;
		}

		Map<Date, VitalEntries> vitalsByDate = getVitalsByDateTime(vitalEntries);

		Set<Date> datesWithVitals = vitalsByDate.keySet();
		for (Date date : datesWithVitals) {
			VitalEntries vitalEntriesForDate = vitalsByDate.get(date);
			VitalTableRow tableRow = createVitalTableRow(vitalEntriesForDate);
			if (tableRow != null) {
				tableRow.setEntryTime(date);
				tableData.add(tableRow);
			}
		}

		return tableData;
	}
	
	private VitalTableRow createVitalTableRow(VitalEntries vitalEntries) {
		VitalTableRow tableRow = new VitalTableRow();
		boolean hasData = false;
		for (VitalEntry vitalEntry : vitalEntries) {
			tableRow.setPatientIdentifier(vitalEntry.getPatientIdentifier());

			String section = vitalEntry.getSection();

			if (VitalSectionCode.BloodPressure.equalsIgnoreCase(section)) {
				hasData = true;
				tableRow = setBloodPressure(vitalEntry, tableRow);
			} else if (VitalSectionCode.Temperature.equalsIgnoreCase(section)) {
				hasData = true;
				tableRow = setTemperature(vitalEntry, tableRow);
			} else if (VitalSectionCode.Respiration.equalsIgnoreCase(section)) {
				hasData = true;
				tableRow = setRespiration(vitalEntry, tableRow);
			} else if (VitalSectionCode.Weight.equalsIgnoreCase(section)) {
				hasData = true;
				tableRow = setWeight(vitalEntry, tableRow);
			} else if (VitalSectionCode.Pain.equalsIgnoreCase(section)) {
				hasData = true;
				tableRow = setPain(vitalEntry, tableRow);
			} else if (VitalSectionCode.PulseOximetry.equalsIgnoreCase(section)) {
				hasData = true;
				tableRow = setPulseOximetry(vitalEntry, tableRow);
			} else if (VitalSectionCode.Pulse.equalsIgnoreCase(section)) {
				hasData = true;
				tableRow = setPulse(vitalEntry, tableRow);
			}
		}
        
		if( hasData )
		    return tableRow;
		else
			return null;
	}

	private VitalTableRow setPulseOximetry(VitalEntry vitalEntry, VitalTableRow tableRow) {
		VitalObservation vitalObservation = vitalEntry.getObservationByType(ObservationTypeCode.PULSE_OXIMETRY);
        if(vitalObservation != null)
		    tableRow.setPulseOximetry(vitalObservation.getValue());
		return tableRow;
	}

	private VitalTableRow setPain(VitalEntry vitalEntry, VitalTableRow tableRow) {
		VitalObservation vitalObservation = vitalEntry.getObservationByType(ObservationTypeCode.PAIN);
        if(vitalObservation != null)
		    tableRow.setPain(vitalObservation.getValue());
		return tableRow;
	}

	private VitalTableRow setWeight(VitalEntry vitalEntry, VitalTableRow tableRow) {
		VitalObservation vitalObservation = vitalEntry.getObservationByType(ObservationTypeCode.WEIGHT);
        if(vitalObservation != null)
		    tableRow.setWeight(vitalObservation.getValue());
		return tableRow;
	}

	private VitalTableRow setHeight(VitalEntry vitalEntry, VitalTableRow tableRow) {
		VitalObservation vitalObservation = vitalEntry.getObservationByType(ObservationTypeCode.HEIGHT);
        if(vitalObservation != null)
            tableRow.setHeight(vitalObservation.getValue());
		return tableRow;
	}

	private VitalTableRow setRespiration(VitalEntry vitalEntry, VitalTableRow tableRow) {
		VitalObservation vitalObservation = vitalEntry.getObservationByType(ObservationTypeCode.RESPIRATION);
        if(vitalObservation != null)
		    tableRow.setRespiration(vitalObservation.getValue());
		return tableRow;
	}

	private VitalTableRow setTemperature(VitalEntry vitalEntry, VitalTableRow tableRow) {
		VitalObservation vitalObservation = vitalEntry.getObservationByType(ObservationTypeCode.BODY_TEMPERATURE);
        if(vitalObservation != null)
            tableRow.setTemperature(vitalObservation.getValue());
		return tableRow;
	}

	private VitalTableRow setBloodPressure(VitalEntry vitalEntry, VitalTableRow tableRow) {
		VitalObservation vitalObservation = vitalEntry.getObservationByType(ObservationTypeCode.INTRAVASCULAR_SYSTOLIC);
        if(vitalObservation != null)
		    tableRow.setSystolic(vitalObservation.getValue());

		vitalObservation = vitalEntry.getObservationByType(ObservationTypeCode.INTRAVASCULAR_DIASTOLIC);
        if(vitalObservation != null)
		    tableRow.setDiastolic(vitalObservation.getValue());

		vitalObservation = vitalEntry.getObservationByType(ObservationTypeCode.HEART_BEAT);
		if (vitalObservation != null)
			tableRow.setPulse(vitalObservation.getValue());

		return tableRow;
	}
	
	private VitalTableRow setPulse(VitalEntry vitalEntry, VitalTableRow tableRow) {
		VitalObservation vitalObservation = vitalEntry.getObservationByType(ObservationTypeCode.HEART_BEAT);
        if(vitalObservation != null)
		    tableRow.setPulse(vitalObservation.getValue());
		return tableRow;
	}
	
	private Map<Date, VitalEntries> getVitalsByDateTime(VitalEntries vitalEntries) {
		Map<Date, VitalEntries> vitalsByDateTime = new HashMap<Date, VitalEntries>();

		for (VitalEntry vitalEntry : vitalEntries) {

			if (NullChecker.isNotNullish(vitalEntry.getSection())) {
				Date date = vitalEntry.getEntryDate();
				VitalEntries vitalEntriesByDate = vitalsByDateTime.get(date);
				if (vitalEntriesByDate == null) {
					vitalEntriesByDate = new VitalEntries();
					vitalEntriesByDate.add(vitalEntry);
					vitalsByDateTime.put(date, vitalEntriesByDate);
				} else {
					vitalEntriesByDate.add(vitalEntry);
				}
			}
		}
		return vitalsByDateTime;
	}

	private VitalEntry getVitalEntryBySection(VitalEntries vitalEntriesByDate, VitalEntry vitalEntry) {

		if (NullChecker.isNotNullish(vitalEntry.getSection())) {
			for (VitalEntry entry : vitalEntriesByDate) {
				if (vitalEntry.getSection().equalsIgnoreCase(entry.getSection())) {
					return entry;
				}
			}
		}

		return null;
	}

	private Date getDateWithoutTime(Date date) {
		String format = "MM/dd/yyyy";
		String dateStr = DateHelper.format(date, format);
		return DateHelper.parse(dateStr, format);
	}

}
