package com.agilex.healthcare.mobilehealthplatform.domain.filter;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.domain.Event;
import com.agilex.healthcare.mobilehealthplatform.domain.Events;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.utility.DateHelper;

public class EventFiltererTest {

	@Test
	public void filterByDate() {
		EventFilter filter = new EventFilter(null, DateHelper.parseDate("1/15/2012"), DateHelper.parseDate("2/15/2012"));
		Events events = createSampleData();
		Events filteredEvents = EventFilterer.applyFilter(events, filter);
		assertEquals(1, filteredEvents.size());
		assertEquals("b", filteredEvents.get(0).getId());
	}

	@Test
	public void filterByDateSetWithAccessors() {
		EventFilter filter = new EventFilter(null, DateHelper.parseDate("1/15/2012"), DateHelper.parseDate("2/15/2012"));
		Events events = createSampleData();
		Events filteredEvents = EventFilterer.applyFilter(events, filter);
		assertEquals(1, filteredEvents.size());
		assertEquals("b", filteredEvents.get(0).getId());
	}

	@Test
	public void filterByDateWithDateFilterConstructor() {
		DateFilter datefilter = DateFilterFactory.createFilterFromDate("1/15/2012", "2/15/2012");
		EventFilter filter = new EventFilter(null, datefilter);
		Events events = createSampleData();
		Events filteredEvents = EventFilterer.applyFilter(events, filter);
		assertEquals(1, filteredEvents.size());
		assertEquals("b", filteredEvents.get(0).getId());
	}

	@Test
	public void filterByDateWithNullDateFilterConstructor() {
		DateFilter datefilter = null;
		EventFilter filter = new EventFilter(null, datefilter);
		Events events = createSampleData();
		Events filteredEvents = EventFilterer.applyFilter(events, filter);
		assertEquals(4, filteredEvents.size());
	}

	@Test
	public void toStringDoesNotError() {
		DateFilter datefilter = DateFilterFactory.createFilterFromDate("1/15/2012", "2/15/2012");
		EventFilter filter = new EventFilter("a", datefilter);
		filter.toString();
	}

	@Test
	public void filterByType() {
		EventFilter filter = new EventFilter("qqq");
		Events events = createSampleData();
		Events filteredEvents = EventFilterer.applyFilter(events, filter);
		assertEquals(1, filteredEvents.size());
		assertEquals("d", filteredEvents.get(0).getId());
	}

	@Test
	public void nullFilter() {
		EventFilter filter = null;
		Events events = createSampleData();
		Events filteredEvents = EventFilterer.applyFilter(events, filter);
		assertEquals(4, filteredEvents.size());
	}

	@Test
	public void nullInput() {
		EventFilter filter = new EventFilter("qqq");
		Events events = null;
		Events filteredEvents = EventFilterer.applyFilter(events, filter);
		assertNull(filteredEvents);
	}

	private Events createSampleData() {
		Events events = new Events();
		Event event;

		event = new Event();
		event.setId("a");
		event.setType("zzz");
		event.setDate(DateHelper.parseDate("1/1/2012"));
		events.add(event);

		event = new Event();
		event.setId("b");
		event.setType("zzz");
		event.setDate(DateHelper.parseDate("2/1/2012"));
		events.add(event);

		event = new Event();
		event.setId("c");
		event.setType("zzz");
		event.setDate(DateHelper.parseDate("3/1/2012"));
		events.add(event);

		event = new Event();
		event.setId("d");
		event.setType("qqq");
		event.setDate(DateHelper.parseDate("4/1/2012"));
		events.add(event);

		return events;
	}
}
