package com.agilex.healthcare.utility;

import org.junit.Assert;
import org.junit.Test;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;


public class DateHelperTest {
	@Test
	public void testBeginningOfToday() {

		Calendar today = Calendar.getInstance();
		Calendar midnightToCheck = new GregorianCalendar();
		midnightToCheck.setTime(DateHelper.beginOfDate(today.getTime()));
		
		assertEquals(0, midnightToCheck.get(Calendar.HOUR));
		assertEquals(0, midnightToCheck.get(Calendar.MINUTE));
		assertEquals(0, midnightToCheck.get(Calendar.SECOND));
		assertEquals(0, midnightToCheck.get(Calendar.MILLISECOND));
		
		assertEquals(today.get(Calendar.MONTH), midnightToCheck.get(Calendar.MONTH));
		assertEquals(today.get(Calendar.DAY_OF_MONTH), midnightToCheck.get(Calendar.DAY_OF_MONTH));
		assertEquals(today.get(Calendar.DAY_OF_YEAR), midnightToCheck.get(Calendar.DAY_OF_YEAR));
	}
	
	@Test
	public void validateDeltaInDays() {
		assertEquals(1, DateHelper.calculateDeltaInDays(DateHelper.parseDate("01/01/2012"), DateHelper.parseDate("01/02/2012")));
	}
	
	@Test
	public void validateNegativeDeltaInDays() {
		assertEquals(-1, DateHelper.calculateDeltaInDays(DateHelper.parseDate("01/02/2012"), DateHelper.parseDate("01/01/2012")));
	}
	
	@Test
	public void validateGreaterThan() {
		Assert.assertTrue(DateHelper.greaterThanOrEqual(new Date(), DateHelper.getYesterday()));
	}

	@Test
	public void validateConvertPureVistaDateFormatToKnownVistaDateFormat() {
		Assert.assertEquals(DateHelper.convertPureVistaDateToKnownVistaDateFormat("3140903.105404"), "20140903.105404");
		Assert.assertEquals(DateHelper.convertPureVistaDateToKnownVistaDateFormat("3131004.020810"), "20131004.020810");
		Assert.assertEquals(DateHelper.convertPureVistaDateToKnownVistaDateFormat("3120513.051121"), "20120513.051121");
		Assert.assertEquals(DateHelper.convertPureVistaDateToKnownVistaDateFormat("3111223.081335"), "20111223.081335");

		Assert.assertEquals(DateHelper.convertPureVistaDateToKnownVistaDateFormat(""), "");
		Assert.assertEquals(DateHelper.convertPureVistaDateToKnownVistaDateFormat(null), "");

		boolean fail = false;
		try {
			DateHelper.convertPureVistaDateToKnownVistaDateFormat("adfs");
		} catch (RuntimeException e) {
			fail = true;
		}
		assertTrue(fail);

		fail = false;
		try {
			DateHelper.convertPureVistaDateToKnownVistaDateFormat("!@#");
		} catch (RuntimeException e) {
			fail = true;
		}
		assertTrue(fail);
	}
}
