package com.agilex.healthcare.utility;

import static org.junit.Assert.assertEquals;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.safety.Whitelist;
import org.junit.Test;
import org.owasp.esapi.ESAPI;

public class ESAPILearningTest {

	@Test
	public void testForNoteTitle() {
        String input = "ACROMEGALY  <C&P ACROMEGALY>";
        String expected = "ACROMEGALY  &lt;C&amp;P ACROMEGALY&gt;";
		
		String canonicalized = ESAPI.encoder().canonicalize(input);
        String encoded = ESAPI.encoder().encodeForHTML(canonicalized);
        
        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(encoded, "", Whitelist.basic(), outputSettings);
        
        assertEquals(expected, output);
	}

	@Test
	public void testForScript() {
        String input = "ACROMEGALY  <C&P ACROMEGALY><script>alert('1')</script>";
        String expected = "ACROMEGALY  &lt;C&amp;P ACROMEGALY&gt;&lt;script&gt;alert('1')&lt;/script&gt;";
		
		String canonicalized = ESAPI.encoder().canonicalize(input);
        String encoded = ESAPI.encoder().encodeForHTML(canonicalized);
        
        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(encoded, "", Whitelist.basic(), outputSettings);
        
        assertEquals(expected, output);
	}
}
