package com.agilex.healthcare.utility;

import org.junit.Before;
import org.junit.Test;

import java.util.Calendar;
import java.util.GregorianCalendar;

import static org.junit.Assert.*;


public class VistADateParserTest {
	private VistADateParser parser;

	@Before
	public void before() {
		parser = new VistADateParser();
	}

	@Test
	public void testToday() {
		Calendar check = new GregorianCalendar();
		Calendar today = Calendar.getInstance();
		today.set(Calendar.HOUR_OF_DAY, 0);
		today.set(Calendar.MINUTE, 0);
		today.set(Calendar.SECOND, 0);
		today.set(Calendar.MILLISECOND, 0);

		check.setTime(parser.parseAnyVistaDateString("T@U"));
		assertEquals(check.get(Calendar.MONTH), today.get(Calendar.MONTH));
		assertEquals(check.get(Calendar.DAY_OF_MONTH), today.get(Calendar.DAY_OF_MONTH));
		assertEquals(check.get(Calendar.YEAR), today.get(Calendar.YEAR));
		assertEquals(check.get(Calendar.HOUR_OF_DAY), today.get(Calendar.HOUR_OF_DAY));
		assertEquals(check.get(Calendar.MINUTE), today.get(Calendar.MINUTE));
		assertEquals(check.get(Calendar.SECOND), today.get(Calendar.SECOND));
		assertEquals(check.get(Calendar.MILLISECOND), today.get(Calendar.MILLISECOND));

		check.setTime(parser.parseAnyVistaDateString("T"));
		assertEquals(check.get(Calendar.MONTH), today.get(Calendar.MONTH));
		assertEquals(check.get(Calendar.DAY_OF_MONTH), today.get(Calendar.DAY_OF_MONTH));
		assertEquals(check.get(Calendar.YEAR), today.get(Calendar.YEAR));
		assertEquals(check.get(Calendar.HOUR_OF_DAY), today.get(Calendar.HOUR_OF_DAY));
		assertEquals(check.get(Calendar.MINUTE), today.get(Calendar.MINUTE));
		assertEquals(check.get(Calendar.SECOND), today.get(Calendar.SECOND));
		assertEquals(check.get(Calendar.MILLISECOND), today.get(Calendar.MILLISECOND));

		check.setTime(parser.parseAnyVistaDateString("T@130240"));
		assertEquals(check.get(Calendar.MONTH), today.get(Calendar.MONTH));
		assertEquals(check.get(Calendar.DAY_OF_MONTH), today.get(Calendar.DAY_OF_MONTH));
		assertEquals(check.get(Calendar.YEAR), today.get(Calendar.YEAR));
		assertEquals(check.get(Calendar.HOUR_OF_DAY), 13);
		assertEquals(check.get(Calendar.MINUTE), 02);
		assertEquals(check.get(Calendar.SECOND), 40);
		assertEquals(check.get(Calendar.MILLISECOND), 0);

		check.setTime(parser.parseAnyVistaDateString("TODAY"));
		assertEquals(check.get(Calendar.MONTH), today.get(Calendar.MONTH));
		assertEquals(check.get(Calendar.DAY_OF_MONTH), today.get(Calendar.DAY_OF_MONTH));
		assertEquals(check.get(Calendar.YEAR), today.get(Calendar.YEAR));
		assertEquals(check.get(Calendar.HOUR_OF_DAY), today.get(Calendar.HOUR_OF_DAY));
		assertEquals(check.get(Calendar.MINUTE), today.get(Calendar.MINUTE));
		assertEquals(check.get(Calendar.SECOND), today.get(Calendar.SECOND));
		assertEquals(check.get(Calendar.MILLISECOND), today.get(Calendar.MILLISECOND));
	}

	@Test
	public void testPositivelyRelativeToToday() {
		Calendar check = new GregorianCalendar();
		Calendar today = Calendar.getInstance();
		today.set(Calendar.HOUR_OF_DAY, 0);
		today.set(Calendar.MINUTE, 0);
		today.set(Calendar.SECOND, 0);
		today.set(Calendar.MILLISECOND, 0);

		check.setTime(parser.parseAnyVistaDateString("T+3"));
		today.add(Calendar.DAY_OF_MONTH, 3);
		assertEquals(check.get(Calendar.MONTH), today.get(Calendar.MONTH));
		assertEquals(check.get(Calendar.DAY_OF_MONTH), today.get(Calendar.DAY_OF_MONTH));
		assertEquals(check.get(Calendar.YEAR), today.get(Calendar.YEAR));
		assertEquals(check.get(Calendar.HOUR_OF_DAY), today.get(Calendar.HOUR_OF_DAY));
		assertEquals(check.get(Calendar.MINUTE), today.get(Calendar.MINUTE));
		assertEquals(check.get(Calendar.SECOND), today.get(Calendar.SECOND));
		assertEquals(check.get(Calendar.MILLISECOND), today.get(Calendar.MILLISECOND));

		check.setTime(parser.parseAnyVistaDateString("T+3@122242"));
		assertEquals(check.get(Calendar.MONTH), today.get(Calendar.MONTH));
		assertEquals(check.get(Calendar.DAY_OF_MONTH), today.get(Calendar.DAY_OF_MONTH));
		assertEquals(check.get(Calendar.YEAR), today.get(Calendar.YEAR));
		assertEquals(check.get(Calendar.HOUR_OF_DAY), 12);
		assertEquals(check.get(Calendar.MINUTE), 22);
		assertEquals(check.get(Calendar.SECOND), 42);
		assertEquals(check.get(Calendar.MILLISECOND), 0);
	}

	@Test
	public void testNegativeRelativeToToday() {
		Calendar check = new GregorianCalendar();
		Calendar today = Calendar.getInstance();
		today.set(Calendar.HOUR_OF_DAY, 0);
		today.set(Calendar.MINUTE, 0);
		today.set(Calendar.SECOND, 0);
		today.set(Calendar.MILLISECOND, 0);

		check.setTime(parser.parseAnyVistaDateString("T-2"));
		today.add(Calendar.DAY_OF_MONTH, -2);
		assertEquals(check.get(Calendar.MONTH), today.get(Calendar.MONTH));
		assertEquals(check.get(Calendar.DAY_OF_MONTH), today.get(Calendar.DAY_OF_MONTH));
		assertEquals(check.get(Calendar.YEAR), today.get(Calendar.YEAR));
		assertEquals(check.get(Calendar.HOUR_OF_DAY), today.get(Calendar.HOUR_OF_DAY));
		assertEquals(check.get(Calendar.MINUTE), today.get(Calendar.MINUTE));
		assertEquals(check.get(Calendar.SECOND), today.get(Calendar.SECOND));
		assertEquals(check.get(Calendar.MILLISECOND), today.get(Calendar.MILLISECOND));

		check.setTime(parser.parseAnyVistaDateString("T-2@150545"));
		assertEquals(check.get(Calendar.MONTH), today.get(Calendar.MONTH));
		assertEquals(check.get(Calendar.DAY_OF_MONTH), today.get(Calendar.DAY_OF_MONTH));
		assertEquals(check.get(Calendar.YEAR), today.get(Calendar.YEAR));
		assertEquals(check.get(Calendar.HOUR_OF_DAY), 15);
		assertEquals(check.get(Calendar.MINUTE), 05);
		assertEquals(check.get(Calendar.SECOND), 45);
		assertEquals(check.get(Calendar.MILLISECOND), 0);
	}

	@Test
	public void testNow() {
		Calendar check = new GregorianCalendar();
		Calendar now = Calendar.getInstance();

		check.setTime(parser.parseAnyVistaDateString("N"));
		assertEquals(check.get(Calendar.MONTH), now.get(Calendar.MONTH));
		assertEquals(check.get(Calendar.DAY_OF_MONTH), now.get(Calendar.DAY_OF_MONTH));
		assertEquals(check.get(Calendar.YEAR), now.get(Calendar.YEAR));
		assertTrue(check.getTimeInMillis() - now.getTimeInMillis() < 1000);

		check.setTime(parser.parseAnyVistaDateString("NOW"));
		assertEquals(check.get(Calendar.MONTH), now.get(Calendar.MONTH));
		assertEquals(check.get(Calendar.DAY_OF_MONTH), now.get(Calendar.DAY_OF_MONTH));
		assertEquals(check.get(Calendar.YEAR), now.get(Calendar.YEAR));
		assertTrue(check.getTimeInMillis() - now.getTimeInMillis() < 1000);
	}

	@Test
	public void testPositivelyRelativeToNow() {
		Calendar check = new GregorianCalendar();
		Calendar now = Calendar.getInstance();

		check.setTime(parser.parseAnyVistaDateString("N+5"));
		now.add(Calendar.DAY_OF_MONTH, 5);
		assertEquals(check.get(Calendar.MONTH), now.get(Calendar.MONTH));
		assertEquals(check.get(Calendar.DAY_OF_MONTH), now.get(Calendar.DAY_OF_MONTH));
		assertEquals(check.get(Calendar.YEAR), now.get(Calendar.YEAR));
		assertTrue(check.getTimeInMillis() - now.getTimeInMillis() < 1000);
	}

	@Test
	public void testNegativeRelativeToNow() {
		Calendar check = new GregorianCalendar();
		Calendar now = Calendar.getInstance();

		check.setTime(parser.parseAnyVistaDateString("N-6"));
		now.add(Calendar.DAY_OF_MONTH, -6);
		assertEquals(check.get(Calendar.MONTH), now.get(Calendar.MONTH));
		assertEquals(check.get(Calendar.DAY_OF_MONTH), now.get(Calendar.DAY_OF_MONTH));
		assertEquals(check.get(Calendar.YEAR), now.get(Calendar.YEAR));
		assertTrue(check.getTimeInMillis() - now.getTimeInMillis() < 1000);
	}

	@Test
	public void testTwoDigit() {
		Calendar check = new GregorianCalendar();

		check.setTime(parser.parseAnyVistaDateString("02/14/14"));
		assertEquals(check.get(Calendar.MONTH), 2-1);
		assertEquals(check.get(Calendar.DAY_OF_MONTH), 14);
		assertEquals(check.get(Calendar.YEAR), 2014);

	}

	@Test
	public void testFullDate() {
		Calendar check = new GregorianCalendar();

		check.setTime(parser.parseAnyVistaDateString("March 13, 2012"));
		assertEquals(check.get(Calendar.MONTH), 3-1);
		assertEquals(check.get(Calendar.DAY_OF_MONTH), 13);
		assertEquals(check.get(Calendar.YEAR), 2012);
		assertEquals(check.get(Calendar.HOUR_OF_DAY), 0);
		assertEquals(check.get(Calendar.MINUTE), 0);
		assertEquals(check.get(Calendar.SECOND), 0);
		assertEquals(check.get(Calendar.MILLISECOND), 0);
	}

	@Test
	public void testShortenedMonth() {
		Calendar check = new GregorianCalendar();

		check.setTime(parser.parseAnyVistaDateString("Sep 8, 2016@09:30"));
		assertEquals(check.get(Calendar.MONTH), 9-1);
		assertEquals(check.get(Calendar.DAY_OF_MONTH), 8);
		assertEquals(check.get(Calendar.YEAR), 2016);
		assertEquals(check.get(Calendar.HOUR_OF_DAY), 9);
		assertEquals(check.get(Calendar.MINUTE), 30);
		assertEquals(check.get(Calendar.SECOND), 0);
		assertEquals(check.get(Calendar.MILLISECOND), 0);

		check.setTime(parser.parseAnyVistaDateString("Aug 28, 2017@09:30:33"));
		assertEquals(check.get(Calendar.MONTH), 8-1);
		assertEquals(check.get(Calendar.DAY_OF_MONTH), 28);
		assertEquals(check.get(Calendar.YEAR), 2017);
		assertEquals(check.get(Calendar.HOUR_OF_DAY), 9);
		assertEquals(check.get(Calendar.MINUTE), 30);
		assertEquals(check.get(Calendar.SECOND), 33);
		assertEquals(check.get(Calendar.MILLISECOND), 0);
	}

	@Test
	public void testDefaultVista() {
		Calendar check = new GregorianCalendar();

		check.setTime(parser.parseAnyVistaDateString("20110530.081055"));
		assertEquals(check.get(Calendar.MONTH), 5-1);
		assertEquals(check.get(Calendar.DAY_OF_MONTH), 30);
		assertEquals(check.get(Calendar.YEAR), 2011);
		assertEquals(check.get(Calendar.HOUR_OF_DAY), 8);
		assertEquals(check.get(Calendar.MINUTE), 10);
		assertEquals(check.get(Calendar.SECOND), 55);
		assertEquals(check.get(Calendar.MILLISECOND), 0);

		check.setTime(parser.parseAnyVistaDateString("20110530"));
		assertEquals(check.get(Calendar.MONTH), 5-1);
		assertEquals(check.get(Calendar.DAY_OF_MONTH), 30);
		assertEquals(check.get(Calendar.YEAR), 2011);
		assertEquals(check.get(Calendar.HOUR_OF_DAY), 0);
		assertEquals(check.get(Calendar.MINUTE), 0);
		assertEquals(check.get(Calendar.SECOND), 0);
		assertEquals(check.get(Calendar.MILLISECOND), 0);
	}

	@Test
	public void testParseAnyVistaDateStringForExceptions() {
		boolean fail;

		String nullString = null;
		assertNull(parser.parseAnyVistaDateString(nullString));
		assertNull(parser.parseAnyVistaDateString(""));

		fail = true;
		try {
			parser.parseAnyVistaDateString("adf23423424");
		} catch (RuntimeException e) {
			fail = false;
		}
		assertFalse(fail);

		fail = true;
		try {
			parser.parseAnyVistaDateString("32c4089181243");
		} catch (RuntimeException e) {
			fail = false;
		}
		assertFalse(fail);

		fail = true;
		try {
			parser.parseAnyVistaDateString("?345o09");
		} catch (RuntimeException e) {
			fail = false;
		}
		assertFalse(fail);

		fail = true;
		try {
			parser.parseAnyVistaDateString("123.234.3354");
		} catch (RuntimeException e) {
			fail = false;
		}
		assertFalse(fail);

		fail = true;
		try {
			parser.parseAnyVistaDateString("Tomahawk");
		} catch (RuntimeException e) {
			fail = false;
		}
		assertFalse(fail);

		fail = true;
		try {
			parser.parseAnyVistaDateString("Nugget");
		} catch (RuntimeException e) {
			fail = false;
		}
		assertFalse(fail);
	}
}
