package com.agilex.healthcare.utility.date.datemarshallers;

import static org.junit.Assert.assertEquals;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.safety.Whitelist;
import org.junit.Test;

public class JSoupLearningTest {
	
	String report = "Detailed Report\n" + 
			"  MYOCARDIAL PERFUSION SCAN W/PHARM STRESS\n" + 
			"     \n" + 
			"  Exm Date: MAY 24, 2008@11:00\n" + 
			"  Req Phys: RADIOLOGIST,ONE                Pat Loc: NUCLEAR MEDICINE (Req'g Loc)\n" + 
			"                                           Img Loc: NUCLEAR MEDICINE\n" + 
			"                                           Service: Unknown\n" + 
			"  (Case 46 WAITING )   MYOCARDIAL PERFUSION SCAN W/PHARM(NM   Broad   )\n" + 
			"       Reason for Study: SEE CLINICAL HISTORY\n" + 
			"      Clinical History:\n" + 
			"        dyspnea on exertion for past few months GXT \n" + 
			"      Report Status: Verified                   Date Reported: MAY 24, 2008\n" + 
			"                                                Date Verified: APR 30, 2010\n" + 
			"      Verifier E-Sig:/ES/ONE RADIOLOGIST\n" + 
			"      Report:\n" + 
			"        Myocardial perfusion scan, May 24, 2008.  \n" + 
			"              Radiopharmaceutical: 3 mCi of thallium-201 and 35.4 mCi \n" + 
			"              technetium 99m myoview.  \n" + 
			"              Findings: Patient underwent pharmacological stress utilizing 38.9 \n" + 
			"              mg of adenosine. Patient reported no chest pain and EKG was \n" + 
			"              negative for inducible ischemia at stress.  \n" + 
			"              A small mild to moderate defect is present in the inferior wall \n" + 
			"              at stress. This area undergoes near complete perfusion at rest.  \n" + 
			"              There is a question of a large mild defect in the lateral wall at \n" + 
			"              stress. This appears to undergo complete perfusion at rest. \n" + 
			"              Gated study demonstrates hypokinetic wall motion of the septum, \n" + 
			"              proximal inferior wall, and apex. Left ventricular ejection \n" + 
			"              fraction is decreased to 41%.  End-diastolic volume is 84 cc and \n" + 
			"              end-systolic volume is 49 cc.  \n" + 
			"      Impression:\n" + 
			"        Pharmacological stress with adenosine.  \n" + 
			"              Findings suggest a small area of mild to moderate ischemia of the \n" + 
			"              inferior wall.  \n" + 
			"              Question mild ischemia of the lateral wall.  \n" + 
			"              Decreased left ventricular ejection fraction of 41%.  \n" + 
			"              Hypokinetic wall motion of the septum, proximally inferior wall, and apex.  \n" + 
			"  Primary Interpreting Staff:\n" + 
			"    ONE RADIOLOGIST, Staff Physician (Verifier)\n" + 
			"  /SDR\n" + 
			"     \n" + 
			"Facility: CAMP MASTER\n" + 
			"===============================================================================";

	@Test
	public void test() {
		Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
		String output = Jsoup.clean(report, "", Whitelist.basic(), outputSettings);
		assertEquals(report, output);
		
	}

    @Test
    public void testNoFilterEvasion() {
        String input = "<SCRIPT SRC=\"http://ha.ckers.org/xss.jpg\"></SCRIPT>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testBodyTag() {
        String input = "<IMG SRC=# onmouseover=\"alert('xxs')\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }



    @Test
    public void testDefaultSrcTag() {
        String input = "<SCRIPT SRC=\"http://ha.ckers.org/xss.jpg\"></SCRIPT>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }



    @Test
    public void testExtraneousOpenBrackets() {
        String input = "<<SCRIPT>alert(\"XSS\");//<</SCRIPT>@gmail.com";
        String expected = "&lt;@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testFrame() {
        String input = "<FRAMESET><FRAME SRC=\"javascript:alert('XSS');\"></FRAMESET>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }




    @Test
    public void testiFrame() {
        String input = "<IFRAME SRC=\"javascript:alert('XSS');\"></IFRAME>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }



    @Test
    public void testiFrameEventHandlers() {
        String input = "<IFRAME SRC=# onmouseover=\"alert(document.cookie)\"></IFRAME>\n@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }



    @Test
    public void testImageMasquerading() {
        String input = "<SCRIPT SRC=\"http://ha.ckers.org/xss.jpg\"></SCRIPT>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testMalFormedAnchorTagWithQuotes() {
        String input = "<a onmouseover=\"alert(document.cookie)\">xxs link</a>@gmail.com";
        String expected = "<a rel=\"nofollow\">xxs link</a>@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }

    @Test
    public void testMalFormedAnchorTagWithoutQuotes() {
        String input = "<a onmouseover=alert(document.cookie)>xxs link</a>@gmail.com";
        String expected = "<a rel=\"nofollow\">xxs link</a>@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }

    @Test
    public void testMalformedIMGTags() {
        String input = "<IMG \"\"\"><SCRIPT>alert(\"XSS\")</SCRIPT>\">@gmail.com";
        String expected = "&quot;&gt;@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testNoneAlphaNonDigitWithSpaces() {
        String input = "<SCRIPT/XSS SRC=\"http://ha.ckers.org/xss.js\"></SCRIPT>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }



    @Test
    public void testNoneAlphaNonDigitWithoutSpaces() {
        String input = "<SCRIPT/SRC=\"http://ha.ckers.org/xss.js\"></SCRIPT>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }



    @Test
    public void testOnErrorAlert() {
        String input = "<IMG SRC=/ onerror=\"alert(String.fromCharCode(88,83,83))\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testQuoteEncapsulation1() {
        String input = "<SCRIPT a=\">\" SRC=\"http://ha.ckers.org/xss.js\"></SCRIPT>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }

    @Test
    public void testQuoteEncapsulation2() {
        String input = "<SCRIPT a=\">\" '' SRC=\"http://ha.ckers.org/xss.js\"></SCRIPT>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testQuoteEncapsulation3() {
        String input = "<SCRIPT \"a='>'\" SRC=\"http://ha.ckers.org/xss.js\"></SCRIPT>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testQuoteEncapsulation4() {
        String input = "<SCRIPT a=\">'>\" SRC=\"http://ha.ckers.org/xss.js\"></SCRIPT>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testQuoteEncapsulation5() {
        String input = "<SCRIPT>document.write(\"<SCRI\");</SCRIPT>PT SRC=\"http://ha.ckers.org/xss.js\"></SCRIPT>@gmail.com";
        String expected = "PT SRC=&quot;http://ha.ckers.org/xss.js&quot;&gt;@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testDefaultHashSrcTag() {
        String input = "<IMG onmouseover=\"alert('xxs')\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testMETA() {
        String input = "<META HTTP-EQUIV=\"refresh\" CONTENT=\"0;url=javascript:alert('XSS');\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testXSSInEmbeddedSVG() {
        String input = "<EMBED SRC=\"data:image/svg+xml;base64,PHN2ZyB4bWxuczpzdmc9Imh0dH A6Ly93d3cudzMub3JnLzIwMDAvc3ZnIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcv MjAwMC9zdmciIHhtbG5zOnhsaW5rPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5L3hs aW5rIiB2ZXJzaW9uPSIxLjAiIHg9IjAiIHk9IjAiIHdpZHRoPSIxOTQiIGhlaWdodD0iMjAw IiBpZD0ieHNzIj48c2NyaXB0IHR5cGU9InRleHQvZWNtYXNjcmlwdCI+YWxlcnQoIlh TUyIpOzwvc2NyaXB0Pjwvc3ZnPg==\" type=\"image/svg+xml\" AllowScriptAccess=\"always\"></EMBED>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    //IE6 and below

    @Test
    public void testXSSUsingJSDirective() {
        String input = "<IMG SRC=\"javascript:alert('XSS');\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }

    @Test
    public void testXSSUsingJSDirectiveNoQuotesAndNoSemicolon() {
        String input = "<IMG SRC=javascript:alert('XSS')>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }

    @Test
    public void testXSSUsingJSDirectiveCaseInsensitive() {
        String input = "<IMG SRC=JaVaScRiPt:alert('XSS')>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }

    @Test
    public void testGraveAccentObfuscation() {
        String input = "<IMG SRC=`javascript:alert(\"RSnake says, 'XSS'\")`>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }




    @Test
    public void testfromCharCode() {
        String input = "<IMG SRC=javascript:alert(String.fromCharCode(88,83,83))>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }



    @Test
    public void testHEXCharsWithoutSemicolon() {
        String input = "<IMG SRC=&#x6A&#x61&#x76&#x61&#x73&#x63&#x72&#x69&#x70&#x74&#x3A&#x61&#x6C&#x65&#x72&#x74&#x28&#x27&#x58&#x53&#x53&#x27&#x29>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }



    @Test
    public void testEmbeddedTab() {
        String input = "<IMG SRC=\"jav\tascript:alert('XSS');\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testEmbeddedNewlineToBreakupXSS() {
        String input = "<IMG SRC=\"jav&#x0A;ascript:alert('XSS');\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }



    @Test
    public void testNullBreaksupJSDirective() {
        String input = "<IMG SRC=java\\0script:alert(\\\"XSS\\\")>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }

    @Test
    public void testSpacesAndMetaCharsBeforeJSInImages() {
        String input = "<IMG SRC=\" &#14;  javascript:alert('XSS');\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }



    @Test
    public void testInputImage() {
        String input = "<INPUT TYPE=\"IMAGE\" SRC=\"javascript:alert('XSS');\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testBodyImage() {
        String input = "<BODY BACKGROUND=\"javascript:alert('XSS')\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testIMGDynsrc() {
        String input = "<IMG DYNSRC=\"javascript:alert('XSS')\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testIMGlowsrc() {
        String input = "<SCRIPT SRC=\"http://ha.ckers.org/xss.jpg\"></SCRIPT>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testVBscriptInImage() {
        String input = "<IMG SRC='vbscript:msgbox(\"XSS\")'>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testListStyleImage() {
        String input = "<STYLE>li {list-style-image: url(\"javascript:alert('XSS')\");}</STYLE><UL><LI>XSS</br>@gmail.com";
        String expected = "<ul><li>XSS<br />@gmail.com</li></ul>";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }


    @Test
    public void testRemoteStyleSheet1() {
        String input = "<LINK REL=\"stylesheet\" HREF=\"http://ha.ckers.org/xss.css\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }

    @Test
    public void testRemoteStyleSheet2() {
        String input = "<STYLE>@import'http://ha.ckers.org/xss.css';</STYLE>@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }

    @Test
    public void testRemoteStyleSheet3() {
        String input = "<META HTTP-EQUIV=\"Link\" Content=\"<http://ha.ckers.org/xss.css>; REL=stylesheet\">@gmail.com";
        String expected = "@gmail.com";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);

    }
    
    @Test
    public void testDataWithAngularBrackets() {
        String input = "ACROMEGALY  &lt;C&amp;P ACROMEGALY&gt;";
        String expected = "ACROMEGALY  &lt;C&amp;P ACROMEGALY&gt;";

        Document.OutputSettings outputSettings = new Document.OutputSettings().prettyPrint(false);
        String output = Jsoup.clean(input, "", Whitelist.basic(), outputSettings);
        assertEquals(expected, output);
    }
}
