package com.agilex.system.health.services;

import com.agilex.system.health.ConnectionStatus;
import org.junit.Test;

import static junit.framework.Assert.*;

public class WebServiceTest {
    @Test
    public void okConnectionTest() {
        WebService service = new WebService("HDR", "http://172.16.45.55:8089/hdr/mockPathwaysHttpBinding", 5000, 60000);
        ConnectionStatus status = service.runConnectionTest();
        assertEquals(200, status.getCode());
        assertEquals("OK", status.getCodeDescription());
        assertTrue(status.isSuccess());
    }

    @Test
    public void notFoundConnectionTest() {
        WebService service = new WebService("HDR", "http://172.16.45.55:8089/invalid/mockPathwaysHttpBinding", 5000, 60000);
        ConnectionStatus status = service.runConnectionTest();
        assertEquals(404, status.getCode());
        assertEquals("Not Found", status.getCodeDescription());
        assertFalse(status.isSuccess());
    }

    @Test
    public void malformedUrlConnectionTest() {
        WebService service = new WebService("HDR", "htp:ha.invalid.com:8089/hdr/mockPathwaysHttpBinding", 5000, 60000);
        ConnectionStatus status = service.runConnectionTest();
        assertFalse(status.isSuccess());
        assertEquals("java.net.MalformedURLException", status.getExceptionType());
    }

    @Test
    public void connectionTimeoutConnectionTest() {
        WebService service = new WebService("HDR", "http://hadev.invalid.com:8089/invalid/mockPathwaysHttpBinding", 100, 60000);
        ConnectionStatus status = service.runConnectionTest();
        assertFalse(status.isSuccess());
        assertEquals("java.net.SocketTimeoutException", status.getExceptionType());
    }

}
