package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.admission;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.admission.AdmissionDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.admission.AdmissionDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.Admission;
import com.agilex.healthcare.mobilehealthplatform.domain.Admissions;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.utility.DateHelper;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class AdmissionDataLayerMdwsVistaTest {
	
	@Resource
	MdwsConfiguration mdwsVistaConfiguration;
	
	@BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("777", "cprs1234", "cprs4321$", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}

	@Test
	public void testGetPatientAdmissionsFromVista() {
		AdmissionDataLayer dl = new AdmissionDataLayerMdws(mdwsVistaConfiguration);
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("01/01/1998"), DateHelper.getToday());
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-777", "100022");
		Admissions admissions = dl.fetchAdmissions(patientIdentifier, filter);
		
		assertNotNull(admissions);
		assertEquals(2, admissions.size());
		
		Admission one = admissions.get(0);
		assertEquals(MdwsDateTranslationUtil.extractDateOrNull("20140311.115231"), one.getAdmissionDate());
		assertEquals("7A GEN MED", one.getFacilityName());
		assertEquals("158", one.getLocationId());
		
		Admission two = admissions.get(1);
		assertEquals(MdwsDateTranslationUtil.extractDateOrNull("20020130.114524"), two.getAdmissionDate());
		assertEquals("BCMA", two.getFacilityName());
		assertEquals("11", two.getLocationId());
	}
	
	@Test
	public void testNoPatientAdmissionsFromVista() {
		AdmissionDataLayer dl = new AdmissionDataLayerMdws(mdwsVistaConfiguration);
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("01/01/1998"), DateHelper.getToday());
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-500", "156");
		Admissions admissions = dl.fetchAdmissions(patientIdentifier, filter);
		
		assertNotNull(admissions);
		assertEquals(0, admissions.size());
	}
}
