package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.consult;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultFilter;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.Consult;
import com.agilex.healthcare.mobilehealthplatform.domain.Consults;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.enumeration.ConsultStatus;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.utility.DateHelper;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class ConsultDataLayerMdwsVistaTest {
	
	@Resource
	MdwsConfiguration mdwsVistaConfiguration;
	
	@BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("777", "cprs1234", "cprs4321$", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}

	@Test
	public void testGetPatientConsultsFromVista() {
		ConsultDataLayer dl = new ConsultDataLayerMdws(mdwsVistaConfiguration);
		ConsultFilter filter = ConsultFilterFactory.createEmptyFilter();
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-777", "100022");
		Consults consults = dl.fetchPatientConsults(patientIdentifier, filter);
		
		assertNotNull(consults);
		assertEquals(1, consults.size());
		
		Consult c = consults.get(0);
		assertEquals(MdwsDateTranslationUtil.extractDateOrNull("20131002.120744"), c.getCreatedDate());
		assertEquals(new Long(619), c.getConsultId());
		assertEquals("619", c.getDataIdentifier().getUniqueId());
		Assert.assertTrue(c.getDaysOnList() > 0); //This day will change, so just ensure that it's older than today.
		assertEquals(ConsultStatus.COMPLETE, c.getStatus());
		assertEquals("COMPLETE", c.getStatusDescription());
		assertEquals(false, c.getStatusIcon());
		assertNotNull(c.getText());
		Assert.assertTrue(c.getText().length() > 0);
		assertEquals("CARDIOLOGY Cons", c.getTitle());
	}
	
	@Test
	public void testGetPatientConsultsDateFilter() throws URISyntaxException {
		ConsultDataLayer dl = new ConsultDataLayerMdws(mdwsVistaConfiguration);
		
		ConsultFilter filter = ConsultFilterFactory.createFilterFromUri(new URI("http://localhost?startDate=09/10/2013&endDate=08/07/2014"));
		//"createdDate": "09/10/2013 09:28:21"
		//"createdDate": "08/04/2014 12:57:40"
		//"createdDate": "08/04/2014 12:57:41"
		//"createdDate": "08/07/2014 10:11:40"
		List<ConsultStatus> statuses = new ArrayList<ConsultStatus>();
		statuses.add(ConsultStatus.PENDING);
		statuses.add(ConsultStatus.ACTIVE);
		statuses.add(ConsultStatus.SCHEDULED);
		filter.setStatuses(statuses);
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-777", "8");
		Consults consultsUnfiltered = dl.fetchPatientConsults(patientIdentifier, filter);
		
		filter = ConsultFilterFactory.createFilterFromUri(new URI("http://localhost?startDate=08/01/2014&endDate=08/06/2014"));
		filter.setStatuses(statuses);
		Consults consultsFiltered = dl.fetchPatientConsults(patientIdentifier, filter);
		
		assertNotNull(consultsUnfiltered);
		assertEquals(4, consultsUnfiltered.size());
		assertNotNull(consultsFiltered);
		assertEquals(2, consultsFiltered.size());
	}
	
	@Test
	public void testNoPatientConsultsFromVista() {
		ConsultDataLayer dl = new ConsultDataLayerMdws(mdwsVistaConfiguration);
		ConsultFilter filter = ConsultFilterFactory.createEmptyFilter();
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-500", "156");
		Consults consults = dl.fetchPatientConsults(patientIdentifier, filter);
		
		assertNotNull(consults);
		assertEquals(0, consults.size());
	}
}
