package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.facility;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.Iterator;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.facility.ClinicDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.Clinic;
import com.agilex.healthcare.mobilehealthplatform.domain.Clinics;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class ClinicDataLayerMdwsVistaTest {
	private static final String SITE_CODE = "777";
	@Resource
	MdwsConfiguration mdwsVistaConfiguration;
	
	@BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser(SITE_CODE, "cprs1234", "cprs4321$", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}

	@Test
	public void fetchClinics() {
		ClinicDataLayerMdws dataLayer = new ClinicDataLayerMdws(mdwsVistaConfiguration);
		Clinics clinics = dataLayer.fetchClinics("", null);
		assertNotNull(clinics);
		assertTrue(clinics.size() > 0);
		
		Iterator<Clinic> it = clinics.iterator();
		while (it.hasNext()) {
			Clinic clinic = it.next();
			assertEquals(SITE_CODE, clinic.getFacilityCode());
			assertNotNull(clinic.getId());
			assertNotNull(clinic.getName());
		}
	}
	
	@Test
	public void testFetchClinicsWithSearchIndex() {
		ClinicDataLayerMdws dataLayer = new ClinicDataLayerMdws(mdwsVistaConfiguration);
		Clinics clinics = dataLayer.fetchClinics("", "card");
		assertNotNull(clinics);
		assertEquals("CARDIOLOGY", clinics.get(0).getName());
	}
}
