package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.notetitle;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.Iterator;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.notetitle.NoteTitleDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitle;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitles;
import com.agilex.healthcare.mobilehealthplatform.mdws.SearchIndex.Direction;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class NoteTitleDataLayerMdwsVistaTest {
	private static final String SITE_CODE = "777";
	@Resource
	MdwsConfiguration mdwsVistaConfiguration;
	
	@BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser(SITE_CODE, "cprs1234", "cprs4321$", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}

	@Test
	public void getNoteTitles() {
		NoteTitleDataLayerMdws dataLayer = new NoteTitleDataLayerMdws(mdwsVistaConfiguration);
		NoteTitles noteTitles = dataLayer.getNoteTitles(null, null);
		assertNotNull(noteTitles);
		assertEquals(44, noteTitles.size());
		
		Iterator<NoteTitle> it = noteTitles.iterator();
		while (it.hasNext()) {
			NoteTitle noteTitle = it.next();
			assertNotNull(noteTitle.getIen());
			assertNotNull(noteTitle.getName());
		}
	}
	
	@Test
	public void getNoteTitlesWithSearchIndex() {
		NoteTitleDataLayerMdws dataLayer = new NoteTitleDataLayerMdws(mdwsVistaConfiguration);
		NoteTitles noteTitles = dataLayer.getNoteTitles("br", null);
		assertNotNull(noteTitles);
		assertEquals(44, noteTitles.size());
		assertEquals("BRAIN  <C&P BRAIN AND SPINAL CORD>", noteTitles.get(0).getName());
	}
	
	@Test
	public void getNoteTitlesForward() {
		NoteTitleDataLayerMdws dataLayer = new NoteTitleDataLayerMdws(mdwsVistaConfiguration);
		NoteTitles noteTitles = dataLayer.getNoteTitles("br", Direction.FORWARD.toString());
		assertNotNull(noteTitles);
		assertEquals(44, noteTitles.size());
		assertEquals("BRAIN  <C&P BRAIN AND SPINAL CORD>", noteTitles.get(0).getName());
		assertEquals("C&P BRAIN AND SPINAL CORD", noteTitles.get(1).getName());
	}
	
	@Test
	public void getNoteTitlesBackward() {
		NoteTitleDataLayerMdws dataLayer = new NoteTitleDataLayerMdws(mdwsVistaConfiguration);
		NoteTitles noteTitles = dataLayer.getNoteTitles("br", Direction.BACKWARD.toString());
		assertNotNull(noteTitles);
		assertEquals(44, noteTitles.size());
		//We won't include brain as this is a BACKWARD's lookup.
		assertEquals("BONES  <C&P BONES>", noteTitles.get(0).getName());
		assertEquals("BLADDER  <C&P LIVER, GALL BLADDER, AND PANCREAS>", noteTitles.get(1).getName());
	}
	
	@Test
	public void isConsultNoteFalse() {
		NoteTitleDataLayerMdws dataLayer = new NoteTitleDataLayerMdws(mdwsVistaConfiguration);
		boolean result = dataLayer.isConsultNote("1");
		assertFalse(result);
	}
	
	@Test
	public void isPrfNoteFalse() {
		NoteTitleDataLayerMdws dataLayer = new NoteTitleDataLayerMdws(mdwsVistaConfiguration);
		boolean result = dataLayer.isPrfNote("1");
		assertFalse(result);
	}
	
	@Test
	public void isConsultNoteTrue() {
		NoteTitleDataLayerMdws dataLayer = new NoteTitleDataLayerMdws(mdwsVistaConfiguration);
		boolean result = dataLayer.isConsultNote("1637");
		assertTrue(result);
	}
	
	@Test
	public void isPrfNoteTrue() {
		NoteTitleDataLayerMdws dataLayer = new NoteTitleDataLayerMdws(mdwsVistaConfiguration);
		boolean result = dataLayer.isPrfNote("1356");
		assertTrue(result);
	}

	@Test
	public void isOneVisitNoteFalse() {
		NoteTitleDataLayerMdws dataLayer = new NoteTitleDataLayerMdws(mdwsVistaConfiguration);
		boolean result = dataLayer.isOneVisitNote("1", "1301", "BRAIN  <C&P BRAIN AND SPINAL CORD>", "114", "3140501.213926", "A");
		assertFalse(result);
	}
	
	@Test
	public void isSurgeryNoteFalse() {
		NoteTitleDataLayerMdws dataLayer = new NoteTitleDataLayerMdws(mdwsVistaConfiguration);
		boolean result = dataLayer.isSurgeryNote("1");
		assertFalse(result);
	}
}
