package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.order;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.Date;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.order.OrderDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.TestUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.Order;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderResultsHistory;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderServices;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderStatuses;
import com.agilex.healthcare.mobilehealthplatform.domain.Orders;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.enumeration.OrderGrouping;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.utility.DateHelper;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class OrderDataLayerMdwsVistaTest {
	
	private final PatientIdentifier VISTA_TEST_PATIENT_ID = new PatientIdentifier("dfn-688", "100844");
    private final String VISTA_TEST_SITE = "688";
	
	@Resource
	MdwsConfiguration mdwsOrderMgmtConfiguration;
	
	@BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("688", "doc123", "doc123!!", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}
	
	@Test
	public void fetchPatientOrderResultHistory() {
		OrderDataLayerMdws dl = new OrderDataLayerMdws(mdwsOrderMgmtConfiguration);
		OrderResultsHistory history = dl.fetchPatientOrderResultHistory(new PatientIdentifier("dfn-777", "1"), "10894;1");
		
		assertNotNull(history);
		assertEquals("10894;1", history.getOrderIen());
		assertNotNull(history.getResultHistory());
	}

	@Test
    public void fetchOrderStatusesFromVistaTest() {
        OrderDataLayerMdws dl = new OrderDataLayerMdws(mdwsOrderMgmtConfiguration);
        OrderStatuses statuses = dl.fetchOrderStatuses(VISTA_TEST_SITE);
        assertTrue(String.format("[size: %s]", statuses.size()), statuses.size() >= 0);
        
        assertEquals("1", statuses.get(0).getIen());
		assertEquals("All", statuses.get(0).getName());
		assertEquals("0", statuses.get(0).getParent());
    }

    @Test
    public void fetchOrderServicesFromVistaTest() {
        OrderDataLayerMdws dl = new OrderDataLayerMdws(mdwsOrderMgmtConfiguration);
        OrderServices services = dl.fetchOrderServices("777");
        assertTrue(String.format("[size: %s]", services.size()), services.size() >= 0);
        
        assertEquals("1", services.get(0).getIen());
		assertEquals("ALL SERVICES", services.get(0).getName());
		assertEquals("0", services.get(0).getParent());
    }

    @Test
	public void fetchOrdersByCustomFromVistaTest() {
		OrderDataLayerMdws dl = new OrderDataLayerMdws(mdwsOrderMgmtConfiguration);
		Orders orders = dl.fetchOrdersByCustom(VISTA_TEST_PATIENT_ID, "", "", "1", "1");
		
		Date orderedDate = DateHelper.parseVistaDefaultDateTIme("20140106.2100");
//		Date startDate = DateHelper.parseVistaDefaultDateTIme("20140106.2100");
//		Date stopDate = DateHelper.parseVistaDefaultDateTIme("20140205.0000");
		
		Order order = orders.get(0);
		TestUtil.assertNullDataIdentifier(order.getDataIdentifier());
		assertEquals(orderedDate, order.getOrderedDate());
		assertEquals(OrderGrouping.Medication, order.getOrderGrouping());
		assertEquals("34044;1", order.getOrderId());
		assertNotNull(order.getOrderProvider());
		assertEquals("983", order.getOrderProvider().getDuz());
		assertEquals("PROVIDER,ONE", order.getOrderProvider().getName());
		assertNotNull(order.getOrderSignStatus());
		assertEquals("ELECTRONIC", order.getOrderSignStatus().getName());
		assertEquals("1", order.getOrderSignStatus().getValue());
		assertNotNull(order.getOrderStatus());
		assertEquals("e", order.getOrderStatus().getCode());
		assertEquals("Expired", order.getOrderStatus().getName());
		assertEquals("exp", order.getOrderStatus().getShortName());
		assertEquals("7", order.getOrderStatus().getValue());
		assertEquals("Outpatient", order.getOrderSubType());
		assertEquals("OMEGA-3 FISH OIL CONC CAP,ORAL  1000MG	TAKE 1 CAPSULE BY MOUTH EVERY 12 HOURS	Quantity: 6 Refills: 0	", order.getOrderText());
		assertEquals("Outpatient", order.getOrderType());
		assertEquals("4", order.getOrderTypeId());
		TestUtil.assertNullPatientIdentifier(order.getPatientIdentifier());
		assertNotNull(order.getProvider());
		assertEquals("PROVIDER,ONE", order.getProvider().getDisplayName());
		assertEquals("ONE", order.getProvider().getFirstName());
		assertEquals("PROVIDER", order.getProvider().getLastName());
		assertNotNull(order.getProvider().getUserIdentifier());
		assertEquals("default", order.getProvider().getUserIdentifier().getAssigningAuthority());
		assertEquals("983", order.getProvider().getUserIdentifier().getUniqueId());
		assertNull(order.getSourceSystem());
		//assertEquals(startDate, order.getStartDate());
		assertEquals("Expired", order.getStatus());
		//assertEquals(stopDate, order.getStopDate());
		TestUtil.assertNullPatientIdentifier(order.getSurrogateIdentifier());
		assertEquals("", order.getVerifyingClerk());
		assertEquals("", order.getVerifyingNurse());
	}
    
    @Test(expected=MdwsResponseException.class)
	public void fetchOrdersByCustomFromVistaTestException() {
		OrderDataLayerMdws dl = new OrderDataLayerMdws(mdwsOrderMgmtConfiguration);
		dl.fetchOrdersByCustom(VISTA_TEST_PATIENT_ID, "1/1/1980", "1/1/2014", "1", ""); //Must supply service
	}
}
