package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.pathology;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.lab.PathologyLabDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.pathology.PathologyDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.PathologyReports;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;

@ContextConfiguration( locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class PathologyDataLayerMdwsVistaTest {

    @Resource
    MdwsConfiguration mdwsVistaConfiguration;

    private static String assigningAuthority = "dfn-688";
    private static String patientId = "224";
    private static String START_DATE = "01/01/2002";
    private static String END_DATE = "01/01/2008";

    @BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("777", "cprs1234", "cprs4321$", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}

	@Test
    public void verifyFetchPathologyReportFromVista(){
        PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
        DateFilter dateFilter = DateFilterFactory.createFilterFromDate(START_DATE, END_DATE);
        PathologyDataLayer pathologyDataLayer = new PathologyLabDataLayerMdws(mdwsVistaConfiguration);
        PathologyReports pathologyReports = pathologyDataLayer.getPatientPathologyReports(patientIdentifier, dateFilter);
        assertNotNull(pathologyReports);
        assertTrue(pathologyReports.size() > 0);
    }
	
	@Test
    public void verifyNoFetchPathologyReportFromVista(){
        PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
        DateFilter dateFilter = DateFilterFactory.createFilterFromDate("01/02/2002", "01/03/2002");
        PathologyDataLayer pathologyDataLayer = new PathologyLabDataLayerMdws(mdwsVistaConfiguration);
        PathologyReports pathologyReports = pathologyDataLayer.getPatientPathologyReports(patientIdentifier, dateFilter);
        assertNotNull(pathologyReports);
        Assert.assertEquals(0, pathologyReports.size());
    }
	
	@Test(expected=MdwsResponseException.class)
    public void verifyExceptionFetchPathologyReportFromVista(){
        PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
        DateFilter dateFilter = DateFilterFactory.createFilterFromDate("01/02/2002", "01/02/2002");
        PathologyDataLayer pathologyDataLayer = new PathologyLabDataLayerMdws(mdwsVistaConfiguration);
        pathologyDataLayer.getPatientPathologyReports(patientIdentifier, dateFilter); //Can't have begin and end dates on the same day
    }
}
