package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.patient;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientSearchCriteria;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.TestUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.Patients;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;


@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class PatientSearchMdwsVistaTest {

	@Resource(name = "patientTest")
	Patient patientTest;

	@Resource
	MdwsConfiguration mdwsVistaConfiguration;

	private PatientDataLayerMdws datalayer;

	@Before
	public void setup(){
		datalayer = new PatientDataLayerMdws(mdwsVistaConfiguration);
	}

	@BeforeClass
	public static void login(){
		AuthenticationUtil.loginUser("500", "doc123", "doc123!!", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}
	
	@Test
	public void retrievePatientTestSpringData() {

		assertEqualsIgnoreCase(patientTest.getDisplayName(), "Ill,Patient");
	}

	@Test
	public void searchPatientBySsn() {
		System.out.println("Patient Test is "+patientTest.getLastName());
		PatientSearchCriteria criteria = new PatientSearchCriteria();
		criteria.setVistaSiteCode("500");
		Patient prototype = new Patient();
		criteria.setPrototype(prototype);
		prototype.setSsn(patientTest.getSsn());
		Patients patients = datalayer.patientSearch(criteria);
		System.out.println("Patient Size is "+patients.getSize());
		Patient patient = patients.get(0);
		System.out.println("spring data "+patientTest.getDisplayName()+" mdws data "+patient.getDisplayName());
		assertEqualsIgnoreCase(patientTest.getDisplayName(), patient.getDisplayName());
		System.out.println("after spring data "+patientTest.getDisplayName()+" mdws data "+patient.getDisplayName());
	}


	@Test
	public void searchPatientBySsnWithSearchString() {
		System.out.println("Patient Test is "+patientTest.getLastName());
		PatientSearchCriteria criteria = new PatientSearchCriteria();
		criteria.setVistaSiteCode("500");
		criteria.setSearchString(patientTest.getSsn());
		Patients patients = datalayer.patientSearch(criteria);
		Patient patient = patients.get(0);
		System.out.println("spring data ssn "+patientTest.getLastName()+" mdws data "+patient.getLastName());
		assertEqualsIgnoreCase(patientTest.getLastName(), patient.getLastName());
	}

	@Test
	public void searchPatientByInitialPlusLastFourSsn() {
		PatientSearchCriteria criteria = new PatientSearchCriteria();
		criteria.setVistaSiteCode("500");
		criteria.setLastInitialPlusLastFourOfSsn(extractInitialSsn(patientTest));
		Patients patients = datalayer.patientSearch(criteria);
		Patient patient = patients.get(0);
		assertEquals(patientTest.getLastName().toLowerCase(), patient.getLastName().toLowerCase());
	}

	private String extractInitialSsn(Patient patient) {
		return patient.getLastName().substring(0, 1) + patient.getSsn().substring(5);
	}

	@Test
	public void searchPatientByInitialPlusLastFourSsnWithSearchString() {
		PatientSearchCriteria criteria = new PatientSearchCriteria();
		criteria.setVistaSiteCode("500");
		criteria.setSearchString(extractInitialSsn(patientTest));
		Patients patients = datalayer.patientSearch(criteria);
		Patient patient = patients.get(0);
		assertEquals(patientTest.getLastName().toLowerCase(), patient.getLastName().toLowerCase());
	}

	@Test
	public void searchPatientByLastFourSsnWithSearchString() {
		PatientSearchCriteria criteria = new PatientSearchCriteria();
		criteria.setVistaSiteCode("500");
		criteria.setSearchString(extractInitialSsn(patientTest));
		Patients patients = datalayer.patientSearch(criteria);
		Patient patient = patients.get(0);
		assertEquals(patientTest.getLastName().toLowerCase(), patient.getLastName().toLowerCase());
	}

	@Test
	public void searchPatientByLastFourSsn() {
		PatientSearchCriteria criteria = new PatientSearchCriteria();
		criteria.setVistaSiteCode("500");
		criteria.setLastFourOfSsn(extractInitialSsn(patientTest));
		Patients patients = datalayer.patientSearch(criteria);
		Patient patient = patients.get(0);
		assertEquals(patientTest.getLastName().toLowerCase(), patient.getLastName().toLowerCase());
	}

	private void assertEqualsIgnoreCase(String expected, String actual) {
		if (expected != null)
			expected = expected.toLowerCase();
		if (actual != null)
			actual = actual.toLowerCase();
		assertEquals(expected, actual);
	}

	@Test
	public void searchPatientByWard(){
		PatientSearchCriteria criteria = new PatientSearchCriteria();
		criteria.setVistaSiteCode("500");
		criteria.setWard("33");
		
		Patients patients = datalayer.patientSearch(criteria);
		assertTrue(patients.getSize() > 0);
	}
	
	@Test
	public void searchPatientByWardByLastName(){
		Patient prototype = new Patient();
		prototype.setLastName("VERYILL");
		PatientSearchCriteria criteria = new PatientSearchCriteria();
		criteria.setPrototype(prototype);
		criteria.setVistaSiteCode("500");
		criteria.setWard("33");
		
		Patients patients = datalayer.patientSearch(criteria);
		assertTrue(patients.getSize() > 0);
		assertEquals("VERYILL", patients.get(0).getLastName());
		assertNull(patients.get(0).getAdmittedDateTime());
		assertNull("veryill", patients.get(0).getAge());
		assertNull(patients.get(0).getDateOfBirth());
		assertEquals("VERYILL,PATIENT", patients.get(0).getDisplayName());
		assertEquals("PATIENT", patients.get(0).getFirstName());
		assertNull(patients.get(0).getGender());
		assertNull(patients.get(0).getIcn());
		assertEquals(true, patients.get(0).isInpatient());
		assertNull(patients.get(0).getLocationIEN());
		assertNull(patients.get(0).getMiddleInitial());
		assertNull(patients.get(0).getMiddleName());
		TestUtil.assertPatientIdentifier(patients.get(0).getPatientIdentifier(), "default", "100843");
		
		assertEquals("104", patients.get(0).getRoombed());
		assertEquals(false, patients.get(0).isSensitive());
		assertEquals(false, patients.get(0).isServiceConnected());
		assertEquals("0", patients.get(0).getServiceConnectedPercent());
		assertEquals("", patients.get(0).getSsn());
		assertEquals(false, patients.get(0).isTextMessagingAllowed());
		assertNull(patients.get(0).getTextMessagingPhoneNumber());
		assertNull(patients.get(0).getWardLocation());
	}

	@Test
	public void searchPatientByClinic(){
		// Can't find a clinic with patients at the moment so just assuming no exceptions == success
		PatientSearchCriteria criteria = new PatientSearchCriteria();
		criteria.setVistaSiteCode("500");
		criteria.setClinic("16");
		/*Patients patients = */datalayer.patientSearch(criteria);
		assertTrue(true);
	}
}
