package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.progressnote;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.Date;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.progressnote.ProgressNoteDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.progressnote.ProgressNoteDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.TestUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNote;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNotes;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.utility.DateHelper;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class ProgressNoteDataLayerMdwsVistaTest {
	
	@Resource
	MdwsConfiguration mdwsVistaConfiguration;
	
	@BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("777", "cprs1234", "cprs4321$", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}

	@Test
	public void testGetPatientAllergiesFromVista() {
		ProgressNoteDataLayer dl = new ProgressNoteDataLayerMdws(mdwsVistaConfiguration);
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("03/01/2001"), DateHelper.parseDate("03/26/2011"));
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-500", "237");
		ProviderNotes providerNotes = dl.fetchPatientProgressNotes(patientIdentifier, filter);
		
		assertNotNull(providerNotes);
		assertEquals(11, providerNotes.size());
		
		ProviderNote one = providerNotes.get(0);
		assertEquals("3846", one.getId());
		assertNull(one.getApprovedBy());
		assertNotNull(one.getAuthor());
		assertEquals("FIFTYNINE LABTECH", one.getAuthor().getName());
		assertNotNull(one.getDataIdentifier());
		assertNull(one.getDataIdentifier().getSystemId());
		assertNull(one.getDataIdentifier().getUniqueId());
		assertNull(one.getDischargeDate());
		assertEquals("PROGRESS NOTE", one.getDocumentType());
		assertNull(one.getEntryDate());
		assertEquals("ADVANCE DIRECTIVE COMPLETED", one.getLocalTitle());
		assertNull(one.getLocation());
		TestUtil.assertNullPatientIdentifier(one.getPatientIdentifier());
		
		assertNull(one.getSourceSystem());
		assertEquals("ADVANCE DIRECTIVE", one.getStandardTitle());
		TestUtil.assertNullPatientIdentifier(one.getSurrogateIdentifier());
		assertNotNull(one.getText());
		assertTrue(one.getText().length() > 0);
		Date timestamp = DateHelper.parseVistaDateTime("20070516.085600");
		assertEquals(timestamp, one.getTimestamp());
	}
	
	@Test
	public void testNoPatientAllergiesFromVista() {
		ProgressNoteDataLayer dl = new ProgressNoteDataLayerMdws(mdwsVistaConfiguration);
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("01/01/1998"), DateHelper.parseDate("01/02/1998"));
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-500", "156");
		ProviderNotes providerNotes = dl.fetchPatientProgressNotes(patientIdentifier, filter);
		
		assertNotNull(providerNotes);
		assertEquals(0, providerNotes.size());
	}
	
	@Test(expected=MdwsResponseException.class)
	public void testNoPatientAllergiesFromVistaException() {
		ProgressNoteDataLayer dl = new ProgressNoteDataLayerMdws(mdwsVistaConfiguration);
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("01/01/1998"), DateHelper.parseDate("01/01/1998"));
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-500", "156");
		dl.fetchPatientProgressNotes(patientIdentifier, filter);
	}
}
