package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.radiology;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.radiology.RadiologyDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.TestUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.RadiologyResult;
import com.agilex.healthcare.mobilehealthplatform.domain.RadiologyResults;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.utility.DateHelper;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class RadiologyDataLayerMdwsVistaTest {
	
	//NOTE: patient ids that have radiology data are : 21, 237, 736.
	
	@Resource
	MdwsConfiguration mdwsVistaConfiguration;

	@BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("500", "ov1234", "ov1234!!", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}

	@Test
	public void verifyRetrievalFromVista() {
		PatientIdentifier identifier = new PatientIdentifier("dfn-500", "237");
		//String patientId = "237";
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("01/01/1990"), DateHelper.getToday());

		RadiologyDataLayerMdws dl = new RadiologyDataLayerMdws(mdwsVistaConfiguration); 
		RadiologyResults results = dl.getResults(identifier, filter);
		
		assertNotNull(results);
		assertTrue(results.size() >= 2);
		
		RadiologyResult one = results.get(0);
		TestUtil.assertDataIdentifier(one.getDataIdentifier(), null, "6909098.8783-1");
		assertEquals("500", one.getFacilityName());
		assertNull(one.getInterpretation());
		TestUtil.assertNullPatientIdentifier(one.getPatientIdentifier());
		assertNotNull(one.getReport());
		assertTrue(one.getReport().length() > 0);
		assertEquals(MdwsDateTranslationUtil.extractDateOrNull("20090901.121600"), one.getResultedDate());
		assertEquals("500", one.getSourceSystem());
		TestUtil.assertNullPatientIdentifier(one.getSurrogateIdentifier());
		assertEquals("6909098.8783-1", one.getTestId());
		assertEquals("KNEE 2 VIEWS", one.getTestName());
		assertEquals("KNEE 2 VIEWS", one.getTestShortName());
		
		RadiologyResult two = results.get(1);
		TestUtil.assertDataIdentifier(two.getDataIdentifier(), null, "7009782.8859-1");
		assertEquals("500", two.getFacilityName());
		assertNull(two.getInterpretation());
		TestUtil.assertNullPatientIdentifier(two.getPatientIdentifier());
		assertNotNull(two.getReport());
		assertTrue(two.getReport().length() > 0);
		assertEquals(MdwsDateTranslationUtil.extractDateOrNull("19990217.114000"), two.getResultedDate());
		assertEquals("500", two.getSourceSystem());
		TestUtil.assertNullPatientIdentifier(two.getSurrogateIdentifier());
		assertEquals("7009782.8859-1", two.getTestId());
		assertEquals("ANKLE 2 VIEWS", two.getTestName());
		assertEquals("ANKLE 2 VIEWS", two.getTestShortName());
	}
	
	@Test
	public void verifyRetrievalFromVistaWithDateFilter() {
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("01/01/2000"), DateHelper.getToday());
		PatientIdentifier identifier = new PatientIdentifier("dfn-500", "237");
		//String patientId = "237"; // 21 and 736
		RadiologyDataLayerMdws dl = new RadiologyDataLayerMdws(mdwsVistaConfiguration);
		RadiologyResults results = dl.getResults(identifier, filter);
		
		assertNotNull(results);
		assertTrue(results.size() >= 1);
	}
}
