package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.report;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.Date;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.report.ReportDocumentDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.report.ReportDocumentDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.TestUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ReportDocument;
import com.agilex.healthcare.mobilehealthplatform.domain.ReportDocuments;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.utility.DateHelper;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class ReportDocumentDataLayerMdwsVistaTest {
	
	@Resource
	MdwsConfiguration mdwsVistaConfiguration;
	
	@BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("777", "cprs1234", "cprs4321$", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}

	@Test
	public void getPatientAllReportDocumentsFromVista() {
		ReportDocumentDataLayer dl = new ReportDocumentDataLayerMdws(mdwsVistaConfiguration);
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("01/01/1998"), DateHelper.parseDate("03/05/2014"));
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-500", "237");
		ReportDocuments reportDocuments = dl.getPatientAllReportDocuments(patientIdentifier, filter);
		
		assertNotNull(reportDocuments);
		assertTrue(reportDocuments.size() > 0);
		
		ReportDocument one = reportDocuments.get(0);
		assertNull(one.getApprovedBy());
		assertNotNull(one.getAuthor());
		assertEquals("", one.getAuthor().getId());
		assertEquals("ONE PROGRAMMER", one.getAuthor().getName());
		assertEquals("", one.getAuthor().getSignature());
		TestUtil.assertNullDataIdentifier(one.getDataIdentifier());
		assertNull(one.getDischargeDate());
		assertEquals("PROGRESS NOTE", one.getDocumentType());
		assertNull(one.getEntryDate());
		assertNull(one.getExam());
		assertNull(one.getFacility());
		assertNull(one.getFacilityName());
		assertEquals("CARDIOLOGY NOTE", one.getLocalTitle());
		assertNull(one.getLocation());
		TestUtil.assertNullPatientIdentifier(one.getPatientIdentifier());
		assertEquals("6811", one.getRecordId());
		assertNull(one.getResultedDate());
		assertNull(one.getSourceSystem());
		assertNull(one.getSpecimen());
		assertEquals("CARDIOLOGY NOTE", one.getStandardTitle());
		TestUtil.assertNullPatientIdentifier(one.getSurrogateIdentifier());
		assertNull(one.getTestId());
		assertNull(one.getTestName());
		assertNull(one.getTestShortName());
		assertNotNull(one.getText());
		assertTrue(one.getText().length() > 0);
		Date timestamp = DateHelper.parseVistaDefaultDateTIme("20140304.1329");
		assertEquals(timestamp, one.getTimestamp());
		assertNull(one.getTitle());
	}
	
	@Test
	public void getNoPatientAllReportDocumentsFromVista() {
		ReportDocumentDataLayer dl = new ReportDocumentDataLayerMdws(mdwsVistaConfiguration);
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("01/01/1998"), DateHelper.getToday());
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-500", "156");
		ReportDocuments reportDocuments = dl.getPatientAllReportDocuments(patientIdentifier, filter);
		
		assertNotNull(reportDocuments);
		assertEquals(0, reportDocuments.size());
	}
}
