package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.surgery;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.surgery.SurgeryDataLayer;
import com.agilex.healthcare.mobilehealthplatform.datalayer.surgery.SurgeryDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.TestUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.Surgery;
import com.agilex.healthcare.mobilehealthplatform.domain.Surgeries;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.SurgeryReport;
import com.agilex.healthcare.mobilehealthplatform.domain.SurgeryReports;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.utility.DateHelper;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class SurgeryDataLayerMdwsVistaTest {
	
	@Resource
	MdwsConfiguration mdwsVistaConfiguration;
	
	@BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("777", "cprs1234", "cprs4321$", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}

	@Test
	public void testGetPatientSurgeriesFromVista() {
		SurgeryDataLayer dl = new SurgeryDataLayerMdws(mdwsVistaConfiguration);
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("01/01/1998"), DateHelper.getToday());
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-500", "237");
		Surgeries surgerys = dl.fetchSurgeries(patientIdentifier, filter);
		
		assertNotNull(surgerys);
		assertEquals(2, surgerys.size());
		
		Surgery one = surgerys.get(0);
		TestUtil.assertDataIdentifier(one.getDataIdentifier(), null, "10007");
		assertEquals(MdwsDateTranslationUtil.extractDateOrNull("20061208.000000"), one.getProcedureDate());
		assertEquals("CAMP MASTER", one.getFacility());
		assertEquals("LEFT INGUINAL HERNIA REPAIR WITH MESH", one.getProcedure());
		TestUtil.assertNullPatientIdentifier(one.getPatientIdentifier());
		TestUtil.assertNullPatientIdentifier(one.getSurrogateIdentifier());
		assertNull(one.getSourceSystem());
		
		Surgery two = surgerys.get(1);
		TestUtil.assertDataIdentifier(two.getDataIdentifier(), null, "10003");
		assertEquals(MdwsDateTranslationUtil.extractDateOrNull("20020311.000000"), two.getProcedureDate());
		assertEquals("CAMP MASTER", two.getFacility());
		assertEquals("RIH", two.getProcedure());
		TestUtil.assertNullPatientIdentifier(two.getPatientIdentifier());
		TestUtil.assertNullPatientIdentifier(two.getSurrogateIdentifier());
		assertNull(two.getSourceSystem());
	}
	
	@Test
	public void testNoPatientSurgeriesFromVista() {
		SurgeryDataLayer dl = new SurgeryDataLayerMdws(mdwsVistaConfiguration);
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("01/01/1998"), DateHelper.getToday());
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-500", "156");
		Surgeries surgerys = dl.fetchSurgeries(patientIdentifier, filter);
		
		assertNotNull(surgerys);
		assertEquals(0, surgerys.size());
	}
	
	@Test
	public void fetchPatientSurgeryReportsFromVista() {
		SurgeryDataLayer dl = new SurgeryDataLayerMdws(mdwsVistaConfiguration);
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("01/01/1998"), DateHelper.getToday());
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-500", "237");
		SurgeryReports surgeryReports = dl.fetchPatientSurgeryReports(patientIdentifier, filter);
		
		assertNotNull(surgeryReports);
		assertEquals(2, surgeryReports.size());
		
		SurgeryReport one = surgeryReports.get(0);
		assertNotNull(one.getAuthor());
		assertEquals("", one.getAuthor().getId());
		assertEquals("PROVIDER,ONE", one.getAuthor().getName());
		assertEquals("", one.getAuthor().getSignature());
		TestUtil.assertNullDataIdentifier(one.getDataIdentifier());
		assertEquals("SURGERY REPORT", one.getDocumentType());
		assertEquals("10007", one.getId());
		assertEquals("CAMP MASTER", one.getLocation());
		TestUtil.assertNullPatientIdentifier(one.getPatientIdentifier());
		TestUtil.assertNullPatientIdentifier(one.getSurrogateIdentifier());
		assertNull(one.getSourceSystem());
		assertNotNull(one.getText());
		assertTrue(one.getText().length() > 0);
		assertEquals(MdwsDateTranslationUtil.extractDateOrNull("20061208.073000"), one.getTimestamp());
		assertEquals("LEFT INGUINAL HERNIA REPAIR WITH MESH", one.getTitle());
		
		SurgeryReport two = surgeryReports.get(1);
		assertNotNull(two.getAuthor());
		assertEquals("", two.getAuthor().getId());
		assertEquals("TDNURSE,ONE", two.getAuthor().getName());
		assertEquals("", two.getAuthor().getSignature());
		TestUtil.assertNullDataIdentifier(two.getDataIdentifier());
		assertEquals("SURGERY REPORT", two.getDocumentType());
		assertEquals("10003", two.getId());
		assertEquals("CAMP MASTER", two.getLocation());
		TestUtil.assertNullPatientIdentifier(two.getPatientIdentifier());
		TestUtil.assertNullPatientIdentifier(two.getSurrogateIdentifier());
		assertNull(two.getSourceSystem());
		assertEquals("No reports are available for this case.\n", two.getText());
		assertEquals(MdwsDateTranslationUtil.extractDateOrNull("20020311.091500"), two.getTimestamp());
		assertEquals("RIH", two.getTitle());
	}
	
	@Test
	public void testNoFetchPatientSurgeryReportsFromVista() {
		SurgeryDataLayer dl = new SurgeryDataLayerMdws(mdwsVistaConfiguration);
		DateFilter filter = DateFilterFactory.createFilterFromDate(DateHelper.parseDate("01/01/1998"), DateHelper.getToday());
		PatientIdentifier patientIdentifier = new PatientIdentifier("DFN-500", "156");
		SurgeryReports surgeryReports = dl.fetchPatientSurgeryReports(patientIdentifier, filter);
		
		assertNotNull(surgeryReports);
		assertEquals(0, surgeryReports.size());
	}
}
