package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.vital;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.Date;

import javax.annotation.Resource;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.TestUtil;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vital.VitalDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vital.VitalEntryFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.VitalEntries;
import com.agilex.healthcare.mobilehealthplatform.domain.VitalEntry;
import com.agilex.healthcare.mobilehealthplatform.domain.VitalObservation;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.utility.DateHelper;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class VitalDataLayerMdwsVistaTest {
	
	private final PatientIdentifier PATIENT_ID = new PatientIdentifier("DFN-500", "237");
	//private final String PULSE_VITAL_ID_FOR_PATIENT_ID_237 = "25420";
	
	@Resource
	MdwsConfiguration mdwsVistaConfiguration;
	
	@BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("500", "ov1234", "ov1234!!", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}

	@Test
	public void verifyGetVitalEntriesFromVista() {
		
		VitalDataLayerMdws dl = new VitalDataLayerMdws(mdwsVistaConfiguration);
		VitalEntries entries = dl.fetchVitalEntries(PATIENT_ID, new VitalEntryFilter());
		
		assertNotNull(entries);
		assertTrue(String.format("[size: %s]", entries.size()), entries.size() >= 334);
	}
	
	@Test
	public void verifyGetVitalEntriesFromVistaBySectionFilter() {
		
		VitalDataLayerMdws dl = new VitalDataLayerMdws(mdwsVistaConfiguration);
		VitalEntryFilter filter = new VitalEntryFilter();
		filter.setSection("temperature");
		VitalEntries entries = dl.fetchVitalEntries(PATIENT_ID, filter);
		
		assertNotNull(entries);
		assertTrue(String.format("[size: %s]", entries.size()), entries.size() >= 34);
	}

	@Test
	public void verifyGetVitalEntriesFromVistaByStartDateFilter() {
		
		VitalDataLayerMdws dl = new VitalDataLayerMdws(mdwsVistaConfiguration);
		VitalEntryFilter filter = new VitalEntryFilter();
		filter.setSection("temperature");
		filter.setStartDate(DateHelper.parseDate("01/25/2011"));
		VitalEntries entries = dl.fetchVitalEntries(PATIENT_ID, filter);
		
		assertNotNull(entries);
		assertTrue(String.format("[size: %s]", entries.size()), entries.size() >= 1);
		
		Date entryDate = DateHelper.parseVistaDefaultDateTIme("20110208.1000");
		VitalEntry entry = entries.get(0);
		TestUtil.assertDataIdentifier(entry.getDataIdentifier(), "VISTA", "");
		assertEquals(entryDate, entry.getDate());
		assertNull(entry.getNotes());
		TestUtil.assertNullPatientIdentifier(entry.getPatientIdentifier());
		assertEquals("", entry.getQualifier());
		assertEquals("temperature", entry.getSection());
		assertEquals("500", entry.getSourceSystem());
		TestUtil.assertNullPatientIdentifier(entry.getSurrogateIdentifier());
		
		assertNotNull(entry.getVitalObservations());
		assertEquals(1, entry.getVitalObservations().size());
		VitalObservation vo = entry.getVitalObservations().get(0);
		TestUtil.assertNullDataIdentifier(vo.getDataIdentifier());
		TestUtil.assertNullPatientIdentifier(vo.getPatientIdentifier());
		assertNull(vo.getSourceSystem());
		TestUtil.assertNullPatientIdentifier(vo.getSurrogateIdentifier());
		assertEquals("8310-5", vo.getType());
		assertEquals("97.1", vo.getValue());
		assertNull(vo.getValueUnits());
		
		assertNotNull(vo.getVitalEntry());
	}
	//Ignored this test as this method is not used by PVS
	@Ignore
	@Test
	public void getLatestVitalEntriesFromVista() {
		
		VitalDataLayerMdws dl = new VitalDataLayerMdws(mdwsVistaConfiguration);
		VitalEntryFilter filter = new VitalEntryFilter();
		filter.setSection("temperature");
		filter.setStartDate(DateHelper.parseDate("01/25/2011"));
		VitalEntries entries = dl.getLatestVitalEntries(PATIENT_ID, filter);
		
		assertNotNull(entries);
		assertTrue(String.format("[size: %s]", entries.size()), entries.size() >= 1);
		
		Date entryDate = DateHelper.parseVistaDefaultDateTIme("20110208.1000");
		VitalEntry entry = entries.get(0);
		TestUtil.assertDataIdentifier(entry.getDataIdentifier(), "VISTA", null);
		assertEquals(entryDate, entry.getDate());
		assertNull(entry.getNotes());
		TestUtil.assertNullPatientIdentifier(entry.getPatientIdentifier());
		assertNull(entry.getQualifier());
		assertEquals("temperature", entry.getSection());
		assertNull(entry.getSourceSystem());
		TestUtil.assertNullPatientIdentifier(entry.getSurrogateIdentifier());
		
		assertNotNull(entry.getVitalObservations());
		assertEquals(1, entry.getVitalObservations().size());
		VitalObservation vo = entry.getVitalObservations().get(0);
		TestUtil.assertNullDataIdentifier(vo.getDataIdentifier());
		TestUtil.assertNullPatientIdentifier(vo.getPatientIdentifier());
		assertNull(vo.getSourceSystem());
		TestUtil.assertNullPatientIdentifier(vo.getSurrogateIdentifier());
		assertEquals("8310-5", vo.getType());
		assertEquals("97.1 F", vo.getValue());
		assertNull(vo.getValueUnits());
		
		assertNotNull(vo.getVitalEntry());
	}
}
