package com.agilex.healthcare.mobilehealthplatform.datalayer.admission;

import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.domain.code.SourceSystemCode;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.mdws.CdwPatientDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import java.util.List;

public class AdmissionDataLayerCdw extends CdwPatientDataLayer<Admission, Admissions> implements AdmissionDataLayer {

    @PersistenceContext(unitName = "cdwPersistenceUnit")
    protected EntityManager cdwEntityManager;

    private static final Log logger = LogFactory.getLog(AdmissionDataLayerCdw.class);

    public static final String GET_ADMISSIONS_FOR_A_PATIENT = "select * from App.Admissions where PatientICN = :patientICN";
    AdmissionDataLayerMdws dataLayer;

	public AdmissionDataLayerCdw(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
		dataLayer = new AdmissionDataLayerMdws(mdwsConfiguration);
	}
	
    @Override
    public Admissions fetchAdmissions(PatientIdentifier patientIdentifier, DateFilter dateFilter) {
        logger.debug("Retrieve list of Admissions for a Patient : " + patientIdentifier.getUniqueId());
        Query query = this.cdwEntityManager.createNativeQuery(GET_ADMISSIONS_FOR_A_PATIENT, AdmissionPo.class);
        PatientIdentifier alteredPatientIdentifier = alterForCdw(patientIdentifier);
        query.setParameter("patientICN", alteredPatientIdentifier.getUniqueId());
        
        @SuppressWarnings("unchecked")
		List<AdmissionPo> results = query.getResultList();
        Admissions admissions = convertAdmissionPostoAdmissions(results);
        admissions = setDataIdentifier(admissions, SourceSystemCode.CDW);
        return admissions;
    }

	@Override
	public Admission fetchAdmission(PatientIdentifier patientIdentifier, String admissionId) {
        Admission admission = dataLayer.fetchAdmission(alterForCdw(patientIdentifier), admissionId);
		return setIdentifiers(admission, patientIdentifier, SourceSystemCode.CDW);
	}

    public Admissions convertAdmissionPostoAdmissions(List<AdmissionPo> results){
        Admissions admissions = new Admissions();
        if (results != null && results.size() > 0) {
            for (AdmissionPo admissionPo : results) {
                admissions.add(admissionPo.create());
            }
        }
        return admissions;
    }
    public Admissions setDataIdentifier(Admissions admissions, String systemId){
        Admissions modifiedAdmissions = new Admissions();
        if (admissions != null && admissions.size() > 0) {
            for (Admission admission: admissions) {
                admission.setDataIdentifier(new DataIdentifier(systemId,admission.getInpatientSID()));
                modifiedAdmissions.add(admission);
            }
        }
        return modifiedAdmissions;
    }
}
