package com.agilex.healthcare.mobilehealthplatform.datalayer.admission;

import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.AdmissionTranslator;
import com.agilex.healthcare.utility.NullSafeStringComparer;

public class AdmissionDataLayerMdws extends MdwsEmrDataLayer implements AdmissionDataLayer {

	public AdmissionDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

	/**
	 * Calls the MDWS service EmrSvc.getAdmissions for the given patient
	 */
	@Override
	public Admissions fetchAdmissions(PatientIdentifier patientIdentifier, DateFilter dateFilter) {
		EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);
		TaggedInpatientStayArray inpatientStays = authenticatedConnection.getPort().getAdmissions();
		
		super.disconnectFromCdwIfApplicable(authenticatedConnection);
		Admissions admissions = translate(inpatientStays);
		Admissions filtered = filter(admissions, dateFilter);
		return filtered;
	}

	/**
	 * Calls the MDWS service EmrSvc.getAdmissions
	 */
	@Override
	public Admission fetchAdmission(PatientIdentifier patientIdentifier, String admissionId) {
		Admissions admissions = fetchAdmissions(patientIdentifier, DateFilterFactory.createEmptyFilter());
		
		Admission admission = null;
		for(Admission current : admissions) {
			if (NullSafeStringComparer.areEqualIgnoreCase(admissionId, current.getUniqueId())) {
				admission = current;
			}
		}
		
		return admission;
	}

	private Admissions translate(TaggedInpatientStayArray inpatientStays) {
		AdmissionTranslator translator = new AdmissionTranslator();
		return translator.translate(inpatientStays);
	}

	private Admissions filter(Admissions admissions, DateFilter filter) {
		return DateFilterer.filterByDate(admissions, filter);
	}
}
