package com.agilex.healthcare.mobilehealthplatform.datalayer.admission;

import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNote;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseValidator;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedTextArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.TaggedTextTranslator;

public class DischargeNoteDataLayerMdws extends MdwsEmrDataLayer implements DischargeNoteDataLayer {

	public DischargeNoteDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

	/**
	 * Calls the MDWS service EmrSvc.getDischargesReports for the given patient with a hard-coded nrpts of 10 and the given startDate and endDate.
	 */
	@Override
	public ProviderNote fetchDischargeNote(PatientIdentifier patientIdentifier, DateFilter dateFilter) {
		EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);
		
		String siteType = new MdwsConfigurationHelper().extractSiteTypeFromAssigningAuthority(patientIdentifier.getAssigningAuthority());
		
		String startDate = MdwsDateTranslationUtil.startDateBySystem(dateFilter.getStartDate(), siteType);
		String endDate = MdwsDateTranslationUtil.endDateBySystem(dateFilter.getEndDate(), siteType);
		
		TaggedTextArray reports = authenticatedConnection.getPort().getDischargesReports(startDate, endDate, 10);
		super.disconnectFromCdwIfApplicable(authenticatedConnection);
		MdwsResponseValidator.validateMdwsObject(reports);
		
		return translate(reports);
	}
	
	private ProviderNote translate(TaggedTextArray report) {
		TaggedTextTranslator translator = new TaggedTextTranslator();
		String reportText = translator.translate(report);
		
		return new ProviderNote(reportText);
	}	
}
