package com.agilex.healthcare.mobilehealthplatform.datalayer.admission;


import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNotes;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseValidator;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedNoteArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.NoteTranslator;
import com.agilex.healthcare.utility.DateHelper;

public class DischargeSummariesDataLayerMdws extends MdwsEmrDataLayer implements DischargeSummariesDataLayer{
    private static final int NUMBER_OF_RECORDS = 1000;
    private static final String DISCHARGE_SUMMARY = "DISCHARGE SUMMARY";

    public DischargeSummariesDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
        super(mdwsConfiguration);
    }

    /**
	 * Calls the MDWS service EmrSvc.getDischargeSummaries for the given patient with a hard-coded nNotes of 1000 and the given startDate and endDate.
	 */
	@Override
    public ProviderNotes fetchPatientDischargeSummaries(PatientIdentifier patientIdentifier, DateFilter dateFilter) {
        EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);
        TaggedNoteArrays taggedNoteArrays = authenticatedConnection.getPort().getDischargeSummaries(DateHelper.formatDateTimeInVistaFormat(dateFilter.getStartDate()), DateHelper.formatDateTimeInVistaFormat(dateFilter.getEndDate()), NUMBER_OF_RECORDS);
        MdwsResponseValidator.validateMdwsObject(taggedNoteArrays);
        return translate(taggedNoteArrays);
    }


    private ProviderNotes translate(TaggedNoteArrays taggedNoteArrays){
        NoteTranslator dischargeSummariesTranslator = new NoteTranslator();
        return dischargeSummariesTranslator.translateFromTaggedNoteArrays(taggedNoteArrays, DISCHARGE_SUMMARY);
    }
}
