package com.agilex.healthcare.mobilehealthplatform.datalayer.consult;

import org.apache.commons.lang.NotImplementedException;

import com.agilex.healthcare.mobilehealthplatform.domain.Consults;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderIdentifier;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseValidator;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedConsultArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.ConsultTranslator;

public class ConsultDataLayerMdws extends MdwsEmrDataLayer implements ConsultDataLayer {
	private Consults consults = new Consults();
//    private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(ConsultDataLayerMdws.class);

	public ConsultDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

	/**
	 * Calls the MDWS service EmrSvc.getConsultsForPatient for the given patient
	 */
	@Override
	public Consults fetchPatientConsults(PatientIdentifier patientId, ConsultFilter filter) {
		EmrConnection authenticatedSite = super.getAuthenticatedConnectionWithPatientSelect(patientId);
		TaggedConsultArrays mdwsTaggedConsultArrays = authenticatedSite.getPort().getConsultsForPatient();
		MdwsResponseValidator.validateMdwsObject(mdwsTaggedConsultArrays);

		ConsultTranslator translator = new ConsultTranslator();
		consults = translator.translateFromTaggedConsultArrays(mdwsTaggedConsultArrays, filter);
		Consults.sortConsultsOldestFirst(consults);
		return consults;
	}
	
	/**
	 * Throws an NotImplementedException
	 * @throws NotImplementedException
	 */
	@Override
	public Consults fetchConsultsWrittenByProvider(ProviderIdentifier providerIdentifier, ConsultFilter filter) {
		throw new NotImplementedException("Not suppport in Vista mode.");
	}

	/**
	 * Throws an NotImplementedException
	 * @throws NotImplementedException
	 */
	@Override
	public Consults fetchConsultsBySpecialties(ConsultFilter filter, String facilityCode) {
		throw new NotImplementedException("Not suppport in Vista mode.");
	}

}

