package com.agilex.healthcare.mobilehealthplatform.datalayer.facility;

import com.agilex.healthcare.facilitydefinition.datalayer.ward.WardDataLayer;
import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.Wards;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseValidator;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedHospitalLocationArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.HospitalLocationTranslator;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserUtil;

public class WardDataLayerMdws extends MdwsEmrDataLayer implements WardDataLayer {

	public WardDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

	/**
	 * The facilityCode passed in will be ignored since the connection we will be using was created during login using
	 * the facility they chose.  As a result, we reuse that connection and do not create a new one with the facility
	 * passed in.
	 * We also don't support filtering any of the data from this method.<br/><br/>
	 * 
	 * Calls the MDWS service EmrSvc.getWards
	 */
	@Override
	public Wards fetchWards(String ignored, String ignored2) {
		MhpUser mhpUser = MhpUserUtil.getMhpUser();
		String userFacilityCode = mhpUser.getVistaLocation(); //Ignore the facilityCode passed in.
		EmrConnection connection = super.getAuthenticatedConnection(userFacilityCode);
		TaggedHospitalLocationArray tagged = connection.getPort().getWards();
		MdwsResponseValidator.validateMdwsObject(tagged);
		HospitalLocationTranslator translator = new HospitalLocationTranslator();
		Wards wards = translator.translateWardsFromTaggedHospitalLocationArray(tagged, userFacilityCode);
		return wards;
	}
}
