package com.agilex.healthcare.mobilehealthplatform.datalayer.lab;

import com.agilex.healthcare.mobilehealthplatform.datalayer.pathology.PathologyDataLayer;
import com.agilex.healthcare.mobilehealthplatform.domain.PathologyReports;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseValidator;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedCytologyRptArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedSurgicalPathologyRptArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.CytologyReportTranslator;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.SurgicalPathologyReportTranslator;
import com.agilex.healthcare.utility.DateHelper;

public class PathologyLabDataLayerMdws extends MdwsEmrDataLayer implements PathologyDataLayer{

    private static int NUMBER_OF_REPORTS = 2000;

    public PathologyLabDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
        super(mdwsConfiguration);
    }

    /**
	 * Calls the MDWS service EmrSvc.getCytologyReports for the given patient with a hard coded nrpts of 2000.
	 */
	@Override
    public PathologyReports getPatientPathologyReports(PatientIdentifier patientIdentifier, DateFilter dateFilter){
        EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);
        TaggedSurgicalPathologyRptArrays taggedSurgicalPathologyRptArrays = authenticatedConnection.getPort().getSurgicalPathologyReports(DateHelper.formatDateTimeInVistaFormat(dateFilter.getStartDate()), DateHelper.formatDateTimeInVistaFormat(dateFilter.getEndDate()), NUMBER_OF_REPORTS);
        TaggedCytologyRptArrays taggedCytologyRptArrays = authenticatedConnection.getPort().getCytologyReports(DateHelper.formatDateTimeInVistaFormat(dateFilter.getStartDate()), DateHelper.formatDateTimeInVistaFormat(dateFilter.getEndDate()), NUMBER_OF_REPORTS);
        MdwsResponseValidator.validateMdwsObject(taggedSurgicalPathologyRptArrays);
        MdwsResponseValidator.validateMdwsObject(taggedCytologyRptArrays);
        return  translateAndCombineResults(taggedSurgicalPathologyRptArrays, taggedCytologyRptArrays, dateFilter);
    }

    private PathologyReports translateAndCombineResults(TaggedSurgicalPathologyRptArrays taggedSurgicalPathologyRptArrays, TaggedCytologyRptArrays taggedCytologyRptArrays, DateFilter dateFilter) {
        PathologyReports surgicalPathologyReports = translateFromSurgicalPathologyReports(taggedSurgicalPathologyRptArrays, dateFilter);
        PathologyReports cytoPathologyReports = translateFromCytologyReports(taggedCytologyRptArrays, dateFilter);
        surgicalPathologyReports.addAll(cytoPathologyReports);
        return surgicalPathologyReports;
    }

    private PathologyReports translateFromSurgicalPathologyReports(TaggedSurgicalPathologyRptArrays taggedSurgicalPathologyRptArrays, DateFilter dateFilter){
        SurgicalPathologyReportTranslator surgicalPathologyReportTranslator = new SurgicalPathologyReportTranslator();
        return surgicalPathologyReportTranslator.translate(taggedSurgicalPathologyRptArrays, dateFilter);
    }

    private PathologyReports translateFromCytologyReports(TaggedCytologyRptArrays taggedCytologyRptArrays, DateFilter dateFilter){
        CytologyReportTranslator cytologyReportTranslator = new CytologyReportTranslator();
        return cytologyReportTranslator.translate(taggedCytologyRptArrays, dateFilter);
    }
}
