package com.agilex.healthcare.mobilehealthplatform.datalayer.order;

import com.agilex.healthcare.mobilehealthplatform.domain.OrderResultsHistory;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderServices;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderStatuses;
import com.agilex.healthcare.mobilehealthplatform.domain.Orders;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsOrderMgmtDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.OrderMgmtConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.TaggedOrdersArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.TaggedOrdersServiceArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.TaggedOrdersStatusArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.TextArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.OrderCustomTranslator;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.OrderResultsHistoryTranslator;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.OrderServicesTranslator;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.OrderStatusesTranslator;

public class OrderDataLayerMdws extends MdwsOrderMgmtDataLayer implements
		OrderDataLayer {

	public OrderDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

	/**
	 * Calls the MDWS service OrderMgmtSvc.getOrderHistory
	 */
	@Override
	public OrderResultsHistory fetchPatientOrderResultHistory(PatientIdentifier patientIdentifier, String orderIen) {
		OrderResultsHistoryTranslator translator = new OrderResultsHistoryTranslator();

		//getting the order management connection (it needs to be connected to a site and have a patient selected)
		OrderMgmtConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);

		//getting the patient's active and recent activity for orders
		TextArray textArray = authenticatedConnection.getPort().getOrderHistory(orderIen);

		//translate the SOAP call into a usable object
		OrderResultsHistory result = translator.translate(orderIen, textArray);
		return result;
	}

	/**
	 * Calls the MDWS service OrderMgmtSvc.getOrdersStatuses
	 */
	@Override
    public OrderStatuses fetchOrderStatuses(String siteCode) {
        OrderStatusesTranslator translator = new OrderStatusesTranslator();

        //getting the order management connection (it needs to be connected to a site)
        OrderMgmtConnection authenticatedConnection = super.getAuthenticatedConnection(siteCode);

        TaggedOrdersStatusArray ordersStatuses = authenticatedConnection.getPort().getOrdersStatuses();
		OrderStatuses retvalue = translator.translateFromTaggedOrderArrays(ordersStatuses);
		return retvalue;
    }

	/**
	 * Calls the MDWS service OrderMgmtSvc.getOrdersServices
	 */
	@Override
    public OrderServices fetchOrderServices(String siteCode) {
    	OrderServicesTranslator translator = new OrderServicesTranslator();

        //getting the order management connection (it needs to be connected to a site)
        OrderMgmtConnection authenticatedConnection = super.getAuthenticatedConnection(siteCode);

        TaggedOrdersServiceArray ordersServices = authenticatedConnection.getPort().getOrdersServices();
        OrderServices retvalue = translator.translate(ordersServices);
		return retvalue;
    }

	/**
	 * Calls the MDWS service OrderMgmtSvc.getOrdersByCustom
	 */
    public Orders fetchOrdersByCustom(PatientIdentifier patientIdentifier, String startDate, String endDate, String status, String service) {
        OrderCustomTranslator translator = new OrderCustomTranslator();
        //getting the order management connection (it needs to be connected to a site and have a patient selected)
        OrderMgmtConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);

        TaggedOrdersArray taggedOrderArrays = authenticatedConnection.getPort().getOrdersByCustom(status, service, startDate, endDate);

        Orders result = translator.translateFromTaggedOrderArrays(taggedOrderArrays);

        return result;
    }
}
