package com.agilex.healthcare.mobilehealthplatform.datalayer.progressnote;


import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNotes;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseValidator;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedNoteArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.NoteTranslator;
import com.agilex.healthcare.utility.DateHelper;

public class ProgressNoteDataLayerMdws extends MdwsEmrDataLayer implements ProgressNoteDataLayer{
    private static final int NUMBER_OF_RECORDS = 1000;
    private static final String PROGRESS_NOTE = "PROGRESS NOTE";

    public ProgressNoteDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
        super(mdwsConfiguration);
    }

    /**
	 * Calls the MDWS service EmrSvc.getNotesWithText for the given patient
	 */
	@Override
    public ProviderNotes fetchPatientProgressNotes(PatientIdentifier patientIdentifier, DateFilter dateFilter) {
        EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);
        TaggedNoteArrays taggedNoteArrays = authenticatedConnection.getPort().getNotesWithText(DateHelper.formatDateTimeInVistaFormat(dateFilter.getStartDate()), DateHelper.formatDateTimeInVistaFormat(dateFilter.getEndDate()), NUMBER_OF_RECORDS);
        MdwsResponseValidator.validateMdwsObject(taggedNoteArrays);
        return translate(taggedNoteArrays);
    }

    private ProviderNotes translate(TaggedNoteArrays taggedNoteArrays){
        NoteTranslator noteTranslator = new NoteTranslator();
        return noteTranslator.translateFromTaggedNoteArrays(taggedNoteArrays, PROGRESS_NOTE);
    }
}
