package com.agilex.healthcare.mobilehealthplatform.datalayer.provider;

import java.util.*;

import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.ProviderTranslator;

public class ProviderDataLayerMdws extends MdwsEmrDataLayer implements ProviderDataLayer {

	public ProviderDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

	/**
	 * Calls the MDWS service EmrSvc.getPatientCareTeam for the given patient
	 */
	@Override
	public PatientProviders fetchPrimaryCareProviders(PatientIdentifier patientIdentifier, String siteCode) {
		EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);
		//TODO: MDWS refactoring - method not implemented in MDWS3.0.2.5
		//PatientCareTeamTO careTeamTo = authenticatedConnection.getPort().getPatientCareTeam(siteCode);
		super.disconnectFromCdwIfApplicable(authenticatedConnection);

//		MdwsResponseValidator.validateMdwsObject(careTeamTo);
//
//		return translate(careTeamTo);
		throw new UnsupportedOperationException("CMPM-236 - Not Implemented in MDWS3.0.2.5");
	}

	/**
	 * Calls the MDWS service EmrSvc.getMentalHealthVisits for the given patient
	 */
	@Override
	public PatientProviders fetchMentalHealthProviders(PatientIdentifier patientIdentifier, DateFilter filter, String facilitySystemCode) {
		EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);

		TaggedVisitArrays visitArrays = authenticatedConnection.getPort().getMentalHealthVisits();
		MdwsResponseValidator.validateMdwsObject(visitArrays);
		PatientProviders facilityFilteredProviders = translate(visitArrays, facilitySystemCode);
		PatientProviders dateFilteredProviders = filterByDate(facilityFilteredProviders, filter);
		PatientProviders uniqueProviders = filterOutDuplicates(dateFilteredProviders);

		return uniqueProviders;
	}

	private PatientProviders translate(TaggedVisitArrays visitArrays, String facilitySystemCode) {
		ProviderTranslator translator = new ProviderTranslator();
		return translator.translate(visitArrays, facilitySystemCode);
	}

	//TODO: MDWS refactoring - method not implemented in MDWS3.0.2.5
//	private PatientProviders translate(PatientCareTeamTO arrays) {
//		ProviderTranslator translator = new ProviderTranslator();
//		return translator.translate(arrays);
//	}

	private PatientProviders filterOutDuplicates(PatientProviders providers) {
		HashMap<String, PatientProvider> providerHashMap = new HashMap<String, PatientProvider>();

		PatientProviders result = new PatientProviders();

		for (PatientProvider provider : providers) {
			// check if entry already in our dictionary
			String key = getProviderKey(provider);
			if (!providerHashMap.containsKey(key)) {
				// if not, then add
				result.add(provider);
				// then add to dictionary so we don't add again
				providerHashMap.put(key, provider);
			}

		}
		return result;
	}

	private String getProviderKey(PatientProvider provider) {
		return String.format("ProviderName={%1s},ProviderClass={%2s}", provider.getProviderName(), provider.getPersonClass());
	}

	private PatientProviders filterByDate(PatientProviders providers, DateFilter dateFilter) {
		return DateFilterer.filterByDate(providers, dateFilter);
	}

	/**
	 * Throws an UnsupportedOperationException
	 * @throws UnsupportedOperationException
	 */
	@Override
	public PatientProviders fetchPrimaryCareProviders(PatientIdentifier patientIdentifier) {
		throw new UnsupportedOperationException();
	}

}
