package com.agilex.healthcare.mobilehealthplatform.datalayer.radiology;

import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.RadiologyResult;
import com.agilex.healthcare.mobilehealthplatform.domain.RadiologyResults;
import com.agilex.healthcare.mobilehealthplatform.domain.code.SourceSystemCode;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterer;
import com.agilex.healthcare.mobilehealthplatform.mdws.CdwRawPatientDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseValidator;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedRadiologyReportArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.RadiologyTranslator;

public class RadiologyDataLayerCdw extends CdwRawPatientDataLayer<RadiologyResult, RadiologyResults> implements RadiologyDataLayer {

	public RadiologyDataLayerCdw(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

	@Override
	public RadiologyResults getResults(PatientIdentifier patientIdentifier, DateFilter dateFilter) {
		EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier.getUniqueId());

		String startDate = MdwsDateTranslationUtil.startDateBySystem(dateFilter.getStartDate(), SourceSystemCode.CDW);
		String endDate = MdwsDateTranslationUtil.endDateBySystem(dateFilter.getEndDate(), SourceSystemCode.CDW);
		String siteCode = super.getSiteCodeFromPatientIdentifier(patientIdentifier);

		TaggedRadiologyReportArrays radiologyReportArrays = authenticatedConnection.getPort().getRadiologyReportsBySite(startDate, endDate, siteCode);
		super.returnConnection(authenticatedConnection);
		
		MdwsResponseValidator.validateMdwsObject(radiologyReportArrays);
		
		RadiologyResults radiologyResults = translateAndFilter(radiologyReportArrays, dateFilter);
		return super.setIdentifiers(radiologyResults, patientIdentifier, SourceSystemCode.CDW_RAW);
	}

	@Override
	public RadiologyResult fetchResult(PatientIdentifier patientIdentifier, String resultId) {
		RadiologyResults results = getResults(patientIdentifier, DateFilterFactory.createEmptyFilter());
		
		RadiologyResult search = null;
		for(RadiologyResult current : results) {
			if(current.getUniqueId().equals(resultId)) {
				search = current;
			}
		}
		
		return search;
	}

	private RadiologyResults translateAndFilter(TaggedRadiologyReportArrays radiologyReportArrays, DateFilter dateFilter) {
		RadiologyTranslator translator = new RadiologyTranslator();
		RadiologyResults translated =  translator.translate(radiologyReportArrays);
		return DateFilterer.filterByDate(translated, dateFilter);
	}
}
