package com.agilex.healthcare.mobilehealthplatform.datalayer.report;

import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ReportDocuments;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.ReportDocumentTranslator;
import com.agilex.healthcare.utility.DateHelper;

public class ReportDocumentDataLayerMdws extends MdwsEmrDataLayer implements ReportDocumentDataLayer {

	private static final int NUMBER_OF_RECORDS = 2000;
	private static final String DISCHARGE_SUMMARY = "DISCHARGE SUMMARY";
	private static final String PATHOLOGY_REPORT = "PATHOLOGY REPORT";
	private static final String PROGRESS_NOTE = "PROGRESS NOTE";
	private static final String RADIOLOGY_REPORT = "RADIOLOGY REPORT";
	private static final String SURGERY_REPORT = "SURGERY REPORT";

	public ReportDocumentDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

	/**
	 * Calls the following MDWS services for the given patient<br/>
	 * EmrSvc.getDischargeSummaries<br/>
	 * EmrSvc.getSurgicalPathologyReports<br/>
	 * EmrSvc.getCytologyReports<br/>
	 * EmrSvc.getNotesWithText<br/>
	 * EmrSvc.getRadiologyReports<br/>
	 * EmrSvc.getSurgeryReportsWithText<br/>
	 */
	@Override
	public ReportDocuments getPatientAllReportDocuments(PatientIdentifier patientIdentifier, DateFilter dateFilter) {
		ReportDocuments reportDocuments = new ReportDocuments();
		EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);
		String startDate = "";
		String endDate = "";
		if(dateFilter.getEndDate() != null){
			endDate = DateHelper.formatDateTimeInVistaFormat(dateFilter.getEndDate());
			if(dateFilter.getStartDate() != null){
				startDate = DateHelper.formatDateTimeInVistaFormat(dateFilter.getStartDate());
			}
		}
		TaggedNoteArrays dischargeSummariesTaggedNoteArrays = authenticatedConnection.getPort().getDischargeSummaries(startDate, endDate, NUMBER_OF_RECORDS);
		TaggedSurgicalPathologyRptArrays taggedSurgicalPathologyRptArrays = authenticatedConnection.getPort().getSurgicalPathologyReports(DateHelper.formatDateTimeInVistaFormat(dateFilter.getStartDate()), DateHelper.formatDateTimeInVistaFormat(dateFilter.getEndDate()), NUMBER_OF_RECORDS);
		TaggedCytologyRptArrays taggedCytologyRptArrays = authenticatedConnection.getPort().getCytologyReports(DateHelper.formatDateTimeInVistaFormat(dateFilter.getStartDate()), DateHelper.formatDateTimeInVistaFormat(dateFilter.getEndDate()), NUMBER_OF_RECORDS);
		TaggedNoteArrays taggedNoteArrays = authenticatedConnection.getPort().getNotesWithText(DateHelper.formatDateTimeInVistaFormat(dateFilter.getStartDate()), DateHelper.formatDateTimeInVistaFormat(dateFilter.getEndDate()), NUMBER_OF_RECORDS);
		TaggedRadiologyReportArrays taggedRadiologyReports = authenticatedConnection.getPort().getRadiologyReports(startDate, endDate, NUMBER_OF_RECORDS);
		TaggedSurgeryReportArrays taggedSurgeryReportArrays = authenticatedConnection.getPort().getSurgeryReportsWithText();

		ReportDocumentTranslator translator = new ReportDocumentTranslator();
		translator.add(reportDocuments, dischargeSummariesTaggedNoteArrays, DISCHARGE_SUMMARY);
		translator.add(reportDocuments, taggedSurgicalPathologyRptArrays, dateFilter, PATHOLOGY_REPORT);
		translator.add(reportDocuments, taggedCytologyRptArrays, dateFilter, PATHOLOGY_REPORT);
		translator.add(reportDocuments, taggedNoteArrays, PROGRESS_NOTE);
		translator.add(reportDocuments, taggedRadiologyReports, RADIOLOGY_REPORT);
		translator.add(reportDocuments, taggedSurgeryReportArrays, dateFilter, SURGERY_REPORT);

		reportDocuments.sortDescending("timestamp");
		return reportDocuments;
	}
}
