package com.agilex.healthcare.mobilehealthplatform.datalayer.surgery;

import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedSurgeryReportArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.SurgeryReportTranslator;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.SurgeryTranslator;
import com.agilex.healthcare.utility.NullSafeStringComparer;

public class SurgeryDataLayerMdws extends MdwsEmrDataLayer implements SurgeryDataLayer {
    private static final String SURGERY_REPORT = "SURGERY REPORT";

	public SurgeryDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

	private Surgeries translate(TaggedSurgeryReportArrays surgeryReports) {
		SurgeryTranslator translator = new SurgeryTranslator();
		return translator.translate(surgeryReports);
	}

	private Surgeries filter(Surgeries surgeries, DateFilter filter) {
		return DateFilterer.filterByDate(surgeries, filter);
	}

	/**
	 * Calls the MDWS service EmrSvc.getSurgeryReports for the given patient
	 */
	@Override
	public Surgeries fetchSurgeries(PatientIdentifier patientIdentifier, DateFilter dateFilter) {
		EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);

		TaggedSurgeryReportArrays surgeryReports = authenticatedConnection.getPort().getSurgeryReports();
		
		MdwsResponseValidator.validateMdwsObject(surgeryReports);
		Surgeries surgeries = translate(surgeryReports);
		
		return filter(surgeries, dateFilter);
	}
	
	/**
	 * Calls the MDWS service EmrSvc.getSurgeryReports for the given patient
	 */
	@Override
	public Surgery fetchSurgery(PatientIdentifier patientIdentifier, String surgeryId) {
		Surgeries surgeries = fetchSurgeries(patientIdentifier, DateFilterFactory.createEmptyFilter());
		
		Surgery match = null;
		for(Surgery surgery : surgeries) {
			if(NullSafeStringComparer.areEqualIgnoreCase(surgery.getUniqueId(), surgeryId)) {
				match = surgery;
				break;
			}
		}
		
		return match;
	}

	/**
	 * Calls the MDWS service EmrSvc.getSurgeryReportsWithText for the given patient
	 */
	@Override
    public SurgeryReports fetchPatientSurgeryReports(PatientIdentifier patientIdentifier, DateFilter dateFilter) {
        EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);
        TaggedSurgeryReportArrays taggedSurgeryReportArrays = authenticatedConnection.getPort().getSurgeryReportsWithText();
        MdwsResponseValidator.validateMdwsObject(taggedSurgeryReportArrays);
        return translateToSurgeryReports(taggedSurgeryReportArrays, dateFilter);
    }

    private SurgeryReports translateToSurgeryReports(TaggedSurgeryReportArrays taggedSurgeryReportArrays, DateFilter dateFilter){
        SurgeryReportTranslator surgeryReportTranslator = new SurgeryReportTranslator();
        return surgeryReportTranslator.translate(taggedSurgeryReportArrays, dateFilter, SURGERY_REPORT);
    }
}
