package com.agilex.healthcare.mobilehealthplatform.mdws;

import java.util.concurrent.atomic.AtomicBoolean;

import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.PatientTO;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.*;

public class MdwsEmrDataLayer {

	protected MdwsConfiguration mdwsConfiguration;
	private final AtomicBoolean isConnectedToCDW = new AtomicBoolean(false);

	protected MdwsEmrDataLayer(MdwsConfiguration mdwsConfiguration) {
		this.mdwsConfiguration = mdwsConfiguration;
	}

	protected EmrConnection getAuthenticatedConnection(String siteCode) {
		EmrConnectionManager connectionManager = EmrConnectionManagerFactory.getInstance();
		EmrConnection connection = connectionManager.getAuthenticatedSiteConnection(mdwsConfiguration, siteCode);
		boolean connectedToCDW = (siteCode.equals(MdwsConfigurationHelper.CDW_SITECODE));
		isConnectedToCDW.set(connectedToCDW);
		return connection;
	}

	protected EmrConnection getAuthenticatedConnectionWithPatientSelect(PatientIdentifier patientIdentifier) {
		return getAuthenticatedConnectionWithPatientSelect(getSiteCodeFromPatientIdentifier(patientIdentifier), patientIdentifier.getUniqueId());
	}
		
	/**
	 * Calls the MDWS service EmrSvc.select
	 */
	protected EmrConnection getAuthenticatedConnectionWithPatientSelect(String siteCode, String patientId) {
        EmrConnection connection = getAuthenticatedConnection(siteCode);
        PatientTO result = null;
		try {
			result = connection.getPort().select(patientId);
		}
		catch (Exception e) {
			throw new MdwsResponseException(MdwsResponseFaultTOFilter.getResponseType(e.getMessage()), e.getMessage());
		}
        if (result.getFault() != null) {
        	throw new MdwsResponseException(result.getFault());
        }
        return connection;
    }
    
    protected String getSiteCodeFromPatientIdentifier(PatientIdentifier patientIdentifier){
		String siteCode = new MdwsConfigurationHelper().extractSiteCodeFromAssigningAuthority(patientIdentifier.getAssigningAuthority());
		return siteCode;
	}

    /**
     * This method exists in this class instead of @EmrConnection because the logic in method @getSiteCodeFromPatientIdentifier 
     * which swaps the site code to 204 if the patient identifier is ICN.
     */
    protected void disconnectFromCdwIfApplicable(EmrConnection emrConnection) {
		if (isConnectedToCDW.get() == true){
			emrConnection.disconnect();
			isConnectedToCDW.compareAndSet(true, false);
		}
	}

}
