package com.agilex.healthcare.mobilehealthplatform.mdws;


import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.PatientTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfigurationHelper;

public class MdwsOrderMgmtDataLayer {

	private static final String CDW_SITECODE = "204";
	
    protected MdwsConfiguration mdwsConfiguration;

    public MdwsOrderMgmtDataLayer(MdwsConfiguration mdwsConfiguration) {
        this.mdwsConfiguration = mdwsConfiguration;
    }

    protected OrderMgmtConnection getAuthenticatedConnection(String siteCode) {
        OrderMgmtConnectionManager connectionManager = OrderMgmtConnectionManagerFactory.getInstance();
        return connectionManager.getAuthenticatedSiteConnection(mdwsConfiguration, siteCode);
    }

    protected OrderMgmtConnection getAuthenticatedConnectionWithPatientSelect(PatientIdentifier patientIdentifier) {
		String siteCode = null;

		if ("ICN".equals(patientIdentifier.getAssigningAuthority())) {
			siteCode = CDW_SITECODE;
		} else {
			siteCode = new MdwsConfigurationHelper().extractSiteCodeFromAssigningAuthority(patientIdentifier.getAssigningAuthority());
		}

		String patientId = patientIdentifier.getUniqueId();
		OrderMgmtConnection connection = getAuthenticatedConnectionWithPatientSelected(siteCode, patientId);
		
		return connection;
	}
    
    /**
	 * Calls the MDWS service OrderMgmtSvc.select
	 */
	 protected OrderMgmtConnection getAuthenticatedConnectionWithPatientSelected(String siteCode, String patientId){
        OrderMgmtConnection connection = getAuthenticatedConnection(siteCode);
        PatientTO result = null;
		try {
			result = connection.getPort().select(patientId);
		}
        catch (Exception e) {
        	throw new MdwsResponseException(MdwsResponseFaultTOFilter.getResponseType(e.getMessage()), e.getMessage());
		}
        
		if (result.getFault() != null) {
			throw new MdwsResponseException(result.getFault());
		}
        return connection;
    }
}
