package com.agilex.healthcare.mobilehealthplatform.mdws.connection;


import java.io.Serializable;
import java.net.URL;
import java.util.Date;
import java.util.UUID;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;

import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.transport.http.HTTPConduit;
import org.apache.cxf.transports.http.configuration.HTTPClientPolicy;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.DataSourceArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.OrderMgmtSvc;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.OrderMgmtSvcSoap;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.security.AppUser;

public class OrderMgmtConnection implements Serializable {
//	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(LVOrderMgmtConnection.class);
    private static final long serialVersionUID = -978507159875320581L;

    private String id;
    private MdwsConfiguration configuration;
    private OrderMgmtSvcSoap port;
    private Date refreshedTimestamp;
    
    public OrderMgmtConnection() {
        this.id = UUID.randomUUID().toString();
    }

    public OrderMgmtConnection(OrderMgmtSvcSoap port) {
        this.id = UUID.randomUUID().toString();
        this.port = port;
    }


    /**
	 * Calls the MDWS service OrderMgmtSvc.connect if the authenticatedFlag is false - it will usually be true
	 * since most apps will login to a site with AuthorizationServices and that connection is used from that point on.
	 */
	public OrderMgmtConnection(MdwsConfiguration configuration, String siteCode, boolean authenticateFlag) {
        this.id = UUID.randomUUID().toString();
        this.configuration = configuration;
        this.port = establishConnection(configuration, siteCode, authenticateFlag);
        this.refreshedTimestamp = new Date();
    }

    private OrderMgmtSvcSoap establishConnection(MdwsConfiguration configuration, String siteCode, boolean authenticated) {
    	String baseUrl = "";
    	MhpUser mhpUser = null;
    	
		if (authenticated){
			mhpUser = getMhpUser();
			baseUrl = mhpUser.getMdwsInfo().getBaseUrl();
		}else{
			baseUrl = configuration.getEndPointBaseUrl();
		}
		
		String url = baseUrl + configuration.getEndPointUrl();
		
		URL wsdlLocation = constructWsdlUrl(url);
        OrderMgmtSvc schedulingSvc = null;
        try {
        	schedulingSvc = new OrderMgmtSvc(wsdlLocation, new QName("http://mdws.domain/OrderMgmtSvc", "OrderMgmtSvc"));
        } catch (Throwable t) {
			throw new MdwsResponseException(MdwsResponseException.UserMessages.CONNECTION_ERROR, t.getMessage());
		}
        OrderMgmtSvcSoap port = schedulingSvc.getOrderMgmtSvcSoap();

        ((BindingProvider)port).getRequestContext().put(BindingProvider.SESSION_MAINTAIN_PROPERTY, true);
        ((BindingProvider)port).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, url);
        ((BindingProvider)port).getRequestContext().put("javax.xml.ws.client.connectionTimeout", configuration.getWsConnectTimeout());
        ((BindingProvider)port).getRequestContext().put("javax.xml.ws.client.receiveTimeout", configuration.getWsRequestTimeout());

        if(authenticated) {
        	setSessionOnPort(port, mhpUser.getMdwsInfo().getSessionId());
        }
        else {
	        DataSourceArray dsa = null;
	        try {
	        	dsa = port.connect(siteCode);
	        } catch (Throwable t) {
				throw new MdwsResponseException(MdwsResponseException.UserMessages.CONNECTION_ERROR, t.getMessage());
			}
	        if (dsa.getFault() != null)
				throw new MdwsResponseException(MdwsResponseException.UserMessages.CONNECTION_ERROR, dsa.getFault());
        }

        return port;
    }
    
    private URL constructWsdlUrl(String endPointUrl){
        String wsdlLocation = endPointUrl + "?WSDL";
        URL wsdlLocationURL = null;
        try {
            wsdlLocationURL = new URL(wsdlLocation);
        } catch (Exception e) {
        	throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_WSDL, e.getMessage());
        }

        return wsdlLocationURL;
    }

    public Date getRefreshedTimestamp() {
        return refreshedTimestamp;
    }

    public void refreshTimestamp() {
        this.refreshedTimestamp = new Date();
    }

    public String getId() {
        return id;
    }

    public OrderMgmtSvcSoap getPort() {
        return port;
    }

    public MdwsConfiguration getConfiguration() {
        return this.configuration;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((id == null) ? 0 : id.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        OrderMgmtConnection other = (OrderMgmtConnection) obj;
        if (id == null) {
            if (other.id != null)
                return false;
        } else if (!id.equals(other.id))
            return false;
        return true;
    }
    
	private  MhpUser getMhpUser(){
        SecurityContext ctx = SecurityContextHolder.getContext();

        AppUser appUser = (AppUser) ctx.getAuthentication().getPrincipal();
        MhpUser mhpUser = appUser.getMhpUser();
		mhpUser.setUserName(appUser.getUsername());

        return mhpUser;
	}
	
	private void setSessionOnPort(OrderMgmtSvcSoap port, String sessionId) {
		Client client = ClientProxy.getClient(port);

		HTTPClientPolicy policy = new HTTPClientPolicy();
		HTTPConduit httpConduit = (HTTPConduit)client.getConduit();
		
		policy.setCookie("ASP.NET_SessionId=" + sessionId);
		
		httpConduit.setClient(policy);
	}
}
