package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Admission;
import com.agilex.healthcare.mobilehealthplatform.domain.Admissions;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.HospitalLocationTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.InpatientStayTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.SiteTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedInpatientStayArray;

public class AdmissionTranslator {
	
	public Admissions translate(TaggedInpatientStayArray arr) {
		Admissions retValue = new Admissions();
		
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedInpatientStayArray was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getStays() == null || arr.getStays().getInpatientStayTO() == null)
			return retValue;
		
		Iterator<InpatientStayTO> it = arr.getStays().getInpatientStayTO().iterator();
		while (it.hasNext()) {
			InpatientStayTO inpatientStay = it.next();
			
			//Perform Error Checking
			if (inpatientStay.getFault() != null)
				throw new MdwsResponseException(inpatientStay.getFault());
			
			Admission admission = createAdmission(inpatientStay);
			
			retValue.add(admission);
		}
		return retValue;
	}

	private Admission createAdmission(InpatientStayTO inpatientStay) {
		Admission admission = new Admission();
		
		String movementCheckinId = inpatientStay.getMovementCheckinId();
		String dischargeTimestamp = inpatientStay.getDischargeTimestamp();
		String admitTimestamp = inpatientStay.getAdmitTimestamp();
		Date dtDischargeTimestamp = MdwsDateTranslationUtil.extractDateOrNull(dischargeTimestamp);
		Date dtAdmitTimestamp = MdwsDateTranslationUtil.extractDateOrNull(admitTimestamp);
		
		admission.setUniqueId(movementCheckinId);
		admission.setDischargeDate(dtDischargeTimestamp);
		admission.setAdmissionDate(dtAdmitTimestamp);
		

		HospitalLocationTO location = inpatientStay.getLocation();
		if (location != null) {
			//Perform Error Checking
			if (location.getFault() != null)
				throw new MdwsResponseException(location.getFault());
			
			String facilityName = location.getName();
			String id = location.getId();
			String type = location.getType();
			
			admission.setFacilityName(facilityName);
			admission.setLocationId(id);
			admission.setLocationType(type);

			SiteTO facility = location.getFacility();
			if (facility != null) {
				//Perform Error Checking
				if (facility.getFault() != null)
					throw new MdwsResponseException(facility.getFault());
				
				String sitecode = facility.getSitecode();
				
				admission.setFacilityCode(sitecode);
			}
		}
		return admission;
	}
}
