package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Allergies;
import com.agilex.healthcare.mobilehealthplatform.domain.Allergy;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.AllergyTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.ArrayOfSymptomTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.SymptomTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedAllergyArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedAllergyArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedText;
import com.agilex.healthcare.utility.NullChecker;

public class AllergyTranslator {

	public Allergies translate(TaggedAllergyArrays arr) {
		Allergies retValue = new Allergies();
		
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedAllergyArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedAllergyArray() == null)
			return retValue;
		
		Iterator<TaggedAllergyArray> itTagged = arr.getArrays().getTaggedAllergyArray().iterator();
		while (itTagged.hasNext()) {
			TaggedAllergyArray taggedAllergy = itTagged.next();
			//Perform Error Checking
			if (taggedAllergy.getFault() != null)
				throw new MdwsResponseException(taggedAllergy.getFault());
			
			if (taggedAllergy.getAllergies() == null || taggedAllergy.getAllergies().getAllergyTO() == null)
				continue;
			
			Iterator<AllergyTO> it = taggedAllergy.getAllergies().getAllergyTO().iterator();
			while (it.hasNext()) {
				AllergyTO allergyTo = it.next();
				//Perform Error Checking
				if (allergyTo.getFault() != null)
					throw new MdwsResponseException(allergyTo.getFault());

				Allergy allergy = createAllergy(allergyTo);
				retValue.add(allergy);
			}
		}
		return retValue;
	}

	private Allergy createAllergy(AllergyTO allergyTo) {
		String allergenType = allergyTo.getAllergenType();
		String allergenName = allergyTo.getAllergenName();
		String comment = allergyTo.getComment();
		String allergenId = allergyTo.getAllergenId();
		String severity = allergyTo.getSeverity();
		String sDate = allergyTo.getTimestamp();
		Date extractDate = MdwsDateTranslationUtil.extractDateOrNull(sDate);
		String sourceSystem = extractFacility(allergyTo);
		//The original code had reaction being set twice - sticking with 2nd value since that's what was finally in the object before.
		//String reaction = allergyTo.getReaction();
		String reaction2 = extractReaction(allergyTo);
		
		
		Allergy allergy = new Allergy();
		allergy.setCategory(allergenType);
		allergy.setSubstance(allergenName);
		allergy.setNote(comment);
		allergy.setUniqueId(allergenId);
		allergy.setSeverity(severity);
		allergy.setOnsetDate(extractDate);
		allergy.setSourceSystem(sourceSystem);
		//The original code had reaction being set twice - sticking with 2nd value since that's what was finally in the object before.
		//allergy.setReaction(reaction);
		allergy.setReaction(reaction2);
		return allergy;
	}
	
	private String extractFacility(AllergyTO to) {
		TaggedText facility = to.getFacility();
		
		if(facility == null)
			return "";
		
		//Perform Error Checking
		if (facility.getFault() != null)
			throw new MdwsResponseException(facility.getFault());
		return facility.getText();
	}
	
	private String extractReaction(AllergyTO to) {
		String retValue = "";
		ArrayOfSymptomTO symptoms = to.getReactions();
		if(symptoms == null || symptoms.getSymptomTO() == null)
			return retValue;
		
		Iterator<SymptomTO> it = symptoms.getSymptomTO().iterator();
		
		while (it.hasNext()) {
			SymptomTO symptom = it.next();
			//Perform Error Checking
			if (symptom.getFault() != null)
				throw new MdwsResponseException(symptom.getFault());
			
			if(NullChecker.isNotNullish(symptom.getName()))
				retValue += (retValue.isEmpty() ? "" : ",") + symptom.getName();
		}
		return retValue;
	}
}
