package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.*;

import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.*;

public class AppointmentTranslator {
	
	public Appointments translate(TaggedAppointmentArrays arr) {
		Appointments retValue = new Appointments();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedAppointmentArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedAppointmentArray() == null)
			return retValue;
		
		Iterator<TaggedAppointmentArray> itTagged = arr.getArrays().getTaggedAppointmentArray().iterator();
		while (itTagged.hasNext()) {
			TaggedAppointmentArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());
			
			if (tagged.getAppts() == null || tagged.getAppts().getAppointmentTO() == null)
				continue;
			
			Iterator<AppointmentTO> it = tagged.getAppts().getAppointmentTO().iterator();
			while (it.hasNext()) {
				AppointmentTO to = it.next();
				//Perform Error Checking
				if (to.getFault() != null)
					throw new MdwsResponseException(to.getFault());
				
				Appointment appt = createAppointment(to);
				retValue.add(appt);
			}
		}
		
		return retValue;
	}

	private Appointment createAppointment(AppointmentTO to) {
		Appointment appt = new Appointment();
		appt.setUniqueId(to.getId());
		
		String clinicVisitId = to.getVisitId();
		if (clinicVisitId != null) {
			appt.setStatus("kept");
		} else {
			String currentStatus = to.getCurrentStatus();
			appt.setStatus(currentStatus);
		}
		
		Date extractDate = MdwsDateTranslationUtil.extractDateOrNull(to.getTimestamp());
		String facilityName = extractFacility(to);
		String providerName = to.getProviderName();
		
		appt.setAppointmentStartDate(extractDate);
		appt.setFacilityName(facilityName);
//				TODO Integration - re-enable when MDWS changes are merged
		appt.setClinicVisitId(clinicVisitId);
		appt.setProviderName(providerName);
		
		HospitalLocationTO mdwsHospitalLocation = to.getClinic();
		if (mdwsHospitalLocation != null) {
			String clinicName = mdwsHospitalLocation.getName();
			String clinicId = mdwsHospitalLocation.getId();
			String clinicType = mdwsHospitalLocation.getType();
			String reasonForVisit = "";
			
			TaggedText stopCode = mdwsHospitalLocation.getStopCode();
			if(stopCode != null) {
				//Perform Error Checking
				if (stopCode.getFault() != null)
					throw new MdwsResponseException(stopCode.getFault());
				else
					reasonForVisit = stopCode.getText();
			}
			
			appt.setClinicName(clinicName);
			appt.setClinicId(clinicId);
			appt.setReasonForVisit(reasonForVisit);
			appt.setClinicType(clinicType);
		}
		return appt;
	}
	
	private String extractFacility(AppointmentTO appointment) {
		TaggedText facility = appointment.getFacility();
		
		if(facility == null)
			return "";
		
		//Perform Error Checking
		if (facility.getFault() != null)
			throw new MdwsResponseException(facility.getFault());
		return facility.getText();
	}

}
