package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.Consult;
import com.agilex.healthcare.mobilehealthplatform.domain.Consults;
import com.agilex.healthcare.mobilehealthplatform.enumeration.ConsultStatus;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.ConsultTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedConsultArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedConsultArrays;

public class ConsultTranslator {

	public Consults translateFromTaggedConsultArrays(TaggedConsultArrays arr, ConsultFilter filter) {
		Consults retValue = new Consults();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedConsultArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedConsultArray() == null)
			return retValue;
		
		if (filter == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_INPUT, "filter cannot be null");
		
		Iterator<TaggedConsultArray> itTagged = arr.getArrays().getTaggedConsultArray().iterator();
		while (itTagged.hasNext()) {
			TaggedConsultArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getConsults() == null || tagged.getConsults().getConsultTO() == null)
				continue;

			Iterator<ConsultTO> it = tagged.getConsults().getConsultTO().iterator();
			while (it.hasNext()) {
				ConsultTO to = it.next();
				//Perform Error Checking
				if (to.getFault() != null)
					throw new MdwsResponseException(to.getFault());
				
				Consult consult = createConsult(to, filter);
				//NOTICE that unlike the other translators, this one can return null for a Consult.
				//They have written their code such that they do not want empty objects returned and it will actually
				//fail later when they attempt to sort the collection since the "creation date" would be null.
				if (consult != null)
					retValue.add(consult);
			}
		}

		return retValue;
	}
	
	public Consult createConsult(ConsultTO mdwsConsult, ConsultFilter filter) {
		Consult consult = new Consult();
		assert mdwsConsult != null;
		String timestamp = mdwsConsult.getTimestamp();
		Date createdDate = MdwsDateTranslationUtil.extractDateOrNull(timestamp);
		
		ConsultStatus status = ConsultStatus.lookUpByName(mdwsConsult.getStatus());
		if (!returnedStatusMatchesRequest(status, filter))
			return null;
		Date startDate = filter.getStartDate();
		Date endDate = filter.getEndDate();
		if (startDate != null && createdDate.before(startDate))
			return null;
		if (endDate != null && createdDate.after(endDate))
			return null;
		
		String uniqueId = mdwsConsult.getId();
		Long consultId = Long.valueOf(uniqueId);
		String text = mdwsConsult.getText();
		String title = mdwsConsult.getTitle();
		
		consult.setConsultId(consultId);
		consult.setConsultIEN(uniqueId);
		consult.setUniqueId(uniqueId);
		consult.setCreatedDate(createdDate);
		consult.setStatus(status);
		consult.setText(text);
		consult.setTitle(title);
		return consult;
	}
	
	private boolean returnedStatusMatchesRequest(ConsultStatus status, ConsultFilter filter) {
		boolean match = true;
		List<ConsultStatus> statuses = filter.getStatuses();
		boolean filterByStatus = (filter.getStatuses().size() > 0);
		
		if (filterByStatus && (!statuses.contains(status)))
			match = false;
		
		return match;
	}	
}
