package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Facility;
import com.agilex.healthcare.mobilehealthplatform.domain.PathologyReport;
import com.agilex.healthcare.mobilehealthplatform.domain.PathologyReports;
import com.agilex.healthcare.mobilehealthplatform.domain.Specimen;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.CytologyRpt;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.LabSpecimenTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedCytologyRptArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedCytologyRptArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedText;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;

public class CytologyReportTranslator {

    public PathologyReports translate(TaggedCytologyRptArrays arr, DateFilter dateFilter){
    	PathologyReports retValue = new PathologyReports();
    	if (arr == null)
    		throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedCytologyRptArrays was null");
    	if (arr.getFault() != null)
    		throw new MdwsResponseException(arr.getFault());
    	if (arr.getArrays() == null || arr.getArrays().getTaggedCytologyRptArray() == null)
    		return retValue;

    	Iterator<TaggedCytologyRptArray> itTagged = arr.getArrays().getTaggedCytologyRptArray().iterator();
    	while (itTagged.hasNext()) {
    		TaggedCytologyRptArray tagged = itTagged.next();
    		//Perform Error Checking
    		if (tagged.getFault() != null)
    			throw new MdwsResponseException(tagged.getFault());

    		if (tagged.getRpts() == null || tagged.getRpts().getCytologyRpt() == null)
    			continue;

    		Iterator<CytologyRpt> it = tagged.getRpts().getCytologyRpt().iterator();
    		while (it.hasNext()) {
    			CytologyRpt mdwsCytologyRpt = it.next();
    			//Perform Error Checking
    			if (mdwsCytologyRpt.getFault() != null)
    				throw new MdwsResponseException(mdwsCytologyRpt.getFault());
    			
    			if(isCytologyReportWithinRequiredDateRange(mdwsCytologyRpt, dateFilter)){
    				PathologyReport pathologyReport = createPathologyReport(mdwsCytologyRpt);
    				retValue.add(pathologyReport);
                }
    		}
    	}

    	return retValue;
    }

    private PathologyReport createPathologyReport(CytologyRpt cytologyRpt){
        TaggedText cytologyRptFacility = cytologyRpt.getFacility();
		LabSpecimenTO labSpecimenTO = cytologyRpt.getSpecimen();
		
		//Perform Error Checking
		if (cytologyRptFacility == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "cytologyRptFacility was null");
		if (cytologyRptFacility.getFault() != null)
			throw new MdwsResponseException(cytologyRptFacility.getFault());
		//Perform Error Checking
		if (labSpecimenTO == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "labSpecimenTO was null");
		if (labSpecimenTO.getFault() != null)
			throw new MdwsResponseException(labSpecimenTO.getFault());

        String title = cytologyRpt.getTitle();
		String facilityCode = cytologyRptFacility.getText();
		String facilityName = cytologyRptFacility.getTag();
		String id = labSpecimenTO.getId();
		String name = labSpecimenTO.getName();
		String collectionDate = labSpecimenTO.getCollectionDate();
		String accessionNum = labSpecimenTO.getAccessionNum();
		String exam = cytologyRpt.getExam();
		
    	Facility facility = new Facility();
    	Specimen specimen = new Specimen();
        PathologyReport pathologyReport = new PathologyReport();

        facility.setFacilityCode(facilityCode);
        facility.setName(facilityName);
        
        specimen.setId(id);
        specimen.setName(name);
        if(NullChecker.isNotNullish(collectionDate))
            specimen.setCollectionDate(DateHelper.parseVistaDateTime(collectionDate));
        specimen.setAccessionNumber(accessionNum);

        pathologyReport.setTitle(title);
		pathologyReport.setSpecimen(specimen);
        pathologyReport.setExam(exam);
        pathologyReport.setFacility(facility);
        
        return pathologyReport;
    }

    private boolean isCytologyReportWithinRequiredDateRange(CytologyRpt cytologyRpt, DateFilter dateFilter){
        Date dateTime = null;
        if(NullChecker.isNotNullish(cytologyRpt.getTimestamp())){
            dateTime = DateHelper.parseVistaDateTime(cytologyRpt.getTimestamp());
        }
        else if(NullChecker.isNotNullish(cytologyRpt.getSpecimen().getCollectionDate())){
            dateTime = DateHelper.parseVistaDateTime(cytologyRpt.getSpecimen().getCollectionDate());
        }
        if(dateTime != null){
            return (dateTime.equals(dateFilter.getStartDate()) || dateTime.after(dateFilter.getStartDate())) && (dateTime.equals(dateFilter.getEndDate()) || dateTime.before(dateFilter.getEndDate()));
        }
        return false;
    }
}
