package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Address;
import com.agilex.healthcare.mobilehealthplatform.domain.Addresses;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientDemographics;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.AddressTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.ArrayOfAddressTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.ArrayOfDemographicSetTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.ArrayOfPhoneNumTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.ArrayOfString;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.DemographicSetTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.PhoneNumTO;

public class DemographicTranslator {

    private final String WORK_PHONE = "Work Phone";
    private final String HOME_PHONE = "Home Phone";

    public PatientDemographics translate(ArrayOfDemographicSetTO arr) {
    	
    	PatientDemographics retValue = new PatientDemographics();
    	if (arr == null)
    		throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "ArrayOfDemographicSetTO was null");
    	//NOTICE No Fault is returned here - like other Translators.
    	//if (arr.getFault() != null)
    	//	throw new MdwsResponseException(arr.getFault());
    	if (arr.getDemographicSetTO() == null)
    		return retValue;

    	Iterator<DemographicSetTO> itDemographic = arr.getDemographicSetTO().iterator();
    	if (itDemographic.hasNext()) { //NOTICE this is an if and not a while - like other Translators.
    		DemographicSetTO mdwsDemographicSet = itDemographic.next();
    		//NOTICE No Fault is returned here - like other Translators.
        	//Perform Error Checking
    		//if (demographicSet.getFault() != null)
    		//	throw new MdwsResponseException(demographicSet.getFault());

    		setPhoneNumbers(retValue, mdwsDemographicSet);
	        setAddress(retValue, mdwsDemographicSet);
	        setEmailAddress(retValue, mdwsDemographicSet);
    	}

    	return retValue;
    }

    void setEmailAddress(PatientDemographics patientDemographics, DemographicSetTO mdwsDemographicSet) {
    	ArrayOfString mdwsEmailAddresses = mdwsDemographicSet.getEmailAddresses();
    	if (mdwsEmailAddresses == null || mdwsEmailAddresses.getString() == null) {
    		patientDemographics.setEmailAddress("");
    		return;
    	}
        String emailAddresses = "";
        for (String emailAddress : mdwsEmailAddresses.getString()) {
            emailAddresses += (emailAddresses.length() > 0 ? "," : "") + emailAddress;
        }
        patientDemographics.setEmailAddress(emailAddresses);
    }

    void setPhoneNumbers(PatientDemographics patientDemographics, DemographicSetTO mdwsDemographicSet) {
        ArrayOfPhoneNumTO mdwsPhones = mdwsDemographicSet.getPhones();
        if (mdwsPhones == null || mdwsPhones.getPhoneNumTO() == null)
        	return;
        
        Iterator<PhoneNumTO> it = mdwsPhones.getPhoneNumTO().iterator();
        while (it.hasNext()) {
        	PhoneNumTO mdwsPhone = it.next();
        	
        	//Perform Error Checking
    		if (mdwsPhone.getFault() != null)
    			throw new MdwsResponseException(mdwsPhone.getFault());
        	
        	String description = mdwsPhone.getDescription();
            String phoneNumber = this.formatPhoneNumber(mdwsPhone);
            
			if (WORK_PHONE.equalsIgnoreCase(description))
                patientDemographics.setPhoneNumberWork(phoneNumber);
            else if (HOME_PHONE.equalsIgnoreCase(description))
                patientDemographics.setPhoneNumberHome(phoneNumber);
            else
                patientDemographics.setPhoneNumberMobile(phoneNumber);
        }
    }

    void setAddress(PatientDemographics patientDemographics, DemographicSetTO mdwsDemographicSet) {
        ArrayOfAddressTO mdwsAddresses = mdwsDemographicSet.getAddresses();
        if (mdwsAddresses == null || mdwsAddresses.getAddressTO() == null)
        	return;

        Addresses addresses = new Addresses();
        Iterator<AddressTO> it = mdwsAddresses.getAddressTO().iterator();
        while (it.hasNext()) {
        	AddressTO mdwsAddress = it.next();
        	//Perform Error Checking
    		if (mdwsAddress.getFault() != null)
    			throw new MdwsResponseException(mdwsAddress.getFault());
        	
    		//NOTE: It appears from the prior code that the last address in the list must be the one that they
    		//want to set in the patientDemographics (while still keeping all of the other addresses).
        	Address address = formatAddress(mdwsAddress);
            addresses.add(address);
            patientDemographics.setAddress(address);
        }
        patientDemographics.setAddresses(addresses);
    }

    Address formatAddress(AddressTO mdwsAddress) {
    	//Perform Error Checking
		if (mdwsAddress.getFault() != null)
			throw new MdwsResponseException(mdwsAddress.getFault());
		
        Address address = new Address();

        String streetAddress1 = mdwsAddress.getStreetAddress1();
        String streetAddress2 = mdwsAddress.getStreetAddress2();
        String streetAddress3 = mdwsAddress.getStreetAddress3();
        String city = mdwsAddress.getCity();
        String state = mdwsAddress.getState();
        String zipcode = mdwsAddress.getZipcode();
		
        address.setStreetAddressLine1(streetAddress1);
        address.setStreetAddressLine2(streetAddress2);
        address.setStreetAddressLine3(streetAddress3);
        address.setCity(city);
        address.setState(state);
        address.setZipCode(zipcode);

        return address;
    }

    String formatPhoneNumber(PhoneNumTO mdwsPhoneNumber) {
    	//Perform Error Checking
		if (mdwsPhoneNumber.getFault() != null)
			throw new MdwsResponseException(mdwsPhoneNumber.getFault());
		
        String areaCode = mdwsPhoneNumber.getAreaCode() == null ? "" : mdwsPhoneNumber.getAreaCode();
        String exchange = mdwsPhoneNumber.getExchange() == null ? "" : mdwsPhoneNumber.getExchange();
        String number = mdwsPhoneNumber.getNumber() == null ? "" : mdwsPhoneNumber.getNumber();

        // (123)-456-7890
        return new StringBuilder().append("(").append(areaCode).append(")-").append(exchange).append("-").append(number).toString();
    }

}
