package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Facilities;
import com.agilex.healthcare.mobilehealthplatform.domain.Facility;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.SiteArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.SiteTO;

public class FacilityTranslator {
	
	public Facilities translateSitesToFacilities(SiteArray arr) {
		Facilities retValue = new Facilities();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "SiteArray was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getSites() == null || arr.getSites().getSiteTO() == null)
			return retValue;

		Iterator<SiteTO> mdwsSites = arr.getSites().getSiteTO().iterator();
		while (mdwsSites.hasNext()) {
			SiteTO mdwsSite = mdwsSites.next();
			//Perform Error Checking
			if (mdwsSite.getFault() != null)
				throw new MdwsResponseException(mdwsSite.getFault());

			Facility facility = createFacility(mdwsSite);
			retValue.add(facility);
		}

		return retValue;
	}
	
	private Facility createFacility(SiteTO site) {
		Facility facility = new Facility();
		
		String address = site.getAddress();
		String parentSiteId = site.getParentSiteId();
		String name = site.getName();
		String siteType = site.getSiteType();
		String systemName = site.getSystemName();
		String state = site.getState();
		String city = site.getCity();
		
		facility.setAddress(address);
		facility.setParentSiteCode(parentSiteId);
		facility.setName(name);
		facility.setType(siteType);
		facility.setFacilityCode(systemName);
		facility.setState(state);
		facility.setCity(city);
		
		return facility;
	}
}
