package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Clinic;
import com.agilex.healthcare.mobilehealthplatform.domain.Clinics;
import com.agilex.healthcare.mobilehealthplatform.domain.HospitalLocation;
import com.agilex.healthcare.mobilehealthplatform.domain.HospitalLocations;
import com.agilex.healthcare.mobilehealthplatform.domain.Ward;
import com.agilex.healthcare.mobilehealthplatform.domain.Wards;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.HospitalLocationTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedHospitalLocationArray;

public class HospitalLocationTranslator {

	public HospitalLocations translateFromTaggedHospitalLocationArray(TaggedHospitalLocationArray arr) {
		HospitalLocations retValue = new HospitalLocations();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedHospitalLocationArray was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getLocations() == null || arr.getLocations().getHospitalLocationTO() == null)
			return retValue;

		Iterator<HospitalLocationTO> hospitalLocationTOs = arr.getLocations().getHospitalLocationTO().iterator();
		while (hospitalLocationTOs.hasNext()) {
			HospitalLocationTO hospitalLocationTO = hospitalLocationTOs.next();
			//Perform Error Checking
			if (hospitalLocationTO.getFault() != null)
				throw new MdwsResponseException(hospitalLocationTO.getFault());

			HospitalLocation hospitalLocation = new HospitalLocation();
			String id = hospitalLocationTO.getId();
			String name = hospitalLocationTO.getName();
			hospitalLocation.setIen(id);
			hospitalLocation.setName(name);
			retValue.add(hospitalLocation);
		}

		return retValue;
	}

	public Wards translateWardsFromTaggedHospitalLocationArray(TaggedHospitalLocationArray arr, String siteCode) {
		Wards retValue = new Wards();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedHospitalLocationArray was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getLocations() == null || arr.getLocations().getHospitalLocationTO() == null)
			return retValue;

		Iterator<HospitalLocationTO> hospitalLocationTOs = arr.getLocations().getHospitalLocationTO().iterator();
		while (hospitalLocationTOs.hasNext()) {
			HospitalLocationTO hospitalLocationTO = hospitalLocationTOs.next();
			//Perform Error Checking
			if (hospitalLocationTO.getFault() != null)
				throw new MdwsResponseException(hospitalLocationTO.getFault());

			Ward ward = new Ward();
			String ien = hospitalLocationTO.getId();
			String name = hospitalLocationTO.getName();
			
			ward.setId(ien);
			ward.setName(name);
			ward.setFacilityCode(siteCode);
			retValue.add(ward);
		}

		return retValue;
	}

	public Clinics translateClinicsFromTaggedHospitalLocationArray(TaggedHospitalLocationArray arr, String siteCode) {
		Clinics retValue = new Clinics();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedHospitalLocationArray was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getLocations() == null || arr.getLocations().getHospitalLocationTO() == null)
			return retValue;

		Iterator<HospitalLocationTO> hospitalLocationTOs = arr.getLocations().getHospitalLocationTO().iterator();
		while (hospitalLocationTOs.hasNext()) {
			HospitalLocationTO hospitalLocationTO = hospitalLocationTOs.next();
			//Perform Error Checking
			if (hospitalLocationTO.getFault() != null)
				throw new MdwsResponseException(hospitalLocationTO.getFault());

			Clinic clinic = new Clinic();
			String ien = hospitalLocationTO.getId();
			String name = hospitalLocationTO.getName();
			
			clinic.setId(ien);
			clinic.setName(name);
			clinic.setFacilityCode(siteCode);
			retValue.add(clinic);
		}

		return retValue;
	}
}
