package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Medication;
import com.agilex.healthcare.mobilehealthplatform.domain.Medications;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.MedicationTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.TaggedMedicationArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.TaggedMedicationArrays;
import com.agilex.healthcare.utility.DateHelper;

public class MedicationRefillTranslator {

	public Medications translateTaggedMedications(TaggedMedicationArrays arr) {		
		Medications retValue = new Medications();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedMedicationArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedMedicationArray() == null)
			return retValue;

		Iterator<TaggedMedicationArray> itMedArr = arr.getArrays().getTaggedMedicationArray().iterator();
		while (itMedArr.hasNext()) {
			TaggedMedicationArray medArray = itMedArr.next();
			//Perform Error Checking
			if (medArray.getFault() != null)
				throw new MdwsResponseException(medArray.getFault());

			if (medArray.getMeds() == null || medArray.getMeds().getMedicationTO() == null)
				continue;

			Iterator<MedicationTO> itMedTO = medArray.getMeds().getMedicationTO().iterator();
			while (itMedTO.hasNext()) {
				MedicationTO mdwsMedication = itMedTO.next();
				//Perform Error Checking
				if (mdwsMedication.getFault() != null)
					throw new MdwsResponseException(mdwsMedication.getFault());

				Medication med = createMedication(mdwsMedication);
				retValue.add(med);
			}
		}

		return retValue;
	}
	
	public Medication createMedication(MedicationTO mdwsMedication) {
		Medication medication = new Medication();
		
		String id = mdwsMedication.getId();
		String name = mdwsMedication.getName();
		String daysSupply = mdwsMedication.getDaysSupply();
		String orderId = mdwsMedication.getOrderId();
		String rxNum = mdwsMedication.getRxNum();
		String refills = mdwsMedication.getRefills();
		String quantity = mdwsMedication.getQuantity();
		String sig = mdwsMedication.getSig();
		String expirationDate = mdwsMedication.getExpirationDate();
		Date dtExpirationDate = parseDateOrNull(expirationDate);
		String lastFillDate = mdwsMedication.getLastFillDate();
		Date dtLastFillDate = parseDateOrNull(lastFillDate);
		String status = mdwsMedication.getStatus();
		
		medication.setUniqueId(id);
		medication.setDrugName(name);
		medication.setDaysSupply(daysSupply);
		medication.setOrderNumber(orderId);
		medication.setPrescriptionId(rxNum);
		medication.setRefills(refills);
		medication.setQuantity(quantity);
		medication.setSig(sig);
		medication.setExpiredDate(dtExpirationDate);
		medication.setLastFilledDate(dtLastFillDate);
		medication.setEndDate(dtExpirationDate);
		
		medication.setStatus(status);

		return medication;
	}
	
	private Date parseDateOrNull(String vistaDate) {
		try {
			return DateHelper.parse(vistaDate, DateHelper.VISTA_DOB_DATE_FORMAT);
		} catch (RuntimeException e) {
			return null;
		}
	}
 } 
