package com.agilex.healthcare.mobilehealthplatform.mdws.translator;


import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Author;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNote;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNotes;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.AuthorTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.NoteTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedNoteArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedNoteArrays;
import com.agilex.healthcare.utility.DateHelper;

public class NoteTranslator {

    public ProviderNotes translateFromTaggedNoteArrays(TaggedNoteArrays arr, String documentType){
    	ProviderNotes retValue = new ProviderNotes();
    	if (arr == null)
    		throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedNoteArrays was null");
    	if (arr.getFault() != null)
    		throw new MdwsResponseException(arr.getFault());
    	if (arr.getArrays() == null || arr.getArrays().getTaggedNoteArray() == null)
    		return retValue;

    	Iterator<TaggedNoteArray> itTagged = arr.getArrays().getTaggedNoteArray().iterator();
    	while (itTagged.hasNext()) {
    		TaggedNoteArray tagged = itTagged.next();
    		//Perform Error Checking
    		if (tagged.getFault() != null)
    			throw new MdwsResponseException(tagged.getFault());

    		if (tagged.getNotes() == null || tagged.getNotes().getNoteTO() == null)
    			continue;

    		Iterator<NoteTO> it = tagged.getNotes().getNoteTO().iterator();
    		while (it.hasNext()) {
    			NoteTO mdwsNote = it.next();
    			//Perform Error Checking
    			if (mdwsNote.getFault() != null)
    				throw new MdwsResponseException(mdwsNote.getFault());

    			ProviderNote providerNote = createProviderNote(mdwsNote, documentType);
    			retValue.add(providerNote);
    		}
    	}

    	return retValue;
    }

    private ProviderNote createProviderNote(NoteTO noteTO, String documentType){
    	String id = noteTO.getId();
        String timestamp = noteTO.getTimestamp();
        Date dtTimestamp = DateHelper.parseVistaDateTime(timestamp);
        String admitTimestamp = noteTO.getAdmitTimestamp();
        Date dtAdmitTimestamp = DateHelper.parseVistaDateTime(admitTimestamp);
        String dischargeTimestamp = noteTO.getDischargeTimestamp();
        Date dtDischargeTimestamp = DateHelper.parseVistaDateTime(dischargeTimestamp);
		String localTitle = noteTO.getLocalTitle();
		String standardTitle = noteTO.getStandardTitle();
		String text = noteTO.getText();
		AuthorTO author = noteTO.getAuthor();
		AuthorTO approvedBy = noteTO.getApprovedBy();
		
		ProviderNote providerNote = new ProviderNote();
        providerNote.setId(id);
        providerNote.setTimestamp(dtTimestamp);
        providerNote.setEntryDate(dtAdmitTimestamp);
        providerNote.setDischargeDate(dtDischargeTimestamp);
        providerNote.setLocalTitle(localTitle);
        providerNote.setStandardTitle(standardTitle);
        
        if(author != null) {
    		//Perform Error Checking
    		if (author.getFault() != null)
    			throw new MdwsResponseException(author.getFault());
			String authorId = author.getId();
			String authorName = author.getName();
			String authorSignature = author.getSignature();
			Author providerNoteAuthor = new Author(authorId,authorName,authorSignature);
			providerNote.setAuthor(providerNoteAuthor);
		}
        
        providerNote.setText(text);
        if(approvedBy != null) {
    		//Perform Error Checking
    		if (approvedBy.getFault() != null)
    			throw new MdwsResponseException(approvedBy.getFault());
			String approvedById = approvedBy.getId();
			String approvedByName = approvedBy.getName();
			String approvedBySignature = approvedBy.getSignature();
			Author providerNoteApprovedByAuthor = new Author(approvedById, approvedByName, approvedBySignature);
			providerNote.setApprovedBy(providerNoteApprovedByAuthor);
		}

//            providerNote.setLocation("7A GEN MED");
        providerNote.setDocumentType(documentType);
        return providerNote;
    }
}
