package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.Order;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderLocation;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderSignStatus;
import com.agilex.healthcare.mobilehealthplatform.domain.Orders;
import com.agilex.healthcare.mobilehealthplatform.domain.OrdersStatus;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.OrderTypeTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.OrdersTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.TaggedOrdersArray;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;

public class OrderCustomTranslator {
	
	public Orders translateFromTaggedOrderArrays(TaggedOrdersArray arr) {
		Orders retValue = new Orders();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedOrderArray was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getOrders() == null || arr.getOrders().getOrdersTO() == null)
			return retValue;

		Iterator<OrdersTO> itOrderTO = arr.getOrders().getOrdersTO().iterator();
		while (itOrderTO.hasNext()) {
			OrdersTO mdwsOrder = itOrderTO.next();
			//Perform Error Checking
			if (mdwsOrder.getFault() != null)
				throw new MdwsResponseException(mdwsOrder.getFault());

			Order order = createOrder(mdwsOrder);
			if (order != null)
    			retValue.add(order);
		}

		return retValue;
	}
	
	private Order createOrder(OrdersTO mdwsOrder){
		OrderTypeTO type = mdwsOrder.getType();
		if(type == null || type.getName1() == null)
			return null;
		//Perform Error Checking
		if (type.getFault() != null)
			throw new MdwsResponseException(type.getFault());
		
		String orderIen = mdwsOrder.getOrderIen();
		String dateEntered = mdwsOrder.getDateEntered();
		String startDate = mdwsOrder.getStartDate();
		String stopDate = mdwsOrder.getStopDate();
		String verifyingNurse = mdwsOrder.getVerifyingNurse();
		String verifyingClerk = mdwsOrder.getVerifyingClerk();
		
		OrderProvider provider = OrderTranslatorUtil.createProvider(mdwsOrder);
		OrdersStatus status = OrderTranslatorUtil.createOrderStatus(mdwsOrder);
		OrderSignStatus sigStatus = OrderTranslatorUtil.createSignatureStatus(mdwsOrder);
		String orderText = OrderTranslatorUtil.createOrderText(mdwsOrder);
		OrderLocation location = OrderTranslatorUtil.createLocation(mdwsOrder);
		MhpUser mhpUserProvider = OrderTranslatorUtil.createMhpUser(mdwsOrder);
		
		boolean flag = mdwsOrder.isFlag();
		
		Order order = new Order();
		order.setOrderId(orderIen);
		order.setVerifyingNurse(verifyingNurse);
		order.setVerifyingClerk(verifyingClerk == null ? "":verifyingClerk);
		
		if (NullChecker.isNotNullish(dateEntered) && validDate(dateEntered)) {
			Date orderedDate = DateHelper.parseAnyVistaDateString(dateEntered);
			order.setOrderedDate(orderedDate);
		}	
		if (NullChecker.isNotNullish(startDate) && validDate(startDate)) {
			Date dtStartDate = DateHelper.parseAnyVistaDateString(startDate);
			order.setStartDate(dtStartDate);
		}
		if (NullChecker.isNotNullish(stopDate) && validDate(stopDate)) {
			Date dtStopDate = DateHelper.parseAnyVistaDateString(stopDate);
			order.setStopDate(dtStopDate);
		}
		
		order.setOrderProvider(provider);
		order.setOrderStatus(status);
		if (null != order.getOrderStatus()) {
			order.setStatus(order.getOrderStatus().getName());  //order.status is a convenience field for order.orderstatus.name
		}
		if (null != mhpUserProvider) {
			order.setProvider(mhpUserProvider);
		}
		order.setOrderSignStatus(sigStatus);
		order.setOrderText(orderText);
		order.setOrderLocation(location);
		order.setFlag(flag);
		OrderTranslatorUtil.setOrderType(order, mdwsOrder);
		
		return order;
	}
	
	private boolean validDate(String date ) {
		if (!"-1".equalsIgnoreCase(date) && !"Invalid Date".equalsIgnoreCase(date)) {
			return true;
		}
		return false;
	}
}
