package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.agilex.healthcare.mobilehealthplatform.domain.OrderPickListItem;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderPickListItemEntry;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderPickLists;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.OrderPickListTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.PickListEntry;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.TaggedOrderPickListArray;

public class OrderPickListTranslator {

    public OrderPickLists translateFromTaggedOrderPickListArray(TaggedOrderPickListArray arr){
    	OrderPickLists retValue = new OrderPickLists();
    	if (arr == null)
    		throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "LVTaggedOrderPickListArray was null");
    	if (arr.getFault() != null)
    		throw new MdwsResponseException(arr.getFault());
    	if (arr.getPickLists() == null || arr.getPickLists().getOrderPickListTO() == null)
    		return retValue;

    	Iterator<OrderPickListTO> itOrderPickListTO = arr.getPickLists().getOrderPickListTO().iterator();
    	while (itOrderPickListTO.hasNext()) {
    		OrderPickListTO orderPickListTO = itOrderPickListTO.next();
    		//Perform Error Checking
    		if (orderPickListTO.getFault() != null)
    			throw new MdwsResponseException(orderPickListTO.getFault());

    		OrderPickListItem orderPickListItem = createOrderPickListItem(orderPickListTO);
    		retValue.add(orderPickListItem);
    	}

    	return retValue;
    }

    private OrderPickListItem createOrderPickListItem(OrderPickListTO orderPickListTO) {
    	OrderPickListItem orderPickListItem = new OrderPickListItem();
        String pickListName = orderPickListTO.getPickListName();
    	orderPickListItem.setPickListName(pickListName);
        
    	if (orderPickListTO.getEntries() == null || orderPickListTO.getEntries().getPickListEntry() == null)
    		return orderPickListItem;
    	
    	Iterator<PickListEntry> it = orderPickListTO.getEntries().getPickListEntry().iterator();
    	List<OrderPickListItemEntry> orderPickListItemEntries = new ArrayList<OrderPickListItemEntry>();
        while (it.hasNext()) {
        	PickListEntry lvPickListEntry = it.next();
        	
        	String entryIen = lvPickListEntry.getEntryIen();
            String entryName = lvPickListEntry.getEntryName();
            boolean isDefaultEntry = lvPickListEntry.isIsDefaultEntry();
			
            OrderPickListItemEntry orderPickListItemEntry = new OrderPickListItemEntry();
            orderPickListItemEntry.setIen(entryIen);
            orderPickListItemEntry.setName(entryName);
            orderPickListItemEntry.setDefaultEntry(isDefaultEntry);
            orderPickListItemEntries.add(orderPickListItemEntry);
        }
    	
    	orderPickListItem.setOrderPickListItemEntry(orderPickListItemEntries);
        return orderPickListItem;
    }
}
