package com.agilex.healthcare.mobilehealthplatform.mdws.translator;


import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.Order;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderSignStatus;
import com.agilex.healthcare.mobilehealthplatform.domain.Orders;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.OrderTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.OrderTypeTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedOrderArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedOrderArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.UserTO;
import com.agilex.healthcare.utility.DateHelper;

public class OrderTranslator {
				  
	public Orders translateFromTaggedOrderArrays(TaggedOrderArrays arr) throws Exception{
		Orders retValue = new Orders();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedOrderArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedOrderArray() == null)
			return retValue;

		Iterator<TaggedOrderArray> itTagged = arr.getArrays().getTaggedOrderArray().iterator();
		while (itTagged.hasNext()) {
			TaggedOrderArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getItems() == null || tagged.getItems().getOrderTO() == null)
				continue;

			Iterator<OrderTO> it = tagged.getItems().getOrderTO().iterator();
			while (it.hasNext()) {
				OrderTO mdwsOrder = it.next();
				//Perform Error Checking
				if (mdwsOrder.getFault() != null)
					throw new MdwsResponseException(mdwsOrder.getFault());

				Order order = createOrder(mdwsOrder);
				retValue.add(order);
			}
		}

		return retValue;
	}

	private Order createOrder(OrderTO orderTO) throws Exception {
		Order order = new Order();
		
		String id = orderTO.getId();
		String timestamp = orderTO.getTimestamp();
		Date orderedDate = DateHelper.parseVistaDateTimeOrNull(timestamp);
		String startDate = orderTO.getStartDate();
		Date dtStartDate = DateHelper.parseVistaDateTimeOrNull(startDate);
		String stopDate = orderTO.getStopDate();
		Date dtStopDate = DateHelper.parseVistaDateTimeOrNull(stopDate);
		String dateSigned = orderTO.getDateSigned();
		Date dtDateSigned = DateHelper.parseVistaDateTimeOrNull(dateSigned);
		String verifyingNurse = orderTO.getVerifyingNurse();
		String dateVerified = orderTO.getDateVerified();
		Date dtDateVerified = DateHelper.parseVistaDateTimeOrNull(dateVerified);
		String verifyingClerk = orderTO.getVerifyingClerk();
		String sigStatus = orderTO.getSigStatus();
		String text = orderTO.getText();
		String detail = orderTO.getDetail();
		String errMsg = orderTO.getErrMsg();
		boolean flag = orderTO.isFlag();
		
		UserTO provider = orderTO.getProvider();
		//Perform Error Checking
  		if (provider != null && provider.getFault() != null)
  			throw new MdwsResponseException(provider.getFault());
		OrderTypeTO type = orderTO.getType();
		//Perform Error Checking
  		if (type != null && type.getFault() != null)
  			throw new MdwsResponseException(type.getFault());
		
		MhpUser mhpUser = OrderTranslatorUtil.createMhpUser(orderTO);
		
		
		order.setOrderId(id);
		
		if(null != timestamp)
			order.setOrderedDate(orderedDate);
		if(null != startDate)
			order.setStartDate(dtStartDate);
		if(null != stopDate)
			order.setStopDate(dtStopDate);
		if(null != dateSigned)
			order.setDateSigned(dtDateSigned);
		if(null != dateVerified)
			order.setDateVerified(dtDateVerified);
		
		order.setStatus(orderTO.getStatus());
		
		order.setVerifyingNurse(verifyingNurse);
		
		OrderSignStatus orderSignStatus = new OrderSignStatus();
		orderSignStatus.setName(sigStatus);
		order.setOrderSignStatus(orderSignStatus);
		order.setVerifyingClerk(verifyingClerk);
		if(null != provider)
			order.setProvider(mhpUser);
		
		order.setOrderText(text);
		order.setOrderDetail(detail);
		if(null != type)
			//OrderTranslatorUtil.setOrderType(order, orderTO);
		
		order.setErrorMessage(errMsg);
		order.setFlag(flag);
		return order;
	}
}
