package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.Order;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderLocation;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderSignStatus;
import com.agilex.healthcare.mobilehealthplatform.domain.OrdersStatus;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.enumeration.MedicationSubType;
import com.agilex.healthcare.mobilehealthplatform.enumeration.OrderGrouping;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.OrderTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.AddressTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.ArrayOfAddressTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.ArrayOfPhoneNumTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.ArrayOfString;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.ArrayOfTaggedText;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.DemographicSetTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.FaultTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.HospitalLocationTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.OrderStatus;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.OrderTypeTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.OrdersMgmtTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.OrdersTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.PhoneNumTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.SignatureStatus;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.TaggedText;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.TaggedTextArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.UserTO;
import com.agilex.healthcare.utility.NullChecker;

public class OrderTranslatorUtil {
	public static OrderProvider createProvider(OrdersTO mdwsOrder) {
		//Perform Error Checking
		if (mdwsOrder.getFault() != null)
			throw new MdwsResponseException(mdwsOrder.getFault());
		
		UserTO mdwsUser = mdwsOrder.getProvider();
		if(mdwsUser == null)
			return null;
		//Perform Error Checking
		if (mdwsUser.getFault() != null)
			throw new MdwsResponseException(mdwsUser.getFault());
		
		String name = mdwsUser.getName();
		String ssn = mdwsUser.getSSN();
		String duz = mdwsUser.getDUZ();
		String siteId = mdwsUser.getSiteId();
		
		OrderProvider orderProvider = new OrderProvider();
		orderProvider.setName(name);
		orderProvider.setSsn(ssn);
		orderProvider.setDuz(duz);
		orderProvider.setSiteId(siteId);
		
		return orderProvider;
	}

	public static OrdersStatus createOrderStatus(OrdersTO mdwsOrder) {
		//Perform Error Checking
		if (mdwsOrder.getFault() != null)
			throw new MdwsResponseException(mdwsOrder.getFault());
		
		OrderStatus mdwsOrderStatus = mdwsOrder.getStatus();
		if (mdwsOrderStatus == null)
			return null;
		//NOTICE there is no error checking like other fields
	  	//Perform Error Checking
//	  	if (mdwsOrderStatus.getFault() != null)
//	  		throw new MdwsResponseException(mdwsOrderStatus.getFault());
		  	
		String code = mdwsOrderStatus.getCode();
		String value = mdwsOrderStatus.getValue();
		String name = mdwsOrderStatus.getName();
		String shortName = mdwsOrderStatus.getShortName();
		
		OrdersStatus ordersStatus = new OrdersStatus();
		ordersStatus.setCode(code);
		ordersStatus.setValue(value);
		ordersStatus.setName(name);
		ordersStatus.setShortName(shortName);
		
		return ordersStatus;
	}
		
	public static OrderSignStatus createSignatureStatus(OrdersTO mdwsOrder) {
		//Perform Error Checking
		if (mdwsOrder.getFault() != null)
			throw new MdwsResponseException(mdwsOrder.getFault());
		
		SignatureStatus signatureStatus = mdwsOrder.getSigStatus();
		if (signatureStatus == null)
			return null;
		//NOTICE there is no error checking like other fields
	  	//Perform Error Checking
	  	//if (signatureStatus.getFault() != null)
	  	//	throw new MdwsResponseException(signatureStatus.getFault());
		
		OrderSignStatus orderSignStatus = new OrderSignStatus();
		String name = signatureStatus.getName();
		String value = signatureStatus.getValue();
		
		orderSignStatus.setName(name);
		orderSignStatus.setValue(value);
		return orderSignStatus;
	}
	
	public static String createOrderText(OrdersTO mdwsOrder) {
		//Perform Error Checking
		if (mdwsOrder.getFault() != null)
			throw new MdwsResponseException(mdwsOrder.getFault());
		
		ArrayOfString arrayOfStrings = mdwsOrder.getOrderText();
		if(arrayOfStrings == null || arrayOfStrings.getString().size() == 0){
			return "";
		}
		StringBuilder buffy = new StringBuilder();
		Iterator<String> it = arrayOfStrings.getString().iterator();
		while (it.hasNext()) {
			String text = it.next();
			buffy.append(text).append("\t");
		}
		return buffy.toString();
	}

	public static OrderLocation createLocation(OrdersTO mdwsOrder) {
		//Perform Error Checking
		if (mdwsOrder.getFault() != null)
			throw new MdwsResponseException(mdwsOrder.getFault());
		
		HospitalLocationTO hospitalLocationTO = mdwsOrder.getLocation();
		if(hospitalLocationTO == null)
			return null;
		//Perform Error Checking
		if (hospitalLocationTO.getFault() != null)
			throw new MdwsResponseException(hospitalLocationTO.getFault());
		
		String id = hospitalLocationTO.getId();
		String name = hospitalLocationTO.getName();
		boolean askForCheckIn = hospitalLocationTO.isAskForCheckIn();
		
		OrderLocation orderLocation = new OrderLocation();
		orderLocation.setId(id);
		orderLocation.setName(name);
		orderLocation.setAskedForCheckIn(askForCheckIn);
		return orderLocation;
	}
	
	public static MhpUser createMhpUser(OrdersTO mdwsOrder) {
		//Perform Error Checking
		if (mdwsOrder.getFault() != null)
			throw new MdwsResponseException(mdwsOrder.getFault());
		
		UserTO mdwsUser = mdwsOrder.getProvider();
		
		return createMhpUser(mdwsUser);
	}

	public static MhpUser createMhpUser(OrderTO mdwsOrder) throws Exception{
		//Perform Error Checking
		if (mdwsOrder.getFault() != null)
			throw new MdwsResponseException(mdwsOrder.getFault());
		
		com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.UserTO mdwsUser = mdwsOrder.getProvider();
		UserTO userTO = convertEmrServiceUserTOToOrderMgmntSvcUserTO(mdwsUser);
		
		return createMhpUser(userTO);
	}

	private static UserTO convertEmrServiceUserTOToOrderMgmntSvcUserTO(
			com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.UserTO mdwsUser) throws Exception{
			UserTO userTO = new UserTO();
			copyOrderTypeTO(mdwsUser, userTO);
		return userTO;
	}

	protected static MhpUser createMhpUser(UserTO mdwsUser) {
		if(mdwsUser == null)
			return null;
		//Perform Error Checking
		if (mdwsUser.getFault() != null)
			throw new MdwsResponseException(mdwsUser.getFault());
		
		String name = mdwsUser.getName();
		String duz = mdwsUser.getDUZ();
		PatientIdentifier patientIdentifier = new PatientIdentifier("default", duz);
		String title = mdwsUser.getTitle();
		
		MhpUser mhpUser = new MhpUser();
		
		mhpUser.setDisplayName(name);
		mhpUser.setUserIdentifier(patientIdentifier);
		mhpUser.setTitle(title);
		parseAndSetNames(mhpUser, mdwsUser);
	
		return mhpUser;
	}
	
	private static void parseAndSetNames(MhpUser mhpUser, UserTO mdwsUser) {
		//Perform Error Checking
		if (mdwsUser.getFault() != null)
			throw new MdwsResponseException(mdwsUser.getFault());
		
		// Assuming "LAST,FIRST MIDDLE" Syntax
		String name = mdwsUser.getName();
		String username = mdwsUser.getUsername();
		
		mhpUser.setAccessCode(username);
		if (NullChecker.isNotNullish(name)) {
			String[] splitNames = name.trim().split(",");
			mhpUser.setLastName(splitNames[0]);
			String[] firstAndMiddle = splitNames[1].toString().trim().split(" ");
			switch (firstAndMiddle.length) {
				case 1:
					mhpUser.setFirstName(firstAndMiddle[0]);
					break;
				case 2:
					mhpUser.setFirstName(firstAndMiddle[0]);
					mhpUser.setMiddleInitials(firstAndMiddle[1]);
					break;
				default:
					mhpUser.setFirstName(firstAndMiddle[0]);
			}
		}
	}
	
	public static void setOrderType(Order order, OrdersTO mdwsOrder) {
		//Perform Error Checking
		if (mdwsOrder.getFault() != null)
			throw new MdwsResponseException(mdwsOrder.getFault());
		
		OrderTypeTO orderTypeTO = mdwsOrder.getType();
		
		setOrderType(order, orderTypeTO);
	}
		
	public static void setOrderType(Order order, OrdersMgmtTO mdwsOrder) {
		//Perform Error Checking
		if (mdwsOrder.getFault() != null)
			throw new MdwsResponseException(mdwsOrder.getFault());
		
		OrderTypeTO orderTypeTO = mdwsOrder.getType();
		
		setOrderType(order, orderTypeTO);
	}

	protected static void setOrderType(Order order, OrderTypeTO orderTypeTO) {
		if(orderTypeTO == null)
			return;
		//Perform Error Checking
		if (orderTypeTO.getFault() != null)
			throw new MdwsResponseException(orderTypeTO.getFault());

		String id = orderTypeTO.getId();
		String name1 = orderTypeTO.getName1();
		String name2 = orderTypeTO.getName2();
		
		order.setOrderTypeId(id);
	
		OrderGrouping orderGrouping = OrderGrouping.Other;
		String orderType = "";
		String orderSubType = "";
	
		if (name1.equals("Lab")) {
			orderGrouping = OrderGrouping.Laboratory;
			orderType = OrderGrouping.Laboratory.name();
			orderSubType = name2;
		} else if (name2.equals("Radiology")) {
			orderGrouping = OrderGrouping.Radiology;
			orderType = orderSubType = OrderGrouping.Radiology.name();
		} else if (name1.contains("Meds")) {
			orderGrouping = OrderGrouping.Medication;
			MedicationSubType type =  MedicationSubType.dataOf(name1);
			orderType = type == null ? "" :type.name();
			orderSubType = orderType;
		} else {
			orderType = name1;
			orderSubType = name2;
		}
		order.setOrderGrouping(orderGrouping);
		order.setOrderType(orderType);
		order.setOrderSubType(orderSubType);
	}
	
    private static OrderTypeTO  copyOrderTypeTO(com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.OrderTypeTO orderTypeTO, OrderTypeTO ormOrderTypeTO){
    	ormOrderTypeTO.setId(orderTypeTO.getId());
    	ormOrderTypeTO.setName1(orderTypeTO.getName1());
    	ormOrderTypeTO.setName2(orderTypeTO.getName2());
    	if (orderTypeTO.getFault() != null) {
    		FaultTO ormFaultTO = new FaultTO();
    		ormFaultTO = copyFaultTO(orderTypeTO.getFault(), ormFaultTO);
    	}
    	return ormOrderTypeTO;
    }

    private static void copyOrderTypeTO(com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.UserTO userTO, UserTO ormUserTO){
    	ormUserTO.setDigitalPager(userTO.getDigitalPager());
    	ormUserTO.setDomain(userTO.getDomain());
    	ormUserTO.setDUZ(userTO.getDUZ());
    	ormUserTO.setEmailAddress(userTO.getEmailAddress());
    	ormUserTO.setGreeting(userTO.getGreeting());
    	ormUserTO.setName(userTO.getName());
    	ormUserTO.setOffice(userTO.getOffice());
    	ormUserTO.setOrderRole(userTO.getOrderRole());
    	ormUserTO.setPager(userTO.getPager());
    	ormUserTO.setPhone(userTO.getPhone());
    	ormUserTO.setService(userTO.getService());
    	ormUserTO.setSiteId(userTO.getSiteId());
    	ormUserTO.setSiteMessage(userTO.getSiteMessage());
    	ormUserTO.setSSN(userTO.getSSN());
    	ormUserTO.setTitle(userTO.getTitle());
    	ormUserTO.setUserClass(userTO.getUserClass());
    	ormUserTO.setUsername(userTO.getUsername());
    	if (userTO.getDemographics() != null) {
    		copyDemographics(userTO, ormUserTO);
    	}
    	if (userTO.getIds() != null) {
    		copyIds(userTO, ormUserTO);
    	}
    	ormUserTO.setUsername(userTO.getUsername());
    	if (userTO.getFault() != null) {
    		FaultTO ormFaultTO = new FaultTO();
    		ormFaultTO = copyFaultTO(userTO.getFault(), ormFaultTO);
    		ormUserTO.setFault(ormFaultTO);
    	}
    }

	private static void copyIds(com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.UserTO userTO, UserTO ormUserTO) {
		if (userTO.getIds() != null) {
			TaggedTextArray ormTaggedTextArray = new TaggedTextArray();
			ArrayOfTaggedText ormArrayOfTaggedTextP = new ArrayOfTaggedText();
			
			TaggedText ormTaggedTextP = null;
			
			Iterator<com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedText> itTaggedResults = userTO.getIds().getResults().getTaggedText().iterator();
			while (itTaggedResults.hasNext()) {
				com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedText taggedResults = itTaggedResults.next();
				
				ormTaggedTextP = new TaggedText();
				ormTaggedTextP.setTag(taggedResults.getTag());
				ormTaggedTextP.setText(taggedResults.getText());
				ArrayOfString ormArrayOfString = new ArrayOfString();
				ormArrayOfString.getString().addAll(taggedResults.getTextArray().getString());
				ormTaggedTextP.setTextArray(ormArrayOfString);
		    	if (taggedResults.getFault() != null) {
		    		FaultTO ormFaultTO = new FaultTO();
		    		ormFaultTO = copyFaultTO(taggedResults.getFault(), ormFaultTO);
		    		ormTaggedTextP.setFault(ormFaultTO);
		    	}
				
				if (taggedResults.getTaggedResults() == null || taggedResults.getTaggedResults().getTaggedText() == null)
					continue;
				
				ArrayOfTaggedText ormArrayOfTaggedTextC = new ArrayOfTaggedText();
				TaggedText ormTaggedTextC = null;
				Iterator<com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedText> itTaggedText = taggedResults.getTaggedResults().getTaggedText().iterator();
				while (itTaggedText.hasNext()) {
					com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedText taggedText = itTaggedText.next();

					ormTaggedTextC = new TaggedText();
					ormTaggedTextC.setTag(taggedText.getTag());
					ormTaggedTextC.setText(taggedText.getText());
					ArrayOfString ormArrayOfStringC = new ArrayOfString();
					ormArrayOfStringC.getString().addAll(taggedText.getTextArray().getString());
					ormTaggedTextC.setTextArray(ormArrayOfStringC);
					ormArrayOfTaggedTextC.getTaggedText().add(ormTaggedTextC);
			    	if (taggedText.getFault() != null) {
			    		FaultTO ormFaultTO = new FaultTO();
			    		ormFaultTO = copyFaultTO(taggedText.getFault(), ormFaultTO);
			    		ormTaggedTextC.setFault(ormFaultTO);
			    	}
				}
				ormTaggedTextP.setTaggedResults(ormArrayOfTaggedTextC);
				ormArrayOfTaggedTextP.getTaggedText().add(ormTaggedTextP);
			}
			ormTaggedTextArray.setResults(ormArrayOfTaggedTextP);
		}
				
	}

	private static void copyDemographics(
			com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.UserTO userTO,
			UserTO ormUserTO) {
		DemographicSetTO ormDemographicSetTO = new DemographicSetTO();
		
		ormDemographicSetTO.setTag(userTO.getDemographics().getTag());
		
		if (userTO.getDemographics().getAddresses() != null) {
			ArrayOfAddressTO ormArrayOfAddressTO = new ArrayOfAddressTO();
		    List<AddressTO> addressTOs = new ArrayList<AddressTO>();
		    Iterator<com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.AddressTO> it = userTO.getDemographics().getAddresses().getAddressTO().iterator();
		    while (it.hasNext()) {
		    	com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.AddressTO addressTO = it.next();
		    	AddressTO ormAddressTO = new AddressTO();
		    	ormAddressTO = copyAddress(addressTO, ormAddressTO);
		    	addressTOs.add(ormAddressTO);
		    }
		    ormArrayOfAddressTO.getAddressTO().addAll(addressTOs);
		    ormDemographicSetTO.setAddresses(ormArrayOfAddressTO);
		}

		if (userTO.getDemographics().getEmailAddresses() != null) {
			ArrayOfString ormArrayOfString = new ArrayOfString();
		    List<String> emailStrings = userTO.getDemographics().getEmailAddresses().getString();
		    ormArrayOfString.getString().addAll(emailStrings);
		    ormDemographicSetTO.setEmailAddresses(ormArrayOfString);
		}

		if (userTO.getDemographics().getNames() != null) {
			ArrayOfString ormArrayOfString = new ArrayOfString();
		    List<String> nameStrings = userTO.getDemographics().getNames().getString();
		    ormArrayOfString.getString().addAll(nameStrings);
		    ormDemographicSetTO.setNames(ormArrayOfString);
		}

		if (userTO.getDemographics().getPhones() != null) {
			ArrayOfPhoneNumTO ormArrayOfPhoneNumTO = new ArrayOfPhoneNumTO();
		    List<PhoneNumTO> phoneNumTOs = new ArrayList<PhoneNumTO>();
		    Iterator<com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.PhoneNumTO> it = userTO.getDemographics().getPhones().getPhoneNumTO().iterator();
		    while (it.hasNext()) {
		    	com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.PhoneNumTO phoneNumTO = it.next();
		    	PhoneNumTO ormpPhoneNumTO = new PhoneNumTO();
		    	ormpPhoneNumTO = copyPhone(phoneNumTO, ormpPhoneNumTO);
		    	phoneNumTOs.add(ormpPhoneNumTO);
		    }
		    ormArrayOfPhoneNumTO.getPhoneNumTO().addAll(phoneNumTOs);
		    ormDemographicSetTO.setPhones(ormArrayOfPhoneNumTO);
		}
		ormUserTO.setDemographics(ormDemographicSetTO);
	}
    
	private static AddressTO copyAddress(com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.AddressTO addressTO, AddressTO ormAddressTO) {
		
		ormAddressTO.setCity(addressTO.getCity());
		ormAddressTO.setCountry(addressTO.getCountry());
		ormAddressTO.setCounty(addressTO.getCounty());
		ormAddressTO.setDescription(addressTO.getDescription());
		ormAddressTO.setState(addressTO.getState());
		ormAddressTO.setStreetAddress1(addressTO.getStreetAddress1());
		ormAddressTO.setStreetAddress2(addressTO.getStreetAddress2());
		ormAddressTO.setStreetAddress3(addressTO.getStreetAddress3());
		ormAddressTO.setType(addressTO.getType());
		ormAddressTO.setValidFrom(addressTO.getValidFrom());
		ormAddressTO.setValidTo(addressTO.getValidTo());
		ormAddressTO.setZipcode(addressTO.getZipcode());
    	if (addressTO.getFault() != null) {
    		FaultTO ormFaultTO = new FaultTO();
    		ormFaultTO = copyFaultTO(addressTO.getFault(), ormFaultTO);
    		ormAddressTO.setFault(ormFaultTO);
    	}
		return ormAddressTO;
	}
	
	private static PhoneNumTO copyPhone(com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.PhoneNumTO phoneNumTO, PhoneNumTO ormPhoneNumTO) {
		
		ormPhoneNumTO.setAreaCode(phoneNumTO.getAreaCode());
		ormPhoneNumTO.setDescription(phoneNumTO.getDescription());
		ormPhoneNumTO.setExchange(phoneNumTO.getExchange());
		ormPhoneNumTO.setNumber(phoneNumTO.getNumber());
		ormPhoneNumTO.setType(phoneNumTO.getType());
    	if (phoneNumTO.getFault() != null) {
    		FaultTO ormFaultTO = new FaultTO();
    		ormFaultTO = copyFaultTO(phoneNumTO.getFault(), ormFaultTO);
    		ormPhoneNumTO.setFault(ormFaultTO);
    	}
		return ormPhoneNumTO;
	}
	
	private static FaultTO copyFaultTO(com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.FaultTO faultTO, FaultTO ormFaultTO) {
		
		ormFaultTO.setInnerMessage(faultTO.getInnerMessage());
		ormFaultTO.setInnerStackTrace(faultTO.getInnerStackTrace());
		ormFaultTO.setInnerType(faultTO.getInnerType());
		ormFaultTO.setMessage(faultTO.getMessage());
		ormFaultTO.setStackTrace(faultTO.getStackTrace());
		ormFaultTO.setSuggestion(faultTO.getSuggestion());
		ormFaultTO.setType(faultTO.getType());
		return ormFaultTO;
	}

	
}
