package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.*;

import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.*;
import com.agilex.healthcare.utility.*;

public class PatientTranslator {
	
	public Patients translateFromTaggedPatientArray(TaggedPatientArray arr) {
		Patients retValue = new Patients();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedPatientArray was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getPatients() == null || arr.getPatients().getPatientTO() == null)
			return retValue;

		Iterator<PatientTO> itPatientTO = arr.getPatients().getPatientTO().iterator();
		while (itPatientTO.hasNext()) {
			PatientTO mdwsPatient = itPatientTO.next();
			//Perform Error Checking
			if (mdwsPatient.getFault() != null)
				throw new MdwsResponseException(mdwsPatient.getFault());

			Patient patient = createPatient(mdwsPatient);
			retValue.add(patient);
		}

		return retValue;
	}
	
	public Patients translateFromTaggedPatientArrays(TaggedPatientArrays arr) {
		Patients retValue = new Patients();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedPatientArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedPatientArray() == null)
			return retValue;

		Iterator<TaggedPatientArray> itTagged = arr.getArrays().getTaggedPatientArray().iterator();
		while (itTagged.hasNext()) {
			TaggedPatientArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getPatients() == null || tagged.getPatients().getPatientTO() == null)
				continue;

			Iterator<PatientTO> it = tagged.getPatients().getPatientTO().iterator();
			while (it.hasNext()) {
				PatientTO mdwsPatient = it.next();
				//Perform Error Checking
				if (mdwsPatient.getFault() != null)
					throw new MdwsResponseException(mdwsPatient.getFault());

				Patient patient = createPatient(mdwsPatient);
				retValue.add(patient);
			}
		}

		return retValue;
	}
	
	public Patient translateFromPatientTO(PatientTO mdwsPatient) {
		//Perform Error Checking
		if (mdwsPatient.getFault() != null)
			throw new MdwsResponseException(mdwsPatient.getFault());

		Patient patient = createPatient(mdwsPatient);
		return patient;
	}

	private Patient createPatient(PatientTO mdwsPatient) {
		String patientName = mdwsPatient.getPatientName();
		String gender = mdwsPatient.getGender();
		String ssn = mdwsPatient.getSsn();
		String localPid = mdwsPatient.getLocalPid();
		boolean inpatient = mdwsPatient.isInpatient();
		boolean serviceConnected = mdwsPatient.isServiceConnected();
		int scPercent = mdwsPatient.getScPercent();
		String serviceConnectedPercent = String.valueOf(scPercent);
		String cwad = mdwsPatient.getCwad();
		boolean restricted = mdwsPatient.isRestricted();
		String mpiPid = mdwsPatient.getMpiPid();
		String admitTimestamp = mdwsPatient.getAdmitTimestamp();
		
		HospitalLocationTO location = mdwsPatient.getLocation();
		//Perform Error Checking
		if (location != null && location.getFault() != null)
			throw new MdwsResponseException(location.getFault());
		
		Patient patient = new Patient();
		patient.setDisplayName(patientName);
		patient.setGender(gender);
		patient.setSsn(ssn);

		PatientIdentifier patientIdentifier = new PatientIdentifier("default", localPid);
		patient.setPatientIdentifier(patientIdentifier);

		setDateOfBirth(patient, mdwsPatient);
		setPatientAge(patient, mdwsPatient);
		setPatientNames(patient, mdwsPatient);
		setPatientMiddleInitial(patient);
		
		if (location != null) {
			String id = location.getId();
			String name = location.getName();
			String room = location.getRoom();
			if ((id != null) && (!id.isEmpty()))
				patient.setLocationIEN(id);
			patient.setWardLocation(name);
            patient.setRoombed(room);
		}
		patient.setInpatient(inpatient);
		patient.setServiceConnected(serviceConnected);
		patient.setServiceConnectedPercent(serviceConnectedPercent);
		patient.setCwad(cwad);
		patient.setSensitive(restricted);
		if (mpiPid != null)
            patient.setIcn(mpiPid);
        if (admitTimestamp != null && admitTimestamp.length() > 0) {
            Date parseVistaDateTime = DateHelper.parseVistaDateTime(admitTimestamp);
			patient.setAdmittedDateTime(parseVistaDateTime);
        }
		return patient;
	}

	private void setPatientAge(Patient patient, PatientTO mdwsPatient) {
		try {
			int age = mdwsPatient.getAge();
			if (age == 0) {
				long calculatedAge = calculateAge(patient.getDateOfBirth().getDate());
				patient.setAge(String.valueOf(calculatedAge));
			}
		} catch (Exception e) {}

	}

	private long calculateAge(Date dob) {
		return DateHelper.calculateDeltaInYears(dob, new Date());
	}

	private void setPatientNames(Patient patient, PatientTO mdwsPatient) {
		// Assuming "LAST,FIRST MIDDLE" Syntax
		String patientName = mdwsPatient.getPatientName();
		if (NullChecker.isNotNullish(patientName)) {
			String[] splitNames = patientName.trim().split(",");
			patient.setLastName(splitNames[0]);
			String[] firstAndMiddle = splitNames[1].toString().trim().split(" ");

			switch (firstAndMiddle.length) {
                case 1:
                    patient.setFirstName(firstAndMiddle[0]);
                    break;
                case 2:
                    patient.setFirstName(firstAndMiddle[0]);
                    patient.setMiddleName(firstAndMiddle[1]);
                    break;
                case 3: // rare, but possible
                    patient.setFirstName(firstAndMiddle[0]);
                    patient.setMiddleName(firstAndMiddle[1] + " " + firstAndMiddle[2]);
                    break;
                case 4: // even more rare, but possible
                    patient.setFirstName(firstAndMiddle[0]);
                    patient.setMiddleName(firstAndMiddle[1] + " " + firstAndMiddle[2] + " " + firstAndMiddle[3]);
                    break;
                default:
                    patient.setFirstName(firstAndMiddle[0]); // first name is required by vista;
			}
		}
	}

	private void setPatientMiddleInitial(Patient patient) {
		String middleName = patient.getMiddleName();
		if (NullChecker.isNotNullish(middleName)) {
			char initial = middleName.charAt(0);
			patient.setMiddleInitial(String.valueOf(initial).toUpperCase());
		}
	}

	private void setDateOfBirth(Patient patient, PatientTO mdwsPatient) {
		String dob = mdwsPatient.getDob();
		Date dobDate = MdwsDateTranslationUtil.extractDateOrNull(dob);
		if (dobDate == null)
			patient.setDateOfBirth(null);
		else
			patient.setDateOfBirth(new DOBDate(dobDate));
	}
}
