package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Problem;
import com.agilex.healthcare.mobilehealthplatform.domain.Problems;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.ProblemTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedProblemArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedProblemArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedText;
import com.agilex.healthcare.utility.Hasher;
import com.agilex.healthcare.utility.NullChecker;

public class ProblemTranslator {
	public Problems translateMdwsTaggedProblemArrays(TaggedProblemArrays arr) {
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedProblemArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedProblemArrays.getArrays was null");
		if (arr.getArrays().getTaggedProblemArray() == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedProblemArrays.getArrays.getTaggedProblemArray was null");
		
		Problems retValue = new Problems();
		
		Iterator<TaggedProblemArray> itTaggedProblemArray = arr.getArrays().getTaggedProblemArray().iterator();
		
		while (itTaggedProblemArray.hasNext()) {
			TaggedProblemArray taggedProblemArray = itTaggedProblemArray.next();
			//Perform Error Checking
			if (taggedProblemArray.getFault() != null)
				throw new MdwsResponseException(taggedProblemArray.getFault());
			
			//String tag = taggedProblemArray.getTag();
			if (taggedProblemArray.getProblems() == null || taggedProblemArray.getProblems().getProblemTO() == null)
				continue;
			
			Iterator<ProblemTO> itProblems = taggedProblemArray.getProblems().getProblemTO().iterator();
			while (itProblems.hasNext()) {
				ProblemTO problemTO = itProblems.next();
				//Perform Error Checking
				if (problemTO.getFault() != null)
					throw new MdwsResponseException(problemTO.getFault());
				
				Problem problem = createProblem(problemTO);
				retValue.add(problem);
			}
		}

		return retValue;
	}

	private Problem createProblem(ProblemTO problemTo) {
		assert problemTo != null;
		String providerNarrative = providerNarrative(problemTo);
		String status = problemTo.getStatus();
		boolean active = isActive(status);
		String comment = problemTo.getComment();
		
		Problem problem = new Problem();
		setId(problem, problemTo);

		problem.setDescription(providerNarrative);
		problem.setActive(active);
		problem.setComments(comment);
		setDates(problem, problemTo);
		setFacility(problem, problemTo);

		return problem;
	}

	private void setId(Problem problem, ProblemTO mdwsProblem) {
		String id = mdwsProblem.getId();

		if (NullChecker.isNullish(id))
			id = createRepeatableUniqueId(mdwsProblem);

		problem.setUniqueId(id);
		problem.setSystemId("vista");
	}

	private String createRepeatableUniqueId(ProblemTO prob) {
		//We want to create something unique yet at the same time we need to be able to refer back to it in case they
		//ask for a single Vital.  So, we will create a unique id by taking all of the possible data that we know about
		//to create a String that contains everything and creating a hash from that String.
		
		String uniqueId = "" +
				prob.getComment() +
				":" + prob.getExposures() +
				":" + prob.getId() +
				":" + prob.getModifiedDate() +
				":" + prob.getNoteNarrative() +
				":" + prob.getOnsetDate() +
				":" + prob.getPriority() +
				":" + prob.getProviderNarrative() +
				":" + prob.getResolvedDate() +
				":" + prob.getStatus() +
				":" + prob.getTimestamp() +
				":" + (prob.getAcuity() == null ? "" : prob.getAcuity().getTag()) +
				":" + (prob.getAcuity() == null ? "" : prob.getAcuity().getText()) +
				":" + (prob.getFacility() == null ? "" : prob.getFacility().getTag()) +
				":" + (prob.getFacility() == null ? "" : prob.getFacility().getText()) +
				":" + (prob.getType() == null ? "" : prob.getType().getCategory()) +
				":" + (prob.getType() == null ? "" : prob.getType().getDataId()) +
				":" + (prob.getType() == null ? "" : prob.getType().getDataName()) +
				":" + (prob.getType() == null ? "" : prob.getType().getDataType()) +
				":" + (prob.getType() == null ? "" : prob.getType().getId()) +
				":" + (prob.getType() == null ? "" : prob.getType().getName()) +
				":" + (prob.getType() == null ? "" : prob.getType().getShortName());
		Hasher hasher = new Hasher();
		String retvalue = hasher.hashStringToString(uniqueId);
		return retvalue;
	}

	private boolean isActive(String mdwsStatusString) {
		String vistaActiveStatus = "A";
		String cdwActiveStatus = "Y";
		if (vistaActiveStatus.equals(mdwsStatusString) || cdwActiveStatus.equals(mdwsStatusString))
			return true;
		else
			return false;
	}

	void setDates(Problem problem, ProblemTO problemTo) {
		assert problem != null;
		assert problemTo != null;

		String onsetDate = problemTo.getOnsetDate();
		String resolvedDate = problemTo.getResolvedDate();
		String modifiedDate = problemTo.getModifiedDate();
		Date dtOnsetDate = MdwsDateTranslationUtil.extractDateOrNull(onsetDate);
		Date dtResolvedDate = MdwsDateTranslationUtil.extractDateOrNull(resolvedDate);
		Date dtModifiedDate = MdwsDateTranslationUtil.extractDateOrNull(modifiedDate);
		
		if (onsetDate != null)
			problem.setOnsetDate(dtOnsetDate);

		if (resolvedDate != null)
			problem.setResolutionDate(dtResolvedDate);

		if (modifiedDate != null)
			problem.setLastUpdatedDate(dtModifiedDate);
	}

	private void setFacility(Problem problem, ProblemTO problemTo) {
		TaggedText facility = problemTo.getFacility();
		if (facility == null)
			return;
		//Perform Error Checking
		if (facility.getFault() != null)
			throw new MdwsResponseException(facility.getFault());
		
		String tag = facility.getTag();
		String sourceSystem = stripSiteCodeFromFacilityName(tag);
		problem.setSourceSystem(sourceSystem);
	}

	private String stripSiteCodeFromFacilityName(String facilityName) {
		if (facilityName != null && facilityName.contains("("))
			return matchFacilityName(facilityName);

		return facilityName;
	}

	String matchFacilityName(String facilityName) {
		// Matches something like "(321) City Name, ST" -> {"(321)",
		// "City Name, ST"}
		String altered = facilityName.replaceAll("\\([0-9]{3,4}\\)", "");
		return altered.trim();
	}

	private String providerNarrative(ProblemTO problemTo) {
		if (problemTo == null)
			return "Blank Narrative";
		//Perform Error Checking
		if (problemTo.getFault() != null)
			throw new MdwsResponseException(problemTo.getFault());
		
		String providerNarrative = problemTo.getProviderNarrative();
		return NullChecker.isNotNullish(providerNarrative) ? providerNarrative.trim() : "Blank Narrative";
	}
}
