package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.PatientProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientProviders;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
//import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.PatientCareTeamMemberTO;
//import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.PatientCareTeamTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.PersonTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.SiteTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedVisitArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedVisitArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.UserTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.VisitTO;
//TODO: MDWS refactoring - method not implemented in MDWS3.0.2.5
public class ProviderTranslator {
//    public PatientProviders translate(PatientCareTeamTO arr) {
//    	PatientProviders retValue = new PatientProviders();
//    	if (arr == null)
//    		throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "PatientCareTeamTO was null");
//    	if (arr.getFault() != null)
//    		throw new MdwsResponseException(arr.getFault());
//    	if (arr.getMembers() == null || arr.getMembers().getPatientCareTeamMemberTO() == null)
//    		return retValue;
//
//    	Iterator<PatientCareTeamMemberTO> itMember = arr.getMembers().getPatientCareTeamMemberTO().iterator();
//    	while (itMember.hasNext()) {
//    		PatientCareTeamMemberTO member = itMember.next();
//    		//Perform Error Checking
//    		if (member.getFault() != null)
//    			throw new MdwsResponseException(member.getFault());
//
//    		PatientProvider provider = createPatientProvider(member);
//    		retValue.add(provider);
//    	}
//
//    	return retValue;
//    }
//
//	private PatientProvider createPatientProvider(PatientCareTeamMemberTO member) {
//		PersonTO person = member.getPerson();
//		//Perform Error Checking
//		if (person == null)
//			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "person was null");
//		if (person.getFault() != null)
//			throw new MdwsResponseException(person.getFault());
//		
//		String id = person.getId();
//		String name = person.getName();
//		String primaryStandardPosition = member.getPrimaryStandardPosition();
//		String teamStartDate = member.getTeamStartDate();
//		Date visitDate = MdwsDateTranslationUtil.extractDateOrNull(teamStartDate);
//		
//		PatientProvider provider = new PatientProvider();
//
//		provider.setUniqueId(id);
//		provider.setProviderName(name);
//		provider.setPersonClass(primaryStandardPosition);
//		provider.setVisitDate(visitDate);
//		return provider;
//	}
//
    public PatientProviders translate(TaggedVisitArrays arr, String facilitySystemCode) {
    	PatientProviders retValue = new PatientProviders();
    	if (arr == null)
    		throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "XXX was null");
    	if (arr.getFault() != null)
    		throw new MdwsResponseException(arr.getFault());
    	if (arr.getArrays() == null || arr.getArrays().getTaggedVisitArray() == null)
    		return retValue;

    	Iterator<TaggedVisitArray> itTagged = arr.getArrays().getTaggedVisitArray().iterator();
    	while (itTagged.hasNext()) {
    		TaggedVisitArray tagged = itTagged.next();
    		//Perform Error Checking
    		if (tagged.getFault() != null)
    			throw new MdwsResponseException(tagged.getFault());

    		if (tagged.getVisits() == null || tagged.getVisits().getVisitTO() == null)
    			continue;

    		Iterator<VisitTO> it = tagged.getVisits().getVisitTO().iterator();
    		while (it.hasNext()) {
    			VisitTO visitTO = it.next();
    			//Perform Error Checking
    			if (visitTO.getFault() != null)
    				throw new MdwsResponseException(visitTO.getFault());

    			PatientProvider provider = createPatientProvider(visitTO, facilitySystemCode);
                if (provider != null)
                	retValue.add(provider);
    		}
    	}

    	return retValue;
    }

	private PatientProvider createPatientProvider(VisitTO visitTO, String facilitySystemCode) {
		boolean filterByFacility = true;
		if (facilitySystemCode == null || facilitySystemCode.equals(""))
    		filterByFacility  = false;

		SiteTO facility = visitTO.getFacility();
		UserTO mdwsProvider = visitTO.getProvider();
		//Perform Error Checking
		if (facility == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "facility was null");
		if (mdwsProvider == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "mdwsProvider was null");
		if (facility.getFault() != null)
			throw new MdwsResponseException(facility.getFault());
		if (mdwsProvider.getFault() != null)
			throw new MdwsResponseException(mdwsProvider.getFault());

		String siteCode = facility.getSitecode();
		String name = mdwsProvider.getName();
		String personClass = mdwsProvider.getUserClass();
		String timestamp = visitTO.getTimestamp();
		Date visitDate = MdwsDateTranslationUtil.extractDateOrNull(timestamp);
		
		if (filterByFacility == true && siteCode.equals(facilitySystemCode) == false)
    		return null;
		
		PatientProvider provider = new PatientProvider();
		provider.setProviderName(name);
		provider.setPersonClass(personClass);
		provider.setVisitDate(visitDate);
		return provider;
	}
}
