package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Calendar;
import java.util.Date;

import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.Order;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderLocation;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderSignStatus;
import com.agilex.healthcare.mobilehealthplatform.domain.OrdersStatus;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.OrderStatus;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.OrdersTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.ordermgmtservice.UserTO;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;

public class RadiologyOrderTranslator {

    public Order translateFromOrderTO(OrdersTO orderTO) {
		if (orderTO == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "orderTO was null");
		if (orderTO.getFault() != null)
			throw new MdwsResponseException(orderTO.getFault());

		String orderIen = orderTO.getOrderIen();
        String dateEntered = orderTO.getDateEntered();
        String startDate = orderTO.getStartDate();
        String stopDate = orderTO.getStopDate();
        boolean flag = orderTO.isFlag();
		String verifyingNurse = orderTO.getVerifyingNurse();
		String verifyingClerk = orderTO.getVerifyingClerk();
		
		OrdersStatus orderStatus = OrderTranslatorUtil.createOrderStatus(orderTO);
		OrderSignStatus orderSignStatus = OrderTranslatorUtil.createSignatureStatus(orderTO);
		MhpUser mhpUser = OrderTranslatorUtil.createMhpUser(orderTO);
		OrderProvider orderProvider = OrderTranslatorUtil.createProvider(orderTO);
        String orderText = OrderTranslatorUtil.createOrderText(orderTO);
        OrderLocation orderLocation = OrderTranslatorUtil.createLocation(orderTO);
        
        
        Order order = new Order();
        order.setOrderId(orderIen);
        
        if (NullChecker.isNotNullish(dateEntered) && validDate(dateEntered)) {
        	Date dtDateEntered = DateHelper.parseVistaDefaultDateTIme(dateEntered);
            order.setOrderedDate(dtDateEntered);
        }
        
        if (NullChecker.isNotNullish(startDate) && validDate(startDate)) {
            if (startDate.equals("TODAY") && order.getOrderedDate() != null) {
                order.setStartDate(order.getOrderedDate());
            }
            else if (startDate.equals("TODAY")) {
				//Shouldn't hit this but it's here to prevent an exception in the else statement.
				Date dtStartDate = Calendar.getInstance().getTime();
				order.setStartDate(dtStartDate);
			}
            else {
				Date dtStartDate = DateHelper.parseVistaDefaultDateTIme(startDate);
				order.setStartDate(dtStartDate);
			}
        }
        if (NullChecker.isNotNullish(stopDate) && validDate(stopDate)) {
        	Date dtStopDate = DateHelper.parseVistaDefaultDateTIme(stopDate);
			order.setStopDate(dtStopDate);
        }
		        
        order.setOrderStatus(orderStatus);
        OrderStatus status = orderTO.getStatus();
		if (null != status) {
			String name = status.getName();
			order.setStatus(name);
		}
        
        order.setOrderSignStatus(orderSignStatus);
        order.setVerifyingClerk(verifyingClerk);
        order.setVerifyingNurse(verifyingNurse);
		UserTO provider = orderTO.getProvider();
		if (null != provider) {
			//Perform Error Checking
			if (provider.getFault() != null) {
				throw new MdwsResponseException(provider.getFault());
			}
			order.setProvider(mhpUser);
		}
		
		order.setOrderProvider(orderProvider);
        order.setOrderText(orderText);
        order.setOrderLocation(orderLocation);
        order.setFlag(flag);
        OrderTranslatorUtil.setOrderType(order, orderTO);

        return order;
    }
    
	private boolean validDate(String date ) {
		if (!"-1".equalsIgnoreCase(date) && !"Invalid Date".equalsIgnoreCase(date)) {
			return true;
		}
		return false;
	}
    
}
