package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.*;

import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.*;

public class RadiologyTranslator {
	
	public RadiologyResults translate(TaggedRadiologyReportArrays arr) {
		RadiologyResults retValue = new RadiologyResults();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedRadiologyReportArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedRadiologyReportArray() == null)
			return retValue;

		Iterator<TaggedRadiologyReportArray> itTagged = arr.getArrays().getTaggedRadiologyReportArray().iterator();
		while (itTagged.hasNext()) {
			TaggedRadiologyReportArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getRpts() == null || tagged.getRpts().getRadiologyReportTO() == null)
				continue;

			Iterator<RadiologyReportTO> it = tagged.getRpts().getRadiologyReportTO().iterator();
			while (it.hasNext()) {
				RadiologyReportTO report = it.next();
				//Perform Error Checking
				if (report.getFault() != null)
					throw new MdwsResponseException(report.getFault());

				RadiologyResult result = createRadiologyResult(report);
				retValue.add(result);
			}
		}

		return retValue;
	}
	
	private RadiologyResult createRadiologyResult(RadiologyReportTO report) {
		String title = report.getTitle();
		String id = report.getId();
		String text = report.getText();
		String impression = report.getImpression();
		String timestamp = report.getTimestamp();
		Date dtResultedDate = MdwsDateTranslationUtil.extractDateOrNull(timestamp);
		
		RadiologyResult result = new RadiologyResult();
		result.setTestShortName(title);
		result.setUniqueId(id);
		result.setTestId(id);
		result.setReport("REPORT:\n "+text + "\n\nIMPRESSION:\n "+impression);
		result.setTestName(title);
		result.setResultedDate(dtResultedDate);

		setFacility(result, report);
		
		return result;
	}
	
	private void setFacility(RadiologyResult result, RadiologyReportTO report) {
		TaggedText facility = report.getFacility();
		if (facility == null)
			return;
		//Perform Error Checking
		if (facility.getFault() != null)
			throw new MdwsResponseException(facility.getFault());
		
		String tag = facility.getTag(); // returns the station number TODO: Need to translate this somehow to a real facility name
		
		result.setSourceSystem(tag);
		result.setFacilityName(tag);
	}
}
