package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Author;
import com.agilex.healthcare.mobilehealthplatform.domain.Facility;
import com.agilex.healthcare.mobilehealthplatform.domain.ReportDocument;
import com.agilex.healthcare.mobilehealthplatform.domain.ReportDocuments;
import com.agilex.healthcare.mobilehealthplatform.domain.Specimen;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.AuthorTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.CytologyRpt;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.LabSpecimenTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.NoteTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.RadiologyReportTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.SurgeryReportTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.SurgicalPathologyRpt;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedCytologyRptArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedCytologyRptArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedNoteArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedNoteArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedRadiologyReportArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedRadiologyReportArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedSurgeryReportArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedSurgeryReportArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedSurgicalPathologyRptArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedSurgicalPathologyRptArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedText;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;

public class ReportDocumentTranslator {

	/**
	 * (1) Discharge Summary & Progress Note
	 * 
	 * @param reportDocuments Unlike the other translators which return a collection, this expects you to pass the collection
	 * into the method and it will then populate that collection for you.  Passing in null for this will result in
	 * an exception being thrown.
	 */
	public void add(ReportDocuments reportDocuments, TaggedNoteArrays arr, String documentType) {
		if (reportDocuments == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_INPUT, "reportDocuments was null");
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedNoteArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedNoteArray() == null)
			return;

		Iterator<TaggedNoteArray> itTagged = arr.getArrays().getTaggedNoteArray().iterator();
		while (itTagged.hasNext()) {
			TaggedNoteArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getNotes() == null || tagged.getNotes().getNoteTO() == null)
				continue;

			Iterator<NoteTO> it = tagged.getNotes().getNoteTO().iterator();
			while (it.hasNext()) {
				NoteTO noteTO = it.next();
				//Perform Error Checking
				if (noteTO.getFault() != null)
					throw new MdwsResponseException(noteTO.getFault());

				ReportDocument reportDocument = createReportDocument(noteTO);
				reportDocument.setDocumentType(documentType);
				reportDocuments.add(reportDocument); //NOTICE this is different than the other translators in that the collection is passed in as a parameter
			}
		}
	}

	private ReportDocument createReportDocument(NoteTO noteTO) {
		String id = noteTO.getId();
		String noteTimestamp = noteTO.getTimestamp();
		String admitTimestamp = noteTO.getAdmitTimestamp();
		String dischargeTimestamp = noteTO.getDischargeTimestamp();
		String localTitle = noteTO.getLocalTitle();
		String standardTitle = noteTO.getStandardTitle();
		String text = noteTO.getText();
		AuthorTO author = noteTO.getAuthor();
		AuthorTO approvedBy = noteTO.getApprovedBy();
		
		ReportDocument reportDocument = new ReportDocument();
		reportDocument.setRecordId(id);
		if (noteTimestamp != null) {
			Date dtTimestamp = DateHelper.parseVistaDateTime(noteTimestamp);
			reportDocument.setTimestamp(dtTimestamp);
		}
		if (admitTimestamp != null) {
			Date dtAdmitTimestamp = DateHelper.parseVistaDateTime(admitTimestamp);
			Date reportTimestamp = reportDocument.getTimestamp();
			
			reportDocument.setEntryDate(dtAdmitTimestamp);
			if(reportTimestamp == null) {
				Date entryDate = reportDocument.getEntryDate();
				reportDocument.setTimestamp(entryDate);
			}
		}
		if (dischargeTimestamp != null) {
			Date dtDischargeTimestamp = DateHelper.parseVistaDateTime(dischargeTimestamp);
			reportDocument.setDischargeDate(dtDischargeTimestamp);
		}
		
		reportDocument.setLocalTitle(localTitle);
		reportDocument.setStandardTitle(standardTitle);
		
		if (author != null) {
			//Perform Error Checking
			if (author.getFault() != null)
				throw new MdwsResponseException(author.getFault());

			String authorId = author.getId();
			String authorName = author.getName();
			String authorSignature = author.getSignature();
			Author reportAuthor = new Author(authorId, authorName, authorSignature);
			reportDocument.setAuthor(reportAuthor);
		}
		
		reportDocument.setText(text);
		if (approvedBy != null) {
			//Perform Error Checking
			if (approvedBy.getFault() != null)
				throw new MdwsResponseException(approvedBy.getFault());
			
			String approvedById = approvedBy.getId();
			String approvedByName = approvedBy.getName();
			String approvedBySignature = approvedBy.getSignature();
			Author reportApprovedBy = new Author(approvedById, approvedByName, approvedBySignature);
			reportDocument.setApprovedBy(reportApprovedBy);
		}
		
		return reportDocument;
	}

//----------------------------------------------------------------------------------------------------------------------
	
	/**
	 * (2) Surgical Pathology Reports
	 * 
	 * @param reportDocuments Unlike the other translators which return a collection, this expects you to pass the collection
	 * into the method and it will then populate that collection for you.  Passing in null for this will result in
	 * an exception being thrown.
	 */
	public void add(ReportDocuments reportDocuments, TaggedSurgicalPathologyRptArrays arr, DateFilter dateFilter, String documentType) {
		if (reportDocuments == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_INPUT, "reportDocuments was null");
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedSurgicalPathologyRptArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedSurgicalPathologyRptArray() == null)
			return;

		Iterator<TaggedSurgicalPathologyRptArray> itTagged = arr.getArrays().getTaggedSurgicalPathologyRptArray().iterator();
		while (itTagged.hasNext()) {
			TaggedSurgicalPathologyRptArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getRpts() == null || tagged.getRpts().getSurgicalPathologyRpt() == null)
				continue;

			Iterator<SurgicalPathologyRpt> it = tagged.getRpts().getSurgicalPathologyRpt().iterator();
			while (it.hasNext()) {
				SurgicalPathologyRpt surgicalPathologyRpt = it.next();
				//Perform Error Checking
				if (surgicalPathologyRpt.getFault() != null)
					throw new MdwsResponseException(surgicalPathologyRpt.getFault());

				ReportDocument reportDocument = createReportDocument(surgicalPathologyRpt, dateFilter);
				if (reportDocument != null) {
					reportDocument.setDocumentType(documentType);
					reportDocuments.add(reportDocument);
				}
			}
		}
	}

	private ReportDocument createReportDocument(SurgicalPathologyRpt surgicalPathologyRpt, DateFilter dateFilter) {
		String title = surgicalPathologyRpt.getTitle();
		String timestamp = surgicalPathologyRpt.getTimestamp();
		Date dtTimestamp = DateHelper.parseVistaDateTime(timestamp);
		String exam = surgicalPathologyRpt.getExam();
		
		if (isReportWithinRequiredDateRange(dtTimestamp, dateFilter) == false) {
			return null;
		}
		
		ReportDocument reportDocument = new ReportDocument();
		reportDocument.setTitle(title);
		if (timestamp != null)
			reportDocument.setTimestamp(dtTimestamp);
		
		setFacility(reportDocument, surgicalPathologyRpt);
		setSpecimen(reportDocument, surgicalPathologyRpt);
		reportDocument.setExam(exam);
		
		return reportDocument;
	}

	private void setFacility(ReportDocument reportDocument, SurgicalPathologyRpt surgicalPathologyRpt) {
		Facility facility = new Facility();
		
		TaggedText reportFacility = surgicalPathologyRpt.getFacility();
		if (reportFacility != null) {
			//Perform Error Checking
			if (reportFacility.getFault() != null)
				throw new MdwsResponseException(reportFacility.getFault());
		
			String text = reportFacility.getText();
			String tag = reportFacility.getTag();
			
			facility.setFacilityCode(text);
			facility.setName(tag);
		}
		
		reportDocument.setFacility(facility);
	}

	private void setSpecimen(ReportDocument reportDocument, SurgicalPathologyRpt surgicalPathologyRpt) {
		Specimen specimen = new Specimen();
		
		LabSpecimenTO labSpecimenTO = surgicalPathologyRpt.getSpecimen();
		if (labSpecimenTO != null) {
			//Perform Error Checking
			if (labSpecimenTO.getFault() != null)
				throw new MdwsResponseException(labSpecimenTO.getFault());
			
			String id = labSpecimenTO.getId();
			String name = labSpecimenTO.getName();
			String collectionDate = labSpecimenTO.getCollectionDate();
			Date dtCollectionDate = DateHelper.parseVistaDateTime(collectionDate);
			String accessionNum = labSpecimenTO.getAccessionNum();
			
			if (NullChecker.isNotNullish(collectionDate)) {
				specimen.setCollectionDate(dtCollectionDate);
				if (reportDocument.getTimestamp() == null)
					reportDocument.setTimestamp(dtCollectionDate);
			}
			
			specimen.setId(id);
			specimen.setName(name);
			specimen.setAccessionNumber(accessionNum);
		}
		
		reportDocument.setSpecimen(specimen);
	}

	private boolean isReportWithinRequiredDateRange(Date dateTime, DateFilter dateFilter){
		Date startDate = dateFilter.getStartDate();
		Date endDate = dateFilter.getEndDate();
		return (dateTime.equals(startDate) || dateTime.after(startDate)) && (dateTime.equals(endDate) || dateTime.before(endDate));
	}
	
//----------------------------------------------------------------------------------------------------------------------

	/**
	 * (3) Cytology Report
	 * 
	 * @param reportDocuments Unlike the other translators which return a collection, this expects you to pass the collection
	 * into the method and it will then populate that collection for you.  Passing in null for this will result in
	 * an exception being thrown.
	 */
	public void add(ReportDocuments reportDocuments, TaggedCytologyRptArrays arr, DateFilter dateFilter, String documentType) {
		if (reportDocuments == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_INPUT, "reportDocuments was null");
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedCytologyRptArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedCytologyRptArray() == null)
			return;

		Iterator<TaggedCytologyRptArray> itTagged = arr.getArrays().getTaggedCytologyRptArray().iterator();
		while (itTagged.hasNext()) {
			TaggedCytologyRptArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getRpts() == null || tagged.getRpts().getCytologyRpt() == null)
				continue;

			Iterator<CytologyRpt> it = tagged.getRpts().getCytologyRpt().iterator();
			while (it.hasNext()) {
				CytologyRpt cytologyRpt = it.next();
				//Perform Error Checking
				if (cytologyRpt.getFault() != null)
					throw new MdwsResponseException(cytologyRpt.getFault());

				ReportDocument reportDocument = createReportDocument(cytologyRpt, dateFilter);
				if (reportDocument != null) {
					reportDocument.setDocumentType(documentType);
					reportDocuments.add(reportDocument);
				}
			}
		}
	}

	private ReportDocument createReportDocument(CytologyRpt cytologyRpt, DateFilter dateFilter){
		if(isReportWithinRequiredDateRange(cytologyRpt, dateFilter) == false)
			return null;
			
		String title = cytologyRpt.getTitle();
		String exam = cytologyRpt.getExam();
		
		ReportDocument reportDocument = new ReportDocument();
		reportDocument.setTitle(title);
		setFacility(reportDocument, cytologyRpt);
		setSpecimen(reportDocument, cytologyRpt);
		reportDocument.setExam(exam);
		
		return reportDocument;
	}

	private void setFacility(ReportDocument reportDocument, CytologyRpt cytologyRpt) {
		Facility facility = new Facility();
		TaggedText reportFacility = cytologyRpt.getFacility();
		if (reportFacility != null) {
			//Perform Error Checking
			if (reportFacility.getFault() != null)
				throw new MdwsResponseException(reportFacility.getFault());
	
			String text = reportFacility.getText();
			String tag = reportFacility.getTag();
			facility.setFacilityCode(text);
			facility.setName(tag);
		}
		reportDocument.setFacility(facility);
	}

	private void setSpecimen(ReportDocument reportDocument, CytologyRpt cytologyRpt) {
		Specimen specimen = new Specimen();
		LabSpecimenTO labSpecimenTO = cytologyRpt.getSpecimen();
		if (labSpecimenTO != null) {
			//Perform Error Checking
			if (labSpecimenTO.getFault() != null)
				throw new MdwsResponseException(labSpecimenTO.getFault());
	
			String id = labSpecimenTO.getId();
			String name = labSpecimenTO.getName();
			String collectionDate = labSpecimenTO.getCollectionDate();
			String accessionNum = labSpecimenTO.getAccessionNum();
			specimen.setId(id);
			specimen.setName(name);
			if(NullChecker.isNotNullish(collectionDate)){
				Date dtCollectionDate = DateHelper.parseVistaDateTime(collectionDate);
				specimen.setCollectionDate(dtCollectionDate);
				reportDocument.setTimestamp(dtCollectionDate);
			}
			specimen.setAccessionNumber(accessionNum);
		}
		reportDocument.setSpecimen(specimen);
	}

	private boolean isReportWithinRequiredDateRange(CytologyRpt cytologyRpt, DateFilter dateFilter){
		String timestamp = cytologyRpt.getTimestamp();
		Date dateTime = DateHelper.parseVistaDateTime(timestamp);
		LabSpecimenTO specimen = cytologyRpt.getSpecimen();
		
		if (specimen != null) {
			//Perform Error Checking
			if (specimen.getFault() != null)
				throw new MdwsResponseException(specimen.getFault());
	
			if (dateTime == null) {
				String collectionDate = specimen.getCollectionDate();
				dateTime = DateHelper.parseVistaDateTime(collectionDate);
			}
		}
		
		if (dateTime != null) {
			boolean retValue = isReportWithinRequiredDateRange(dateTime, dateFilter);
			return retValue;
		}
		return false;
	}
	
//----------------------------------------------------------------------------------------------------------------------

	/**
	 * (4) Radiology Report
	 * 
	 * @param reportDocuments Unlike the other translators which return a collection, this expects you to pass the collection
	 * into the method and it will then populate that collection for you.  Passing in null for this will result in
	 * an exception being thrown.
	 */
	public void add(ReportDocuments reportDocuments, TaggedRadiologyReportArrays arr, String documentType) {
		if (reportDocuments == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_INPUT, "reportDocuments was null");
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedRadiologyReportArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedRadiologyReportArray() == null)
			return;

		Iterator<TaggedRadiologyReportArray> itTagged = arr.getArrays().getTaggedRadiologyReportArray().iterator();
		while (itTagged.hasNext()) {
			TaggedRadiologyReportArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getRpts() == null || tagged.getRpts().getRadiologyReportTO() == null)
				continue;

			Iterator<RadiologyReportTO> it = tagged.getRpts().getRadiologyReportTO().iterator();
			while (it.hasNext()) {
				RadiologyReportTO mdwsReport = it.next();
				//Perform Error Checking
				if (mdwsReport.getFault() != null)
					throw new MdwsResponseException(mdwsReport.getFault());

				ReportDocument reportDocument = createReportDocument(mdwsReport);
				reportDocument.setDocumentType(documentType);
				reportDocuments.add(reportDocument);
			}
		}
	}

	private ReportDocument createReportDocument(RadiologyReportTO mdwsReport) {
		String title = mdwsReport.getTitle();
		String id = mdwsReport.getId();
		String timestamp = mdwsReport.getTimestamp();
		Date dtTimestamp = DateHelper.parseVistaDateTime(timestamp);
		String text = mdwsReport.getText();
		String impression = mdwsReport.getImpression();
		Date dtResultedDate = MdwsDateTranslationUtil.extractDateOrNull(timestamp);
		
		ReportDocument reportDocument = new ReportDocument();
		reportDocument.setTestShortName(title);
		reportDocument.setUniqueId(id);
		if (timestamp != null)
			reportDocument.setTimestamp(dtTimestamp);
		
		reportDocument.setTestId(id);
		reportDocument.setReport("REPORT:\n " + text + "\n\nIMPRESSION:\n " + impression);
		reportDocument.setTestName(title);
		
		setFacility(reportDocument, mdwsReport);
		reportDocument.setResultedDate(dtResultedDate);
		
		return reportDocument;
	}

	private void setFacility(ReportDocument reportDocument, RadiologyReportTO mdwsReport) {
		TaggedText facility = mdwsReport.getFacility();
		if(facility != null) {
			//Perform Error Checking
			if (facility.getFault() != null)
				throw new MdwsResponseException(facility.getFault());
			
			String facilityName = facility.getTag(); // returns the station number TODO: Need to translate this somehow to a real facility name
			reportDocument.setSourceSystem(facilityName);
			reportDocument.setFacilityName(facilityName);
		} 
	}

//----------------------------------------------------------------------------------------------------------------------
	
	/**
	 * (5) Surgery Report
	 * 
	 * @param reportDocuments Unlike the other translators which return a collection, this expects you to pass the collection
	 * into the method and it will then populate that collection for you.  Passing in null for this will result in
	 * an exception being thrown.
	 */
	public void add(ReportDocuments reportDocuments, TaggedSurgeryReportArrays arr, DateFilter dateFilter, String documentType) {
		if (reportDocuments == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_INPUT, "reportDocuments was null");
		
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedSurgeryReportArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedSurgeryReportArray() == null)
			return;

		Iterator<TaggedSurgeryReportArray> itTagged = arr.getArrays().getTaggedSurgeryReportArray().iterator();
		while (itTagged.hasNext()) {
			TaggedSurgeryReportArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getRpts() == null || tagged.getRpts().getSurgeryReportTO() == null)
				continue;

			Iterator<SurgeryReportTO> it = tagged.getRpts().getSurgeryReportTO().iterator();
			while (it.hasNext()) {
				SurgeryReportTO surgeryReportTO = it.next();
				//Perform Error Checking
				if (surgeryReportTO.getFault() != null)
					throw new MdwsResponseException(surgeryReportTO.getFault());

				ReportDocument reportDocument = createReportDocument(surgeryReportTO, dateFilter);
				if (reportDocument != null) {
					reportDocument.setDocumentType(documentType);
					reportDocuments.add(reportDocument);
				}
			}
		}
	}

	private ReportDocument createReportDocument(SurgeryReportTO surgeryReportTO, DateFilter dateFilter) {
		String timestamp = surgeryReportTO.getTimestamp();
		Date dateTime = DateHelper.parseVistaDateTime(timestamp);
		if (isReportWithinRequiredDateRange(dateTime, dateFilter) == false)
			return null;
		
		String id = surgeryReportTO.getId();
		String title = surgeryReportTO.getTitle();
		String text = surgeryReportTO.getText();
		AuthorTO reportAuthor = surgeryReportTO.getAuthor();
		TaggedText facility = surgeryReportTO.getFacility();
		
		ReportDocument reportDocument = new ReportDocument();
		reportDocument.setRecordId(id);
		reportDocument.setTitle(title);
		if (timestamp != null)
			reportDocument.setTimestamp(dateTime);
		
		if (reportAuthor != null) {
			//Perform Error Checking
			if (reportAuthor.getFault() != null)
				throw new MdwsResponseException(reportAuthor.getFault());
			
			String authorId = reportAuthor.getId();
			String authorName = reportAuthor.getName();
			String authorSignature = reportAuthor.getSignature();
			Author author = new Author(authorId, authorName, authorSignature);
			reportDocument.setAuthor(author);
		}
		
		reportDocument.setText(text);
		
		if (facility != null) {
			//Perform Error Checking
			if (facility.getFault() != null)
				throw new MdwsResponseException(facility.getFault());

			String tag = facility.getTag();
			reportDocument.setLocation(tag);
		}
		
		return reportDocument;
	}
}
