package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.SecureMessageFolder;
import com.agilex.healthcare.mobilehealthplatform.domain.SecureMessageFolders;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.FolderTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.MailboxTO;

public class SecureMessageFolderTranslator {
	
	public static SecureMessageFolders translateMhvArrayOfFolderTo(MailboxTO arr) {
		SecureMessageFolders retValue = new SecureMessageFolders();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "MailboxTO was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getFolders() == null || arr.getFolders().getFolderTO() == null)
			return retValue;

		Iterator<FolderTO> itFolderTO = arr.getFolders().getFolderTO().iterator();
		while (itFolderTO.hasNext()) {
			FolderTO mhvFolder = itFolderTO.next();
			//Perform Error Checking
			if (mhvFolder.getFault() != null)
				throw new MdwsResponseException(mhvFolder.getFault());

			SecureMessageFolder folder = createSecureMessageFolder(mhvFolder);
			retValue.add(folder);
		}

		return retValue;
	}

	private static SecureMessageFolder createSecureMessageFolder(FolderTO mhvFolder) {		
		int id = mhvFolder.getId();
		String uniqueId = Integer.toString(id);
		String name = mhvFolder.getName();
		boolean isSystemFolder = mhvFolder.isIsSystemFolder();
		
		SecureMessageFolder folder = new SecureMessageFolder();
		folder.setUniqueId(uniqueId);
		folder.setName(name);
		folder.setSystemFolder(isSystemFolder);
		return folder;
	}
}
