package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.xml.datatype.XMLGregorianCalendar;

import com.agilex.healthcare.mobilehealthplatform.domain.SecureMessage;
import com.agilex.healthcare.mobilehealthplatform.domain.SecureMessages;
import com.agilex.healthcare.mobilehealthplatform.domain.code.AssigningAuthorityCode;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.AddresseeTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.ArrayOfMessageTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.FolderTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.MessageTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.SecureMessageThreadsTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.SmUserTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.ThreadTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.mhvservice.TriageGroupTO;
import com.agilex.healthcare.utility.DateHelper;

public class SecureMessageTranslator {
	public static SecureMessages translateMhvSecureMessageThreads(SecureMessageThreadsTO arr) {
		SecureMessages retValue = new SecureMessages();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "SecureMessageThreadsTO was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getMessageThreads() == null || arr.getMessageThreads().getThreadTO() == null)
			return retValue;

		Iterator<ThreadTO> itThreadTO = arr.getMessageThreads().getThreadTO().iterator();
		while (itThreadTO.hasNext()) {
			ThreadTO mhvThread = itThreadTO.next();
			//Perform Error Checking
			if (mhvThread.getFault() != null)
				throw new MdwsResponseException(mhvThread.getFault());

			SecureMessages messages = translateThread(mhvThread); //NOTICE this pattern differs from the other translators.
			retValue.addAll(messages);
		}

		return retValue;
	}

	public static SecureMessages translateThread(ThreadTO mhvThread) {
		SecureMessages retValue = new SecureMessages();
		if (mhvThread == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "ThreadTO was null");
		if (mhvThread.getFault() != null)
			throw new MdwsResponseException(mhvThread.getFault());
		if (mhvThread.getMessages() == null || mhvThread.getMessages().getMessageTO() == null)
			return retValue;

		Iterator<MessageTO> itMessageTO = mhvThread.getMessages().getMessageTO().iterator();
		while (itMessageTO.hasNext()) {
			MessageTO mhvMessage = itMessageTO.next();
			//Perform Error Checking
			if (mhvMessage.getFault() != null)
				throw new MdwsResponseException(mhvMessage.getFault());

			SecureMessage secureMessage = toSecureMessage(mhvMessage); //NOTICE this deviates from other translators.
			setValuesFromThread(secureMessage, mhvThread);
			retValue.add(secureMessage);
		}

		return retValue;
	}

	public static SecureMessage toSecureMessage(MessageTO mhvMessage) {
		if (mhvMessage == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "MessageTO was null");
		if (mhvMessage.getFault() != null)
			throw new MdwsResponseException(mhvMessage.getFault());
		
		int threadId = mhvMessage.getThreadId();
		int threadOplock = mhvMessage.getThreadOplock();
		int id = mhvMessage.getId();
		int oplock = mhvMessage.getOplock();
		String senderName = mhvMessage.getSenderName();
		int senderId = mhvMessage.getSenderId();
		String recipientName = mhvMessage.getRecipientName();
		int recipientId = mhvMessage.getRecipientId();
		String body = mhvMessage.getBody();
		XMLGregorianCalendar sentDate = mhvMessage.getSentDate();
		XMLGregorianCalendar escalationNotificationDate = mhvMessage.getEscalationNotificationDate();
		XMLGregorianCalendar escalatedDate = mhvMessage.getEscalatedDate();
		XMLGregorianCalendar completedDate = mhvMessage.getCompletedDate();
		Date dtDateSent = DateHelper.toDate(sentDate);
		Date dtEsclation = DateHelper.toDate(escalationNotificationDate);
		Date dtEscalated = DateHelper.toDate(escalatedDate);
		Date dtCompleted = DateHelper.toDate(completedDate);
		
		SecureMessage secureMessage = new SecureMessage();
		secureMessage.setThreadId(Integer.toString(threadId));
		secureMessage.setThreadOplock(Integer.toString(threadOplock));
		secureMessage.setUniqueId(Integer.toString(id));
		secureMessage.setMessageOplock(Integer.toString(oplock));
		secureMessage.setFrom(senderName);
		secureMessage.setFromId(Integer.toString(senderId));
		secureMessage.setTo(recipientName);
		secureMessage.setToId(Integer.toString(recipientId));
		secureMessage.setMessageBody(body);
		secureMessage.setDateSent(dtDateSent);
		secureMessage.setEscalationNotificationDate(dtEsclation);
		secureMessage.setEscalatedDate(dtEscalated);
		secureMessage.setCompletedDate(dtCompleted);
		setAddresseeValues(secureMessage, mhvMessage);

		return secureMessage;
	}

	private static void setAddresseeValues(SecureMessage secureMessage, MessageTO mhvMessage) {
		if (mhvMessage.getAddressees() == null || mhvMessage.getAddressees().getAddresseeTO() == null)
			return;
		Iterator<AddresseeTO> it = mhvMessage.getAddressees().getAddresseeTO().iterator();
		if (it.hasNext()) { //NOTICE this is an IF statement and not a WHILE.  They only wanted the first address.
			AddresseeTO mhvAddressee = it.next();
			if (mhvAddressee.getFault() != null)
				throw new MdwsResponseException(mhvAddressee.getFault());
			
			int id = mhvAddressee.getId();
			secureMessage.setAddresseeId(Integer.toString(id));
			int oplock = mhvAddressee.getOplock();
			secureMessage.setAddresseeOplock(Integer.toString(oplock));
			
			setFolder(secureMessage, mhvAddressee);
			setOwner(secureMessage, mhvAddressee);
			XMLGregorianCalendar mhvReadDate = mhvAddressee.getReadDate();
			Date readDate = DateHelper.toDate(mhvReadDate);
			secureMessage.setReadDate(readDate);
		}
	}

	private static void setFolder(SecureMessage secureMessage, AddresseeTO mhvAddressee) {
		FolderTO mhvFolder = mhvAddressee.getFolder();
		if (mhvFolder != null) {
			//Perform Error Checking
			if (mhvFolder.getFault() != null)
				throw new MdwsResponseException(mhvFolder.getFault());

			String name = mhvFolder.getName();
			int id = mhvFolder.getId();
			
			secureMessage.setFolder(name);
			secureMessage.setFolderId(Integer.toString(id));
		}
	}

	private static void setOwner(SecureMessage secureMessage, AddresseeTO mhvAddressee) {
		SmUserTO mhvOwner = mhvAddressee.getOwner();
		if (mhvOwner != null) {
			//Perform Error Checking
			if (mhvOwner.getFault() != null)
				throw new MdwsResponseException(mhvOwner.getFault());

			int id = mhvOwner.getId();
			String assigningAuthority = AssigningAuthorityCode.SECURE_MESSAGE;
			
			secureMessage.setAddresseeOwnerId(Integer.toString(id));
			secureMessage.setPatientIdentifier(assigningAuthority, Integer.toString(id));
		}
	}

	private static void setValuesFromThread(SecureMessage secureMessage, ThreadTO mhvThread) {
		if (mhvThread != null) {
			TriageGroupTO mailGroup = mhvThread.getMailGroup();
			String subject = mhvThread.getSubject();
			int messageCategory = mhvThread.getMessageCategory();
			
			if (mailGroup != null) {
				//Perform Error Checking
				if (mailGroup.getFault() != null)
					throw new MdwsResponseException(mailGroup.getFault());
				int id = mailGroup.getId();
				secureMessage.setMailGroup(Integer.toString(id));
			}
			secureMessage.setSubject(subject);
			secureMessage.setSubjectCategory(Integer.toString(messageCategory));
			boolean canEditToField = canEditToField(mhvThread);
			secureMessage.setReply(canEditToField);
		}
	}
	
	private static boolean canEditToField(ThreadTO mhvThread) {
		ArrayOfMessageTO messages = mhvThread.getMessages();
		
		if(messages != null) {
			List<MessageTO> listOfMessages = messages.getMessageTO();
			if(listOfMessages != null)
				return listOfMessages.size() > 1;
		}
		
		return false;
	}
}
