package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Author;
import com.agilex.healthcare.mobilehealthplatform.domain.SurgeryReport;
import com.agilex.healthcare.mobilehealthplatform.domain.SurgeryReports;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.AuthorTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.SurgeryReportTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedSurgeryReportArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedSurgeryReportArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedText;
import com.agilex.healthcare.utility.DateHelper;

public class SurgeryReportTranslator {
    public SurgeryReports translate(TaggedSurgeryReportArrays arr, DateFilter dateFilter, String documentType){
    	SurgeryReports retValue = new SurgeryReports();
    	if (arr == null)
    		throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedSurgeryReportArrays was null");
    	if (arr.getFault() != null)
    		throw new MdwsResponseException(arr.getFault());
    	if (arr.getArrays() == null || arr.getArrays().getTaggedSurgeryReportArray() == null)
    		return retValue;

    	Iterator<TaggedSurgeryReportArray> itTagged = arr.getArrays().getTaggedSurgeryReportArray().iterator();
    	while (itTagged.hasNext()) {
    		TaggedSurgeryReportArray tagged = itTagged.next();
    		//Perform Error Checking
    		if (tagged.getFault() != null)
    			throw new MdwsResponseException(tagged.getFault());

    		if (tagged.getRpts() == null || tagged.getRpts().getSurgeryReportTO() == null)
    			continue;

    		Iterator<SurgeryReportTO> it = tagged.getRpts().getSurgeryReportTO().iterator();
    		while (it.hasNext()) {
    			SurgeryReportTO mdwsSurgeryReportTO = it.next();
    			//Perform Error Checking
    			if (mdwsSurgeryReportTO.getFault() != null)
    				throw new MdwsResponseException(mdwsSurgeryReportTO.getFault());

    			SurgeryReport surgeryReport = createSurgeryReport(mdwsSurgeryReportTO, dateFilter, documentType);
    			if (surgeryReport != null)
    				retValue.add(surgeryReport);
    		}
    	}

    	return retValue;
    }

    private boolean isSurgeryReportWithinRequiredDateRange(Date dateTime, DateFilter dateFilter){
        Date startDate = dateFilter.getStartDate();
		Date endDate = dateFilter.getEndDate();
		return (dateTime.equals(startDate) || dateTime.after(startDate)) && (dateTime.equals(endDate) || dateTime.before(endDate));
    }

    private SurgeryReport createSurgeryReport(SurgeryReportTO mdwsSurgeryReportTO, DateFilter dateFilter, String documentType){
    	String timestamp = mdwsSurgeryReportTO.getTimestamp();
    	Date surgeryReportTimestamp = DateHelper.parseVistaDateTime(timestamp);
    	if (isSurgeryReportWithinRequiredDateRange(surgeryReportTimestamp, dateFilter) == false)
    		return null;
    	
    	String id = mdwsSurgeryReportTO.getId();
    	String title = mdwsSurgeryReportTO.getTitle();
    	String text = mdwsSurgeryReportTO.getText();
		
    	SurgeryReport surgeryReport = new SurgeryReport();
        surgeryReport.setId(id);
        surgeryReport.setTitle(title);
        surgeryReport.setTimestamp(surgeryReportTimestamp);
        surgeryReport.setText(text);
        surgeryReport.setDocumentType(documentType);
        
        setAuthor(surgeryReport, mdwsSurgeryReportTO);
        setFacility(surgeryReport, mdwsSurgeryReportTO);
        
        return surgeryReport;
    }
    
    private void setAuthor(SurgeryReport surgeryReport, SurgeryReportTO mdwsSurgeryReportTO) {
    	AuthorTO authorTO = mdwsSurgeryReportTO.getAuthor();
		if (authorTO == null)
    		return;
    	//Perform Error Checking
		if (authorTO.getFault() != null)
			throw new MdwsResponseException(authorTO.getFault());
		
    	String authorId = authorTO.getId();
		String authorName = authorTO.getName();
		String authorSignature = authorTO.getSignature();
		
		Author author = new Author(authorId, authorName, authorSignature);
		surgeryReport.setAuthor(author);
    }
    
    private void setFacility(SurgeryReport surgeryReport, SurgeryReportTO mdwsSurgeryReportTO) {
    	TaggedText facility = mdwsSurgeryReportTO.getFacility();
    	if (facility == null)
    		return;
    	//Perform Error Checking
		if (facility.getFault() != null)
			throw new MdwsResponseException(facility.getFault());
		
		String tag = facility.getTag();
		surgeryReport.setLocation(tag);
    }
}
