package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.*;

import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.*;

public class SurgeryTranslator {
	
	public Surgeries translate(TaggedSurgeryReportArrays arr) {
		Surgeries retValue = new Surgeries();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedSurgeryReportArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedSurgeryReportArray() == null)
			return retValue;

		Iterator<TaggedSurgeryReportArray> itTagged = arr.getArrays().getTaggedSurgeryReportArray().iterator();
		while (itTagged.hasNext()) {
			TaggedSurgeryReportArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getRpts() == null || tagged.getRpts().getSurgeryReportTO() == null)
				continue;

			Iterator<SurgeryReportTO> it = tagged.getRpts().getSurgeryReportTO().iterator();
			while (it.hasNext()) {
				SurgeryReportTO surgeryReport = it.next();
				//Perform Error Checking
				if (surgeryReport.getFault() != null)
					throw new MdwsResponseException(surgeryReport.getFault());

				Surgery surgery = createSurgery(surgeryReport);
				retValue.add(surgery);
			}
		}

		return retValue;
	}		
	
	private Surgery createSurgery(SurgeryReportTO surgeryReport) {
		String id = surgeryReport.getId();
		String timestamp = surgeryReport.getTimestamp();
		Date procedureDate = MdwsDateTranslationUtil.extractDateOrNull(timestamp);
		String title = surgeryReport.getTitle();
		TaggedText facility = surgeryReport.getFacility();
		String facilityTag = "";
		if(facility != null) {
			//Perform Error Checking
			if (facility.getFault() != null)
				throw new MdwsResponseException(facility.getFault());
			facilityTag = facility.getTag();
		}
		
		Surgery surgery = new Surgery();
		surgery.setUniqueId(id);
		surgery.setProcedureDate(procedureDate);
		surgery.setProcedure(title);
		surgery.setFacility(facilityTag);
		
		return surgery;
	}
}
