package com.agilex.healthcare.mobilehealthplatform.mdws.translator;


import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Facility;
import com.agilex.healthcare.mobilehealthplatform.domain.PathologyReport;
import com.agilex.healthcare.mobilehealthplatform.domain.PathologyReports;
import com.agilex.healthcare.mobilehealthplatform.domain.Specimen;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.LabSpecimenTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.SurgicalPathologyRpt;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedSurgicalPathologyRptArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedSurgicalPathologyRptArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedText;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;

public class SurgicalPathologyReportTranslator {

    public PathologyReports translate(TaggedSurgicalPathologyRptArrays arr, DateFilter dateFilter){
    	PathologyReports retValue = new PathologyReports();
    	if (arr == null)
    		throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedSurgicalPathologyRptArrays was null");
    	if (arr.getFault() != null)
    		throw new MdwsResponseException(arr.getFault());
    	if (arr.getArrays() == null || arr.getArrays().getTaggedSurgicalPathologyRptArray() == null)
    		return retValue;

    	Iterator<TaggedSurgicalPathologyRptArray> itTagged = arr.getArrays().getTaggedSurgicalPathologyRptArray().iterator();
    	while (itTagged.hasNext()) {
    		TaggedSurgicalPathologyRptArray tagged = itTagged.next();
    		//Perform Error Checking
    		if (tagged.getFault() != null)
    			throw new MdwsResponseException(tagged.getFault());

    		if (tagged.getRpts() == null || tagged.getRpts().getSurgicalPathologyRpt() == null)
    			continue;

    		Iterator<SurgicalPathologyRpt> it = tagged.getRpts().getSurgicalPathologyRpt().iterator();
    		while (it.hasNext()) {
    			SurgicalPathologyRpt surgicalPathologyRpt = it.next();
    			//Perform Error Checking
    			if (surgicalPathologyRpt.getFault() != null)
    				throw new MdwsResponseException(surgicalPathologyRpt.getFault());

    			PathologyReport pathologyReport = createPathologyReport(surgicalPathologyRpt, dateFilter);
    			if (pathologyReport != null)
    				retValue.add(pathologyReport);
    		}
    	}

    	return retValue;
    }

    private PathologyReport createPathologyReport(SurgicalPathologyRpt surgicalPathologyRpt, DateFilter dateFilter) {
    	String timestamp = surgicalPathologyRpt.getTimestamp();
    	Date dateTime = DateHelper.parseVistaDateTime(timestamp);
		if(isSurgicalReportWithinRequiredDateRange(dateTime, dateFilter) == false)
    		return null;
		
		String title = surgicalPathologyRpt.getTitle();
		String exam = surgicalPathologyRpt.getExam();
		
		PathologyReport pathologyReport = new PathologyReport();
        pathologyReport.setTitle(title);
        if (timestamp != null){
            pathologyReport.setTimestamp(dateTime);
        }
        
        setFacility(pathologyReport, surgicalPathologyRpt);
		setSpecimen(pathologyReport, surgicalPathologyRpt);
        
        pathologyReport.setExam(exam);
        return pathologyReport;
    }
    
    private void setFacility(PathologyReport pathologyReport, SurgicalPathologyRpt surgicalPathologyRpt) {
    	TaggedText mdwsFacility = surgicalPathologyRpt.getFacility();
    	if (mdwsFacility == null)
    		return;
    	//Perform Error Checking
		if (mdwsFacility.getFault() != null)
			throw new MdwsResponseException(mdwsFacility.getFault());
		
		String mdwsFacilityCode = mdwsFacility.getText();
		String mdwsFacilityName = mdwsFacility.getTag();
		
		Facility facility = new Facility();
        facility.setFacilityCode(mdwsFacilityCode);
        facility.setName(mdwsFacilityName);
        pathologyReport.setFacility(facility);
    }
    
    private void setSpecimen(PathologyReport pathologyReport, SurgicalPathologyRpt surgicalPathologyRpt) {
    	LabSpecimenTO labSpecimenTO = surgicalPathologyRpt.getSpecimen();
    	if (labSpecimenTO == null)
    		return;
    	//Perform Error Checking
		if (labSpecimenTO.getFault() != null)
			throw new MdwsResponseException(labSpecimenTO.getFault());
		String labSpecimenId = labSpecimenTO.getId();
        String labSpecimenName = labSpecimenTO.getName();
        String labSpecimenCollectionDate = labSpecimenTO.getCollectionDate();
        String labSpecimenAccessionNum = labSpecimenTO.getAccessionNum();
        
        Specimen specimen = new Specimen();
        specimen.setId(labSpecimenId);
        specimen.setName(labSpecimenName);
        if(NullChecker.isNotNullish(labSpecimenCollectionDate)){
            Date collectionDate = DateHelper.parseVistaDateTime(labSpecimenCollectionDate);
			specimen.setCollectionDate(collectionDate);
        }
        specimen.setAccessionNumber(labSpecimenAccessionNum);
        pathologyReport.setSpecimen(specimen);
    }

    private boolean isSurgicalReportWithinRequiredDateRange(Date dateTime, DateFilter dateFilter){
        Date startDate = dateFilter.getStartDate();
		Date endDate = dateFilter.getEndDate();
		return (dateTime.equals(startDate) || dateTime.after(startDate)) && (dateTime.equals(endDate) || dateTime.before(endDate));
    }
}
