package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Appointment;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointments;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.HospitalLocationTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedVisitArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.VisitTO;
import com.agilex.healthcare.utility.NullChecker;

public class VisitTranslator {
	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(VisitTranslator.class);
	
	public Appointments translate(TaggedVisitArray arr) {
		Appointments retValue = new Appointments();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedVisitArray was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getVisits() == null || arr.getVisits().getVisitTO() == null)
			return retValue;

		Iterator<VisitTO> it = arr.getVisits().getVisitTO().iterator();
		while (it.hasNext()) {
			VisitTO visitTO = it.next();
			//Perform Error Checking
			if (visitTO.getFault() != null)
				throw new MdwsResponseException(visitTO.getFault());

			Appointment appointment = createAppointment(visitTO);
			if (appointment != null)
				retValue.add(appointment);
		}

		return retValue;
	}
	
	private Appointment createAppointment(VisitTO visitTO) {
		
		HospitalLocationTO location = visitTO.getLocation();
		if (location == null) {
			LOGGER.debug("Removing Bad Data from MDWS results - appointment didn't have a location");
			return null;
		}
		//Perform Error Checking
		if (location.getFault() != null)
			throw new MdwsResponseException(location.getFault());
		
		String id = location.getId();
		String name = location.getName();
		
		if (NullChecker.isNullish(id) || NullChecker.isNullish(name)) {
			LOGGER.debug("Removing Bad Data from MDWS results - appointment didn't have a clinic id or a clinic name");
			return null;
		}
		
		Appointment appointment = new Appointment();
		String status = visitTO.getStatus();
		String timestamp = visitTO.getTimestamp();
		Date visitDate = MdwsDateTranslationUtil.extractDateOrNull(timestamp);
		String type = visitTO.getType();
		
		appointment.setClinicId(id);
		appointment.setClinicName(name);
		
		appointment.setStatus(status);
		appointment.setAppointmentStartDate(visitDate);
		appointment.setAppointmentEndDate(visitDate);
		appointment.setStatusCode(type);
		
		return appointment;
	}
}
