package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.VitalEntries;
import com.agilex.healthcare.mobilehealthplatform.domain.VitalEntry;
import com.agilex.healthcare.mobilehealthplatform.domain.VitalObservation;
import com.agilex.healthcare.mobilehealthplatform.domain.code.ObservationTypeCode;
import com.agilex.healthcare.mobilehealthplatform.domain.code.VitalSectionCode;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.ObservationTypeTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedText;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedVitalSignArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedVitalSignArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedVitalSignSetArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedVitalSignSetArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.VitalSignSetTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.VitalSignTO;
import com.agilex.healthcare.utility.Hasher;
import com.agilex.healthcare.utility.NullChecker;

public class VitalsTranslator {
	private static final String PULSE = "PULSE";
	private static final String BLOOD_PRESSURE = "BLOOD PRESSURE";
	private static final String PULSE_OXIMETRY = "PULSE OXIMETRY";
	private static final String PULSE_OXIMETRY_MISSPELLED = "PULSE OXYMETRY";
	private static final String RESPIRATION = "RESPIRATION";
	private static final String PAIN = "PAIN";
	private static final String TEMPERATURE = "TEMPERATURE";
	private static final String HEIGHT = "HEIGHT";
	private static final String WEIGHT = "WEIGHT";

	public VitalEntries translateArrays(TaggedVitalSignArrays arr) {
		VitalEntries retValue = new VitalEntries();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedVitalSignArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedVitalSignArray() == null)
			return retValue;

		Iterator<TaggedVitalSignArray> itTagged = arr.getArrays().getTaggedVitalSignArray().iterator();
		while (itTagged.hasNext()) {
			TaggedVitalSignArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getVitals() == null || tagged.getVitals().getVitalSignTO() == null)
				continue;

			Iterator<VitalSignTO> it = tagged.getVitals().getVitalSignTO().iterator();
			while (it.hasNext()) {
				VitalSignTO vitalSignTo = it.next();
				//Perform Error Checking
				if (vitalSignTo.getFault() != null)
					throw new MdwsResponseException(vitalSignTo.getFault());

				VitalEntry vitalEntry = createVitalEntry(vitalSignTo);
				retValue.add(vitalEntry);
			}
		}

		return retValue;
	}

	private VitalEntry createVitalEntry(VitalSignTO vitalSignTo) {
		String timestamp = vitalSignTo.getTimestamp();
		Date entryDate = MdwsDateTranslationUtil.extractDateOrNull(timestamp);
		String id = vitalSignTo.getId();
		TaggedText facility = vitalSignTo.getFacility();
		
		VitalEntry vitalEntry = new VitalEntry();
		vitalEntry.setEntryDate(entryDate);
		setVitalObservationValues(vitalEntry, vitalSignTo);
		vitalEntry.setUniqueId(id);
		if (facility != null) {
			//Perform Error Checking
			if (facility.getFault() != null)
				throw new MdwsResponseException(facility.getFault());
			String text = facility.getText();
			vitalEntry.setSourceSystem(text);
		}

		return vitalEntry;
	}

	// /

	public VitalEntries translateSets(TaggedVitalSignSetArrays arr) {
		VitalEntries retValue = new VitalEntries();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedVitalSignSetArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedVitalSignSetArray() == null)
			return retValue;

		Iterator<TaggedVitalSignSetArray> itTagged = arr.getArrays().getTaggedVitalSignSetArray().iterator();
		while (itTagged.hasNext()) {
			TaggedVitalSignSetArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getSets() == null || tagged.getSets().getVitalSignSetTO() == null)
				continue;

			Iterator<VitalSignSetTO> itVitalSignSetTO = tagged.getSets().getVitalSignSetTO().iterator();
			while (itVitalSignSetTO.hasNext()) {
				VitalSignSetTO vitalSignSetTO = itVitalSignSetTO.next();
				//Perform Error Checking
				if (vitalSignSetTO.getFault() != null)
					throw new MdwsResponseException(vitalSignSetTO.getFault());
				
				String qualifiers = vitalSignSetTO.getQualifiers();
				String timestamp = vitalSignSetTO.getTimestamp();
				String facilityTag = null;
				TaggedText facility = vitalSignSetTO.getFacility();
				String facilityText = "";
				String units = vitalSignSetTO.getUnits();
				
				if (facility != null) {
					//Perform Error Checking
					if (facility.getFault() != null)
						throw new MdwsResponseException(facility.getFault());
					facilityTag = facility.getTag();
					facilityText = facility.getText();
				}
				
				if (vitalSignSetTO.getVitalSigns() == null || vitalSignSetTO.getVitalSigns().getVitalSignTO() == null)
					continue;
				
				Iterator<VitalSignTO> it = vitalSignSetTO.getVitalSigns().getVitalSignTO().iterator();
				while (it.hasNext()) {
					VitalSignTO vitalSignTO = it.next();
					//Perform Error Checking
					if (vitalSignTO.getFault() != null)
						throw new MdwsResponseException(vitalSignTO.getFault());
					
					VitalEntry vitalEntry = createVitalEntry(vitalSignTO, qualifiers, timestamp, facilityTag, facilityText, units);
					retValue.add(vitalEntry);
				}
			}
		}

		return retValue;
	}

	private VitalEntry createVitalEntry(VitalSignTO vitalSign, String qualifiers, String timestamp, String facilityTag, String facilityText, String units) {
		Date entryDate = MdwsDateTranslationUtil.extractDateOrNull(timestamp);

		VitalEntry vitalEntry = new VitalEntry();
		setIdentifier(vitalEntry, vitalSign, qualifiers, timestamp, facilityTag, facilityText, units);
		setVitalObservationValues(vitalEntry, vitalSign);

		vitalEntry.setQualifier(qualifiers);
		vitalEntry.setEntryDate(entryDate);
		vitalEntry.setSourceSystem(facilityTag);

		
		return vitalEntry;
	}

	private void setIdentifier(VitalEntry vitalEntry, VitalSignTO vitalSign, String qualifiers, String timestamp, String facilityTag, String facilityText, String units) {
		//Handle scenarios where vitalEntry doesn't have a data identifier.
		//We want to create something unique yet at the same time we need to be able to refer back to it in case they
		//ask for a single Vital.  So, we will create a unique id by taking all of the possible data that we know about
		//to create a String that contains everything and creating a hash from that String.
		String id = null;
		ObservationTypeTO type = vitalSign.getType();
		if (type != null) {
			//Perform Error Checking
			if (type.getFault() != null)
				throw new MdwsResponseException(type.getFault());
			id = type.getId();
		}
		if (NullChecker.isNullish(id))
			id = createRepeatableUniqueId(vitalSign, qualifiers, timestamp, facilityTag, facilityText, units);
		
		vitalEntry.setUniqueId(id);
	}

	private String createRepeatableUniqueId(VitalSignTO vitalSign, String qualifiers, String timestamp, String facilityTag, String facilityText, String units) {
		String uniqueId = "" +
				qualifiers + ":" + timestamp + ":" + facilityTag + ":" + facilityText + ":" + units +
				":" + vitalSign.getId() +
				":" + vitalSign.getComment() +
				":" + vitalSign.getQualifiers() +
				":" + vitalSign.getTimestamp() +
				":" + vitalSign.getUnits() +
				":" + vitalSign.getValue1() +
				":" + vitalSign.getValue2() +
				
				":" + (vitalSign.getFacility() == null ? "" : vitalSign.getFacility().getText()) +
				":" + (vitalSign.getLocation() == null ? "" : vitalSign.getLocation().getId()) +
				":" + (vitalSign.getObserver() == null ? "" : vitalSign.getObserver().getId()) +
				":" + (vitalSign.getRecorder() == null ? "" : vitalSign.getRecorder().getId()) +
				":" + (vitalSign.getType() == null ? "" : vitalSign.getType().getCategory()) +
				":" + (vitalSign.getType() == null ? "" : vitalSign.getType().getId()) +
				":" + (vitalSign.getType() == null ? "" : vitalSign.getType().getName());
		
		Hasher hasher = new Hasher();
		String retvalue = hasher.hashStringToString(uniqueId);
		return retvalue;
	}

	private void setVitalObservationValues(VitalEntry vitalEntry, VitalSignTO vitalSign) {

		String typeName = null;
		String value1 = vitalSign.getValue1();
		ObservationTypeTO type = vitalSign.getType();
		
		if (type != null) {
			//Perform Error Checking
			if (type.getFault() != null)
				throw new MdwsResponseException(type.getFault());
			typeName = type.getName();
		}
		
		if (VitalsTranslator.BLOOD_PRESSURE.equalsIgnoreCase(typeName)) {
			vitalEntry.setSection(VitalSectionCode.BloodPressure);
			setBloodPressureObservationValues(vitalEntry, vitalSign);
			return;
		}

		VitalObservation vitalObservation = new VitalObservation();

		if (VitalsTranslator.PULSE.equalsIgnoreCase(typeName)) {
			vitalObservation.setType(ObservationTypeCode.HEART_BEAT);
			vitalEntry.setSection(VitalSectionCode.Pulse);
		} else if (VitalsTranslator.TEMPERATURE.equalsIgnoreCase(typeName)) {
			vitalObservation.setType(ObservationTypeCode.BODY_TEMPERATURE);
			vitalEntry.setSection(VitalSectionCode.Temperature);
		} else if (VitalsTranslator.HEIGHT.equalsIgnoreCase(typeName)) {
			vitalObservation.setType(ObservationTypeCode.HEIGHT);
			vitalEntry.setSection(VitalSectionCode.Height);
		} else if (VitalsTranslator.WEIGHT.equalsIgnoreCase(typeName)) {
			vitalObservation.setType(ObservationTypeCode.WEIGHT);
			vitalEntry.setSection(VitalSectionCode.Weight);
		} else if (VitalsTranslator.PAIN.equalsIgnoreCase(typeName)) {
			vitalObservation.setType(ObservationTypeCode.PAIN);
			vitalEntry.setSection(VitalSectionCode.Pain);
		} else if (VitalsTranslator.RESPIRATION.equalsIgnoreCase(typeName)) {
			vitalObservation.setType(ObservationTypeCode.RESPIRATION);
			vitalEntry.setSection(VitalSectionCode.Respiration);
		} else if (VitalsTranslator.PULSE_OXIMETRY.equalsIgnoreCase(typeName) || VitalsTranslator.PULSE_OXIMETRY_MISSPELLED.equalsIgnoreCase(typeName)) {
			vitalObservation.setType(ObservationTypeCode.PULSE_OXIMETRY);
			vitalEntry.setSection(VitalSectionCode.PulseOximetry);
		} else {
			// logger.debug("Vital type from CDW was not translated properly");
		}

		vitalObservation.setVitalEntry(vitalEntry);
		vitalObservation.setValue(value1);
		vitalEntry.addObservation(vitalObservation);
	}

	void setBloodPressureObservationValues(VitalEntry vitalEntry, VitalSignTO vitalSign) {
		if (vitalSign.getValue1() == null)
			return;
		
		String[] systolicDiastolic = vitalSign.getValue1().split("/");

		if (systolicDiastolic.length == 2) {
			VitalObservation observation = new VitalObservation();
			observation.setType(ObservationTypeCode.INTRAVASCULAR_SYSTOLIC);
			observation.setValue(systolicDiastolic[0]);
			observation.setValueUnits("mmHg");
			vitalEntry.addObservation(observation);

			observation = new VitalObservation();
			observation.setType(ObservationTypeCode.INTRAVASCULAR_DIASTOLIC);
			observation.setValue(systolicDiastolic[1]);
			observation.setValueUnits("mmHg");
			vitalEntry.addObservation(observation);
		}
	}
}
