package com.agilex.healthcare.mobilehealthplatform.restservice;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.datalayer.facility.FacilityDataService;
import com.agilex.healthcare.mobilehealthplatform.datalayer.facility.FacilityFilter;
import com.agilex.healthcare.mobilehealthplatform.datalayer.facility.FacilityFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.domain.Facilities;
import com.agilex.healthcare.mobilehealthplatform.utils.PilotFacilitiesHelper;

/**
 * REST Services for working with facilities
 */
@Path("/facility-service/")
@Component
@Scope("request")
public class FacilityResource {
	@Resource
	PilotFacilitiesHelper pilotFacilitiesHelper;
	
	/**
	 * Retrieves the facilities for the app passed in.<br/>
	 * Calls CDW<br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>term</td>
	 * 		<td>Filters based on term</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>parentFacilitiesOnly</td>
	 * 		<td>Filters based on parent facilities only</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param app
	 * @return JSON containing facilities for the app passed in.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("facilities")
	public Facilities fetchFacilities(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @QueryParam("app") String app) {
		FacilityFilterFactory facilityFilterFactory = new FacilityFilterFactory(pilotFacilitiesHelper.getPilotSites(app));
		FacilityFilter filter = facilityFilterFactory.createFromUri(uriInfo.getRequestUri());
		FacilityDataService dataService = new FacilityDataService();
		return dataService.fetchFacilities(filter);
	}
	
}
