package com.agilex.healthcare.mobilehealthplatform.restservice;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.datalayer.order.OrderServicesDataService;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderServices;

@Path("/order-services")
@Component
@Scope("request")
public class OrderServicesResource {
	/**
	 * Calls the MDWS service OrderMgmtSvc.getOrdersServices<br/>
	 * If the property order_services_days_to_cache is -1, we will no longer cache the data in Mongo.  Otherwise, after
	 * retrieving this data from MDWS we cache it in Mongo.  We then compare the dates of every call to what we have
	 * stored as the last cached date in Mongo.  If the timespan between today and this last cached date is less than
	 * the property we give them the data from Mongo.  If it's more, we retrieve the data from MDWS, update Mongo, and
	 * give them the new data.
	 * 
	 * @param assigningAuthority
	 * @param patientId
	 * @param site
	 * @param uriInfo
	 * @param headers
	 * @return
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{site}")
	public OrderServices fetchOrderServices(@PathParam("assigning-authority") String assigningAuthority, 
											   @PathParam("patient-id") String patientId, 
											   @PathParam("site") String site,
											   @Context UriInfo uriInfo, 
											   @Context HttpHeaders headers) {
		OrderServicesDataService ds = new OrderServicesDataService();
		OrderServices retvalue = ds.fetchOrderServices(site);
		return retvalue;
	}
}
