package com.agilex.healthcare.mobilehealthplatform.restservice;

import com.agilex.healthcare.mobilehealthplatform.datalayer.lab.LabTestsDataLayer;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTest;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTests;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.ChemistryLabTestLinkBuilder;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;
import java.net.URI;
import java.util.List;

/**
 * 
 * Patient Labs rest service
 *
 */
@Path("/patient/{assigning-authority}/{patient-id}/lab-tests")
@Component
@Scope("request")
public class PatientLabTestsResource {
	@Resource
	LabTestsDataLayer labTestsDataLayer;

	@GET
	@Path("site/{site-code}")
	@Produces({ "application/xml", "application/json" })
	public LabTests getLabTests(
			@PathParam("assigning-authority") String assigningAuthority,
			@PathParam("patient-id") String patientId,
			@PathParam("site-code") String siteCode,
			@QueryParam("lab-category") List<String> labCategoryList,
			@Context UriInfo uriInfo,
			@Context HttpHeaders headers) {

		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);

		LabTests labTests = labTestsDataLayer.getLabTests(patientIdentifier, siteCode, labCategoryList);
		fillLinks(labTests, uriInfo.getBaseUri());

		return labTests;
	}

	@GET
	@Path("id/{lab-test-ien}/test-name/{lab-test-name}/site/{site-code}")
	@Produces({ "application/xml", "application/json" })
	public LabTest getLabTest(
			@PathParam("assigning-authority") String assigningAuthority,
			@PathParam("patient-id") String patientId,
			@PathParam("lab-test-ien") String labTestIen,
			@PathParam("lab-test-name") String labTestName,
			@PathParam("site-code") String siteCode,
			@QueryParam("startDate") String startDate,
			@QueryParam("endDate") String endDate,
			@Context UriInfo uriInfo,
			@Context HttpHeaders headers) {

		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);

		LabTest labTest = labTestsDataLayer.getLabResults(patientIdentifier, labTestIen, labTestName, siteCode, startDate, endDate);
		fillLinks(labTest, uriInfo.getBaseUri());

		return labTest;
	}

	private void fillLinks(LabTests labTests, URI baseUri) {
		ChemistryLabTestLinkBuilder linkBuilder = new ChemistryLabTestLinkBuilder(baseUri);

		for (LabTest labTest : labTests) {
			linkBuilder.fillLinks(labTest, null);
		}
	}

	private void fillLinks(LabTest labTest, URI baseUri) {
		ChemistryLabTestLinkBuilder linkBuilder = new ChemistryLabTestLinkBuilder(baseUri);
		linkBuilder.fillLinks(labTest, null);
	}
}
