package com.agilex.healthcare.mobilehealthplatform.restservice;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.datalayer.order.OrderResultsHistoryDataService;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderResultsHistory;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;

@Path("/patient/{assigning-authority}/{patient-id}/order-results-history")
@Component
@Scope("request")
public class PatientOrderResultsHistoryResource {
	/**
	 * Retrieves the OrderResults History for a patient.<br/>
	 * Calls the MDWS service OrderMgmtSvc.getOrderHistory
	 * 
	 * @param assigningAuthority Assigning Authority associated with the patient's login
	 * @param patientId Identifier of the Patient
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{site}")
	public OrderResultsHistory fetchPatientOrderResultsHistory(@PathParam("assigning-authority") String assigningAuthority, 
											   @PathParam("patient-id") String patientId, 
											   @PathParam("site") String site,
											   @QueryParam("order-ien") String orderIen,
											   @Context UriInfo uriInfo, 
											   @Context HttpHeaders headers) {
		OrderResultsHistoryDataService ds = new OrderResultsHistoryDataService();
		PatientIdentifier pi = new PatientIdentifier(assigningAuthority, patientId);
		OrderResultsHistory retvalue = ds.fetchPatientOrderResultHistory(pi, orderIen);
        return retvalue;
	}
}
