package com.agilex.healthcare.mobilehealthplatform.restservice;

import com.agilex.healthcare.mobilehealthplatform.datalayer.pathology.PathologyDataService;
import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriInfo;

/**
 * 
 * Rest service to retrieve patient pathology labs
 *
 */
@Path("/patient/{assigning-authority}/{patient-id}/lab-pathology")
@Component
@Scope("request")
public class PatientPathologyLabsResource {

    /**
     * This resource retrieves list of Pathology Reports for a patient from VistA.
     * @param assigningAuthority
     * @param patientId
     * @return {@link PathologyReports}
     */
    @GET
    @Produces({ "application/xml", "application/json" })
    @Path("pathology-reports")
    public PathologyReports fetchPatientPathologyReports(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @Context UriInfo uriInfo){
        PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
        DateFilter dateFilter = DateFilterFactory.createFilterFromUri(uriInfo);
        PathologyDataService pathologyDataService = new PathologyDataService();
        return pathologyDataService.getPathologyReportsForPatient(patientIdentifier, dateFilter);
    }
}
