package com.agilex.healthcare.mobilehealthplatform.restservice;

import com.agilex.healthcare.mobilehealthplatform.datalayer.surgery.SurgeryDataService;
import com.agilex.healthcare.mobilehealthplatform.domain.Author;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.SurgeryReport;
import com.agilex.healthcare.mobilehealthplatform.domain.SurgeryReports;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.utility.DateHelper;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriInfo;

@Path("/surgery-reports/site/{site-id}")
@Component
@Scope("request")
public class SurgeryReportResource {

    /**
     * This resource returns Surgery Reports for a patient from VistA.
     * @param siteId
     * @param assigningAuthority
     * @param patientId
     * @param uriInfo
     * @return
     */
    @GET
    @Produces({ "application/xml", "application/json" })
    @Path("patient/{assigning-authority}/{patient-id}")
    public SurgeryReports fetchSurgeryReportsForPatient(@PathParam("site-id") String siteId, @PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @Context UriInfo uriInfo){
        SurgeryDataService surgeryDataService = new SurgeryDataService();
        return surgeryDataService.getPatientSurgeryReports(new PatientIdentifier(assigningAuthority, patientId), DateFilterFactory.createFilterFromUri(uriInfo));
    }
}
