package com.agilex.healthcare.mobilehealthplatform.restservice.progressnote;

import com.agilex.healthcare.mobilehealthplatform.datalayer.progressnote.ProgressNoteDataService;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNotes;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriInfo;

@Path("/progress-note/site/{site-id}")
@Component
@Scope("request")
public class ProgressNoteResource {

    /**
     * This resource returns Progress Notes for a patient from VistA.
     * @param siteCode
     * @param assigningAuthority
     * @param patientId
     * @param uriInfo
     * @return
     */
    @GET
    @Path("/patient/{assigning-authority}/{patient-id}/notes")
    @Produces({ "application/json", "application/xml" })
    public ProviderNotes fetchPatientProgressNotes(@PathParam("site-id") String siteCode, @PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @Context UriInfo uriInfo){
        ProgressNoteDataService progressNoteDataService = new ProgressNoteDataService();
        ProviderNotes providerNotes = progressNoteDataService.getPatientProgressNotes(new PatientIdentifier(assigningAuthority, patientId), DateFilterFactory.createFilterFromUri(uriInfo));
        return providerNotes;
    }
}
