package com.agilex.healthcare.mobilehealthplatform.web;

import agilex.commons.helper.PropertyHelper;
import com.agilex.healthcare.mobilehealthplatform.domain.LinkTitles;
import com.agilex.healthcare.mobilehealthplatform.domain.ResourceDirectory;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.ResourceDirectoryBuilder;
import com.agilex.healthcare.utility.NullChecker;
import org.codehaus.jackson.map.ObjectMapper;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.json.MappingJacksonHttpMessageConverter;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.oauth2.client.DefaultOAuth2ClientContext;
import org.springframework.security.oauth2.client.OAuth2RestTemplate;
import org.springframework.security.oauth2.client.token.AccessTokenProvider;
import org.springframework.security.oauth2.client.token.AccessTokenProviderChain;
import org.springframework.security.oauth2.client.token.AccessTokenRequest;
import org.springframework.security.oauth2.client.token.DefaultAccessTokenRequest;
import org.springframework.security.oauth2.client.token.grant.client.ClientCredentialsAccessTokenProvider;
import org.springframework.security.oauth2.client.token.grant.code.AuthorizationCodeAccessTokenProvider;
import org.springframework.security.oauth2.client.token.grant.code.AuthorizationCodeResourceDetails;
import org.springframework.security.oauth2.client.token.grant.implicit.ImplicitAccessTokenProvider;
import org.springframework.security.oauth2.client.token.grant.password.ResourceOwnerPasswordAccessTokenProvider;
import org.springframework.security.oauth2.common.AuthenticationScheme;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.stereotype.Controller;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

@Controller
public class LoginLogoutController{

	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(LoginLogoutController.class);

	private static final String KEY_AUTH_CODE = "code";
	private static final String KEY_GRANT_TYPE = "grant_type";
	private static final String KEY_SCOPE = "scope";
	private static final String KEY_REDIRECT_URI = "redirect_uri";
	private static final String KEY_ORIGINAL_REDIRECT_URI = "original_redirect_uri";
	private static final String KEY_CLIENT_ID = "client_id";
	private static final String KEY_STATE = "state";
	private static final String KEY_RESPONSE_TYPE = "response_type";
	private static final String KEY_CLIENT_SECRET = "client_secret";

	private static final String CLIENT_SECRET = "oauth.client_secret";
	private static final String GRANT_TYPE = "oauth.grant_type";
	private static final String SCOPE = "oauth.scope";
	private static final String STATE = "oauth.state";
	private static final String CLIENT_ID = "oauth.client_id";

    private static final String KEY_ENABLE_REDIRECT_SECURITY = "oauth.enable_redirect_security";
    private static final String KEY_REDIRECT_URI_ROOTS = "oauth.permitted_redirect_uri_roots";

	@Resource
	private PropertyHelper propertyHelper;

	@Resource
	ResourceDirectoryBuilder resourceDirectoryBuilder;

	@RequestMapping(value = "/oauthlogin", method = RequestMethod.GET)
	public void redirectToLoginPage(@RequestParam(value = "error", required = false) boolean error,
			@RequestParam(value = KEY_REDIRECT_URI, required = false) String originalRedirectUri,
			HttpServletRequest req, HttpServletResponse response) throws Exception{
		logger.debug("In Oauth login");

		String baseUri = buildBaseUriString(req);
		ResourceDirectory resourceDirectory = buildResourceDirectory(req);
		String mhpAuthorizeUrl = resourceDirectory.getLink().getLinkByTitle(LinkTitles.OAuthAuthorize).getHref().toString();

		UriBuilder authorizeUrlBuilder = UriBuilder.fromPath(mhpAuthorizeUrl);

		String redirectUri = baseUri + "/oauthtoken";
		if (NullChecker.isNotNullish(originalRedirectUri)) {
			StringBuilder sb = new StringBuilder(redirectUri);
			sb.append("?");
			sb.append(KEY_ORIGINAL_REDIRECT_URI);
			sb.append("=");
			sb.append(originalRedirectUri);
			redirectUri = sb.toString();
		}

		URI uri = authorizeUrlBuilder
				.queryParam(KEY_RESPONSE_TYPE, KEY_AUTH_CODE)
				.queryParam(KEY_STATE, propertyHelper.getProperty(STATE))
				.queryParam(KEY_CLIENT_ID, propertyHelper.getProperty(CLIENT_ID))
				.queryParam(KEY_REDIRECT_URI, redirectUri)
				.queryParam(KEY_SCOPE, propertyHelper.getProperty(SCOPE)).build();

		redirect(response, uri.toString());
	}

	@RequestMapping(value = "/oauthtoken", method = RequestMethod.GET)
	public void getOauthToken(@RequestParam(value = KEY_AUTH_CODE, required = false) String code,
			@RequestParam(value = KEY_ORIGINAL_REDIRECT_URI, required = false) String originalRedirectUri,
			HttpServletRequest request, HttpServletResponse response) {

		String contextPath = NullChecker.isNotNullish(request.getContextPath()) ? request.getContextPath() : "";
		String redirectUri = NullChecker.isNotNullish(originalRedirectUri) ? originalRedirectUri : contextPath + "/launchpad/";

		if (NullChecker.isNotNullish(code)) {
			logger.debug("In Oauth token for code: " + code);
			SecurityContext ctx = SecurityContextHolder.getContext();
			ctx.setAuthentication(null);
			String token = getAccessToken(request, code, originalRedirectUri);
			redirectUri = redirectUri + "?token=" + token;
		} else {
			// START
			// the code below is not necessary when oauth login is completely handled by separate authoriziation service
			HttpSession session = request.getSession(false);
			if (session != null) {
				session.invalidate();
			}
			// END
		}
		redirect(response, redirectUri);
	}


	private String getAccessToken(HttpServletRequest req, String code, String originalRedirectUri) {
		String baseUri = buildBaseUriString(req);

		ResourceDirectory resourceDirectory = buildResourceDirectory(req);

		String redirectUri = baseUri + "/oauthtoken";
		if (NullChecker.isNotNullish(originalRedirectUri)) {
			StringBuilder sb = new StringBuilder(redirectUri);
			sb.append("?");
			sb.append(KEY_ORIGINAL_REDIRECT_URI);
			sb.append("=");
			sb.append(originalRedirectUri);
			redirectUri = sb.toString();
		}

		MultiValueMap<String, String> formData = new LinkedMultiValueMap<String, String>();
		formData.add(KEY_GRANT_TYPE, propertyHelper.getProperty(GRANT_TYPE));
		formData.add(KEY_CLIENT_ID, propertyHelper.getProperty(CLIENT_ID));
		formData.add(KEY_SCOPE, propertyHelper.getProperty(SCOPE));
		formData.add(KEY_AUTH_CODE, code);
		formData.add(KEY_CLIENT_SECRET, propertyHelper.getProperty(CLIENT_SECRET));
		formData.add(KEY_REDIRECT_URI, redirectUri);
		AuthorizationCodeResourceDetails resource = new AuthorizationCodeResourceDetails();

		String mhpTokenUrl = resourceDirectory.getLink().getLinkByTitle(LinkTitles.OAuthToken).getHref().toString();
		resource.setAccessTokenUri(mhpTokenUrl);
		resource.setClientId(propertyHelper.getProperty(CLIENT_ID));
		resource.setScope(Arrays.asList(propertyHelper.getProperty(SCOPE)));
		resource.setClientSecret(propertyHelper.getProperty(CLIENT_SECRET));
		resource.setClientAuthenticationScheme(AuthenticationScheme.query);

		AccessTokenRequest request = new DefaultAccessTokenRequest();
		request.setAuthorizationCode(code);
		request.setPreservedState(new Object());
		request.setAll(formData.toSingleValueMap());

		OAuth2RestTemplate template = new OAuth2RestTemplate(resource, new DefaultOAuth2ClientContext(request));
		addJSONMessageConverter(template);

		OAuth2AccessToken accessToken = template.getAccessToken();

		return accessToken.getValue();
	}

	// This was added because of incompatability introduced by Yammer metrics library. Yammer metrics library has dependency on Jackson library and it was
	// interfering with object mapper
	private void addJSONMessageConverter(OAuth2RestTemplate template) {
		List<HttpMessageConverter<?>> messageConverters = new ArrayList<HttpMessageConverter<?>>();
		MappingJacksonHttpMessageConverter jsonMessageConverter = new MappingJacksonHttpMessageConverter();
		jsonMessageConverter.setObjectMapper(new ObjectMapper());
		messageConverters.add(jsonMessageConverter);

		AuthorizationCodeAccessTokenProvider authorizationCodeAccessTokenProvider = new AuthorizationCodeAccessTokenProvider();
		authorizationCodeAccessTokenProvider.setMessageConverters(messageConverters);

		AccessTokenProviderChain accessTokenProvider = new AccessTokenProviderChain(Arrays.<AccessTokenProvider> asList(
				authorizationCodeAccessTokenProvider, new ImplicitAccessTokenProvider(),
				new ResourceOwnerPasswordAccessTokenProvider(), new ClientCredentialsAccessTokenProvider()));

		template.setAccessTokenProvider(accessTokenProvider);
	}


	private ResourceDirectory buildResourceDirectory(HttpServletRequest req) {
		URI baseUri = UriBuilder.fromUri(buildBaseUriString(req)).build();
		URI restBaseUri = UriBuilder.fromUri(baseUri).path("rest").build();
		URI requestUri = UriBuilder.fromUri(restBaseUri).path("public").path("resource-directory").build();

		return resourceDirectoryBuilder.getResourceDirectory(restBaseUri, requestUri);
	}

	private String buildBaseUriString(HttpServletRequest req) {
		StringBuilder sb = new StringBuilder();
		sb.append(req.getScheme());
		sb.append("://");
		sb.append(req.getServerName());
		sb.append(":");
		sb.append(req.getServerPort());
		sb.append(req.getContextPath());

		return sb.toString();
	}

	private void redirect(HttpServletResponse response, String redirectUri){

        if(redirectSecurityIsEnabled() && !redirectUriIsPermitted(redirectUri)) {
            logger.debug("Redirect URI path is not white-listed");
            throw new WebApplicationException(Response.Status.BAD_REQUEST);
        }

        try {
			response.sendRedirect(redirectUri);
		} catch (IOException e) {
			throw new RuntimeException("Exception while redirecting to " + redirectUri, e);
		}
	}

    /*
     * This method ensures that the URI matches permitted base uris
     * using properties configured on the classpath.
     *
     * The property associated with KEY_REDIRECT_URI_ROOTS can be a comma
     * separated list of base URIs
     *
     * @param redirectUri
     */
    boolean redirectUriIsPermitted(String redirectUri) {
        if(NullChecker.isNullish(redirectUri))
            return false;

        String redirectUriRoots = propertyHelper.getProperty(KEY_REDIRECT_URI_ROOTS);

        if(NullChecker.isNotNullish(redirectUriRoots)) {
            String[] roots = redirectUriRoots.split(",");

            String ignoreCaseRedirectUri = redirectUri.toLowerCase();
            for (String root : roots) {
                String ignoreCaseRoot = root.toLowerCase();
                if (ignoreCaseRedirectUri.startsWith(ignoreCaseRoot))
                    return true;
            }
        }

        return false;
    }

    boolean redirectSecurityIsEnabled() {
        String securityEnabled = propertyHelper.getProperty(KEY_ENABLE_REDIRECT_SECURITY);

        return "true".equalsIgnoreCase(securityEnabled);
    }


}
