package com.agilex.healthcare.mvisum.dataservice;

import com.agilex.healthcare.utility.StreamHelper;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Properties;

public class MVisumDataService {

    private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(MVisumDataService.class);
    private Properties mVisumProps;

    public String getMVisumUrl(String facilityCode) {
        if (System.getProperty(facilityCode) != null) {
            return System.getProperty(facilityCode);
        } else {
            //Load MVisum properties
            loadMVisumProperties();
            return mVisumProps.getProperty(facilityCode);
        }
    }

    public void loadMVisumProperties() {
        if (mVisumProps == null) {
            InputStreamReader inputStream = null;
            InputStream input = null;
            try {                
                String propertyFile = (System.getProperty("mvisumpropfile") != null) ?
                    System.getProperty("mvisumpropfile") :
                    "com/agilex/healthcare/mobilehealthplatform/restservice/mvisum.properties";

                inputStream = new InputStreamReader(
                    MVisumDataService.class.getClassLoader().getResourceAsStream(propertyFile));

                mVisumProps = new Properties();
                mVisumProps.load(inputStream);
                
            } catch (IOException e) {
                throw new RuntimeException("Could not load mVisum properties");
            
            } finally {
                if (inputStream != null) {
                    try {
                        inputStream.close();
                    } catch (IOException e) {
                        logger.error("InputStreamReader close on mvisumpropfile");
                    }
                }
                StreamHelper.safeClose(input);
            }
        }
    }
}
