package com.agilex.healthcare.mobilehealthplatform.clientapi;

import java.io.InputStream;
import java.net.URI;

import com.agilex.healthcare.mobilehealthplatform.domain.Allergies;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTestGroups;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientDemographics;
import com.agilex.healthcare.mobilehealthplatform.domain.Patients;
import com.agilex.healthcare.mobilehealthplatform.domain.Problems;
import com.agilex.healthcare.utility.NullChecker;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.ClientResponse.Status;
import com.sun.jersey.api.client.UniformInterfaceException;
import com.sun.jersey.api.client.WebResource;

public class PatientResourceClient {
	private static org.apache.commons.logging.Log log = org.apache.commons.logging.LogFactory.getLog(PatientResourceClient.class);
	private JerseyClientHandle jerseyClientHandle = null;

	public PatientResourceClient(JerseyClientHandle jerseyClientHandle) {
		log.debug("initializing patient resource client");
		this.jerseyClientHandle = jerseyClientHandle;
	}

	public Patients patientSearch(Patient prototype) {
		Patients patients = null;
		if (prototype != null) {
			try {
				com.sun.jersey.api.client.WebResource resourceHandle = jerseyClientHandle.resource().path("patients");
				resourceHandle = addPatientSearchQueryParams(resourceHandle, prototype);
				log.info("get uri " + resourceHandle.getURI().toString());
				patients = resourceHandle.get(Patients.class);
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
		} else {
			patients = new Patients();
		}

		return patients;
	}

	public Patients patientSearch(String searchString) {
		Patients patients = null;
		if (NullChecker.isNotNullish(searchString)) {
			try {
				com.sun.jersey.api.client.WebResource resourceHandle = jerseyClientHandle.resource().path("patients");
				resourceHandle = addPatientSearchQueryParam(resourceHandle, "searchstring", searchString);
				log.info("get uri " + resourceHandle.getURI().toString());
				patients = resourceHandle.get(Patients.class);
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
		} else {
			patients = new Patients();
		}

		return patients;
	}

	public Patient getPatient(URI uri) {
		Patient retvalue = null;
		try {
			com.sun.jersey.api.client.WebResource resourceHandle = jerseyClientHandle.resource(uri);
			log.info("get uri " + resourceHandle.getURI().toString());
			retvalue = resourceHandle.get(Patient.class);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
		return retvalue;
	}

	public Patient getPatient(Patient patient) {
		return getPatient(patient.getSelfUri());
	}

	public PatientDemographics getDemographics(URI uri) {
		PatientDemographics retvalue = null;
		com.sun.jersey.api.client.WebResource resourceHandle = jerseyClientHandle.resource(uri);
		log.info("get uri " + resourceHandle.getURI().toString());
		try {
			retvalue = resourceHandle.get(PatientDemographics.class);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
		return retvalue;
	}

	public PatientDemographics getDemographics(Patient patient) {
		return getDemographics(patient.getDemographicsUri());
	}

	private WebResource addPatientSearchQueryParams(WebResource resource, Patient prototype) {
		resource = addPatientSearchQueryParam(resource, "lastname", prototype.getLastName());
		resource = addPatientSearchQueryParam(resource, "firstname", prototype.getFirstName());
		resource = addPatientSearchQueryParam(resource, "patientid", prototype.getId());
		resource = addPatientSearchQueryParam(resource, "ssn", prototype.getSsn());
		return resource;
	}

	private WebResource addPatientSearchQueryParam(WebResource resource, String queryParamKey, String queryParamValue) {
		if (NullChecker.isNotNullish(queryParamValue)) {
			resource = resource.queryParam(queryParamKey, queryParamValue);
		}
		return resource;
	}

	public InputStream getPatientImage(Patient patient) {
		return getPatientImage(patient.getPatientImageUri());
	}

	public InputStream getPatientImage(URI uri) {
		InputStream image = null;
		try {
			com.sun.jersey.api.client.WebResource resourceHandle = jerseyClientHandle.resource(uri);
			log.info("get uri " + resourceHandle.getURI().toString());
			ClientResponse response = resourceHandle.get(ClientResponse.class);
			if (response.getClientResponseStatus() == Status.OK) {
				image = response.getEntityInputStream();
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
		return image;
	}

	public Problems getPatientActiveProblems(Patient patient) {
		return getPatientActiveProblems(patient.getProblemsUri());
	}

	public Problems getPatientActiveProblems(URI uri) {
		Problems retvalue = null;
		try {
			WebResource resourceHandle = jerseyClientHandle.resource(uri);
			log.info("get uri " + resourceHandle.getURI().toString());
			Problems problems = resourceHandle.get(Problems.class);
			return problems;
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
		return retvalue;
	}

	public Allergies getPatientAllergies(Patient patient) {
		return getPatientAllergies(patient.getAllergiesUri());
	}

	public Allergies getPatientAllergies(URI uri) {
		Allergies retvalue = null;
		try {
			WebResource resourceHandle = jerseyClientHandle.resource(uri);
			log.info("get uri " + resourceHandle.getURI().toString());
			retvalue = resourceHandle.get(Allergies.class);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
		return retvalue;
	}

	public LabTestGroups getPatientLabTestGroups(URI uri) {
		LabTestGroups retvalue = null;
		try {
			WebResource resourceHandle = jerseyClientHandle.resource(uri);
			log.info("get uri " + resourceHandle.getURI().toString());
			retvalue = resourceHandle.get(LabTestGroups.class);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
		return retvalue;
	}
}
