package com.agilex.healthcare.mobilehealthplatform.restservice;

import static org.junit.Assert.assertNotNull;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.w3c.dom.Document;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.UniformInterfaceException;

public class ConnectivityTest {
	private static MobileHealthClientTestVersion mobileHealthClient;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void setup() {
		mobileHealthClient = TestHelper.createMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void tearDown() {
		if (mobileHealthClient != null) {
			try {
				mobileHealthClient.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			mobileHealthClient.getJerseyClient().destroy();
			mobileHealthClient = null;
		}

	}

	@Test
	public void canAccessServiceViaJersey() {
		String uri = getPatientSearchUri();

		Client jerseyclient = mobileHealthClient.getJerseyClient();

		Document result = jerseyclient.resource(uri).get(Document.class);
		assertNotNull(result);
	}

	@Test
	public void canAccessServiceViaMhpClient() {
		String uri = getPatientSearchUri();
		Document result = mobileHealthClient.getAsXml(uri);
		assertNotNull(result);
	}

	private String getPatientSearchUri() {
		URI uri = UriBuilder.fromPath(IntegrationTestConfiguration.getMobileHealthServerBaseUri()).path("patients")
				.queryParam("searchstring", "younger").build();
		return uri.toString();
	}
}
