package com.agilex.healthcare.mobilehealthplatform.restservice;

import javax.ws.rs.core.MediaType;

import org.junit.Test;

import com.agilex.healthcare.testutility.TestHelper;
import com.agilex.healthcare.utility.StreamHelper;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.ClientResponse.Status;
import com.sun.jersey.api.client.WebResource;

public abstract class GeneralResourceTestBase {
	private boolean defaultDebug = true;
	private WebResource cachedResource = null;
	
	private Client jerseyClient;
	
	private final String GET = "GET";
	private final String HEAD = "HEAD";

	abstract protected String getResourcePath();

	protected WebResource getResourceHandle() {
		if (cachedResource == null) {
			String uri = getResourcePath();
			if (defaultDebug)
				System.out.println("initializing resource with URI " + uri);
			cachedResource = jerseyClient.resource(uri);
		}
		return cachedResource;
	}

	protected String getCanonicalResourceUri() {
		return getResourcePath();
	}

	private void runTest(String httpAction, String mediaType, boolean debug) {
		if (debug)
			System.out.println("running test for media type " + mediaType);


		jerseyClient = TestHelper.createMobileHealthClient().getJerseyClient();
		ClientResponse response = getResourceHandle().accept(mediaType).get(ClientResponse.class);
		if (debug) {
			System.out.println(response.toString());
			System.out.println((StreamHelper.streamToString(response.getEntityInputStream(), false)));
		}
		jerseyClient.destroy();
		jerseyClient = null;
		if (response.getClientResponseStatus() != Status.OK)
			throw new RuntimeException(response.toString());
	}

	private void runMediaTest(String httpAction, String mediaType) {
		this.defaultDebug = true;
		runTest(httpAction, mediaType, false);
	}

	@Test
	public void getAsXml() {
		runMediaTest(GET, MediaType.APPLICATION_XML);
	}

	@Test
	public void headAsXml() {
		runMediaTest(HEAD, MediaType.APPLICATION_XML);
	}

	@Test
	public void getAsJson() {
		runMediaTest(GET, MediaType.APPLICATION_JSON);
	}

	@Test
	public void headAsJson() {
		runMediaTest(HEAD, MediaType.APPLICATION_JSON);
	}

	@Test(expected = Exception.class)
	public void getUnknownMediaType() {
		runMediaTest(GET, "unknown");
	}

	@Test(expected = Exception.class)
	public void headUnknownMediaType() {
		runMediaTest(HEAD, "unknown");
	}
}
