package com.agilex.healthcare.mobilehealthplatform.restservice;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.agilex.healthcare.utility.XpathHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class MhpUserResourceTest {

	private static String mhpResourceUri = IntegrationTestConfiguration.getMobileHealthServerBaseUri() + "/mhpuser";
	//private Patient testPatient = IntegrationTestConfiguration.getTestPatient();

	private static MobileHealthClientTestVersion client;
    private static Document mhpUserXml = null;
    private static String deleteTokenURI = null;

	@BeforeClass
	public static void startUp() {
        client = TestHelper.createProviderMobileHealthClient();
        mhpUserXml = client.getAsXml(mhpResourceUri);

		deleteTokenURI = TestHelper.getDeleteTokenURI();
    }

	@AfterClass
	public static void destroy() {
	    if (client != null) {
	    	try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
	    	client = null;
	    }
	}

	@Test
	public void validatePatientContextLink() {
		Node patientLinkNode = XpathHelper.getNode(mhpUserXml, "//m:mhpUser/atom:link[@title='user-context']");
		assertNotNull(patientLinkNode);

        String patientContextUrl = patientLinkNode.getAttributes().getNamedItem("href").getNodeValue();
        assertEquals("http://localhost:8080/UserContext/rest/context/user/system/777/id/1", patientContextUrl);
	}

	@Test
	public void validatePortalFavoritesLink() {
		Node portalFavoritesNode = XpathHelper.getNode(mhpUserXml, "//m:mhpUser/atom:link[@title='portal-favorites']");
		assertNotNull(portalFavoritesNode);

        String portalFavoritesUri = portalFavoritesNode.getAttributes().getNamedItem("href").getNodeValue();
        assertEquals("http://localhost:4000/portal-service/favorites-service/site/777/id/1/items", portalFavoritesUri);
	}

	@Test
	public void validateLocationServiceResourceDirectoryLink() {
		Node patientLinkNode = XpathHelper.getNode(mhpUserXml, "//m:mhpUser/atom:link[@title='location-service-resource-directory']");
		assertNotNull(patientLinkNode);
	}

	@Test
	public void validatePatientSearchResourceDirectoryLink() {
		Node patientLinkNode = XpathHelper.getNode(mhpUserXml, "//m:mhpUser/atom:link[@title='patient-search-resource-directory']");
		assertNotNull(patientLinkNode);
	}

	@Test
	public void validateClinicalEventResourceDirectoryLink() {
		Node patientLinkNode = XpathHelper.getNode(mhpUserXml, "//m:mhpUser/atom:link[@title='clinical-event-resource-directory']");
		assertNotNull(patientLinkNode);
	}

	@Test
	public void validateSchedulingResourceDirectoryLink() {
		Node patientLinkNode = XpathHelper.getNode(mhpUserXml, "//m:mhpUser/atom:link[@title='scheduling-resource-directory']");
		assertNotNull(patientLinkNode);
	}
}