package com.agilex.healthcare.mobilehealthplatform.restservice.admission;


import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.LinkTitles;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNotes;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class DischargeSummaryResourceProdModeTest {
    private static MobileHealthClientTestVersion client;
    private static String START_DATE = "01/01/2002";
    private static String END_DATE = "01/01/2008";
    private static String deleteTokenURI = null;
    
    @BeforeClass
    public static void checkModeOfOperation(){
        client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
        deleteTokenURI = TestHelper.getDeleteTokenURI();
    }

    @AfterClass
    public static void destroy(){
        if (client != null) {
        	try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
            client = null;
        }
    }

    @Test
    public void testFetchPatientDischargeSummariesFromVista(){
        Patient patient = getTestPatient();
        URI dischargeSummariesUri = getDischargeSummarysUri(patient.getLink().getLinkByTitle(LinkTitles.patientDischargeSummaries).getHref().toString());
        ProviderNotes dischargeSummaries = client.getJerseyClient().resource(dischargeSummariesUri).type(MediaType.APPLICATION_XML).accept("application/xml").get(ProviderNotes.class);
        assertNotNull(dischargeSummaries);
        assertTrue(dischargeSummaries.size() > 0);
    }
    
    private Patient getTestPatient(){
        Patient searchPatient = new Patient();
        searchPatient.setSsn("666-00-0001");
        return client.getPatientResourceClient().patientSearch(searchPatient).get(0);
    }

    private URI getDischargeSummarysUri(String dischargeSummaryUri){
        StringBuilder sb = new StringBuilder(dischargeSummaryUri);
        sb.append("?startDate=");
        sb.append(START_DATE);
        sb.append("&endDate=");
        sb.append(END_DATE);
        return UriBuilder.fromUri(sb.toString()).build();
    }
}
