package com.agilex.healthcare.mobilehealthplatform.restservice.locationservice;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Clinic;
import com.agilex.healthcare.mobilehealthplatform.domain.Clinics;
import com.agilex.healthcare.mobilehealthplatform.domain.ResourceDirectory;
import com.agilex.healthcare.mobilehealthplatform.domain.Specialties;
import com.agilex.healthcare.mobilehealthplatform.domain.Specialty;
import com.agilex.healthcare.mobilehealthplatform.domain.Ward;
import com.agilex.healthcare.mobilehealthplatform.domain.Wards;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.agilex.healthcare.utility.NullChecker;
import com.sun.jersey.api.client.UniformInterfaceException;

public class LocationServiceResourceTest {
	private static final String SITE = "777";
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
    public static void checkModeOfOperation(){
        client = TestHelper.createProviderMobileHealthClient();
        deleteTokenURI = TestHelper.getDeleteTokenURI();
    }
	
	@AfterClass
	public static void tearDown() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}

//--------------------------------------------Clinics-------------------------------------------------------------------

	@Test
	public void fetchEmptyClinics() {
		try {
			URI uri = getClinicsUri("invalid");
			Clinics clinics = client.getJerseyClient().resource(uri).get(Clinics.class);
			assertEquals(0, clinics.size());
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void fetchClinics() {
		try {
			URI uri = getClinicsUri(SITE);
			Clinics clinics = client.getJerseyClient().resource(uri).get(Clinics.class);
			assertTrue(clinics.size() > 0);
			
			for (Clinic clinic : clinics) {
				assertNotNull(clinic.getId());
				assertNotNull(clinic.getFacilityCode());
				assertNotNull(clinic.getName());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchMdwsClinics() {
		try {
			URI uri = getMdwsClinicsUri(SITE);
			Clinics clinics = client.getJerseyClient().resource(uri).get(Clinics.class);
			assertTrue(clinics.size() > 0);
			
			for (Clinic clinic : clinics) {
				assertNotNull(clinic.getId());
				assertNotNull(clinic.getFacilityCode());
				assertNotNull(clinic.getName());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void fetchClinicsWithFilter() {
		try {
			String searchString = "card";
			URI uri = getClinicsUri(SITE, searchString);
			Clinics clinics = client.getJerseyClient().resource(uri).get(Clinics.class);
			assertTrue(clinics.size() > 0);
			
			for (Clinic clinic : clinics) {
				assertTrue(clinic.getName().toLowerCase().contains(searchString.toLowerCase()));
				assertNotNull(clinic.getId());
				assertNotNull(clinic.getFacilityCode());
				assertNotNull(clinic.getName());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void fetchMdwsClinicsWithSearchIndex() {
		try {
			URI uri = getClinicsUri(SITE, "card");
			URI uriNoSearchIndex = getMdwsClinicsUri("500", "");
			Clinics allClinics = client.getJerseyClient().resource(uriNoSearchIndex).get(Clinics.class);
			Clinics clinics = client.getJerseyClient().resource(uri).get(Clinics.class);
			assertNotNull(allClinics);
			assertNotNull(clinics);
			assertTrue(clinics.size() > 0);
			assertTrue(allClinics.size() > 0);
			assertTrue(allClinics.size() > clinics.size());
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	//---------utility methods---------

	private URI getClinicsUri(String facilityCode) {
		return getClinicsUri(facilityCode, null);
	}
	
	private URI getClinicsUri(String facilityCode, String searchString) {
		String uri = String.format("%s/location-service/site/%s/clinics", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode);
		
		if (NullChecker.isNotNullish(searchString)) {
			uri = uri.concat("?searchstring=" + searchString);	
		}
		
		URI clinicsUri = UriBuilder.fromUri(uri).build();
		return clinicsUri;
	}
	
	private URI getMdwsClinicsUri(String facilityCode) {
		return getMdwsClinicsUri(facilityCode, null);
	}
	
	private URI getMdwsClinicsUri(String facilityCode, String searchString) {
		String uri = String.format("%s/location-service/site/%s/mdws-clinics", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode);
		
		if (NullChecker.isNotNullish(searchString)) {
			uri = uri.concat("?searchstring=" + searchString);	
		}
		
		URI clinicsUri = UriBuilder.fromUri(uri).build();
		return clinicsUri;
	}
	
//--------------------------------------------End Clinics---------------------------------------------------------------

//--------------------------------------------Wards---------------------------------------------------------------------

	@Test
	public void fetchWards() {
		try {
			URI uri = getWardsUri(SITE);
			Wards wards = client.getJerseyClient().resource(uri).get(Wards.class);
			assertTrue(wards.size() > 0);

			for (Ward ward : wards) {
				assertNotNull(ward.getId());
				assertNotNull(ward.getFacilityCode());
				assertNotNull(ward.getName());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchWardsWithFilter() {
		try {
			String searchString = "NHC";
			URI uri = getWardsUri(SITE, searchString);
			Wards wards = client.getJerseyClient().resource(uri).get(Wards.class);
			assertTrue(wards.size() > 0);

			String previousName = "";
			for (Ward ward : wards) {
				assertTrue(ward.getName().toLowerCase().contains(searchString.toLowerCase()));
				assertNotNull(ward.getId());
				assertNotNull(ward.getFacilityCode());
				assertNotNull(ward.getName());
				assertTrue("sorted in ascending order", previousName.compareTo(ward.getName()) < 0);
				previousName = ward.getName();
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchMdwsWards() {
		try {
			URI uri = getMdwsWardsUri(SITE);
			Wards wards = client.getJerseyClient().resource(uri).get(Wards.class);
			assertTrue(wards.size() > 0);

			for (Ward ward : wards) {
				assertNotNull(ward.getId());
				assertNotNull(ward.getFacilityCode());
				assertNotNull(ward.getName());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	//---------utility methods---------
	
	private URI getWardsUri(String facilityCode) {
		URI wardsUri = getWardsUri(facilityCode, null);
		return wardsUri;		
	}
	
	private URI getWardsUri(String facilityCode, String searchString) {
		String uri = String.format("%s/location-service/site/%s/wards", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode);
		
		if (NullChecker.isNotNullish(searchString)) {
			uri = uri.concat("?searchstring=" + searchString);	
		}
		
		URI wardsUri = UriBuilder.fromUri(uri).build();
		return wardsUri;
	}
	
	private URI getMdwsWardsUri(String facilityCode) {
		String uri = String.format("%s/location-service/site/%s/mdws-wards", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode);
		
		URI wardsUri = UriBuilder.fromUri(uri).build();
		return wardsUri;
	}
	
//--------------------------------------------End Wards-----------------------------------------------------------------
	
//--------------------------------------------Resource Directory--------------------------------------------------------
	@Test
	public void fetchResourceDirectory() {
		try {
			URI uri = getResourceDirectoryUri(SITE);
			ResourceDirectory dir = client.getJerseyClient().resource(uri).get(ResourceDirectory.class);
			
			assertNotNull(dir);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	private URI getResourceDirectoryUri(String facilityCode) {
		String uri = String.format("%s/location-service/site/%s/resource-directory", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
//--------------------------------------------End Resource Directory----------------------------------------------------
	
//--------------------------------------------Specialties---------------------------------------------------------------
	
	@Test
	public void fetchSpecialties() {
		try {
			URI uri = getSpecialtiesUri(SITE);
			Specialties specialties = client.getJerseyClient().resource(uri).get(Specialties.class);
			assertNotNull(specialties);
			assertTrue(specialties.size() > 0);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchSpecialtiesBySystem() {
		try {
			URI uri = getSpecialtiesUri("640", "CDW");
			Specialties specialties = client.getJerseyClient().resource(uri).get(Specialties.class);
			assertNotNull(specialties);
			assertTrue(specialties.size() > 0);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchSpecialty() {
		try {
			URI uri = getSpecialtyUri(SITE, "CARDIOLOGY");
			Specialty specialty = client.getJerseyClient().resource(uri).get(Specialty.class);
			assertNotNull(specialty);
			assertEquals("CARDIOLOGY", specialty.getName());
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	private URI getSpecialtiesUri(String facilityCode) {
		URI uri = getSpecialtiesUri(facilityCode, null);
		return uri;
	}
	
	private URI getSpecialtiesUri(String facilityCode, String system) {
		String uri = String.format("%s/location-service/site/%s/specialties", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode);
		
		if (NullChecker.isNotNullish(system)) {
			uri += "/system/" + system;
		}
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getSpecialtyUri(String facilityCode, String name) {
		String uri = String.format("%s/location-service/site/%s/specialty?name=%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, name);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
//--------------------------------------------End Specialties-----------------------------------------------------------
}
