package com.agilex.healthcare.mobilehealthplatform.restservice.pathology;


import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.LinkTitles;
import com.agilex.healthcare.mobilehealthplatform.domain.PathologyReports;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.Patients;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientPathologyLabsResourceTest {

    private static MobileHealthClientTestVersion client;
    private static String START_DATE = "01/01/2002";
    private static String END_DATE = "01/01/2008";
    private static String deleteTokenURI = null;

    @BeforeClass
    public static void checkModeOfOperation(){
        client = TestHelper.createProviderMobileHealthClient();
        deleteTokenURI = TestHelper.getDeleteTokenURI();
    }

    @AfterClass
    public static void destroy(){
        if (client != null) {
        	try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
            client = null;
        }
    }

    @Test
    public void testFetchPatientPathologyReports(){
        try {
			Patient patient = getTestPatient();
			URI pathologyReportsSearchUri = getPathologyReportsSearchUri(patient.getLink().getLinkByTitle(LinkTitles.patientPathologyReports).getHref().toString());
			PathologyReports pathologyReports = client.getJerseyClient().resource(pathologyReportsSearchUri).type(MediaType.APPLICATION_XML).accept("application/xml").get(PathologyReports.class);
			assertNotNull(pathologyReports);
			assertTrue(pathologyReports.size() > 0);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }
    
    private Patient getTestPatient(){
        Patient searchPatient = new Patient();
        searchPatient.setLastName("Eleven");
        searchPatient.setFirstName("Patient");
        Patients patients = client.getPatientResourceClient().patientSearch(searchPatient);
        return patients.get(0);
    }

    private URI getPathologyReportsSearchUri(String pathologyReportsSearchUri){
        StringBuilder sb = new StringBuilder(pathologyReportsSearchUri);
        sb.append("?startDate=");
        sb.append(START_DATE);
        sb.append("&endDate=");
        sb.append(END_DATE);
        return UriBuilder.fromUri(sb.toString()).build();
    }
}
