package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Admission;
import com.agilex.healthcare.mobilehealthplatform.domain.Admissions;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNote;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNotes;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientAdmissionsResourceTest {
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;
	
	@BeforeClass
	public static void validateMode(){
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}
	
	@AfterClass
	public static void destroy(){
		if (client!=null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client=null;
		}
	}
	
	@Test
	public void testFetchAdmissionsByScope(){
		try {
			String assigningAuthority = "dfn-777";
			String patientId = "8";
			String scope = "operational";
			URI admissionsURI = getAdmissionsByScopeUri(assigningAuthority, patientId, scope);
			Admissions admissions = client.getJerseyClient().resource(admissionsURI).get(Admissions.class);
			Assert.assertNotNull(admissions);
			Assert.assertEquals(10, admissions.getSize());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}

	@Test
	public void testFetchAdmissionsBySystem(){
		try {
			String assigningAuthority = "dfn-777";
			String patientId = "8";
			String system = "vista";
			URI admissionsURI = getAdmissionsBySystemUri(assigningAuthority, patientId, system);
			Admissions admissions = client.getJerseyClient().resource(admissionsURI).get(Admissions.class);
			Assert.assertNotNull(admissions);
			Assert.assertEquals(10, admissions.getSize());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}
	
	//Not sure, how to test it
	@Ignore
	@Test
	public void testFetchAdmission(){
		try {
			String assigningAuthority = "dfn-777";
			String patientId = "8";
			String system = "vista";
			String admissionId = "3062538e-5eca-4c14-b2b1-f4c045758c03";
			URI admissionURI = getAdmissionUri(assigningAuthority, patientId, system, admissionId);
			Admission admission = client.getJerseyClient().resource(admissionURI).get(Admission.class);
			Assert.assertNotNull(admission);
			Assert.assertEquals(1, admission.getInpatientSID());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}

	//Not sure, how to test it
	@Ignore
	@Test
	public void testFetchDischargeNote(){
		try {
			String assigningAuthority = "dfn-777";
			String patientId = "8";
			String system = "vista";
			String dischargeNoteId = "21";
			URI dischargeNoteURI = getDischargeNoteUri(assigningAuthority, patientId, system, dischargeNoteId);
			ProviderNote providerNote = client.getJerseyClient().resource(dischargeNoteURI).get(ProviderNote.class);
			Assert.assertNotNull(providerNote);
			Assert.assertEquals(1, providerNote.getId());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}

	@Test
	public void testFetchPatientDischargeSummaries(){
		try {
			String assigningAuthority = "dfn-777";
			String patientId = "8";
			String system = "vista";
			URI dischargeSummariesURI = getDischargeNotesUri(assigningAuthority, patientId, system);
			ProviderNotes providerNotes = client.getJerseyClient().resource(dischargeSummariesURI).get(ProviderNotes.class);
			Assert.assertNotNull(providerNotes);
			Assert.assertEquals(2, providerNotes.getSize());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}
	
	private URI getAdmissionsByScopeUri(String assigningAuthority, String patientId, String scope){
		String uriFormat = String.format("%s/patient/%s/%s/admissions/scope/%s", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				scope);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;		
	}
	
	private URI getAdmissionsBySystemUri(String assigningAuthority, String patientId, String system){
		String uriFormat = String.format("%s/patient/%s/%s/admissions/system/%s", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
		
	}

	private URI getAdmissionUri(String assigningAuthority, String patientId, String system, String admissionId){
		String uriFormat = String.format("%s/patient/%s/%s/admissions/system/%s/id/%s", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system,
				admissionId);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}

	private URI getDischargeNoteUri(String assigningAuthority, String patientId, String system, String dischargeNoteId){
		String uriFormat = String.format("%s/patient/%s/%s/admissions/system/%s/id/%s/note", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system,
				dischargeNoteId);
		
		StringBuilder sb = new StringBuilder(uriFormat);
        sb.append("?startDate=");
        sb.append("01/01/2002");
        sb.append("&endDate=");
        sb.append("01/01/2008");	
		
		URI uri = UriBuilder.fromUri(sb.toString()).build();
		return uri;
		
	}

	private URI getDischargeNotesUri(String assigningAuthority, String patientId, String siteId) {
		String uriFormat = String.format("%s/patient/%s/%s/admissions/site/%s/discharge-summaries", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				siteId);
		
		StringBuilder sb = new StringBuilder(uriFormat);
        sb.append("?startDate=");
        sb.append("01/01/2002");
        sb.append("&endDate=");
        sb.append("01/01/2008");	
		
		URI uri = UriBuilder.fromUri(sb.toString()).build();
		return uri;
		
	}
	
}