package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Allergies;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientAllergiesResourceTest {
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;
	
	@BeforeClass
	public static void validateMode(){
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}
	
	@AfterClass
	public static void destroy(){
		if (client!=null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client=null;
		}
	}

	@Test
	public void testFetchAllergiesByScope(){
		try {
			String assigningAuthority = "dfn-777";
			String patientId = "8";
			String scope = "operational";
			URI allergiesURI = UriBuilder.fromUri(getAllergiesByScopeUri(assigningAuthority, patientId, scope)).build();
			Allergies allergies = client.getJerseyClient().resource(allergiesURI).get(Allergies.class);
			Assert.assertNotNull(allergies);
			Assert.assertEquals(3, allergies.getSize());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}

	@Test
	public void testFetchAllergiesBySystem(){
		try {
			String assigningAuthority = "dfn-777";
			String patientId = "8";
			String system = "vista";
			URI allergiesURI = UriBuilder.fromUri(getAllergiesBySystemUri(assigningAuthority, patientId, system)).build();
			Allergies allergies = client.getJerseyClient().resource(allergiesURI).get(Allergies.class);
			Assert.assertNotNull(allergies);
			Assert.assertEquals(3, allergies.getSize());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}
	
	public String getAllergiesByScopeUri(String assigningAuthority, String patientId, String scope){
		return String.format("%s/patient/%s/%s/allergies/scope/%s", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				scope);
	}
	
	public String getAllergiesBySystemUri(String assigningAuthority, String patientId, String system){
		return String.format("%s/patient/%s/%s/allergies/system/%s", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system);
	}
}
