package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointment;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointments;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNote;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientAppointmentsResourceTest {
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;
	
	@BeforeClass
	public static void validateMode(){
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}
	
	@AfterClass
	public static void destroy(){
		if (client!=null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client=null;
		}
	}
	
	
	@Test
	public void testFetchAppointmentsByScope(){
		try {
			String assigningAuthority = "dfn-688";
			String patientId = "100844";
			String scope = "operational";
			URI appointmentsURI = UriBuilder.fromUri(getAppointmentsByScopeUri(assigningAuthority, patientId, scope)).build();
			Appointments appointments = client.getJerseyClient().resource(appointmentsURI).get(Appointments.class);
			Assert.assertNotNull(appointments);
			Assert.assertEquals(5, appointments.getSize());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}

	
	@Test
	public void testFetchAppointmentsBySystem(){
		try {
			String assigningAuthority = "dfn-688";
			String patientId = "100844";
			String system = "hdr";
			URI appointmentsURI = UriBuilder.fromUri(getAppointmentsBySystemUri(assigningAuthority, patientId, system)).build();
			Appointments appointments = client.getJerseyClient().resource(appointmentsURI).get(Appointments.class);
			Assert.assertNotNull(appointments);
			Assert.assertEquals(5, appointments.getSize());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}

	//Not sure, how to test
	@Ignore
	@Test
	public void testFetchClinicNoteForAppointment(){
		try {
			String assigningAuthority = "dfn-688";
			String patientId = "100844";
			String system = "vista";
			String appmntId = "1";
			String visitId = "2";
			URI clinicNoteForAppointmenttUr = UriBuilder.fromUri(getClinicNoteForAppointmenttUri(assigningAuthority, patientId, system, appmntId, visitId)).build();
			ProviderNote providerNote = client.getJerseyClient().resource(clinicNoteForAppointmenttUr).get(ProviderNote.class);
			Assert.assertNotNull(providerNote);
			//Assert.assertEquals(1, providerNote.getUniqueId());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}

	//Not sure, how to test
	@Ignore
	@Test
	public void testGetMentalHealthAppointments(){
		try {
			String assigningAuthority = "dfn-688";
			String patientId = "100844";
			String scope = "operational";
			URI mentalHealthAppointmentsURI = UriBuilder.fromUri(getMentalHealthAppointmentsByScopeUri(assigningAuthority, patientId, scope)).build();
			Appointments appointments = client.getJerseyClient().resource(mentalHealthAppointmentsURI).get(Appointments.class);
			Assert.assertNotNull(appointments);
			Assert.assertEquals(2, appointments.getSize());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}

	
	@Test
	public void testGetFutureAppointments(){
		try {
			String assigningAuthority = "dfn-688";
			String patientId = "100844";
			URI futureAppointmentsURI = UriBuilder.fromUri(getFutureAppointmentsURI(assigningAuthority, patientId)).build();
			Appointments appointments = client.getJerseyClient().resource(futureAppointmentsURI).get(Appointments.class);
			Assert.assertNotNull(appointments);
			Assert.assertEquals(5, appointments.getSize());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}
	
	
	//Not sure how to test
	@Ignore
	@Test
	public void testFetchAppointment(){
		try {
			String assigningAuthority = "dfn-688";
			String patientId = "100844";
			String system = "hdr";
			URI appointmentURI = UriBuilder.fromUri(getAppointmentUri(assigningAuthority, patientId, system)).build();
			Appointment appointment = client.getJerseyClient().resource(appointmentURI).get(Appointment.class);
			Assert.assertNotNull(appointment);
			Assert.assertEquals(1, appointment.getUniqueId());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}
	
	private String getAppointmentsByScopeUri(String assigningAuthority, String patientId, String scope){
		String uriFormat = String.format("%s/patient/%s/%s/appointments/scope/%s", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				scope);
		
//		StringBuilder sb = new StringBuilder(uriFormat);
//        sb.append("?startDate=");
//        sb.append("01/01/2002");
//        sb.append("&endDate=");
//        sb.append("01/01/2008");	

		//sreturn sb.toString();
		return uriFormat;
		
	}
	
	public String getAppointmentsBySystemUri(String assigningAuthority, String patientId, String system){
		return String.format("%s/patient/%s/%s/appointments/system/%s", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system);
	}

	public String getClinicNoteForAppointmenttUri(String assigningAuthority, String patientId, String system, String appmntId, String visitId){
		return String.format("%s/patient/%s/%s/appointments/system/%s/id/%s/visit/%s/note", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system,
				appmntId,
				visitId);
	}
	
	public String getMentalHealthAppointmentsByScopeUri(String assigningAuthority, String patientId, String scope){
		return String.format("%s/patient/%s/%s/appointments/mental-health/scope/%s", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				scope);
	}
	
	public String getFutureAppointmentsURI(String assigningAuthority, String patientId){
		String uriFormat = String.format("%s/patient/%s/%s/appointments/future", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId);
		
		StringBuilder sb = new StringBuilder(uriFormat);
		sb.append("?startDate=");
		sb.append("01/01/2010");
		sb.append("&endDate=");
		sb.append("01/01/2017");	
		return sb.toString();
		
	}
	
	public String getAppointmentUri(String assigningAuthority, String patientId, String system){
		return String.format("%s/patient/%s/%s/appointments/system/%s/id/%s", 
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system,
				"1");
	}

}
