package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Consults;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientConsultsResourceTest {

	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;
	
	@BeforeClass
	public static void startup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
        deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
    public void testFetchPatientConsultsOnOperationScope() {
        try {
			final String assigningAuthority = "dfn-777";
			final String scope ="operational";
			final String patientId = "237";
			URI uri = createPatientConsultsUri(assigningAuthority,scope,patientId);
			Consults consults = client.getJerseyClient().resource(uri).get(Consults.class);
			assertNotNull(consults);
			assertTrue(consults.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }
	
    private URI createPatientConsultsUri(String assigningAuthority, String scope, String patientId) {
		String uriFormat = String.format("%s/patient/%s/%s/consults/scope/%s",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										scope);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}

}
