package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResult;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTest;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTests;
import com.agilex.healthcare.mobilehealthplatform.domain.LinkTitles;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;
import com.sun.jersey.api.client.UniformInterfaceException;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import javax.ws.rs.core.UriBuilder;
import java.net.URI;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import static junit.framework.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

public class PatientLabTestsResourceTest {

	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void startup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}

	@Test
	public void testGetLabTestsTest() {
		try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteCode = "777";
			URI uri = getLabTestsUri(assigningAuthority, patientId, siteCode, null);
			LabTests labTests = client.getJerseyClient().resource(uri.toString()).get(LabTests.class);
			assertNotNull(labTests);
			assertTrue(labTests.size() > 0);

			for (LabTest labTest: labTests) {
				assertNotNull(labTest.getIen());
				assertNotNull(labTest.getTestname());
				assertNotNull(labTest.getAccessionAreaIen());
				assertNotNull(labTest.getAccessionAreaName());
				assertNotNull(labTest.getAccessionSubscript());
				assertNotNull(labTest.getSiteCode());
				assertNotNull(labTest.getMostCurrentResultDate());
				assertNotNull(labTest.getLink().getLinkByTitle(LinkTitles.PatientChemHemLabResultsForTest));
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void testGetLabTestsWithLabCategoryFilterTest() {
		try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteCode = "777";
			List<String> labCategoryList = new LinkedList<String>();
			labCategoryList.add("HE");

			URI uri = getLabTestsUri(assigningAuthority, patientId, siteCode, labCategoryList);
			LabTests labTests = client.getJerseyClient().resource(uri.toString()).get(LabTests.class);
			assertNotNull(labTests);
			assertTrue(labTests.size() > 0);

			for (LabTest labTest: labTests) {
				assertNotNull(labTest.getIen());
				assertNotNull(labTest.getTestname());
				assertNotNull(labTest.getAccessionAreaIen());
				assertNotNull(labTest.getAccessionAreaName());
				assertNotNull(labTest.getAccessionSubscript());
				assertEquals(labTest.getAccessionSubscript(), "HE");
				assertNotNull(labTest.getSiteCode());
				assertNotNull(labTest.getMostCurrentResultDate());
				assertNotNull(labTest.getLink().getLinkByTitle(LinkTitles.PatientChemHemLabResultsForTest));
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void testGetLabResultsTest() {
		try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteCode = "777";
			final String testId = "1";
			final String testName = "WBC";
			URI uri = getLabResultsUri(assigningAuthority, patientId, testId, testName, siteCode, null, null);
			LabTest labTest = client.getJerseyClient().resource(uri.toString()).get(LabTest.class);
			assertNotNull(labTest.getLabResults());
			assertTrue(labTest.getLabResults().size() > 0);
			assertNotNull(labTest.getLink().getLinkByTitle(LinkTitles.PatientChemHemLabResultsForTest));


			assertNotNull(labTest.getTestname());
			for (LabResult labResult : labTest.getLabResults()) {
				assertNotNull(labResult.getValue());
				assertNotNull(labResult.getFlag());
				assertNotNull(labResult.getSpecimenId());
				assertNotNull(labResult.getSpecimenName());
				assertNotNull(labResult.getTestname());

				//assertNotNull(labResult.getReferenceHigh());
				//assertNotNull(labResult.getReferenceLow());
				//assertNotNull(labResult.getValueUnits());
				assertNotNull(labResult.getResultedDate());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void testGetLabResultsWithDateFilterTest() {
		try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteCode = "777";
			final String testId = "1";
			final String testName = "WBC";
			final String startDateString = "01/01/2005";
			final String endDateString = "09/01/2008";
			final Date startDate = DateHelper.parseDate(startDateString);
			final Date endDate = DateHelper.parseDate(endDateString);
			URI uri = getLabResultsUri(assigningAuthority, patientId, testId, testName, siteCode, startDateString, endDateString);
			LabTest labTest = client.getJerseyClient().resource(uri.toString()).get(LabTest.class);
			assertNotNull(labTest.getLabResults());
			assertTrue(labTest.getLabResults().size() > 0);
			assertNotNull(labTest.getLink().getLinkByTitle(LinkTitles.PatientChemHemLabResultsForTest));

			assertNotNull(labTest.getTestname());
			for (LabResult labResult : labTest.getLabResults()) {
				assertNotNull(labResult.getValue());
				assertNotNull(labResult.getFlag());
				assertNotNull(labResult.getSpecimenId());
				assertNotNull(labResult.getSpecimenName());
				//assertNotNull(labResult.getReferenceHigh());
				//assertNotNull(labResult.getReferenceLow());
				//assertNotNull(labResult.getValueUnits());

				assertNotNull(labResult.getResultedDate());
				assertTrue(labResult.getResultedDate().getTime() >= startDate.getTime());
				assertTrue(labResult.getResultedDate().getTime() <= endDate.getTime());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	public URI getLabTestsUri(String assigningAuthority, String patientId, String siteCode, List<String> labCategoryList) {
		String uriFormat = String.format("%s/patient/%s/%s/lab-tests/site/%s",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				siteCode);

		if (labCategoryList != null && !labCategoryList.isEmpty()) {
			boolean firstArg = true;
			for (String labCategory : labCategoryList) {
				if (firstArg) {
					uriFormat += "?";
					firstArg = false;
				} else {
					uriFormat += "&";
				}
				uriFormat += "lab-category=" + labCategory;
			}
		}

		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}

	private URI getLabResultsUri(String assigningAuthority, String patientId, String testId, String testName, String siteCode, String startDate, String endDate) {
		String uriFormat = String.format("%s/patient/%s/%s/lab-tests/id/%s/test-name/%s/site/%s",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				testId,
				testName,
				siteCode);

		boolean firstArg = true;
		if (NullChecker.isNotNullish(startDate)) {
			uriFormat += "?";
			firstArg = false;

			uriFormat += "startDate=" + startDate;
		}
		if (NullChecker.isNotNullish(endDate)) {
			if (firstArg) {
				uriFormat += "?";
				firstArg = false;
			} else {
				uriFormat += "&";
			}

			uriFormat += "endDate=" + endDate;
		}

		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}

}
