package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertEquals;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Medication;
import com.agilex.healthcare.mobilehealthplatform.domain.Medications;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientMedicationsResourceTest {
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;
	
	@BeforeClass
	public static void startup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
        deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
    public void testFetchMedicationsByScope() {
        try {
			final String assigningAuthority = "dfn-777";
			final String scope ="operational";
			final String patientId = "8";
			URI uri = getMedicationsByScopeUri(assigningAuthority,scope,patientId);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }

	@Test
    public void testFetchMedicationsBySystem() {
        try {
        	final String assigningAuthority = "dfn-777";
        	final String patientId = "8";
        	final String system = "vista";
			URI uri = getMedicationsBySystemUri(assigningAuthority, patientId, system);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }

	@Test
    public void testFetchActiveMedicationsByScope() {
        try {
			final String assigningAuthority = "dfn-777";
			final String scope ="operational";
			final String patientId = "8";
			URI uri = getActiveMedicationsByScopeUri(assigningAuthority,scope,patientId);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }

	@Test
    public void testFetchActiveMedicationsBySystem() {
        try {
        	final String assigningAuthority = "dfn-777";
        	final String patientId = "8";
        	final String system = "vista";
			URI uri = getActiveMedicationsBySystemUri(assigningAuthority, patientId, system);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }
	
	//Not sure, how to test it
	@Ignore
	@Test
    public void testFetchOutpatientMedications() {
        try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			URI uri = getOutpatientMedicationsUri(assigningAuthority, patientId);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }

	//Not sure, how to test it
	@Ignore
	@Test
    public void testFetchCurrentOutpatientMedications() {
        try {
        	final String assigningAuthority = "dfn-777";
        	final String patientId = "8";
			URI uri = getCurrentOutpatientMedicationsUri(assigningAuthority, patientId);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }

	@Test
    public void testFetchOutpatientMedicationsByScope() {
        try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String scope ="operational";
			URI uri = getOutpatientMedicationsByScopeUri(assigningAuthority, patientId, scope);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }
	
	@Test
    public void testFetchCurrentOutpatientMedicationsByScope() {
        try {
        	final String assigningAuthority = "dfn-777";
        	final String patientId = "8";
        	final String scope ="operational";
			URI uri = getCurrentOutpatientMedicationsByScopeUri(assigningAuthority, patientId, scope);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }

	@Test
    public void testFetchOutpatientMedicationsBySystem() {
        try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String system ="vista";
			URI uri = getOutpatientMedicationsBySystemUri(assigningAuthority, patientId, system);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }
	
	@Test
    public void testFetchCurrentOutpatientMedicationsBySystem() {
        try {
        	final String assigningAuthority = "dfn-777";
        	final String patientId = "8";
        	final String system ="vista";
			URI uri = getCurrentOutpatientMedicationsBySystemUri(assigningAuthority, patientId, system);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }
	
	//Not sure how to test them -- error - Could not find stored procedure &apos;App.MedicationsList&apos;.
	@Ignore
	@Test
    public void testFetchInpatientMedications() {
        try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "237";
			URI uri = getInpatientMedicationsUri(assigningAuthority, patientId);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }

	
	@Ignore
	@Test
    public void testFetchCurrentInpatientMedications() {
        try {
        	final String assigningAuthority = "dfn-777";
        	final String patientId = "237";
			URI uri = getCurrentInpatientMedicationsUri(assigningAuthority, patientId);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }
	
	@Ignore
	@Test
    public void testFetchInpatientMedicationsByScope() {
        try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "237";
			final String scope ="operational";
			URI uri = getInpatientMedicationsByScopeUri(assigningAuthority, patientId, scope);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }
	
	@Ignore
	@Test
    public void testFetchCurrentInpatientMedicationsByScope() {
        try {
        	final String assigningAuthority = "dfn-777";
        	final String patientId = "237";
        	final String scope ="operational";
			URI uri = getCurrentInpatientMedicationsByScopeUri(assigningAuthority, patientId, scope);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }

	@Ignore
	@Test
    public void testFetchInpatientMedicationsBySystem() {
        try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "237";
			final String system ="vista";
			URI uri = getInpatientMedicationsBySystemUri(assigningAuthority, patientId, system);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }
	
	@Ignore
	@Test
    public void testFetchCurrentInpatientMedicationsBySystem() {
        try {
        	final String assigningAuthority = "dfn-777";
        	final String patientId = "237";
        	final String system ="vista";
			URI uri = getCurrentInpatientMedicationsBySystemUri(assigningAuthority, patientId, system);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medications medications = client.getJerseyClient().resource(uri).get(Medications.class);
			assertNotNull(medications);
			assertTrue(medications.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }
	
	//@Ignore
	@Test
    public void testGetMedication() {
        try {
        	final String assigningAuthority = "dfn-777";
        	final String patientId = "8";
        	final String system = "vista";
        	final String medicationId = "eb23a21049b20452448bc2527a5c301129e0d25f";
			URI uri = getMedicationUri(assigningAuthority, patientId, system, medicationId);
			uri = UriBuilder.fromUri(uri.toString()).build();
			Medication medication = client.getJerseyClient().resource(uri).get(Medication.class);
			assertNotNull(medication);
			assertEquals(medicationId, medication.getId());
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }

    private URI getMedicationUri(String assigningAuthority, String patientId, String system, String medicationId) {
		String uriFormat = String.format("%s/patient/%s/%s/medications/system/%s/id/%s",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										system,
										medicationId);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
    private URI getCurrentInpatientMedicationsBySystemUri(String assigningAuthority, String patientId, String system) {
		String uriFormat = String.format("%s/patient/%s/%s/medications/inpatient/system/%s/active",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										system);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
    private URI getInpatientMedicationsBySystemUri(String assigningAuthority, String patientId, String system) {

		String uriFormat = String.format("%s/patient/%s/%s/medications/inpatient/system/%s",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
		
	}
	
    private URI getCurrentInpatientMedicationsByScopeUri(String assigningAuthority, String patientId, String scope) {
		String uriFormat = String.format("%s/patient/%s/%s/medications/inpatient/scope/%s/active",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										scope);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
    private URI getInpatientMedicationsByScopeUri(String assigningAuthority, String patientId, String scope) {

		String uriFormat = String.format("%s/patient/%s/%s/medications/inpatient/scope/%s",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				scope);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
		
	}
	
    private URI getInpatientMedicationsUri(String assigningAuthority, String patientId) {

		String uriFormat = String.format("%s/patient/%s/%s/medications/inpatient",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
		
	}
	
    private URI getCurrentInpatientMedicationsUri(String assigningAuthority, String patientId) {
		String uriFormat = String.format("%s/patient/%s/%s/medications/inpatient/active",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
	
    private URI getCurrentOutpatientMedicationsBySystemUri(String assigningAuthority, String patientId, String system) {
		String uriFormat = String.format("%s/patient/%s/%s/medications/outpatient/system/%s/active",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										system);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
    private URI getOutpatientMedicationsBySystemUri(String assigningAuthority, String patientId, String system) {

		String uriFormat = String.format("%s/patient/%s/%s/medications/outpatient/system/%s",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
		
	}
	
    private URI getCurrentOutpatientMedicationsByScopeUri(String assigningAuthority, String patientId, String scope) {
		String uriFormat = String.format("%s/patient/%s/%s/medications/outpatient/scope/%s/active",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										scope);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
    private URI getOutpatientMedicationsByScopeUri(String assigningAuthority, String patientId, String scope) {

		String uriFormat = String.format("%s/patient/%s/%s/medications/outpatient/scope/%s",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				scope);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
		
	}
	
    private URI getOutpatientMedicationsUri(String assigningAuthority, String patientId) {

		String uriFormat = String.format("%s/patient/%s/%s/medications/outpatient",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId);
		
		StringBuilder sb = new StringBuilder(uriFormat);
        sb.append("?startDate=");
        sb.append("01/01/2002");
        sb.append("&endDate=");
        sb.append("01/01/2014");	
		URI uri = UriBuilder.fromUri(sb.toString()).build();
		return uri;
		
	}
	
    private URI getCurrentOutpatientMedicationsUri(String assigningAuthority, String patientId) {
		String uriFormat = String.format("%s/patient/%s/%s/medications/outpatient/active",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
    private URI getActiveMedicationsBySystemUri(String assigningAuthority, String patientId, String system) {

		String uriFormat = String.format("%s/patient/%s/%s/medications/system/%s/active",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
		
	}
	
    private URI getActiveMedicationsByScopeUri(String assigningAuthority, String scope, String patientId) {
		String uriFormat = String.format("%s/patient/%s/%s/medications/scope/%s/active",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										scope);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
    private URI getMedicationsBySystemUri(String assigningAuthority, String patientId, String system) {

		String uriFormat = String.format("%s/patient/%s/%s/medications/system/%s",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
		
	}
	
    private URI getMedicationsByScopeUri(String assigningAuthority, String scope, String patientId) {
		String uriFormat = String.format("%s/patient/%s/%s/medications/scope/%s",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										scope);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
    
}
