package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResult;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResults;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTestGroups;
import com.agilex.healthcare.mobilehealthplatform.domain.LabTests;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientMicrobiologyLabsResourceTest {
	private static final String OPERATIONAL = "operational";
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void setup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void fetchResultsByScope() {
		try {
			URI uri = getFetchResultsByScopeUri("dfn-777", "8", OPERATIONAL);
			LabResults labResults = client.getJerseyClient().resource(uri).get(LabResults.class);
			assertNotNull(labResults);
			assertTrue(labResults.size() > 0);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getLabResultById() {
		try {
			URI uri = getFetchResultsByScopeUri("dfn-777", "8", OPERATIONAL);
			LabResults labResults = client.getJerseyClient().resource(uri).get(LabResults.class);
			assertNotNull(labResults);
			assertTrue(labResults.size() > 0);
			
			LabResult expected = labResults.get(0);
			uri = getLabResultByIdUri("dfn-777", "8", "vista", expected.getUniqueId());
			LabResult actual = client.getJerseyClient().resource(uri).get(LabResult.class);
			assertNotNull(actual);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void fetchResultsBySystem(){
		try {
			String assigningAuthority = "dfn-777";
			String patientId = "8";
			String system = "vista";
			URI labResultsURI = UriBuilder.fromUri(getLabResultsBySystemUri(assigningAuthority, patientId, system)).build();
			LabResults labResults = client.getJerseyClient().resource(labResultsURI).get(LabResults.class);
			Assert.assertNotNull(labResults);
			Assert.assertTrue(labResults.getSize() > 0);
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}
	
	//Not sure, how to test it
	@Ignore
	@Test
    public void getLabTestsByScope() {
        try {
			final String assigningAuthority = "dfn-777";
			final String scope ="operational";
			final String patientId = "8";
			URI uri = getLabTestsByScopeUri(assigningAuthority,scope,patientId);
			uri = UriBuilder.fromUri(uri.toString()).build();
			LabTests labTests = client.getJerseyClient().resource(uri).get(LabTests.class);
			assertNotNull(labTests);
			assertTrue(labTests.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }

	@Ignore
	@Test
	public void getLabTestsBySystem(){
		try {
			String assigningAuthority = "dfn-777";
			String patientId = "8";
			String system = "vista";
			URI labTestsURI = UriBuilder.fromUri(getLabTestsBySystemUri(assigningAuthority,patientId, system)).build();
			LabTests labTests = client.getJerseyClient().resource(labTestsURI).get(LabTests.class);
			Assert.assertNotNull(labTests);
			Assert.assertTrue(labTests.getSize() > 0);
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}

	@Test
    public void getLabTestsInGroupsByScope() {
        try {
			final String assigningAuthority = "dfn-777";
			final String scope ="operational";
			final String patientId = "8";
			URI uri = getLabTestsInGroupsByScopeUri(assigningAuthority,scope,patientId);
			uri = UriBuilder.fromUri(uri.toString()).build();
			LabTestGroups labTestGroups = client.getJerseyClient().resource(uri).get(LabTestGroups.class);
			assertNotNull(labTestGroups);
			assertTrue(labTestGroups.size() > 0);
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }

	@Test
	public void getLabTestsInGroupsBySystem(){
		try {
			String assigningAuthority = "dfn-777";
			String patientId = "8";
			String system = "vista";
			URI labTestsURI = UriBuilder.fromUri(getLabTestsInGroupsBySystemUri(assigningAuthority,patientId, system)).build();
			LabTestGroups labTestGroups = client.getJerseyClient().resource(labTestsURI).get(LabTestGroups.class);
			Assert.assertNotNull(labTestGroups);
			Assert.assertTrue(labTestGroups.getSize() > 0);
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}

//----------------------------------------------------------------------------------------------------------------------

	private URI getFetchResultsByScopeUri(String assigningAuthority, String patientId, String scope) {
		String uri = String.format("%s/patient/%s/%s/lab-microbiology/results/scope/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, scope);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getLabResultByIdUri(String assigningAuthority, String patientId, String system, String id) {
		String uri = String.format("%s/patient/%s/%s/lab-microbiology/results/system/%s/id/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, system, id);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
    public String getLabTestsInGroupsBySystemUri(String assigningAuthority, String patientId, String system){
		return String.format("%s/patient/%s/%s/lab-microbiology/groups/system/%s",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system);
	}
	
    private URI getLabTestsInGroupsByScopeUri(String assigningAuthority, String scope, String patientId) {
		String uriFormat = String.format("%s/patient/%s/%s/lab-microbiology/groups/scope/%s",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										scope);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
	public String getLabTestsBySystemUri(String assigningAuthority, String patientId, String system){
		return String.format("%s/patient/%s/%s/lab-microbiology/tests/system/%s",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system);
	}
	
    private URI getLabTestsByScopeUri(String assigningAuthority, String scope, String patientId) {
		String uriFormat = String.format("%s/patient/%s/%s/lab-microbiology/tests/scope/%s",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										scope);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
	public String getLabResultsBySystemAndTestUri(String assigningAuthority, String patientId, String system, String loinc_code) {
		return String.format("%s/patient/%s/%s/lab/results/system/%s/loinc/%s",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system,
				loinc_code);
	}
    
	public String getLabResultsBySystemUri(String assigningAuthority, String patientId, String system) {
		return String.format("%s/patient/%s/%s/lab-microbiology/results/system/%s",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				system);
	}
}
