package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderResultsHistory;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientOrderResultsHistoryResourceTest {
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;
	
	private final String assigningAuthority = "dfn-777";
	private final String patientId = "1";
	private final String siteId ="777";
	private final String orderIen = "10894;1";
		
	@BeforeClass
	public static void startup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void fetchPatientOrderResultHistory(){
		try {
			URI uri = getUri(assigningAuthority, patientId, siteId, orderIen);
			OrderResultsHistory history = client.getJerseyClient().resource(uri).get(OrderResultsHistory.class);
			Assert.assertNotNull(history);
			assertEquals("10894;1", history.getOrderIen());
			assertNotNull(history.getResultHistory());
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}
	
	@Test
	public void fetchPatientOrderResultHistoryNoOrderIEN(){
		boolean passed = false;
		try {
			URI uri = getUri(assigningAuthority, patientId, siteId, "");
			OrderResultsHistory history = client.getJerseyClient().resource(uri).get(OrderResultsHistory.class);
			Assert.assertNotNull(history);
			assertEquals("10894;1", history.getOrderIen());
			assertNotNull(history.getResultHistory());
    	} catch (UniformInterfaceException e) {
    		String msg = AssertJersey.getMessage(e);
    		Assert.assertTrue("Expected message to contain 'Must not be null or empty'", msg.contains("Must not be null or empty"));
    		passed = true;
    	}
		
		if (passed == false)
			Assert.fail("Expected an assertion because no order ien was specified");
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
	private URI getUri(String assigningAuthority, String patientId, String siteId, String orderIen) {
		String uriFormat = String.format("%s/patient/%s/%s/order-results-history/site/%s?order-ien=%s",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										siteId,
										orderIen);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
}
