package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Admissions;
import com.agilex.healthcare.mobilehealthplatform.domain.AtomLink;
import com.agilex.healthcare.mobilehealthplatform.domain.HospitalLocations;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResult;
import com.agilex.healthcare.mobilehealthplatform.domain.Order;
import com.agilex.healthcare.mobilehealthplatform.domain.Orders;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientOrdersResourceTest {
	
	URI ordersUri = null;

	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;
	
	@BeforeClass
	public static void startup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void fetchCreatinineTestResultsTest(){
		final String assigningAuthority = "dfn-777";
		final String patientId = "8";
		final String siteId ="777";
		URI uri = UriBuilder.fromUri(getCreatenineTestResultsUri(assigningAuthority,patientId, siteId)).build();
		LabResult creatinineTest = client.getJerseyClient().resource(uri).get(LabResult.class);
		Assert.assertNotNull(creatinineTest);
		Assert.assertEquals("Check lower ref range", ".9", creatinineTest.getReferenceLow());
		Assert.assertEquals("Check higher ref range", "1.4", creatinineTest.getReferenceHigh());
	}
	
	@Test
    public void testFetchOrdersBySite() {
        try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteId ="777";
			final String serviceIEN = "2";
			final String statusIEN = "1";
			final String startDate = "02/02/2010";
			final String endDate = "02/02/2012";
			URI uri = getOrdersBySiteUri(assigningAuthority, patientId, siteId, serviceIEN, statusIEN, startDate, endDate);
			Orders orders = client.getJerseyClient().resource(uri).get(Orders.class);
			assertNotNull(orders);
			assertTrue(orders.size() > 0);

			for (Order order : orders) {
				AtomLink link = order.getLink().getLinkByTitle("order-results-history");
				assertNotNull(link);
				assertNotNull(link.getHref());
			}
        }
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
    }


	@Test
	public void testFetchOrdersBySiteNoDates() {
		try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteId ="777";
			final String serviceIEN = "2";
			final String statusIEN = "1";
			final String startDate = "";
			final String endDate = "";
			URI uri = getOrdersBySiteUri(assigningAuthority, patientId, siteId, serviceIEN, statusIEN, startDate, endDate);
			Orders orders = client.getJerseyClient().resource(uri).get(Orders.class);
			assertNotNull(orders);
			assertTrue(orders.size() > 0);

			for (Order order : orders) {
				AtomLink link = order.getLink().getLinkByTitle("order-results-history");
				assertNotNull(link);
				assertNotNull(link.getHref());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void testFetchOrdersBySiteNoEndDate() {
		try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteId ="777";
			final String serviceIEN = "2";
			final String statusIEN = "1";
			final String startDate = "02/02/2010";
			final String endDate = "";
			URI uri = getOrdersBySiteUri(assigningAuthority, patientId, siteId, serviceIEN, statusIEN, startDate, endDate);
			Orders orders = client.getJerseyClient().resource(uri).get(Orders.class);
			assertNotNull(orders);
			assertTrue(orders.size() > 0);

			for (Order order : orders) {
				AtomLink link = order.getLink().getLinkByTitle("order-results-history");
				assertNotNull(link);
				assertNotNull(link.getHref());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void testFetchOrdersBySiteNoStartDate() {
		try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteId ="777";
			final String serviceIEN = "2";
			final String statusIEN = "1";
			final String startDate = "";
			final String endDate = "02/02/2012";
			URI uri = getOrdersBySiteUri(assigningAuthority, patientId, siteId, serviceIEN, statusIEN, startDate, endDate);
			ClientResponse response = client.getJerseyClient().resource(uri).get(ClientResponse.class);
			assertNotNull(response);
			assertTrue(response.getStatus() == 400);
			String responseMsg = AssertJersey.getJSONResponse(response);
			String expectedMsg = "{\"errors\":[{\"fieldName\":\"startDate\",\"errorMessage\":\"startDate is required if endDate is provided\"}],\"object-type\":\"ValidationErrors\"}";
			assertEquals(expectedMsg, responseMsg);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void testFetchOrdersBySiteNoServiceIEN() {
		try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteId ="777";
			final String serviceIEN = "";
			final String statusIEN = "1";
			final String startDate = "02/02/2010";
			final String endDate = "02/02/2012";
			URI uri = getOrdersBySiteUri(assigningAuthority, patientId, siteId, serviceIEN, statusIEN, startDate, endDate);
			ClientResponse response = client.getJerseyClient().resource(uri).get(ClientResponse.class);
			assertNotNull(response);
			assertTrue(response.getStatus() == 400);
			String responseMsg = AssertJersey.getJSONResponse(response);
			String expectedMsg = "{\"errors\":[{\"fieldName\":\"service-ien\",\"errorMessage\":\"Must not be null or empty.\"}],\"object-type\":\"ValidationErrors\"}";
			assertEquals(expectedMsg, responseMsg);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void testFetchOrdersBySiteNoStatusIEN() {
		try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteId ="777";
			final String serviceIEN = "2";
			final String statusIEN = "";
			final String startDate = "02/02/2010";
			final String endDate = "02/02/2012";
			URI uri = getOrdersBySiteUri(assigningAuthority, patientId, siteId, serviceIEN, statusIEN, startDate, endDate);
			ClientResponse response = client.getJerseyClient().resource(uri).get(ClientResponse.class);
			assertNotNull(response);
			assertTrue(response.getStatus() == 400);
			String responseMsg = AssertJersey.getJSONResponse(response);
			String expectedMsg = "{\"errors\":[{\"fieldName\":\"status-ien\",\"errorMessage\":\"Must not be null or empty.\"}],\"object-type\":\"ValidationErrors\"}";
			assertEquals(expectedMsg, responseMsg);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void testFetchAdmissions(){
		try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteId ="777";
			URI uri = UriBuilder.fromUri(getAdmissionsUri(assigningAuthority,patientId, siteId)).build();
			Admissions admissions = client.getJerseyClient().resource(uri).get(Admissions.class);
			Assert.assertNotNull(admissions);
			Assert.assertTrue(admissions.getSize() > 0);
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}

	@Test
	public void testFetchClinics(){
		try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteId ="777";
			URI uri = UriBuilder.fromUri(getClinicsUri(assigningAuthority,patientId, siteId)).build();
			HospitalLocations hospitalLocations = client.getJerseyClient().resource(uri).get(HospitalLocations.class);
			Assert.assertNotNull(hospitalLocations);
			Assert.assertTrue(hospitalLocations.getSize() > 0);
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}

	@Test
	public void testFetchLocations(){
		try {
			final String assigningAuthority = "dfn-777";
			final String patientId = "8";
			final String siteId ="777";
			URI uri = UriBuilder.fromUri(getLocationsUri(assigningAuthority,patientId, siteId)).build();
			HospitalLocations hospitalLocations = client.getJerseyClient().resource(uri).get(HospitalLocations.class);
			Assert.assertNotNull(hospitalLocations);
			Assert.assertTrue(hospitalLocations.getSize() > 0);
    	} catch (UniformInterfaceException e) {
    		AssertJersey.fail(e);
    	}
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
	private URI getCreatenineTestResultsUri(String assigningAuthority, String patientId, String siteId) {
		String uriFormat = String.format("%s/patient/%s/%s/orders/site/%s/creatininetest",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										siteId);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
    private URI getLocationsUri(String assigningAuthority, String patientId, String siteId) {
		String uriFormat = String.format("%s/patient/%s/%s/orders/site/%s/locations",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										siteId);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}

    private URI getClinicsUri(String assigningAuthority, String patientId, String siteId) {
		String uriFormat = String.format("%s/patient/%s/%s/orders/site/%s/clinics",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										siteId);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
    
    private URI getAdmissionsUri(String assigningAuthority, String patientId, String siteId) {
		String uriFormat = String.format("%s/patient/%s/%s/orders/site/%s/admissions",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										siteId);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
	public URI getAcceptOrderUri(String assigningAuthority, String patientId, String siteId){
		String uriFormat = String.format("%s/patient/%s/%s/orders/site/%s/accept-order",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				siteId);
		
		StringBuilder sb = new StringBuilder(uriFormat);
        sb.append("?orderType=");
        sb.append("Labs");
        sb.append("&desiredDateTime=");
        sb.append("01/01/2002");	
        sb.append("&patientLocation=");
        sb.append("loc");	
        sb.append("&procedures=");
        sb.append("proc1");	
        sb.append("&orderIen=");
        sb.append("24324");	
		
		URI uri = UriBuilder.fromUri(sb.toString()).build();
		return uri;
		
	}
	
    public URI getSaveRadiologyOrderUri(String assigningAuthority, String patientId, String siteId) {
		String uriFormat = String.format("%s/patient/%s/%s/orders/site/%s/save-patient",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				siteId);
		
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
		
	}
	
    public URI getProcedureMessageUri(String assigningAuthority, String patientId, String siteId, String procedureIen) {
		String uriFormat = String.format("%s/patient/%s/%s/orders/site/%s/procedure-message",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				siteId);

		StringBuilder sb = new StringBuilder(uriFormat);
        sb.append("?procedureIen=");
        sb.append(procedureIen);
		
		URI uri = UriBuilder.fromUri(sb.toString()).build();
		return uri;
		
	}
	
    public URI getOrderImagingTypesUri(String assigningAuthority, String patientId, String siteId) {
		String uriFormat = String.format("%s/patient/%s/%s/orders/site/%s/imaging-types",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
				assigningAuthority,
				patientId,
				siteId);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
	
    private URI getOrdersBySiteUri(String assigningAuthority, String patientId, String siteId, String serviceIEN, String statusIEN, String startDate, String endDate) {
		String uriFormat = String.format("%s/patient/%s/%s/orders/site/%s?service-ien=%s&status-ien=%s&startDate=%s&endDate=%s",
										IntegrationTestConfiguration.getMobileHealthServerBaseUri(),
										assigningAuthority,
										patientId,
										siteId,
										serviceIEN,
										statusIEN,
										startDate,
										endDate);
		URI uri = UriBuilder.fromUri(uriFormat).build();
		return uri;
	}
}
