package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import static org.junit.Assert.*;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Problem;
import com.agilex.healthcare.mobilehealthplatform.domain.Problems;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientProblemsResourceTest {
	private static final String VISTA = "vista";
	private static final String OPERATIONAL = "operational";
	private static final String PATIENT_ID = "1";
	private static final String ASSIGNING_AUTHORITY = "dfn-777";
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void setup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void fetchProblem() {
		try {
			URI uri = fetchProblemsByScopeUri(ASSIGNING_AUTHORITY, PATIENT_ID, OPERATIONAL);
			Problems problems = client.getJerseyClient().resource(uri).get(Problems.class);
			assertNotNull(problems);
			assertTrue(problems.size() > 0);
			
			Problem expected = problems.get(0);
			uri = fetchProblemUri(ASSIGNING_AUTHORITY, PATIENT_ID, VISTA, expected.getUniqueId());
			Problem actual = client.getJerseyClient().resource(uri).get(Problem.class);
			assertNotNull(actual);
			
			assertEquals(expected.getUniqueId(), actual.getUniqueId());
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchProblemsByScope() {
		try {
			URI uri = fetchProblemsByScopeUri(ASSIGNING_AUTHORITY, PATIENT_ID, OPERATIONAL);
			Problems obj = client.getJerseyClient().resource(uri).get(Problems.class);
			assertNotNull(obj);
			assertTrue(obj.size() > 0);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchProblemsBySystem() {
		try {
			URI uri = fetchProblemsBySystemUri(ASSIGNING_AUTHORITY, PATIENT_ID, VISTA);
			Problems obj = client.getJerseyClient().resource(uri).get(Problems.class);
			assertNotNull(obj);
			assertTrue(obj.size() > 0);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

//----------------------------------------------------------------------------------------------------------------------

	private URI fetchProblemUri(String assigningAuthority, String patientId, String systemId, String id) {
		String uri = String.format("%s/patient/%s/%s/problems/system/%s/id/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, systemId, id);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	private URI fetchProblemsByScopeUri(String assigningAuthority, String patientId, String scope) {
		String uri = String.format("%s/patient/%s/%s/problems/scope/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, scope);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	private URI fetchProblemsBySystemUri(String assigningAuthority, String patientId, String systemId) {
		String uri = String.format("%s/patient/%s/%s/problems/system/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, systemId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
}
