package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.RadiologyResult;
import com.agilex.healthcare.mobilehealthplatform.domain.RadiologyResults;
import com.agilex.healthcare.mobilehealthplatform.domain.RadiologyTests;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientRadiologyResourceTest {
	private static final String OPERATIONAL = "operational";
	private static final String PATIENT_ID = "1";
	private static final String ASSIGNING_AUTHORITY = "dfn-777";
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void setup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void fetchRadiologyResult() {
		try {
			URI uri = fetchRadiologyResultUri(ASSIGNING_AUTHORITY, PATIENT_ID, "vista", "7039491.8876-1");
			RadiologyResult obj = client.getJerseyClient().resource(uri).get(RadiologyResult.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	
	@Test
	public void fetchRadiologyResultsByScope() {
		try {
			URI uri = fetchRadiologyResultsByScopeUri(ASSIGNING_AUTHORITY, PATIENT_ID, OPERATIONAL);
			RadiologyResults obj = client.getJerseyClient().resource(uri).get(RadiologyResults.class);
			assertNotNull(obj);
			assertTrue(obj.size() > 0);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	
	@Test
	public void fetchRadiologyResultsBySystem() {
		try {
			URI uri = fetchRadiologyResultsBySystemUri(ASSIGNING_AUTHORITY, PATIENT_ID, "vista");
			RadiologyResults obj = client.getJerseyClient().resource(uri).get(RadiologyResults.class);
			assertNotNull(obj);
			assertTrue(obj.size() > 0);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	
	@Test
	public void getRadiologyTestsByScope() {
		try {
			URI uri = getRadiologyTestsByScopeUri(ASSIGNING_AUTHORITY, PATIENT_ID, OPERATIONAL);
			RadiologyTests obj = client.getJerseyClient().resource(uri).get(RadiologyTests.class);
			assertNotNull(obj);
			assertTrue(obj.size() > 0);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	
	@Test
	public void getRadiologyTestsBySystem() {
		try {
			URI uri = getRadiologyTestsBySystemUri(ASSIGNING_AUTHORITY, PATIENT_ID, "vista");
			RadiologyTests obj = client.getJerseyClient().resource(uri).get(RadiologyTests.class);
			assertNotNull(obj);
			assertTrue(obj.size() > 0);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

//----------------------------------------------------------------------------------------------------------------------

	private URI fetchRadiologyResultUri(String assigningAuthority, String patientId, String systemId, String id) {
		String uri = String.format("%s/patient/%s/%s/radiology/results/system/%s/id/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, systemId, id);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	private URI fetchRadiologyResultsByScopeUri(String assigningAuthority, String patientId, String scope) {
		String uri = String.format("%s/patient/%s/%s/radiology/results/scope/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, scope);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	private URI fetchRadiologyResultsBySystemUri(String assigningAuthority, String patientId, String systemId) {
		String uri = String.format("%s/patient/%s/%s/radiology/results/system/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, systemId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	private URI getRadiologyTestsByScopeUri(String assigningAuthority, String patientId, String scope) {
		String uri = String.format("%s/patient/%s/%s/radiology/tests/scope/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, scope);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	private URI getRadiologyTestsBySystemUri(String assigningAuthority, String patientId, String systemId) {
		String uri = String.format("%s/patient/%s/%s/radiology/tests/system/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, systemId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
}
