package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import static org.junit.Assert.assertNotNull;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientDemographics;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifiers;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientResourceTest {
	private static final String ASSIGNING_AUTHORITY = "dfn-777";
	private static final String PATIENT_ID = "1";
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void setup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void fetchPatientIdentifiers() {
		try {
			URI uri = getFetchPatientIdentifiersUri(ASSIGNING_AUTHORITY, PATIENT_ID);
			PatientIdentifiers obj = client.getJerseyClient().resource(uri).get(PatientIdentifiers.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getDemographics() {
		try {
			URI uri = getDemographicsUri(ASSIGNING_AUTHORITY, PATIENT_ID);
			PatientDemographics obj = client.getJerseyClient().resource(uri).get(PatientDemographics.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getDemographicsByScope() {
		try {
			URI uri = getDemographicsByScopeUri(ASSIGNING_AUTHORITY, PATIENT_ID, "operational");
			PatientDemographics obj = client.getJerseyClient().resource(uri).get(PatientDemographics.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getPatient() {
		try {
			URI uri = getPatientUri(ASSIGNING_AUTHORITY, PATIENT_ID);
			Patient obj = client.getJerseyClient().resource(uri).get(Patient.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

//----------------------------------------------------------------------------------------------------------------------

	private URI getFetchPatientIdentifiersUri(String assigningAuthority, String patientId) {
		String uri = String.format("%s/patient/%s/%s/identifiers", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	private URI getDemographicsUri(String assigningAuthority, String patientId) {
		String uri = String.format("%s/patient/%s/%s/demographics", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	private URI getDemographicsByScopeUri(String assigningAuthority, String patientId, String scope) {
		String uri = String.format("%s/patient/%s/%s/demographics/scope/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, scope);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	private URI getPatientUri(String assigningAuthority, String patientId) {
		String uri = String.format("%s/patient/%s/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
}
