package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import static org.junit.Assert.assertNotNull;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Surgeries;
import com.agilex.healthcare.mobilehealthplatform.domain.Surgery;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientSurgeryResourceTest {
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void setup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}

	@Test
	public void fetchSurgeriesByScope() {
		try {
			URI uri = getFetchSurgeriesByScopeUri("dfn-777", "237", "operational");
			Surgeries obj = client.getJerseyClient().resource(uri).get(Surgeries.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchSurgeriesBySystem() {
		try {
			URI uri = getFetchSurgeriesBySystemUri("dfn-777", "237", "vista");
			Surgeries obj = client.getJerseyClient().resource(uri).get(Surgeries.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchSurgery() {
		try {
			URI uri = getFetchSurgeryUri("dfn-777", "237", "vista", "10007");
			Surgery obj = client.getJerseyClient().resource(uri).get(Surgery.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

//----------------------------------------------------------------------------------------------------------------------

	private URI getFetchSurgeriesByScopeUri(String assigningAuthority, String patientId, String scope) {
		String uri = String.format("%s/patient/%s/%s/surgeries/scope/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, scope);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getFetchSurgeriesBySystemUri(String assigningAuthority, String patientId, String system) {
		String uri = String.format("%s/patient/%s/%s/surgeries/system/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, system);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getFetchSurgeryUri(String assigningAuthority, String patientId, String systemId, String id) {
		String uri = String.format("%s/patient/%s/%s/surgeries/system/%s/id/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, systemId, id);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
}
