package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;
import java.util.Date;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.AtomLink;
import com.agilex.healthcare.mobilehealthplatform.domain.AtomLinks;
import com.agilex.healthcare.mobilehealthplatform.domain.Consult;
import com.agilex.healthcare.mobilehealthplatform.domain.Consults;
import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.ResourceDirectory;
import com.agilex.healthcare.mobilehealthplatform.domain.Specialties;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.agilex.healthcare.utility.DateHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class ProviderConsultsResourceTest {

	private static final String SPECIALTIES = "specialties";
	private static final String LOCATION_SERVICE_RESOURCE_DIRECTORY = "location-service-resource-directory";
	private static final String CONSULTS_BY_SPECIALTIES = "consults-by-specialties";
	private static final String MHPUSER = "mhpuser";
	private static final String CURRENT_PROVIDER_CONSULTS = "current-provider-consults";
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;
	
	@BeforeClass
	public static void startup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
        deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	
	@Test
    public void testMhpUserLinkExists() throws Exception{
		AtomLinks links = retrieveResourceDirectory();
		AtomLink mhpUserLink = links.getLinkByTitle(MHPUSER);
		assertNotNull(mhpUserLink);
    }

	private AtomLinks retrieveResourceDirectory() {
		URI resourceDirectoryUri = IntegrationTestConfiguration.getHealthAdapterResourceDirectoryUri();
		ResourceDirectory resourceDirectory = client.getJerseyClient().resource(resourceDirectoryUri).accept("application/xml").get(ResourceDirectory.class);
		AtomLinks links = resourceDirectory.getLink();
		return links;
	}
	
	@Test
    public void testProviderConsultsLinkExists() throws Exception{
		AtomLinks links = retrieveResourceDirectory();
		AtomLink mhpUserLink = links.getLinkByTitle(MHPUSER);
		MhpUser mhpUser = retrieveMhpUser(mhpUserLink);
		AtomLink providerConsultsLink = mhpUser.getLink().getLinkByTitle(CURRENT_PROVIDER_CONSULTS);
		assertNotNull(providerConsultsLink);
    }
	
	@Test
    public void testProviderConsultsSpecailitiesLinkExists() throws Exception{
		AtomLinks links = retrieveResourceDirectory();
		AtomLink mhpUserLink = links.getLinkByTitle(MHPUSER);
		MhpUser mhpUser = retrieveMhpUser(mhpUserLink);
		AtomLink providerConsultsLink = mhpUser.getLink().getLinkByTitle(CONSULTS_BY_SPECIALTIES);
		assertNotNull(providerConsultsLink);
    }
	
	@Test
    public void testSpecialitiesLinkExists() throws Exception{
		AtomLinks links = retrieveResourceDirectory();
		AtomLink mhpUserLink = links.getLinkByTitle(MHPUSER);
		MhpUser mhpUser = retrieveMhpUser(mhpUserLink);
		AtomLink locationServiceResourceDirectoryLink = mhpUser.getLink().getLinkByTitle(LOCATION_SERVICE_RESOURCE_DIRECTORY);
		ResourceDirectory locationServiceDirectory = client.getJerseyClient().resource(locationServiceResourceDirectoryLink.getHref()).accept("application/xml").get(ResourceDirectory.class);
		AtomLinks locationServiceLinks = locationServiceDirectory.getLink();
		AtomLink specialtiesLink = locationServiceLinks.getLinkByTitle(SPECIALTIES);
		assertNotNull(specialtiesLink);
    }
	
	@Test
    public void testFetchingOfSpecialities() throws Exception{
		AtomLinks links = retrieveResourceDirectory();
		AtomLink mhpUserLink = links.getLinkByTitle(MHPUSER);
		MhpUser mhpUser = retrieveMhpUser(mhpUserLink);
		AtomLink locationServiceResourceDirectoryLink = mhpUser.getLink().getLinkByTitle(LOCATION_SERVICE_RESOURCE_DIRECTORY);
		ResourceDirectory locationServiceDirectory = client.getJerseyClient().resource(locationServiceResourceDirectoryLink.getHref()).accept("application/xml").get(ResourceDirectory.class);
		AtomLinks locationServiceLinks = locationServiceDirectory.getLink();
		AtomLink specialtiesLink = locationServiceLinks.getLinkByTitle(SPECIALTIES);
		Specialties specialties = client.getJerseyClient().resource(specialtiesLink.getHref()).accept("application/xml").get(Specialties.class);
		assertTrue(specialties.getSize() > 0);
	}
	

	@Test
    public void testFetchingConsultsByProvider() throws Exception{
		AtomLinks links = retrieveResourceDirectory();
		AtomLink mhpUserLink = links.getLinkByTitle(MHPUSER);
		MhpUser mhpUser = retrieveMhpUser(mhpUserLink);
		AtomLink providerConsultsLink = mhpUser.getLink().getLinkByTitle(CURRENT_PROVIDER_CONSULTS);
		final String startDateString = "10/07/2012";
		final String endDateString = "10/07/2014";
		
		URI providerConsultsUriWithDateRange = addDateQueryParameters(providerConsultsLink.getHref(), startDateString, endDateString);
		Consults consults = client.getJerseyClient().resource(providerConsultsUriWithDateRange).accept("application/xml").get(Consults.class);
		
		assertConsults(consults, startDateString, endDateString);
	}

	@Test
    public void testFetchingConsultsByProviderWithDateRange() throws Exception{
		AtomLinks links = retrieveResourceDirectory();
		AtomLink mhpUserLink = links.getLinkByTitle(MHPUSER);
		MhpUser mhpUser = retrieveMhpUser(mhpUserLink);
		AtomLink providerConsultsLink = mhpUser.getLink().getLinkByTitle(CURRENT_PROVIDER_CONSULTS);
		final String startDateString = "01/07/2013";
		final String endDateString = "10/07/2014";
		
		URI providerConsultsUriWithDateRange = addDateQueryParameters(providerConsultsLink.getHref(), startDateString, endDateString);
		Consults consults = client.getJerseyClient().resource(providerConsultsUriWithDateRange).accept("application/xml").get(Consults.class);
		
		assertConsults(consults, startDateString, endDateString);
	}
	
	@Test
    public void testFetchingConsultsByIEN() throws Exception{
		AtomLinks links = retrieveResourceDirectory();
		AtomLink mhpUserLink = links.getLinkByTitle(MHPUSER);
		MhpUser mhpUser = retrieveMhpUser(mhpUserLink);
		AtomLink providerConsultsLink = mhpUser.getLink().getLinkByTitle(CURRENT_PROVIDER_CONSULTS);
		final String startDateString = "10/07/2012";
		final String endDateString = "10/07/2014";
		
		URI providerConsultsUriWithDateRange = addDateQueryParameters(providerConsultsLink.getHref(), startDateString, endDateString);
		Consults consults = client.getJerseyClient().resource(providerConsultsUriWithDateRange).accept("application/xml").get(Consults.class);
		Consult consult = consults.get(0);
		URI consultUri = consult.getLink().getUriByTitle("patient-consult");
		Consult actualConsult = client.getJerseyClient().resource(consultUri).accept("application/xml").get(Consult.class);
		assertEquals(consult.getConsultIEN(), actualConsult.getConsultIEN());
	}

	@Test
    public void testFetchingConsultsBySpecialities() throws Exception{
		AtomLinks links = retrieveResourceDirectory();
		AtomLink mhpUserLink = links.getLinkByTitle(MHPUSER);
		MhpUser mhpUser = retrieveMhpUser(mhpUserLink);
		AtomLink consultsBySpecialitiesLink = mhpUser.getLink().getLinkByTitle(CONSULTS_BY_SPECIALTIES);
		final String startDateString = "10/07/2012";
		final String endDateString = "12/07/2015";
		
		URI consultsBySpecialties = addDateQueryParameters(consultsBySpecialitiesLink.getHref(), startDateString, endDateString);
		consultsBySpecialties = addSpecialties(consultsBySpecialties, "1500002658");
		Consults consults = client.getJerseyClient().resource(consultsBySpecialties).accept("application/xml").get(Consults.class);
		
		assertConsults(consults, startDateString, endDateString);
		assertEquals(1500002658, consults.get(0).getRequestServiceSID().longValue());
	}

	
	
	private void assertConsults(Consults consults, String startDateString, String endDateString) {
		assertTrue(consults.size() > 0);
		Date startDate = DateHelper.parse(startDateString, "MM/dd/yyyy");
		Date endDate = DateHelper.parse(endDateString, "MM/dd/yyyy");
		for (Consult consult : consults) {
			Date createdDate = consult.getCreatedDate();
			if (!(startDate.before(createdDate) && endDate.after(createdDate))) {
				Assert.fail("Consult is not within the range");
			}
		}
	}

	
	private URI addDateQueryParameters(URI uri, String startDate, String endDate) throws Exception{
		StringBuffer uriWithDates = new StringBuffer();
		uriWithDates.append(uri.toString());
		uriWithDates.append("?startDate=" + startDate);
		uriWithDates.append("&");
		uriWithDates.append("endDate=" + endDate);

		return new URI(uriWithDates.toString());
	}
	
	
	private URI addSpecialties(URI uri, String specialties) throws Exception{
		StringBuffer uriWithSpecialties = new StringBuffer();
		uriWithSpecialties.append(uri.toString());
		uriWithSpecialties.append("&specialties=" + specialties);
		return new URI(uriWithSpecialties.toString());
		
	}

	private MhpUser retrieveMhpUser(AtomLink mhpUserLink){
		MhpUser mhpUser = client.getJerseyClient().resource(mhpUserLink.getHref()).accept("application/xml").get(MhpUser.class);
		return mhpUser;
	}
}
