package com.agilex.healthcare.mobilehealthplatform.restservice.patients;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.Patients;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientSearchBySearchStringViaClientApiTest {

	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;
	
	@BeforeClass
	public static void setup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	private Patients performTest(String searchString, int expectedNumberOfRows) {
		return performTest(searchString, expectedNumberOfRows, null);
	}
	private Patients performTest(String searchString, int expectedNumberOfRows, String expectedLastName) {
		Patients patients = client.getPatientResourceClient().patientSearch(searchString);
		assertTrue(patients.size() == expectedNumberOfRows);
		
		if (expectedLastName != null) {
			for (Patient patient : patients)
				assertEquals(expectedLastName.toLowerCase(), patient.getLastName().toLowerCase());
		}
		return patients;
	}

	@Test
	public void searchByLastName() {
		String searchString = IntegrationTestConfiguration.getProdTestPatient().getLastName();
		String expectedLastName = IntegrationTestConfiguration.getProdTestPatient().getLastName();
		int expectedNumOfRecords = 1;
		performTest(searchString, expectedNumOfRecords, expectedLastName);
	}

	@Test
	public void searchByLastFirstName() {
		String searchString = IntegrationTestConfiguration.getProdTestPatient().getLastName()+","+IntegrationTestConfiguration.getProdTestPatient().getFirstName();
		String expectedLastName = IntegrationTestConfiguration.getProdTestPatient().getLastName();
		int expectedNumOfRecords = 1;
		performTest(searchString, expectedNumOfRecords, expectedLastName);
	}
	
	@Test
	public void searchByLastFirstMiddleName() {
		String lastName = IntegrationTestConfiguration.getProdTestPatientWithMiddleName().getLastName();
		String firstName = IntegrationTestConfiguration.getProdTestPatientWithMiddleName().getFirstName();
		String middleName = IntegrationTestConfiguration.getProdTestPatientWithMiddleName().getMiddleName();
		
		String searchString = lastName
				+ "%2C" //","
				+ "%20" //" "
				+ firstName
				+ "%20" //" "
				+ middleName;
		int expectedNumOfRecords = 1;
		Patients patients = performTest(searchString, expectedNumOfRecords, lastName);
		assertEquals(firstName.toLowerCase(), patients.get(0).getFirstName().toLowerCase());
		assertEquals(middleName.toLowerCase(), patients.get(0).getMiddleName().toLowerCase());
	}
	
	@Test
	public void searchBySsn() {
		String ssn = IntegrationTestConfiguration.getProdTestPatient().getSsn();
		String searchString = ssn.substring(0, 3) + "-" + ssn.substring(3, 5) + "-" + ssn.substring(5, 9);
		String expectedLastName = IntegrationTestConfiguration.getProdTestPatient().getLastName();
		int expectedNumOfRecords = 1;
		performTest(searchString, expectedNumOfRecords, expectedLastName);
	}
	@Test
	public void searchBySsnWithoutDash() {
		String ssn = IntegrationTestConfiguration.getProdTestPatient().getSsn();
		String searchString = ssn.replaceAll("-", "");
		String expectedLastName = IntegrationTestConfiguration.getProdTestPatient().getLastName();
		int expectedNumOfRecords = 1;
		performTest(searchString, expectedNumOfRecords, expectedLastName);
	}
	@Test
	public void searchBySsnPattern() {
		String ssn = IntegrationTestConfiguration.getProdTestPatient().getSsn();
		String lastName = IntegrationTestConfiguration.getProdTestPatient().getLastName();
		String searchString = lastName.substring(0, 1) + ssn.substring(5, 9);
		int expectedNumOfRecords = 21;
		performTest(searchString, expectedNumOfRecords);
	}
}
