package com.agilex.healthcare.mobilehealthplatform.restservice.patients.links;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.springframework.http.MediaType;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.agilex.healthcare.utility.XpathHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientsResourceLinksTest {
	private static Node patientAsXml;

	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;
	
	protected static Node getPatient() {
		return patientAsXml;
	}

	protected static void setPatient(Node patientAsXml) {
		PatientsResourceLinksTest.patientAsXml = patientAsXml;
	}

	@BeforeClass
	public static void loadPatients() {
		try {
			deleteTokenURI = TestHelper.getDeleteTokenURI();
			System.out.println("load patients");

			URI rootPatientsUri = IntegrationTestConfiguration.getResourceUri("patients");
			URI patientSearchUri = UriBuilder.fromUri(rootPatientsUri).queryParam("lastname", "seventeen").build();

			client = TestHelper.createMobileHealthClient();

			Document patientsAsXml = client.getJerseyClient().resource(patientSearchUri).accept(MediaType.APPLICATION_XML.toString()).get(Document.class);
			setPatient(XpathHelper.getNode(patientsAsXml, "m:patients/m:patient"));
			System.out.println("loaded patients");
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}

	@Test
	public void patientHasLinks() {
		NodeList linkNodes = XpathHelper.getNodeList(getPatient(), "atom:link");
		assertTrue(linkNodes.getLength() > 0);
		System.out.println("patient has " + linkNodes.getLength() + " atom links");
	}

	@Test(expected = AssertionError.class)
	public void patientDoesNotHaveBunkLink() {
		validateLinkExists("invalid-title");
	}

	@Test
	public void validatePatientIdentifiers() {
		validateLinkExists("patient-identifiers");
	}
	
	@Test
	public void validateDemographics() {
		validateLinkExists("patient-demographics");
	}

	@Test
	public void validateImage() {
		validateLinkExists("patient-image");
	}

	@Test
	public void validateProblems() {
		validateLinkExists("patient-problems");
	}

	@Test
	public void validateAllergies() {
		validateLinkExists("patient-allergies");
	}

	@Test
	public void validateSurgeries() {
		validateLinkExists("patient-surgeries");
	}

	@Test
	public void validateSurgery() {
		validateLinkExists("patient-surgeries");
	}

	@Test
	public void validateAppointmentsPast() {
		validateLinkExists("patient-appointments-past");
	}

	@Test
	public void validateAppointments() {
		validateLinkExists("patient-appointments");
	}

	@Test
	public void validateCalendarEvents() {
		validateLinkExists("patient-calendar-events");
	}

	@Test
	public void validateCalendarPublicKey() {
		validateLinkExists("patient-calendar-publickey");
	}

	@Test
	public void validateAdmissionsLogs() {
		validateLinkExists("patient-admisions");
	}

	@Test
	public void validateRadiologyTest() {
		validateLinkExists("patient-radiology-tests");
	}

	@Test
	public void validateRadiologyResult() {
		validateLinkExists("patient-radiology-results");
	}

	@Test
	public void validateMedications() {
		validateLinkExists("patient-medications");
	}

	@Test
	public void validateActiveMedications() {
		validateLinkExists("patient-medications-active");
	}

	@Test
	public void validateOutpatienMedications() {
		validateLinkExists("patient-medications-outpatient");
	}

	@Test
	public void validateInpatientMedications() {
		validateLinkExists("patient-medications-inpatient");
	}

	@Test
	public void validateActiveOutpatientMedications() {
		validateLinkExists("patient-medications-outpatient-active");
	}

	@Test
	public void validateActiveInpatientMedications() {
		validateLinkExists("patient-medications-inpatient-active");
	}

	@Test
	public void validateLabTests() {
		validateLinkExists("patient-lab-tests");
	}

	@Test
	public void validateLabMicroResults() {
		validateLinkExists("patient-lab-micro-results");
	}

	@Test
	public void validateLabMicroTests() {
		validateLinkExists("patient-lab-micro-tests");
	}

	@Test
	public void validateLabMicroTestsByGroup() {
		validateLinkExists("patient-lab-micro-tests-by-group");
	}

	@Test
	public void validateNotifications() {
		validateLinkExists("patient-notifications");
	}

	@Test
	public void validateDevices() {
		validateLinkExists("patient-devices");
	}

	@Test
	public void validateMedicationRefills() {
		validateLinkExists("patient-medication-refills");
	}
	
	private void validateLinkExists(String title) {
		Node linkNode = XpathHelper.getNode(getPatient(), "atom:link[@title='" + title + "']");
		assertNotNull(linkNode);
	}

}
