package com.agilex.healthcare.mobilehealthplatform.restservice.radiology;


import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.RadiologyResults;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class RadiologyResourceProdModeTest {
    private static String assigningAuthority = "dfn-777";
    private static String uniqueId = "148";
    private static Patient patient;
    private static MobileHealthClientTestVersion client;
    private static String deleteTokenURI = null;

    @BeforeClass
    public static void checkModeOfOperation(){
        client = TestHelper.createLVProviderMobileHealthClient();
        patient = getTestPatient(createPatientIdentifier());
        deleteTokenURI = TestHelper.getDeleteTokenURI();
    }

    @AfterClass
    public static void destroy(){
        //client.getJerseyClient().resource(deleteTokenURI).delete();
        if (client != null) {
        	try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
            client = null;
        }
    }

    @Test
    public void fetchAndVerifyRadiologyReports(){
		URI uri = patient.getLink().getLinkByTitle("patient-radiology-results-operational").getHref();
		uri = UriBuilder.fromUri(uri).queryParam("startDate", "1/1/1995").queryParam("enddate", "1/1/2000").build();
		RadiologyResults radiologyResults = client.getJerseyClient().resource(uri).get(RadiologyResults.class);
		assertTrue(radiologyResults.size() > 0);
		assertNotNull(radiologyResults);

    }
    
    public static PatientIdentifier createPatientIdentifier(){
        PatientIdentifier patientIdentifier = new PatientIdentifier();
        patientIdentifier.setAssigningAuthority(assigningAuthority);
        patientIdentifier.setUniqueId(uniqueId);
        return patientIdentifier;
    }
    public static Patient getTestPatient(PatientIdentifier patientIdentifier){
        URI patientUri = UriBuilder.fromPath(IntegrationTestConfiguration.getMobileHealthServerBaseUri()).path("patient").path(patientIdentifier.getAssigningAuthority()).path(patientIdentifier.getUniqueId()).build();
        return client.getJerseyClient().resource(patientUri).get(Patient.class);
    }
}
