package com.agilex.healthcare.mobilehealthplatform.restservice.surgery;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.LinkTitles;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.Patients;
import com.agilex.healthcare.mobilehealthplatform.domain.SurgeryReports;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class SurgeryReportResourceTest {

	private static MobileHealthClientTestVersion client;
	private static String START_DATE = "01/01/2002";
	private static String END_DATE = "01/01/2008";
	private static String deleteTokenURI = null;
	
	@BeforeClass
	public static void checkModeOfOperation(){
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy(){
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}

	@Test
	public void testFetchSurgeryReportsForPatientFromVista(){
		try {
			Patient patient = getTestPatient();
			URI surgeryReportUri = getSurgeryReportUri(patient.getLink().getLinkByTitle(LinkTitles.patientSurgeryReports).getHref().toString());
			SurgeryReports surgeryReports = client.getJerseyClient().resource(surgeryReportUri).get(SurgeryReports.class);
			assertNotNull(surgeryReports);
			assertTrue(surgeryReports.size() > 0);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	private Patient getTestPatient(){
		Patient searchPatient = new Patient();
		searchPatient.setSsn("666-00-0001");
		Patients patients = client.getPatientResourceClient().patientSearch(searchPatient);
		return patients.get(0);
	}

	private URI getSurgeryReportUri(String surgeryReportsUri){
		StringBuilder sb = new StringBuilder(surgeryReportsUri);
		sb.append("?startDate=");
		sb.append(START_DATE);
		sb.append("&endDate=");
		sb.append(END_DATE);
		return UriBuilder.fromUri(sb.toString()).build();
	}
}
