package com.agilex.healthcare.mobilehealthplatform.restservice.system;

import static org.junit.Assert.assertNotNull;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.AtomLink;
import com.agilex.healthcare.mobilehealthplatform.domain.LinkTitles;
import com.agilex.healthcare.mobilehealthplatform.domain.ResourceDirectory;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class ResourceDirectoryResourceTest {
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void setup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void fetchResourceDirectory() {
		try {
			URI uri = getResourceDirectoryUri();
			ResourceDirectory obj = client.getJerseyClient().resource(uri).accept("application/json").get(ResourceDirectory.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getResourcesAsJsonOnSubpath() {
		try {
			URI uri = getResourceDirectoryJsonUri();
			ResourceDirectory obj = client.getJerseyClient().resource(uri).get(ResourceDirectory.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchSingleResourceLink() {
		try {
			URI uri = getResourceDirectoryLinkTitleUri(LinkTitles.UserSession);
			AtomLink obj = client.getJerseyClient().resource(uri).get(AtomLink.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchSingleResourceUri() {
		try {
			URI uri = getResourceDirectoryLinkTitleHrefUri(LinkTitles.UserSession);
			String obj = client.getJerseyClient().resource(uri).get(String.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

//----------------------------------------------------------------------------------------------------------------------

	private URI getResourceDirectoryUri() {
		String uri = String.format("%s/public/resource-directory", IntegrationTestConfiguration.getMobileHealthServerBaseUri());
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getResourceDirectoryJsonUri() {
		String uri = String.format("%s/public/resource-directory/json", IntegrationTestConfiguration.getMobileHealthServerBaseUri());
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getResourceDirectoryLinkTitleUri(String linkTitle) {
		String uri = String.format("%s/public/resource-directory/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), linkTitle);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getResourceDirectoryLinkTitleHrefUri(String linkTitle) {
		String uri = String.format("%s/public/resource-directory/%s/href", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), linkTitle);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
}
