package com.agilex.healthcare.mobilehealthplatform.restservice.system;

import static org.junit.Assert.assertNotNull;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.w3c.dom.Document;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class SystemMetricsResourceTest {
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void setup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void getSystemMetrics() {
		try {
			URI uri = getSystemMetricsUri();
			Document doc = client.getJerseyClient().resource(uri).get(Document.class);
			assertNotNull(doc);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getSystemMetric() {
		try {
			//This just returns system properties, so grab one that we know will exist.
			URI uri = getSystemMetricsUri("property-java.version");
			String str = client.getJerseyClient().resource(uri).get(String.class);
			assertNotNull(str);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

//----------------------------------------------------------------------------------------------------------------------

	private URI getSystemMetricsUri() {
		String uri = String.format("%s/metrics", IntegrationTestConfiguration.getMobileHealthServerBaseUri());
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getSystemMetricsUri(String key) {
		String uri = String.format("%s/metrics/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), key);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
}
