package com.agilex.healthcare.mobilehealthplatform.restservice.user;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.UserSession;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class UserSessionResourceTest {
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void setup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void getUserSession() {
		try {
			URI uri = getUserSessionUri();
			UserSession userSession = client.getJerseyClient().resource(uri).get(UserSession.class);
			assertNotNull(userSession);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void logoff() {
		try {
			URI uri = getUserSessionLogoffUri();
			String msg = client.getJerseyClient().resource(uri).get(String.class);
			assertEquals("session invalidated", msg);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

//----------------------------------------------------------------------------------------------------------------------

	private URI getUserSessionUri() {
		String uri = String.format("%s/user-session", IntegrationTestConfiguration.getMobileHealthServerBaseUri());
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getUserSessionLogoffUri() {
		String uri = String.format("%s/user-session/logoff", IntegrationTestConfiguration.getMobileHealthServerBaseUri());
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
}
