package com.agilex.healthcare.notewriter.restservice;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;
import java.util.Iterator;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitle;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitles;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.agilex.healthcare.utility.NullChecker;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.UniformInterfaceException;

public class ClinicalEventResourceNoteTitlesTest {
	private static final String SITE = "777";
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void checkWhetherTestsAreRunningInCIHMode() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}
	
	@AfterClass
	public static void tearDown() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void isNoteTitleConsultTrue() {
		try {
			URI uri = getNoteWriterIsNoteTitleConsultUri(SITE, "1", "1637");
			String response = client.getJerseyClient().resource(uri).get(String.class);

			assertNotNull(response);
			assertTrue("Expected result to be true", response.contains("true"));
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void isNoteTitleConsultFalse() {
		try {
			URI uri = getNoteWriterIsNoteTitleConsultUri(SITE, "1", "1");
			String response = client.getJerseyClient().resource(uri).get(String.class);

			assertNotNull(response);
			assertTrue("Expected result to be false", response.contains("false"));
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void isNoteTitleOneVisitNoteTrue() {
		try {
			URI uri = getNoteWriterIsNoteTitleOneVisitNoteUri(SITE, "1", "IGNORED", "1", "1301", "BRAIN%20%20%3CC%26P%20BRAIN%20AND%20SPINAL%20CORD%3E", "114", "3140501.213926", "A");
			String response = client.getJerseyClient().resource(uri).get(String.class);

			assertNotNull(response);
			assertTrue("Expected result to be false", response.contains("false"));
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void isNoteTitlePatientRecordFlagTrue() {
		try {
			URI uri = getNoteWriterIsNoteTitlePatientRecordFlagUri(SITE, "1", "1356");
			String response = client.getJerseyClient().resource(uri).get(String.class);

			assertNotNull(response);
			assertTrue("Expected result to be true", response.contains("true"));
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void isNoteTitlePatientRecordFlagFalse() {
		try {
			URI uri = getNoteWriterIsNoteTitlePatientRecordFlagUri(SITE, "1", "1");
			String response = client.getJerseyClient().resource(uri).get(String.class);

			assertNotNull(response);
			assertTrue("Expected result to be false", response.contains("false"));
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void isNoteTitleSurgeryNoteFalse() {
		try {
			URI uri = getNoteWriterIsNoteTitleSurgeryNoteUri(SITE, "1", "1");
			String response = client.getJerseyClient().resource(uri).get(String.class);

			assertNotNull(response);
			assertTrue("Expected result to be false", response.contains("false"));
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getNoteTitles() {
		try {
			String staffId = "staff046";
			URI uri = getNoteWriterNoteTitlesUri(SITE, staffId, null);
			NoteTitles titles = client.getJerseyClient().resource(uri).get(NoteTitles.class);
			
			assertTrue(titles.size() > 0);

			String previousName = "";
			for (int i = 0; i < titles.getSize(); i++) {
				NoteTitle title = titles.get(i);
				
				assertTrue(NullChecker.isNotNullish(title.getIen()));
				assertTrue(NullChecker.isNotNullish(title.getName()));
				
				assertTrue(previousName.compareTo(title.getName()) <= 0);
				previousName = title.getName();
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void getNoteTitlesWithFilter() {
		try {
			String staffId = "staff046";
			String searchString = "ph";
			URI uri = getNoteWriterNoteTitlesUri(SITE, staffId, searchString);
			NoteTitles titles = client.getJerseyClient().resource(uri).get(NoteTitles.class);
			
			assertTrue(titles.size() > 0);
			
			String previousName = "";
			for (int i = 0; i < titles.getSize(); i++) {
				NoteTitle title = titles.get(i);
				
				assertTrue(NullChecker.isNotNullish(title.getIen()));
				assertTrue(NullChecker.isNotNullish(title.getName()));
				assertTrue(String.format("Didn't find '%s' in '%s'", searchString.toLowerCase(), title.getName().toLowerCase()), title.getName().toLowerCase().contains(searchString.toLowerCase()));
				
				assertTrue(previousName.compareTo(title.getName()) <= 0);
				previousName = title.getName();
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getNoteTitlesWithFilterDecoded() {
		try {
			String staffId = "staff046";
			String searchString = "c%26p";
			String searchStringDecoded = "c&p";
			URI uri = getNoteWriterNoteTitlesUri(SITE, staffId, searchString);
			NoteTitles titles = client.getJerseyClient().resource(uri).get(NoteTitles.class);
			
			assertTrue(titles.size() > 0);
			
			String previousName = "";
			for (int i = 0; i < titles.getSize(); i++) {
				NoteTitle title = titles.get(i);
				
				assertTrue(NullChecker.isNotNullish(title.getIen()));
				assertTrue(NullChecker.isNotNullish(title.getName()));
				assertTrue(String.format("Didn't find '%s' in '%s'", searchStringDecoded.toLowerCase(), title.getName().toLowerCase()), title.getName().toLowerCase().contains(searchStringDecoded.toLowerCase()));
				
				assertTrue(previousName.compareTo(title.getName()) <= 0);
				previousName = title.getName();
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getMdwsNoteTitles() {
		try {
			URI uri = getNoteWriterMdwsNoteTitlesUri(SITE, "1", null, null);
			NoteTitles noteTitles = client.getJerseyClient().resource(uri).get(NoteTitles.class);
			
			assertNotNull(noteTitles);
			assertEquals(44, noteTitles.size());
			
			Iterator<NoteTitle> it = noteTitles.iterator();
			while (it.hasNext()) {
				NoteTitle noteTitle = it.next();
				assertNotNull(noteTitle.getIen());
				assertNotNull(noteTitle.getName());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void getMdwsNoteTitlesWithSearchIndex() {
		try {
			URI uri = getNoteWriterMdwsNoteTitlesUri(SITE, "1", "br", null);
			NoteTitles noteTitles = client.getJerseyClient().resource(uri).get(NoteTitles.class);
			assertNotNull(noteTitles);
			assertEquals(44, noteTitles.size());
			assertEquals("BRAIN  <C&P BRAIN AND SPINAL CORD>", noteTitles.get(0).getName());
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getMdwsNoteTitlesWithSearchIndexForwards() {
		try {
			URI uri = getNoteWriterMdwsNoteTitlesUri(SITE, "1", "br", "1");
			NoteTitles noteTitles = client.getJerseyClient().resource(uri).get(NoteTitles.class);
			assertNotNull(noteTitles);
			assertEquals(44, noteTitles.size());
			assertEquals("BRAIN  <C&P BRAIN AND SPINAL CORD>", noteTitles.get(0).getName());
			assertEquals("C&P BRAIN AND SPINAL CORD", noteTitles.get(1).getName());
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getMdwsNoteTitlesWithSearchIndexBackwards() {
		try {
			URI uri = getNoteWriterMdwsNoteTitlesUri(SITE, "1", "br", "-1");
			NoteTitles noteTitles = client.getJerseyClient().resource(uri).get(NoteTitles.class);
			assertNotNull(noteTitles);
			assertEquals(44, noteTitles.size());
			//We won't include brain as this is a BACKWARD's lookup.
			assertEquals("BRAIN  <C&P BRAIN AND SPINAL CORD>", noteTitles.get(0).getName());
			assertEquals("C&P BRAIN AND SPINAL CORD", noteTitles.get(1).getName());
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getLastSelectedNoteTitleNoneReturned() {
		try {
			String facilityCode = "invalid";
			String staffId = "invalid";

			URI uri = getNoteWriterLastSelectedNoteTitleUri(facilityCode, staffId);
			ClientResponse response = client.getJerseyClient().resource(uri).get(ClientResponse.class);

			assertEquals(204, response.getStatus());
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
//----------------------------------------------------------------------------------------------------------------------

	private URI getNoteWriterIsNoteTitleConsultUri(String facilityCode, String staffId, String ien) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/note-title/is-consult/ien/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId, ien);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getNoteWriterIsNoteTitleOneVisitNoteUri(String facilityCode, String staffId, String assigningAuthority, String patientId, String ien, String noteTitle, String locationIen, String vistaTimestamp, String serviceCategory) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/patient/%s/%s/note-title/is-one-visit-note/ien/%s/noteTitle/%s/locationIen/%s/vistaTimestamp/%s/serviceCategory/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId, assigningAuthority, patientId, ien, noteTitle, locationIen, vistaTimestamp, serviceCategory);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getNoteWriterIsNoteTitlePatientRecordFlagUri(String facilityCode, String staffId, String ien) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/note-title/is-patient-record-flag/ien/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId, ien);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getNoteWriterIsNoteTitleSurgeryNoteUri(String facilityCode, String staffId, String ien) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/note-title/is-surgery-note/ien/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId, ien);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getNoteWriterNoteTitlesUri(String facilityCode, String staffId, String searchString) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/note-titles/progress-notes", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId);
		
		if (NullChecker.isNotNullish(searchString)) {
			uri = uri.concat("?searchstring=" + searchString);
		}
		
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getNoteWriterMdwsNoteTitlesUri(String facilityCode, String staffId, String searchString, String direction) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/note-titles/mdws-progress-notes", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId);
		
		if (NullChecker.isNotNullish(searchString)) {
			uri = uri.concat("?searchstring=" + searchString);
		}
		if (NullChecker.isNotNullish(direction)) {
			uri = uri.concat("&direction=" + direction);
		}
		
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	static URI getNoteWriterLastSelectedNoteTitleUri(String facilityCode, String staffId) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/note-titles/last-selected-progress-note-title", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId);
		
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
}
