package com.agilex.healthcare.notewriter.restservice;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.FacilityProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.FacilityProviders;
import com.agilex.healthcare.mobilehealthplatform.domain.LinkTitles;
import com.agilex.healthcare.mobilehealthplatform.domain.Modifier;
import com.agilex.healthcare.mobilehealthplatform.domain.Modifiers;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitRelatedTo;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitRelatedTos;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitType;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitTypes;
import com.agilex.healthcare.notewriter.datalayer.SearchFilter;
import com.agilex.healthcare.notewriter.domain.EncounterCodes;
import com.agilex.healthcare.notewriter.domain.EncounterDiagnosisICDCode;
import com.agilex.healthcare.notewriter.domain.EncounterDiagnosisICDCodes;
import com.agilex.healthcare.notewriter.domain.EncounterProcedureCPTCode;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.agilex.healthcare.utility.NullChecker;
import com.sun.jersey.api.client.UniformInterfaceException;

public class EncounterFormResourceTest {

	private static final String SITE = "777";
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void setup() {
		client = TestHelper.createLVProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}
	
	@AfterClass
	public static void tearDown() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}

	@Test
	public void getDiagnosisCodes() {
		try {
			URI uri = getClinicalCodesUri(SITE, "nip", null, LinkTitles.Diagnoses);
			EncounterCodes codes = client.getJerseyClient().resource(uri).get(EncounterCodes.class);
			assertTrue(codes.size() > 0);
			assertTrue(codes.size() <= SearchFilter.DEFAULT_MAX_RECORDS);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void getDiagnosisCodesMaxRecords() {
		try {
			URI uri = getClinicalCodesUri(SITE, "nip", "1", LinkTitles.Diagnoses);
			EncounterCodes codes = client.getJerseyClient().resource(uri).get(EncounterCodes.class);

			assertTrue(codes.size() > 0);
			assertTrue(codes.size() <= 1);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getFacilityProviders() {
		if (client != null) {
			try {
				URI uri = getFacilityProvidersUri(SITE);
				FacilityProviders provs = client.getJerseyClient().resource(uri).get(FacilityProviders.class);
				assertTrue(provs.size() > 0);
				assertTrue(provs.size() <= SearchFilter.DEFAULT_MAX_RECORDS);
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
		}
	}
	
	@Test
	public void getFacilityProvidersMdws() {
		if (client != null) {
			try {
				URI uri = getFacilityProvidersMdwsUri(SITE);
				FacilityProviders provs = client.getJerseyClient().resource(uri).get(FacilityProviders.class);
				assertTrue(provs.size() > 0); //No limit on the number of records returned
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
		}
	}
	
	@Test
	public void getModifiersByCptCode() {
		if (client != null) {
			try {
				URI uri = getModifiersByCptCodeUri(SITE, "00100");
				Modifiers modifiers = client.getJerseyClient().resource(uri).get(Modifiers.class);
				assertTrue(modifiers.size() > 0); //No limit on the number of records returned
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
		}
	}
	
	@Test
	public void getPatientProblemListDiagnoses() {
		try {
			String staffId = "1";
			String assigningAuthority = "EDIPI";
			String patientId = "1";
			URI uri = getProblemListDiagnosesUri(SITE, staffId, assigningAuthority, patientId);
			EncounterDiagnosisICDCodes codes = client.getJerseyClient().resource(uri).get(EncounterDiagnosisICDCodes.class);
			
			assertTrue(codes.size() > 0);

			for (int i = 0; i < codes.getSize(); i++) {
				EncounterDiagnosisICDCode code = codes.get(i);
				
				assertTrue(NullChecker.isNotNullish(code.getDescription()));
				assertTrue(NullChecker.isNotNullish(code.getCode()));
				assertTrue(NullChecker.isNotNullish(code.getCodeType()));
				assertTrue(NullChecker.isNotNullish(code.getIen()));
				assertTrue(NullChecker.isNotNullish(code.getShortDescription()));
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getProcedureCodes() {
		try {
			URI uri = getClinicalCodesUri(SITE, "Osteotomy", null, LinkTitles.Procedures);
			EncounterCodes codes = client.getJerseyClient().resource(uri).get(EncounterCodes.class);
			assertTrue(codes.size() > 0);
			assertTrue(codes.size() <= SearchFilter.DEFAULT_MAX_RECORDS);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getProcedureCodesMaxRecords() {
		try {
			URI uri = getClinicalCodesUri(SITE, "Osteotomy", "1", LinkTitles.Procedures);
			EncounterCodes codes = client.getJerseyClient().resource(uri).get(EncounterCodes.class);

			assertTrue(codes.size() > 0);
			assertTrue(codes.size() <= 1);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getProviders() {
		try {
			URI uri = getProvidersUri(SITE);
			FacilityProviders facilityProviders= client.getJerseyClient().resource(uri).get(FacilityProviders.class);
			
			assertTrue(facilityProviders.size() > 0);

			for (int i = 0; i < facilityProviders.getSize(); i++) {
				FacilityProvider facilityProvider = facilityProviders.get(i);
				
				assertNotNull(facilityProvider.getFacilityCode());
				assertNotNull(facilityProvider.getProviderIen());
				assertNotNull(facilityProvider.getProviderName());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getProvidersMdws() {
		try {
			URI uri = getMdwsProvidersUri(SITE);
			FacilityProviders facilityProviders= client.getJerseyClient().resource(uri).get(FacilityProviders.class);
			
			assertTrue(facilityProviders.size() > 0);
	
			for (int i = 0; i < facilityProviders.getSize(); i++) {
				FacilityProvider facilityProvider = facilityProviders.get(i);
				
				assertNotNull(facilityProvider.getFacilityCode());
				assertNotNull(facilityProvider.getProviderIen());
				assertNotNull(facilityProvider.getProviderName());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void getVisitRelatedTo() {
		try {
			String staffId = "1";
			String assigningAuthority = "EDIPI";
			String patientId = "1";
			String locationIen = "32";
			URI uri = getVisitRelatedToUri(SITE, staffId, assigningAuthority, patientId, locationIen);
			VisitRelatedTos visitRelatedTos = client.getJerseyClient().resource(uri).get(VisitRelatedTos.class);
			
			assertTrue(visitRelatedTos.size() > 0);

			for (int i = 0; i < visitRelatedTos.getSize(); i++) {
				VisitRelatedTo visitRelatedTo = visitRelatedTos.get(i);
				
				assertTrue(NullChecker.isNotNullish(visitRelatedTo.getTitle()));
				assertNotNull(visitRelatedTo.getEnabled());
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getVisitTypes() {
		try {
			String staffId = "1";
			String assigningAuthority = "EDIPI";
			String patientId = "1";
			String locationIen = "195";
			URI uri = getVisitTypesUri(SITE, staffId, assigningAuthority, patientId, locationIen);
			
			VisitTypes visitTypes = client.getJerseyClient().resource(uri).get(VisitTypes.class);
			
			assertTrue(visitTypes.size() > 0);

			for (int i = 0; i < visitTypes.getSize(); i++) {
				VisitType visitType = visitTypes.get(i);
				
				assertTrue(NullChecker.isNotNullish(visitType.getName()));
				assertTrue(NullChecker.isNotNullish(visitType.getSections()));
				
				for (EncounterProcedureCPTCode section : visitType.getSections()) {
					assertNotNull(section.getCode());
					assertNotNull(section.getDescription());
					assertNotNull(section.getShortDescription());
					
					for (Modifier modifier : section.getModifiers()) {
						assertNotNull(modifier.getCode());
						assertNotNull(modifier.getIen());
						assertNotNull(modifier.getName());
					}
				}
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
	private URI getClinicalCodesUri(String facilityCode, String searchString, String maxRecords, String linkUriName) {
		StringBuilder sb = new StringBuilder("%s/note-writer/encounter-form-service/site/%s/%s");
		
		boolean start = true;
		if (NullChecker.isNotNullish(searchString)) {
			sb.append(start ? "?" : "&");
			start = false;

			sb.append("searchstring=");
			sb.append(searchString);
		}
		if (NullChecker.isNotNullish(maxRecords)) {
			sb.append(start ? "?" : "&");
			start = false;
			
			sb.append("maxrecords=");
			sb.append(maxRecords);
		}
		
		String suri = String.format(sb.toString(), IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, linkUriName);
		URI uri = UriBuilder.fromUri(suri).build();
		return uri;
	}
	
	private URI getFacilityProvidersUri(String facilityCode) {
		String suri = String.format("%s/note-writer/encounter-form-service/site/%s/providers", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode);
		URI uri = UriBuilder.fromUri(suri).build();
		return uri;
	}
	
	private URI getFacilityProvidersMdwsUri(String facilityCode) {
		String suri = String.format("%s/note-writer/encounter-form-service/site/%s/mdws-providers", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode);
		URI uri = UriBuilder.fromUri(suri).build();
		return uri;
	}

	private URI getMdwsProvidersUri(String facilityCode) {
		String suri = String.format("%s/note-writer/encounter-form-service/site/%s/mdws-providers", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode);
		URI uri = UriBuilder.fromUri(suri).build();
		return uri;
	}
	
	private URI getModifiersByCptCodeUri(String facilityCode, String cptCode) {
		String suri = String.format("%s/note-writer/encounter-form-service/site/%s/modifiers?cpt-code=%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, cptCode);
		URI uri = UriBuilder.fromUri(suri).build();
		return uri;
	}
	
	private URI getProblemListDiagnosesUri(String facilityCode, String staffId, String assigningAuthority, String patientId) {
		String suri = String.format("%s/note-writer/encounter-form-service/site/%s/id/%s/patient/%s/%s/problem-list-diagnoses", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId, assigningAuthority, patientId);
		URI uri = UriBuilder.fromUri(suri).build();
		return uri;
	}

	private URI getProvidersUri(String facilityCode) {
		String suri = String.format("%s/note-writer/encounter-form-service/site/%s/providers", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode);
		URI uri = UriBuilder.fromUri(suri).build();
		return uri;
	}
	
	private URI getVisitRelatedToUri(String facilityCode, String staffId, String assigningAuthority, String patientId, String locationIen) {
		String suri = String.format("%s/note-writer/encounter-form-service/site/%s/id/%s/patient/%s/%s/visit-related-to?location-ien=%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId, assigningAuthority, patientId, locationIen);
		URI uri = UriBuilder.fromUri(suri).build();
		return uri;
	}
	
	private URI getVisitTypesUri(String facilityCode, String staffId, String assigningAuthority, String patientId, String locationIen) {
		String suri = String.format("%s/note-writer/encounter-form-service/site/%s/visit-types?location-ien=%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, locationIen);
		URI uri = UriBuilder.fromUri(suri).build();
		return uri;
	}
}
