package com.agilex.healthcare.testutility;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import org.junit.Assert;

import com.agilex.healthcare.mobilehealthplatform.domain.DataIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;

public class IdentifierUtil {
	/**
	 * A patient identifier comes back as an object when it's null with all of the fields set to null.
	 */
	public static void assertNullPatientIdentifier(PatientIdentifier patientIdentifier) {
		assertNotNull(patientIdentifier);
		assertNull(patientIdentifier.getAssigningAuthority());
		assertNull(patientIdentifier.getFirstName());
		assertNull(patientIdentifier.getLastName());
		assertNull(patientIdentifier.getSsn());
		assertNull(patientIdentifier.getUniqueId());
	}
	
	/**
	 * A patient identifier comes back as an object when it's null with all of the fields set to null.
	 */
	public static void assertPatientIdentifier(PatientIdentifier patientIdentifier, String assigningAuthority, String uniqueId) {
		assertNotNull(patientIdentifier);
		assertEquals(assigningAuthority, patientIdentifier.getAssigningAuthority());
		assertNull(patientIdentifier.getFirstName());
		assertNull(patientIdentifier.getLastName());
		assertNull(patientIdentifier.getSsn());
		assertEquals(uniqueId, patientIdentifier.getUniqueId());
	}
	
	/**
	 * Validates that 2 PatientIdentifiers are equal
	 */
	public static void assertPatientIdentifier(PatientIdentifier expected, PatientIdentifier actual) {
		if (expected == null && actual == null)
			return;
		else if (expected == null && actual != null)
			Assert.fail("expected identifier was null but actual wasn't");
		else if (expected != null && actual == null)
			Assert.fail("expected identifier was not null but actual was null");
		
		assertPatientIdentifier(actual, expected.getAssigningAuthority(), expected.getUniqueId());
	}
	
	/**
	 * A data identifier comes back as an object when it's null with all of the fields set to null.
	 */
	public static void assertNullDataIdentifier(DataIdentifier dataIdentifier) {
		assertNotNull(dataIdentifier);
		assertNull(dataIdentifier.getSystemId());
		assertNull(dataIdentifier.getUniqueId());
	}
	
	/**
	 * A data identifier comes back as an object when it's null with all of the fields set to null.
	 * Because we found that the uniqueId returned from mdws could be different for every single time
	 * you called it, we added in ability to just check if uniqueId is not null by passing in an empty string
	 * rather than null.
	 */
	public static void assertDataIdentifier(DataIdentifier dataIdentifier, String systemId, String uniqueId) {
		assertNotNull(dataIdentifier);
		if (systemId == null)
			assertNull(dataIdentifier.getSystemId());
		else
			assertEquals(systemId, dataIdentifier.getSystemId());
		
		if (uniqueId == null)
			assertNull(dataIdentifier.getUniqueId());
		else if (uniqueId.isEmpty())
			assertNotNull(dataIdentifier.getUniqueId());
		else
			assertEquals(uniqueId, dataIdentifier.getUniqueId());
	}
	
	/**
	 * Validates that 2 PatientIdentifiers are equal
	 */
	public static void assertDataIdentifier(DataIdentifier expected, DataIdentifier actual) {
		if (expected == null && actual == null)
			return;
		else if (expected == null && actual != null)
			Assert.fail("expected identifier was null but actual wasn't");
		else if (expected != null && actual == null)
			Assert.fail("expected identifier was not null but actual was null");
		
		assertDataIdentifier(actual, expected.getSystemId(), expected.getUniqueId());
	}
}
